(* For Mathematica 4.0 *)

integrand = 1 / (a x^2 + b x + c);
delta = 4 a c - b^2;
assums1 = {Element[a, Reals], Element[b, Reals], Element[c, Reals]};


(******************)
(* Case delta > 0 *)
(******************)

assums2 = Append[assums1, delta > 0];
int = Sign[a] 2 Pi / Sqrt[delta]; (* Proposed result *)

(* Case a > 0 *)
assums = Append[assums2, a > 0];
tmp1 = Integrate[integrand, {x, -Infinity, Infinity}, Assumptions->assums];
int - tmp1 //FullSimplify[#, assums]& (* Returns 0 *)

(* Case a < 0 *)
assums = Append[assums2, a < 0];
tmp1 = Integrate[integrand, {x, -Infinity, Infinity}, Assumptions->assums];
int - tmp1 //FullSimplify[#, assums]& (* Returns 0 *)

(* Note that the case a == 0 is not possible because delta > 0. *)


(*******************)
(* Case delta == 0 *)
(*******************)

assums = Append[assums1, delta == 0];
Integrate[integrand, {x, -Infinity, Infinity}, 
  Assumptions->assums]  (* Complains about indeterminate expression *)


(******************)
(* Case delta < 0 *)
(******************)

assums2 = Append[assums1, delta < 0];
int   = - Sign[a] 2 Pi / Sqrt[delta]; (* Proposed result *)
intPV = + Sign[a] 2 Pi / Sqrt[delta]; (* Proposed result for principal value *)

(* Case a > 0 *)
assums = Append[assums2, a > 0];

tmp1 = Integrate[integrand, {x, -Infinity, Infinity}, Assumptions->assums];
int - tmp1 //FullSimplify[#, assums]& (* Returns 0 *)

tmp2 = Integrate[integrand, {x, -Infinity, Infinity}, Assumptions->assums, 
  PrincipalValue->True];
intPV - tmp2 //FullSimplify[#, assums]& (* Returns 0 *)

(* Case a == 0 *)
assums = Append[assums2, a < 0];

tmp1 = Integrate[integrand, {x, -Infinity, Infinity}, Assumptions->assums];
int - tmp1 //FullSimplify[#, assums]& (* Returns 0 *)

tmp2 = Integrate[integrand, {x, -Infinity, Infinity}, Assumptions->assums, 
  PrincipalValue->True];
intPV - tmp2 //FullSimplify[#, assums]& (* Returns 0 *)

(* Case a < 0 *)
assums = Append[assums2, a < 0];

tmp1 = Integrate[integrand, {x, -Infinity, Infinity}, Assumptions->assums];
int - tmp1 //FullSimplify[#, assums]& (* Returns 0 *)

tmp2 = Integrate[integrand, {x, -Infinity, Infinity}, Assumptions->assums, 
  PrincipalValue->True];
intPV - tmp2 //FullSimplify[#, assums]& (* Returns 0 *)
