(* For Mathematica 4.0 *)

integrand = 1/(a*Sin[phi]+b*Cos[phi]+c);
assums1 = {Element[a, Reals], Element[b, Reals], Element[c, Reals]};
aa = c+b;
bb = -2 a;
cc = c-b;
delta = 4 aa cc - bb^2;


(******************)
(* Case delta > 0 *)
(******************)

int = 4 Pi Sign[cc] / Sqrt[delta]; (* Proposed result *)
assums2 = Append[assums1, delta > 0];

(* aa > 0 *)
assums = Append[assums2, aa > 0];

tmp1 = Integrate[integrand, {phi,0,2Pi}, Assumptions->assums];
FullSimplify[int - tmp1, assums] (* Gives 0 *)

(* aa > 0 *)
assums = Append[assums2, aa < 0];
tmp1 = Integrate[integrand, {phi,0,2Pi}, Assumptions->assums];
FullSimplify[int - tmp1, assums] (* Gives 0 *)

(* Note that aa == 0 is not possible for delta > 0 *)


(*******************)
(* Case delta == 0 *)
(*******************)

assums = Append[assums1, delta == 0];
tmp1=Integrate[integrand, {phi,0,2Pi}, Assumptions->assums]
FullSimplify[tmp1, assums] (* Complains about "division by zero" and returns
                              ComplexInfinity. *)


(******************)
(* Case delta < 0 *)
(******************)

assums = Append[assums1, delta < 0];

int   = 4 Pi Sign[cc] / Sqrt[delta]; (* Proposed result *)
tmp1 = Integrate[integrand, {phi,0,2Pi}, Assumptions->assums];
FullSimplify[int - tmp1, assums] (* Gives 0 *)

intPV = 4 Pi Sign[cc] / Sqrt[delta]; (* Proposed result for principal value *)
tmp2 = Integrate[integrand, {phi,0,2Pi}, Assumptions->assums, 
  PrincipalValue->True];
FullSimplify[intPV - tmp2, assums] (* Gives 0 *)

(* Integration with splitted integration range: *)
Integrate[integrand, {phi,0,Pi}, Assumptions->assums] + 
  Integrate[integrand, {phi,Pi,2Pi}, Assumptions->assums]
(* Returns Indeterminate *)
