# This file does a complete set of plots for the following parameters.  It is
# included by gnuplot files in the directories for the different values of tau.

# $0 = tau
# $1 = p_0
# $2 = p_1
# $3 = "maximum x range"
# $4 = "maximum x range for ending (=right) detail graph"
# $5 = "maximum x range for beginning (=left) detail graph"
#    = $4-"minimum x range for ending detail graph"
# $6 = "xtics increment for convergence graph"
# $7 = "xtics increment for detail graphs"
# $8 = "distance between measurement points (measured in recording points)"

set multiplot
set lmargin 0
set bmargin 0
set rmargin 0
set tmargin 0

set label 2 "tau = $0, p_0 = $1, p_1 = $2:" at screen -.065,1.08 left

set pointsize 0.5

# Convergence graph (i.e. (g_1-|c_1|^2)^2 plotted over the time, but only at
# the measurement points):
set size 1.2,0.5
set origin 0.0,0.52
set xrange [0:$3]
set yrange [0:0.4]
unset xtics
set ytics nomirror autofreq
plot \
     "$1_$2.rcrd" using 1:(($$4-$$3)**2) every $8 notitle with points pointtype 7

# Graph displaying the evolution of the guess:
set size 1.2,0.2
set origin 0.0,0.28
set yrange [-0.1:1.1]
set xtics nomirror $6
set ytics nomirror (0, 1)
plot \
     "$1_$2.rcrd" using 1:4 every $8 notitle with points pointtype 7

# Graph displaying the begining of the evolution:
# (1:2="|c_1_tilde|^2", 1:3="|c_1|^2", 1:4="g_1"):
set size 0.585,0.2
set origin 0.0,0.0
set xrange [0:$5]
set xtics nomirror $7
plot \
     "$1_$2.rcrd" using 1:2 notitle with lines linetype 0, \
     "$1_$2.rcrd" using 1:3 notitle with lines linetype 2, \
     "$1_$2.rcrd" using 1:4 every $8 notitle with points pointtype 7

# Graph displaying the begining of the evolution:
set origin 0.615,0.0
set xrange [$4-$5:$4]
unset ytics
plot \
     "$1_$2.rcrd" using 1:2 notitle with lines linetype 0, \
     "$1_$2.rcrd" using 1:3 notitle with lines linetype 2, \
     "$1_$2.rcrd" using 1:4 every $8 notitle with points pointtype 7

unset multiplot
