#ifndef MIN_S_VP_HPP
#define MIN_S_VP_HPP

#include "s-vp.hpp"
#include <cstdlib>
#include <vector>

/** This class takes care of the creation and maintenance of a vector
    containing an approximation of the superoptimal parameters of a
    postestimation procedure described in 0309302232.  */
class min_s_vps_tp {
public:
    min_s_vps_tp(int, int);
    ~min_s_vps_tp();
    void sample();
    void output();
private:
    inline int round_to_int(double);
    void try_to_store(int, int, int, int, int, int, int);

    int n_steps_mb; //< Number of steps used when sampling over a complete
                    //< parameter range (see also the documentation of this
                    //< class's constructor.
    std::vector<s_vp_tp*> V; //< Vector containing sets of superoptimal
                             //< parameters.
};

/** Rounds the floating point value \p x to an integer value. */
int min_s_vps_tp::round_to_int(double x) {
	return int(x + 0.5); 
}

#endif
