#ifndef S_VP_HPP
#define S_VP_HPP

/** A set containing the parameters \t a_0, \t a_1, \t b_0, \t b_1, \t beta_0,
    \t beta_1, \t gamma_0, \t gamma_1, and the corresponding disturbance and
    the postestimation deviation calculated according to 0309302232. */
class s_vp_tp {
public:
    s_vp_tp(int, int, int, int, int, int, int);
    inline double s();
    inline double vp();
    inline double a_0();
    inline double a_1();
    inline double b_0();
    inline double b_1();
    inline double beta_0();
    inline double beta_1();
    inline double gamma_0();
    inline double gamma_1();
private:
    bool a_m_eq_b_m(int);
    bool a_m_eq_0(int);
    bool b_m_eq_0(int);
    bool a_m_eq_b_m_eq_0(int);
    void update_P(int);
    void update_s();
    void update_vp();
    int maxstep_mb;
    int i_a_mb[1]; ///< a_mb * maxstep_mb (but only first element)
    int i_b_mb[1]; ///< b_mb * maxstep_mb (but only first element)
    int i_beta_mb[2]; ///< beta_mb * maxstep_mb
    int i_gamma_mb[2]; ///< gamma_mb * maxstep_mb
    double s_mb; ///< Disturbance
    double vp_mb; ///< Postestimation deviation
    double a_mb[2]; ///< {a_0, a_1}
    double b_mb[2]; ///< {b_0, b_1}
    double beta_mb[2]; ///< {beta_0, beta_1}
    double gamma_mb[2]; ///< {gamma_0, gamma_1}
    double P_mb[2]; ///< {P_0, P_1} as defined in 0309302232
};

double s_vp_tp::s() { return s_mb; }
double s_vp_tp::vp() { return vp_mb; }
double s_vp_tp::a_0() { return a_mb[0]; }
double s_vp_tp::a_1() { return a_mb[1]; }
double s_vp_tp::b_0() { return b_mb[0]; }
double s_vp_tp::b_1() { return b_mb[1]; }
double s_vp_tp::beta_0() { return beta_mb[0]; }
double s_vp_tp::beta_1() { return beta_mb[1]; }
double s_vp_tp::gamma_0() { return gamma_mb[0]; }
double s_vp_tp::gamma_1() { return gamma_mb[1]; }

#endif
