#include <iostream>
#include <vector>
#include <limits>
using namespace std;

static double
read_c12() {
    double c12;
    double tmp;

    // QQQ: What happens when there is no value during read?
    cin >> tmp;
    cin >> tmp;
    cin >> c12;
    cin >> tmp;
    return c12;
}

static void
clear_line(vector<double> &line) {
    for (int i = 0; i < line.size(); ++i)
        line[i] = 0.;
}

static void
add_to_line(vector<double> &line) {
    for (int i = 0; i < line.size(); ++i)
        line[i] += read_c12();
}

static void
divide_line(vector<double> &line, long n) {
    for (int i = 0; i < line.size(); ++i)
        line[i] /= n;
}

static void
output_line(vector<double> &line) {
    for (int i = 0; i < line.size(); ++i)
        cout << line[i] << " ";
    cout << "\n";
}

/** QQQ
    Reads |c_1|^2 from \p input into \p data until the input stream is
    exhausted or the limit of \p max_N values has been reached. Returns the
    number of actually read values. */
static void
create_table(unsigned long n_blocks, unsigned long n_lines_per_block,
             unsigned long n_c12s_per_line)
{
    vector<double> line(n_c12s_per_line);

    // QQQ: why segfault when input file is not .rcrd but, falsely, .in?
    cin.ignore(numeric_limits<int>::max(), '\n'); // Ignores first line
    
    for (int i = 0; i < n_blocks; ++i) {
        clear_line(line);
        for (int j = 0; j < n_lines_per_block; ++j)
            add_to_line(line);
        divide_line(line, n_lines_per_block);
        output_line(line);
    }
}

class usage_error_tp : public std::runtime_error {
    public:
    usage_error_tp(std::string s) : 
        runtime_error("Usage: "+s+" n_blocks n_lines_per_block \\\\\n"
                      "       n_c12s_per_line < data.rcrd > data.table\n"
                      "\n"
                      "The result of the program is described by the following algorithm:\n"
                      "1. \"n_blocks\"*\"n_lines_per_block\"*\"n_c12s_per_line\" subsequent values of\n"
                      "   |c_1|^2 from a Trasim rcrd file are read from standard input.\n"
                      "2. The values of |c_1|^2 are subdivided into \"n_blocks\" subsequent blocks.\n"
                      "3. Each block is condensed into a so called averaged line as follows:\n"
                      "3.a. The block is subdivided into \"n_lines_per_block\" sub blocks, called\n"
                      "     lines.\n"
                      "3.b. The corresponding values in each line are added. E.g. the first value in\n"
                      "     line 1, the first value in line 2, ..., and the first value in line\n"
                      "     \"n_lines_per_block\" are added.\n"
                      "3.c. The sums are divided by \"n_lines_per_block\". The result is stored into\n"
                      "     the averaged line.\n"
                      "4. The values in each averaged line are output to standard output. They are\n"
                      "   separated by blanks. The averaged lines are separated by newlines.\n"
) {}
};

class n_blocks_error_tp : public std::runtime_error {
    public:
    n_blocks_error_tp() :
        runtime_error("\"n_blocks\" needs to be of type unsigned long and greater or equal to one.") {}
};

static unsigned long to_n_blocks(char *n_blocks_string) {
    char *endptr;
    unsigned long n_blocks = 
        strtoul(n_blocks_string, &endptr, 10);
    if (strlen(endptr) != 0 || n_blocks < 1)
        throw n_blocks_error_tp();
    return n_blocks;
}

class n_lines_per_block_error_tp : public std::runtime_error {
    public:
    n_lines_per_block_error_tp() :
        runtime_error("\"n_lines_per_block\" needs to be of type unsigned long and greater or equal to one.") {}
};

static unsigned long to_n_lines_per_block(char *n_lines_per_block_string) {
    char *endptr;
    unsigned long n_lines_per_block = 
        strtoul(n_lines_per_block_string, &endptr, 10);
    if (strlen(endptr) != 0 || n_lines_per_block < 1)
        throw n_lines_per_block_error_tp();
    return n_lines_per_block;
}

class n_c12s_per_line_error_tp : public std::runtime_error {
    public:
    n_c12s_per_line_error_tp() :
        runtime_error("\"n_c12s_per_line\" needs to be of type unsigned long and greater or equal to one.") {}
};

static unsigned long to_n_c12s_per_line(char *n_c12s_per_line_string) {
    char *endptr;
    unsigned long n_c12s_per_line = 
        strtoul(n_c12s_per_line_string, &endptr, 10);
    if (strlen(endptr) != 0 || n_c12s_per_line < 1)
        throw n_c12s_per_line_error_tp();
    return n_c12s_per_line;
}

static void
create_table(int argc, char *argv[]) {
    if (argc != 4)
        throw usage_error_tp(argv[0]);
    create_table(to_n_blocks(argv[1]), 
                 to_n_lines_per_block(argv[2]),
                 to_n_c12s_per_line(argv[3]));
}

int main(int argc, char **argv) {
    try {
        create_table(argc, argv);
    } catch (const exception &e) {
        cerr << e.what() << "\n";
        return 1;
    }
    return 0;
}

