// QQQ: accepts rcrd(QQQ:rename?) files generated by Trasim 1.0(QQQ:define)

// QQQ: create a class. Document what kind of transform is used (refer to
// calculation #)

#include "power.hpp"
#include <fftw3.h>
#include <iostream>
#include <string>
#include <fstream>
#include <limits>
#include <vector>
#include <cmath>
using namespace std;

static double pi = 2*acos(0.);

static inline bool even(unsigned long x) {
    return x%2 == 0;
}

/** Reads |c_1|^2 from \p input into \p data until the input stream is
    exhausted or the limit of \p max_N values has been reached. Returns the
    number of actually read values. */
static unsigned long read_data(double *data, unsigned long max_N, 
                               istream &input)
{
    // QQQ: why segfault when input file is not .rcrd but, falsely, .in?
    input.ignore(numeric_limits<int>::max(), '\n'); // Ignores first line
    double tmp;
    unsigned long j = 0;
    unsigned long N = 0;
    while (input >> tmp) {
        if (j % 4 == 2) {
            if (N == max_N)
                break;
            data[N] = tmp, ++N; // Read |c_1|^2
        }
        ++j;
    }
    return N;
}

/** Preforms Fourier transform of \p N values of \p data. The result is stored
    in \p data itself. */
static void fourier_transform_data(double *data, unsigned long N, 
                                   double Delta_t)
{
    fftw_plan p = fftw_plan_r2r_1d(N, data, data, FFTW_R2HC, FFTW_ESTIMATE);
    fftw_execute(p);
    fftw_destroy_plan(p);
    for (unsigned long i = 0; i < N; ++i)
        data[i] *= Delta_t / sqrt(2*pi);
}

/** Converts the fourier spectrum contained in \p data into the first part of
    the power spectrum. The result is stored in the values data[0], data[1],
    ..., data[N/2]. */
static void fourier_to_power(double *data, unsigned long N) {
    for (unsigned long i = 0; i <= N/2; ++i) {
        if (i == 0)
            data[i] = pow(data[i], 2);
        else if (even(N) && i == N/2)
            data[i] = pow(data[i], 2);
        else
            data[i] = pow(data[i], 2) + pow(data[N-i], 2);
    }
}

/** Condenses the first part of the power spectrum in \p data to \p out_N
    values by averaging over adjacent values or by "stretching" values. The
    result is output to \p output in the format of \p out_N lines, each of
    which contains one value. */
static void condense_power_and_output(double *data, unsigned long N, 
                                      unsigned long out_N, double Delta_t,
                                      ostream &output) 
{
    if (out_N > N/2+1)
        // This error could be avoided by implementing functionality to
        // "stretch" the spectrum.
        throw out_N_too_large_error_tp();

    output << "# omega/(dim(Delta_t)^-1) amplitude\n";

    unsigned long j_min = 0, j_max;
    for (unsigned long n = 0; n < out_N; ++n) {
        j_max = (unsigned long)((N/2) * (double(n+1)/out_N));
        if (j_max >= N/2)
            j_max = N/2; // Might be necessary because of numeric. inaccuracies
        if (j_min <= j_max) {
            unsigned long block_size = j_max-j_min+1;
            double total = 0.;
            for (unsigned long j = j_min; j <= j_max; ++j)
                total += data[j];
            output 
                << 2*pi*((j_min+j_max)/2.)/N/Delta_t << " " // Average omega
                << total / block_size << "\n"; // QQQ: also output "x-position", i.e. freq?
        } else
            output << 0 << "\n";
        j_min = j_max+1;
    }
    output << flush;
}

/** Converts |c_1|^2 data from a Trasim rcrd-file (read from \p input) to the
    condensed first half of the discrete power spectrum of |c_1|^2 (written to
    \p output). \p max_N is the maximum number of number of samples of |c_1|^2
    to be read from \p input. \p out_N is the number of desired values in the
    output of the power spectrum. If the number of values in the condensed
    first half of the power spectrum is larger than \p out_N, then blocks of
    adjacent values are sampled to get the desired number of output values. */
void rcrd_to_powr(unsigned long max_N, unsigned long out_N, double Delta_t,
                  istream &input, ostream &output)
{
    // data first contains the values of |c_1|^2, then the discrete fourier
    // transform, and then the first half of the discrete power spectrum which,
    // finally, is ouput as condensed first half of the power spectrum
    // consisting of out_N values:
    double *data = (double *) fftw_malloc(sizeof(double) * max_N);
    if (data == 0)
        throw runtime_error("Not enough memory.\n");

    unsigned long N = read_data(data, max_N, input);
    fourier_transform_data(data, N, Delta_t);
    fourier_to_power(data, N);
    condense_power_and_output(data, N, out_N, Delta_t, output);

    fftw_free(data);
}

static unsigned long to_max_N(char *max_N_string) {
    char *endptr;
    unsigned long max_N = strtoul(max_N_string, &endptr, 10);
    // QQQ: check negativity?
    if (strlen(endptr) != 0 || max_N < 2)
        throw max_N_error_tp();
    return max_N;
}

static unsigned long to_out_N(char *out_N_string) {
    char *endptr;
    unsigned long out_N = strtoul(out_N_string, &endptr, 10);
    // QQQ: check negativity?
    if (strlen(endptr) != 0 || out_N < 2)
        throw out_N_error_tp();
    return out_N;
}

static double to_Delta_t(char *Delta_t_string) {
    char *endptr;
    double Delta_t = strtod(Delta_t_string, &endptr);
    if (strlen(endptr) != 0 || Delta_t <= 0.)
        throw Delta_t_error_tp();
    return Delta_t;
}

/** Calls \t rcrd_to_power(max_N, out_N, cin, cout), where \t max_N is
    extracted from \p argv[1], \t out_N is extracted from \p argv[2]. \p
    argv[0] is the name of the program. \p argc is the size of \p
    argv. Returns 0 only if the execution was sucessful. */
void rcrd_to_powr(int argc, char *argv[]) {
    if (argc != 4)
        throw usage_error_tp(argv[0]);
    rcrd_to_powr(to_max_N(argv[1]), to_out_N(argv[2]), to_Delta_t(argv[3]), 
                 cin, cout);
}
