#ifndef POWER__POWER_HPP
#define POWER__POWER_HPP

#include <iostream>
#include <stdexcept>
#include <string>

class out_N_too_large_error_tp : public std::runtime_error {
    public:
    out_N_too_large_error_tp() : 
        runtime_error("The number of output values in the condensed first part of the power spectrum,\n"
                      "out_N, needs to be smaller or equal to N/2+1, where N is the number of samples.") 
    {}
};

class out_of_memory_error_tp : public std::runtime_error {
    public:
    out_of_memory_error_tp() : runtime_error("Not enough memory.") {}
};

class usage_error_tp : public std::runtime_error {
    public:
    usage_error_tp(std::string s) : 
        runtime_error("Usage: "+s+" max_N out_N Delta_t < data.rcrd > data.powr\n"
                      "\n"
                      "Reads a Trasim rcrd file from standard input and outputs the condensed first\n"
                      "part of the discrete power spectrum of |c_1|^2 to standard output (for a more\n"
                      "detailed description, see the provided documentation). The parameter \"max_N\"\n"
                      "specifies the maximum number of samples of |c_1|^2 that should be processed\n"
                      "(type: the \"unsigned long\" C++ type). Note that the fast fourier transform is\n"
                      "especially quick if the number of processed samples is a product of small\n"
                      "factors. The parameter \"out_N\" (type: \"unsigned long\") specifies the number of\n"
                      "values in the condensed spectrum, and the parameter \"Delta_t\" specifies the\n"
                      "distance in time between samples (in the same units as those that specify the\n"
                      "time in the rcrd file).") {}
};

class max_N_error_tp : public std::runtime_error {
    public:
    max_N_error_tp() : 
        runtime_error("\"max_N\" needs to be of type unsigned long and greater or equal to two.") {}
};

class out_N_error_tp : public std::runtime_error {
    public:
    out_N_error_tp() :
        runtime_error("\"out_N\" needs to be of type unsigned long and greater or equal to two.") {}
};

class Delta_t_error_tp : public std::runtime_error {
    public:
    Delta_t_error_tp() :
        runtime_error("\"Delta_t\" needs to be of type double and greater than zero.") {}
};

void rcrd_to_powr(unsigned long, unsigned long, double, 
                  std::istream &, std::ostream &);
void rcrd_to_powr(int, char *[]);

#endif
