#ifndef TRASIM__FREQUENCY_TRACKING_HPP
#define TRASIM__FREQUENCY_TRACKING_HPP

#include "params.hpp"
#include "measurement.hpp"
#include "h2.hpp"
#include "tracking.hpp"
#include <stdexcept>
#include <fstream>
#include <vector>

/** Frequency Tracking is a tracking method that consists of two steps: QQQ (or
    just use calculation number? Additionally, but not solely, because variable
    names need to be explained here) */
class frequency_tracking_tp : public tracking_tp {
public:
    class E_1_less_than_E_0_error_tp;
    class Delta_t_too_small_error_tp;
    class N_too_small_error_tp;
    class n_rec_points_too_small_error_tp;
    frequency_tracking_tp(params_tp &params, std::ostream &, std::ostream &);
    unsigned long track();
    double beta();
    double omega();
    double omega_tilde();
    double omega_tilde_aprx();
private:
    void evolve_rec();
    void evolve(bool);
    void record_and_evolve(bool, int);
    void update_omega_tilde_aprx(int);
    void stream_misc1();
    void stream_misc2();
    double acos_f(double, double);
    const std::vector<operator_tp> measurement_operators();

    measurement_tp measurement_mb; ///< Measurement used

    double Delta_t_mb; ///< Defines the distance between 
                       ///< measurements. dim(Delta_t) = 
                       ///< dim(tracking_tp::E_0())/h_bar

    int N_mb; ///< 2*N is the total number of measurements.

    double omega_tilde_aprx_mb; ///< Current approximation for the frequency
                                ///< omega_tilde

    const double beta_g_mb; ///< Arbitrary value of beta that is used for
                            ///< calculating the guess
    unsigned long M_mb[2]; ///< {M(Delta_t/2), M(Delta_t)}
    int n_rec_points_mb; ///< Number of recording points per Delta_t/2
    double Delta_t_rec_mb; ///< Distance between recording points (QQQ:dim?)
    operator_tp U_mb; ///< exp(-i*H*Delta_t/hbar) = Unitary evolution
                      ///< between measurements
    operator_tp U_half_mb; ///< exp(-i*H*Delta_t/2/hbar) = Unitary evolution
                           ///< between measurements
    operator_tp U_rec_mb; ///< exp(-i*H*Delta_t_rec/hbar) = Unitary evolution
                          ///< between recording points
    double phase_g_mb; ///< Current value of the argument of the cosine in the
                       ///< guess
    std::vector<int> 
    dont_record_mb; ///< Data between the measurements number
                    ///< 2*dont_record_mb[0]-1 and 2*dont_record_mb[1]-1 is not
                    ///< recorded. The first measurements is denoted by 0.
                    ///< -1 refers to t=0, where no measurement is performed.
    // QQQ: ^^^ mention possible inconsistencies?

#if 0 // QQQ
    skip_rec_points_mb; ///< The recording points in the interval
                        ///< [skip_rec_points_mb[0], < skip_rec_points_mb[1]],
                        ///< are not stored and not written to the output 
                        ///< stream. However, the usual caluculations at each 
                        ///< recording point are done in order to keep the 
                        ///< resulting data consistent with data where other 
                        ///< or no recording points have been omitted.
#endif
};

class frequency_tracking_tp::E_1_less_than_E_0_error_tp : 
    public std::runtime_error 
{
public:
    E_1_less_than_E_0_error_tp() : 
        runtime_error("E_1 needs to be greater than or equal to E_0.") {}
};

class frequency_tracking_tp::Delta_t_too_small_error_tp : 
    public std::runtime_error 
{
public:
    Delta_t_too_small_error_tp() : 
        runtime_error("Delta_t needs to be greater than 0.") {}
};

class frequency_tracking_tp::N_too_small_error_tp : 
    public std::runtime_error
{
public:
    N_too_small_error_tp() : 
        runtime_error("N needs to be greater than or equal to 1.") {}
};

class frequency_tracking_tp::n_rec_points_too_small_error_tp : public std::runtime_error {
public:
    n_rec_points_too_small_error_tp() : 
        runtime_error("n_rec_points needs to be at least 1.") {}
};

#endif
