#ifndef TRASIM__MEASUREMENT_HPP
#define TRASIM__MEASUREMENT_HPP

#include "h2.hpp"
#include "params.hpp"
#include <vector>
 
/** A measurement. It consists of a bunch of measurement operators. 

    \author Felix E. Klee (felix.klee@inka.de)
    \date 2004 */
class measurement_tp {
public:
    measurement_tp(const std::vector<operator_tp> &M, 
                   double completeness_eps = -1);
    measurement_tp(params_tp &params, const std::string &, 
                   double completeness_eps = -1);
    int measure(state_tp &);
    void measure(state_tp &, int);
    bool complete(double);
    class incomplete_error_tp;
    inline state_tp state();
    inline const std::vector<operator_tp> &M() const;
    inline const std::vector<operator_tp> &E() const;
    class N_out_of_range_error_tp;
private:
    void update_effects();
    std::vector<operator_tp> M_mb; ///< Measurement operators
    std::vector<operator_tp> E_mb; ///< Effect operators

    friend bool operator==(const measurement_tp &, const measurement_tp &);
    friend bool approx_eq(const measurement_tp &, const measurement_tp &,
                          approx_tp comp = aprx_r, double eps = -1);
    friend std::ostream &operator<<(std::ostream &, const measurement_tp &);
};

class measurement_tp::N_out_of_range_error_tp : public std::runtime_error {
    public:
    N_out_of_range_error_tp(const std::string &sect) : 
        runtime_error("The number of measurement operators, "+sect+
					  ":N, is out of range.") {}
};

class measurement_tp::incomplete_error_tp : public std::runtime_error {
public:
    incomplete_error_tp() : 
        runtime_error("The set of measurement operators is incomplete.") {}
};

/** Returns the set of measurement operators. */
inline const std::vector<operator_tp> &measurement_tp::M() const {
    return M_mb; 
}

/** Returns the set of effect operators. */
inline const std::vector<operator_tp> &measurement_tp::E() const {
    return E_mb; 
}

std::ostream &operator<<(std::ostream &, const measurement_tp &);

bool operator==(const measurement_tp &, const measurement_tp &);
bool operator!=(const measurement_tp &, const measurement_tp &);
bool approx_eq(const measurement_tp &, const measurement_tp &, 
               approx_tp, double);

#endif
