#include <fstream>
#include "misc.hpp"

using namespace std;

/** \file

    Miscellaneous stuff. */

mt19937_tp mt19937;

double pi = 2*acos(0.);


/** Picks randomly a value i out of {0, 1, 2, ..., p.size()-1} with probability
    p[i]. It is assumed but not checked that p[0] + p[1] + ... + p[p.size()-1] =
    1. If this is only nearly the case the probability for p.size()-1 might be
    slightly larger or smaller than p[p.size()-1]. On the other hand, if the
    difference to 1 is large then the higher values for i might be considerably
    less probable or more probable than "intended". If \p p is empty then then
    the result is -1. */
int pick_randomly(const double_vect &p) {
    double a = 0;
    double rand = mt19937.genrand_real2(); // Random # out of [0,1)
    int j = 0;
    for (double_vect::const_iterator i = p.begin(); 
         i != p.end(); ++i, ++j) 
    {
        a += *i;
        if (rand < a)
            return j;
    }
    return p.size()-1;
}

/** Reads the first \p N white space separated integer values from the text
    file named \p filename and stores them in the vector \p v. May throw
    exceptions of type \c runtime_error if the file cannot be read. */
void read_int_vector(string filename, vector<int> &v, size_t N) {
    ifstream file(filename.c_str());
    if (!file)
        throw runtime_error(string("Cannot open file \"")+
                            filename+string("\"."));
    v.clear();
    try {
        v.reserve(N);
    } catch(...) {
        throw runtime_error(string("Not enough memory to read ") +
                            stringify(N) + string(" values from the file ") +
                            filename + string("."));
    }

    for (size_t i = 0; i < N; ++i) {
        int tmp;
        file >> tmp;
        if (!file)
            throw runtime_error(string("Reading file \"")+filename+
                                "\" failed.");
        v.push_back(tmp);
    }
}
