#ifndef TRASIM__MT19937_HPP
#define TRASIM__MT19937_HPP

/** This is a C++ interface to the original 2002/1/26-version of mt19937
    (Mersenne Twister pseudorandom number generator). The main function was
    removed and code was encapsulated into a C++ class and prettified where
    neccessary. Also some minor additions were supplemented.

    \sa The original mt19937/readme_mt.txt
    \sa The original mt19937/mt19937ar.c

    \author Makoto Matsumoto
    \author Takuji Nishimura
    \author Isaku Wada
    \author Felix E. Klee (felix.klee@inka.de) (only C++-interface and some minor additions)

    \date 2004
*/

class mt19937_tp {
public:
    mt19937_tp();
    void init_genrand(unsigned long s);
    void init_by_array(unsigned long init_key[], int key_length);
    void init_by_time();
    unsigned long genrand_int32();
    long genrand_int31();
    double genrand_real1();
    double genrand_real2();
    double genrand_real3();
    double genrand_res53();
private:
    /* Period parameters: */  
    static const int N = 624;
    static const int M = 397;
    static const unsigned long MATRIX_A;   ///< constant vector a
    static const unsigned long UPPER_MASK; ///< most significant w-r bits
    static const unsigned long LOWER_MASK; ///< least significant r bits

    unsigned long mt[N]; ///< the array for the state vector
    int mti; ///< mti==N+1 means mt[N] is not initialized
};

#endif
