#ifndef TRASIM__TRASIM_HPP
#define TRASIM__TRASIM_HPP

#include "tracking.hpp"
#include <stdexcept>
#include <string>

/** The main program. 

    \author Felix E. Klee (felix.klee@inka.de)
    \date 2004 */
class trasim_tp {
public:
    class invalid_seed_error_tp;
    class cmd_line_error_tp;
    class procedure_error_tp;
    class open_file_error_tp;
    trasim_tp(int, char **);
    ~trasim_tp();
    void run();
private:
    void load_random_seed(params_tp &);
    tracking_tp *t;
    std::string rcrd_file_name_mb; ///< Output file name for data recorded
                                   ///< during the simulation of tracking
    std::string misc_file_name_mb; ///< Output file name for misc. data
    std::ofstream misc_stream_mb; ///< Stream for file misc_file_name
    std::ofstream rcrd_stream_mb; ///< Stream for file rcrd_file_name
    unsigned long random_seed_mb; ///< Seed used to initialize the RNG
};

class trasim_tp::invalid_seed_error_tp : public std::runtime_error {
public:
    invalid_seed_error_tp(const std::string &s) : 
        runtime_error("The random seed \""+s+"\" is invalid.")
    {}
};

class trasim_tp::cmd_line_error_tp : public std::runtime_error {
public:
    cmd_line_error_tp(const std::string &app_name) : 
        runtime_error("Usage: "+app_name+" input_file.in")
    {}
};

class trasim_tp::procedure_error_tp : public std::runtime_error {
public:
    procedure_error_tp(const std::string &procedure) :
        runtime_error("Unknown procedure \""+procedure+"\".")
    {}
};

class trasim_tp::open_file_error_tp : public std::runtime_error {
public:
    open_file_error_tp(const std::string &file_name) :
        runtime_error("Cannot open \""+file_name+"\".")
    {}
};

#endif
