#include "uniform-tracking_aks-guess.hpp"
#include "misc.hpp"
#include <stdexcept>

using namespace std;

const string uniform_tracking_tp::aks_guess_tp::
sec_name() const { return "aks-guess"; }

uniform_tracking_tp::aks_guess_tp::
aks_guess_tp(params_tp &params, const operator_tp &M0) : 
    nser_guess_tp(params, sec_name())
{
    p_0_mb = norm(M0(0,0)); // |M0(0,0)|^2
    p_1_mb = norm(M0(1,1)); // |M0(1,1)|^2
    Delta_p_mb = p_1_mb-p_0_mb;
    p_bar_mb = (p_1_mb+p_0_mb) / 2;
    if (approx_eq(Delta_p_mb, 0, aprx_a, 1e-10))
        throw invalid_M_error_tp(); // Delta_p_mb == 0 would cause div. by zero
}

/** Updates the guess based on \p outcomes_mb. See the description of this
    class for how this is done. */
void uniform_tracking_tp::aks_guess_tp::
update_guess() {
    int N_0 = 0; // Number of outcomes 0
    for (vector<int>::const_iterator i = outcomes_mb.begin();
         i != outcomes_mb.end(); ++i)
    {
        if (*i == 0)
            N_0 ++;
    }
    double r = double(N_0) / N_mb;
    g_mb = (r - p_0_mb) / Delta_p_mb;
}

void uniform_tracking_tp::aks_guess_tp::stream_misc(ostream &o) const {
    nser_guess_tp::stream_misc(o);
    o << "p_0 = " << p_0_mb << "\n";
    o << "p_1 = " << p_1_mb << "\n";
    o << "Delta_p = " << Delta_p_mb << "\n";
    o << "p_bar = " << p_bar_mb << "\n";
}
