#ifndef TRASIM__UNIFORM_TRACKING__AKS_GUESS_HPP
#define TRASIM__UNIFORM_TRACKING__AKS_GUESS_HPP

#include "uniform-tracking_nser-guess.hpp"

/** N-series guess where the guess is calculated from \c outcomes_mb as
    follows:
    
    g_1 = (r - p_0) / Delta_p,

    where

    - r = N_0 / N, with N_0 = "# of outcomes 0", N = "# of measurements in
      N-series"
    - Delta_p = p_1 - p_0
    - p_0 = |M0_00|^2, p_1 = |M0_11|^2, with M0 = 0-th measurement operator

    For more information about this guess and the origin of its name, see
    0403060139.

    Note that this guess is supposed to be used with minimal measurements
    consisting of two measurements operators diagonal in the {|0>,|1>}
    basis. However, you can also try it out with other measurements as long as
    the following requirements is met: Delta_p != 0 <=> |M0_00|^2 !=
    |M0_11|^2. */
class uniform_tracking_tp::aks_guess_tp : 
    public uniform_tracking_tp::nser_guess_tp 
{
public:
    aks_guess_tp(params_tp &, const operator_tp &);
    void stream_misc(std::ostream &) const;
	class invalid_M_error_tp;
protected:
    void update_guess();
    virtual const std::string sec_name() const;
private:
    double p_0_mb; ///< sqrt(M0_00)
    double p_1_mb; ///< sqrt(M0_11)
    double Delta_p_mb; ///< p_1 - p_0
    double p_bar_mb; ///< (p_1 + p_0) / 2
};

class uniform_tracking_tp::aks_guess_tp::
	invalid_M_error_tp : public std::runtime_error 
{
public:
	invalid_M_error_tp() : runtime_error("The selected guess demands "
										 "sqrt(|M_11|) != sqrt(|M_00|).") {}
};

#endif
