#include "uniform-tracking_guess.hpp"
#include <stdexcept>

using namespace std;

uniform_tracking_tp::guess_tp::guess_tp(double guess) : guess_mb(guess) {}

/** Returns a guess for |c_1|^2 that is based on the last measurement
    outcomes. This function is called at equidistant points in time (once after
    each measurement, i.e. each call to new_outcome(), and maybe several times
    in between measurements). */
double uniform_tracking_tp::guess_tp::guess() {
    return guess_mb; // Default behavior - Override this function in order to do
                     // something else (i.e. guess() might update the guess each
                     // time in order to get a smoother time evolution of the
                     // guess).
}

/** Called after a measurement with result \p m. */
void uniform_tracking_tp::guess_tp::new_outcome(int m) {
    // This default implementation does nothing and is only useful for guesses
    // that ignore measurement results (i.e. stupid guesses).
    (void) m;
}

/** Dump miscellaneous data to \p o. */
void uniform_tracking_tp::guess_tp::stream_misc(ostream &o) const {
    o << "\n\n";
    o << "[" << sec_name() << "]\n";
}
