#include "uniform-tracking_min-dev-nser-guess.hpp"
#include <stdexcept>

using namespace std;

const string uniform_tracking_tp::min_dev_nser_guess_tp::
sec_name() const { return "minimum-deviation-n-series-guess"; }

uniform_tracking_tp::min_dev_nser_guess_tp::
min_dev_nser_guess_tp(params_tp &params, const operator_tp &M0) : 
    nser_guess_tp(params, sec_name())
{
    p_0_mb = norm(M0(0,0)); // |M0(0,0)|^2
    p_1_mb = norm(M0(1,1)); // |M0(1,1)|^2
    if (approx_eq(p_0_mb, 0, aprx_a, 1e-20) || 
        approx_eq(p_0_mb, 1, aprx_a, 1e-20) || 
        approx_eq(p_1_mb, 0, aprx_a, 1e-20) || 
        approx_eq(p_1_mb, 1, aprx_a, 1e-20))
        throw invalid_M_error_tp(); // see QQQ on why this is neccessary

    Delta_p_mb = p_1_mb-p_0_mb;
    p_bar_mb = (p_1_mb+p_0_mb) / 2;
}

/** Updates the guess based on \p outcomes_mb. See the description of this class
    how this is done. */
void uniform_tracking_tp::min_dev_nser_guess_tp::
update_guess() {
    int N_0 = 0; // Number of outcomes 0
    for (vector<int>::const_iterator i = outcomes_mb.begin();
         i != outcomes_mb.end(); ++i)
    {
        if (*i == 0)
            N_0 ++;
    }

	double a = pow(p_0_mb, double(N_0)) * pow(1 - p_0_mb, double(N_mb - N_0));
	double b = pow(p_1_mb, double(N_0)) * pow(1 - p_1_mb, double(N_mb - N_0));

	g_mb = b / (a + b);
}

void uniform_tracking_tp::min_dev_nser_guess_tp::stream_misc(ostream &o) const {
    nser_guess_tp::stream_misc(o);
    o << "p_0 = " << p_0_mb << "\n";
    o << "p_1 = " << p_1_mb << "\n";
    o << "Delta_p = " << Delta_p_mb << "\n";
    o << "p_bar = " << p_bar_mb << "\n";
}
