#ifndef TRASIM__UNIFORM_TRACKING__MIN_DEV_NSER_GUESS_HPP
#define TRASIM__UNIFORM_TRACKING__MIN_DEV_NSER_GUESS_HPP

#include "uniform-tracking_nser-guess.hpp"

/** N-series guess where the guess is calculated from \p outcomes_mb as
    follows:
    
    g = 1 / ((p_0/p_1)^N_0 * ((1-p_0)/(1-p_1))^(N-N_0) + 1),

    where

	- N = "# of measurements in N-series"
    - N_0 = "# of outcomes 0 in N-series"
    - p_0 = |M0_00|^2, p_1 = |M0_11|^2, with M0 = 0-th measurement operator

    See also my calculation 0305181811 and PRA63052102. */
class uniform_tracking_tp::min_dev_nser_guess_tp : 
    public uniform_tracking_tp::nser_guess_tp 
{
public:
    min_dev_nser_guess_tp(params_tp &, const operator_tp &);
    void stream_misc(std::ostream &o) const;
	class invalid_M_error_tp;
protected:
    void update_guess();
    virtual const std::string sec_name() const;
private:
    double p_0_mb; ///< sqrt(M0_00)
    double p_1_mb; ///< sqrt(M0_11)
    double Delta_p_mb; ///< p_1 - p_0
    double p_bar_mb; ///< (p_1 + p_0) / 2
};

class uniform_tracking_tp::min_dev_nser_guess_tp::
	invalid_M_error_tp : public std::runtime_error 
{
public:
	invalid_M_error_tp() : runtime_error("The selected guess demands \n"
										 "|M_00|^2 != 0 && |M_00|^2 != 1 &&"
                                         "|M_11|^2 != 0 && |M_11|^2 != 1.") {}
};

#endif
