#ifndef TRASIM__UNIFORM_TRACKING__MIN_DEV_PROJ_GUESS_HPP
#define TRASIM__UNIFORM_TRACKING__MIN_DEV_PROJ_GUESS_HPP

#include "uniform-tracking_guess.hpp"

/** Guess that minimizes the postestimation deviation at points in time
    immediately after projective measurements (it does not necessarily minimize
    the tracking deviation as defined in 04032700). Only the last measurement
    outcome is processed for each guess.

    Note that no check is performed whether the measurement operators are
    really projective because the user should be able to use non projective
    measurement operators "at his own risk" (the results of the guess will then
    most likely be junk). */
class uniform_tracking_tp::min_dev_proj_guess_tp : 
    public uniform_tracking_tp::guess_tp 
{
public:
    min_dev_proj_guess_tp(params_tp &, const measurement_tp &meas);
    virtual void new_outcome(int outcome);
protected:
    virtual const std::string sec_name() const;
private:
    const measurement_tp &meas_mb; ///< Measurement used
    state_tp one_mb; ///< |1>
};

#endif
