#include "uniform-tracking_nser-guess.hpp"
#include <stdexcept>

using namespace std;

/** Initializes an N-series guess. sec_name2 is the name of the section in the
	parameter file that contains the data for the guess. */
uniform_tracking_tp::nser_guess_tp::
nser_guess_tp(params_tp &params, const string &sec_name2) : 
	g_mb(0.5)
{
    int tmp = params.get<int>(sec_name2+":N");
    if (tmp < 1)
        throw N_smaller_than_one_error_tp(sec_name2);
	else
		N_mb = tmp;
    outcomes_mb.reserve(N_mb);
}
                                       
uniform_tracking_tp::nser_guess_tp::
~nser_guess_tp() {}

/** 1. Adds the outcome of the last measurement to \p outcomes_mb. \n
    2. If this results in \p outcomes_mb to be full (i.e. have the size \p
       N_mb) then it
       - uses this vector to calculate a new value for the current guess with
         the function \p update_guess()
       - clears \p outcomes_mb.
    3. Sets \p g_mb as the current guess. */
void uniform_tracking_tp::nser_guess_tp::
new_outcome(int outcome) {
    outcomes_mb.push_back(outcome);
    if (outcomes_mb.size() == N_mb) {
        update_guess();
        outcomes_mb.clear();
    }
    guess_mb = g_mb;
}

/** \fn void uniform_tracking_tp::nser_guess_tp::update_guess()
    
    Updates the value for the guess \p g_m from the vector of outcomes \p
    outcomes_mb. */
