#ifndef TRASIM__UNIFORM_TRACKING__NSER_GUESS_HPP
#define TRASIM__UNIFORM_TRACKING__NSER_GUESS_HPP

#include "uniform-tracking_guess.hpp"

/** Superclass for N-series guesses. These guesses work as follows:
    
    1. Create vector \p outcomes_mb that will contain N measurement outcomes. \n
    2. Initialize a variable \p g_mb that will hold the current guess. The
       initial value might for example be 1/2. \n
    3. Add the outcome of the last measurement to \p outcomes_mb. \n
    4. If \p outcomes_mb is full then

       - use this vector to calculate a new value for the current guess and 
         store it in \p g_mb,
       - clear \p outcomes_mb.

    5. Return \p g_mb as the current guess. \n
    6. After a measurement is performed continue at 3 if desired. */
class uniform_tracking_tp::nser_guess_tp : 
    public uniform_tracking_tp::guess_tp 
{
public:
    nser_guess_tp(params_tp &params, const std::string &sec_name2);
    virtual ~nser_guess_tp();
    void new_outcome(int);
	class N_smaller_than_one_error_tp;
protected:
    virtual void update_guess() = 0;
    std::vector<int> outcomes_mb; // Outcomes
    unsigned int N_mb; // Number of measurements in an N-series
    double g_mb; // Current guess
};

class uniform_tracking_tp::nser_guess_tp::
	N_smaller_than_one_error_tp : public std::runtime_error 
{
public:
	N_smaller_than_one_error_tp(const std::string &guess_name) : 
		runtime_error(guess_name+":N is not allowed to be smaller than one.") {}
};

#endif
