#include <mt19937.hpp>
#include "../test-suite/suite.hpp"
#include <iostream>
#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <fstream>
#include <stdexcept>
using namespace std;

/** Taken from main() in the original mt19937ar.c. Only the printf functions
    where changed to print into a file and the MT functions where adapted to the
    C++ interface.  See also the readme_mt.txt in the original mt19937. */
static void test1_main() {
    FILE *out = fopen("test1.out", "w");
    assert(out != 0);
    mt19937_tp mt;
    int i;
    unsigned long init[4]={0x123, 0x234, 0x345, 0x456};
	int length=4;
    mt.init_by_array(init, length);
    fprintf(out, "1000 outputs of genrand_int32()\n");
    for (i=0; i<1000; i++) {
        fprintf(out, "%10lu ", mt.genrand_int32());
        if (i%5==4) fprintf(out, "\n");
    }
    fprintf(out, "\n1000 outputs of genrand_real2()\n");
    for (i=0; i<1000; i++) {
        fprintf(out, "%10.8f ", mt.genrand_real2());
        if (i%5==4) fprintf(out, "\n");
    }
    assert(fclose(out) == 0);
}

/** Verifies that the output by test1_main is identical to the original
    mt19937ar.out. */
static void test1_compare() {
    ifstream orig_stream(TEST_SUITE_3RD_PARTY_DIR "/mt19937/mt19937ar.out");
    assert(orig_stream);
    ifstream test_stream("test1.out");
    assert(test_stream);

    char orig_c, test_c;
    while (orig_stream.get(orig_c)) {
        test_stream.get(test_c);
        assert(orig_c == test_c);
    }
}

/** Test described in readme_mt.txt in the original mt19937. */
static void test1() {
    test1_main();
    test1_compare();
}

int main() {
    vector<test_tp> tests;
    tests.push_back(test1);
    return run_tests(tests, "mt19937");
}
