#include <params.hpp>
#include "../test-suite/suite.hpp"
#include <complex>
#include <iostream>
#include <string>
#include <stdexcept>
#include <cassert>
#include <cstdlib>
using namespace std;

/* Test 1: Error when trying to load a non existent file */
static void test1() {
    try {
        params_tp params(string("non-existing-file"));
        assert(false);
    } catch (const params_tp::open_file_error_tp &e) {
        assert(true);
    }
}

/* Test 2: Loading and displaying some values and production of errors because
   of missing keys and incompatible types to convert to. */
static void test2() {
    params_tp *params = 0;
    params = new params_tp(string("test.in"));

    assert(params->get<string>(string("bla:hihi")) == "huhu");
    assert(params->get<bool>(string("bla:bool1")) == false);
    assert(params->get<bool>(string("bla:bool2")) == true);

    vector<int> v_int_expected;
    v_int_expected.push_back(4);
    v_int_expected.push_back(5);
    v_int_expected.push_back(6);
    vector<int> v_int = params->get_vector<int>(string("bla:an_integer_list"));
    assert(v_int == v_int_expected);

    vector<complex<double> > v_cpx_expected;
    v_cpx_expected.push_back(complex<double>(4,3));
    v_cpx_expected.push_back(complex<double>(5,1));
    v_cpx_expected.push_back(complex<double>(6,7));
    vector<complex<double> > v_cpx = 
        params->get_vector<complex<double> >(string("bla:a_complex_list"));
    assert(v_cpx == v_cpx_expected);
    
    vector<string> v_str_expected;
    v_str_expected.push_back("hi");
    v_str_expected.push_back("ha");
    v_str_expected.push_back("\"ho");
    v_str_expected.push_back("ho\"");
    v_str_expected.push_back("hu");
    vector<string> v_str = 
        params->get_vector<string>(string("bla:a_string_list"));
    assert(v_str == v_str_expected);

    // Default value test:
    assert(params->get<int>(string("bla:non-existent"),-5) == -5);
    assert(params->get<string>(string("blub:non-existent2"), "hello") == 
           "hello");

    try {
        params->get<string>(string("unknown key"));
        assert(false);
    } catch (const params_tp::get_key_error_tp &e) {
        assert(true);
    }

    delete params;
}

int main() {
    vector<test_tp> tests;
    tests.push_back(test1);
    tests.push_back(test2);
    return run_tests(tests, "params");
}
