#include <measurement.hpp>
#include <h2.hpp>
#include <params.hpp>
#include <uniform-tracking.hpp>
#include <uniform-tracking_guess.hpp>
#include <uniform-tracking_aks-guess.hpp>
#include <../test-suite/suite.hpp>
#include <iostream>
#include <cmath>
#include <complex>
#include <exception>
#include <cstdlib>
#include <cassert>
#include <cstring>
#include <fstream>
using namespace std;

typedef complex<double> dx;

/** Some simple tests of error messages, output, etc.. */
static void test1() {
    ostringstream some_stream;

    // N == 0 is too small:
    params_tp params1("test1-1.in");
    try {
        uniform_tracking_tp ut(params1, some_stream, some_stream);
        assert(false);
    } catch (const uniform_tracking_tp::nser_guess_tp::
			 N_smaller_than_one_error_tp &e) 
	{
        assert(true);
    }

    // N == -1 doesn't make sense:
    params_tp params2("test1-2.in");
    try {
        uniform_tracking_tp ut(params2, some_stream, some_stream);
        assert(false);
    } catch (const uniform_tracking_tp::nser_guess_tp::
			 N_smaller_than_one_error_tp &e) 
    {
        assert(true);
    }

    // |M0_00|^2 == |M0_11|^2 is not allowed:
    params_tp params3("test1-3.in");
    try {
        uniform_tracking_tp ut(params3, some_stream, some_stream);
        assert(false);
    } catch (const uniform_tracking_tp::aks_guess_tp::invalid_M_error_tp &e) {
        assert(true);
    }

    // QQQ:  Only one measurement operator:
    params_tp params4("test1-4.in");
    try {
        uniform_tracking_tp ut(params4, some_stream, some_stream);
        assert(false);
    } catch (const uniform_tracking_tp::aks_guess_tp::
             invalid_M_error_tp &e)
    {
        assert(true);
    }

    // Is miscellaneous data correctly dumped?:
    string stream_misc_expected5 = 
        "\n\n[aks-guess]\np_0 = 0.65\np_1 = 0.35\nDelta_p = -0.3\np_bar = 0.5\n";
    params_tp params5("test1-5.in");
    uniform_tracking_tp ut5(params5, some_stream, some_stream);
    ut5.track();
    ostringstream out5;
    out5.precision(5);
    ut5.guess().stream_misc(out5);
    assert(out5.str() == stream_misc_expected5);
}

/** Tests of different parameter combinations. This also involves interactive
    graphical tests where the user has to visually compare results and say
    whether they are roughly similar. */
static void test2() {
    ostringstream some_stream;

    // Intermediate Regime
    ofstream rcrd_stream1("test2-1.rcrd");
    assert(rcrd_stream1);
    mt19937.init_genrand(8);
    params_tp params1("test2-1.in");
    uniform_tracking_tp ut1(params1, some_stream, rcrd_stream1);
    ut1.track();

    // Quantum Jump Regime
    ofstream rcrd_stream2("test2-2.rcrd");
    assert(rcrd_stream2);
    params_tp params2("test2-2.in");
    uniform_tracking_tp ut2(params2, some_stream, rcrd_stream2);
    ut2.track();

    // Rabi Regime
    ofstream rcrd_stream3("test2-3.rcrd");
    assert(rcrd_stream3);
    params_tp params3("test2-3.in");
    uniform_tracking_tp ut3(params3, some_stream, rcrd_stream3);
    ut3.track();

    vector<string> gnuplot_files;
    gnuplot_files.push_back("test2-1.gnuplot");
    gnuplot_files.push_back("test2-2.gnuplot");
    gnuplot_files.push_back("test2-3.gnuplot");
    cout << "\n";
    assert_graphical_outputs_roughly_similar(gnuplot_files);
    cout << "\n";
}

/** Tests of the AKS guess. This is also a test of the \p nser_guess_tp
    superclass of all N-series guesses. */
int main() {
    vector<test_tp> tests;
    tests.push_back(test1);
    tests.push_back(test2);
    return run_tests(tests, "uniform-tracking_aks-guess");
}
