(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     32375,        912]*)
(*NotebookOutlinePosition[     33062,        937]*)
(*  CellTagsIndexPosition[     33018,        933]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[BoxData[{
    RowBox[{
      RowBox[{"facts", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              SubscriptBox["p", "0"], "\[GreaterEqual]", "0"}], ",", 
            RowBox[{
              SubscriptBox["p", "1"], "\[GreaterEqual]", "0"}], ",", 
            RowBox[{
              SubscriptBox["p", "0"], "\[LessEqual]", "1"}], ",", 
            RowBox[{
              SubscriptBox["p", "1"], "\[LessEqual]", "1"}], ",", 
            RowBox[{"\[CurlyPhi]", "\[GreaterEqual]", "0"}], ",", 
            RowBox[{"\[CurlyTheta]", "\[GreaterEqual]", "0"}], ",", 
            RowBox[{"\[CurlyTheta]2", "\[GreaterEqual]", "0"}], ",", 
            RowBox[{
              RowBox[{"\[CurlyTheta]", "/", "2"}], "\[Equal]", 
              "\[CurlyTheta]2"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"complexVars", "=", 
        RowBox[{"{", 
          RowBox[{
            SubscriptBox["pBK", 
              RowBox[{"0", ",", "0"}]], ",", 
            SubscriptBox["pBK", 
              RowBox[{"0", ",", "1"}]], ",", 
            SubscriptBox["pBK", 
              RowBox[{"1", ",", "0"}]], ",", 
            SubscriptBox["pBK", 
              RowBox[{"1", ",", "1"}]]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"simpl", "[", "expr_", "]"}], ":=", 
        RowBox[{"Simplify", "[", 
          RowBox[{
            RowBox[{"ComplexExpand", "[", 
              RowBox[{"expr", ",", "complexVars"}], "]"}], ",", "facts"}], 
          "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"fsimpl", "[", "expr_", "]"}], ":=", 
        RowBox[{"FullSimplify", "[", 
          RowBox[{"ComplexExpand", "[", "expr", "]"}], "]"}]}], 
      ";"}]}], "Input"],

Cell["m=+:", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"subst", "=", 
        RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"pm\[Psi]", "\[Rule]", 
              RowBox[{
                SubsuperscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], "1"], 
                RowBox[{
                  SubscriptBox["p", "i"], 
                  SuperscriptBox[
                    RowBox[{"Abs", "[", 
                      RowBox[{
                        RowBox[{
                          SubscriptBox["c", "0"], 
                          SubscriptBox["pBK", 
                            RowBox[{"i", ",", "0"}]]}], "+", 
                        RowBox[{
                          SubscriptBox["c", "1"], 
                          SubscriptBox["pBK", 
                            RowBox[{"i", ",", "1"}]]}]}], "]"}], "2"]}]}]}], 
            ",", "\[IndentingNewLine]", 
            RowBox[{"c1m2P", "\[Rule]", 
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{
                    FractionBox["1", 
                      SqrtBox["pm\[Psi]"]], 
                    RowBox[{"(", 
                      RowBox[{
                        SubsuperscriptBox["\[Sum]", 
                          RowBox[{"i", "=", "0"}], "1"], 
                        RowBox[{
                          SqrtBox[
                            SubscriptBox["p", "i"]], 
                          RowBox[{"Conjugate", "[", 
                            SubscriptBox["pBK", 
                              RowBox[{"i", ",", "1"}]], "]"}], 
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{
                                  SubscriptBox["c", "0"], 
                                  SubscriptBox["pBK", 
                                    RowBox[{"i", ",", "0"}]]}], "+", 
                                RowBox[{
                                  SubscriptBox["c", "1"], 
                                  SubscriptBox["pBK", 
                                    RowBox[{"i", ",", "1"}]]}]}], ")"}], 
                            "2"]}]}], ")"}]}], "]"}], "2"]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
              SubscriptBox["c", "0"], "\[Rule]", 
              RowBox[{"Cos", "[", 
                RowBox[{"\[CurlyTheta]", "/", "2"}], "]"}]}], ",", 
            RowBox[{
              SubscriptBox["c", "1"], "\[Rule]", 
              RowBox[{
                RowBox[{"Exp", "[", 
                  RowBox[{"\[ImaginaryI]", " ", "\[CurlyPhi]"}], "]"}], 
                RowBox[{"Sin", "[", 
                  RowBox[{"\[CurlyTheta]", "/", "2"}], "]"}]}]}], ",", 
            RowBox[{"\[CurlyTheta]", "\[Rule]", 
              RowBox[{"2", "\[CurlyTheta]2"}]}]}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"pm\[Psi]", 
          SuperscriptBox[
            RowBox[{"(", 
              RowBox[{"gmP", "-", "c1m2P"}], ")"}], "2"]}], "//.", "subst"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"integrand", "=", 
        RowBox[{"simpl", "[", "%", "]"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              SubsuperscriptBox["\[Sum]", 
                RowBox[{"i", "=", "0"}], "1"], 
              RowBox[{
                SubscriptBox["p", "i"], 
                SuperscriptBox[
                  RowBox[{"Abs", "[", 
                    RowBox[{
                      RowBox[{
                        SubscriptBox["pBK", 
                          RowBox[{"i", ",", "0"}]], 
                        RowBox[{"Cos", "[", 
                          FractionBox["\[CurlyTheta]", "2"], "]"}]}], "+", 
                      RowBox[{
                        SubscriptBox["pBK", 
                          RowBox[{"i", ",", "1"}]], 
                        SuperscriptBox["\[ExponentialE]", 
                          RowBox[{"\[ImaginaryI]", " ", "\[CurlyPhi]"}]], 
                        RowBox[{"Sin", "[", 
                          FractionBox["\[CurlyTheta]", "2"], "]"}]}]}], "]"}],
                   "2"]}]}], ")"}], 
          RowBox[{"-", "1"}]], 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                SubsuperscriptBox["\[Sum]", 
                  RowBox[{"i", "=", "0"}], "1"], 
                RowBox[{"gmP", " ", 
                  SubscriptBox["p", "i"], 
                  SuperscriptBox[
                    RowBox[{"Abs", "[", 
                      RowBox[{
                        RowBox[{
                          SubscriptBox["pBK", 
                            RowBox[{"i", ",", "0"}]], 
                          RowBox[{"Cos", "[", 
                            FractionBox["\[CurlyTheta]", "2"], "]"}]}], "+", 
                        RowBox[{
                          SubscriptBox["pBK", 
                            RowBox[{"i", ",", "1"}]], 
                          SuperscriptBox["\[ExponentialE]", 
                            RowBox[{"\[ImaginaryI]", " ", "\[CurlyPhi]"}]], 
                          RowBox[{"Sin", "[", 
                            FractionBox["\[CurlyTheta]", "2"], "]"}]}]}], 
                      "]"}], "2"]}]}], "-", 
              RowBox[{
                RowBox[{"Abs", "[", 
                  RowBox[{
                    SubsuperscriptBox["\[Sum]", 
                      RowBox[{"i", "=", "0"}], "1"], 
                    RowBox[{
                      SqrtBox[
                        SubscriptBox["p", "i"]], 
                      RowBox[{"Conjugate", "[", 
                        SubscriptBox["pBK", 
                          RowBox[{"i", ",", "1"}]], "]"}], 
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              SubscriptBox["pBK", 
                                RowBox[{"i", ",", "0"}]], 
                              RowBox[{"Cos", "[", 
                                FractionBox["\[CurlyTheta]", "2"], "]"}]}], 
                            "+", 
                            RowBox[{
                              SubscriptBox["pBK", 
                                RowBox[{"i", ",", "1"}]], 
                              SuperscriptBox["\[ExponentialE]", 
                                RowBox[{
                                "\[ImaginaryI]", " ", "\[CurlyPhi]"}]], 
                              RowBox[{"Sin", "[", 
                                FractionBox["\[CurlyTheta]", "2"], "]"}]}]}], 
                          ")"}], "2"]}]}], "]"}], "^", "2"}]}], ")"}], "^", 
          "2"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"integrand", "\[Equal]", 
        RowBox[{"simpl", "[", "%", "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"simpl", "[", "%", "]"}]}], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        SubscriptBox["\[PartialD]", "gmP"], "integrand"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"dintegrand", "=", 
        RowBox[{"simpl", "[", "%", "]"}]}], ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(dintegrand\\)\\\" is similar to existing symbol \\\"\\!\\(integrand\
\\)\\\".\"\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"dintegrand", "\[Equal]", 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["\[Sum]", 
                    RowBox[{"i", "=", "0"}], "1"], 
                  RowBox[{"gmP", " ", 
                    SubscriptBox["p", "i"], " ", 
                    SuperscriptBox[
                      RowBox[{"Abs", "[", 
                        SubscriptBox["a", "i"], "]"}], "2"]}]}], "-", 
                RowBox[{
                  RowBox[{"Abs", "[", 
                    RowBox[{
                      SubsuperscriptBox["\[Sum]", 
                        RowBox[{"i", "=", "0"}], "1"], 
                      RowBox[{
                        SqrtBox[
                          SubscriptBox["p", "i"]], 
                        RowBox[{"Conjugate", "[", 
                          SubscriptBox["pBK", 
                            RowBox[{"i", ",", "1"}]], "]"}], 
                        SuperscriptBox[
                          SubscriptBox["a", "i"], "2"]}]}], "]"}], "^", 
                  "2"}]}], ")"}]}]}], "/.", 
        RowBox[{
          SubscriptBox["a", "i_"], "\[Rule]", 
          RowBox[{
            RowBox[{
              SubscriptBox["pBK", 
                RowBox[{"i", ",", "0"}]], 
              RowBox[{"Cos", "[", 
                FractionBox["\[CurlyTheta]", "2"], "]"}]}], "+", 
            RowBox[{
              SubscriptBox["pBK", 
                RowBox[{"i", ",", "1"}]], 
              SuperscriptBox["\[ExponentialE]", 
                RowBox[{"\[ImaginaryI]", " ", "\[CurlyPhi]"}]], 
              RowBox[{"Sin", "[", 
                FractionBox["\[CurlyTheta]", "2"], "]"}]}]}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"simpl", "[", "%", "]"}]}], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate", "[", 
      RowBox[{"dintegrand", ",", 
        RowBox[{"{", 
          RowBox[{"\[CurlyPhi]", ",", "0", ",", 
            RowBox[{"2", "\[Pi]"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["$Aborted"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"i", "=", "0"}], "1"], 
          RowBox[{"gmP", " ", 
            SubscriptBox["p", "i"], " ", 
            SuperscriptBox[
              RowBox[{"Abs", "[", 
                SubscriptBox["a", "i"], "]"}], "2"]}]}], "/.", 
        RowBox[{
          SubscriptBox["a", "i_"], "\[Rule]", 
          RowBox[{
            RowBox[{
              SubscriptBox["pBK", 
                RowBox[{"i", ",", "0"}]], 
              RowBox[{"Cos", "[", 
                FractionBox["\[CurlyTheta]", "2"], "]"}]}], "+", 
            RowBox[{
              SubscriptBox["pBK", 
                RowBox[{"i", ",", "1"}]], 
              SuperscriptBox["\[ExponentialE]", 
                RowBox[{"\[ImaginaryI]", " ", "\[CurlyPhi]"}]], 
              RowBox[{"Sin", "[", 
                FractionBox["\[CurlyTheta]", "2"], "]"}]}]}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"Abs", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Conjugate", "[", 
                SubscriptBox["pBK", 
                  RowBox[{"0", ",", "1"}]], "]"}], 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SubscriptBox["pBK", 
                        RowBox[{"0", ",", "0"}]], " ", 
                      RowBox[{"Cos", "[", "\[CurlyTheta]", "]"}]}], "+", 
                    RowBox[{
                      SubscriptBox["pBK", 
                        RowBox[{"0", ",", "1"}]], " ", 
                      SuperscriptBox["\[ExponentialE]", 
                        RowBox[{"\[ImaginaryI]", " ", "\[CurlyPhi]"}]], " ", 
                      RowBox[{"Sin", "[", "\[CurlyTheta]", "]"}]}]}], ")"}], 
                "2"]}], "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                SubscriptBox["pBK", 
                  RowBox[{"1", ",", "1"}]], "]"}], 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SubscriptBox["pBK", 
                        RowBox[{"1", ",", "0"}]], " ", 
                      RowBox[{"Cos", "[", "\[CurlyTheta]", "]"}]}], "+", 
                    RowBox[{
                      SubscriptBox["pBK", 
                        RowBox[{"1", ",", "1"}]], " ", 
                      SuperscriptBox["\[ExponentialE]", 
                        RowBox[{"\[ImaginaryI]", " ", "\[CurlyPhi]"}]], " ", 
                      RowBox[{"Sin", "[", "\[CurlyTheta]", "]"}]}]}], ")"}], 
                "2"]}]}], "]"}], "^", "2"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"Abs", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "+", 
                  RowBox[{"b", " ", 
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", " ", "p"}], "]"}]}]}], ")"}], "^", "2"}], 
            "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"c", "+", 
                  RowBox[{"d", " ", 
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", " ", "p"}], "]"}]}]}], ")"}], "^", "2"}]}],
           "]"}], "^", "2"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Integrate", "[", 
      RowBox[{"%", ",", 
        RowBox[{"{", 
          RowBox[{"\[CurlyPhi]", ",", "0", ",", 
            RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{
                SuperscriptBox[
                  RowBox[{"Abs", "[", 
                    SubscriptBox["pBK", 
                      RowBox[{"0", ",", "1"}]], "]"}], "2"], "+", 
                SuperscriptBox[
                  RowBox[{"Abs", "[", 
                    SubscriptBox["pBK", 
                      RowBox[{"1", ",", "1"}]], "]"}], "2"]}], "\[Equal]", 
              "1"}], "}"}]}]}], "]"}]}], "Input"],

Cell[BoxData["$Aborted"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "+", 
                  RowBox[{"b", " ", 
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", " ", "p"}], "]"}]}]}], ")"}], "^", "2"}], 
            "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"c", "+", 
                  RowBox[{"d", " ", 
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", " ", "p"}], "]"}]}]}], ")"}], "^", "2"}]}],
           ")"}], 
        RowBox[{"Conjugate", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{"a", "+", 
                  RowBox[{"b", " ", 
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", " ", "p"}], "]"}]}]}], ")"}], "^", "2"}], 
            "+", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"c", "+", 
                  RowBox[{"d", " ", 
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", " ", "p"}], "]"}]}]}], ")"}], "^", "2"}]}],
           "]"}]}], "//", "Expand"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"int", "=", 
        RowBox[{
          RowBox[{"Integrate", "[", 
            RowBox[{"#", ",", 
              RowBox[{"{", 
                RowBox[{"p", ",", "0", ",", 
                  RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], "&"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"int", "/@", "%%"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        SuperscriptBox["a", "2"], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"a", "+", 
              RowBox[{"b", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{
        SuperscriptBox["c", "2"], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"a", "+", 
              RowBox[{"b", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{"2", " ", "a", " ", "b", " ", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", "p"}]], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"a", "+", 
              RowBox[{"b", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{"2", " ", "c", " ", "d", " ", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", "p"}]], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"a", "+", 
              RowBox[{"b", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{
        SuperscriptBox["b", "2"], " ", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "p"}]], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"a", "+", 
              RowBox[{"b", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{
        SuperscriptBox["d", "2"], " ", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "p"}]], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"a", "+", 
              RowBox[{"b", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{
        SuperscriptBox["a", "2"], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"c", "+", 
              RowBox[{"d", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{
        SuperscriptBox["c", "2"], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"c", "+", 
              RowBox[{"d", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{"2", " ", "a", " ", "b", " ", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", "p"}]], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"c", "+", 
              RowBox[{"d", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{"2", " ", "c", " ", "d", " ", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", "p"}]], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"c", "+", 
              RowBox[{"d", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{
        SuperscriptBox["b", "2"], " ", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "p"}]], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"c", "+", 
              RowBox[{"d", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], "2"]}], 
      "+", 
      RowBox[{
        SuperscriptBox["d", "2"], " ", 
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"2", " ", "\[ImaginaryI]", " ", "p"}]], " ", 
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"c", "+", 
              RowBox[{"d", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{"\[ImaginaryI]", " ", "p"}]]}]}], "]"}], 
          "2"]}]}]], "Output"],

Cell[BoxData["$Aborted"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Conjugate", "/@", 
        RowBox[{"(", 
          RowBox[{"a", "+", 
            RowBox[{"b", " ", 
              RowBox[{"Exp", "[", 
                RowBox[{"I", " ", "p"}], "]"}]}]}], ")"}]}], "//", 
      RowBox[{
        RowBox[{"Simplify", "[", 
          RowBox[{"#", ",", 
            RowBox[{"p", "\[GreaterEqual]", "0"}]}], "]"}], 
        "&"}]}], "\[IndentingNewLine]", 
    RowBox[{"Integrate", "[", 
      RowBox[{"%", ",", 
        RowBox[{"{", 
          RowBox[{"p", ",", "0", ",", 
            RowBox[{"2", "\[Pi]"}]}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Conjugate", "[", "a", "]"}], "+", 
      RowBox[{
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{
            RowBox[{"-", "\[ImaginaryI]"}], " ", "p"}]], " ", 
        RowBox[{"Conjugate", "[", "b", "]"}]}]}]], "Output"],

Cell[BoxData[
    RowBox[{"2", " ", "\[Pi]", " ", 
      RowBox[{"Conjugate", "[", "a", "]"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate", "[", " ", 
      RowBox[{
        RowBox[{
          RowBox[{"(", 
            RowBox[{"Conjugate", "/@", 
              RowBox[{"(", 
                RowBox[{"a", "+", 
                  RowBox[{"b", " ", 
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", " ", "p"}], "]"}]}]}], ")"}]}], ")"}], "^",
           "2"}], ",", 
        RowBox[{"{", 
          RowBox[{"p", ",", "0", ",", 
            RowBox[{"2", "Pi"}]}], "}"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"p", "\[GreaterEqual]", "0"}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"Integrate", "[", 
      RowBox[{
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              RowBox[{"Conjugate", "[", "a", "]"}], "+", 
              RowBox[{
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{
                    RowBox[{"-", "\[ImaginaryI]"}], " ", 
                    RowBox[{"Conjugate", "[", "p", "]"}]}]], " ", 
                RowBox[{"Conjugate", "[", "b", "]"}]}]}], ")"}], "2"], ",", 
        RowBox[{"{", 
          RowBox[{"p", ",", "0", ",", 
            RowBox[{"2", " ", "\[Pi]"}]}], "}"}], ",", 
        RowBox[{"Assumptions", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"p", "\[GreaterEqual]", "0"}], "}"}]}]}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"dvmPPre", "=", 
      RowBox[{"Integrate", "[", 
        RowBox[{"dintegrand", ",", 
          RowBox[{"{", 
            RowBox[{"\[CurlyPhi]", ",", "0", ",", 
              RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
          RowBox[{"Assumptions", "\[Rule]", "facts"}]}], "]"}]}]], "Input"],

Cell[BoxData["$Aborted"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"vmPPreAlt", "=", 
      RowBox[{"Integrate", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"Sin", "[", "\[CurlyTheta]", "]"}], "integrand"}], ",", 
          RowBox[{"{", 
            RowBox[{"\[CurlyTheta]", ",", "0", ",", "\[Pi]"}], "}"}], ",", 
          RowBox[{"Assumptions", "\[Rule]", "facts"}]}], "]"}]}]], "Input"],

Cell[BoxData["$Aborted"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"vmP", "=", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"4", "\[Pi]"}]], 
        RowBox[{"Integrate", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Sin", "[", "\[CurlyTheta]", "]"}], "vmpPre"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[CurlyTheta]", ",", "0", ",", "\[Pi]"}], "}"}], ",", 
            RowBox[{"Assumptions", "\[Rule]", "facts"}]}], "]"}]}]}]], "Input"],

Cell[BoxData["$Aborted"], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"vmP", "=", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"vmP", ",", "facts"}], "]"}], "&"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"dgmvmP", "=", 
      RowBox[{
        RowBox[{
          SubscriptBox["\[PartialD]", "gmP"], "vmP"}], "//", 
        RowBox[{
          RowBox[{"FullSimplify", "[", 
            RowBox[{"#", ",", "facts"}], "]"}], "&"}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(dgmvmP\\)\\\" is similar to existing symbol \\\"\\!\\(dgmvm\\)\\\".\
\"\>"}]], "Message"],

Cell[BoxData[
    RowBox[{
      RowBox[{"-", "p1"}], "+", 
      RowBox[{"gmP", " ", 
        RowBox[{"(", 
          RowBox[{"p0", "+", "p1"}], ")"}]}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"Solve", "[", 
          RowBox[{
            RowBox[{"dgmvmP", "\[Equal]", "0"}], ",", "gmP"}], "]"}], "//", 
        RowBox[{
          RowBox[{"FullSimplify", "[", 
            RowBox[{"#", ",", "facts"}], "]"}], "&"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"gmP", "/.", "%"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      FractionBox["p1", 
        RowBox[{"p0", "+", "p1"}]], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"Integrate", "[", 
      RowBox[{
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"a", "+", 
              RowBox[{"b", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{
                    RowBox[{"-", "\[ImaginaryI]"}], " ", "\[CurlyPhi]"}]]}]}],
             "]"}], "2"], ",", 
        RowBox[{"{", 
          RowBox[{"\[CurlyPhi]", ",", "0", ",", 
            RowBox[{"2", "\[Pi]"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{
      SubsuperscriptBox["\[Integral]", "0", 
        RowBox[{"2", " ", "\[Pi]"}]], 
      RowBox[{
        SuperscriptBox[
          RowBox[{"Conjugate", "[", 
            RowBox[{"a", "+", 
              RowBox[{"b", " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{
                    RowBox[{"-", "\[ImaginaryI]"}], " ", "\[CurlyPhi]"}]]}]}],
             "]"}], "2"], 
        RowBox[{"\[DifferentialD]", "\[CurlyPhi]"}]}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"Conjugate", "/@", 
      RowBox[{"(", 
        RowBox[{"a", "+", 
          RowBox[{"b", " ", 
            SuperscriptBox["\[ExponentialE]", 
              RowBox[{
                RowBox[{"-", "\[ImaginaryI]"}], " ", "\[CurlyPhi]"}]]}]}], 
        ")"}]}], "\[IndentingNewLine]", 
    RowBox[{"Simplify", "[", 
      RowBox[{"%", ",", 
        RowBox[{"\[CurlyPhi]", "\[GreaterEqual]", "0"}]}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"Integrate", "[", 
      RowBox[{
        SuperscriptBox["%", "2"], ",", 
        RowBox[{"{", 
          RowBox[{"\[CurlyPhi]", ",", "0", ",", 
            RowBox[{"2", "\[Pi]"}]}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Conjugate", "[", "a", "]"}], "+", 
      RowBox[{
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", 
            RowBox[{"Conjugate", "[", "\[CurlyPhi]", "]"}]}]], " ", 
        RowBox[{"Conjugate", "[", "b", "]"}]}]}]], "Output"],

Cell[BoxData[
    RowBox[{
      RowBox[{"Conjugate", "[", "a", "]"}], "+", 
      RowBox[{
        SuperscriptBox["\[ExponentialE]", 
          RowBox[{"\[ImaginaryI]", " ", "\[CurlyPhi]"}]], " ", 
        RowBox[{"Conjugate", "[", "b", "]"}]}]}]], "Output"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"4", " ", "\[Pi]", " ", 
              SuperscriptBox[
                RowBox[{"Conjugate", "[", "a", "]"}], "2"]}], "-", 
            RowBox[{"4", " ", "\[ImaginaryI]", " ", 
              RowBox[{"Conjugate", "[", "a", "]"}], " ", 
              RowBox[{"Conjugate", "[", "b", "]"}]}], "-", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox[
                RowBox[{"Conjugate", "[", "b", "]"}], "2"]}]}], ")"}]}], "+", 
      
      RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{"(", 
          RowBox[{
            RowBox[{"4", " ", "\[ImaginaryI]", " ", 
              RowBox[{"Conjugate", "[", "a", "]"}], " ", 
              RowBox[{"Conjugate", "[", "b", "]"}]}], "+", 
            RowBox[{"\[ImaginaryI]", " ", 
              SuperscriptBox[
                RowBox[{"Conjugate", "[", "b", "]"}], "2"]}]}], 
          ")"}]}]}]], "Output"]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowToolbars->"EditBar",
WindowSize->{1025, 740},
WindowMargins->{{-4, Automatic}, {Automatic, 0}},
Magnification->1.25
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 1782, 45, 115, "Input"],
Cell[3502, 96, 20, 0, 39, "Text"],
Cell[3525, 98, 2874, 64, 169, "Input"],
Cell[6402, 164, 347, 10, 74, "Input"],

Cell[CellGroupData[{
Cell[6774, 178, 3772, 84, 216, "Input"],
Cell[10549, 264, 31, 0, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10617, 269, 248, 7, 74, "Input"],
Cell[10868, 278, 267, 6, 47, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11172, 289, 1844, 45, 135, "Input"],
Cell[13019, 336, 31, 0, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13087, 341, 213, 5, 52, "Input"],
Cell[13303, 348, 35, 0, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13375, 353, 4063, 102, 156, "Input"],
Cell[17441, 457, 35, 0, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17513, 462, 1562, 44, 77, "Input"],
Cell[19078, 508, 4641, 121, 106, "Output"],
Cell[23722, 631, 35, 0, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23794, 636, 608, 17, 56, "Input"],
Cell[24405, 655, 277, 7, 36, "Output"],
Cell[24685, 664, 107, 2, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24829, 671, 644, 17, 35, "Input"],
Cell[25476, 690, 753, 18, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26266, 713, 315, 7, 35, "Input"],
Cell[26584, 722, 35, 0, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26656, 727, 366, 8, 35, "Input"],
Cell[27025, 737, 35, 0, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27097, 742, 450, 11, 57, "Input"],
Cell[27550, 755, 35, 0, 35, "Output"]
}, Open  ]],
Cell[27600, 758, 158, 4, 35, "Input"],

Cell[CellGroupData[{
Cell[27783, 766, 261, 7, 36, "Input"],
Cell[28047, 775, 259, 6, 47, "Message"],
Cell[28309, 783, 167, 5, 35, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28513, 793, 364, 10, 56, "Input"],
Cell[28880, 805, 110, 3, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29027, 813, 508, 13, 40, "Input"],
Cell[29538, 828, 502, 13, 55, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[30077, 846, 683, 18, 83, "Input"],
Cell[30763, 866, 305, 7, 36, "Output"],
Cell[31071, 875, 259, 6, 36, "Output"],
Cell[31333, 883, 1026, 26, 89, "Output"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

