(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    235760,       5385]*)
(*NotebookOutlinePosition[    237112,       5426]*)
(*  CellTagsIndexPosition[    237068,       5422]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Tools for the generation of the SV table and the corresponding plot\
\
\>", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
Disturbance and postestimation deviation according to 0303111934 \
and 0303122046\
\>", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"s", "[", 
          RowBox[{"p0_", ",", "p1_", ",", "\[Beta]20_"}], "]"}], ":=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"p0", "\[Equal]", "p1"}], "\[Or]", 
              RowBox[{"p0", "\[Equal]", "0"}], "\[Or]", 
              RowBox[{"p0", "\[Equal]", "1"}], "\[Or]", 
              RowBox[{"p1", "\[Equal]", "0"}], "\[Or]", 
              RowBox[{"p1", "\[Equal]", "1"}]}], ",", 
            RowBox[{"-", "1"}], ",", 
            RowBox[{
              RowBox[{
                RowBox[{
                  FractionBox["1", "12"], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", "1"}], "+", 
                            SubscriptBox["p", "1"]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", "3"}], "+", 
                            RowBox[{"2", " ", 
                              SubscriptBox["\[Beta]2", "0"]}]}], ")"}]}], "-",
                       
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", "1"}], "+", 
                            SubscriptBox["p", "0"]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{"1", "+", 
                            RowBox[{"2", " ", 
                              SubscriptBox["\[Beta]2", "0"]}]}], ")"}]}], "+",
                       
                      RowBox[{"4", " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", "2"}], "+", 
                            RowBox[{
                              SubscriptBox["p", "1"], " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"-", "2"}], "+", 
                                  SubscriptBox["\[Beta]2", "0"]}], ")"}], " ",
                               
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"-", "1"}], "+", 
                                  SubscriptBox["\[Beta]2", "0"]}], ")"}]}], 
                            "+", 
                            RowBox[{
                              SubscriptBox["\[Beta]2", "0"], " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"2", " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SqrtBox[
                                        RowBox[{
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "0"]}], ")"}], " ", 
                                        
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "1"]}], ")"}]}]]}], 
                                      ")"}], " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["\[Beta]2", "0"]}], 
                                      ")"}]}], "+", 
                                  RowBox[{
                                    SubscriptBox["p", "0"], " ", 
                                    RowBox[{"(", 
                                      RowBox[{"1", "+", 
                                        SubscriptBox["\[Beta]2", "0"]}], 
                                      ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
                      RowBox[{
                        FractionBox["1", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{"-", 
                                  SubscriptBox["p", "0"]}], "+", 
                                SubscriptBox["p", "1"]}], ")"}], "3"]], 
                        RowBox[{"(", 
                          RowBox[{"6", " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{
                                  RowBox[{"-", 
                                    SuperscriptBox[
                                      RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "1"]}], ")"}], 
                                      "4"]}], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["\[Beta]2", "0"]}], 
                                      ")"}], "2"]}], "+", 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "0"]}], ")"}], "4"],
                                   " ", 
                                  SubsuperscriptBox["\[Beta]2", "0", "2"]}], 
                                "+", 
                                RowBox[{"4", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      SubscriptBox["p", "0"]}], ")"}], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "1"]}], ")"}], "3"],
                                   " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      SubscriptBox["\[Beta]2", "0"]}], ")"}], 
                                  " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      RowBox[{"2", " ", 
                                        SubscriptBox["\[Beta]2", "0"]}]}], 
                                    ")"}]}], "-", 
                                RowBox[{"4", " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "0"]}], ")"}], "3"],
                                   " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      SubscriptBox["p", "1"]}], ")"}], " ", 
                                  SubscriptBox["\[Beta]2", "0"], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      RowBox[{"2", " ", 
                                        SubscriptBox["\[Beta]2", "0"]}]}], 
                                    ")"}]}], "+", 
                                RowBox[{
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "0"]}], ")"}], "2"],
                                   " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "1"]}], ")"}], "2"],
                                   " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "3"}], "+", 
                                      RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        RowBox[{"1", "-", 
                                        SubscriptBox["p", "0"]}], "]"}]}], 
                                      "-", 
                                      RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        RowBox[{"1", "-", 
                                        SubscriptBox["p", "1"]}], "]"}]}], 
                                      "+", 
                                      RowBox[{
                                        SubscriptBox["\[Beta]2", "0"], " ", 
                                        RowBox[{"(", 
                                        RowBox[{"6", "+", 
                                        RowBox[{"Log", "[", 
                                        FractionBox["1", 
                                        SuperscriptBox[
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "0"]}], ")"}], 
                                        "12"]], "]"}], "+", 
                                        RowBox[{"Log", "[", 
                                        SuperscriptBox[
                                        RowBox[{"(", 
                                        RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "1"]}], ")"}], 
                                        "12"], "]"}], "+", 
                                        RowBox[{"12", " ", 
                                        RowBox[{"Log", "[", 
                                        FractionBox[
                                        RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "0"]}], 
                                        RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["p", "1"]}]], "]"}], " ",
                                         
                                        SubscriptBox["\[Beta]2", "0"]}]}], 
                                        ")"}]}]}], ")"}]}]}], ")"}]}], 
                          ")"}]}]}], ")"}]}], "+", 
                RowBox[{
                  FractionBox["1", "12"], " ", 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{
                        SubscriptBox["p", "1"], " ", 
                        RowBox[{"(", 
                          RowBox[{"3", "-", 
                            RowBox[{"2", " ", 
                              SubscriptBox["\[Beta]2", "0"]}]}], ")"}]}], "+",
                       
                      RowBox[{
                        SubscriptBox["p", "0"], " ", 
                        RowBox[{"(", 
                          RowBox[{"1", "+", 
                            RowBox[{"2", " ", 
                              SubscriptBox["\[Beta]2", "0"]}]}], ")"}]}], "+",
                       
                      RowBox[{"4", " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              RowBox[{"-", 
                                SubscriptBox["p", "1"]}], " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"-", "2"}], "+", 
                                  SubscriptBox["\[Beta]2", "0"]}], ")"}], " ",
                               
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"-", "1"}], "+", 
                                  SubscriptBox["\[Beta]2", "0"]}], ")"}]}], 
                            "+", 
                            RowBox[{
                              SubscriptBox["\[Beta]2", "0"], " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"2", " ", 
                                    SqrtBox[
                                      RowBox[{
                                        SubscriptBox["p", "0"], " ", 
                                        SubscriptBox["p", "1"]}]], " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["\[Beta]2", "0"]}], 
                                      ")"}]}], "-", 
                                  RowBox[{
                                    SubscriptBox["p", "0"], " ", 
                                    RowBox[{"(", 
                                      RowBox[{"1", "+", 
                                        SubscriptBox["\[Beta]2", "0"]}], 
                                      ")"}]}]}], ")"}]}]}], ")"}]}], "+", 
                      RowBox[{
                        FractionBox["1", 
                          SuperscriptBox[
                            RowBox[{"(", 
                              RowBox[{
                                SubscriptBox["p", "0"], "-", 
                                SubscriptBox["p", "1"]}], ")"}], "3"]], 
                        RowBox[{"(", 
                          RowBox[{"6", " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{
                                  RowBox[{"-", 
                                    SubsuperscriptBox["p", "1", "4"]}], " ", 
                                  SuperscriptBox[
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["\[Beta]2", "0"]}], 
                                      ")"}], "2"]}], "+", 
                                RowBox[{"4", " ", 
                                  SubsuperscriptBox["p", "0", "3"], " ", 
                                  SubscriptBox["p", "1"], " ", 
                                  RowBox[{"(", 
                                    RowBox[{"1", "-", 
                                      RowBox[{"2", " ", 
                                        SubscriptBox["\[Beta]2", "0"]}]}], 
                                    ")"}], " ", 
                                  SubscriptBox["\[Beta]2", "0"]}], "+", 
                                RowBox[{
                                  SubsuperscriptBox["p", "0", "4"], " ", 
                                  SubsuperscriptBox["\[Beta]2", "0", "2"]}], 
                                "+", 
                                RowBox[{"4", " ", 
                                  SubscriptBox["p", "0"], " ", 
                                  SubsuperscriptBox["p", "1", "3"], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      SubscriptBox["\[Beta]2", "0"]}], ")"}], 
                                  " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      RowBox[{"2", " ", 
                                        SubscriptBox["\[Beta]2", "0"]}]}], 
                                    ")"}]}], "+", 
                                RowBox[{
                                  SubsuperscriptBox["p", "0", "2"], " ", 
                                  SubsuperscriptBox["p", "1", "2"], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "3"}], "+", 
                                      RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        SubscriptBox["p", "0"], "]"}]}], "-", 
                                      
                                      RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        SubscriptBox["p", "1"], "]"}]}], "+", 
                                      
                                      RowBox[{"6", " ", 
                                        SubscriptBox["\[Beta]2", "0"], " ", 
                                        RowBox[{"(", 
                                        RowBox[{"1", "-", 
                                        RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        SubscriptBox["p", "0"], "]"}]}], "+", 
                                        
                                        RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        SubscriptBox["p", "1"], "]"}]}], "+", 
                                        
                                        RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        FractionBox[
                                        SubscriptBox["p", "0"], 
                                        SubscriptBox["p", "1"]], "]"}], " ", 
                                        SubscriptBox["\[Beta]2", "0"]}]}], 
                                        ")"}]}]}], ")"}]}]}], ")"}]}], 
                          ")"}]}]}], ")"}]}]}], "/.", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[Beta]2", "0"], "\[Rule]", "\[Beta]20"}], 
                  ",", 
                  RowBox[{
                    SubscriptBox["p", "0"], "\[Rule]", "p0"}], ",", 
                  RowBox[{
                    SubscriptBox["p", "1"], "\[Rule]", "p1"}]}], "}"}]}]}], 
          "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"vp", "[", 
          RowBox[{"p0_", ",", "p1_", ",", "\[Beta]20_"}], "]"}], ":=", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"p0", "\[Equal]", "p1"}], "\[Or]", 
              RowBox[{"p0", "\[Equal]", "0"}], "\[Or]", 
              RowBox[{"p0", "\[Equal]", "1"}], "\[Or]", 
              RowBox[{"p1", "\[Equal]", "0"}], "\[Or]", 
              RowBox[{"p1", "\[Equal]", "1"}]}], ",", 
            RowBox[{"-", "1"}], ",", "\[IndentingNewLine]", 
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{"-", 
                    RowBox[{"(", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{"1", "-", 
                            SubscriptBox["p", "0"]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{"1", "-", 
                            SubscriptBox["p", "1"]}], ")"}], " ", 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{
                              SuperscriptBox[
                                RowBox[{"(", 
                                  RowBox[{
                                    RowBox[{"-", "1"}], "+", 
                                    SubscriptBox["p", "0"]}], ")"}], "3"], 
                              " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"-", "1"}], "+", 
                                  SubscriptBox["\[Beta]2", "0"]}], ")"}], " ",
                               
                              SubscriptBox["\[Beta]2", "0"]}], "-", 
                            RowBox[{
                              SuperscriptBox[
                                RowBox[{"(", 
                                  RowBox[{
                                    RowBox[{"-", "1"}], "+", 
                                    SubscriptBox["p", "1"]}], ")"}], "3"], 
                              " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"-", "1"}], "+", 
                                  SubscriptBox["\[Beta]2", "0"]}], ")"}], " ",
                               
                              SubscriptBox["\[Beta]2", "0"]}], "+", 
                            RowBox[{
                              SuperscriptBox[
                                RowBox[{"(", 
                                  RowBox[{
                                    RowBox[{"-", "1"}], "+", 
                                    SubscriptBox["p", "0"]}], ")"}], "2"], 
                              " ", 
                              RowBox[{"(", 
                                RowBox[{"1", "-", 
                                  SubscriptBox["p", "1"]}], ")"}], " ", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{"-", "2"}], "+", 
                                  RowBox[{"Log", "[", 
                                    RowBox[{"1", "-", 
                                      SubscriptBox["p", "0"]}], "]"}], "-", 
                                  RowBox[{"Log", "[", 
                                    RowBox[{"1", "-", 
                                      SubscriptBox["p", "1"]}], "]"}], "+", 
                                  RowBox[{"3", " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "3"}], "+", 
                                        RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        RowBox[{"1", "-", 
                                        SubscriptBox["p", "0"]}], "]"}]}], 
                                        "-", 
                                        RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        RowBox[{"1", "-", 
                                        SubscriptBox["p", "1"]}], "]"}]}]}], 
                                      ")"}], " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["\[Beta]2", "0"]}], 
                                      ")"}], " ", 
                                    SubscriptBox["\[Beta]2", "0"]}]}], 
                                ")"}]}], "+", 
                            RowBox[{
                              RowBox[{"(", 
                                RowBox[{"1", "-", 
                                  SubscriptBox["p", "0"]}], ")"}], " ", 
                              SuperscriptBox[
                                RowBox[{"(", 
                                  RowBox[{
                                    RowBox[{"-", "1"}], "+", 
                                    SubscriptBox["p", "1"]}], ")"}], "2"], 
                              " ", 
                              RowBox[{"(", 
                                RowBox[{"2", "+", 
                                  RowBox[{"Log", "[", 
                                    RowBox[{"1", "-", 
                                      SubscriptBox["p", "0"]}], "]"}], "-", 
                                  RowBox[{"Log", "[", 
                                    RowBox[{"1", "-", 
                                      SubscriptBox["p", "1"]}], "]"}], "+", 
                                  RowBox[{"3", " ", 
                                    RowBox[{"(", 
                                      RowBox[{"3", "+", 
                                        RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        RowBox[{"1", "-", 
                                        SubscriptBox["p", "0"]}], "]"}]}], 
                                        "-", 
                                        RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        RowBox[{"1", "-", 
                                        SubscriptBox["p", "1"]}], "]"}]}]}], 
                                      ")"}], " ", 
                                    RowBox[{"(", 
                                      RowBox[{
                                        RowBox[{"-", "1"}], "+", 
                                        SubscriptBox["\[Beta]2", "0"]}], 
                                      ")"}], " ", 
                                    SubscriptBox["\[Beta]2", "0"]}]}], 
                                ")"}]}]}], ")"}]}], ")"}]}], "/", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"-", 
                              SubscriptBox["p", "0"]}], "+", 
                            SubscriptBox["p", "1"]}], ")"}], "3"], " ", 
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"-", "2"}], "+", 
                          SubscriptBox["p", "0"], "+", 
                          SubscriptBox["p", "1"]}], ")"}]}], ")"}]}], "+", 
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      SubscriptBox["p", "0"], " ", 
                      SubscriptBox["p", "1"], " ", 
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{
                            RowBox[{"-", 
                              SubsuperscriptBox["p", "0", "3"]}], " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{"-", "1"}], "+", 
                                SubscriptBox["\[Beta]2", "0"]}], ")"}], " ", 
                            SubscriptBox["\[Beta]2", "0"]}], "+", 
                          RowBox[{
                            SubsuperscriptBox["p", "1", "3"], " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{"-", "1"}], "+", 
                                SubscriptBox["\[Beta]2", "0"]}], ")"}], " ", 
                            SubscriptBox["\[Beta]2", "0"]}], "+", 
                          RowBox[{
                            SubsuperscriptBox["p", "0", "2"], " ", 
                            SubscriptBox["p", "1"], " ", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{"-", "2"}], "+", 
                                RowBox[{"Log", "[", 
                                  SubscriptBox["p", "0"], "]"}], "-", 
                                RowBox[{"Log", "[", 
                                  SubscriptBox["p", "1"], "]"}], "+", 
                                RowBox[{"3", " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "3"}], "+", 
                                      RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        SubscriptBox["p", "0"], "]"}]}], "-", 
                                      
                                      RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        SubscriptBox["p", "1"], "]"}]}]}], 
                                    ")"}], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      SubscriptBox["\[Beta]2", "0"]}], ")"}], 
                                  " ", 
                                  SubscriptBox["\[Beta]2", "0"]}]}], ")"}]}], 
                          "+", 
                          RowBox[{
                            SubscriptBox["p", "0"], " ", 
                            SubsuperscriptBox["p", "1", "2"], " ", 
                            RowBox[{"(", 
                              RowBox[{"2", "+", 
                                RowBox[{"Log", "[", 
                                  SubscriptBox["p", "0"], "]"}], "-", 
                                RowBox[{"Log", "[", 
                                  SubscriptBox["p", "1"], "]"}], "+", 
                                RowBox[{"3", " ", 
                                  RowBox[{"(", 
                                    RowBox[{"3", "+", 
                                      RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        SubscriptBox["p", "0"], "]"}]}], "-", 
                                      
                                      RowBox[{"2", " ", 
                                        RowBox[{"Log", "[", 
                                        SubscriptBox["p", "1"], "]"}]}]}], 
                                    ")"}], " ", 
                                  RowBox[{"(", 
                                    RowBox[{
                                      RowBox[{"-", "1"}], "+", 
                                      SubscriptBox["\[Beta]2", "0"]}], ")"}], 
                                  " ", 
                                  SubscriptBox["\[Beta]2", "0"]}]}], 
                              ")"}]}]}], ")"}]}], ")"}], "/", 
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{
                            SubscriptBox["p", "0"], "-", 
                            SubscriptBox["p", "1"]}], ")"}], "3"], " ", 
                      RowBox[{"(", 
                        RowBox[{
                          SubscriptBox["p", "0"], "+", 
                          SubscriptBox["p", "1"]}], ")"}]}], ")"}]}]}], "/.", 
              
              RowBox[{"{", 
                RowBox[{
                  RowBox[{
                    SubscriptBox["\[Beta]2", "0"], "\[Rule]", "\[Beta]20"}], 
                  ",", 
                  RowBox[{
                    SubscriptBox["p", "0"], "\[Rule]", "p0"}], ",", 
                  RowBox[{
                    SubscriptBox["p", "1"], "\[Rule]", "p1"}]}], "}"}]}]}], 
          "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["SV table creation tools", "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tableWidth", "[", "table_", "]"}], ":=", 
        RowBox[{
          RowBox[{"Dimensions", "[", 
            RowBox[{"table", ",", "2"}], "]"}], "\[LeftDoubleBracket]", "2", 
          "\[RightDoubleBracket]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"tableHeight", "[", "table_", "]"}], ":=", 
        RowBox[{
          RowBox[{"Dimensions", "[", 
            RowBox[{"table", ",", "2"}], "]"}], "\[LeftDoubleBracket]", "1", 
          "\[RightDoubleBracket]"}]}], ";"}]}], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tableI", "[", 
          RowBox[{"y_", ",", "table_"}], "]"}], ":=", 
        RowBox[{"Ceiling", "[", 
          RowBox[{
            RowBox[{"Min", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"y", "/", 
                    RowBox[{"tableHeight", "[", "table", "]"}]}], "*", 
                  "1000"}], ",", "1"}], "]"}], "*", 
            RowBox[{"tableHeight", "[", "table", "]"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"tableJ", "[", 
          RowBox[{"x_", ",", "table_"}], "]"}], ":=", 
        RowBox[{"Ceiling", "[", 
          RowBox[{
            RowBox[{"Min", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"x", "/", 
                    RowBox[{"tableWidth", "[", "table", "]"}]}], "*", 
                  "1000"}], ",", "1"}], "]"}], "*", 
            RowBox[{"tableWidth", "[", "table", "]"}]}], "]"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"appendInTable", "[", 
        RowBox[{"i_", ",", "j_", ",", "a_", ",", "table_"}], "]"}], ":=", 
      RowBox[{"ReplacePart", "[", 
        RowBox[{"table", ",", 
          RowBox[{"ReplacePart", "[", 
            RowBox[{
              RowBox[{
              "table", "\[LeftDoubleBracket]", "i", "\[RightDoubleBracket]"}],
               ",", 
              RowBox[{"Append", "[", 
                RowBox[{
                  RowBox[{
                    RowBox[{
                    "table", "\[LeftDoubleBracket]", "i", 
                      "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", "j", 
                    "\[RightDoubleBracket]"}], ",", "a"}], "]"}], ",", "j"}], 
            "]"}], ",", "i"}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"insertIntoTable", "[", 
          RowBox[{"p0_", ",", "p1_", ",", "\[Beta]20_", ",", "table_"}], 
          "]"}], ":=", 
        RowBox[{"With", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"vp", "=", 
                  RowBox[{"vp", "[", 
                    RowBox[{"p0", ",", "p1", ",", "\[Beta]20"}], "]"}]}], ",",
                 
                RowBox[{"sp", "=", 
                  RowBox[{"s", "[", 
                    RowBox[{"p0", ",", "p1", ",", "\[Beta]20"}], "]"}]}]}], 
              "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"vp", ">", "0"}], "&&", 
                  RowBox[{"sp", ">", "0"}]}], ",", 
                RowBox[{"appendInTable", "[", 
                  RowBox[{
                    RowBox[{"tableI", "[", 
                      RowBox[{"vp", ",", "table"}], "]"}], ",", 
                    RowBox[{"tableJ", "[", 
                      RowBox[{"sp", ",", "table"}], "]"}], ",", 
                    RowBox[{"{", 
                      RowBox[{"p0", ",", "p1", ",", "\[Beta]20"}], "}"}], ",",
                     "table"}], "]"}], ",", "table"}], "]"}]}], "]"}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"fillTableVarP0", "[", 
        RowBox[{"\[Beta]20_", ",", "p1_", ",", 
          RowBox[{"{", 
            RowBox[{"p0_", ",", "p0Max_", ",", "p0\[CapitalDelta]_"}], "}"}], 
          ",", "table_"}], "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"p0", "\[LessEqual]", "p0Max"}], ",", 
          RowBox[{"fillTableVarP0", "[", 
            RowBox[{"\[Beta]20", ",", "p1", ",", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"p0", "+", "p0\[CapitalDelta]"}], ",", "p0Max", ",",
                   "p0\[CapitalDelta]"}], "}"}], ",", 
              RowBox[{"insertIntoTable", "[", 
                RowBox[{"p0", ",", "p1", ",", "\[Beta]20", ",", "table"}], 
                "]"}]}], "]"}], ",", "table"}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"fillTableVarP1P0", "[", 
        RowBox[{"\[Beta]20_", ",", 
          RowBox[{"{", 
            RowBox[{"p1_", ",", "p1Max_", ",", "p1\[CapitalDelta]_"}], "}"}], 
          ",", 
          RowBox[{"{", 
            RowBox[{"p0_", ",", "p0Max_", ",", "p0\[CapitalDelta]_"}], "}"}], 
          ",", "table_"}], "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"p1", "\[LessEqual]", "p1Max"}], ",", 
          RowBox[{"fillTableVarP1P0", "[", 
            RowBox[{"\[Beta]20", ",", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"p1", "+", "p1\[CapitalDelta]"}], ",", "p1Max", ",",
                   "p1\[CapitalDelta]"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"p0", ",", "p0Max", ",", "p0\[CapitalDelta]"}], "}"}],
               ",", 
              RowBox[{"fillTableVarP0", "[", 
                RowBox[{"\[Beta]20", ",", "p1", ",", 
                  RowBox[{"{", 
                    RowBox[{"p0", ",", 
                      RowBox[{"Min", "[", 
                        RowBox[{"p1", ",", "p0Max"}], "]"}], ",", 
                      "p0\[CapitalDelta]"}], "}"}], ",", "table"}], "]"}]}], 
            "]"}], ",", "table"}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"fillTableVar\[Beta]20P1P0", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
            "\[Beta]20_", ",", "\[Beta]20Max_", ",", 
              "\[Beta]20\[CapitalDelta]_"}], "}"}], ",", "p1VarData_", ",", 
          "p0VarData_", ",", "table_"}], "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"\[Beta]20", "\[LessEqual]", "\[Beta]20Max"}], ",", 
          RowBox[{"fillTableVar\[Beta]20P1P0", "[", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"\[Beta]20", "+", "\[Beta]20\[CapitalDelta]"}], ",",
                   "\[Beta]20Max", ",", "\[Beta]20\[CapitalDelta]"}], "}"}], 
              ",", "p1VarData", ",", "p0VarData", ",", 
              RowBox[{"fillTableVarP1P0", "[", 
                RowBox[{
                "\[Beta]20", ",", "p1VarData", ",", "p0VarData", ",", 
                  "table"}], "]"}]}], "]"}], ",", "table"}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["SV table processing tools", "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"markCellEntriesInRow", "[", "row_", "]"}], ":=", 
        RowBox[{
          RowBox[{
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"Length", "[", "#", "]"}], ">", "0"}], ",", "0", 
                ",", "1"}], "]"}], "&"}], "/@", "row"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"markCellEntries", "[", "table_", "]"}], ":=", 
        RowBox[{"markCellEntriesInRow", "/@", "table"}]}], ";"}]}], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"leftMostNonEmptyCell", "[", "row_", "]"}], ":=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"Length", "[", "row", "]"}], ">", "0"}], ",", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"Length", "[", 
                    RowBox[{"First", "[", "row", "]"}], "]"}], ">", "0"}], 
                ",", 
                RowBox[{"First", "[", "row", "]"}], ",", 
                RowBox[{"leftMostNonEmptyCell", "[", 
                  RowBox[{"Rest", "[", "row", "]"}], "]"}]}], "]"}], ",", 
            RowBox[{"{", "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"bottomMostNonEmptyCells", "[", "table_", "]"}], ":=", 
      RowBox[{"With", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"cells", "=", 
              RowBox[{"Map", "[", 
                RowBox[{"leftMostNonEmptyCell", ",", 
                  RowBox[{"Transpose", "[", "table", "]"}]}], "]"}]}], "}"}], 
          ",", 
          RowBox[{"Delete", "[", 
            RowBox[{"cells", ",", 
              RowBox[{"Position", "[", 
                RowBox[{"cells", ",", 
                  RowBox[{"{", "}"}]}], "]"}]}], "]"}]}], "]"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"averageP0P1\[Beta]20", "[", "list_", "]"}], ":=", 
        RowBox[{
          RowBox[{"Apply", "[", 
            RowBox[{"Plus", ",", "list"}], "]"}], "/", 
          RowBox[{"Length", "[", "list", "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"p0p1\[Beta]20Plussvppq\[CapitalDelta]p\[Beta]20", "[", 
          RowBox[{"{", 
            RowBox[{"p0_", ",", "p1_", ",", "\[Beta]20_"}], "}"}], "]"}], ":=",
         
        RowBox[{"{", 
          RowBox[{
            RowBox[{"s", "[", 
              RowBox[{"p0", ",", "p1", ",", "\[Beta]20"}], "]"}], ",", 
            RowBox[{"vp", "[", 
              RowBox[{"p0", ",", "p1", ",", "\[Beta]20"}], "]"}], ",", "p0", 
            ",", "p1", ",", "\[Beta]20", ",", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"p0", "+", "p1"}], ")"}], "/", "2"}], ",", 
            RowBox[{"p1", "-", "p0"}]}], "}"}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generation and processing of the SV table", "Subsection"],

Cell[CellGroupData[{

Cell["The SV table", "Subsubsection"],

Cell["\<\
This section contains the calculation of the SV table as described \
in the introduction.\
\>", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"{", "}"}], ",", 
          RowBox[{"{", "100", "}"}], ",", 
          RowBox[{"{", "200", "}"}]}], "]"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"svTable", "=", 
        RowBox[{"fillTableVar\[Beta]20P1P0", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"0", ",", "1", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"0", ",", "1", ",", "0.01"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"0", ",", "1", ",", "0.01"}], "}"}], ",", "%"}], 
          "]"}]}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Density plot of the SV table", "Subsubsection"],

Cell["\<\
Below, a density plot of the SV table calculated in the last \
section is displayed. All cells in the table that are non empty are displayed \
as black rectangles and all empty cells are displayed as white \
rectangles.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"densityTable", "=", 
        RowBox[{"markCellEntries", "[", "svTable", "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ListDensityPlot", "[", 
        RowBox[{"densityTable", ",", 
          RowBox[{"Mesh", "\[Rule]", "False"}], ",", 
          RowBox[{"Frame", "\[Rule]", "False"}], ",", 
          RowBox[{"Axes", "\[Rule]", "True"}], ",", 
          RowBox[{"Ticks", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        RowBox[{"10", "*", 
                          RowBox[{"100", "/", "12"}]}], ",", 
                        "\"\<\!\(1\/12\)\>\""}], "}"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{"{", 
                      RowBox[{
                        RowBox[{"10", "*", 
                          RowBox[{"100", "/", "12"}]}], ",", 
                        "\"\<\!\(1\/12\)\>\""}], "}"}]}], "}"}]}], "}"}]}], 
          ",", 
          RowBox[{"AspectRatio", "\[Rule]", 
            RowBox[{"1", "/", "1.5"}]}], ",", 
          RowBox[{"AxesLabel", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"\"\<s\>\"", ",", "\"\<v'\>\""}], "}"}]}]}], "]"}], 
      ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .66667 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% DensityGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0192308 0.00480769 0.0128205 0.00641026 [
[.01923 -0.0125 -3 -9 ]
[.01923 -0.0125 3 0 ]
[.41987 -0.0125 -11.0938 -19.9375 ]
[.41987 -0.0125 11.0938 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 10 4.90625 ]
[-0.0125 .01282 -6 -4.5 ]
[-0.0125 .01282 0 4.5 ]
[-0.0125 .54701 -22.1875 -9.96875 ]
[-0.0125 .54701 0 9.96875 ]
[0 .69167 -8 0 ]
[0 .69167 8 9.8125 ]
[ 0 0 0 0 ]
[ 1 .66667 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01923 0 m
.01923 .00625 L
s
[(0)] .01923 -0.0125 0 1 Mshowa
.41987 0 m
.41987 .00625 L
s
gsave
.41987 -0.0125 -72.0938 -23.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 27.9375 translate 1 -1 scale
63.000 16.938 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
69.062 11.188 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
64.750 16.938 moveto
(\\200\\200\\200\\200) show
73.250 16.938 moveto
(\\200\\200) show
77.062 16.938 moveto
(\\200) show
66.062 22.938 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(12) show
81.188 16.938 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
69.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .01282 m
.00625 .01282 L
s
[(0)] -0.0125 .01282 1 0 Mshowa
0 .54701 m
.00625 .54701 L
s
gsave
-0.0125 .54701 -83.1875 -13.9688 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 27.9375 translate 1 -1 scale
63.000 16.938 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
69.062 11.188 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
64.750 16.938 moveto
(\\200\\200\\200\\200) show
73.250 16.938 moveto
(\\200\\200) show
77.062 16.938 moveto
(\\200) show
66.062 22.938 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(12) show
81.188 16.938 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
0 .66667 L
s
gsave
0 .69167 -69 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(v) show
69.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(') show
75.000 11.250 moveto
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .66667 L
0 .66667 L
closepath
clip
newpath
% Start of gray image
p
.01923 .01282 translate
.96154 .64103 scale
200 string
200 100 8 [200 0 0 100 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFF00FFFFFF00FFFFFF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00FFFFFF00FFFF00FFFFFF00FFFF00FFFFFF00FFFF
0000FFFF0000FFFFFF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00FFFF00FFFF00FFFF00FF00
FFFF00FFFFFF00FFFF00FF0000FF00FFFFFF00FFFF00FFFF0000FFFF0000FFFF0000FFFF
0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
00FF00FF00FF00FF00FF00FF00FF00FFFF00FF00FFFF00FFFFFFFFFFFFFFFF00FFFF00FF
FF00FFFF00FFFFFF00FF0000FF0000FFFF0000FFFF00FFFF0000FFFF0000FFFF0000FFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000FF000000FF0000FF00
FFFFFFFFFFFFFFFFFFFFFFFF00FFFF00FF00FFFF00FF00FFFF00FFFF00FF00FFFF00FFFF
0000FF0000FF0000FFFF0000FF0000FFFF0000FFFF0000FF0000FFFFFF0000FFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000FF00FF0000
FF00FF00FFFF00FF00FF00FFFFFFFF0000FF0000FF0000FF0000FF00FFFF00FF0000FF00
FF00FF0000FF0000FF000000FF0000FFFF0000FFFF0000FFFF0000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000FF0000FFFFFFFFFFFFFFFF
FF0000000000FF0000000000FF0000FFFFFFFFFF0000FF0000FF0000FF0000FF0000FF00
FF00000000FF000000FF0000FF000000FF000000FF000000FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000FF00000000000000
00FFFFFFFFFFFF000000FF000000FF000000FF000000000000000000FF0000FF00FF0000
0000FF0000FF000000FF000000FF000000FF000000FFFF00FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF0000000000000000000000000000FFFFFFFFFFFFFF000000000000
000000FF00FF00FF0000FF00000000FF0000FF0000FF0000FF0000FF0000000000000000
00FF00000000FF000000000000FF00000000FFFF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000FF00FF0000FFFFFFFFFF00
FF00000000FF00FF00FFFF00000000FF0000FF00FFFF0000FF0000FF000000000000FF00
000000000000000000000000000000FFFF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF0000000000000000000000FFFFFFFFFFFF0000000000000000000000FFFF
FF00FF00000000FF0000FF00000000000000FF0000FF000000000000FF00000000000000
000000000000FF00000000FFFF0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000FFFFFFFF0000000000000000
FF00FF00000000FF0000FF00FF000000000000000000FF0000FF0000FF00000000000000
0000000000000000FFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF00000000000000000000FFFFFFFF0000000000000000000000FF00FF0000000000
0000FFFF000000000000FF0000000000000000FF00000000000000000000000000000000
00000000FF0000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFF0000000000000000000000FFFFFF000000000000000000FF0000000000
00000000000000000000000000000000000000000000FF00000000000000000000000000
00000000000000FFFF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FF00000000000000FFFFFF0000000000000000000000FFFF000000000000FF00FF0000FF
00000000000000000000000000000000FF00000000000000000000000000000000000000
00000000FFFF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF000000000000000000FFFFFF000000000000000000FF000000FF00000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00FF00FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000
00000000FFFF00000000000000000000FFFF000000000000000000000000000000FF0000
000000000000000000000000000000000000000000FF000000000000FF0000000000FFFF
0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000
0000000000000000FFFF0000000000000000FF00000000000000FF000000000000000000
0000000000000000000000000000000000000000000000000000000000000000000000FF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000
0000000000000000000000FF0000000000000000FF000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000FFFF00
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000
000000000000000000000000FFFF00000000000000FF0000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000FFFFFF00FFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000
0000000000000000000000000000FF000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000FF0000FF00FFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000
000000000000000000000000000000FF0000000000000000000000000000000000000000
00000000000000000000000000000000000000000000000000FF000000FFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000FF00FF0000FFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000FFFF00FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000FFFF00FFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000FF0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000000000000000000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000FF0000FF00FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000FF00FFFF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000FF0000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00FF00FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000FF00FF00
FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000FF00FF00
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000FF00FFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000FF00FF00FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000FF00FF00FFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000FF00FF00FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000FF00FF00FFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000000000000000000000000000
00000000000000000000000000000000000000FF000000FF0000FFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000FF00FF00FFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000000000000000000000
000000000000000000000000000000000000FF00FF0000FFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000000000000000
0000000000000000000000000000000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFF0000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFF0000000000000000000000000000000000000000000000000000
00000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFF0000000000000000000000000000000000000000000000000000
00000000000000000000000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF000000000000000000000000000000000000000000000000000000
00000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF000000000000000000000000000000000000000000000000000000
000000000000000000FF0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFF00000000000000000000000000000000000000000000000000000000
000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFF00000000000000000000000000000000000000000000000000000000
0000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFF0000000000000000000000000000000000000000000000000000000000
00000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFF0000000000000000000000000000000000000000000000000000000000
00000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFF0000000000000000000000000000000000000000000000000000000000
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000000000000000000000000000000000000000000000000000000000
0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000000000000000000000000000000000000000000000000000000000
00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF00000000000000000000000000000000000000000000000000000000FFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFF00000000000000000000000000000000000000000000000000000000FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFF000000000000000000000000000000000000000000000000000000FFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFF00000000000000000000000000000000000000000000000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFF00000000000000000000000000000000000000000000000000FFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFF00000000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFF000000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF0000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF00000000000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF0000000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFF00000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFF000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFF00000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FF000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FF0000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FF000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FF00000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
00000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 192},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000`1000`40n007h00Ooooo8Ooo003ooolQ
ool007coo`@000;oo`@009[oo`00O_oo00D00?ooooooo`0009goo`00O_oo00<00?ooool00_oo00<0
0?ooool0V_oo001nool00`00ooooo`03ool00`00ooooo`2Iool007koo`03003ooooo00Coo`03003o
oooo09Soo`00O?oo0`000ooo00@00?ooool009[oo`00O_oo00<00?ooool00_oo0P00Vooo003ooolQ
ool007[oo`h009Soo`00oooo8Ooo003ooolQool002Goo`800?Woo`009?oo00@00?ooool005Ooo`@0
09goo`009?oo00@00?ooool005Woo`03003ooooo09coo`009?oo00@00?ooool005Woo`03003ooooo
09coo`009?oo00@00?ooool005Woo`03003ooooo09coo`009?oo00@00?ooool005Woo`03003ooooo
09coo`009Ooo0P00F?oo0`00W_oo0021ool00`00ooooo`2Lool00?ooob7oo`00oooo4?oo0`003_oo
000QoooV000;ool00`00ooooo`0;ool001[oo`8000Goo`03003ooooo00;oo`03003ooooo05Soo`03
003ooooo08coo`8000koo`006Ooo00@00?ooool000Coo`03003ooooo00;oo`03003ooooo05Soo`03
003ooooo08_oo`03003ooooo00koo`006Ooo00@00?ooool000Coo`<00>coo`<000goo`006Ooo00@0
0?ooool000Coo`03003ooooo0:Koo`03003ooooo00;oo`05003oooooool0001<ool001Woo`04003o
oooo0004ool00`00ooooo`2Vool00`00ooooo`02ool01@00oooooooo0000C?oo000Iool01000oooo
o`001?oo00<00?ooool0R_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`04003ooooo0003ool2
0002ool20002ool20004ool00`00ooooo`1>ool001[oo`8000Goo`03003ooooo06coo`04003ooooo
0002ool01`00oooooooo003oo`0000;oo`03003ooooo00;oo`05003ooooo003oo`0200000ooo0000
0003ool01000ooooo`000ooo0P000_oo0P000_oo0`000_oo0P00FOoo000Qool00`00ooooo`1/ool0
1000ooooo`000_oo00L00?ooooooo`00ool00002ool00`00ooooo`02ool01@00ooooo`00ool00P00
00?oo`0000000ooo00@00?ooool000?oo`8000;oo`8000;oo`<000;oo`8005Woo`008Ooo00<00?oo
ool0DOoo00d00?oo00000?oo003oo`00ool00?oo003oo`0200001ooo003ooooo003oo`0000?oo`03
003ooooo00Koo`8000;oo`04003ooooo0002ool20003ool01@00ool00000ool00`000_oo0P000_oo
00@00?ooooooo`8000;oo`8000;oo`<005goo`008Ooo00<00?ooool0?Ooo2P0000?oo`0000000P00
00Goo`00003oo`0000goo`8000;oo`03003oo`0000;oo`04003ooooo0002ool01P00ooooo`00ool0
00?oo`03003ooooo00800005ool00000003oo`020002ool200001?oo000000000_oo0P000_oo0P00
0_oo0P000ooo0P00HOoo000Qool00`00ooooo`0mool:00000ooo0000000200001Ooo00000?oo0000
3Ooo0P000_oo00<00?oo00000_oo00@00?ooool000;oo`06003ooooo003oo`000ooo00<00?ooool0
0P0000Goo`0000000?oo008000;oo`800004ool000000002ool20002ool20002ool20003ool2001Q
ool0027oo`03003ooooo04Woo`<00003ool00?oo00800005ool00?ooool00002ool01@00ool00?oo
00001Ooo0P0000Coo`00003oo`<00005ool00000ool00002ool00`00ooooo`0200001Ooo003oo`00
ool00`0000Coo`00003oo`<00004ool000000002ool20002ool20003ool2001Uool0027oo`03003o
oooo03Woo``00003ool0000000[oo`H00003ool0000000<000;oo`8000Goo`<00004ool00000ool2
00000ooo00000002ool200000ooo003oo`0500001Ooo00000000ool00P0000?oo`0000000P0000Go
o`0000000?oo00@006Soo`008Ooo00<00?ooool0A?oo100000?oo`0000001`001ooo0`0000?oo`00
00000P0000Goo`0000000?oo00X00003ool0000000;oo`03003oo`0000<00005ool00000003oo`03
00001Ooo00000000ool0100000Coo`00000000;oo`8006Soo`008Ooo00<00?ooool0A?oo100000?o
o`0000001`001ooo0`0000?oo`0000000P0000Goo`0000000?oo00X00003ool0000000;oo`03003o
o`0000<00005ool00000003oo`0300001Ooo00000000ool0100000Coo`00000000;oo`8006Soo`00
8Ooo00<00?ooool0<ooo40002?oo2P0000Goo`00ool00?oo00800003ool0000000<00004ool00000
ool20002ool200001?oo00000?oo2P0000?oo`0000000`0000?oo`000000100000?oo`0000000`00
0_oo00<00?ooool0J_oo000Qool00`00ooooo`0lool900000ooo003oo`030005ool00`00ool00004
00001?oo003oo`000_oo1@0000Goo`00003oo`0000?oo`800004ool00000ool700000ooo0000000@
0002ool00`00ooooo`1]ool0027oo`03003ooooo03coo`T00003ool00?oo00<000Goo`03003oo`00
00@00004ool00?oo0002ool500001Ooo00000?oo00000ooo0P0000Coo`00003oo`L00003ool00000
010000;oo`03003ooooo06goo`008Ooo00<00?ooool0<?oo30001ooo30000ooo00<00?ooool01000
00Coo`00003oo`P00003ool0000000;oo`H00003ool0000000d00003ool00000008000;oo`<007;o
o`008Ooo00<00?ooool0=ooo3@001?oo2@0000Coo`00ooooo`@00005ool00000ool00002ool:0000
1?oo00000?oo0P0000?oo`0000003`000_oo0`00MOoo000Qool00`00ooooo`0gool=0004ool90000
1?oo003ooooo100000Goo`00003oo`0000;oo`X00004ool00000ool200000ooo0000000?0002ool3
001eool0027oo`03003ooooo02goo`/000Coo``00004ool00000ool70003ool600000ooo00000007
00000ooo0000000D0002ool200000ooo003oo`1hool0027oo`03003ooooo037oo``000Coo`X00003
ool0000001`00003ool0000001@000;oo`03003ooooo07Koo`008Ooo00<00?ooool0<Ooo30001?oo
2P0000?oo`000000700000?oo`00000050000_oo00<00?ooool0M_oo000Qool00`00ooooo`0Zool8
0004ool<0002ool700000ooo003oo`0200000ooo0000000@00000ooo0000000G0003ool00`00oooo
o`1iool0027oo`03003ooooo02goo`X000?oo`X00005ool00000003oo`0a00000ooo003oo`02001m
ool0027oo`03003ooooo02goo`X000?oo`X00005ool00000003oo`0a00000ooo003oo`02001mool0
027oo`03003ooooo02Ooo`L000;oo`/000;ooa400003ool0000001L000;oo`H00003ool0000000@0
00;oo`80087oo`008Ooo00<00?ooool09ooo2`000_oo2@0000?oo`0000001P0000?oo`000000;`00
Q?oo000Qool00`00ooooo`0UoolA00000ooo000000060002oole0002ool00`00ooooo`21ool0027o
o`03003ooooo02Gooa400003ool0000000H000;oocD000;oo`03003ooooo087oo`008Ooo00<00?oo
ool09?oo4`000_oo200000?oo`000000;P001?oo00<00?ooool0Pooo000Qool00`00ooooo`0SoolF
00000ooo0000000b00001Ooo00000?oo0000ROoo000Qool00`00ooooo`0SoolF00000ooo0000000b
00001Ooo00000?oo0000ROoo000Qool00`00ooooo`0RoolH00000ooo0000000`00001?oo00000000
Rooo000Qool00`00ooooo`0Room60002ool01000ool00000S_oo000Qool00`00ooooo`0Room60002
ool01000ool00000S_oo000Qool00`00ooooo`0Ooom=0002ool00`00ooooo`2;ool0027oo`03003o
oooo01oood/000;oo`03003ooooo08goo`008Ooo00<00?ooool07oooB`000_oo00<00?ooool0SOoo
000Qool00`00ooooo`0Noom<002Bool0027oo`03003ooooo01goodL00003ool0000009Goo`008Ooo
00<00?ooool07OooA`0000?oo`000000UOoo000Qool00`00ooooo`0Loom700000ooo003oo`2Fool0
027oo`03003ooooo01_oodH00003ool00?oo09Soo`008Ooo00<00?ooool06oooAP0000?oo`00ool0
V?oo000Qool00`00ooooo`0Joom100000ooo00000002ool00`00ool0002Iool0027oo`03003ooooo
01[ood@000;oo`03003ooooo09Woo`008Ooo00<00?ooool06Ooo@@0000Coo`00ooooo`8009coo`00
8Ooo00<00?ooool06Ooo@@0000Coo`00ooooo`8009coo`008Ooo00<00?ooool06?oo@00000Goo`00
003oo`0009ooo`008Ooo00<00?ooool06?oo@@0000?oo`00ool0X?oo000Qool00`00ooooo`0Hoom1
00000ooo003oo`2Pool0027oo`03003ooooo01Oood800:?oo`008Ooo00<00?ooool05Ooo?@0000Go
o`00003oo`000:Goo`008Ooo00<00?ooool05Ooo?@0000Goo`00003oo`000:Goo`008Ooo00<00?oo
ool05Ooo?@0000?oo`000000Yooo000Qool00`00ooooo`0Dooln002Zool0027oo`03003ooooo01Co
och00:[oo`008Ooo00<00?ooool05?oo>@0000Koo`00ool00?oo002Yool0027oo`03003ooooo01?o
ocX00004ool00?oo002[ool0027oo`03003ooooo01?oocX00004ool00?oo002[ool0027oo`03003o
oooo01;ooc/00003ool00?oo0:coo`008Ooo00<00?ooool04_oo=`000_oo00<00?oo0000[_oo000Q
ool00`00ooooo`0Aoolf00001Ooo003ooooo0000/?oo000Qool00`00ooooo`0Aoolf00001Ooo003o
oooo0000/?oo000Qool00`00ooooo`0Aoole00000ooo003oo`02002aool0027oo`03003ooooo013o
oc@00004ool00?oo002dool0027oo`03003ooooo013ooc@00004ool00?oo002dool0027oo`03003o
oooo013oobh00003ool0000000800003ool000000;Koo`008Ooo00<00?ooool03ooo<P0000Coo`00
ool00;Ooo`008Ooo00<00?ooool03ooo<P0000Coo`00ool00;Ooo`008Ooo00<00?ooool03ooo;P00
00Coo`00ooooo`800;Woo`008Ooo00<00?ooool03_oo<@00_Ooo000Qool00`00ooooo`0>oola002m
ool0027oo`03003ooooo00koobh00003ool00?oo0;goo`008Ooo00<00?ooool03Ooo<000_ooo000Q
ool00`00ooooo`0=ool`002oool0027oo`03003ooooo00goobh00<7oo`008Ooo00<00?ooool03Ooo
:`0000?oo`00ool0`Ooo000Qool00`00ooooo`0=ool[00000ooo003oo`31ool0027oo`03003ooooo
00coob`00<Coo`008Ooo00<00?ooool03?oo9`0000Coo`0000000<Goo`008Ooo00<00?ooool02_oo
:@00bOoo000Qool00`00ooooo`0:oolY0039ool0027oo`03003ooooo00[oobP00<[oo`008Ooo00<0
0?ooool02Ooo:000booo000Qool00`00ooooo`09oolX003;ool0027oo`03003ooooo00WoobD00003
ool00?oo0<_oo`008Ooo00<00?ooool02Ooo9000cooo000Qool00`00ooooo`09oolT003?ool0027o
o`03003ooooo00Soob@00=3oo`008Ooo00<00?ooool02?oo8P00d_oo000Qool00`00ooooo`08oolR
003Bool0027oo`03003ooooo00Sooal00=Goo`008Ooo00<00?ooool01ooo7`00e_oo000Qool00`00
ooooo`07oolO003Fool0027oo`03003ooooo00Oooah00=Ooo`008Ooo00<00?ooool01ooo7000fOoo
000Qool00`00ooooo`06oolL003Jool0027oo`03003ooooo00Kooa`00=[oo`008Ooo00<00?ooool0
1_oo6000g_oo000Qool00`00ooooo`06oolG003Oool0027oo`03003ooooo00KooaL00=ooo`008Ooo
00<00?ooool01Ooo5P00hOoo000Qool00`00ooooo`05oolE003Rool0027oo`03003ooooo00GooaD0
0>;oo`008Ooo00<00?ooool01Ooo4`00i?oo000Qool00`00ooooo`04oolA003Wool0027oo`03003o
oooo00Cooa400>Ooo`008Ooo00<00?ooool01?oo4000j?oo000Qool00`00ooooo`04ool>003Zool0
027oo`03003ooooo00Coo`h00>[oo`008Ooo00<00?ooool00ooo3@00k?oo000Qool00`00ooooo`03
ool<003]ool0027oo`03003ooooo00?oo``00>goo`008Ooo00<00?ooool00ooo2P00kooo000>ool4
0002ool40009ool00`00ooooo`03ool7003bool0013oo`05003oooooool0000<ool00`00ooooo`02
ool7003cool0013oo`03003ooooo00;oo`03003ooooo00Woo`03003ooooo00;oo`L00??oo`004?oo
00<00?ooool00ooo00<00?ooool02?oo00<00?ooool00_oo1@00mOoo000@ool00`00ooooo`04ool0
0`00ooooo`07ool00`00ooooo`02ool3003gool000koo`<000?oo`04003ooooo0009ool00`00oooo
o`02ool3003gool0013oo`03003ooooo00;oo`8000[oo`03003ooooo00;oo`800?Soo`008Ooo00<0
0?ooool00_oo00<00?ooool0mooo000<ool>0007ool30002ool00`00ooooo`3gool0027oo`03003o
oooo0?coo`008Ooo00<00?ooool0o?oo000Aool4000<ool00`00ooooo`3lool001?oo`03003ooooo
00_oo`03003ooooo0?coo`004ooo00<00?ooool02ooo00<00?ooool0o?oo000Cool00`00ooooo`0;
ool00`00ooooo`3lool001?oo`03003ooooo00_oo`03003ooooo0?coo`004Ooo0`003Ooo00<00?oo
ool0o?oo000Cool00`00ooooo`0;ool00`00ooooo`3lool0027oo`03003ooooo0?coo`008Ooo00<0
0?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`008Ooo00<00?ooool0o?oo
000Qool00`00ooooo`3lool0027oo`03003ooooo0?coo`008Ooo00<00?ooool0o?oo000Qool00`00
ooooo`3lool0027oo`03003ooooo0?coo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0
027oo`03003ooooo0?coo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool0027oo`03003o
oooo0?coo`008Ooo00<00?ooool0o?oo000Qool00`00ooooo`3lool00?ooob7oo`00oooo8Ooo003o
oolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo000Nool00`00ooooo`3o
ool001goo`03003oo`000?ooo`7oo`007Ooo00<00?oo0000oooo0Ooo000Lool01@00oooooooo0000
oooo000Kool20002ool20002ool00`00ooooo`3jool002?oo`03003ooooo0?[oo`008ooo00<00?oo
ool0n_oo0000\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {635.938, 483.375}} -> {-183.86, \
394.825, 0.91001, 0.682507}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Average of the values in the bottom most non empty cells", \
"Subsubsection"],

Cell[BoxData[""], "Input"],

Cell[TextData[{
  "Displayed below is a table describing the average of the values in the \
bottom most non empty cells of the SV table. The average values of ",
  Cell[BoxData[
      \("\!\(p\_0\)", "\!\(p\_1\)", 
      "\[LeftBracketingBar]\!\(\[Beta]\_0\)\!\(\(\[RightBracketingBar]\^2\)\)\
"\)]],
  " were used to calculate the disturbance, postestimation deviation, and the \
quantities  ",
  Cell[BoxData[
      \(TraditionalForm\`p\&_ = \(1\/2\) \((p\_0 + p\_1)\)\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]p = p\_1 - p\_0\)]],
  ". It is interesting to note that the \"optimal\" values of ",
  Cell[BoxData[
      \(TraditionalForm\`\[LeftBracketingBar]\[Beta]\_0\[RightBracketingBar]\^\
2\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`p\&_\)]],
  " seem to be ",
  Cell[BoxData[
      \(TraditionalForm\`0.5\)]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"bottomMostNonEmptyCells", "[", "svTable", "]"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"averageP0P1\[Beta]20", "/@", "%"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"p0p1\[Beta]20Plussvppq\[CapitalDelta]p\[Beta]20", "/@", "%"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"TableForm", "[", 
        RowBox[{"%", ",", 
          RowBox[{"TableHeadings", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"None", ",", 
                RowBox[{"{", 
                  RowBox[{
                  "\"\<s\>\"", ",", "\"\<\!\(v\^\[Prime]\)\>\"", ",", 
                    "\"\<\!\(p\_0\)\>\"", ",", "\"\<\!\(p\_1\)\>\"", ",", 
                    "\"\<\[LeftBracketingBar]\!\(\[Beta]\_0\)\!\(\(\
\[RightBracketingBar]\^2\)\)\>\"", ",", "\"\<\!\(p\&_\)\>\"", ",", 
                    "\"\<\[CapitalDelta]p\>\""}], "}"}]}], "}"}]}]}], "]"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"NumberForm", "[", 
      RowBox[{"%", ",", 
        RowBox[{"{", 
          RowBox[{"5", ",", "5"}], "}"}], ",", 
        RowBox[{"NumberPadding", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{"\"\<\>\"", ",", "\"\<0\>\""}], "}"}]}]}], 
      "]"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[GridBox[{
            {"\<\"s\"\>", "\<\"\\!\\(v\\^\[Prime]\\)\"\>", \
"\<\"\\!\\(p\\_0\\)\"\>", "\<\"\\!\\(p\\_1\\)\"\>", "\<\"\[LeftBracketingBar]\
\\!\\(\[Beta]\\_0\\)\\!\\(\\(\[RightBracketingBar]\\^2\\)\\)\"\>", \
"\<\"\\!\\(p\\&_\\)\"\>", "\<\"\[CapitalDelta]p\"\>"},
            {
              InterpretationBox["\<\"0.00079\"\>",
                0.00079073940682726029,
                AutoDelete->True], 
              InterpretationBox["\<\"0.08025\"\>",
                0.080248430965936091,
                AutoDelete->True], 
              InterpretationBox["\<\"0.39280\"\>",
                0.39280046674445762,
                AutoDelete->True], 
              InterpretationBox["\<\"0.60720\"\>",
                0.6071995332555421,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.49999999999999778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.49999999999999989,
                AutoDelete->True], 
              InterpretationBox["\<\"0.21440\"\>",
                0.21439906651108448,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00195\"\>",
                0.0019545081001128652,
                AutoDelete->True], 
              InterpretationBox["\<\"0.07597\"\>",
                0.075972895732389717,
                AutoDelete->True], 
              InterpretationBox["\<\"0.33519\"\>",
                0.33518518518518536,
                AutoDelete->True], 
              InterpretationBox["\<\"0.66481\"\>",
                0.66481481481481519,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.32963\"\>",
                0.32962962962962983,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00279\"\>",
                0.0027933484111422229,
                AutoDelete->True], 
              InterpretationBox["\<\"0.07307\"\>",
                0.073068485912892861,
                AutoDelete->True], 
              InterpretationBox["\<\"0.30599\"\>",
                0.30598984771573612,
                AutoDelete->True], 
              InterpretationBox["\<\"0.69401\"\>",
                0.69401015228426421,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000044,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.38802\"\>",
                0.3880203045685281,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00371\"\>",
                0.0037114405082854978,
                AutoDelete->True], 
              InterpretationBox["\<\"0.07004\"\>",
                0.070044402645909595,
                AutoDelete->True], 
              InterpretationBox["\<\"0.28000\"\>",
                0.28000000000000019,
                AutoDelete->True], 
              InterpretationBox["\<\"0.72000\"\>",
                0.7200000000000002,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.500000000000001,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.44000\"\>",
                0.44,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00469\"\>",
                0.004691323987145557,
                AutoDelete->True], 
              InterpretationBox["\<\"0.06698\"\>",
                0.066980699505602426,
                AutoDelete->True], 
              InterpretationBox["\<\"0.25680\"\>",
                0.25680000000000019,
                AutoDelete->True], 
              InterpretationBox["\<\"0.74320\"\>",
                0.74320000000000053,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000111,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.48640\"\>",
                0.48640000000000033,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00571\"\>",
                0.0057104054850748613,
                AutoDelete->True], 
              InterpretationBox["\<\"0.06396\"\>",
                0.063959227336166519,
                AutoDelete->True], 
              InterpretationBox["\<\"0.23620\"\>",
                0.23620000000000008,
                AutoDelete->True], 
              InterpretationBox["\<\"0.76380\"\>",
                0.76380000000000059,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000067,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.52760\"\>",
                0.52760000000000051,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00682\"\>",
                0.0068229317783793739,
                AutoDelete->True], 
              InterpretationBox["\<\"0.06084\"\>",
                0.060836591360431333,
                AutoDelete->True], 
              InterpretationBox["\<\"0.21676\"\>",
                0.21676470588235303,
                AutoDelete->True], 
              InterpretationBox["\<\"0.78324\"\>",
                0.78323529411764736,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.5,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.56647\"\>",
                0.56647058823529428,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00792\"\>",
                0.0079206683117165975,
                AutoDelete->True], 
              InterpretationBox["\<\"0.05792\"\>",
                0.057920668311716647,
                AutoDelete->True], 
              InterpretationBox["\<\"0.20000\"\>",
                0.20000000000000004,
                AutoDelete->True], 
              InterpretationBox["\<\"0.80000\"\>",
                0.80000000000000049,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.5,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.60000\"\>",
                0.60000000000000042,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00865\"\>",
                0.00864540102373762,
                AutoDelete->True], 
              InterpretationBox["\<\"0.05608\"\>",
                0.056079035996624629,
                AutoDelete->True], 
              InterpretationBox["\<\"0.19000\"\>",
                0.19000000000000009,
                AutoDelete->True], 
              InterpretationBox["\<\"0.81000\"\>",
                0.8100000000000005,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.62000\"\>",
                0.62000000000000044,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.00955\"\>",
                0.0095500415633707845,
                AutoDelete->True], 
              InterpretationBox["\<\"0.05387\"\>",
                0.053867430959346541,
                AutoDelete->True], 
              InterpretationBox["\<\"0.17852\"\>",
                0.17852272727272739,
                AutoDelete->True], 
              InterpretationBox["\<\"0.82148\"\>",
                0.82147727272727333,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000056,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.64295\"\>",
                0.642954545454546,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01073\"\>",
                0.01072863978093443,
                AutoDelete->True], 
              InterpretationBox["\<\"0.05112\"\>",
                0.051122156134050745,
                AutoDelete->True], 
              InterpretationBox["\<\"0.16500\"\>",
                0.16499999999999995,
                AutoDelete->True], 
              InterpretationBox["\<\"0.83500\"\>",
                0.83500000000000063,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.67000\"\>",
                0.67000000000000071,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01169\"\>",
                0.011689317989248344,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04899\"\>",
                0.048990879183898717,
                AutoDelete->True], 
              InterpretationBox["\<\"0.15500\"\>",
                0.155,
                AutoDelete->True], 
              InterpretationBox["\<\"0.84500\"\>",
                0.84500000000000053,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.69000\"\>",
                0.6900000000000005,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01274\"\>",
                0.012736245841154847,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04677\"\>",
                0.046769858450876549,
                AutoDelete->True], 
              InterpretationBox["\<\"0.14500\"\>",
                0.14499999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.85500\"\>",
                0.85500000000000043,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.71000\"\>",
                0.71000000000000041,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01388\"\>",
                0.013880803440797784,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04446\"\>",
                0.044455327606409648,
                AutoDelete->True], 
              InterpretationBox["\<\"0.13500\"\>",
                0.13499999999999995,
                AutoDelete->True], 
              InterpretationBox["\<\"0.86500\"\>",
                0.86500000000000077,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.73000\"\>",
                0.73000000000000087,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01449\"\>",
                0.014493883049387663,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04326\"\>",
                0.043261694521389109,
                AutoDelete->True], 
              InterpretationBox["\<\"0.13000\"\>",
                0.13,
                AutoDelete->True], 
              InterpretationBox["\<\"0.87000\"\>",
                0.87000000000000055,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.74000\"\>",
                0.74000000000000055,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01581\"\>",
                0.015811621111785613,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04080\"\>",
                0.040798799840265049,
                AutoDelete->True], 
              InterpretationBox["\<\"0.12000\"\>",
                0.11999999999999997,
                AutoDelete->True], 
              InterpretationBox["\<\"0.88000\"\>",
                0.88000000000000056,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.76000\"\>",
                0.76000000000000056,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01652\"\>",
                0.016520951558515996,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03953\"\>",
                0.03952826583151095,
                AutoDelete->True], 
              InterpretationBox["\<\"0.11500\"\>",
                0.11499999999999998,
                AutoDelete->True], 
              InterpretationBox["\<\"0.88500\"\>",
                0.88500000000000045,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.77000\"\>",
                0.77000000000000046,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01727\"\>",
                0.017267500317831436,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03823\"\>",
                0.03823075263224468,
                AutoDelete->True], 
              InterpretationBox["\<\"0.11000\"\>",
                0.10999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.89000\"\>",
                0.89000000000000057,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.78000\"\>",
                0.78000000000000058,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01889\"\>",
                0.018885053887193859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03555\"\>",
                0.035551720553860522,
                AutoDelete->True], 
              InterpretationBox["\<\"0.10000\"\>",
                0.099999999999999978,
                AutoDelete->True], 
              InterpretationBox["\<\"0.90000\"\>",
                0.90000000000000047,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.80000\"\>",
                0.80000000000000049,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.01952\"\>",
                0.019518964771308878,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03455\"\>",
                0.034548726341169587,
                AutoDelete->True], 
              InterpretationBox["\<\"0.09636\"\>",
                0.096363636363636374,
                AutoDelete->True], 
              InterpretationBox["\<\"0.90364\"\>",
                0.90363636363636446,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.5,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000044,
                AutoDelete->True], 
              InterpretationBox["\<\"0.80727\"\>",
                0.80727272727272803,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02069\"\>",
                0.020694377594140757,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03275\"\>",
                0.032754964402501932,
                AutoDelete->True], 
              InterpretationBox["\<\"0.09000\"\>",
                0.090000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.91000\"\>",
                0.91000000000000036,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.82000\"\>",
                0.8200000000000004,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02168\"\>",
                0.021682798190361674,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03131\"\>",
                0.031310029972923828,
                AutoDelete->True], 
              InterpretationBox["\<\"0.08500\"\>",
                0.085000000000000006,
                AutoDelete->True], 
              InterpretationBox["\<\"0.91500\"\>",
                0.91500000000000037,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.83000\"\>",
                0.8300000000000004,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02273\"\>",
                0.022734871030130905,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02983\"\>",
                0.02983260413846775,
                AutoDelete->True], 
              InterpretationBox["\<\"0.08000\"\>",
                0.080000000000000016,
                AutoDelete->True], 
              InterpretationBox["\<\"0.92000\"\>",
                0.92000000000000048,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.84000\"\>",
                0.84000000000000052,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02386\"\>",
                0.023857688810054751,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02832\"\>",
                0.028321470083827453,
                AutoDelete->True], 
              InterpretationBox["\<\"0.07500\"\>",
                0.075000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.92500\"\>",
                0.92500000000000049,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.85000\"\>",
                0.85000000000000053,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02386\"\>",
                0.023857688810054713,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02832\"\>",
                0.028321470083827419,
                AutoDelete->True], 
              InterpretationBox["\<\"0.07500\"\>",
                0.074999999999999997,
                AutoDelete->True], 
              InterpretationBox["\<\"0.92500\"\>",
                0.92500000000000071,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.85000\"\>",
                0.85000000000000075,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02506\"\>",
                0.025059619167440296,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02678\"\>",
                0.026775291315260664,
                AutoDelete->True], 
              InterpretationBox["\<\"0.07000\"\>",
                0.069999999999999993,
                AutoDelete->True], 
              InterpretationBox["\<\"0.93000\"\>",
                0.93000000000000038,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.86000\"\>",
                0.86000000000000043,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02635\"\>",
                0.026350638555814005,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02519\"\>",
                0.025192592126620904,
                AutoDelete->True], 
              InterpretationBox["\<\"0.06500\"\>",
                0.06500000000000003,
                AutoDelete->True], 
              InterpretationBox["\<\"0.93500\"\>",
                0.9350000000000005,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.87000\"\>",
                0.87000000000000044,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02774\"\>",
                0.027742786046049479,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02357\"\>",
                0.023571733292968772,
                AutoDelete->True], 
              InterpretationBox["\<\"0.06000\"\>",
                0.060000000000000012,
                AutoDelete->True], 
              InterpretationBox["\<\"0.94000\"\>",
                0.94000000000000061,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.88000\"\>",
                0.88000000000000056,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02774\"\>",
                0.027742786046049479,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02357\"\>",
                0.023571733292968772,
                AutoDelete->True], 
              InterpretationBox["\<\"0.06000\"\>",
                0.060000000000000012,
                AutoDelete->True], 
              InterpretationBox["\<\"0.94000\"\>",
                0.94000000000000061,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.88000\"\>",
                0.88000000000000056,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.02925\"\>",
                0.029250793931962966,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02191\"\>",
                0.021910881366482768,
                AutoDelete->True], 
              InterpretationBox["\<\"0.05500\"\>",
                0.055000000000000028,
                AutoDelete->True], 
              InterpretationBox["\<\"0.94500\"\>",
                0.94500000000000062,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.89000\"\>",
                0.89000000000000057,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03089\"\>",
                0.030892986816511109,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02021\"\>",
                0.020207969208855477,
                AutoDelete->True], 
              InterpretationBox["\<\"0.05000\"\>",
                0.050000000000000003,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.95000000000000062,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.90000\"\>",
                0.90000000000000058,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03089\"\>",
                0.030892986816511109,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02021\"\>",
                0.020207969208855477,
                AutoDelete->True], 
              InterpretationBox["\<\"0.05000\"\>",
                0.050000000000000003,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.95000000000000062,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.90000\"\>",
                0.90000000000000058,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03269\"\>",
                0.032692602741471677,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01846\"\>",
                0.018460644213564265,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04500\"\>",
                0.044999999999999998,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95500\"\>",
                0.95500000000000085,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000044,
                AutoDelete->True], 
              InterpretationBox["\<\"0.91000\"\>",
                0.91000000000000081,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03269\"\>",
                0.032692602741471635,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01846\"\>",
                0.018460644213564324,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04500\"\>",
                0.045000000000000026,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95500\"\>",
                0.95500000000000052,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000044,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.91000\"\>",
                0.91000000000000048,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03468\"\>",
                0.0346798055727493,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01667\"\>",
                0.016666198713633743,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04000\"\>",
                0.040000000000000008,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000074,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.92000\"\>",
                0.92000000000000071,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03468\"\>",
                0.034679805572749314,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01667\"\>",
                0.016666198713633722,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04000\"\>",
                0.040000000000000015,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000085,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000044,
                AutoDelete->True], 
              InterpretationBox["\<\"0.92000\"\>",
                0.92000000000000082,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03468\"\>",
                0.034679805572749259,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01667\"\>",
                0.016666198713633756,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04000\"\>",
                0.040000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.92000\"\>",
                0.9200000000000006,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03689\"\>",
                0.036894887171421989,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01482\"\>",
                0.014821473669718765,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03500\"\>",
                0.034999999999999996,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96500\"\>",
                0.96500000000000064,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.93000\"\>",
                0.9300000000000006,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03689\"\>",
                0.036894887171421843,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01482\"\>",
                0.014821473669718834,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03500\"\>",
                0.035000000000000017,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96500\"\>",
                0.9650000000000003,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000011,
                AutoDelete->True], 
              InterpretationBox["\<\"0.93000\"\>",
                0.93000000000000027,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03939\"\>",
                0.039393646784125727,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01292\"\>",
                0.01292272048156537,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03000\"\>",
                0.029999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.94000\"\>",
                0.94000000000000061,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03939\"\>",
                0.039393646784125692,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01292\"\>",
                0.012922720481565371,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03000\"\>",
                0.030000000000000006,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000044,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.94000\"\>",
                0.94000000000000061,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03939\"\>",
                0.039393646784125727,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01292\"\>",
                0.01292272048156537,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03000\"\>",
                0.029999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.94000\"\>",
                0.94000000000000061,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04226\"\>",
                0.042257076778775229,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01097\"\>",
                0.010965393432094957,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02500\"\>",
                0.025000000000000005,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97500\"\>",
                0.97500000000000053,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.95000000000000051,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04226\"\>",
                0.042257076778775229,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01097\"\>",
                0.010965393432094957,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02500\"\>",
                0.025000000000000001,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97500\"\>",
                0.97500000000000053,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.95000000000000051,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04226\"\>",
                0.042257076778775195,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01097\"\>",
                0.010965393432094978,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02500\"\>",
                0.024999999999999994,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97500\"\>",
                0.97500000000000042,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.9500000000000004,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841475,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745828,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000054,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000052,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841642,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745411,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.020000000000000007,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000076,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000044,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000074,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841475,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745828,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.019999999999999997,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000054,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000052,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841586,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076245,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818866012,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000054,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000053,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076245,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818866012,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000054,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000053,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076245,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818866012,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000054,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000053,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.0099999999999999985,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923783,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440899,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000044,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000043,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04226\"\>",
                0.042257076778775229,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01097\"\>",
                0.010965393432094957,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02500\"\>",
                0.025000000000000001,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97500\"\>",
                0.97500000000000053,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.95000000000000051,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04226\"\>",
                0.042257076778775229,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01097\"\>",
                0.010965393432094957,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02500\"\>",
                0.025000000000000001,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97500\"\>",
                0.97500000000000053,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.95000000000000051,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04226\"\>",
                0.042257076778775229,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01097\"\>",
                0.010965393432094957,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02500\"\>",
                0.025000000000000001,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97500\"\>",
                0.97500000000000053,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.95000000000000051,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04226\"\>",
                0.042257076778775229,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01097\"\>",
                0.010965393432094957,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02500\"\>",
                0.025000000000000001,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97500\"\>",
                0.97500000000000053,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95000\"\>",
                0.95000000000000051,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841586,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03468\"\>",
                0.034679805572749189,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01667\"\>",
                0.016666198713633756,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04000\"\>",
                0.040000000000000001,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.92000\"\>",
                0.9200000000000006,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841565,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923949,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841586,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841586,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076336,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03269\"\>",
                0.032692602741471545,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01846\"\>",
                0.0184606442135643,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04500\"\>",
                0.044999999999999998,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95500\"\>",
                0.95500000000000063,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.91000\"\>",
                0.91000000000000059,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841586,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076315,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03269\"\>",
                0.032692602741471545,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01846\"\>",
                0.0184606442135643,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04500\"\>",
                0.044999999999999998,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95500\"\>",
                0.95500000000000063,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.91000\"\>",
                0.91000000000000059,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04967\"\>",
                0.049666467578076315,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00685\"\>",
                0.0068506201818865778,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01500\"\>",
                0.014999999999999999,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98500\"\>",
                0.98500000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97000\"\>",
                0.97000000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03468\"\>",
                0.034679805572749189,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01667\"\>",
                0.016666198713633756,
                AutoDelete->True], 
              InterpretationBox["\<\"0.04000\"\>",
                0.040000000000000001,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.92000\"\>",
                0.9200000000000006,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841586,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04753\"\>",
                0.04753174413734064,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00791\"\>",
                0.0079067242771514445,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01750\"\>",
                0.017500000000000002,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98250\"\>",
                0.9825000000000006,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96500\"\>",
                0.96500000000000064,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04561\"\>",
                0.045610485290841586,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00894\"\>",
                0.0089438186241745619,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02000\"\>",
                0.02,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96000\"\>",
                0.96000000000000063,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.03689\"\>",
                0.03689488717142201,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01482\"\>",
                0.014821473669718768,
                AutoDelete->True], 
              InterpretationBox["\<\"0.03500\"\>",
                0.035000000000000003,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96500\"\>",
                0.96500000000000064,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.93000\"\>",
                0.9300000000000006,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04331\"\>",
                0.043312365010572909,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01030\"\>",
                0.010298995562680418,
                AutoDelete->True], 
              InterpretationBox["\<\"0.02333\"\>",
                0.023333333333333334,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97667\"\>",
                0.97666666666666724,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.95333\"\>",
                0.95333333333333392,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05124\"\>",
                0.05123610193830145,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00614\"\>",
                0.0061353253473793463,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01333\"\>",
                0.013333333333333332,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98667\"\>",
                0.98666666666666725,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.97333\"\>",
                0.97333333333333394,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04822\"\>",
                0.048217165505941996,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00756\"\>",
                0.0075568641052115946,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01667\"\>",
                0.016666666666666666,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98333\"\>",
                0.98333333333333395,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96667\"\>",
                0.96666666666666723,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.04822\"\>",
                0.048217165505941961,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00756\"\>",
                0.0075568641052115946,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01667\"\>",
                0.016666666666666666,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98333\"\>",
                0.98333333333333395,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000022,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.96667\"\>",
                0.96666666666666723,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]},
            {
              InterpretationBox["\<\"0.05484\"\>",
                0.054842680104923859,
                AutoDelete->True], 
              InterpretationBox["\<\"0.00468\"\>",
                0.0046755946751440413,
                AutoDelete->True], 
              InterpretationBox["\<\"0.01000\"\>",
                0.01,
                AutoDelete->True], 
              InterpretationBox["\<\"0.99000\"\>",
                0.99000000000000066,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.50000\"\>",
                0.50000000000000033,
                AutoDelete->True], 
              InterpretationBox["\<\"0.98000\"\>",
                0.98000000000000065,
                AutoDelete->True]}
            },
          RowSpacings->1,
          ColumnSpacings->3,
          RowAlignments->Baseline,
          ColumnAlignments->{Left}],
        Function[ BoxForm`e$, 
          TableForm[ 
          BoxForm`e$, TableHeadings -> {None, {"s", "\!\(v\^\[Prime]\)", 
            "\!\(p\_0\)", "\!\(p\_1\)", 
            "\[LeftBracketingBar]\!\(\[Beta]\_0\)\!\(\(\[RightBracketingBar]\^\
2\)\)", "\!\(p\&_\)", "\[CapitalDelta]p"}}]]],
      (NumberForm[ #, {5, 5}, NumberPadding -> {"", "0"}]&)]], "Output",
  FontColor->GrayLevel[0]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
Evaluator->"psi",
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {None, Inherited, None}},
PageHeaderLines->{False, False},
PrintingOptions->{"PrintingMargins"->{{93.5625, 21.5625}, {21.5625, 36}},
"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "felix", \
"tmp"}, "0303170334.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1},
Magnification->1
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 99, 3, 45, "Subsection"],

Cell[CellGroupData[{
Cell[1863, 58, 114, 3, 42, "Subsubsection"],
Cell[1980, 63, 19404, 365, 333, "Input"],
Cell[21387, 430, 12696, 245, 129, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34120, 680, 48, 0, 42, "Subsubsection"],
Cell[34171, 682, 582, 14, 43, "Input"],
Cell[34756, 698, 1029, 28, 43, "Input"],
Cell[35788, 728, 773, 18, 27, "Input"],
Cell[36564, 748, 1334, 32, 43, "Input"],
Cell[37901, 782, 825, 18, 43, "Input"],
Cell[38729, 802, 1277, 29, 59, "Input"],
Cell[40009, 833, 1001, 22, 59, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41047, 860, 50, 0, 42, "Subsubsection"],
Cell[41100, 862, 567, 15, 43, "Input"],
Cell[41670, 879, 1324, 32, 43, "Input"],
Cell[42997, 913, 284, 7, 27, "Input"],
Cell[43284, 922, 713, 17, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[44046, 945, 63, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[44134, 949, 37, 0, 42, "Subsubsection"],
Cell[44174, 951, 113, 3, 32, "Text"],
Cell[44290, 956, 660, 17, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44987, 978, 53, 0, 42, "Subsubsection"],
Cell[45043, 980, 243, 5, 50, "Text"],
Cell[45289, 987, 139, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[45453, 994, 1252, 30, 69, "Input"],
Cell[46708, 1026, 57659, 1050, 200, 47761, 923, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[104416, 2082, 83, 1, 42, "Subsubsection"],
Cell[104502, 2085, 26, 0, 27, "Input"],
Cell[104531, 2087, 876, 25, 52, "Text"],

Cell[CellGroupData[{
Cell[105432, 2116, 1281, 31, 92, "Input"],
Cell[106716, 2149, 129004, 3231, 2655, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

