(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     96921,       2574]*)
(*NotebookOutlinePosition[     97559,       2597]*)
(*  CellTagsIndexPosition[     97515,       2593]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Some helper functions", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
      RowBox[{"NumberForm", "[", 
        RowBox[{
          RowBox[{"MatrixForm", "[", 
            RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        RowBox[{"E", "(", 
          RowBox[{"m", ",", "\[Phi]"}], ")"}], TraditionalForm]]],
  " (called ee here) directly"
}], "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"eeDirect", "[", "nn_", "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
                   "+", "1"}], "\[RightDoubleBracket]"}], "]"}], ".", 
            RowBox[{"mm", "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{
                "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "+", "1"}], "\[RightDoubleBracket]"}]}], ",", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}], "]"}], ".", 
            RowBox[{"adj", "[", "u", "]"}], ".", 
            RowBox[{"eeDirect", "[", 
              RowBox[{"nn", "-", "1"}], "]"}], ".", "u", " ", ".", 
            RowBox[{"mm", "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{
                "m", "\[LeftDoubleBracket]", "nn", "\[RightDoubleBracket]"}], 
                "+", "1"}], "\[RightDoubleBracket]"}]}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{"E", "(", 
            RowBox[{"m", ",", "\[Phi]"}], ")"}], 
          RowBox[{"j", " ", "k"}]], TraditionalForm]]],
  " (called ",
  Cell[BoxData[
      FormBox["ee", TraditionalForm]]],
  " here) according to (1-3)"
}], "Subsection"],

Cell[TextData[{
  "We use the following ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " compatible names for variables: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"ee", "=", "E"}], ",", " ", 
          RowBox[{"nn", "=", "N"}]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ee", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_"}], "]"}], ":=", 
      RowBox[{
        SubsuperscriptBox["\[Sum]", 
          RowBox[{"n", "=", "0"}], 
          RowBox[{"2", " ", 
            RowBox[{"(", 
              RowBox[{"nn", "-", "1"}], ")"}]}]], 
        RowBox[{
          RowBox[{"e", "[", 
            RowBox[{"nn", ",", "j", ",", "k", ",", "n"}], "]"}], 
          SuperscriptBox["x", "n"], 
          SuperscriptBox["y", 
            RowBox[{
              RowBox[{"2", 
                RowBox[{"(", 
                  RowBox[{"nn", "-", "1"}], ")"}]}], "-", 
              "n"}]]}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"e", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      "\[IndentingNewLine]", "\t", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"nn", "\[LessEqual]", "1"}], ",", "\[IndentingNewLine]", 
          "\t", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"adj", "[", 
                  RowBox[{"mm", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{
                      "m", "\[LeftDoubleBracket]", "1", 
                        "\[RightDoubleBracket]"}], "+", "1"}], 
                    "\[RightDoubleBracket]"}], "]"}], ".", 
                RowBox[{"mm", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{
                    "m", "\[LeftDoubleBracket]", "1", 
                      "\[RightDoubleBracket]"}], "+", "1"}], 
                  "\[RightDoubleBracket]"}]}], ")"}], "\[LeftDoubleBracket]", 
            
            RowBox[{
              RowBox[{"j", "+", "1"}], ",", 
              RowBox[{"k", "+", "1"}]}], "\[RightDoubleBracket]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{
              RowBox[{"a", "[", 
                RowBox[{"nn", ",", "j", ",", "k", ",", 
                  RowBox[{"n", "-", "2"}]}], "]"}], 
              RowBox[{"If", "[", 
                RowBox[{
                  RowBox[{"n", "\[GreaterEqual]", "2"}], ",", "1", ",", "0"}],
                 "]"}]}], "+", 
            RowBox[{
              RowBox[{"b", "[", 
                RowBox[{"nn", ",", "j", ",", "k", ",", "n"}], "]"}], 
              RowBox[{"If", "[", 
                RowBox[{
                  RowBox[{"n", "\[LessEqual]", 
                    RowBox[{
                      RowBox[{"2", 
                        RowBox[{"(", 
                          RowBox[{"nn", "-", "1"}], ")"}]}], "-", "2"}]}], 
                  ",", "1", ",", "0"}], "]"}]}], "+", 
            RowBox[{
              RowBox[{"c", "[", 
                RowBox[{"nn", ",", "j", ",", "k", ",", 
                  RowBox[{"n", "-", "1"}]}], "]"}], 
              RowBox[{"If", "[", 
                RowBox[{
                  RowBox[{
                    RowBox[{"n", "\[GreaterEqual]", "1"}], "\[And]", 
                    RowBox[{"n", "\[LessEqual]", 
                      RowBox[{
                        RowBox[{"2", 
                          RowBox[{"(", 
                            RowBox[{"nn", "-", "1"}], ")"}]}], "-", "1"}]}]}],
                   ",", "1", ",", "0"}], "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"a", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Which", "[", "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"j", "\[Equal]", "k"}], ",", 
          RowBox[{"ajj", "[", 
            RowBox[{"nn", ",", "j", ",", "n"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "0"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "1"}]}], ",", 
          RowBox[{"a01", "[", 
            RowBox[{"nn", ",", "n"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "1"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "0"}]}], ",", 
          RowBox[{"Conjugate", "[", 
            RowBox[{"a01", "[", 
              RowBox[{"nn", ",", "n"}], "]"}], "]"}]}], 
        "]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"b", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Which", "[", "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"j", "\[Equal]", "k"}], ",", 
          RowBox[{"bjj", "[", 
            RowBox[{"nn", ",", "j", ",", "n"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "0"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "1"}]}], ",", 
          RowBox[{"b01", "[", 
            RowBox[{"nn", ",", "n"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "1"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "0"}]}], ",", 
          RowBox[{"Conjugate", "[", 
            RowBox[{"b01", "[", 
              RowBox[{"nn", ",", "n"}], "]"}], "]"}]}], 
        "]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"c", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Which", "[", "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"j", "\[Equal]", "k"}], ",", 
          RowBox[{"cjj", "[", 
            RowBox[{"nn", ",", "j", ",", "n"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "0"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "1"}]}], ",", 
          RowBox[{"c01", "[", 
            RowBox[{"nn", ",", "n"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "1"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "0"}]}], ",", 
          RowBox[{"Conjugate", "[", 
            RowBox[{"c01", "[", 
              RowBox[{"nn", ",", "n"}], "]"}], "]"}]}], "]"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ajj", "[", 
        RowBox[{"nn_", ",", "j_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"0", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "]"}], "2"], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", "n"}], 
                "]"}]}], "+", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"1", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "]"}], "2"], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", "n"}], 
                "]"}]}], "+", 
            RowBox[{"2", 
              RowBox[{"Re", "[", 
                RowBox[{
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"0", "+", "1"}], ",", 
                        RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                    "]"}], 
                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"1", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  RowBox[{"e", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", 
                      "n"}], "]"}]}], "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"bjj", "[", 
        RowBox[{"nn_", ",", "j_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"0", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "]"}], "2"], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", "n"}], 
                "]"}]}], "+", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"1", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "]"}], "2"], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", "n"}], 
                "]"}]}], "+", 
            RowBox[{"2", 
              RowBox[{"Re", "[", 
                RowBox[{
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"0", "+", "1"}], ",", 
                        RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                    "]"}], 
                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"1", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"e", "[", 
                      RowBox[{
                        RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", 
                        "n"}], "]"}], "]"}]}], "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"cjj", "[", 
        RowBox[{"nn_", ",", "j_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          RowBox[{"2", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"Abs", "[", 
                          RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                            RowBox[{
                              RowBox[{"1", "+", "1"}], ",", 
                              RowBox[{"j", "+", "1"}]}], 
                            "\[RightDoubleBracket]"}], "]"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Abs", "[", 
                          RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                            RowBox[{
                              RowBox[{"0", "+", "1"}], ",", 
                              RowBox[{"j", "+", "1"}]}], 
                            "\[RightDoubleBracket]"}], "]"}], "2"]}], ")"}], 
                  RowBox[{"Im", "[", 
                    RowBox[{"e", "[", 
                      RowBox[{
                        RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", 
                        "n"}], "]"}], "]"}]}], "+", 
                RowBox[{
                  RowBox[{"Im", "[", 
                    RowBox[{
                      RowBox[{"Conjugate", "[", 
                        RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                          RowBox[{
                            RowBox[{"0", "+", "1"}], ",", 
                            RowBox[{"j", "+", "1"}]}], 
                          "\[RightDoubleBracket]"}], "]"}], 
                      RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                        RowBox[{
                          RowBox[{"1", "+", "1"}], ",", 
                          RowBox[{"j", "+", "1"}]}], 
                        "\[RightDoubleBracket]"}]}], "]"}], 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"e", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", 
                          "n"}], "]"}], "-", 
                      RowBox[{"e", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", 
                          "n"}], "]"}]}], ")"}]}]}], ")"}]}]}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"a01", "[", 
        RowBox[{"nn_", ",", "n_"}], "]"}], ":=", "\[IndentingNewLine]", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"Conjugate", "[", 
                RowBox[{"e", "[", 
                  RowBox[{
                    RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                  "]"}], "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"b01", "[", 
        RowBox[{"nn_", ",", "n_"}], "]"}], ":=", "\[IndentingNewLine]", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"Conjugate", "[", 
                RowBox[{"e", "[", 
                  RowBox[{
                    RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                  "]"}], "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"e", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"c01", "[", 
        RowBox[{"nn_", ",", "n_"}], "]"}], ":=", "\[IndentingNewLine]", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"2", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"Conjugate", "[", 
                      RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                        RowBox[{
                          RowBox[{"1", "+", "1"}], ",", 
                          RowBox[{"0", "+", "1"}]}], 
                        "\[RightDoubleBracket]"}], "]"}], 
                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"1", "+", "1"}], ",", 
                        RowBox[{"1", "+", "1"}]}], 
                      "\[RightDoubleBracket]"}]}], "-", 
                  RowBox[{
                    RowBox[{"Conjugate", "[", 
                      RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                        RowBox[{
                          RowBox[{"0", "+", "1"}], ",", 
                          RowBox[{"0", "+", "1"}]}], 
                        "\[RightDoubleBracket]"}], "]"}], 
                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"0", "+", "1"}], ",", 
                        RowBox[{"1", "+", "1"}]}], 
                      "\[RightDoubleBracket]"}]}]}], ")"}], 
              RowBox[{"Im", "[", 
                RowBox[{"e", "[", 
                  RowBox[{
                    RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                  "]"}], "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{"I", " ", 
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"e", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", 
                      "n"}], "]"}], "-", 
                  RowBox[{"e", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", 
                      "n"}], "]"}]}], ")"}]}], "\[IndentingNewLine]", "-", 
            RowBox[{"I", " ", 
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"Conjugate", "[", 
                RowBox[{
                  RowBox[{"e", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", 
                      "n"}], "]"}], "-", 
                  RowBox[{"e", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", 
                      "n"}], "]"}]}], "]"}]}]}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 1", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]},
              {
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999994,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999996,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.063\"\>",
                      0.062999999999999973,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14699999999999996,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.019\"\>",
                      0.01889999999999999,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.1\"\>",
                      0.10289999999999998,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999996,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999996,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.063\"\>",
                      0.062999999999999973,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14699999999999999,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.019\"\>",
                      0.018899999999999986,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.1\"\>",
                      0.10290000000000001,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.35\"\>",
                      0.35000000000000009,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.092\"\>",
                        -0.091651513899116868,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.092\"\>",
                        0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.15000000000000002,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.24\"\>",
                      0.23915605972938192,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.046\"\>",
                        0.04582575694955842,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.046\"\>",
                        -0.04582575694955842,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.048\"\>",
                      0.047504545830264956,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.029\"\>",
                      0.029251363749079509,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043912878474779246,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.044\"\>",
                        -0.043912878474779246,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.13\"\>",
                      0.13240924181056735,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.35\"\>",
                      0.35000000000000003,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.092\"\>",
                        -0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.092\"\>",
                        0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14999999999999997,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.24\"\>",
                      0.23915605972938181,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.046\"\>",
                        0.045825756949558413,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.046\"\>",
                        -0.045825756949558413,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.048\"\>",
                      0.047504545830264935,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.029\"\>",
                      0.029251363749079481,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043912878474779204,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.044\"\>",
                        -0.043912878474779204,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.13\"\>",
                      0.13240924181056726,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 5", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Phi]", "=", "0.13"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}]},
              {
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}], 
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurment operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"adj", "[", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"0.35", "+", 
                  RowBox[{"0.2", "I"}]}], ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"0", ",", 
                RowBox[{"0.1", "+", 
                  RowBox[{"0.4", "I"}]}]}], "}"}]}], "}"}], "]"}], ".", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"0.35", "+", 
                RowBox[{"0.2", "I"}]}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", 
              RowBox[{"0.1", "+", 
                RowBox[{"0.4", "I"}]}]}], "}"}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"0.16249999999999998`", "\[InvisibleSpace]"}], "+", 
              RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
            RowBox[{
              RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
              RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
              RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
            RowBox[{
              RowBox[{"0.17`", "\[InvisibleSpace]"}], "+", 
              RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]}], "}"}]}], 
      "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm", "=", "tmpM3"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.4\"\>",
                      0.4000000634136231,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.00008\"\>",
                      7.9632645824330266*^-05,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.00008\"\>",
                      7.9632645824330266*^-05,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.5\"\>",
                      0.49999993658637698,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.041\"\>",
                      0.041428469917695353,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0019\"\>",
                        -0.0019281648282308036,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0045\"\>",
                          0.0045433291699860179,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0019\"\>",
                        -0.0019281648282308001,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0045\"\>",
                          0.0045433291699860136,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.097\"\>",
                      0.097139953938601697,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.025786736636375415,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0076\"\>",
                        -0.0075792359025801271,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.004\"\>",
                          0.0039590982118779079,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0076\"\>",
                        -0.0075792359025801236,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.004\"\>",
                          0.0039590982118779079,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.026\"\>",
                      0.026103313692231043,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.012681766177825194,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.00054\"\>",
                        0.00054368666449428526,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0028\"\>",
                          0.0027752308352941473,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.00054\"\>",
                        0.00054368666449428516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0028\"\>",
                          0.0027752308352941455,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.01\"\>",
                      0.010093034328461871,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.4\"\>",
                      0.4000000634136231,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.00008\"\>",
                      7.9632645824330266*^-05,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.00008\"\>",
                      7.9632645824330266*^-05,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.5\"\>",
                      0.49999993658637698,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.041\"\>",
                      0.041428469917695353,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0019\"\>",
                        -0.0019281648282308014,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0045\"\>",
                          0.0045433291699860162,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0019\"\>",
                        -0.0019281648282308014,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0045\"\>",
                          0.0045433291699860162,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.097\"\>",
                      0.097139953938601711,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.025786736636375422,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0076\"\>",
                        -0.0075792359025801288,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.004\"\>",
                          0.0039590982118779096,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0076\"\>",
                        -0.0075792359025801288,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.004\"\>",
                          0.0039590982118779096,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.026\"\>",
                      0.02610331369223105,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.0126817661778252,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.00054\"\>",
                        0.00054368666449428516,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0028\"\>",
                          0.0027752308352941468,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.00054\"\>",
                        0.00054368666449428516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0028\"\>",
                          0.0027752308352941468,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.01\"\>",
                      0.010093034328461871,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 43, 0, 45, "Subsection"],
Cell[1785, 53, 198, 5, 27, "Input"],
Cell[1986, 60, 401, 10, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2424, 75, 220, 7, 45, "Subsection"],
Cell[2647, 84, 1460, 33, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4144, 122, 353, 12, 49, "Subsection"],
Cell[4500, 136, 304, 11, 32, "Text"],
Cell[4807, 149, 663, 19, 35, "Input"],
Cell[5473, 170, 2639, 63, 75, "Input"],
Cell[8115, 235, 2626, 62, 203, "Input"],
Cell[10744, 299, 2334, 55, 61, "Input"],
Cell[13081, 356, 2394, 56, 61, "Input"],
Cell[15478, 414, 2843, 63, 77, "Input"],
Cell[18324, 479, 3358, 76, 107, "Input"],
Cell[21685, 557, 3358, 76, 107, "Input"],
Cell[25046, 635, 3998, 89, 91, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29081, 729, 31, 0, 45, "Subsection"],
Cell[29115, 731, 1032, 29, 43, "Input"],

Cell[CellGroupData[{
Cell[30172, 764, 1564, 38, 123, "Input"],
Cell[31739, 804, 2876, 82, 45, "Output"],
Cell[34618, 888, 2876, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[37543, 976, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[37599, 980, 1127, 31, 59, "Input"],
Cell[38729, 1013, 1178, 34, 90, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39944, 1052, 1564, 38, 123, "Input"],
Cell[41511, 1092, 2876, 82, 45, "Output"],
Cell[44390, 1176, 2876, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[47315, 1264, 31, 0, 45, "Subsection"],
Cell[47349, 1266, 1423, 40, 55, "Input"],

Cell[CellGroupData[{
Cell[48797, 1310, 1564, 38, 123, "Input"],
Cell[50364, 1350, 3052, 82, 45, "Output"],
Cell[53419, 1434, 3053, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[56521, 1522, 31, 0, 45, "Subsection"],
Cell[56555, 1524, 1456, 41, 55, "Input"],

Cell[CellGroupData[{
Cell[58036, 1569, 1564, 38, 123, "Input"],
Cell[59603, 1609, 3555, 88, 45, "Output"],
Cell[63161, 1699, 3556, 88, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[66766, 1793, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[66822, 1797, 1653, 43, 91, "Input"],
Cell[68478, 1842, 840, 22, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[69355, 1869, 153, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[69533, 1876, 760, 22, 27, "Input"],
Cell[70296, 1900, 746, 19, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[71079, 1924, 5935, 159, 169, "Input"],
Cell[77017, 2085, 666, 20, 58, "Output"],
Cell[77686, 2107, 2474, 62, 45, "Output"],
Cell[80163, 2171, 5447, 122, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[85659, 2299, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[85722, 2303, 1564, 38, 123, "Input"],
Cell[87289, 2343, 4796, 112, 83, "Output"],
Cell[92088, 2457, 4793, 112, 83, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

