(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    138111,       3630]*)
(*NotebookOutlinePosition[    138749,       3653]*)
(*  CellTagsIndexPosition[    138705,       3649]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Some helper functions", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
      RowBox[{"NumberForm", "[", 
        RowBox[{
          RowBox[{"MatrixForm", "[", 
            RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          OverscriptBox["E", "~"], "(", 
          RowBox[{"m", ",", "\[Phi]"}], ")"}], TraditionalForm]]],
  " (called eet here) directly"
}], "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"eetDirect", "[", "nn_", "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
          RowBox[{
            RowBox[{"mm", "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{
                "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "+", "1"}], "\[RightDoubleBracket]"}], ".", 
            RowBox[{"adj", "[", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
                   "+", "1"}], "\[RightDoubleBracket]"}], "]"}]}], ",", 
          RowBox[{
            RowBox[{"mm", "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{
                "m", "\[LeftDoubleBracket]", "nn", "\[RightDoubleBracket]"}], 
                "+", "1"}], "\[RightDoubleBracket]"}], ".", "u", ".", 
            RowBox[{"eetDirect", "[", 
              RowBox[{"nn", "-", "1"}], "]"}], ".", 
            RowBox[{"adj", "[", "u", "]"}], " ", ".", 
            RowBox[{"adj", "[", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}], "]"}]}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{
            OverscriptBox["E", "~"], "(", 
            RowBox[{"m", ",", "\[Phi]"}], ")"}], 
          RowBox[{"j", " ", "k"}]], TraditionalForm]]],
  " according to (1-3)"
}], "Subsection"],

Cell[TextData[{
  "We use the following ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " compatible names for variables: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"eet", "=", 
            OverscriptBox["E", "~"]}], ",", " ", 
          RowBox[{"nn", "=", "N"}], ",", 
          RowBox[{"et", "=", 
            OverscriptBox["e", "~"]}], ",", 
          RowBox[{"at", "=", 
            OverscriptBox["a", "~"]}], ",", 
          RowBox[{"bt", "=", 
            OverscriptBox["b", "~"]}], ",", 
          RowBox[{"ct", "=", 
            OverscriptBox["c", "~"]}], ",", 
          RowBox[{"xt", "=", 
            RowBox[{
              OverscriptBox["x", "~"], "=", "x"}]}], ",", 
          RowBox[{"yt", "=", 
            RowBox[{
              OverscriptBox["y", "~"], "=", 
              RowBox[{"-", "y"}]}]}], ",", 
          RowBox[{"mmt", "=", 
            RowBox[{
              RowBox[{"{", 
                OverscriptBox["M", "~"], "}"}], "=", 
              RowBox[{"{", 
                SubsuperscriptBox["M", "j", "\[Dagger]"], "}"}]}]}]}], 
        TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"xt", "=", "x"}], ";", 
      RowBox[{"yt", "=", 
        RowBox[{"-", "y"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"eet", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmt", "=", 
              RowBox[{"Map", "[", 
                RowBox[{"adj", ",", "mm"}], "]"}]}], "}"}], ",", 
          RowBox[{
            SubsuperscriptBox["\[Sum]", 
              RowBox[{"n", "=", "0"}], 
              RowBox[{"2", " ", 
                RowBox[{"(", 
                  RowBox[{"nn", "-", "1"}], ")"}]}]], 
            RowBox[{
              RowBox[{"et", "[", 
                RowBox[{"nn", ",", "j", ",", "k", ",", "n"}], "]"}], 
              SuperscriptBox["xt", "n"], 
              SuperscriptBox["yt", 
                RowBox[{
                  RowBox[{"2", 
                    RowBox[{"(", 
                      RowBox[{"nn", "-", "1"}], ")"}]}], "-", "n"}]]}]}]}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"et", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      "\[IndentingNewLine]", "\t", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"nn", "\[LessEqual]", "1"}], ",", "\[IndentingNewLine]", 
          "\t", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"adj", "[", 
                  RowBox[{"mmt", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{
                      "m", "\[LeftDoubleBracket]", "1", 
                        "\[RightDoubleBracket]"}], "+", "1"}], 
                    "\[RightDoubleBracket]"}], "]"}], ".", 
                RowBox[{"mmt", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{
                    "m", "\[LeftDoubleBracket]", "1", 
                      "\[RightDoubleBracket]"}], "+", "1"}], 
                  "\[RightDoubleBracket]"}]}], ")"}], "\[LeftDoubleBracket]", 
            
            RowBox[{
              RowBox[{"j", "+", "1"}], ",", 
              RowBox[{"k", "+", "1"}]}], "\[RightDoubleBracket]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{
              RowBox[{"at", "[", 
                RowBox[{"nn", ",", "j", ",", "k", ",", 
                  RowBox[{"n", "-", "2"}]}], "]"}], 
              RowBox[{"If", "[", 
                RowBox[{
                  RowBox[{"n", "\[GreaterEqual]", "2"}], ",", "1", ",", "0"}],
                 "]"}]}], "+", 
            RowBox[{
              RowBox[{"bt", "[", 
                RowBox[{"nn", ",", "j", ",", "k", ",", "n"}], "]"}], 
              RowBox[{"If", "[", 
                RowBox[{
                  RowBox[{"n", "\[LessEqual]", 
                    RowBox[{
                      RowBox[{"2", 
                        RowBox[{"(", 
                          RowBox[{"nn", "-", "1"}], ")"}]}], "-", "2"}]}], 
                  ",", "1", ",", "0"}], "]"}]}], "+", 
            RowBox[{
              RowBox[{"ct", "[", 
                RowBox[{"nn", ",", "j", ",", "k", ",", 
                  RowBox[{"n", "-", "1"}]}], "]"}], 
              RowBox[{"If", "[", 
                RowBox[{
                  RowBox[{
                    RowBox[{"n", "\[GreaterEqual]", "1"}], "\[And]", 
                    RowBox[{"n", "\[LessEqual]", 
                      RowBox[{
                        RowBox[{"2", 
                          RowBox[{"(", 
                            RowBox[{"nn", "-", "1"}], ")"}]}], "-", "1"}]}]}],
                   ",", "1", ",", "0"}], "]"}]}]}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"at", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Which", "[", "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"j", "\[Equal]", "k"}], ",", 
          RowBox[{"atjj", "[", 
            RowBox[{"nn", ",", "j", ",", "n"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "0"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "1"}]}], ",", 
          RowBox[{"at01", "[", 
            RowBox[{"nn", ",", "n"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "1"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "0"}]}], ",", 
          RowBox[{"Conjugate", "[", 
            RowBox[{"at01", "[", 
              RowBox[{"nn", ",", "n"}], "]"}], "]"}]}], 
        "]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"bt", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Which", "[", "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"j", "\[Equal]", "k"}], ",", 
          RowBox[{"btjj", "[", 
            RowBox[{"nn", ",", "j", ",", "n"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "0"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "1"}]}], ",", 
          RowBox[{"bt01", "[", 
            RowBox[{"nn", ",", "n"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "1"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "0"}]}], ",", 
          RowBox[{"Conjugate", "[", 
            RowBox[{"bt01", "[", 
              RowBox[{"nn", ",", "n"}], "]"}], "]"}]}], 
        "]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"ct", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Which", "[", "\[IndentingNewLine]", 
        RowBox[{
          RowBox[{"j", "\[Equal]", "k"}], ",", 
          RowBox[{"ctjj", "[", 
            RowBox[{"nn", ",", "j", ",", "n"}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "0"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "1"}]}], ",", 
          RowBox[{"ct01", "[", 
            RowBox[{"nn", ",", "n"}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"j", "\[Equal]", "1"}], "\[And]", 
            RowBox[{"k", "\[Equal]", "0"}]}], ",", 
          RowBox[{"Conjugate", "[", 
            RowBox[{"ct01", "[", 
              RowBox[{"nn", ",", "n"}], "]"}], "]"}]}], "]"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(btjj\\)\\\" is similar to existing symbol \
\\\"\\!\\(atjj\\)\\\".\"\>"}]], "Message"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(bt01\\)\\\" is similar to existing symbol \
\\\"\\!\\(at01\\)\\\".\"\>"}]], "Message"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(ctjj\\)\\\" is similar to existing symbols \\!\\({atjj, btjj}\\).\"\
\>"}]], "Message"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(ct01\\)\\\" is similar to existing symbols \\!\\({at01, bt01}\\).\"\
\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"atjj", "[", 
        RowBox[{"nn_", ",", "j_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmtnn", "=", 
              RowBox[{"mmt", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"0", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "]"}], "2"], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", "n"}], 
                "]"}]}], "+", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"1", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "]"}], "2"], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", "n"}], 
                "]"}]}], "+", 
            RowBox[{"2", 
              RowBox[{"Re", "[", 
                RowBox[{
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"0", "+", "1"}], ",", 
                        RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                    "]"}], 
                  RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"1", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  RowBox[{"et", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", 
                      "n"}], "]"}]}], "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(mmtnn\\)\\\" is similar to existing symbol \\\"\\!\\(mmnn\\)\\\".\"\
\>"}]], "Message"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"btjj", "[", 
        RowBox[{"nn_", ",", "j_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmtnn", "=", 
              RowBox[{"mmt", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          RowBox[{
            RowBox[{
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"0", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "]"}], "2"], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", "n"}], 
                "]"}]}], "+", 
            RowBox[{
              SuperscriptBox[
                RowBox[{"Abs", "[", 
                  RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"1", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "]"}], "2"], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", "n"}], 
                "]"}]}], "+", 
            RowBox[{"2", 
              RowBox[{"Re", "[", 
                RowBox[{
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"0", "+", "1"}], ",", 
                        RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                    "]"}], 
                  RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"1", "+", "1"}], ",", 
                      RowBox[{"j", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"et", "[", 
                      RowBox[{
                        RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", 
                        "n"}], "]"}], "]"}]}], "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ctjj", "[", 
        RowBox[{"nn_", ",", "j_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmtnn", "=", 
              RowBox[{"mmt", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          RowBox[{"2", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox[
                        RowBox[{"Abs", "[", 
                          RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                            RowBox[{
                              RowBox[{"1", "+", "1"}], ",", 
                              RowBox[{"j", "+", "1"}]}], 
                            "\[RightDoubleBracket]"}], "]"}], "2"], "-", 
                      SuperscriptBox[
                        RowBox[{"Abs", "[", 
                          RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                            RowBox[{
                              RowBox[{"0", "+", "1"}], ",", 
                              RowBox[{"j", "+", "1"}]}], 
                            "\[RightDoubleBracket]"}], "]"}], "2"]}], ")"}], 
                  RowBox[{"Im", "[", 
                    RowBox[{"et", "[", 
                      RowBox[{
                        RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", 
                        "n"}], "]"}], "]"}]}], "+", 
                RowBox[{
                  RowBox[{"Im", "[", 
                    RowBox[{
                      RowBox[{"Conjugate", "[", 
                        RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                          RowBox[{
                            RowBox[{"0", "+", "1"}], ",", 
                            RowBox[{"j", "+", "1"}]}], 
                          "\[RightDoubleBracket]"}], "]"}], 
                      RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                        RowBox[{
                          RowBox[{"1", "+", "1"}], ",", 
                          RowBox[{"j", "+", "1"}]}], 
                        "\[RightDoubleBracket]"}]}], "]"}], 
                  RowBox[{"(", 
                    RowBox[{
                      RowBox[{"et", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", 
                          "n"}], "]"}], "-", 
                      RowBox[{"et", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", 
                          "n"}], "]"}]}], ")"}]}]}], ")"}]}]}], 
        "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"at01", "[", 
        RowBox[{"nn_", ",", "n_"}], "]"}], ":=", "\[IndentingNewLine]", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmtnn", "=", 
              RowBox[{"mmt", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmtnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"Conjugate", "[", 
                RowBox[{"et", "[", 
                  RowBox[{
                    RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                  "]"}], "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"bt01", "[", 
        RowBox[{"nn_", ",", "n_"}], "]"}], ":=", "\[IndentingNewLine]", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", "n"}], 
                "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"Conjugate", "[", 
                RowBox[{"et", "[", 
                  RowBox[{
                    RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                  "]"}], "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"et", "[", 
                RowBox[{
                  RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                "]"}]}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ct01", "[", 
        RowBox[{"nn_", ",", "n_"}], "]"}], ":=", "\[IndentingNewLine]", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"2", 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{
                    RowBox[{"Conjugate", "[", 
                      RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                        RowBox[{
                          RowBox[{"1", "+", "1"}], ",", 
                          RowBox[{"0", "+", "1"}]}], 
                        "\[RightDoubleBracket]"}], "]"}], 
                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"1", "+", "1"}], ",", 
                        RowBox[{"1", "+", "1"}]}], 
                      "\[RightDoubleBracket]"}]}], "-", 
                  RowBox[{
                    RowBox[{"Conjugate", "[", 
                      RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                        RowBox[{
                          RowBox[{"0", "+", "1"}], ",", 
                          RowBox[{"0", "+", "1"}]}], 
                        "\[RightDoubleBracket]"}], "]"}], 
                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"0", "+", "1"}], ",", 
                        RowBox[{"1", "+", "1"}]}], 
                      "\[RightDoubleBracket]"}]}]}], ")"}], 
              RowBox[{"Im", "[", 
                RowBox[{"et", "[", 
                  RowBox[{
                    RowBox[{"nn", "-", "1"}], ",", "0", ",", "1", ",", "n"}], 
                  "]"}], "]"}]}], "\[IndentingNewLine]", "+", 
            RowBox[{"I", " ", 
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"0", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"1", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"et", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", 
                      "n"}], "]"}], "-", 
                  RowBox[{"et", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", 
                      "n"}], "]"}]}], ")"}]}], "\[IndentingNewLine]", "-", 
            RowBox[{"I", " ", 
              RowBox[{"Conjugate", "[", 
                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"1", "+", "1"}], ",", 
                    RowBox[{"0", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}], 
              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"0", "+", "1"}], ",", 
                  RowBox[{"1", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"Conjugate", "[", 
                RowBox[{
                  RowBox[{"et", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "0", ",", "0", ",", 
                      "n"}], "]"}], "-", 
                  RowBox[{"et", "[", 
                    RowBox[{
                      RowBox[{"nn", "-", "1"}], ",", "1", ",", "1", ",", 
                      "n"}], "]"}]}], "]"}]}]}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 1", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]},
              {
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999994,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999996,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.063\"\>",
                      0.062999999999999973,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14699999999999996,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.019\"\>",
                      0.01889999999999999,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.1\"\>",
                      0.10289999999999998,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999996,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999996,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.063\"\>",
                      0.062999999999999973,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14699999999999999,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.019\"\>",
                      0.018899999999999986,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.1\"\>",
                      0.10290000000000001,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.35\"\>",
                      0.35000000000000009,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.092\"\>",
                        -0.091651513899116868,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.092\"\>",
                        0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.15000000000000002,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.24\"\>",
                      0.23915605972938192,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.046\"\>",
                        0.04582575694955842,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.046\"\>",
                        -0.04582575694955842,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.048\"\>",
                      0.047504545830264956,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.029\"\>",
                      0.029251363749079509,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043912878474779246,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.044\"\>",
                        -0.043912878474779246,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.13\"\>",
                      0.13240924181056735,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.35\"\>",
                      0.35000000000000003,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.092\"\>",
                        -0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.092\"\>",
                        0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14999999999999997,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.24\"\>",
                      0.23915605972938181,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.046\"\>",
                        0.045825756949558413,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.046\"\>",
                        -0.045825756949558413,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.048\"\>",
                      0.047504545830264935,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.029\"\>",
                      0.029251363749079481,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043912878474779204,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.044\"\>",
                        -0.043912878474779204,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.13\"\>",
                      0.13240924181056726,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 5", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Phi]", "=", "0.13"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}]},
              {
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}], 
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurment operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm", "=", "tmpM3"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eetDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"eet", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"eet", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"eet", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"eet", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.41\"\>",
                      0.41313031422293778,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.49\"\>",
                      0.4868696857770623,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.052\"\>",
                      0.051663506304440013,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.022\"\>",
                        0.021998590925409265,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0024\"\>",
                          0.002423789001482254,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.022\"\>",
                        0.021998590925409265,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0024\"\>",
                          0.0024237890014822522,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.087\"\>",
                      0.08690491755185703,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.032\"\>",
                      0.031531436515928997,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.0063\"\>",
                        0.0063150767961125485,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0014\"\>",
                          0.0014339260950117943,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.0063\"\>",
                        0.0063150767961125502,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0014\"\>",
                          0.001433926095011793,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.02\"\>",
                      0.020358613812677475,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.013181953645826644,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.000044\"\>",
                        4.4109014235668356*^-05,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0025\"\>",
                          0.0025397868101409795,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.000044\"\>",
                        4.4109014235667096*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0025\"\>",
                          0.0025397868101409795,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.0096\"\>",
                      0.0095928468604604255,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.4\"\>",
                      0.4000000634136231,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.00008\"\>",
                      7.9632645824330266*^-05,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.00008\"\>",
                      7.9632645824330266*^-05,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.5\"\>",
                      0.49999993658637698,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.041\"\>",
                      0.041424375521980743,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0045\"\>",
                        -0.0044989638261928042,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.002\"\>",
                          0.0019725269115463081,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0045\"\>",
                        -0.0044989638261928042,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.002\"\>",
                          0.0019725269115463081,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.097\"\>",
                      0.097144048334316321,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026034766356466635,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0083\"\>",
                        -0.0082998262997453984,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0026\"\>",
                          0.0026161169545578472,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0083\"\>",
                        -0.0082998262997453984,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0026\"\>",
                          0.0026161169545578472,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.026\"\>",
                      0.025954954819789631,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.01284678203343529,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0021\"\>",
                        -0.0021374743364740031,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0018\"\>",
                          0.0017523764314350424,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0021\"\>",
                        -0.0021374743364740031,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0018\"\>",
                          0.0017523764314350424,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.0099\"\>",
                      0.0099355324441822591,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 6", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Phi]", "=", "0.13"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"adj", "[", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                  RowBox[{"I", " ", 
                    RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"I", " ", 
                    RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}], 
          "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"-0.13\"\>",
                    -0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}]},
              {
                RowBox[{
                  InterpretationBox["\<\"-0.13\"\>",
                    -0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}], 
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurment operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"adj", "[", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"0.35", "+", 
                  RowBox[{"0.2", "I"}]}], ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"0", ",", 
                RowBox[{"0.1", "+", 
                  RowBox[{"0.4", "I"}]}]}], "}"}]}], "}"}], "]"}], ".", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"0.35", "+", 
                RowBox[{"0.2", "I"}]}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", 
              RowBox[{"0.1", "+", 
                RowBox[{"0.4", "I"}]}]}], "}"}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"0.16249999999999998`", "\[InvisibleSpace]"}], "+", 
              RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
            RowBox[{
              RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
              RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{
              RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
              RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
            RowBox[{
              RowBox[{"0.17`", "\[InvisibleSpace]"}], "+", 
              RowBox[{"0.`", " ", "\[ImaginaryI]"}]}]}], "}"}]}], 
      "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"mm", "=", 
          RowBox[{"Map", "[", 
            RowBox[{"adj", ",", "tmpM3"}], "]"}]}], ";"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
        RowBox[{
          RowBox[{"mm", "=", "tmpM3"}], ";"}], 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eeDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"ee", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.41\"\>",
                      0.41313031422293778,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.49\"\>",
                      0.4868696857770623,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.052\"\>",
                      0.051663506304440013,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.022\"\>",
                        0.021998590925409265,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0024\"\>",
                          0.002423789001482254,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.022\"\>",
                        0.021998590925409265,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0024\"\>",
                          0.0024237890014822522,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.087\"\>",
                      0.08690491755185703,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.032\"\>",
                      0.031531436515928997,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.0063\"\>",
                        0.0063150767961125485,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0014\"\>",
                          0.0014339260950117943,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.0063\"\>",
                        0.0063150767961125502,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0014\"\>",
                          0.001433926095011793,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.02\"\>",
                      0.020358613812677475,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.013181953645826644,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.000044\"\>",
                        4.4109014235668356*^-05,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0025\"\>",
                          0.0025397868101409795,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.000044\"\>",
                        4.4109014235667096*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0025\"\>",
                          0.0025397868101409795,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.0096\"\>",
                      0.0095928468604604255,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.41\"\>",
                      0.41313031422293778,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.49\"\>",
                      0.4868696857770623,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.05\"\>",
                      0.050383033567365267,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.021\"\>",
                        0.020600716731185409,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0043\"\>",
                          0.004319484835013758,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.021\"\>",
                        0.020600716731185409,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0043\"\>",
                          0.004319484835013758,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.088\"\>",
                      0.088185390288931798,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.032\"\>",
                      0.031566745746870205,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.006\"\>",
                        0.006004100232635973,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0029\"\>",
                          0.0029389884604175834,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.006\"\>",
                        0.006004100232635973,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0029\"\>",
                          0.0029389884604175834,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.02\"\>",
                      0.020422975429386057,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.012899101563281267,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00051\"\>",
                        -0.00051383783006243573,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0027\"\>",
                          0.0026871058479251276,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.00051\"\>",
                        -0.00051383783006243573,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0027\"\>",
                          0.0026871058479251276,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.0099\"\>",
                      0.0098832129143362826,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]],

Cell["Adjungated:", "Text"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.41\"\>",
                      0.41313031422293778,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.49\"\>",
                      0.4868696857770623,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.05\"\>",
                      0.050383033567365267,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.021\"\>",
                        0.020600716731185416,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0043\"\>",
                          0.004319484835013758,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.021\"\>",
                        0.020600716731185416,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0043\"\>",
                          0.0043194848350137571,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.088\"\>",
                      0.088185390288931798,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.032\"\>",
                      0.031566745746870205,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.006\"\>",
                        0.0060041002326359739,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0029\"\>",
                          0.0029389884604175838,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.006\"\>",
                        0.006004100232635973,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0029\"\>",
                          0.0029389884604175808,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.02\"\>",
                      0.020422975429386061,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.012899101563281267,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00051\"\>",
                        -0.00051383783006243465,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0027\"\>",
                          0.0026871058479251294,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.00051\"\>",
                        -0.000513837830062434,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0027\"\>",
                          0.0026871058479251272,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.0099\"\>",
                      0.0098832129143362826,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell["Original:", "Text"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.4\"\>",
                      0.4000000634136231,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.00008\"\>",
                      7.9632645824330266*^-05,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.00008\"\>",
                      7.9632645824330266*^-05,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.5\"\>",
                      0.49999993658637698,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.041\"\>",
                      0.041428469917695353,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0019\"\>",
                        -0.0019281648282308014,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0045\"\>",
                          0.0045433291699860162,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0019\"\>",
                        -0.0019281648282308014,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0045\"\>",
                          0.0045433291699860162,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.097\"\>",
                      0.097139953938601711,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.025786736636375422,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.0076\"\>",
                        -0.0075792359025801288,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.004\"\>",
                          0.0039590982118779096,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.0076\"\>",
                        -0.0075792359025801288,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.004\"\>",
                          0.0039590982118779096,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.026\"\>",
                      0.02610331369223105,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.0126817661778252,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.00054\"\>",
                        0.00054368666449428516,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0028\"\>",
                          0.0027752308352941468,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.00054\"\>",
                        0.00054368666449428516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0028\"\>",
                          0.0027752308352941468,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.01\"\>",
                      0.010093034328461871,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 43, 0, 45, "Subsection"],
Cell[1785, 53, 198, 5, 27, "Input"],
Cell[1986, 60, 401, 10, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2424, 75, 252, 8, 47, "Subsection"],
Cell[2679, 85, 1462, 33, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4178, 123, 309, 10, 51, "Subsection"],
Cell[4490, 135, 1134, 34, 35, "Text"],
Cell[5627, 171, 137, 4, 27, "Input"],
Cell[5767, 177, 934, 25, 35, "Input"],
Cell[6704, 204, 2645, 63, 75, "Input"],

Cell[CellGroupData[{
Cell[9374, 271, 2638, 62, 203, "Input"],
Cell[12015, 335, 256, 6, 23, "Message"],
Cell[12274, 343, 256, 6, 23, "Message"],
Cell[12533, 351, 256, 6, 23, "Message"],
Cell[12792, 359, 256, 6, 23, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13085, 370, 2344, 55, 61, "Input"],
Cell[15432, 427, 257, 6, 23, "Message"]
}, Open  ]],
Cell[15704, 436, 2404, 56, 61, "Input"],
Cell[18111, 494, 2853, 63, 77, "Input"],
Cell[20967, 559, 3373, 76, 107, "Input"],
Cell[24343, 637, 3363, 76, 107, "Input"],
Cell[27709, 715, 4004, 89, 91, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31750, 809, 31, 0, 45, "Subsection"],
Cell[31784, 811, 1032, 29, 43, "Input"],

Cell[CellGroupData[{
Cell[32841, 844, 1564, 38, 123, "Input"],
Cell[34408, 884, 2876, 82, 45, "Output"],
Cell[37287, 968, 2876, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[40212, 1056, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[40268, 1060, 1127, 31, 59, "Input"],
Cell[41398, 1093, 1178, 34, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42613, 1132, 1564, 38, 123, "Input"],
Cell[44180, 1172, 2876, 82, 45, "Output"],
Cell[47059, 1256, 2876, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[49984, 1344, 31, 0, 45, "Subsection"],
Cell[50018, 1346, 1423, 40, 55, "Input"],

Cell[CellGroupData[{
Cell[51466, 1390, 1564, 38, 123, "Input"],
Cell[53033, 1430, 3052, 82, 45, "Output"],
Cell[56088, 1514, 3053, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[59190, 1602, 31, 0, 45, "Subsection"],
Cell[59224, 1604, 1456, 41, 55, "Input"],

Cell[CellGroupData[{
Cell[60705, 1649, 1564, 38, 123, "Input"],
Cell[62272, 1689, 3555, 88, 45, "Output"],
Cell[65830, 1779, 3556, 88, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[69435, 1873, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[69491, 1877, 1653, 43, 91, "Input"],
Cell[71147, 1922, 840, 22, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[72024, 1949, 153, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[72202, 1956, 5935, 159, 169, "Input"],
Cell[78140, 2117, 666, 20, 58, "Output"],
Cell[78809, 2139, 2474, 62, 45, "Output"],
Cell[81286, 2203, 5447, 122, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[86782, 2331, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[86845, 2335, 1569, 38, 123, "Input"],
Cell[88417, 2375, 4781, 112, 83, "Output"],
Cell[93201, 2489, 4798, 112, 83, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[98060, 2608, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[98116, 2612, 1717, 44, 91, "Input"],
Cell[99836, 2658, 844, 22, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[100717, 2685, 153, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[100895, 2692, 760, 22, 27, "Input"],
Cell[101658, 2716, 746, 19, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[102441, 2740, 6160, 167, 185, "Input"],
Cell[108604, 2909, 666, 20, 58, "Output"],
Cell[109273, 2931, 2474, 62, 45, "Output"],
Cell[111750, 2995, 5447, 122, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[117246, 3123, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[117309, 3127, 1564, 38, 123, "Input"],
Cell[118876, 3167, 4781, 112, 83, "Output"],
Cell[123660, 3281, 4777, 112, 45, "Output"]
}, Open  ]],
Cell[128452, 3396, 27, 0, 32, "Text"],
Cell[128482, 3398, 4777, 112, 45, "Output"],
Cell[133262, 3512, 25, 0, 32, "Text"],
Cell[133290, 3514, 4793, 112, 83, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

