(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     81119,       2170]*)
(*NotebookOutlinePosition[     81757,       2193]*)
(*  CellTagsIndexPosition[     81713,       2189]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Some helper functions", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
      RowBox[{"NumberForm", "[", 
        RowBox[{
          RowBox[{"MatrixForm", "[", 
            RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        RowBox[{"M", "(", 
          RowBox[{
            OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], ")"}],
         TraditionalForm]]],
  " (called mmm here) directly"
}], "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mmmDirect", "[", "nn_", "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
          RowBox[{"mm", "\[LeftDoubleBracket]", 
            RowBox[{
              RowBox[{
              "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
              "+", "1"}], "\[RightDoubleBracket]"}], ",", 
          RowBox[{
            RowBox[{"mm", "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{
                "m", "\[LeftDoubleBracket]", "nn", "\[RightDoubleBracket]"}], 
                "+", "1"}], "\[RightDoubleBracket]"}], ".", "u", ".", 
            RowBox[{"mmmDirect", "[", 
              RowBox[{"nn", "-", "1"}], "]"}]}]}], "]"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
        RowBox[{
        "mm", " ", "is", " ", "the", " ", "set", " ", "of", " ", 
          "measurement", " ", "operators", " ", "for", " ", "a", " ", 
          "single", " ", "measurement"}], " ", "*)"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{"E", "(", 
            RowBox[{"m", ",", "\[Phi]"}], ")"}], 
          RowBox[{"j", " ", "k"}]], TraditionalForm]]],
  " (called ",
  Cell[BoxData[
      FormBox["ee", TraditionalForm]]],
  " here) according to (1-3)"
}], "Subsection"],

Cell[TextData[{
  "We use the following ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " compatible names for variables: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"mmm", "=", 
            RowBox[{"M", "(", 
              RowBox[{
                OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
              ")"}]}], ",", " ", 
          RowBox[{"nn", "=", "N"}]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mmm", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_"}], "]"}], ":=", 
      RowBox[{
        SubsuperscriptBox["\[Sum]", 
          RowBox[{"n", "=", "0"}], 
          RowBox[{"(", 
            RowBox[{"nn", "-", "1"}], ")"}]], 
        RowBox[{
          RowBox[{"a", "[", 
            RowBox[{"nn", ",", "j", ",", "k", ",", "n"}], "]"}], 
          SuperscriptBox["x", "n"], 
          SuperscriptBox["y", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"nn", "-", "1"}], ")"}], "-", "n"}]]}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"a", "[", 
          RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
        "\[IndentingNewLine]", "\t", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"nn", "\[LessEqual]", "1"}], ",", "\[IndentingNewLine]", 
            "\t", 
            RowBox[{
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
                   "+", "1"}], "\[RightDoubleBracket]"}], 
              "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{"j", "+", "1"}], ",", 
                RowBox[{"k", "+", "1"}]}], "\[RightDoubleBracket]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
              SubsuperscriptBox["\[Sum]", 
                RowBox[{"p", "=", "0"}], "1"], 
              RowBox[{
                RowBox[{
                  RowBox[{"mm", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{
                      "m", "\[LeftDoubleBracket]", "nn", 
                        "\[RightDoubleBracket]"}], "+", "1"}], 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"j", "+", "1"}], ",", 
                    RowBox[{"p", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"a", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", "p", ",", "k", ",", 
                          RowBox[{"n", "-", "1"}]}], "]"}], 
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{"n", "\[GreaterEqual]", "1"}], ",", "1", 
                          ",", "0"}], "]"}]}], "+", 
                    RowBox[{"I", " ", 
                      RowBox[{"a", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", 
                          RowBox[{"1", "-", "p"}], ",", "k", ",", "n"}], 
                        "]"}], 
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{"n", "\[LessEqual]", 
                            RowBox[{"nn", "-", "2"}]}], ",", "1", ",", "0"}], 
                        "]"}]}]}], ")"}]}]}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 1", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]},
              {
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], "\[ImaginaryI]"},
                  {"\[ImaginaryI]", 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], "\[ImaginaryI]"},
                  {"\[ImaginaryI]", 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.25\"\>",
                      0.25099800796022265,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.38\"\>",
                      0.38340579025361626,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.14\"\>",
                      0.13747727084867517,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32078029864690877,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.25\"\>",
                      0.25099800796022265,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.38\"\>",
                      0.38340579025361626,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.14\"\>",
                      0.13747727084867517,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32078029864690877,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.49\"\>",
                        0.49497474683058335,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.21213203435596426,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.066\"\>",
                      0.066203891146696836,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.48\"\>",
                        0.48453390441373473,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20765738760588626,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.066\"\>",
                      -0.066203891146696836,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.055\"\>",
                      -0.054784703604925045,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.33\"\>",
                      -0.32582087381039188,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.49\"\>",
                        0.49497474683058335,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.21213203435596423,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.066\"\>",
                      0.066203891146696836,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.48\"\>",
                        0.48453390441373467,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20765738760588628,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.066\"\>",
                      -0.066203891146696808,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.055\"\>",
                      -0.054784703604925045,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.33\"\>",
                      -0.32582087381039182,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 5", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Phi]", "=", "0.13"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}]},
              {
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}], 
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurement operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm", "=", "tmpM3"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.12\"\>",
                        0.11556295100560166,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.079\"\>",
                          0.078983647783652591,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.14\"\>",
                        -0.14476493145406052,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1054201530221143,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.082\"\>",
                        -0.081812303125227859,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.14\"\>",
                          0.13941908241666626,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.23137397334981527,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.085\"\>",
                          0.085088592206103958,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.038826146894781255,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.06\"\>",
                          0.059520299257066998,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15679433193217271,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.044\"\>",
                          0.043552706959805565,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.017\"\>",
                        -0.017078499979601813,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11321981049548259,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.078\"\>",
                        -0.078018284873153987,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.034\"\>",
                          0.034078738062648661,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.053101810590524066,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.016\"\>",
                          0.015613123710853237,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.085\"\>",
                        -0.084827625921655186,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.054\"\>",
                          0.05406159086005586,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043739384895579103,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.059\"\>",
                          0.059134992020686943,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.045\"\>",
                        0.045357994142567352,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.046\"\>",
                          0.046102257596487922,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.12\"\>",
                        0.11556295100560164,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.079\"\>",
                          0.078983647783652605,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.14\"\>",
                        -0.14476493145406055,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1054201530221143,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.082\"\>",
                        -0.081812303125227887,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.14\"\>",
                          0.13941908241666626,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2313739733498153,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.085\"\>",
                          0.085088592206103958,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.03882614689478122,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.06\"\>",
                          0.059520299257066971,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15679433193217271,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.044\"\>",
                          0.043552706959805586,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.017\"\>",
                        -0.017078499979601827,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1132198104954826,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.078\"\>",
                        -0.078018284873153987,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.034\"\>",
                          0.034078738062648661,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.053101810590524053,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.016\"\>",
                          0.015613123710853214,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.085\"\>",
                        -0.084827625921655214,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.054\"\>",
                          0.054061590860055866,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043739384895579124,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.059\"\>",
                          0.059134992020686949,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.045\"\>",
                        0.045357994142567359,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.046\"\>",
                          0.046102257596487929,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 43, 0, 45, "Subsection"],
Cell[1785, 53, 198, 5, 27, "Input"],
Cell[1986, 60, 401, 10, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2424, 75, 281, 9, 45, "Subsection"],
Cell[2708, 86, 1046, 24, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3791, 115, 353, 12, 49, "Subsection"],
Cell[4147, 129, 453, 15, 32, "Text"],
Cell[4603, 146, 580, 16, 35, "Input"],
Cell[5186, 164, 2497, 56, 95, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7720, 225, 31, 0, 45, "Subsection"],
Cell[7754, 227, 1032, 29, 43, "Input"],

Cell[CellGroupData[{
Cell[8811, 260, 1569, 38, 123, "Input"],
Cell[10383, 300, 2876, 82, 45, "Output"],
Cell[13262, 384, 2876, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[16187, 472, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[16243, 476, 1127, 31, 59, "Input"],
Cell[17373, 509, 1178, 34, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18588, 548, 1569, 38, 123, "Input"],
Cell[20160, 588, 3964, 106, 77, "Output"],
Cell[24127, 696, 3964, 106, 77, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[28140, 808, 31, 0, 45, "Subsection"],
Cell[28174, 810, 1423, 40, 55, "Input"],

Cell[CellGroupData[{
Cell[29622, 854, 1569, 38, 123, "Input"],
Cell[31194, 894, 3052, 82, 45, "Output"],
Cell[34249, 978, 3052, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[37350, 1066, 31, 0, 45, "Subsection"],
Cell[37384, 1068, 1456, 41, 55, "Input"],

Cell[CellGroupData[{
Cell[38865, 1113, 1569, 38, 123, "Input"],
Cell[40437, 1153, 3550, 88, 45, "Output"],
Cell[43990, 1243, 3550, 88, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[47589, 1337, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[47645, 1341, 1653, 43, 91, "Input"],
Cell[49301, 1386, 840, 22, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50178, 1413, 154, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[50357, 1420, 5935, 159, 162, "Input"],
Cell[56295, 1581, 666, 20, 45, "Output"],
Cell[56964, 1603, 2474, 62, 45, "Output"],
Cell[59441, 1667, 5447, 122, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[64937, 1795, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[65000, 1799, 1569, 38, 123, "Input"],
Cell[66572, 1839, 7253, 162, 45, "Output"],
Cell[73828, 2003, 7251, 162, 45, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

