(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     55210,       1507]*)
(*NotebookOutlinePosition[     56161,       1537]*)
(*  CellTagsIndexPosition[     56117,       1533]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Some helper functions", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
      RowBox[{"NumberForm", "[", 
        RowBox[{
          RowBox[{"MatrixForm", "[", 
            RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          OverscriptBox[
            OverscriptBox["E", "~"], "_"], "(", 
          OverscriptBox["m", "\[RightVector]"], ")"}], TraditionalForm]]],
  " (called eetb here) directly"
}], "Subsection"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        RowBox[{"mmm", "=", 
          RowBox[{"M", "(", 
            RowBox[{
              OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
            ")"}]}], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        RowBox[{"mm", "=", 
          RowBox[{"{", 
            RowBox[{
              SubscriptBox["M", "0"], ",", 
              SubscriptBox["M", "1"], ",", "\[Ellipsis]"}], "}"}]}], 
        TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        RowBox[{"eet", "=", 
          RowBox[{
            OverscriptBox["E", "~"], "(", 
            RowBox[{
              OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
            ")"}]}], TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        RowBox[{"nn", "=", "N"}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[CurlyPhi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[CurlyPhi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[CurlyPhi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[CurlyPhi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mmmDirect", "[", "nn_", "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
          RowBox[{"mm", "\[LeftDoubleBracket]", 
            RowBox[{
              RowBox[{
              "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
              "+", "1"}], "\[RightDoubleBracket]"}], ",", 
          RowBox[{
            RowBox[{"mm", "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{
                "m", "\[LeftDoubleBracket]", "nn", "\[RightDoubleBracket]"}], 
                "+", "1"}], "\[RightDoubleBracket]"}], ".", "u", ".", 
            RowBox[{"mmmDirect", "[", 
              RowBox[{"nn", "-", "1"}], "]"}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"eetDirect", "[", "nn_", "]"}], ":=", 
      RowBox[{"ComplexExpand", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"mmmDirect", "[", "nn", "]"}], ".", 
            RowBox[{"adj", "[", 
              RowBox[{"mmmDirect", "[", "nn", "]"}], "]"}]}], ",", 
          RowBox[{"\[CurlyPhi]", "\[GreaterEqual]", "0"}]}], "]"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"eetbDirect", "[", "nn_", "]"}], ":=", 
      RowBox[{
        RowBox[{"1", "/", 
          RowBox[{"(", 
            RowBox[{"2", "\[Pi]"}], ")"}]}], " ", "*", " ", 
        RowBox[{"Integrate", "[", 
          RowBox[{
            RowBox[{"eetDirect", "[", "nn", "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[CurlyPhi]", ",", "0", ",", 
                RowBox[{"2", "\[Pi]"}]}], "}"}]}], "]"}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(eetbDirect\\)\\\" is similar to existing symbol \\\"\\!\\(eetDirect\
\\)\\\".\"\>"}]], "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{
            OverscriptBox[
              OverscriptBox["E", "~"], "_"], "(", "m", ")"}], 
          RowBox[{"j", " ", "k"}]], TraditionalForm]]],
  " (called ",
  Cell[BoxData[
      FormBox["eetb", TraditionalForm]]],
  " here) according to (1-4)"
}], "Subsection"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        RowBox[{"iisc2", "=", 
          SubscriptBox["I", "sc2"]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"eetb", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_"}], "]"}], ":=", 
      RowBox[{
        FractionBox["1", 
          RowBox[{"2", "\[Pi]"}]], 
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"p", "=", "0"}], "1"], 
          RowBox[{
            SubsuperscriptBox["\[Sum]", 
              RowBox[{"n", "=", "0"}], 
              RowBox[{"nn", "-", "1"}]], 
            RowBox[{
              RowBox[{"a", "[", 
                RowBox[{"nn", ",", "j", ",", "p", ",", "n"}], "]"}], 
              RowBox[{
                SubsuperscriptBox["\[Sum]", 
                  RowBox[{"q", "=", 
                    RowBox[{"Floor", "[", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{"n", "+", "1"}], ")"}], "/", "2"}], "]"}]}],
                   
                  RowBox[{"Ceiling", "[", 
                    RowBox[{
                      RowBox[{"(", 
                        RowBox[{"n", "+", "nn", "-", "2"}], ")"}], "/", "2"}],
                     "]"}]], 
                RowBox[{
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"a", "[", 
                      RowBox[{"nn", ",", "k", ",", "p", ",", 
                        RowBox[{
                          RowBox[{"2", "q"}], "-", "n"}]}], "]"}], "]"}], 
                  RowBox[{"iisc2", "[", 
                    RowBox[{"q", ",", 
                      RowBox[{
                        RowBox[{"(", 
                          RowBox[{"nn", "-", "1"}], ")"}], "-", "q"}]}], 
                    "]"}]}]}]}]}]}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"iisc2", "[", 
        RowBox[{"n_", ",", "m_"}], "]"}], ":=", 
      RowBox[{
        FractionBox["2", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"n", "+", "m"}], ")"}], "!"}]], 
        RowBox[{"Gamma", "[", 
          RowBox[{"n", "+", 
            FractionBox["1", "2"]}], "]"}], 
        RowBox[{"Gamma", "[", 
          RowBox[{"m", "+", 
            FractionBox["1", "2"]}], "]"}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"a", "[", 
          RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
        "\[IndentingNewLine]", "\t", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"nn", "\[LessEqual]", "1"}], ",", "\[IndentingNewLine]", 
            "\t", 
            RowBox[{
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
                   "+", "1"}], "\[RightDoubleBracket]"}], 
              "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{"j", "+", "1"}], ",", 
                RowBox[{"k", "+", "1"}]}], "\[RightDoubleBracket]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
              SubsuperscriptBox["\[Sum]", 
                RowBox[{"p", "=", "0"}], "1"], 
              RowBox[{
                RowBox[{
                  RowBox[{"mm", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{
                      "m", "\[LeftDoubleBracket]", "nn", 
                        "\[RightDoubleBracket]"}], "+", "1"}], 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"j", "+", "1"}], ",", 
                    RowBox[{"p", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"a", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", "p", ",", "k", ",", 
                          RowBox[{"n", "-", "1"}]}], "]"}], 
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{"n", "\[GreaterEqual]", "1"}], ",", "1", 
                          ",", "0"}], "]"}]}], "+", 
                    RowBox[{"I", " ", 
                      RowBox[{"a", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", 
                          RowBox[{"1", "-", "p"}], ",", "k", ",", "n"}], 
                        "]"}], 
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{"n", "\[LessEqual]", 
                            RowBox[{"nn", "-", "2"}]}], ",", "1", ",", "0"}], 
                        "]"}]}]}], ")"}]}]}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 1", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eetbDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eetbDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.35\"\>",
                      0.35000000000000003,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14999999999999997,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.074\"\>",
                      0.073747727084867501,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.18\"\>",
                      0.17792197013530911,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.04\"\>",
                      0.039576591040592711,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.084\"\>",
                      0.083823408959407264,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999999,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.35\"\>",
                      0.35000000000000003,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14999999999999997,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.074\"\>",
                      0.073747727084867529,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.18\"\>",
                      0.17792197013530914,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.04\"\>",
                      0.039576591040592732,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.084\"\>",
                      0.083823408959407264,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurment operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm", "=", "tmpM3"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(tmpW\\)\\\" is similar to existing symbol \
\\\"\\!\\(tmpU\\)\\\".\"\>"}]], "Message"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"eetbDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"eetb", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.41\"\>",
                      0.41313031422293778,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.49\"\>",
                      0.4868696857770623,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.053\"\>",
                      0.053186630952723926,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.018\"\>",
                        0.017693779791240111,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0034\"\>",
                          0.003377311619398997,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.018\"\>",
                        0.017693779791240108,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0034\"\>",
                          0.0033773116193989996,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.082\"\>",
                      0.08181874793929847,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.037\"\>",
                      0.036529624881712616,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.00057\"\>",
                        0.00056696449986726812,
                        AutoDelete->True], "-", 
                      RowBox[{
                        RowBox[{"(", 
                          InterpretationBox[
                            RowBox[{"\<\"5.9\"\>", "\[Times]", 
                              SuperscriptBox["10", "\<\"-6\"\>"]}],
                            5.8817977527866798*^-06,
                            AutoDelete->True], ")"}], " ", 
                        "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.00057\"\>",
                        0.00056696449986726942,
                        AutoDelete->True], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          InterpretationBox[
                            RowBox[{"\<\"5.9\"\>", "\[Times]", 
                              SuperscriptBox["10", "\<\"-6\"\>"]}],
                            5.8817977527866103*^-06,
                            AutoDelete->True], ")"}], " ", 
                        "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.017\"\>",
                      0.017427379335290216,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.011\"\>",
                      0.011492035993945439,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.00051\"\>",
                        0.00051171323305065929,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.00029\"\>",
                          0.00029330706776582087,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.00051\"\>",
                        0.00051171323305065929,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00029\"\>",
                          0.00029330706776582065,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.012661480057983817,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.41\"\>",
                      0.41313031422293772,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557537,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557537,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.49\"\>",
                      0.4868696857770623,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.053\"\>",
                      0.053186630952723933,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.018\"\>",
                        0.017693779791240108,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0034\"\>",
                          0.0033773116193989965,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.018\"\>",
                        0.017693779791240108,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0034\"\>",
                          0.0033773116193990009,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.082\"\>",
                      0.081818747939298511,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.037\"\>",
                      0.036529624881712616,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.00057\"\>",
                        0.00056696449986726671,
                        AutoDelete->True], "-", 
                      RowBox[{
                        RowBox[{"(", 
                          InterpretationBox[
                            RowBox[{"\<\"5.9\"\>", "\[Times]", 
                              SuperscriptBox["10", "\<\"-6\"\>"]}],
                            5.8817977527854372*^-06,
                            AutoDelete->True], ")"}], " ", 
                        "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.00057\"\>",
                        0.00056696449986726888,
                        AutoDelete->True], "+", 
                      RowBox[{
                        RowBox[{"(", 
                          InterpretationBox[
                            RowBox[{"\<\"5.9\"\>", "\[Times]", 
                              SuperscriptBox["10", "\<\"-6\"\>"]}],
                            5.8817977527865417*^-06,
                            AutoDelete->True], ")"}], " ", 
                        "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.017\"\>",
                      0.017427379335290226,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.011\"\>",
                      0.011492035993945439,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.00051\"\>",
                        0.00051171323305065854,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.00029\"\>",
                          0.00029330706776582147,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.00051\"\>",
                        0.00051171323305065821,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00029\"\>",
                          0.0002933070677658212,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.01266148005798382,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "felix", \
"uni", "wip", "calculations"}, "0306271600.nb.ps", CharacterEncoding -> \
"ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 43, 0, 45, "Subsection"],
Cell[1785, 53, 198, 5, 27, "Input"],
Cell[1986, 60, 401, 10, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2424, 75, 294, 9, 50, "Subsection"],
Cell[2721, 86, 873, 31, 33, "Text"],
Cell[3597, 119, 571, 15, 27, "Input"],
Cell[4171, 136, 780, 18, 27, "Input"],
Cell[4954, 156, 390, 9, 27, "Input"],

Cell[CellGroupData[{
Cell[5369, 169, 469, 12, 27, "Input"],
Cell[5841, 183, 267, 6, 23, "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6157, 195, 385, 13, 53, "Subsection"],
Cell[6545, 210, 153, 6, 32, "Text"],
Cell[6701, 218, 1644, 41, 44, "Input"],
Cell[8348, 261, 468, 14, 46, "Input"],
Cell[8819, 277, 2497, 56, 95, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11353, 338, 31, 0, 45, "Subsection"],
Cell[11387, 340, 483, 15, 43, "Input"],

Cell[CellGroupData[{
Cell[11895, 359, 1574, 38, 123, "Input"],
Cell[13472, 399, 2876, 82, 45, "Output"],
Cell[16351, 483, 2876, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[19276, 571, 31, 0, 45, "Subsection"],
Cell[19310, 573, 874, 26, 55, "Input"],

Cell[CellGroupData[{
Cell[20209, 603, 1574, 38, 123, "Input"],
Cell[21786, 643, 3055, 82, 45, "Output"],
Cell[24844, 727, 3055, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[27948, 815, 31, 0, 45, "Subsection"],
Cell[27982, 817, 161, 5, 27, "Input"],

Cell[CellGroupData[{
Cell[28168, 826, 153, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[28346, 833, 5935, 159, 169, "Input"],
Cell[34284, 994, 256, 6, 23, "Message"],
Cell[34543, 1002, 666, 20, 58, "Output"],
Cell[35212, 1024, 2474, 62, 45, "Output"],
Cell[37689, 1088, 5447, 122, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[43185, 1216, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[43248, 1220, 1574, 38, 123, "Input"],
Cell[44825, 1260, 5171, 120, 87, "Output"],
Cell[49999, 1382, 5171, 120, 87, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

