(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    173851,       4202]*)
(*NotebookOutlinePosition[    174489,       4225]*)
(*  CellTagsIndexPosition[    174445,       4221]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Some helper functions", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
      RowBox[{"NumberForm", "[", 
        RowBox[{
          RowBox[{"MatrixForm", "[", 
            RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"tForm", "[", "x_", "]"}], ":=", 
      RowBox[{"TableForm", "[", 
        RowBox[{"x", ",", 
          RowBox[{"TableDepth", "\[Rule]", "2"}], ",", 
          RowBox[{"TableHeadings", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"None", ",", 
                RowBox[{"{", 
                  RowBox[{
                  "\"\<n\>\"", ",", 
                    "\"\<M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\
\(0\\\\\\\ 0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\\\\
\\\ 0\)\%*\)\), M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\\\\\\
\ 0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\\\\\\ \
1\)\%*\)\), \[Ellipsis], M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\
\(1\\\\\\\ 1\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\\\\
\\\ 1\)\%*\)\)\>\""}], "}"}]}], "}"}]}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mmmmOut", "[", 
        RowBox[{"mmmmCreator_", ",", "nnMax_"}], "]"}], ":=", 
      RowBox[{"tForm", "[", 
        RowBox[{"Table", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"n", ",", 
                RowBox[{"NumberForm", "[", 
                  RowBox[{
                    RowBox[{"Chop", "[", 
                      RowBox[{
                        RowBox[{"Flatten", "[", 
                          RowBox[{"Table", "[", 
                            RowBox[{
                              RowBox[{"mmmmCreator", "[", 
                                RowBox[{
                                "n", ",", "j", ",", "k", ",", "r", ",", "s"}],
                                 "]"}], ",", 
                              RowBox[{"{", 
                                RowBox[{"s", ",", "0", ",", "1"}], "}"}], ",",
                               
                              RowBox[{"{", 
                                RowBox[{"r", ",", "0", ",", "1"}], "}"}], ",",
                               
                              RowBox[{"{", 
                                RowBox[{"k", ",", "0", ",", "1"}], "}"}], ",",
                               
                              RowBox[{"{", 
                                RowBox[{"j", ",", "0", ",", "1"}], "}"}]}], 
                            "]"}], "]"}], "/.", 
                        RowBox[{"{", 
                          RowBox[{
                            RowBox[{"x", "\[Rule]", 
                              RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                            RowBox[{"y", "\[Rule]", 
                              RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], 
                          "}"}]}], "]"}], ",", "2"}], "]"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"n", ",", "1", ",", "nnMax"}], "}"}]}], "]"}], 
        "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        RowBox[{"M", "(", 
          RowBox[{
            OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], ")"}],
         TraditionalForm]]],
  " (called mmm here) and ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox[
            RowBox[{"M", "(", 
              RowBox[{
                OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
              ")"}], 
            RowBox[{"j", " ", "k"}]], 
          SubsuperscriptBox[
            RowBox[{"M", "(", 
              RowBox[{
                OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
              ")"}], 
            RowBox[{"r", " ", "s"}], "*"]}], TraditionalForm]]],
  " (called mmmm here) directly"
}], "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mmmDirect", "[", "nn_", "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
          RowBox[{"mm", "\[LeftDoubleBracket]", 
            RowBox[{
              RowBox[{
              "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
              "+", "1"}], "\[RightDoubleBracket]"}], ",", 
          RowBox[{
            RowBox[{"mm", "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{
                "m", "\[LeftDoubleBracket]", "nn", "\[RightDoubleBracket]"}], 
                "+", "1"}], "\[RightDoubleBracket]"}], ".", "u", ".", 
            RowBox[{"mmmDirect", "[", 
              RowBox[{"nn", "-", "1"}], "]"}]}]}], "]"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
        RowBox[{
        "mm", " ", "is", " ", "the", " ", "set", " ", "of", " ", 
          "measurement", " ", "operators", " ", "for", " ", "a", " ", 
          "single", " ", "measurement"}], " ", "*)"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"mmmmDirect", "[", 
          RowBox[{"nn_", ",", "j_", ",", "k_", ",", "r_", ",", "s_"}], "]"}], 
        ":=", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"tmp", "=", 
                RowBox[{"mmmDirect", "[", "nn", "]"}]}], "}"}], ",", 
            RowBox[{
              RowBox[{"tmp", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{"j", "+", "1"}], ",", 
                  RowBox[{"k", "+", "1"}]}], "\[RightDoubleBracket]"}], 
              RowBox[{"Conjugate", "[", 
                RowBox[{"tmp", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"r", "+", "1"}], ",", 
                    RowBox[{"s", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                "]"}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(mmmmDirect\\)\\\" is similar to existing symbol \\\"\\!\\(mmmDirect\
\\)\\\".\"\>"}]], "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{"E", "(", 
            RowBox[{"m", ",", "\[Phi]"}], ")"}], 
          RowBox[{"j", " ", "k"}]], TraditionalForm]]],
  " (called ",
  Cell[BoxData[
      FormBox["ee", TraditionalForm]]],
  " here) according to (1-3)"
}], "Subsection"],

Cell[TextData[{
  "We use the following ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " compatible names for variables: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"mmm", "=", 
            RowBox[{"M", "(", 
              RowBox[{
                OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
              ")"}]}], ",", " ", 
          RowBox[{"nn", "=", "N"}]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mmmm", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "r_", ",", "s_"}], "]"}], ":=",
       
      RowBox[{
        SubsuperscriptBox["\[Sum]", 
          RowBox[{"n", "=", "0"}], 
          RowBox[{"2", 
            RowBox[{"(", 
              RowBox[{"nn", "-", "1"}], ")"}]}]], 
        RowBox[{
          RowBox[{"a", "[", 
            RowBox[{"nn", ",", "j", ",", "k", ",", "r", ",", "s", ",", "n"}], 
            "]"}], 
          SuperscriptBox["x", "n"], 
          SuperscriptBox["y", 
            RowBox[{
              RowBox[{"2", 
                RowBox[{"(", 
                  RowBox[{"nn", "-", "1"}], ")"}]}], "-", 
              "n"}]]}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"a", "[", 
          RowBox[{
          "nn_", ",", "j_", ",", "k_", ",", "r_", ",", "s_", ",", "n_"}], 
          "]"}], ":=", "\[IndentingNewLine]", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"mmnn", "=", 
                RowBox[{"mm", "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{
                    "m", "\[LeftDoubleBracket]", "nn", 
                      "\[RightDoubleBracket]"}], "+", "1"}], 
                  "\[RightDoubleBracket]"}]}], "}"}], ",", 
            "\[IndentingNewLine]", "\t", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{
                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"j", "+", "1"}], ",", 
                      RowBox[{"k", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  "*", 
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"r", "+", "1"}], ",", 
                        RowBox[{"s", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                    "]"}]}], ",", "\[IndentingNewLine]", 
                RowBox[{
                  SubsuperscriptBox["\[Sum]", 
                    RowBox[{"p", "=", "0"}], "1"], 
                  RowBox[{
                    SubsuperscriptBox["\[Sum]", 
                      RowBox[{"q", "=", "0"}], "1"], 
                    RowBox[{"(", "\[IndentingNewLine]", 
                      RowBox[{
                        RowBox[{"If", "[", 
                          RowBox[{
                            RowBox[{"n", "\[GreaterEqual]", "2"}], ",", 
                            RowBox[{
                              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                RowBox[{
                                  RowBox[{"j", "+", "1"}], ",", 
                                  RowBox[{"p", "+", "1"}]}], 
                                "\[RightDoubleBracket]"}], 
                              RowBox[{"Conjugate", "[", 
                                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                  RowBox[{
                                    RowBox[{"r", "+", "1"}], ",", 
                                    RowBox[{"q", "+", "1"}]}], 
                                  "\[RightDoubleBracket]"}], "]"}], 
                              RowBox[{"a", "[", 
                                RowBox[{
                                  RowBox[{"nn", "-", "1"}], ",", "p", ",", 
                                  "k", ",", "q", ",", "s", ",", 
                                  RowBox[{"n", "-", "2"}]}], "]"}]}], ",", 
                            "0"}], "]"}], "+", "\[IndentingNewLine]", 
                        RowBox[{"If", "[", 
                          RowBox[{
                            RowBox[{
                            "1", "\[LessEqual]", "n", "\[LessEqual]", 
                              RowBox[{
                                RowBox[{"2", 
                                  RowBox[{"(", 
                                    RowBox[{"nn", "-", "1"}], ")"}]}], "-", 
                                "1"}]}], ",", 
                            RowBox[{"I", 
                              RowBox[{"(", 
                                RowBox[{
                                  RowBox[{
                                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                      RowBox[{
                                        RowBox[{"j", "+", "1"}], ",", 
                                        RowBox[{"1", "-", "p", "+", "1"}]}], 
                                      "\[RightDoubleBracket]"}], 
                                    RowBox[{"Conjugate", "[", 
                                      RowBox[{
                                      "mmnn", "\[LeftDoubleBracket]", 
                                        RowBox[{
                                        RowBox[{"r", "+", "1"}], ",", 
                                        RowBox[{"q", "+", "1"}]}], 
                                        "\[RightDoubleBracket]"}], "]"}]}], 
                                  "-", 
                                  RowBox[{
                                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                      RowBox[{
                                        RowBox[{"j", "+", "1"}], ",", 
                                        RowBox[{"p", "+", "1"}]}], 
                                      "\[RightDoubleBracket]"}], 
                                    RowBox[{"Conjugate", "[", 
                                      RowBox[{
                                      "mmnn", "\[LeftDoubleBracket]", 
                                        RowBox[{
                                        RowBox[{"r", "+", "1"}], ",", 
                                        RowBox[{"1", "-", "q", "+", "1"}]}], 
                                        "\[RightDoubleBracket]"}], "]"}]}]}], 
                                ")"}], 
                              RowBox[{"a", "[", 
                                RowBox[{
                                  RowBox[{"nn", "-", "1"}], ",", "p", ",", 
                                  "k", ",", "q", ",", "s", ",", 
                                  RowBox[{"n", "-", "1"}]}], "]"}]}], ",", 
                            "0"}], "]"}], "+", "\[IndentingNewLine]", 
                        RowBox[{"If", "[", 
                          RowBox[{
                            RowBox[{"n", "\[LessEqual]", 
                              RowBox[{
                                RowBox[{"2", 
                                  RowBox[{"(", 
                                    RowBox[{"nn", "-", "1"}], ")"}]}], "-", 
                                "2"}]}], ",", 
                            RowBox[{
                              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                RowBox[{
                                  RowBox[{"j", "+", "1"}], ",", 
                                  RowBox[{"1", "-", "p", "+", "1"}]}], 
                                "\[RightDoubleBracket]"}], 
                              RowBox[{"Conjugate", "[", 
                                RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                  RowBox[{
                                    RowBox[{"r", "+", "1"}], ",", 
                                    RowBox[{"1", "-", "q", "+", "1"}]}], 
                                  "\[RightDoubleBracket]"}], "]"}], 
                              RowBox[{"a", "[", 
                                RowBox[{
                                  RowBox[{"nn", "-", "1"}], ",", "p", ",", 
                                  "k", ",", "q", ",", "s", ",", "n"}], 
                                "]"}]}], ",", "0"}], "]"}]}], ")"}]}]}]}], 
              "]"}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 1", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"mmmmDirect", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}], "\[IndentingNewLine]", 
    RowBox[{"mmmm", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]},
              {
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"mmmmDirect", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}], "\[IndentingNewLine]", 
    RowBox[{"mmmm", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox["\[ImaginaryI]", 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  RowBox[{"-", 
                    FractionBox["\[ImaginaryI]", 
                      InterpretationBox["\<\"2\"\>",
                        2,
                        Editable->False]]}], ",", 
                  FractionBox[
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox[
                      RowBox[{"0.3", "\[IndentingNewLine]"}]]}], "}"}]}], 
              "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"mmmmDirect", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}], "\[IndentingNewLine]", 
    RowBox[{"mmmm", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.3\"\>",
                    0.29999999999999993,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.46\"\>",
                    0.45825756949558399,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.46\"\>",
                    0.45825756949558399,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.7\"\>",
                    0.70000000000000007,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.21\"\>",
                    0.20999999999999999,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.21\"\>",
                    0.20999999999999999,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.21\"\>",
                    0.20999999999999999,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.21\"\>",
                    0.20999999999999999,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.063\"\>",
                    0.062999999999999987,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.096\"\>",
                    0.096234089594072633,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.096\"\>",
                    0.096234089594072633,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.15\"\>",
                    0.14699999999999999,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.019\"\>",
                    0.018899999999999993,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.044\"\>",
                    0.044099999999999986,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.044\"\>",
                    0.044099999999999986,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.1\"\>",
                    0.10289999999999998,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.3\"\>",
                    0.29999999999999993,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.46\"\>",
                    0.45825756949558399,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.46\"\>",
                    0.45825756949558399,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.7\"\>",
                    0.70000000000000007,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.21\"\>",
                    0.20999999999999994,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.21\"\>",
                    0.20999999999999996,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.21\"\>",
                    0.20999999999999999,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.21\"\>",
                    0.20999999999999996,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.063\"\>",
                    0.062999999999999973,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.096\"\>",
                    0.096234089594072619,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.096\"\>",
                    0.096234089594072633,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.15\"\>",
                    0.14699999999999996,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.019\"\>",
                    0.01889999999999999,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.044\"\>",
                    0.044099999999999986,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.044\"\>",
                    0.044099999999999993,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.1\"\>",
                    0.10289999999999998,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"mmmmDirect", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}], "\[IndentingNewLine]", 
    RowBox[{"mmmm", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.3\"\>",
                    0.29999999999999993,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.46\"\>",
                    0.45825756949558399,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.46\"\>",
                    0.45825756949558399,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.7\"\>",
                    0.70000000000000007,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.11\"\>",
                    0.10500000000000002,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.069\"\>",
                      0.068738635424337599,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.16\"\>",
                      0.16039014932345444,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10500000000000002,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.069\"\>",
                      -0.068738635424337599,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.045\"\>",
                    0.044999999999999998,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10500000000000001,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.069\"\>",
                      -0.068738635424337599,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.16\"\>",
                      -0.16039014932345444,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10500000000000001,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.25\"\>",
                    0.24500000000000008,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.16\"\>",
                      -0.16039014932345444,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10500000000000002,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.069\"\>",
                      0.068738635424337599,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.16\"\>",
                      0.16039014932345444,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10500000000000002,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.0044\"\>",
                    0.0043829552029636841,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.014\"\>",
                      0.013747727084867526,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.032\"\>",
                      0.032078029864690903,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"-0.0044\"\>",
                    -0.0043829552029636841,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.014\"\>",
                      -0.013747727084867526,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.043\"\>",
                    0.043121590627301282,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.1\"\>",
                    0.10061704479703636,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.014\"\>",
                      0.013747727084867526,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.032\"\>",
                      -0.032078029864690903,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.1\"\>",
                    0.10061704479703636,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.23\"\>",
                    0.23477310452641822,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.032\"\>",
                      0.032078029864690903,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"-0.0044\"\>",
                    -0.0043829552029636841,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.014\"\>",
                      -0.013747727084867526,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.032\"\>",
                      -0.032078029864690903,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.0044\"\>",
                    0.0043829552029636841,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.003\"\>",
                    0.0030013637490794872,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0089\"\>",
                      -0.0088761364575662416,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0089\"\>",
                      -0.0088761364575662416,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.018\"\>",
                    0.017850000000000005,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0089\"\>",
                      0.0088761364575662416,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026250000000000013,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026250000000000013,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.053\"\>",
                      0.052789014932345474,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0089\"\>",
                      0.0088761364575662416,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026250000000000013,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026250000000000013,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.053\"\>",
                      0.052789014932345474,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.018\"\>",
                    0.017850000000000005,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.053\"\>",
                      -0.052789014932345474,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.053\"\>",
                      -0.052789014932345474,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10615924181056731,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.3\"\>",
                    0.29999999999999993,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.46\"\>",
                    0.45825756949558399,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.46\"\>",
                    0.45825756949558399,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0\"\>",
                    0,
                    Editable->False], ",", 
                  InterpretationBox["\<\"0.7\"\>",
                    0.70000000000000007,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.11\"\>",
                    0.10499999999999997,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.069\"\>",
                      0.068738635424337585,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.16\"\>",
                      0.16039014932345444,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10499999999999998,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.069\"\>",
                      -0.068738635424337585,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.045\"\>",
                    0.044999999999999984,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.105,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.069\"\>",
                      -0.068738635424337585,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.16\"\>",
                      -0.16039014932345444,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10499999999999998,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.25\"\>",
                    0.24500000000000002,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.16\"\>",
                      -0.16039014932345444,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.105,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.069\"\>",
                      0.068738635424337585,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.16\"\>",
                      0.16039014932345444,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10499999999999998,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.0044\"\>",
                    0.0043829552029636711,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.014\"\>",
                      0.013747727084867521,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.032\"\>",
                      0.032078029864690896,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"-0.0044\"\>",
                    -0.0043829552029636919,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.014\"\>",
                      -0.013747727084867521,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.043\"\>",
                    0.043121590627301268,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.1\"\>",
                    0.10061704479703631,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.014\"\>",
                      0.013747727084867527,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.032\"\>",
                      -0.032078029864690896,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.1\"\>",
                    0.10061704479703631,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.23\"\>",
                    0.23477310452641811,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.032\"\>",
                      0.032078029864690896,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"-0.0044\"\>",
                    -0.0043829552029636919,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.014\"\>",
                      -0.013747727084867521,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.032\"\>",
                      -0.032078029864690896,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.0044\"\>",
                    0.0043829552029636884,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.003\"\>",
                    0.0030013637490794824,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0089\"\>",
                      -0.0088761364575662416,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0089\"\>",
                      -0.0088761364575662381,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.018\"\>",
                    0.017849999999999984,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0089\"\>",
                      0.0088761364575662416,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026249999999999992,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026249999999999999,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.053\"\>",
                      0.052789014932345474,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0089\"\>",
                      0.0088761364575662399,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026250000000000002,
                    AutoDelete->True], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026249999999999989,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.053\"\>",
                      0.052789014932345446,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.018\"\>",
                    0.017849999999999984,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.053\"\>",
                      -0.052789014932345446,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.053\"\>",
                      -0.052789014932345446,
                      AutoDelete->True], " ", "\[ImaginaryI]"}], ",", 
                  InterpretationBox["\<\"0.11\"\>",
                    0.10615924181056724,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 5", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Phi]", "=", "0.13"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}]},
              {
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}], 
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurement operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm", "=", "tmpM3"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"mmmmDirect", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}], "\[IndentingNewLine]", 
    RowBox[{"mmmm", "//", 
      RowBox[{
        RowBox[{"mmmmOut", "[", 
          RowBox[{"#", ",", "4"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.35\"\>",
                    0.34758614223187717,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.13\"\>",
                      -0.1349750021340109,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.00032\"\>",
                        0.00031853058329741067,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.067\"\>",
                      0.067321805954269676,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.14\"\>",
                        0.13509263611023023,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.17\"\>",
                      0.17398132817351936,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.35\"\>",
                        0.34747848945375703,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.13\"\>",
                      -0.1349750021340109,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.00032\"\>",
                        0.00031853058329741067,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.052\"\>",
                    0.05241392118174594,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026018671826480658,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.052521003237258841,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.067\"\>",
                      -0.067242173308445341,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.14\"\>",
                        0.13509263611023023,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.067\"\>",
                      0.067321805954269676,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.14\"\>",
                        0.13509263611023023,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026018671826480658,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.052521003237258841,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.066\"\>",
                    0.065544171991060635,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.17\"\>",
                      0.16874816116156843,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.00032\"\>",
                        0.00031853058329742515,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.17\"\>",
                      0.17398132817351936,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.35\"\>",
                        0.34747848945375703,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.067\"\>",
                      -0.067242173308445341,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.14\"\>",
                        0.13509263611023023,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.17\"\>",
                      0.16874816116156843,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.00032\"\>",
                        0.00031853058329742515,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.43\"\>",
                    0.43445576459531632,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.02\"\>",
                    0.019593212262335177,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.02\"\>",
                      -0.020466298877634136,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0096\"\>",
                        0.0096498464561507249,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0084\"\>",
                      -0.0084029944453574441,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.024\"\>",
                        0.023616726336089751,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.02\"\>",
                      -0.020017651748982321,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.028\"\>",
                        0.02810784922961564,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.02\"\>",
                      -0.020466298877634136,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0096\"\>",
                        0.0096498464561507249,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026130933484559346,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0029\"\>",
                      -0.0028540285485452893,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.029\"\>",
                        0.028807659423989825,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0071\"\>",
                      0.0070662641934825651,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.039219240757594115,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0084\"\>",
                      -0.0084029944453574441,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.024\"\>",
                        0.023616726336089751,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0029\"\>",
                      -0.0028540285485452893,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.029\"\>",
                        0.028807659423989825,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.032\"\>",
                    0.03207029404210484,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.042\"\>",
                      0.042464889803043394,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.012\"\>",
                        0.012073635457632974,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.02\"\>",
                      -0.020017651748982321,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.028\"\>",
                        0.02810784922961564,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0071\"\>",
                      0.0070662641934825651,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.039219240757594115,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.042\"\>",
                      0.042464889803043394,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.012\"\>",
                        0.012073635457632974,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.061\"\>",
                    0.060773984067297684,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.0051\"\>",
                    0.0050501357063459425,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0074\"\>",
                      -0.0074019693514700774,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0034\"\>",
                        0.003379371564049191,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0087\"\>",
                      -0.008679989915570601,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0076\"\>",
                        0.0076414617603280431,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0051\"\>",
                      -0.0050575260767560153,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0033\"\>",
                        0.0033205255731640587,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0074\"\>",
                      -0.0074019693514700774,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0034\"\>",
                        0.003379371564049191,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.013\"\>",
                    0.013110400650186248,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0076\"\>",
                      0.007608841223259763,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.017\"\>",
                        0.017008409643201744,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0052\"\>",
                      0.0051908235419929998,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0083\"\>",
                        0.0082512017012132694,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0087\"\>",
                      -0.008679989915570601,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0076\"\>",
                        0.0076414617603280431,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0076\"\>",
                      0.007608841223259763,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.017\"\>",
                        0.017008409643201744,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026481300809583049,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.014\"\>",
                      0.013717046147582625,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0019\"\>",
                        0.0019454454690373956,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0051\"\>",
                      -0.0050575260767560153,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0033\"\>",
                        0.0033205255731640587,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0052\"\>",
                      0.0051908235419929998,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0083\"\>",
                        0.0082512017012132694,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.014\"\>",
                      0.013717046147582625,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0019\"\>",
                        0.0019454454690373956,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.0072\"\>",
                    0.0072482131624912269,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.0031\"\>",
                    0.0030635719200023015,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0014\"\>",
                      0.0013993585860117665,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0038\"\>",
                        0.0038230835729659721,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0053\"\>",
                      -0.0053485708306391481,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0015\"\>",
                        0.0015463441404603101,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0017\"\>",
                      0.0016887913625211223,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0032\"\>",
                        0.0031562933245083114,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0014\"\>",
                      0.0013993585860117665,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0038\"\>",
                        0.0038230835729659721,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.0054\"\>",
                    0.005410081072330322,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.00051\"\>",
                      -0.00051337643583043388,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0074\"\>",
                        0.007380901712706201,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0047\"\>",
                      0.0047101873989971428,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.00067\"\>",
                        0.00066576023212716465,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0053\"\>",
                      -0.0053485708306391481,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0015\"\>",
                        0.0015463441404603101,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.00051\"\>",
                      -0.00051337643583043388,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0074\"\>",
                        0.007380901712706201,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.01\"\>",
                    0.010118381725824343,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0014\"\>",
                      -0.0013552495717760983,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0064\"\>",
                        0.0063628703831069503,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0017\"\>",
                      0.0016887913625211223,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0032\"\>",
                        0.0031562933245083114,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0047\"\>",
                      0.0047101873989971428,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.00067\"\>",
                        0.00066576023212716465,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0014\"\>",
                      -0.0013552495717760983,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0064\"\>",
                        0.0063628703831069503,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.0042\"\>",
                    0.0041827657881301026,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"],

Cell[BoxData[
    TagBox[GridBox[{
          {"\<\"n\"\>", "\<\"M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\)\\)M(\\!\\(\\(m\\& \
\[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 0\\)\\%*\\)\\), \
M(\\!\\(\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ \
0\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(0\\\\ 1\\)\\%*\\)\\), \[Ellipsis], M(\\!\\(\
\\(m\\& \[RightVector] \\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ \
1\\)\\)\\)M(\\!\\(\\(m\\& \[RightVector] \
\\)\\),\[CurlyPhi]\\!\\(\\()\\_\\(1\\\\ 1\\)\\%*\\)\\)\"\>"},
          {"1", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.35\"\>",
                    0.34758614223187717,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.13\"\>",
                      -0.1349750021340109,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.00032\"\>",
                        0.00031853058329741067,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.067\"\>",
                      0.067321805954269676,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.14\"\>",
                        0.13509263611023023,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.17\"\>",
                      0.17398132817351936,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.35\"\>",
                        0.34747848945375703,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.13\"\>",
                      -0.1349750021340109,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.00032\"\>",
                        0.00031853058329741067,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.052\"\>",
                    0.05241392118174594,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026018671826480658,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.052521003237258841,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.067\"\>",
                      -0.067242173308445341,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.14\"\>",
                        0.13509263611023023,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.067\"\>",
                      0.067321805954269676,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.14\"\>",
                        0.13509263611023023,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026018671826480658,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.052521003237258841,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.066\"\>",
                    0.065544171991060635,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.17\"\>",
                      0.16874816116156843,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.00032\"\>",
                        0.00031853058329742515,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.17\"\>",
                      0.17398132817351936,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.35\"\>",
                        0.34747848945375703,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.067\"\>",
                      -0.067242173308445341,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.14\"\>",
                        0.13509263611023023,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.17\"\>",
                      0.16874816116156843,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.00032\"\>",
                        0.00031853058329742515,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.43\"\>",
                    0.43445576459531632,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"2", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.02\"\>",
                    0.019593212262335177,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.02\"\>",
                      -0.020466298877634136,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0096\"\>",
                        0.009649846456150718,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0084\"\>",
                      -0.0084029944453574458,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.024\"\>",
                        0.023616726336089747,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.02\"\>",
                      -0.020017651748982321,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.028\"\>",
                        0.028107849229615636,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.02\"\>",
                      -0.020466298877634136,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0096\"\>",
                        0.0096498464561507197,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026130933484559343,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0029\"\>",
                      -0.0028540285485452828,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.029\"\>",
                        0.028807659423989832,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0071\"\>",
                      0.0070662641934825746,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.039219240757594115,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0084\"\>",
                      -0.0084029944453574441,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.024\"\>",
                        0.023616726336089747,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0029\"\>",
                      -0.002854028548545281,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.029\"\>",
                        0.028807659423989832,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.032\"\>",
                    0.032070294042104847,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.042\"\>",
                      0.042464889803043401,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.012\"\>",
                        0.012073635457632972,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.02\"\>",
                      -0.020017651748982321,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.028\"\>",
                        0.028107849229615643,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0071\"\>",
                      0.0070662641934825729,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.039219240757594115,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.042\"\>",
                      0.042464889803043394,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.012\"\>",
                        0.012073635457632972,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.061\"\>",
                    0.060773984067297691,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"3", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.0051\"\>",
                    0.0050501357063459417,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0074\"\>",
                      -0.0074019693514700757,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0034\"\>",
                        0.0033793715640491884,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0087\"\>",
                      -0.0086799899155705993,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0076\"\>",
                        0.0076414617603280397,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0051\"\>",
                      -0.0050575260767560118,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0033\"\>",
                        0.0033205255731640578,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0074\"\>",
                      -0.0074019693514700766,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0034\"\>",
                        0.0033793715640491892,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.013\"\>",
                    0.01311040065018625,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0076\"\>",
                      0.0076088412232597664,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.017\"\>",
                        0.017008409643201744,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0052\"\>",
                      0.0051908235419929998,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0083\"\>",
                        0.0082512017012132694,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0087\"\>",
                      -0.0086799899155705976,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0076\"\>",
                        0.0076414617603280397,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0076\"\>",
                      0.0076088412232597664,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.017\"\>",
                        0.017008409643201744,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.026\"\>",
                    0.026481300809583056,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.014\"\>",
                      0.013717046147582627,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0019\"\>",
                        0.0019454454690373954,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0051\"\>",
                      -0.005057526076756011,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0033\"\>",
                        0.0033205255731640613,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0052\"\>",
                      0.0051908235419929998,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0083\"\>",
                        0.0082512017012132694,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.014\"\>",
                      0.013717046147582625,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0019\"\>",
                        0.0019454454690373941,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.0072\"\>",
                    0.0072482131624912261,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]},
          {"4", 
            TagBox[
              RowBox[{"{", 
                RowBox[{
                  InterpretationBox["\<\"0.0031\"\>",
                    0.0030635719200022998,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0014\"\>",
                      0.0013993585860117665,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0038\"\>",
                        0.0038230835729659712,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0053\"\>",
                      -0.0053485708306391473,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0015\"\>",
                        0.0015463441404603116,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0017\"\>",
                      0.0016887913625211238,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0032\"\>",
                        0.0031562933245083101,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0014\"\>",
                      0.0013993585860117665,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0038\"\>",
                        0.0038230835729659712,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.0054\"\>",
                    0.0054100810723303238,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.00051\"\>",
                      -0.00051337643583043322,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0074\"\>",
                        0.0073809017127062027,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0047\"\>",
                      0.0047101873989971437,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.00067\"\>",
                        0.0006657602321271653,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0053\"\>",
                      -0.0053485708306391473,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0015\"\>",
                        0.0015463441404603112,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.00051\"\>",
                      -0.00051337643583043388,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0074\"\>",
                        0.0073809017127062027,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.01\"\>",
                    0.010118381725824344,
                    AutoDelete->True], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0014\"\>",
                      -0.0013552495717761004,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0064\"\>",
                        0.0063628703831069512,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0017\"\>",
                      0.0016887913625211223,
                      AutoDelete->True], "+", 
                    RowBox[{
                      InterpretationBox["\<\"0.0032\"\>",
                        0.0031562933245083092,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"0.0047\"\>",
                      0.0047101873989971446,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.00067\"\>",
                        0.00066576023212716563,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  RowBox[{
                    InterpretationBox["\<\"-0.0014\"\>",
                      -0.0013552495717760998,
                      AutoDelete->True], "-", 
                    RowBox[{
                      InterpretationBox["\<\"0.0064\"\>",
                        0.0063628703831069503,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]}], ",", 
                  InterpretationBox["\<\"0.0042\"\>",
                    0.0041827657881301017,
                    AutoDelete->True]}], "}"}],
              (NumberForm[ #, 2]&)]}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #, TableDepth -> 2, TableHeadings -> {None, {"n", 
        "M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ \
0\)\)\)M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\%*\)\), \
M(\!\(\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 0\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(0\\ 1\)\%*\)\), \[Ellipsis], M(\!\(\
\(m\& \[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\)\)M(\!\(\(m\& \
\[RightVector] \)\),\[CurlyPhi]\!\(\()\_\(1\\ 1\)\%*\)\)"}}]&)]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 43, 0, 45, "Subsection"],
Cell[1785, 53, 198, 5, 27, "Input"],
Cell[1986, 60, 401, 10, 27, "Input"],
Cell[2390, 72, 924, 18, 45, "Input"],
Cell[3317, 92, 1922, 41, 91, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5276, 138, 822, 25, 49, "Subsection"],
Cell[6101, 165, 1046, 24, 43, "Input"],

Cell[CellGroupData[{
Cell[7172, 193, 877, 21, 27, "Input"],
Cell[8052, 216, 267, 6, 23, "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8368, 228, 353, 12, 49, "Subsection"],
Cell[8724, 242, 453, 15, 32, "Text"],
Cell[9180, 259, 722, 21, 35, "Input"],
Cell[9905, 282, 7156, 134, 193, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17098, 421, 31, 0, 45, "Subsection"],
Cell[17132, 423, 1032, 29, 43, "Input"],

Cell[CellGroupData[{
Cell[18189, 456, 307, 8, 43, "Input"],
Cell[18499, 466, 9205, 234, 114, "Output"],
Cell[27707, 702, 9205, 234, 114, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[36961, 942, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[37017, 946, 1127, 31, 59, "Input"],
Cell[38147, 979, 1178, 34, 90, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39362, 1018, 307, 8, 43, "Input"],
Cell[39672, 1028, 12901, 322, 130, "Output"],
Cell[52576, 1352, 12901, 322, 130, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[65526, 1680, 31, 0, 45, "Subsection"],
Cell[65560, 1682, 1494, 42, 69, "Input"],

Cell[CellGroupData[{
Cell[67079, 1728, 307, 8, 43, "Input"],
Cell[67389, 1738, 9566, 234, 114, "Output"],
Cell[76958, 1974, 9565, 234, 114, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[86572, 2214, 31, 0, 45, "Subsection"],
Cell[86606, 2216, 1456, 41, 55, "Input"],

Cell[CellGroupData[{
Cell[88087, 2261, 307, 8, 43, "Input"],
Cell[88397, 2271, 11824, 258, 114, "Output"],
Cell[100224, 2531, 11796, 258, 114, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[112069, 2795, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[112125, 2799, 860, 23, 75, "Input"],
Cell[112988, 2824, 840, 22, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[113865, 2851, 154, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[114044, 2858, 5935, 159, 169, "Input"],
Cell[119982, 3019, 666, 20, 58, "Output"],
Cell[120651, 3041, 2474, 62, 45, "Output"],
Cell[123128, 3105, 5447, 122, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[128624, 3233, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[128687, 3237, 307, 8, 43, "Input"],
Cell[128997, 3247, 22402, 474, 114, "Output"],
Cell[151402, 3723, 22409, 474, 114, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

