(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     76486,       2064]*)
(*NotebookOutlinePosition[     77142,       2088]*)
(*  CellTagsIndexPosition[     77098,       2084]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Some helper functions", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
      RowBox[{"NumberForm", "[", 
        RowBox[{
          RowBox[{"MatrixForm", "[", 
            RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        RowBox[{"M", "(", 
          RowBox[{
            OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], ")"}],
         TraditionalForm]]],
  " (called mmm here) and ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            OverscriptBox["E", "~"], "(", 
            RowBox[{
              OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
            ")"}], "=", 
          RowBox[{
            RowBox[{"M", "(", 
              RowBox[{
                OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
              ")"}], 
            SuperscriptBox[
              RowBox[{"M", "(", 
                RowBox[{
                  OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
                ")"}], "\[Dagger]"]}]}], TraditionalForm]]],
  " (called eet here) directly"
}], "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mmmDirect", "[", "nn_", "]"}], ":=", 
      RowBox[{"If", "[", 
        RowBox[{
          RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
          RowBox[{"mm", "\[LeftDoubleBracket]", 
            RowBox[{
              RowBox[{
              "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
              "+", "1"}], "\[RightDoubleBracket]"}], ",", 
          RowBox[{
            RowBox[{"mm", "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{
                "m", "\[LeftDoubleBracket]", "nn", "\[RightDoubleBracket]"}], 
                "+", "1"}], "\[RightDoubleBracket]"}], ".", "u", ".", 
            RowBox[{"mmmDirect", "[", 
              RowBox[{"nn", "-", "1"}], "]"}]}]}], "]"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
        RowBox[{
        "mm", " ", "is", " ", "the", " ", "set", " ", "of", " ", 
          "measurement", " ", "operators", " ", "for", " ", "a", " ", 
          "single", " ", "measurement"}], " ", "*)"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"eetDirect", "[", "nn_", "]"}], "=", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"mmm", "=", 
                RowBox[{"mmmDirect", "[", "nn", "]"}]}], "}"}], ",", 
            RowBox[{"mmm", ".", 
              RowBox[{"adj", "[", "mmm", "]"}]}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{
            OverscriptBox["E", "~"], "(", 
            RowBox[{"m", ",", "\[CurlyPhi]"}], ")"}], 
          RowBox[{"j", " ", "k"}]], TraditionalForm]]],
  " (called ",
  Cell[BoxData[
      FormBox["eet", TraditionalForm]]],
  " here) according to (1-3)"
}], "Subsection"],

Cell[TextData[{
  "We use the following ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " compatible names for variables: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"mmm", "=", 
            RowBox[{"M", "(", 
              RowBox[{
                OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
              ")"}]}], ",", " ", 
          RowBox[{"nn", "=", "N"}]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"eet", "[", "nn_", "]"}], ":=", 
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"eet", "[", 
              RowBox[{"nn", ",", "j", ",", "k"}], "]"}], "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], ",", 
          RowBox[{"{", 
            RowBox[{"j", ",", "0", ",", "1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "0", ",", "1"}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"eet", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_"}], "]"}], ":=", 
      RowBox[{
        SubsuperscriptBox["\[Sum]", 
          RowBox[{"n", "=", "0"}], 
          RowBox[{"2", 
            RowBox[{"(", 
              RowBox[{"nn", "-", "1"}], ")"}]}]], 
        RowBox[{
          RowBox[{"a", "[", 
            RowBox[{"nn", ",", "j", ",", "k", ",", "n"}], "]"}], 
          SuperscriptBox["x", "n"], 
          SuperscriptBox["y", 
            RowBox[{
              RowBox[{"2", 
                RowBox[{"(", 
                  RowBox[{"nn", "-", "1"}], ")"}]}], "-", 
              "n"}]]}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"a", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mmnn", "=", 
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "nn", 
                    "\[RightDoubleBracket]"}], "+", "1"}], 
                "\[RightDoubleBracket]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
            RowBox[{
              RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
              RowBox[{
                SubsuperscriptBox["\[Sum]", 
                  RowBox[{"p", "=", "0"}], "1"], 
                RowBox[{
                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{"j", "+", "1"}], ",", 
                      RowBox[{"p", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                  RowBox[{"Conjugate", "[", 
                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                      RowBox[{
                        RowBox[{"k", "+", "1"}], ",", 
                        RowBox[{"p", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                    "]"}]}]}], ",", "\[IndentingNewLine]", 
              RowBox[{
                SubsuperscriptBox["\[Sum]", 
                  RowBox[{"p", "=", "0"}], "1"], 
                RowBox[{
                  SubsuperscriptBox["\[Sum]", 
                    RowBox[{"q", "=", "0"}], "1"], 
                  RowBox[{"(", "\[IndentingNewLine]", 
                    RowBox[{
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{"n", "\[GreaterEqual]", "2"}], ",", 
                          RowBox[{
                            RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                              RowBox[{
                                RowBox[{"j", "+", "1"}], ",", 
                                RowBox[{"p", "+", "1"}]}], 
                              "\[RightDoubleBracket]"}], 
                            RowBox[{"Conjugate", "[", 
                              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                RowBox[{
                                  RowBox[{"k", "+", "1"}], ",", 
                                  RowBox[{"q", "+", "1"}]}], 
                                "\[RightDoubleBracket]"}], "]"}], 
                            RowBox[{"a", "[", 
                              RowBox[{
                                RowBox[{"nn", "-", "1"}], ",", "p", ",", "q", 
                                ",", 
                                RowBox[{"n", "-", "2"}]}], "]"}]}], ",", 
                          "0"}], "]"}], "\[IndentingNewLine]", "+", 
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{
                            RowBox[{"n", "\[GreaterEqual]", "1"}], "\[And]", 
                            RowBox[{"n", "\[LessEqual]", 
                              RowBox[{
                                RowBox[{"2", 
                                  RowBox[{"(", 
                                    RowBox[{"nn", "-", "1"}], ")"}]}], "-", 
                                "1"}]}]}], ",", 
                          RowBox[{"I", 
                            RowBox[{"(", 
                              RowBox[{
                                RowBox[{
                                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                    RowBox[{
                                      RowBox[{"j", "+", "1"}], ",", 
                                      RowBox[{"1", "-", "p", "+", "1"}]}], 
                                    "\[RightDoubleBracket]"}], 
                                  RowBox[{"Conjugate", "[", 
                                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                      RowBox[{
                                        RowBox[{"k", "+", "1"}], ",", 
                                        RowBox[{"q", "+", "1"}]}], 
                                      "\[RightDoubleBracket]"}], "]"}]}], "-",
                                 
                                RowBox[{
                                  RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                    RowBox[{
                                      RowBox[{"j", "+", "1"}], ",", 
                                      RowBox[{"p", "+", "1"}]}], 
                                    "\[RightDoubleBracket]"}], 
                                  RowBox[{"Conjugate", "[", 
                                    RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                      RowBox[{
                                        RowBox[{"k", "+", "1"}], ",", 
                                        RowBox[{"1", "-", "q", "+", "1"}]}], 
                                      "\[RightDoubleBracket]"}], "]"}]}]}], 
                              ")"}], 
                            RowBox[{"a", "[", 
                              RowBox[{
                                RowBox[{"nn", "-", "1"}], ",", "p", ",", "q", 
                                ",", 
                                RowBox[{"n", "-", "1"}]}], "]"}]}], ",", 
                          "0"}], "]"}], "\[IndentingNewLine]", "+", 
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{"n", "\[LessEqual]", 
                            RowBox[{
                              RowBox[{"2", 
                                RowBox[{"(", 
                                  RowBox[{"nn", "-", "1"}], ")"}]}], "-", 
                              "2"}]}], ",", 
                          RowBox[{
                            RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                              RowBox[{
                                RowBox[{"j", "+", "1"}], ",", 
                                RowBox[{"1", "-", "p", "+", "1"}]}], 
                              "\[RightDoubleBracket]"}], 
                            RowBox[{"Conjugate", "[", 
                              RowBox[{"mmnn", "\[LeftDoubleBracket]", 
                                RowBox[{
                                  RowBox[{"k", "+", "1"}], ",", 
                                  RowBox[{"1", "-", "q", "+", "1"}]}], 
                                "\[RightDoubleBracket]"}], "]"}], 
                            RowBox[{"a", "[", 
                              RowBox[{
                                RowBox[{"nn", "-", "1"}], ",", "p", ",", "q", 
                                ",", "n"}], "]"}]}], ",", "0"}], "]"}]}], 
                    ")"}]}]}]}], "]"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 1", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eetDirect", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eet", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]},
              {
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]}
              }], "\[NoBreak]", ")"}],
        Function[ BoxForm`e$, 
          MatrixForm[ BoxForm`e$]]],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eetDirect", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eet", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox[
                      RowBox[{"0.3", "\[IndentingNewLine]"}]]}], "}"}]}], 
              "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eetDirect", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eet", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999999,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999999,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.063\"\>",
                      0.062999999999999987,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14699999999999999,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.019\"\>",
                      0.018899999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.1\"\>",
                      0.10289999999999998,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999994,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.21\"\>",
                      0.20999999999999996,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.063\"\>",
                      0.062999999999999973,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14699999999999996,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.019\"\>",
                      0.01889999999999999,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.1\"\>",
                      0.10289999999999998,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eetDirect", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eet", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.35\"\>",
                      0.35000000000000009,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.092\"\>",
                        0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.092\"\>",
                        -0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.15000000000000002,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.24\"\>",
                      0.23915605972938192,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.046\"\>",
                        -0.045825756949558427,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.046\"\>",
                        0.045825756949558427,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.048\"\>",
                      0.047504545830264963,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.029\"\>",
                      0.029251363749079502,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.044\"\>",
                        -0.043912878474779232,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043912878474779232,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.13\"\>",
                      0.13240924181056732,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.3\"\>",
                      0.29999999999999993,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.7\"\>",
                      0.70000000000000007,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.35\"\>",
                      0.34999999999999998,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.092\"\>",
                        0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.092\"\>",
                        -0.09165151389911684,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.15\"\>",
                      0.14999999999999997,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.24\"\>",
                      0.23915605972938175,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.046\"\>",
                        -0.045825756949558399,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.046\"\>",
                        0.045825756949558406,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.048\"\>",
                      0.047504545830264942,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.029\"\>",
                      0.029251363749079481,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.044\"\>",
                        -0.043912878474779204,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043912878474779232,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.13\"\>",
                      0.13240924181056726,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 5", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Phi]", "=", "0.13"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}]},
              {
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}], 
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        Function[ BoxForm`e$, 
          MatrixForm[ BoxForm`e$]]],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurement operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm", "=", "tmpM3"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(tmpW\\)\\\" is similar to existing symbol \
\\\"\\!\\(tmpU\\)\\\".\"\>"}]], "Message"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        Function[ BoxForm`e$, 
          MatrixForm[ BoxForm`e$]]],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eetDirect", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", 
          RowBox[{"eet", "[", "n", "]"}], "]"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.41\"\>",
                      0.41313031422293778,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557523,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.49\"\>",
                      0.4868696857770623,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.052\"\>",
                      0.051663506304440013,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.022\"\>",
                        0.021998590925409258,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0024\"\>",
                          0.0024237890014822488,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.022\"\>",
                        0.021998590925409258,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0024\"\>",
                          0.0024237890014822488,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.087\"\>",
                      0.08690491755185703,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.032\"\>",
                      0.03153143651592899,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.0063\"\>",
                        0.0063150767961125467,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0014\"\>",
                          0.0014339260950117954,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.0063\"\>",
                        0.0063150767961125467,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0014\"\>",
                          0.0014339260950117954,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.02\"\>",
                      0.020358613812677475,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.013181953645826644,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.000044\"\>",
                        4.4109014235668336*^-05,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0025\"\>",
                          0.0025397868101409778,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.000044\"\>",
                        4.4109014235668336*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0025\"\>",
                          0.0025397868101409778,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.0096\"\>",
                      0.0095928468604604238,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.41\"\>",
                      0.41313031422293778,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557537,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.034\"\>",
                      0.033773159027557537,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.49\"\>",
                      0.48686968577706224,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.052\"\>",
                      0.051663506304440013,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.022\"\>",
                        0.021998590925409258,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0024\"\>",
                          0.0024237890014822583,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.022\"\>",
                        0.021998590925409265,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0024\"\>",
                          0.0024237890014822583,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.087\"\>",
                      0.086904917551857003,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.032\"\>",
                      0.031531436515928997,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.0063\"\>",
                        0.0063150767961125485,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0014\"\>",
                          0.0014339260950117913,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.0063\"\>",
                        0.0063150767961125493,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0014\"\>",
                          0.0014339260950117924,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.02\"\>",
                      0.020358613812677475,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.013\"\>",
                      0.013181953645826642,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.000044\"\>",
                        4.4109014235666608*^-05,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.0025\"\>",
                          0.0025397868101409804,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.000044\"\>",
                        4.4109014235667591*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.0025\"\>",
                          0.0025397868101409795,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.0096\"\>",
                      0.0095928468604604272,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
Evaluator->"psi",
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 43, 0, 45, "Subsection"],
Cell[1785, 53, 198, 5, 27, "Input"],
Cell[1986, 60, 401, 10, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2424, 75, 939, 28, 47, "Subsection"],
Cell[3366, 105, 1046, 24, 43, "Input"],
Cell[4415, 131, 387, 10, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4839, 146, 392, 13, 51, "Subsection"],
Cell[5234, 161, 453, 15, 32, "Text"],
Cell[5690, 178, 686, 17, 27, "Input"],
Cell[6379, 197, 659, 19, 35, "Input"],
Cell[7041, 218, 6813, 130, 163, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13891, 353, 31, 0, 45, "Subsection"],
Cell[13925, 355, 1032, 29, 43, "Input"],

Cell[CellGroupData[{
Cell[14982, 388, 492, 13, 43, "Input"],
Cell[15477, 403, 3052, 86, 45, "Output"],
Cell[18532, 491, 3052, 86, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[21633, 583, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[21689, 587, 1127, 31, 59, "Input"],
Cell[22819, 620, 1218, 35, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24074, 660, 492, 13, 43, "Input"],
Cell[24569, 675, 3052, 86, 45, "Output"],
Cell[27624, 763, 3052, 86, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[30725, 855, 31, 0, 45, "Subsection"],
Cell[30759, 857, 1494, 42, 69, "Input"],

Cell[CellGroupData[{
Cell[32278, 903, 492, 13, 43, "Input"],
Cell[32773, 918, 3229, 86, 45, "Output"],
Cell[36005, 1006, 3228, 86, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[39282, 1098, 31, 0, 45, "Subsection"],
Cell[39316, 1100, 1456, 41, 55, "Input"],

Cell[CellGroupData[{
Cell[40797, 1145, 492, 13, 43, "Input"],
Cell[41292, 1160, 3732, 92, 45, "Output"],
Cell[45027, 1254, 3732, 92, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[48808, 1352, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[48864, 1356, 860, 23, 75, "Input"],
Cell[49727, 1381, 880, 23, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50644, 1409, 154, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[50823, 1416, 5935, 159, 169, "Input"],
Cell[56761, 1577, 256, 6, 23, "Message"],
Cell[57020, 1585, 706, 21, 58, "Output"],
Cell[57729, 1608, 2606, 65, 45, "Output"],
Cell[60338, 1675, 5579, 125, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[65966, 1806, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[66029, 1810, 492, 13, 43, "Input"],
Cell[66524, 1825, 4958, 116, 83, "Output"],
Cell[71485, 1943, 4961, 116, 83, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

