(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     63253,       1755]*)
(*NotebookOutlinePosition[     63911,       1779]*)
(*  CellTagsIndexPosition[     63867,       1775]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Miscellaneous", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
      RowBox[{"NumberForm", "[", 
        RowBox[{
          RowBox[{"MatrixForm", "[", 
            RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"(*", " ", 
        RowBox[{
          RowBox[{"Arbitrary", " ", "unitray", " ", "matrix", " ", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"see", " ", "the", " ", "Nielsen"}], "-", "Chuang"}], 
              ")"}], " ", "\[IndentingNewLine]", "  ", "\[Alpha]"}], ",", 
          "\[Beta]", ",", "\[Gamma]", ",", 
          RowBox[{
          "\[Delta]", " ", "need", " ", "to", " ", "be", " ", "real", " ", 
            "valued"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
        RowBox[{
          RowBox[{"unitary", "[", 
            RowBox[{
            "\[Alpha]_", ",", "\[Beta]_", ",", "\[Gamma]_", ",", 
              "\[Delta]_"}], "]"}], ":=", 
          RowBox[{"(", GridBox[{
                {
                  RowBox[{
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", 
                        RowBox[{"(", 
                          RowBox[{"\[Alpha]", "-", 
                            RowBox[{"\[Beta]", "/", "2"}], "-", 
                            RowBox[{"\[Delta]", "/", "2"}]}], ")"}]}], "]"}], 
                    
                    RowBox[{"Cos", "[", 
                      RowBox[{"\[Gamma]", "/", "2"}], "]"}]}], 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Exp", "[", 
                        RowBox[{"I", 
                          RowBox[{"(", 
                            RowBox[{"\[Alpha]", "-", 
                              RowBox[{"\[Beta]", "/", "2"}], "+", 
                              RowBox[{"\[Delta]", "/", "2"}]}], ")"}]}], 
                        "]"}]}], 
                    RowBox[{"Sin", "[", 
                      RowBox[{"\[Gamma]", "/", "2"}], "]"}]}]},
                {
                  RowBox[{
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", 
                        RowBox[{"(", 
                          RowBox[{"\[Alpha]", "+", 
                            RowBox[{"\[Beta]", "/", "2"}], "-", 
                            RowBox[{"\[Delta]", "/", "2"}]}], ")"}]}], "]"}], 
                    
                    RowBox[{"Sin", "[", 
                      RowBox[{"\[Gamma]", "/", "2"}], "]"}]}], 
                  RowBox[{
                    RowBox[{"Exp", "[", 
                      RowBox[{"I", 
                        RowBox[{"(", 
                          RowBox[{"\[Alpha]", "+", 
                            RowBox[{"\[Beta]", "/", "2"}], "+", 
                            RowBox[{"\[Delta]", "/", "2"}]}], ")"}]}], "]"}], 
                    
                    RowBox[{"Cos", "[", 
                      RowBox[{"\[Gamma]", "/", "2"}], "]"}]}]}
                }], ")"}]}], ";"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"diagonal", "[", 
          RowBox[{"\[Alpha]_", ",", "\[Beta]_"}], "]"}], ":=", 
        RowBox[{"(", GridBox[{
              {"\[Alpha]", "0"},
              {"0", "\[Beta]"}
              }], ")"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Psi]", "=", 
        RowBox[{"(", GridBox[{
              {
                RowBox[{"Cos", "[", "\[Theta]", "]"}]},
              {
                RowBox[{
                  RowBox[{"Exp", "[", 
                    RowBox[{"I", " ", "\[CurlyPhi]"}], "]"}], 
                  RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}
              }], ")"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"\[Chi]", "=", 
        RowBox[{"(", GridBox[{
              {
                RowBox[{"Cos", "[", "\[Theta]\[Chi]", "]"}]},
              {
                RowBox[{
                  RowBox[{"Exp", "[", 
                    RowBox[{"I", " ", "\[CurlyPhi]\[Chi]"}], "]"}], 
                  RowBox[{"Sin", "[", "\[Theta]\[Chi]", "]"}]}]}
              }], ")"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"s0", "=", 
        RowBox[{"(", GridBox[{
              {"1"},
              {"0"}
              }], ")"}]}], ";", 
      RowBox[{"s1", "=", 
        RowBox[{"(", GridBox[{
              {"0"},
              {"1"}
              }], ")"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"zero", "=", 
        RowBox[{"(", GridBox[{
              {"0", "0"},
              {"0", "0"}
              }], ")"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"one", "=", 
        RowBox[{"(", GridBox[{
              {"1", "0"},
              {"0", "1"}
              }], ")"}]}], ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \\\"\\!\\(\
\[CurlyPhi]\[Chi]\\)\\\" is similar to existing symbol \\\"\\!\\(\[Theta]\
\[Chi]\\)\\\".\"\>"}]], "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Disturbance and deviation", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"summ", "[", 
          RowBox[{
          "expr_", ",", "uu_", ",", "dd_", ",", "vv_", ",", "gp1_", ",", 
            "nn_"}], "]"}], ":=", 
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"m", "=", "0"}], 
            RowBox[{"nn", "-", "1"}]], 
          RowBox[{"If", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"dd", "\[LeftDoubleBracket]", 
                  RowBox[{"m", "+", "1"}], "\[RightDoubleBracket]"}], 
                "\[Equal]", "zero"}], ",", "0", ",", 
              RowBox[{"expr", "[", 
                RowBox[{
                  RowBox[{"uu", "\[LeftDoubleBracket]", 
                    RowBox[{"m", "+", "1"}], "\[RightDoubleBracket]"}], ",", 
                  RowBox[{"dd", "\[LeftDoubleBracket]", 
                    RowBox[{"m", "+", "1"}], "\[RightDoubleBracket]"}], ",", 
                  RowBox[{"vv", "\[LeftDoubleBracket]", 
                    RowBox[{"m", "+", "1"}], "\[RightDoubleBracket]"}], ",", 
                  RowBox[{"gp1", "\[LeftDoubleBracket]", 
                    RowBox[{"m", "+", "1"}], "\[RightDoubleBracket]"}]}], 
                "]"}]}], "]"}]}]}], ";"}]], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"integrand1", "[", 
          RowBox[{"uum_", ",", "ddm_"}], "]"}], ":=", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"Abs", "[", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"adj", "[", "s1", "]"}], ".", "uum", ".", "ddm", 
                    ".", "\[Psi]"}], ")"}], "\[LeftDoubleBracket]", 
                RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
            "4"], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"adj", "[", "\[Psi]", "]"}], ".", "ddm", ".", "ddm", 
                ".", "\[Psi]"}], ")"}], "\[LeftDoubleBracket]", 
            RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"expr1", "[", 
          RowBox[{"uum_", ",", "ddm_", ",", "vvm_", ",", "gp1m_"}], "]"}], ":=",
         
        RowBox[{
          FractionBox["1", 
            RowBox[{"2", "\[Pi]"}]], 
          RowBox[{"NIntegrate", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"Sin", "[", 
                  RowBox[{"2", "\[Theta]"}], "]"}], 
                RowBox[{"integrand1", "[", 
                  RowBox[{"uum", ",", "ddm"}], "]"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"\[CurlyPhi]", ",", "0", ",", 
                  RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"\[Theta]", ",", "0", ",", 
                  RowBox[{"\[Pi]", "/", "2"}]}], "}"}]}], "]"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"expr2", "[", 
          RowBox[{"uum_", ",", "ddm_", ",", "vvm_", ",", "gp1m_"}], "]"}], ":=",
         
        RowBox[{
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"adj", "[", "s1", "]"}], ".", "uum", ".", "ddm", ".", 
                "ddm", ".", 
                RowBox[{"adj", "[", "uum", "]"}], ".", "s1"}], ")"}], 
            "\[LeftDoubleBracket]", 
            RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "+", 
          RowBox[{
            RowBox[{"Abs", "[", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"adj", "[", "s1", "]"}], ".", "uum", ".", "ddm", 
                    ".", "vvm", ".", "s1"}], ")"}], "\[LeftDoubleBracket]", 
                RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
            "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"expr3", "[", 
          RowBox[{"uum_", ",", "ddm_", ",", "vvm_", ",", "gp1m_"}], "]"}], ":=",
         
        RowBox[{
          SuperscriptBox["gp1m", "2"], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", "s0", "]"}], ".", "ddm", ".", "ddm", 
                  ".", "s0"}], "+", 
                RowBox[{
                  RowBox[{"adj", "[", "s1", "]"}], ".", "ddm", ".", "ddm", 
                  ".", "s1"}]}], ")"}], "\[LeftDoubleBracket]", 
            RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"expr4", "[", 
          RowBox[{"uum_", ",", "ddm_", ",", "vvm_", ",", "gp1m_"}], "]"}], ":=",
         
        RowBox[{"gp1m", " ", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"adj", "[", "s1", "]"}], ".", "uum", ".", "ddm", ".", 
                "ddm", ".", 
                RowBox[{"adj", "[", "uum", "]"}], ".", "s1"}], ")"}], 
            "\[LeftDoubleBracket]", 
            RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"s", "[", 
          RowBox[{"uu_", ",", "dd_", ",", "vv_", ",", "gp1_"}], "]"}], ":=", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"nn", "=", 
                RowBox[{"Length", "[", "uu", "]"}]}], "}"}], ",", 
            RowBox[{
              RowBox[{"summ", "[", 
                RowBox[{
                "expr1", ",", "uu", ",", "dd", ",", "vv", ",", "gp1", ",", 
                  "nn"}], "]"}], "-", 
              RowBox[{
                FractionBox["1", "3"], 
                RowBox[{"summ", "[", 
                  RowBox[{
                  "expr2", ",", "uu", ",", "dd", ",", "vv", ",", "gp1", ",", 
                    "nn"}], "]"}]}], "+", 
              FractionBox["1", "3"]}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"v", "[", 
          RowBox[{"uu_", ",", "dd_", ",", "vv_", ",", "gp1_"}], "]"}], ":=", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"nn", "=", 
                RowBox[{"Length", "[", "uu", "]"}]}], "}"}], ",", 
            RowBox[{
              RowBox[{
                FractionBox["1", "2"], 
                RowBox[{"summ", "[", 
                  RowBox[{
                  "expr3", ",", "uu", ",", "dd", ",", "vv", ",", "gp1", ",", 
                    "nn"}], "]"}]}], "-", 
              RowBox[{"summ", "[", 
                RowBox[{
                "expr4", ",", "uu", ",", "dd", ",", "vv", ",", "gp1", ",", 
                  "nn"}], "]"}], "+", 
              RowBox[{"summ", "[", 
                RowBox[{
                "expr1", ",", "uu", ",", "dd", ",", "vv", ",", "gp1", ",", 
                  "nn"}], "]"}]}]}], "]"}]}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Disturbance and deviation according to 0304092148)", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"summ0304092148", "[", 
          RowBox[{
          "expr_", ",", "\[Psi]_", ",", "mm_", ",", "gp1_", ",", "nn_"}], 
          "]"}], ":=", 
        RowBox[{
          SubsuperscriptBox["\[Sum]", 
            RowBox[{"m", "=", "0"}], 
            RowBox[{"nn", "-", "1"}]], 
          RowBox[{"If", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"mm", "\[LeftDoubleBracket]", 
                  RowBox[{"m", "+", "1"}], "\[RightDoubleBracket]"}], 
                "\[Equal]", "zero"}], ",", "0", ",", 
              RowBox[{"expr", "[", 
                RowBox[{"\[Psi]", ",", 
                  RowBox[{"mm", "\[LeftDoubleBracket]", 
                    RowBox[{"m", "+", "1"}], "\[RightDoubleBracket]"}], ",", 
                  RowBox[{"gp1", "\[LeftDoubleBracket]", 
                    RowBox[{"m", "+", "1"}], "\[RightDoubleBracket]"}]}], 
                "]"}]}], "]"}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"p", "[", 
          RowBox[{"\[Psi]_", ",", "mmm_"}], "]"}], ":=", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"adj", "[", "\[Psi]", "]"}], ".", 
              RowBox[{"adj", "[", "mmm", "]"}], ".", "mmm", ".", "\[Psi]"}], 
            ")"}], "\[LeftDoubleBracket]", 
          RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"c1pabs2", "[", 
          RowBox[{"\[Psi]_", ",", "mmm_"}], "]"}], ":=", 
        FractionBox[
          SuperscriptBox[
            RowBox[{"Abs", "[", 
              RowBox[{
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{"adj", "[", "s1", "]"}], ".", "mmm", ".", 
                    "\[Psi]"}], ")"}], "\[LeftDoubleBracket]", 
                RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
            "2"], 
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"adj", "[", "\[Psi]", "]"}], ".", 
                RowBox[{"adj", "[", "mmm", "]"}], ".", "mmm", ".", "\[Psi]"}],
               ")"}], "\[LeftDoubleBracket]", 
            RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"c1abs2", "[", "\[Psi]_", "]"}], ":=", 
        SuperscriptBox[
          RowBox[{"Abs", "[", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"adj", "[", "\[Psi]", "]"}], ".", "s1"}], ")"}], 
              "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
          "2"]}], ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(c1abs2\\)\\\" is similar to existing symbol \
\\\"\\!\\(c1pabs2\\)\\\".\"\>"}]], "Message"]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"expr03040921481", "[", 
        RowBox[{"\[Psi]_", ",", "mmm_", ",", "gp1m_"}], "]"}], ":=", 
      RowBox[{
        RowBox[{"p", "[", 
          RowBox[{"\[Psi]", ",", "mmm"}], "]"}], 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{
              RowBox[{"c1pabs2", "[", 
                RowBox[{"\[Psi]", ",", "mmm"}], "]"}], "-", 
              RowBox[{"c1abs2", "[", "\[Psi]", "]"}]}], ")"}], 
          "2"]}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"integrand03040921481", "[", 
          RowBox[{"\[Psi]_", ",", "mm_", ",", "gp1_"}], "]"}], ":=", 
        RowBox[{"summ0304092148", "[", 
          RowBox[{
          "expr03040921481", ",", "\[Psi]", ",", "mm", ",", "gp1", ",", 
            RowBox[{"Length", "[", "mm", "]"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"expr03040921482", "[", 
        RowBox[{"\[Psi]_", ",", "mmm_", ",", "gp1m_"}], "]"}], ":=", 
      RowBox[{
        RowBox[{"p", "[", 
          RowBox[{"\[Psi]", ",", "mmm"}], "]"}], 
        SuperscriptBox[
          RowBox[{"(", 
            RowBox[{"gp1m", "-", 
              RowBox[{"c1pabs2", "[", 
                RowBox[{"\[Psi]", ",", "mmm"}], "]"}]}], ")"}], 
          "2"]}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"integrand03040921482", "[", 
          RowBox[{"\[Psi]_", ",", "mm_", ",", "gp1_"}], "]"}], ":=", 
        RowBox[{"summ0304092148", "[", 
          RowBox[{
          "expr03040921482", ",", "\[Psi]", ",", "mm", ",", "gp1", ",", 
            RowBox[{"Length", "[", "mm", "]"}]}], "]"}]}], ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"s0304092148", "[", 
          RowBox[{"mm_", ",", "gp1_"}], "]"}], ":=", 
        RowBox[{
          FractionBox["1", 
            RowBox[{"2", "\[Pi]"}]], 
          RowBox[{"NIntegrate", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"Sin", "[", 
                  RowBox[{"2", "\[Theta]"}], "]"}], 
                RowBox[{"integrand03040921481", "[", 
                  RowBox[{"\[Psi]", ",", "mm", ",", "gp1"}], "]"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"\[CurlyPhi]", ",", "0", ",", 
                  RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"\[Theta]", ",", "0", ",", 
                  RowBox[{"\[Pi]", "/", "2"}]}], "}"}]}], "]"}]}]}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"v0304092148", "[", 
          RowBox[{"mm_", ",", "gp1_"}], "]"}], ":=", 
        RowBox[{
          FractionBox["1", 
            RowBox[{"2", "\[Pi]"}]], 
          RowBox[{"NIntegrate", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"Sin", "[", 
                  RowBox[{"2", "\[Theta]"}], "]"}], 
                RowBox[{"integrand03040921482", "[", 
                  RowBox[{"\[Psi]", ",", "mm", ",", "gp1"}], "]"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"\[CurlyPhi]", ",", "0", ",", 
                  RowBox[{"2", "\[Pi]"}]}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"\[Theta]", ",", "0", ",", 
                  RowBox[{"\[Pi]", "/", "2"}]}], "}"}]}], "]"}]}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(v0304092148\\)\\\" is similar to existing symbol \
\\\"\\!\\(s0304092148\\)\\\".\"\>"}]], "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 1", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"terseForm", "/@", "\[IndentingNewLine]", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"uu0", "=", 
            RowBox[{"unitary", "[", 
              RowBox[{"1.24", ",", 
                RowBox[{"-", "0.407"}], ",", "5.33", ",", 
                RowBox[{"-", "1.78"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
          
          RowBox[{"vv0", "=", 
            RowBox[{"unitary", "[", 
              RowBox[{"3.02", ",", "1.11", ",", 
                RowBox[{"-", "5.65"}], ",", 
                RowBox[{"-", "3.34"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
          
          RowBox[{"dd0", "=", 
            RowBox[{"diagonal", "[", 
              RowBox[{"1", ",", "1"}], "]"}]}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.61\"\>",
                        0.61388934767067971,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.64\"\>",
                          0.64240520791092015,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.39\"\>",
                        -0.39025756549336865,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.24\"\>",
                          0.24115233824407309,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15976118556534802,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.43\"\>",
                          0.43003695335650999,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.88\"\>",
                        -0.87904497130007864,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.13\"\>",
                          0.12970937038876257,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.52\"\>",
                        0.51871085106837878,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.8\"\>",
                          0.79624899096719481,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.22\"\>",
                        0.21801991316128075,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.22\"\>",
                          0.2222474180565204,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15808871029825142,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.27\"\>",
                          0.26820618382855693,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.31\"\>",
                        0.31171513296788506,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.9\"\>",
                          0.89772327501918514,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"mm0", "=", 
        RowBox[{"uu0", ".", "dd0", ".", "vv0"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"gp0", "=", "0.43"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"s", "[", 
        RowBox[{
          RowBox[{"{", "uu0", "}"}], ",", 
          RowBox[{"{", "dd0", "}"}], ",", 
          RowBox[{"{", "vv0", "}"}], ",", 
          RowBox[{"{", "gp0", "}"}]}], "]"}], "//", 
      "Chop"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"s0304092148", "[", 
        RowBox[{
          RowBox[{"{", "mm0", "}"}], ",", 
          RowBox[{"{", "gp0", "}"}]}], "]"}], "//", 
      "Chop"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"v", "[", 
        RowBox[{
          RowBox[{"{", "uu0", "}"}], ",", 
          RowBox[{"{", "dd0", "}"}], ",", 
          RowBox[{"{", "vv0", "}"}], ",", 
          RowBox[{"{", "gp0", "}"}]}], "]"}], "//", 
      "Chop"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"v0304092148", "[", 
        RowBox[{
          RowBox[{"{", "mm0", "}"}], ",", 
          RowBox[{"{", "gp0", "}"}]}], "]"}], "//", "Chop"}]}], "Input"],

Cell[BoxData["0.02584166564492918`"], "Output"],

Cell[BoxData["0.02584166525922965`"], "Output"],

Cell[BoxData["0.08823333375852163`"], "Output"],

Cell[BoxData["0.0882333334335564`"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"terseForm", "/@", "\[IndentingNewLine]", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"uu0", "=", "one"}], ",", "\[IndentingNewLine]", 
          RowBox[{"vv0", "=", "one"}], ",", "\[IndentingNewLine]", 
          RowBox[{"dd0", "=", 
            RowBox[{"diagonal", "[", 
              RowBox[{"0", ",", 
                RowBox[{"Sqrt", "[", "0.5", "]"}]}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"uu1", "=", "one"}], ",", "\[IndentingNewLine]", 
          RowBox[{"vv1", "=", "one"}], ",", "\[IndentingNewLine]", 
          RowBox[{"dd1", "=", 
            RowBox[{"diagonal", "[", 
              RowBox[{"1", ",", 
                RowBox[{"Sqrt", "[", "0.5", "]"}]}], "]"}]}]}], 
        "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.71\"\>",
                      0.70710678118654757,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.71\"\>",
                      0.70710678118654757,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm0", "=", 
        RowBox[{"uu0", ".", "dd0", ".", "vv0"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm1", "=", 
        RowBox[{"uu1", ".", "dd1", ".", "vv1"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"adj", "[", "mm0", "]"}], ".", "mm0"}], "+", 
        RowBox[{
          RowBox[{"adj", "[", "mm1", "]"}], ".", "mm1"}]}], "//", 
      "terseForm"}]], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  1.0000000000000002,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"gp10", "=", "0.43"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"gp11", "=", "0.63"}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"s", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"uu0", ",", "uu1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"dd0", ",", "dd1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"vv0", ",", "vv1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"gp10", ",", "gp11"}], "}"}]}], "]"}], "//", 
      "Chop"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"s0304092148", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mm0", ",", "mm1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"gp10", ",", "gp11"}], "}"}]}], "]"}], "//", 
      "Chop"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"v", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"uu0", ",", "uu1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"dd0", ",", "dd1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"vv0", ",", "vv1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"gp10", ",", "gp11"}], "}"}]}], "]"}], "//", 
      "Chop"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"v0304092148", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mm0", ",", "mm1"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"gp10", ",", "gp11"}], "}"}]}], "]"}], "//", 
      "Chop"}]}], "Input"],

Cell[BoxData["0.0529610278026606`"], "Output"],

Cell[BoxData["0.05296102778629445`"], "Output"],

Cell[BoxData["0.20019436113599404`"], "Output"],

Cell[BoxData["0.20019436113598196`"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[CellGroupData[{

Cell[TextData[{
  "First three ",
  Cell[BoxData[
      FormBox["U", TraditionalForm]]],
  "'s, ",
  Cell[BoxData[
      FormBox["V", TraditionalForm]]],
  "'s and ",
  Cell[BoxData[
      FormBox["D", TraditionalForm]]],
  "'s:"
}], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"terseForm", "/@", "\[IndentingNewLine]", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"uu0", "=", 
              RowBox[{"unitary", "[", 
                RowBox[{"1.24", ",", 
                  RowBox[{"-", "0.407"}], ",", "5.33", ",", 
                  RowBox[{"-", "1.78"}]}], "]"}]}], ")"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(", 
            RowBox[{"vv0", "=", 
              RowBox[{"unitary", "[", 
                RowBox[{"3.02", ",", "1.11", ",", 
                  RowBox[{"-", "5.65"}], ",", 
                  RowBox[{"-", "3.34"}]}], "]"}]}], ")"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(", 
            RowBox[{"dd0", "=", 
              RowBox[{"diagonal", "[", 
                RowBox[{"0.71", ",", "0.24"}], "]"}]}], ")"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(", 
            RowBox[{"uu1", "=", 
              RowBox[{"unitary", "[", 
                RowBox[{"1.57", ",", "4.61", ",", 
                  RowBox[{"-", "0.213"}], ",", "0.749"}], "]"}]}], ")"}], ",",
           "\[IndentingNewLine]", 
          RowBox[{"(", 
            RowBox[{"vv1", "=", 
              RowBox[{"unitary", "[", 
                RowBox[{"5.82", ",", "1.53", ",", "0.700", ",", 
                  RowBox[{"-", "6.20"}]}], "]"}]}], ")"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(", 
            RowBox[{"dd1", "=", 
              RowBox[{"diagonal", "[", 
                RowBox[{"0.10", ",", "0.41"}], "]"}]}], ")"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(", 
            RowBox[{"uu2", "=", 
              RowBox[{"unitary", "[", 
                RowBox[{"3.13", ",", 
                  RowBox[{"-", "0.963"}], ",", "3.63", ",", 
                  RowBox[{"-", "1.65"}]}], "]"}]}], ")"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(", 
            RowBox[{"vv2", "=", 
              RowBox[{"unitary", "[", 
                RowBox[{"5.98", ",", 
                  RowBox[{"-", "3.94"}], ",", "6.02", ",", 
                  RowBox[{"-", "4.34"}]}], "]"}]}], ")"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"(", 
            RowBox[{"dd2", "=", 
              RowBox[{"diagonal", "[", 
                RowBox[{"0", ",", "0"}], "]"}]}], ")"}]}], "}"}]}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.61\"\>",
                        0.61388934767067971,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.64\"\>",
                          0.64240520791092015,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.39\"\>",
                        -0.39025756549336865,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.24\"\>",
                          0.24115233824407309,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15976118556534802,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.43\"\>",
                          0.43003695335650999,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.88\"\>",
                        -0.87904497130007864,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.13\"\>",
                          0.12970937038876257,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.52\"\>",
                        0.51871085106837878,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.8\"\>",
                          0.79624899096719481,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.22\"\>",
                        0.21801991316128075,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.22\"\>",
                          0.2222474180565204,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15808871029825142,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.27\"\>",
                          0.26820618382855693,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.31\"\>",
                        0.31171513296788506,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.9\"\>",
                          0.89772327501918514,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.71\"\>",
                      0.70999999999999996,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.24\"\>",
                      0.23999999999999999,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.44\"\>",
                        0.44258742495187353,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.89\"\>",
                          0.8904026834460359,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.099\"\>",
                        0.099465963581254591,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.037\"\>",
                          0.037496062132185301,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.1\"\>",
                        0.099525555644023822,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.037\"\>",
                          0.037337599819973456,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.44\"\>",
                        -0.44400496606106915,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.89\"\>",
                          0.88969666601968367,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.28\"\>",
                        -0.27851737314116226,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.9\"\>",
                          0.89713386208569268,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.13\"\>",
                        0.12852529336949642,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.32\"\>",
                          0.31789959943674145,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.33\"\>",
                        -0.33135342319917199,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.088\"\>",
                          0.088225933216635397,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.88\"\>",
                        -0.88452545398334392,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.32\"\>",
                          0.31628438927301405,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.1\"\>",
                      0.10000000000000001,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.41\"\>",
                      0.40999999999999998,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.066\"\>",
                        0.065862462213001807,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.23264026830866552,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.91\"\>",
                        0.909795424389784,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.34\"\>",
                          0.33736171595473874,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.92\"\>",
                        -0.91737203547774793,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.32\"\>",
                          0.3161790476232203,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.06\"\>",
                        0.060451407698131641,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.23410464684580962,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.76\"\>",
                        0.76127226298587514,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.64\"\>",
                          0.63501782637507398,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.11\"\>",
                        -0.11494944751539393,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.063\"\>",
                          0.063273424991435886,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.13\"\>",
                        0.1305152846609669,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.014\"\>",
                          0.013515260415838956,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.26\"\>",
                        0.26366439820456961,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.96\"\>",
                          0.95564856685142419,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Construction of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["M", "3"], "=", 
          RowBox[{
            SubscriptBox["U", "3"], 
            SubscriptBox["D", "3"], 
            SubscriptBox["V", "3"]}]}], TraditionalForm]]],
  " so that the set of operators is complete (using diagonalization of ",
  Cell[BoxData[
      FormBox[
        RowBox[{"tmp", "=", 
          RowBox[{"1", "-", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["M", "0", "\[Dagger]"], 
                  SubscriptBox["M", "0"]}], "+", 
                RowBox[{
                  SubsuperscriptBox["M", "1", "\[Dagger]"], 
                  SubscriptBox["M", "1"]}], "+", 
                RowBox[{
                  SubsuperscriptBox["M", "2", "\[Dagger]"], 
                  SubscriptBox["M", "2"]}]}], ")"}]}]}], TraditionalForm]]],
  ", note that it is neccessary that the eigenvalues of ",
  Cell[BoxData[
      FormBox["tmp", TraditionalForm]]],
  " are positive):"
}], "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm0", "=", 
        RowBox[{"uu0", ".", "dd0", ".", "vv0"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm1", "=", 
        RowBox[{"uu1", ".", "dd1", ".", "vv1"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm2", "=", 
        RowBox[{"uu2", ".", "dd2", ".", "vv2"}]}], ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"tmp", "=", 
        RowBox[{"one", "-", 
          RowBox[{"(", 
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "mm0", "]"}], ".", "mm0"}], "+", 
              RowBox[{
                RowBox[{"adj", "[", "mm1", "]"}], ".", "mm1"}], "+", 
              RowBox[{
                RowBox[{"adj", "[", "mm2", "]"}], ".", "mm2"}]}], ")"}]}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"{", 
          RowBox[{"evalues", ",", "evectors"}], "}"}], "=", 
        RowBox[{"Eigensystem", "[", "tmp", "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"(", 
        RowBox[{"uu3", "=", 
          RowBox[{"unitary", "[", 
            RowBox[{"5.43", ",", 
              RowBox[{"-", "0.561"}], ",", 
              RowBox[{"-", "0.581"}], ",", 
              RowBox[{"-", "2.64"}]}], "]"}]}], ")"}], 
      RowBox[{"(*", "arbitrary", "*)"}], "//", 
      "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"(", 
        RowBox[{"dd3", "=", 
          RowBox[{"diagonal", "[", 
            RowBox[{
              RowBox[{"Sqrt", "[", 
                RowBox[{
                "evalues", "\[LeftDoubleBracket]", "1", 
                  "\[RightDoubleBracket]"}], "]"}], ",", 
              RowBox[{"Sqrt", "[", 
                RowBox[{
                "evalues", "\[LeftDoubleBracket]", "2", 
                  "\[RightDoubleBracket]"}], "]"}]}], "]"}]}], ")"}], "//", 
      "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"(", 
        RowBox[{"vv3", "=", 
          RowBox[{"adj", "[", 
            RowBox[{"Transpose", "[", "evectors", "]"}], "]"}]}], ")"}], "//",
       "terseForm"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                RowBox[{
                  InterpretationBox["\<\"0.7\"\>",
                    0.70278286280650326,
                    AutoDelete->True], "+", 
                  RowBox[{
                    InterpretationBox["\<\"0.65\"\>",
                      0.65119378963924157,
                      AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                RowBox[{
                  InterpretationBox["\<\"-0.091\"\>",
                    -0.090615147668758278,
                    AutoDelete->True], "-", 
                  RowBox[{
                    InterpretationBox["\<\"0.27\"\>",
                      0.27172006016072492,
                      AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
              {
                RowBox[{
                  InterpretationBox["\<\"-0.28\"\>",
                    -0.28147420728191869,
                    AutoDelete->True], "-", 
                  RowBox[{
                    InterpretationBox["\<\"0.053\"\>",
                      0.053058144669733041,
                      AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                RowBox[{
                  InterpretationBox["\<\"-0.74\"\>",
                    -0.74020602924380929,
                    AutoDelete->True], "-", 
                  RowBox[{
                    InterpretationBox["\<\"0.61\"\>",
                      0.60831910884858276,
                      AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.92\"\>",
                  0.92058364467933784,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"0.64\"\>",
                  0.64243735348196129,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                RowBox[{
                  InterpretationBox["\<\"-0.47\"\>",
                    -0.47101501721528943,
                    AutoDelete->True], "-", 
                  RowBox[{
                    InterpretationBox["\<\"0.057\"\>",
                      0.056981008012198041,
                      AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                InterpretationBox["\<\"0.88\"\>",
                  0.88028291945464587,
                  AutoDelete->True]},
              {
                InterpretationBox["\<\"0.88\"\>",
                  0.88028291945464576,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"0.47\"\>",
                    0.47101501721528938,
                    AutoDelete->True], "-", 
                  RowBox[{
                    InterpretationBox["\<\"0.057\"\>",
                      0.056981008012198013,
                      AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"mm3", "=", 
        RowBox[{"uu3", ".", "dd3", ".", "vv3"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"(*", 
        RowBox[{"Tests", ":"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{
        RowBox[{
          RowBox[{"adj", "[", "vv3", "]"}], ".", "vv3"}], "//", 
        "terseForm"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"adj", "[", "mm0", "]"}], ".", "mm0"}], "+", 
        RowBox[{
          RowBox[{"adj", "[", "mm1", "]"}], ".", "mm1"}], "+", 
        RowBox[{
          RowBox[{"adj", "[", "mm2", "]"}], ".", "mm2"}], "+", 
        RowBox[{
          RowBox[{"adj", "[", "mm3", "]"}], ".", "mm3"}]}], "//", 
      "terseForm"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999978,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999978,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Some guesses:", "Subsubsection"],

Cell["\<\
Note that for testing purposes we also use weird, e.g. negative, \
guesses.\
\>", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"gp10", "=", "1.47"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"gp11", "=", "0.78"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"gp12", "=", 
        RowBox[{"-", "0.43"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"gp13", "=", 
        RowBox[{"-", "0.26"}]}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Disturbance and deviation with the above values:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"s", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"uu0", ",", "uu1", ",", "uu2", ",", "uu3"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"dd0", ",", "dd1", ",", "dd2", ",", "dd3"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"vv0", ",", "vv1", ",", "vv2", ",", "vv3"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"gp10", ",", "gp11", ",", "gp12", ",", "gp13"}], "}"}]}], 
        "]"}], "//", "Chop"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"s0304092148", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mm0", ",", "mm1", ",", "mm2", ",", "mm3"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"gp10", ",", "gp11", ",", "gp12", ",", "gp13"}], "}"}]}], 
        "]"}], "//", "Chop"}]}], "Input"],

Cell[BoxData["0.14570363388602325`"], "Output"],

Cell[BoxData["0.14570363375915962`"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"v", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"uu0", ",", "uu1", ",", "uu2", ",", "uu3"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"dd0", ",", "dd1", ",", "dd2", ",", "dd3"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"vv0", ",", "vv1", ",", "vv2", ",", "vv3"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"gp10", ",", "gp11", ",", "gp12", ",", "gp13"}], "}"}]}], 
        "]"}], "//", "Chop"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"v0304092148", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"mm0", ",", "mm1", ",", "mm2", ",", "mm3"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"gp10", ",", "gp11", ",", "gp12", ",", "gp13"}], "}"}]}], 
        "]"}], "//", "Chop"}]}], "Input"],

Cell[BoxData["0.6999846217049026`"], "Output"],

Cell[BoxData["0.6999846132222157`"], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
Evaluator->"Local",
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 35, 0, 45, "Subsection"],
Cell[1777, 53, 401, 10, 27, "Input"],
Cell[2181, 65, 198, 5, 27, "Input"],
Cell[2382, 72, 2763, 63, 79, "Input"],
Cell[5148, 137, 278, 8, 45, "Input"],

Cell[CellGroupData[{
Cell[5451, 149, 1492, 45, 197, "Input"],
Cell[6946, 196, 279, 6, 23, "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7274, 208, 47, 0, 45, "Subsection"],
Cell[7324, 210, 1239, 27, 35, "Input"],
Cell[8566, 239, 5688, 145, 226, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14291, 389, 72, 0, 45, "Subsection"],
Cell[14366, 391, 984, 23, 35, "Input"],

Cell[CellGroupData[{
Cell[15375, 418, 1775, 46, 87, "Input"],
Cell[17153, 466, 261, 6, 23, "Message"]
}, Open  ]],
Cell[17429, 475, 1675, 42, 79, "Input"],
Cell[19107, 519, 830, 21, 44, "Input"],

Cell[CellGroupData[{
Cell[19962, 544, 830, 21, 44, "Input"],
Cell[20795, 567, 270, 6, 23, "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[21114, 579, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[21170, 583, 748, 18, 75, "Input"],
Cell[21921, 603, 4337, 102, 45, "Output"]
}, Open  ]],
Cell[26273, 708, 121, 3, 27, "Input"],
Cell[26397, 713, 79, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[26501, 719, 950, 27, 75, "Input"],
Cell[27454, 748, 47, 0, 27, "Output"],
Cell[27504, 750, 47, 0, 27, "Output"],
Cell[27554, 752, 47, 0, 27, "Output"],
Cell[27604, 754, 46, 0, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[27699, 760, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[27755, 764, 775, 17, 123, "Input"],
Cell[28533, 783, 4328, 122, 45, "Output"]
}, Open  ]],
Cell[32876, 908, 252, 7, 43, "Input"],

Cell[CellGroupData[{
Cell[33153, 919, 237, 7, 27, "Input"],
Cell[33393, 928, 665, 20, 58, "Output"]
}, Open  ]],
Cell[34073, 951, 163, 4, 43, "Input"],

Cell[CellGroupData[{
Cell[34261, 959, 1385, 40, 75, "Input"],
Cell[35649, 1001, 46, 0, 27, "Output"],
Cell[35698, 1003, 47, 0, 27, "Output"],
Cell[35748, 1005, 47, 0, 27, "Output"],
Cell[35798, 1007, 47, 0, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[35894, 1013, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[35950, 1017, 250, 11, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[36225, 1032, 2375, 57, 171, "Input"],
Cell[38603, 1091, 12989, 302, 83, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[51641, 1399, 1082, 30, 58, "Subsubsection"],
Cell[52726, 1431, 381, 11, 59, "Input"],
Cell[53110, 1444, 437, 12, 27, "Input"],
Cell[53550, 1458, 193, 5, 27, "Input"],

Cell[CellGroupData[{
Cell[53768, 1467, 1122, 30, 59, "Input"],
Cell[54893, 1499, 1651, 40, 58, "Output"],
Cell[56547, 1541, 686, 20, 58, "Output"],
Cell[57236, 1563, 1192, 30, 58, "Output"]
}, Open  ]],
Cell[58443, 1596, 121, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[58589, 1603, 645, 18, 59, "Input"],
Cell[59237, 1623, 666, 20, 58, "Output"],
Cell[59906, 1645, 682, 20, 58, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[60637, 1671, 38, 0, 42, "Subsubsection"],
Cell[60678, 1673, 99, 3, 32, "Text"],
Cell[60780, 1678, 373, 10, 75, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[61190, 1693, 73, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[61288, 1697, 845, 20, 43, "Input"],
Cell[62136, 1719, 47, 0, 27, "Output"],
Cell[62186, 1721, 47, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[62270, 1726, 845, 20, 43, "Input"],
Cell[63118, 1748, 46, 0, 27, "Output"],
Cell[63167, 1750, 46, 0, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

