(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     33275,        994]*)
(*NotebookOutlinePosition[     33931,       1018]*)
(*  CellTagsIndexPosition[     33887,       1014]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Misc", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"assums", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"a", "\[GreaterEqual]", "0"}], ",", 
            RowBox[{"a", "\[LessEqual]", "1"}], ",", 
            RowBox[{"b", "\[GreaterEqual]", "0"}], ",", 
            RowBox[{"b", "\[LessEqual]", "1"}], ",", 
            RowBox[{"\[Beta]", "\[GreaterEqual]", "0"}], ",", 
            RowBox[{"\[Beta]", "\[LessEqual]", "1"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"assumsU", "=", 
        RowBox[{"Append", "[", 
          RowBox[{"assums", ",", 
            RowBox[{"a", "\[NotEqual]", "b"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"assumsE", "=", 
        RowBox[{"Append", "[", 
          RowBox[{"assums", ",", 
            RowBox[{"a", "\[Equal]", "b"}]}], "]"}]}], ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(assumsU\\)\\\" is similar to existing symbol \
\\\"\\!\\(assums\\)\\\".\"\>"}]], "Message"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(assumsE\\)\\\" is similar to existing symbols \\!\\({assums, \
assumsU}\\).\"\>"}]], "Message"]
}, Open  ]],

Cell["\<\
Numerical integration function that doesn't give warnings when the \
integrand is 0:\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"nintegrate", "[", 
          RowBox[{"f_", ",", 
            RowBox[{"{", 
              RowBox[{"x_", ",", "xmin_", ",", "xmax_"}], "}"}], ",", "wp_"}],
           "]"}], ":=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"NumberQ", "[", "f", "]"}], "&&", 
              RowBox[{"f", "\[Equal]", "0"}]}], ",", "0", ",", 
            RowBox[{"NIntegrate", "[", 
              RowBox[{"f", ",", 
                RowBox[{"{", 
                  RowBox[{"x", ",", "xmin", ",", "xmax"}], "}"}], ",", 
                RowBox[{"WorkingPrecision", "\[Rule]", "wp"}]}], "]"}]}], 
          "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Proof", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"integrand1", "[", 
          RowBox[{"a_", ",", "b_"}], "]"}], "=", 
        FractionBox[
          RowBox[{
            RowBox[{"Sin", "[", 
              RowBox[{"2", "\[Theta]"}], "]"}], 
            SuperscriptBox[
              RowBox[{"Cos", "[", "\[Theta]", "]"}], "4"]}], 
          RowBox[{
            RowBox[{
              SuperscriptBox["a", "2"], 
              SuperscriptBox[
                RowBox[{"Cos", "[", "\[Theta]", "]"}], "2"]}], "+", 
            RowBox[{
              SuperscriptBox["b", "2"], 
              SuperscriptBox[
                RowBox[{"Sin", "[", "\[Theta]", "]"}], "2"]}]}]]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp1U", "=", 
        RowBox[{"Integrate", "[", 
          RowBox[{
            RowBox[{"integrand1", "[", 
              RowBox[{"a", ",", "b"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Theta]", ",", "0", ",", 
                RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
            RowBox[{"Assumptions", "\[Rule]", "assumsU"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp1E", "=", 
        RowBox[{"Integrate", "[", 
          RowBox[{
            RowBox[{"integrand1", "[", 
              RowBox[{"a", ",", "b"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Theta]", ",", "0", ",", 
                RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
            RowBox[{"Assumptions", "\[Rule]", "assumsE"}]}], "]"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"int1", "[", 
          RowBox[{"a_", ",", "b_"}], " ", "]"}], ":=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"a", "\[Equal]", "b"}], ",", 
            FractionBox["1", 
              RowBox[{"3", " ", 
                SuperscriptBox["b", "2"]}]], ",", 
            RowBox[{
              FractionBox["1", 
                RowBox[{"2", " ", 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        SuperscriptBox["a", "2"], "-", 
                        SuperscriptBox["b", "2"]}], ")"}], "3"]}]], 
              RowBox[{"(", 
                RowBox[{
                  SuperscriptBox["a", "4"], "-", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["a", "2"], " ", 
                    SuperscriptBox["b", "2"]}], "+", 
                  RowBox[{"3", " ", 
                    SuperscriptBox["b", "4"]}], "+", 
                  RowBox[{"4", " ", 
                    SuperscriptBox["b", "4"], " ", 
                    RowBox[{"Log", "[", 
                      FractionBox["a", "b"], "]"}]}]}], ")"}]}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp1U", "-", 
        RowBox[{"int1", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"#", ",", "assumsU"}], "]"}], 
        "&"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp1E", "-", 
        RowBox[{"int1", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"#", ",", "assumsE"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData["0"], "Output"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"integrand2", "[", 
          RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
        FractionBox[
          RowBox[{
            RowBox[{"Sin", "[", 
              RowBox[{"2", "\[Theta]"}], "]"}], 
            SuperscriptBox[
              RowBox[{"Sin", "[", "\[Theta]", "]"}], "4"]}], 
          RowBox[{
            RowBox[{
              SuperscriptBox["a", "2"], 
              SuperscriptBox[
                RowBox[{"Cos", "[", "\[Theta]", "]"}], "2"]}], "+", 
            RowBox[{
              SuperscriptBox["b", "2"], 
              SuperscriptBox[
                RowBox[{"Sin", "[", "\[Theta]", "]"}], "2"]}]}]]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp2U", "=", 
        RowBox[{"Integrate", "[", 
          RowBox[{
            RowBox[{"integrand2", "[", 
              RowBox[{"a", ",", "b"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Theta]", ",", "0", ",", 
                RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
            RowBox[{"Assumptions", "\[Rule]", "assumsU"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp2E", "=", 
        RowBox[{"Integrate", "[", 
          RowBox[{
            RowBox[{"integrand2", "[", 
              RowBox[{"a", ",", "b"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Theta]", ",", "0", ",", 
                RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
            RowBox[{"Assumptions", "\[Rule]", "assumsE"}]}], "]"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"int2", "[", 
          RowBox[{"a_", ",", "b_"}], "]"}], "=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"a", "\[Equal]", "b"}], ",", 
            FractionBox["1", 
              RowBox[{"3", 
                SuperscriptBox["b", "2"]}]], ",", 
            FractionBox[
              RowBox[{
                RowBox[{
                  RowBox[{"-", "3"}], " ", 
                  SuperscriptBox["a", "4"]}], "+", 
                RowBox[{"4", " ", 
                  SuperscriptBox["a", "2"], " ", 
                  SuperscriptBox["b", "2"]}], "-", 
                SuperscriptBox["b", "4"], "+", 
                RowBox[{"4", " ", 
                  SuperscriptBox["a", "4"], " ", 
                  RowBox[{"Log", "[", 
                    FractionBox["a", "b"], "]"}]}]}], 
              RowBox[{"2", " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["a", "2"], "-", 
                      SuperscriptBox["b", "2"]}], ")"}], "3"]}]]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp2U", "-", 
        RowBox[{"int2", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"#", ",", "assumsU"}], "]"}], 
        "&"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp2E", "-", 
        RowBox[{"int2", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"#", ",", "assumsE"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData["0"], "Output"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"integrand3", "[", 
          RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
        FractionBox[
          RowBox[{
            RowBox[{"Sin", "[", 
              RowBox[{"2", "\[Theta]"}], "]"}], 
            SuperscriptBox[
              RowBox[{"Cos", "[", "\[Theta]", "]"}], "2"], 
            SuperscriptBox[
              RowBox[{"Sin", "[", "\[Theta]", "]"}], "2"]}], 
          RowBox[{
            RowBox[{
              SuperscriptBox["a", "2"], 
              SuperscriptBox[
                RowBox[{"Cos", "[", "\[Theta]", "]"}], "2"]}], "+", 
            RowBox[{
              SuperscriptBox["b", "2"], 
              SuperscriptBox[
                RowBox[{"Sin", "[", "\[Theta]", "]"}], "2"]}]}]]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp3U", "=", 
        RowBox[{"Integrate", "[", 
          RowBox[{
            RowBox[{"integrand3", "[", 
              RowBox[{"a", ",", "b"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Theta]", ",", "0", ",", 
                RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
            RowBox[{"Assumptions", "\[Rule]", "assumsU"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp3E", "=", 
        RowBox[{"Integrate", "[", 
          RowBox[{
            RowBox[{"integrand3", "[", 
              RowBox[{"a", ",", "b"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Theta]", ",", "0", ",", 
                RowBox[{"\[Pi]", "/", "2"}]}], "}"}], ",", 
            RowBox[{"Assumptions", "\[Rule]", "assumsE"}]}], "]"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(tmp3E\\)\\\" is similar to existing symbol \
\\\"\\!\\(tmp3U\\)\\\".\"\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"int3", "[", 
          RowBox[{"a_", ",", "b_"}], "]"}], "=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"a", "\[Equal]", "b"}], ",", 
            FractionBox["1", 
              RowBox[{"6", " ", 
                SuperscriptBox["b", "2"]}]], ",", 
            FractionBox[
              RowBox[{
                SuperscriptBox["a", "4"], "-", 
                SuperscriptBox["b", "4"], "+", 
                RowBox[{"4", " ", 
                  SuperscriptBox["a", "2"], " ", 
                  SuperscriptBox["b", "2"], " ", 
                  RowBox[{"Log", "[", 
                    FractionBox["b", "a"], "]"}]}]}], 
              RowBox[{"2", " ", 
                SuperscriptBox[
                  RowBox[{"(", 
                    RowBox[{
                      SuperscriptBox["a", "2"], "-", 
                      SuperscriptBox["b", "2"]}], ")"}], "3"]}]]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp3U", "-", 
        RowBox[{"int3", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"#", ",", "assumsU"}], "]"}], 
        "&"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmp3E", "-", 
        RowBox[{"int3", "[", 
          RowBox[{"a", ",", "b"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"#", ",", "assumsE"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData["0"], "Output"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"tmp4", "[", 
          RowBox[{"a_", ",", "b_", ",", "\[Beta]_"}], "]"}], ":=", 
        RowBox[{
          RowBox[{
            SuperscriptBox["a", "4"], 
            SuperscriptBox["\[Beta]", "2"], 
            RowBox[{"int1", "[", 
              RowBox[{"a", ",", "b"}], "]"}]}], "+", 
          RowBox[{
            SuperscriptBox["b", "4"], 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "\[Beta]"}], ")"}], "2"], 
            RowBox[{"int2", "[", 
              RowBox[{"a", ",", "b"}], "]"}]}], "+", 
          RowBox[{"4", 
            SuperscriptBox["a", "2"], 
            SuperscriptBox["b", "2"], "\[Beta]", 
            RowBox[{"(", 
              RowBox[{"1", "-", "\[Beta]"}], ")"}], 
            RowBox[{"int3", "[", 
              RowBox[{"a", ",", "b"}], "]"}]}]}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"expr", "[", 
          RowBox[{"a_", ",", "b_", ",", "\[Beta]_"}], "]"}], ":=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"a", "\[Equal]", "b"}], ",", 
            FractionBox[
              SuperscriptBox["a", "2"], "3"], ",", 
            RowBox[{
              RowBox[{
                FractionBox["1", 
                  RowBox[{"2", " ", 
                    SuperscriptBox[
                      RowBox[{"(", 
                        RowBox[{
                          SuperscriptBox["a", "2"], "-", 
                          SuperscriptBox["b", "2"]}], ")"}], "2"]}]], 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      SuperscriptBox["a", "6"], " ", 
                      SuperscriptBox["\[Beta]", "2"]}], "+", 
                    RowBox[{
                      SuperscriptBox["b", "6"], " ", 
                      SuperscriptBox[
                        RowBox[{"(", 
                          RowBox[{"1", "-", "\[Beta]"}], ")"}], "2"]}], "+", 
                    RowBox[{
                      SuperscriptBox["a", "4"], " ", 
                      SuperscriptBox["b", "2"], "\[Beta]", " ", 
                      RowBox[{"(", 
                        RowBox[{"4", "-", 
                          RowBox[{"7", " ", "\[Beta]"}]}], ")"}]}], "+", 
                    RowBox[{
                      SuperscriptBox["a", "2"], " ", 
                      SuperscriptBox["b", "4"], " ", 
                      RowBox[{"(", 
                        RowBox[{"1", "-", "\[Beta]"}], ")"}], " ", 
                      RowBox[{"(", 
                        RowBox[{
                          RowBox[{"7", " ", "\[Beta]"}], "-", "3"}], 
                        ")"}]}]}], ")"}]}], "+", 
              RowBox[{
                FractionBox[
                  RowBox[{"2", " ", 
                    SuperscriptBox["a", "4"], " ", 
                    SuperscriptBox["b", "4"]}], 
                  SuperscriptBox[
                    RowBox[{"(", 
                      RowBox[{
                        SuperscriptBox["a", "2"], "-", 
                        SuperscriptBox["b", "2"]}], ")"}], "3"]], " ", 
                RowBox[{"(", 
                  RowBox[{"1", "-", 
                    RowBox[{"6", " ", "\[Beta]", 
                      RowBox[{"(", 
                        RowBox[{"1", "-", "\[Beta]"}], ")"}]}]}], ")"}], 
                RowBox[{"Log", "[", 
                  FractionBox["a", "b"], "]"}]}]}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"expr", "[", 
          RowBox[{"a", ",", "b", ",", "\[Beta]"}], "]"}], "-", 
        RowBox[{"tmp4", "[", 
          RowBox[{"a", ",", "b", ",", "\[Beta]"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"#", ",", "assumsU"}], "]"}], 
        "&"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"expr", "[", 
          RowBox[{"a", ",", "b", ",", "\[Beta]"}], "]"}], "-", 
        RowBox[{"tmp4", "[", 
          RowBox[{"a", ",", "b", ",", "\[Beta]"}], "]"}]}], "//", 
      RowBox[{
        RowBox[{"FullSimplify", "[", 
          RowBox[{"#", ",", "assumsE"}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData["0"], "Output"],

Cell[BoxData["0"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsection"],

Cell[CellGroupData[{

Cell["\<\
Comparison of analytical solutions for single integrals with \
numerical solutions\
\>", "Subsubsection"],

Cell[TextData[{
  "Function to create a table of values ",
  Cell[BoxData[
      FormBox[
        RowBox[{"f", "(", 
          RowBox[{
            SubscriptBox["p", "0"], ",", 
            SubscriptBox["p", "1"]}], ")"}], TraditionalForm]]],
  " for ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["p", "0"], ",", 
          RowBox[{
            SubscriptBox["p", "1"], "\[Element]", 
            RowBox[{"(", 
              RowBox[{"0", ",", "1"}], ")"}]}]}], TraditionalForm]]],
  ":"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"abTable", "[", "f_", "]"}], ":=", 
      RowBox[{"Rest", "[", 
        RowBox[{"Flatten", "[", 
          RowBox[{
            RowBox[{"Table", "[", 
              RowBox[{
                RowBox[{"f", "[", 
                  RowBox[{"a", ",", "b"}], "]"}], ",", 
                RowBox[{"{", 
                  RowBox[{"a", ",", "0.1", ",", "1", ",", "0.1"}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{"b", ",", "0.1", ",", "1", ",", "0.1"}], "}"}]}], 
              "]"}], ",", "1"}], "]"}], "]"}]}]], "Input"],

Cell[TextData[{
  "Test of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", 
            RowBox[{"\[Pi]", "/", "2"}]], 
          RowBox[{"d", " ", "\[Theta]", " ", 
            FractionBox[
              RowBox[{"sin", " ", "2", "\[Theta]", " ", 
                SuperscriptBox["cos", "4"], "\[Theta]"}], 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", "0", "2"], 
                  SuperscriptBox["cos", "2"], "\[Theta]"}], "+", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], 
                  SuperscriptBox["sin", "2"], "\[Theta]"}]}]]}]}], 
        TraditionalForm]]],
  ":"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"nint1", "[", 
        RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
      RowBox[{"NIntegrate", "[", 
        RowBox[{
          RowBox[{"integrand1", "[", 
            RowBox[{"a", ",", "b"}], "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"\[Theta]", ",", "0", ",", 
              RowBox[{"\[Pi]", "/", "2"}]}], "}"}]}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{"Numerical", " ", "integration"}], "*)"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(nint1\\)\\\" is similar to existing symbol \\\"\\!\\(int1\\)\\\".\"\
\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"absDiff1", "[", 
        RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
      RowBox[{"Abs", "[", 
        RowBox[{
          RowBox[{"int1", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "-", 
          RowBox[{"nint1", "[", 
            RowBox[{"a", ",", "b"}], "]"}]}], "]"}]}], "\n", 
    RowBox[{
      RowBox[{"absDiff1s", "=", 
        RowBox[{"abTable", "[", "absDiff1", "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Max", "[", "absDiff1s", "]"}], "//", 
      RowBox[{
        RowBox[{"Chop", "[", 
          RowBox[{"#", ",", 
            SuperscriptBox["10", 
              RowBox[{"-", "9"}]]}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(absDiff1s\\)\\\" is similar to existing symbol \
\\\"\\!\\(absDiff1\\)\\\".\"\>"}]], "Message"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[TextData[{
  "Test of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", 
            RowBox[{"\[Pi]", "/", "2"}]], 
          RowBox[{"d", " ", "\[Theta]", " ", 
            FractionBox[
              RowBox[{"sin", " ", "2", "\[Theta]", " ", 
                SuperscriptBox["cos", "2"], "\[Theta]", " ", 
                SuperscriptBox["sin", "2"], "\[Theta]"}], 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", "0", "2"], 
                  SuperscriptBox["cos", "2"], "\[Theta]"}], "+", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], 
                  SuperscriptBox["sin", "2"], "\[Theta]"}]}]]}]}], 
        TraditionalForm]]],
  ":"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"nint2", "[", 
        RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
      RowBox[{"NIntegrate", "[", 
        RowBox[{
          RowBox[{"integrand2", "[", 
            RowBox[{"a", ",", "b"}], "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"\[Theta]", ",", "0", ",", 
              RowBox[{"\[Pi]", "/", "2"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(nint2\\)\\\" is similar to existing symbol \\\"\\!\\(int2\\)\\\".\"\
\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"absDiff2", "[", 
        RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
      RowBox[{"Abs", "[", 
        RowBox[{
          RowBox[{"int2", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "-", 
          RowBox[{"nint2", "[", 
            RowBox[{"a", ",", "b"}], "]"}]}], "]"}]}], "\n", 
    RowBox[{
      RowBox[{"absDiff2s", "=", 
        RowBox[{"abTable", "[", "absDiff2", "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Max", "[", "absDiff2s", "]"}], "//", 
      RowBox[{
        RowBox[{"Chop", "[", 
          RowBox[{"#", ",", 
            SuperscriptBox["10", 
              RowBox[{"-", "9"}]]}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(absDiff2s\\)\\\" is similar to existing symbol \
\\\"\\!\\(absDiff2\\)\\\".\"\>"}]], "Message"],

Cell[BoxData["0"], "Output"]
}, Open  ]],

Cell[TextData[{
  "Test of ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", 
            RowBox[{"\[Pi]", "/", "2"}]], 
          RowBox[{"d", " ", "\[Theta]", " ", 
            FractionBox[
              RowBox[{"sin", " ", "2", "\[Theta]", " ", 
                SuperscriptBox["cos", "2"], "\[Theta]", " ", 
                SuperscriptBox["sin", "2"], "\[Theta]"}], 
              RowBox[{
                RowBox[{
                  SubsuperscriptBox["p", "0", "2"], 
                  SuperscriptBox["cos", "2"], "\[Theta]"}], "+", 
                RowBox[{
                  SubsuperscriptBox["p", "1", "2"], 
                  SuperscriptBox["sin", "2"], "\[Theta]"}]}]]}]}], 
        TraditionalForm]]],
  ":"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"nint3", "[", 
        RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
      RowBox[{"NIntegrate", "[", 
        RowBox[{
          RowBox[{"integrand3", "[", 
            RowBox[{"a", ",", "b"}], "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"\[Theta]", ",", "0", ",", 
              RowBox[{"\[Pi]", "/", "2"}]}], "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(nint3\\)\\\" is similar to existing symbol \\\"\\!\\(int3\\)\\\".\"\
\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"absDiff3", "[", 
        RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
      RowBox[{"Abs", "[", 
        RowBox[{
          RowBox[{"int3", "[", 
            RowBox[{"a", ",", "b"}], "]"}], "-", 
          RowBox[{"nint3", "[", 
            RowBox[{"a", ",", "b"}], "]"}]}], "]"}]}], "\n", 
    RowBox[{
      RowBox[{"absDiff3s", "=", 
        RowBox[{"abTable", "[", "absDiff3", "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Max", "[", "absDiff3s", "]"}], "//", 
      RowBox[{
        RowBox[{"Chop", "[", 
          RowBox[{"#", ",", 
            SuperscriptBox["10", 
              RowBox[{"-", "9"}]]}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(absDiff3s\\)\\\" is similar to existing symbol \
\\\"\\!\\(absDiff3\\)\\\".\"\>"}]], "Message"],

Cell[BoxData["0"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Comparison of the analytical and numerical solution of the weighted sum of \
integrals ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            SubsuperscriptBox["p", "0", "4"], 
            SuperscriptBox["\[Beta]", "2"], 
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", 
                RowBox[{"\[Pi]", "/", "2"}]], 
              RowBox[{
              "d", " ", "\[Theta]", "sin", " ", "2", "\[Theta]", " ", 
                SuperscriptBox["cos", "4"], 
                FractionBox["\[Theta]", 
                  RowBox[{
                    RowBox[{
                      SubsuperscriptBox["p", "0", "2"], 
                      SuperscriptBox["cos", "2"], "\[Theta]"}], "+", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], 
                      SuperscriptBox["sin", "2"], "\[Theta]"}]}]]}]}]}], "+", 
          
          RowBox[{
            SuperscriptBox[
              RowBox[{
                SubsuperscriptBox["p", "1", "4"], "(", 
                RowBox[{"1", "-", "\[Beta]"}], ")"}], "2"], 
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", 
                RowBox[{"\[Pi]", "/", "2"}]], 
              RowBox[{"d", " ", "\[Theta]", " ", 
                FractionBox[
                  RowBox[{
                    RowBox[{"sin", " ", "2", "\[Theta]"}], "+", 
                    RowBox[{
                      SuperscriptBox["sin", "4"], "\[Theta]"}]}], 
                  RowBox[{
                    RowBox[{
                      SubsuperscriptBox["p", "0", "2"], 
                      SuperscriptBox["cos", "2"], "\[Theta]"}], "+", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], 
                      SuperscriptBox["sin", "2"], "\[Theta]"}]}]]}]}]}], "+", 
          
          RowBox[{"4", 
            SubsuperscriptBox["p", "0", "2"], 
            SubsuperscriptBox["p", "1", "2"], 
            RowBox[{"\[Beta]", "(", 
              RowBox[{"1", "-", "\[Beta]"}], ")"}], 
            RowBox[{
              SubsuperscriptBox["\[Integral]", "0", 
                RowBox[{"\[Pi]", "/", "2"}]], 
              RowBox[{"d", " ", "\[Theta]", " ", 
                FractionBox[
                  RowBox[{"sin", " ", "2", "\[Theta]", " ", 
                    SuperscriptBox["cos", "2"], "\[Theta]", " ", 
                    SuperscriptBox["sin", "2"], "\[Theta]"}], 
                  RowBox[{
                    RowBox[{
                      SubsuperscriptBox["p", "0", "2"], 
                      SuperscriptBox["cos", "2"], "\[Theta]"}], "+", 
                    RowBox[{
                      SubsuperscriptBox["p", "1", "2"], 
                      SuperscriptBox["sin", "2"], "\[Theta]"}]}]]}]}]}]}], 
        TraditionalForm]]],
  ":"
}], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"nexpr", "[", 
          RowBox[{"a_", ",", "b_", ",", "\[Beta]_"}], "]"}], ":=", 
        RowBox[{
          RowBox[{
            SuperscriptBox["a", "4"], 
            SuperscriptBox["\[Beta]", "2"], 
            RowBox[{"nint1", "[", 
              RowBox[{"a", ",", "b"}], "]"}]}], "+", 
          RowBox[{
            SuperscriptBox["b", "4"], 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"1", "-", "\[Beta]"}], ")"}], "2"], 
            RowBox[{"nint2", "[", 
              RowBox[{"a", ",", "b"}], "]"}]}], "+", 
          RowBox[{"4", 
            SuperscriptBox["a", "2"], 
            SuperscriptBox["b", "2"], "\[Beta]", 
            RowBox[{"(", 
              RowBox[{"1", "-", "\[Beta]"}], ")"}], 
            RowBox[{"nint3", "[", 
              RowBox[{"a", ",", "b"}], "]"}]}]}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(nexpr\\)\\\" is similar to existing symbol \\\"\\!\\(expr\\)\\\".\"\
\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"absDiff", "[", 
        RowBox[{"a_", ",", "b_", ",", "\[Beta]_"}], "]"}], ":=", 
      RowBox[{"Abs", "[", 
        RowBox[{
          RowBox[{"expr", "[", 
            RowBox[{"a", ",", "b", ",", "\[Beta]"}], "]"}], "-", 
          RowBox[{"nexpr", "[", 
            RowBox[{"a", ",", "b", ",", "\[Beta]"}], "]"}]}], "]"}]}], "\n", 
    RowBox[{
      RowBox[{"absDiffs", "=", 
        RowBox[{"Table", "[", 
          RowBox[{
            RowBox[{"absDiff", "[", 
              RowBox[{"a", ",", "b", ",", "\[Beta]"}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"a", ",", "0.1", ",", "1", ",", "0.1"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"b", ",", "0.1", ",", "1", ",", "0.1"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Beta]", ",", "0", ",", "1", ",", "0.1"}], "}"}]}], 
          "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Max", "[", "absDiffs", "]"}], "//", 
      RowBox[{
        RowBox[{"Chop", "[", 
          RowBox[{"#", ",", 
            SuperscriptBox["10", 
              RowBox[{"-", "9"}]]}], "]"}], "&"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(absDiffs\\)\\\" is similar to existing symbols \\!\\({absDiff, \
absDiff1s, absDiff2s, absDiff3s}\\).\"\>"}]], "Message"],

Cell[BoxData["0"], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
Evaluator->"psi",
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 26, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[1790, 55, 875, 22, 59, "Input"],
Cell[2668, 79, 261, 6, 23, "Message"],
Cell[2932, 87, 264, 6, 23, "Message"]
}, Open  ]],
Cell[3211, 96, 108, 3, 32, "Text"],
Cell[3322, 101, 712, 18, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4071, 124, 27, 0, 45, "Subsection"],
Cell[4101, 126, 1569, 42, 83, "Input"],

Cell[CellGroupData[{
Cell[5695, 172, 1742, 46, 81, "Input"],
Cell[7440, 220, 28, 0, 27, "Output"],
Cell[7471, 222, 28, 0, 27, "Output"]
}, Open  ]],
Cell[7514, 225, 1570, 42, 83, "Input"],

Cell[CellGroupData[{
Cell[9109, 271, 1666, 45, 88, "Input"],
Cell[10778, 318, 28, 0, 27, "Output"],
Cell[10809, 320, 28, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10874, 325, 1658, 44, 83, "Input"],
Cell[12535, 371, 258, 6, 23, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12830, 382, 1511, 41, 88, "Input"],
Cell[14344, 425, 28, 0, 27, "Output"],
Cell[14375, 427, 28, 0, 27, "Output"]
}, Open  ]],
Cell[14418, 430, 905, 24, 29, "Input"],

Cell[CellGroupData[{
Cell[15348, 458, 3317, 80, 102, "Input"],
Cell[18668, 540, 28, 0, 27, "Output"],
Cell[18699, 542, 28, 0, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[18776, 548, 29, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[18830, 552, 115, 3, 42, "Subsubsection"],
Cell[18948, 557, 525, 18, 32, "Text"],
Cell[19476, 577, 598, 15, 27, "Input"],
Cell[20077, 594, 715, 20, 41, "Text"],

Cell[CellGroupData[{
Cell[20817, 618, 474, 12, 27, "Input"],
Cell[21294, 632, 257, 6, 23, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21588, 643, 704, 20, 61, "Input"],
Cell[22295, 665, 265, 6, 23, "Message"],
Cell[22563, 673, 28, 0, 27, "Output"]
}, Open  ]],
Cell[22606, 676, 777, 21, 41, "Text"],

Cell[CellGroupData[{
Cell[23408, 701, 388, 10, 27, "Input"],
Cell[23799, 713, 257, 6, 23, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24093, 724, 704, 20, 61, "Input"],
Cell[24800, 746, 265, 6, 23, "Message"],
Cell[25068, 754, 28, 0, 27, "Output"]
}, Open  ]],
Cell[25111, 757, 777, 21, 41, "Text"],

Cell[CellGroupData[{
Cell[25913, 782, 388, 10, 27, "Input"],
Cell[26304, 794, 257, 6, 23, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26598, 805, 704, 20, 61, "Input"],
Cell[27305, 827, 265, 6, 23, "Message"],
Cell[27573, 835, 28, 0, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[27650, 841, 2859, 68, 80, "Subsubsection"],

Cell[CellGroupData[{
Cell[30534, 913, 909, 24, 29, "Input"],
Cell[31446, 939, 257, 6, 23, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31740, 950, 1171, 29, 61, "Input"],
Cell[32914, 981, 290, 6, 39, "Message"],
Cell[33207, 989, 28, 0, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

