(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     68929,       2606]*)
(*NotebookOutlinePosition[     70019,       2639]*)
(*  CellTagsIndexPosition[     69975,       2635]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Example 1", "Subsubsection"],

Cell["Some fundamental definitions:", "Text"],

Cell[BoxData[{
    \(\(adj[\[Psi]_] := 
        Transpose[Conjugate[\[Psi]]];\)\), "\[IndentingNewLine]", 
    \(\(innerProduct[\[Psi]_, \[Xi]_] := \((adj[\[Psi]] . \[Xi])\)\
\[LeftDoubleBracket]1, 1\[RightDoubleBracket];\)\)}], "Input"],

Cell[TextData[{
  "We let ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(\(\[Psi]\)\(>\)\)\)\  = \(e\^\(i\ \
\[Chi]\)\)(
          cos\ \[CurlyTheta]\/2 | 
            0 > \(+e\^\(i\ \[CurlyPhi]\)\) 
                sin\ \[CurlyTheta]\/2 | \(\(1\)\(>\)\)), \[CurlyTheta] = \
\(3\/4\) \[Pi], \[CurlyPhi] = \(9\/8\) \[Pi], \[Chi] = \(3\/2\) \[Pi]\)]],
  " \nand ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(\(\[Xi]\)\(>\)\)\)\  = \(e\^\(i\ \[Chi]\^\
\[Prime]\)\)(
          cos\ \[CurlyTheta]\^\[Prime]\/2 | 
            0 > \(+e\^\(i\ \[CurlyPhi]\^\[Prime]\)\) 
                sin\ \[CurlyTheta]\^\[Prime]\/2 | \(\(1\)\(>\)\)), \
\[CurlyTheta]\^\[Prime] = \(1\/4\) \[Pi], \[CurlyPhi]\^\[Prime] = \(1\/8\) \
\[Pi], \[Chi]\^\[Prime] = \(1\/5\) \[Pi]\)]],
  ":"
}], "Text"],

Cell[BoxData[{\(\[CurlyTheta] = \(3\/4\) \[Pi]; \[CurlyPhi] = \(9\/8\) \[Pi]; \
\[Chi] = \(3\/2\) \[Pi];\), "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"\[Psi]", "=", 
        RowBox[{\(Exp[I\ \[Chi]]\), 
          RowBox[{"(", 
            RowBox[{
              RowBox[{\(Cos[\[CurlyTheta]\/2]\), 
                RowBox[{"(", GridBox[{
                      {"1"},
                      {"0"}
                      }], ")"}]}], "+", 
              RowBox[{\(Exp[I\ \[CurlyPhi]]\), \(Sin[\[CurlyTheta]\/2]\), 
                RowBox[{"(", GridBox[{
                      {"0"},
                      {"1"}
                      }], ")"}]}]}], ")"}], Cell[""]}]}], 
      ";"}], "\[IndentingNewLine]", \(\[CurlyTheta]p = \(1\/4\) \[Pi]; \
\[CurlyPhi]p = \(1\/8\) \[Pi]; \[Chi]p = \(1\/5\) \[Pi];\), "\
\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"\[Xi]", "=", 
        RowBox[{\(Exp[I\ \[Chi]p]\), 
          RowBox[{"(", 
            RowBox[{
              RowBox[{\(Cos[\[CurlyTheta]p\/2]\), 
                RowBox[{"(", GridBox[{
                      {"1"},
                      {"0"}
                      }], ")"}]}], "+", 
              RowBox[{\(Exp[I\ \[CurlyPhi]p]\), \(Sin[\[CurlyTheta]p\/2]\), 
                RowBox[{"(", GridBox[{
                      {"0"},
                      {"1"}
                      }], ")"}]}]}], ")"}]}]}], ";"}]}], "Input"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(\(\[Psi]\)\(>\)\)\)\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(|\)\(\(\[Xi]\)\(>\)\)\)\)]],
  " are orthogonal:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(innerProduct[\[Psi], \[Xi]] // Simplify\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`P\_\[Psi]\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`P\_\[Xi]\)]],
  " are diametrically opposing points on the Bloch sphere:"
}], "Text"],

Cell[BoxData[{
    \(\(P\[Psi] = {Sin[\[CurlyTheta]] Cos[\[CurlyPhi]], 
          Sin[\[CurlyTheta]] Sin[\[CurlyPhi]], 
          Cos[\[CurlyTheta]]};\)\), "\[IndentingNewLine]", 
    \(\(P\[Xi] = {Sin[\[CurlyTheta]p] Cos[\[CurlyPhi]p], 
          Sin[\[CurlyTheta]p] Sin[\[CurlyPhi]p], 
          Cos[\[CurlyTheta]p]};\)\)}], "Input"],

Cell[BoxData[
    \(<< Graphics`Shapes`\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\($TextStyle = {FontSize \[Rule] 12, 
          FontWeight \[Rule] "\<Bold\>"};\)\), "\[IndentingNewLine]", 
    \(\(axes = {Thickness[0.007], Line[{{0, 0, 0}, {1.2, 0, 0}}], 
          Text["\<x\>", {1.3, 0, 0}], Line[{{0, 0, 0}, {0, 1.2, 0}}], 
          Text["\<y\>", {0, 1.3, 0}], Line[{{0, 0, 0}, {0, 0, 1.2}}], 
          Text["\<z\>", {0, 0, 1.3}]};\)\), "\[IndentingNewLine]", 
    \(\(blochSphere = {Sphere[]};\)\), "\[IndentingNewLine]", 
    \(\(states = {Thickness[0.01], PointSize[0.05], 
          Line[{{0, 0, 0}, P\[Psi]}], Point[P\[Psi]], 
          Text["\<|\[Psi]>\>", 1.7  P\[Psi], Background \[Rule] Automatic], 
          Line[{{0, 0, 0}, P\[Xi]}], Point[P\[Xi]], 
          Text["\<|\[Xi]>\>", 1.3  P\[Xi], 
            Background \[Rule] Automatic]};\)\), "\[IndentingNewLine]", 
    \(\(Graphics3D[{axes, blochSphere, states}, 
        Boxed \[Rule] False];\)\), "\[IndentingNewLine]", 
    \(\(Show[WireFrame[%], ViewPoint -> {1, 1, 0.3}];\)\)}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.21617 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.197819 1.40759 -1.84314e-18 1.40759 [
[ 0 0 0 0 ]
[ 1 1.21617 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.21617 L
0 1.21617 L
closepath
clip
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.40806 .67521 m
.46738 .67961 L
s
.46738 .67961 m
.466 .7221 L
s
.46978 .63751 m
.46738 .67961 L
s
.46738 .67961 m
.52789 .67961 L
s
.52789 .67961 m
.52927 .7221 L
s
.52535 .63751 m
.52789 .67961 L
s
.52789 .67961 m
.5869 .67521 L
s
.40388 .71832 m
.466 .7221 L
s
.466 .7221 m
.46564 .76421 L
s
.466 .7221 m
.52927 .7221 L
s
.52927 .7221 m
.52948 .76421 L
s
.52927 .7221 m
.59106 .71832 L
s
.41542 .63276 m
.46978 .63751 L
s
.47316 .59659 m
.46978 .63751 L
s
.46978 .63751 m
.52535 .63751 L
s
.52167 .59659 m
.52535 .63751 L
s
.52535 .63751 m
.57942 .63276 L
s
.40296 .76122 m
.46564 .76421 L
s
.46564 .76421 m
.46636 .80516 L
s
.46564 .76421 m
.52948 .76421 L
s
.52948 .76421 m
.52847 .80516 L
s
.52948 .76421 m
.59183 .76122 L
s
.35283 .66638 m
.40806 .67521 L
s
.40806 .67521 m
.40388 .71832 L
s
.41542 .63276 m
.40806 .67521 L
s
.5869 .67521 m
.59106 .71832 L
s
.57942 .63276 m
.5869 .67521 L
s
.5869 .67521 m
.64151 .66638 L
s
.34582 .7107 m
.40388 .71832 L
s
.40388 .71832 m
.40296 .76122 L
s
.59106 .71832 m
.59183 .76122 L
s
.59106 .71832 m
.64848 .7107 L
s
.36516 .62328 m
.41542 .63276 L
s
.42586 .59185 m
.41542 .63276 L
s
.56872 .59185 m
.57942 .63276 L
s
.57942 .63276 m
.62911 .62328 L
s
.42586 .59185 m
.47316 .59659 L
s
.47752 .55774 m
.47316 .59659 L
s
.47316 .59659 m
.52167 .59659 L
s
.51687 .55774 m
.52167 .59659 L
s
.52167 .59659 m
.56872 .59185 L
s
.40542 .80303 m
.46636 .80516 L
s
.46636 .80516 m
.46818 .84406 L
s
.46636 .80516 m
.52847 .80516 L
s
.52847 .80516 m
.5262 .84406 L
s
.52847 .80516 m
.58908 .80303 L
s
.34432 .75519 m
.40296 .76122 L
s
.40296 .76122 m
.40542 .80303 L
s
.59183 .76122 m
.58908 .80303 L
s
.59183 .76122 m
.64981 .75519 L
s
.55488 .55344 m
.56872 .59185 L
s
.56872 .59185 m
.61151 .58244 L
s
.38257 .58244 m
.42586 .59185 L
s
.4393 .55344 m
.42586 .59185 L
s
.05 w
.55459 .58358 Mdot
.01 w
.49557 .70086 m
.55459 .58358 L
s
.5 Mabswid
.34854 .79873 m
.40542 .80303 L
s
.40542 .80303 m
.41136 .84275 L
s
.58908 .80303 m
.58271 .84275 L
s
.58908 .80303 m
.64531 .79873 L
s
.4393 .55344 m
.47752 .55774 L
s
.48283 .52201 m
.47752 .55774 L
s
.47752 .55774 m
.51687 .55774 L
s
.51098 .52201 m
.51687 .55774 L
s
.51687 .55774 m
.55488 .55344 L
s
.41136 .84275 m
.46818 .84406 L
s
.46818 .84406 m
.47116 .87984 L
s
.46818 .84406 m
.5262 .84406 L
s
.5262 .84406 m
.52263 .87984 L
s
.5262 .84406 m
.58271 .84275 L
s
.64151 .66638 m
.64848 .7107 L
s
.62911 .62328 m
.64151 .66638 L
s
.64151 .66638 m
.68817 .65312 L
s
.30524 .65312 m
.35283 .66638 L
s
.35283 .66638 m
.34582 .7107 L
s
.36516 .62328 m
.35283 .66638 L
s
.2954 .69922 m
.34582 .7107 L
s
.34582 .7107 m
.34432 .75519 L
s
.64848 .7107 m
.64981 .75519 L
s
.64848 .7107 m
.69792 .69922 L
s
.32244 .60914 m
.36516 .62328 L
s
.38257 .58244 m
.36516 .62328 L
s
.61151 .58244 m
.62911 .62328 L
s
.62911 .62328 m
.67099 .60914 L
s
.40479 .54498 m
.4393 .55344 L
s
.45563 .51864 m
.4393 .55344 L
s
.53802 .51864 m
.55488 .55344 L
s
.55488 .55344 m
.58898 .54498 L
s
.64981 .75519 m
.64531 .79873 L
s
.64981 .75519 m
.69981 .74608 L
s
.29332 .74608 m
.34432 .75519 L
s
.34432 .75519 m
.34854 .79873 L
s
.34647 .56857 m
.38257 .58244 L
s
.40479 .54498 m
.38257 .58244 L
s
.58898 .54498 m
.61151 .58244 L
s
.61151 .58244 m
.64687 .56857 L
s
.35863 .84014 m
.41136 .84275 L
s
.41136 .84275 m
.4209 .87924 L
s
.58271 .84275 m
.5726 .87924 L
s
.58271 .84275 m
.63482 .84014 L
s
.2993 .79227 m
.34854 .79873 L
s
.34854 .79873 m
.35863 .84014 L
s
.64531 .79873 m
.63482 .84014 L
s
.64531 .79873 m
.69356 .79227 L
s
.45563 .51864 m
.48283 .52201 L
s
.48904 .49067 m
.48283 .52201 L
s
.48283 .52201 m
.51098 .52201 L
s
.50402 .49067 m
.51098 .52201 L
s
.51098 .52201 m
.53802 .51864 L
s
.4209 .87924 m
.47116 .87984 L
s
.47116 .87984 m
.47534 .91123 L
s
.47116 .87984 m
.52263 .87984 L
s
.52263 .87984 m
.51771 .91123 L
s
.52263 .87984 m
.5726 .87924 L
s
.37671 .53267 m
.40479 .54498 L
s
.43146 .51208 m
.40479 .54498 L
s
.56188 .51208 m
.58898 .54498 L
s
.58898 .54498 m
.61646 .53267 L
s
.43146 .51208 m
.45563 .51864 L
s
.47466 .48874 m
.45563 .51864 L
s
.51832 .48874 m
.53802 .51864 L
s
.53802 .51864 m
.56188 .51208 L
s
.3747 .87806 m
.4209 .87924 L
s
.4209 .87924 m
.43413 .91115 L
s
.5726 .87924 m
.55868 .91115 L
s
.5726 .87924 m
.61823 .87806 L
s
.63482 .84014 m
.61823 .87806 L
s
.63482 .84014 m
.67902 .83623 L
s
.31349 .83623 m
.35863 .84014 L
s
.35863 .84014 m
.3747 .87806 L
s
.67099 .60914 m
.68817 .65312 L
s
.64687 .56857 m
.67099 .60914 L
s
.67099 .60914 m
.70089 .59068 L
s
.29142 .59068 m
.32244 .60914 L
s
.32244 .60914 m
.30524 .65312 L
s
.34647 .56857 m
.32244 .60914 L
s
.26983 .63563 m
.30524 .65312 L
s
.30524 .65312 m
.2954 .69922 L
s
.68817 .65312 m
.69792 .69922 L
s
.68817 .65312 m
.72236 .63563 L
s
.41239 .50268 m
.43146 .51208 L
s
.46212 .48503 m
.43146 .51208 L
s
.5307 .48503 m
.56188 .51208 L
s
.56188 .51208 m
.58053 .50268 L
s
.32118 .55072 m
.34647 .56857 L
s
.37671 .53267 m
.34647 .56857 L
s
.61646 .53267 m
.64687 .56857 L
s
.64687 .56857 m
.6712 .55072 L
s
.25736 .68398 m
.2954 .69922 L
s
.2954 .69922 m
.29332 .74608 L
s
.69792 .69922 m
.69981 .74608 L
s
.69792 .69922 m
.73466 .68398 L
s
.35795 .51712 m
.37671 .53267 L
s
.41239 .50268 m
.37671 .53267 L
s
.58053 .50268 m
.61646 .53267 L
s
.61646 .53267 m
.63446 .51712 L
s
.25474 .73398 m
.29332 .74608 L
s
.29332 .74608 m
.2993 .79227 L
s
.69981 .74608 m
.69356 .79227 L
s
.69981 .74608 m
.73707 .73398 L
s
.48904 .49067 m
.47466 .48874 L
s
.50402 .49067 m
.48904 .49067 L
s
.51832 .48874 m
.50402 .49067 L
s
.43413 .91115 m
.47534 .91123 L
s
.47534 .91123 m
.48073 .9367 L
s
.47534 .91123 m
.51771 .91123 L
s
.51771 .91123 m
.51143 .9367 L
s
.51771 .91123 m
.55868 .91115 L
s
.47466 .48874 m
.46212 .48503 L
s
.5307 .48503 m
.51832 .48874 L
s
.33585 .8763 m
.3747 .87806 L
s
.3747 .87806 m
.39674 .91098 L
s
.61823 .87806 m
.59559 .91098 L
s
.61823 .87806 m
.65625 .8763 L
s
.40039 .49106 m
.41239 .50268 L
s
.45256 .47982 m
.41239 .50268 L
s
.54002 .47982 m
.58053 .50268 L
s
.58053 .50268 m
.592 .49106 L
s
.26237 .7837 m
.2993 .79227 L
s
.2993 .79227 m
.31349 .83623 L
s
.69356 .79227 m
.67902 .83623 L
s
.69356 .79227 m
.7292 .7837 L
s
.46212 .48503 m
.45256 .47982 L
s
.54002 .47982 m
.5307 .48503 L
s
.39674 .91098 m
.43413 .91115 L
s
.43413 .91115 m
.45103 .93687 L
s
.55868 .91115 m
.54095 .93687 L
s
.55868 .91115 m
.59559 .91098 L
s
.45256 .47982 m
.44698 .47353 L
s
.54532 .47353 m
.54002 .47982 L
s
.28031 .83109 m
.31349 .83623 L
s
.31349 .83623 m
.33585 .8763 L
s
.67902 .83623 m
.65625 .8763 L
s
.67902 .83623 m
.71098 .83109 L
s
.35139 .49935 m
.35795 .51712 L
s
.35795 .51712 m
.32118 .55072 L
s
.40039 .49106 m
.35795 .51712 L
s
.63446 .51712 m
.6712 .55072 L
s
.592 .49106 m
.63446 .51712 L
s
.63446 .51712 m
.64015 .49935 L
s
.31069 .52986 m
.32118 .55072 L
s
.32118 .55072 m
.29142 .59068 L
s
.6712 .55072 m
.70089 .59068 L
s
.6712 .55072 m
.68059 .52986 L
s
.39719 .47813 m
.40039 .49106 L
s
.44698 .47353 m
.40039 .49106 L
s
.54532 .47353 m
.592 .49106 L
s
.592 .49106 m
.5946 .47813 L
s
.59559 .91098 m
.56709 .93721 L
s
.59559 .91098 m
.62557 .91073 L
s
.36605 .91073 m
.39674 .91098 L
s
.39674 .91098 m
.42452 .93721 L
s
.27702 .56873 m
.29142 .59068 L
s
.29142 .59068 m
.26983 .63563 L
s
.70089 .59068 m
.72236 .63563 L
s
.70089 .59068 m
.71397 .56873 L
s
.44698 .47353 m
.44608 .46672 L
s
.54593 .46672 m
.54532 .47353 L
s
.25219 .61455 m
.26983 .63563 L
s
.26983 .63563 m
.25736 .68398 L
s
.72236 .63563 m
.73466 .68398 L
s
.72236 .63563 m
.73852 .61455 L
s
.45103 .93687 m
.48073 .9367 L
s
.48073 .9367 m
.48732 .95443 L
s
.48073 .9367 m
.51143 .9367 L
s
.51143 .9367 m
.50382 .95443 L
s
.51143 .9367 m
.54095 .93687 L
s
.65625 .8763 m
.62557 .91073 L
s
.65625 .8763 m
.68281 .87403 L
s
.30821 .87403 m
.33585 .8763 L
s
.33585 .8763 m
.36605 .91073 L
s
.2377 .66549 m
.25736 .68398 L
s
.25736 .68398 m
.25474 .73398 L
s
.73466 .68398 m
.73707 .73398 L
s
.73466 .68398 m
.75274 .66549 L
s
.44608 .46672 m
.45015 .46004 L
s
.54156 .46004 m
.54593 .46672 L
s
.44608 .46672 m
.39719 .47813 L
s
.54593 .46672 m
.5946 .47813 L
s
.42452 .93721 m
.45103 .93687 L
s
.45103 .93687 m
.47145 .95461 L
s
.54095 .93687 m
.51958 .95461 L
s
.54095 .93687 m
.56709 .93721 L
s
.73707 .73398 m
.7292 .7837 L
s
.73707 .73398 m
.75555 .71928 L
s
.23464 .71928 m
.25474 .73398 L
s
.25474 .73398 m
.26237 .7837 L
s
.40391 .46507 m
.39719 .47813 L
s
.39719 .47813 m
.35139 .49935 L
s
.5946 .47813 m
.64015 .49935 L
s
.5946 .47813 m
.58728 .46507 L
s
.45015 .46004 m
.45899 .45421 L
s
.53246 .45421 m
.54156 .46004 L
s
.45015 .46004 m
.40391 .46507 L
s
.54156 .46004 m
.58728 .46507 L
s
.34522 .91042 m
.36605 .91073 L
s
.36605 .91073 m
.40345 .93771 L
s
.62557 .91073 m
.58765 .93771 L
s
.62557 .91073 m
.64551 .91042 L
s
.40345 .93771 m
.42452 .93721 L
s
.42452 .93721 m
.45757 .95496 L
s
.56709 .93721 m
.53326 .95496 L
s
.56709 .93721 m
.58765 .93771 L
s
.24356 .77335 m
.26237 .7837 L
s
.26237 .7837 m
.28031 .83109 L
s
.7292 .7837 m
.71098 .83109 L
s
.7292 .7837 m
.74643 .77335 L
s
.35934 .48092 m
.35139 .49935 L
s
.35139 .49935 m
.31069 .52986 L
s
.64015 .49935 m
.68059 .52986 L
s
.64015 .49935 m
.63131 .48092 L
s
.51943 .44987 m
.53246 .45421 L
s
.45899 .45421 m
.47183 .44987 L
s
.42068 .45335 m
.45899 .45421 L
s
.53246 .45421 m
.56998 .45335 L
s
.71098 .83109 m
.68281 .87403 L
s
.71098 .83109 m
.7255 .82495 L
s
.26433 .82495 m
.28031 .83109 L
s
.28031 .83109 m
.30821 .87403 L
s
.31867 .50771 m
.31069 .52986 L
s
.31069 .52986 m
.27702 .56873 L
s
.68059 .52986 m
.71397 .56873 L
s
.68059 .52986 m
.67142 .50771 L
s
.50382 .44756 m
.51943 .44987 L
s
.47183 .44987 m
.48734 .44756 L
s
.51943 .44987 m
.54397 .44444 L
s
.47183 .44987 m
.44628 .44444 L
s
.42068 .45335 m
.40391 .46507 L
s
.40391 .46507 m
.35934 .48092 L
s
.58728 .46507 m
.63131 .48092 L
s
.58728 .46507 m
.56998 .45335 L
s
.48734 .44756 m
.50382 .44756 L
s
.50382 .44756 m
.51202 .43961 L
s
.48734 .44756 m
.47801 .43961 L
s
.01 w
.49557 .70086 m
.40751 .87586 L
s
.007 w
.49557 .70086 m
.49488 1.01819 L
s
.49557 .70086 m
.7647 .64367 L
s
.49557 .70086 m
.22096 .64367 L
s
.5 Mabswid
.50382 .95443 m
.51958 .95461 L
s
.48732 .95443 m
.50382 .95443 L
s
.47145 .95461 m
.48732 .95443 L
s
.39 .93832 m
.40345 .93771 L
s
.40345 .93771 m
.44696 .95545 L
s
.58765 .93771 m
.54359 .95545 L
s
.58765 .93771 m
.60046 .93832 L
s
.51958 .95461 m
.53326 .95496 L
s
.45757 .95496 m
.47145 .95461 L
s
.29614 .87135 m
.30821 .87403 L
s
.30821 .87403 m
.34522 .91042 L
s
.68281 .87403 m
.64551 .91042 L
s
.68281 .87403 m
.6936 .87135 L
s
.71397 .56873 m
.73852 .61455 L
s
.71397 .56873 m
.7053 .54493 L
s
.28426 .54493 m
.27702 .56873 L
s
.27702 .56873 m
.25219 .61455 L
s
.53326 .95496 m
.54359 .95545 L
s
.44696 .95545 m
.45757 .95496 L
s
.33744 .91006 m
.34522 .91042 L
s
.34522 .91042 m
.39 .93832 L
s
.64551 .91042 m
.60046 .93832 L
s
.64551 .91042 m
.65226 .91006 L
s
.25841 .59137 m
.25219 .61455 L
s
.25219 .61455 m
.2377 .66549 L
s
.73852 .61455 m
.75274 .66549 L
s
.73852 .61455 m
.73068 .59137 L
s
.38279 .46392 m
.42068 .45335 L
s
.56998 .45335 m
.60702 .46392 L
s
.56998 .45335 m
.54397 .44444 L
s
.44628 .44444 m
.42068 .45335 L
s
.38279 .46392 m
.35934 .48092 L
s
.35934 .48092 m
.31867 .50771 L
s
.63131 .48092 m
.67142 .50771 L
s
.63131 .48092 m
.60702 .46392 L
s
.54359 .95545 m
.54952 .95605 L
s
.44069 .95605 m
.44696 .95545 L
s
.38612 .939 m
.39 .93832 L
s
.39 .93832 m
.44069 .95605 L
s
.60046 .93832 m
.54952 .95605 L
s
.60046 .93832 m
.60362 .939 L
s
.75274 .66549 m
.75555 .71928 L
s
.75274 .66549 m
.74555 .64496 L
s
.24314 .64496 m
.2377 .66549 L
s
.2377 .66549 m
.23464 .71928 L
s
.54397 .44444 m
.56866 .45072 L
s
.54397 .44444 m
.51202 .43961 L
s
.47801 .43961 m
.44628 .44444 L
s
.44628 .44444 m
.42051 .45072 L
s
.43958 .9567 m
.44069 .95605 L
s
.54952 .95605 m
.55026 .9567 L
s
.2399 .70293 m
.23464 .71928 L
s
.23464 .71928 m
.24356 .77335 L
s
.75555 .71928 m
.74643 .77335 L
s
.75555 .71928 m
.74851 .70293 L
s
.67142 .50771 m
.7053 .54493 L
s
.67142 .50771 m
.64161 .48677 L
s
.34737 .48677 m
.31867 .50771 L
s
.31867 .50771 m
.28426 .54493 L
s
.51202 .43961 m
.52033 .44347 L
s
.51202 .43961 m
.47801 .43961 L
s
.47801 .43961 m
.46849 .44347 L
s
.24932 .76189 m
.24356 .77335 L
s
.24356 .77335 m
.26433 .82495 L
s
.74643 .77335 m
.7255 .82495 L
s
.74643 .77335 m
.73893 .76189 L
s
.444 .95734 m
.43958 .9567 L
s
.38612 .939 m
.43958 .9567 L
s
.55026 .9567 m
.54548 .95734 L
s
.60362 .939 m
.55026 .9567 L
s
.27108 .81824 m
.26433 .82495 L
s
.26433 .82495 m
.29614 .87135 L
s
.7255 .82495 m
.6936 .87135 L
s
.7255 .82495 m
.71715 .81824 L
s
.34737 .48677 m
.38279 .46392 L
s
.42051 .45072 m
.38279 .46392 L
s
.60702 .46392 m
.64161 .48677 L
s
.60702 .46392 m
.56866 .45072 L
s
.39318 .93969 m
.38612 .939 L
s
.33744 .91006 m
.38612 .939 L
s
.65226 .91006 m
.60362 .939 L
s
.60362 .939 m
.59583 .93969 L
s
.30386 .86849 m
.29614 .87135 L
s
.29614 .87135 m
.33744 .91006 L
s
.6936 .87135 m
.65226 .91006 L
s
.6936 .87135 m
.68451 .86849 L
s
.34547 .90969 m
.33744 .91006 L
s
.65226 .91006 m
.64316 .90969 L
s
.31674 .52198 m
.28426 .54493 L
s
.28426 .54493 m
.25841 .59137 L
s
.7053 .54493 m
.73068 .59137 L
s
.7053 .54493 m
.67145 .52198 L
s
.45374 .9579 m
.444 .95734 L
s
.54548 .95734 m
.53542 .9579 L
s
.39318 .93969 m
.444 .95734 L
s
.59583 .93969 m
.54548 .95734 L
s
.56866 .45072 m
.59214 .47016 L
s
.56866 .45072 m
.52033 .44347 L
s
.46849 .44347 m
.42051 .45072 L
s
.42051 .45072 m
.39595 .47016 L
s
.46797 .95831 m
.45374 .9579 L
s
.53542 .9579 m
.52095 .95831 L
s
.45374 .9579 m
.41146 .94032 L
s
.57689 .94032 m
.53542 .9579 L
s
.29332 .56866 m
.25841 .59137 L
s
.25841 .59137 m
.24314 .64496 L
s
.73068 .59137 m
.74555 .64496 L
s
.73068 .59137 m
.69418 .56866 L
s
.41146 .94032 m
.39318 .93969 L
s
.34547 .90969 m
.39318 .93969 L
s
.64316 .90969 m
.59583 .93969 L
s
.59583 .93969 m
.57689 .94032 L
s
.31674 .52198 m
.34737 .48677 L
s
.64161 .48677 m
.67145 .52198 L
s
.64161 .48677 m
.59214 .47016 L
s
.39595 .47016 m
.34737 .48677 L
s
.48522 .95854 m
.46797 .95831 L
s
.52095 .95831 m
.50358 .95854 L
s
.54816 .9408 m
.52095 .95831 L
s
.43969 .9408 m
.46797 .95831 L
s
.52033 .44347 m
.5283 .46091 L
s
.52033 .44347 m
.46849 .44347 L
s
.46849 .44347 m
.4593 .46091 L
s
.50358 .95854 m
.48522 .95854 L
s
.51269 .94106 m
.50358 .95854 L
s
.47489 .94106 m
.48522 .95854 L
s
.27931 .62468 m
.24314 .64496 L
s
.24314 .64496 m
.2399 .70293 L
s
.74555 .64496 m
.74851 .70293 L
s
.74555 .64496 m
.70766 .62468 L
s
.37065 .90934 m
.34547 .90969 L
s
.30386 .86849 m
.34547 .90969 L
s
.68451 .86849 m
.64316 .90969 L
s
.64316 .90969 m
.61699 .90934 L
s
.2763 .68674 m
.2399 .70293 L
s
.2399 .70293 m
.24932 .76189 L
s
.74851 .70293 m
.73893 .76189 L
s
.74851 .70293 m
.71033 .68674 L
s
.71715 .81824 m
.68451 .86849 L
s
.68451 .86849 m
.65293 .86576 L
s
.33413 .86576 m
.30386 .86849 L
s
.27108 .81824 m
.30386 .86849 L
s
.41146 .94032 m
.37065 .90934 L
s
.61699 .90934 m
.57689 .94032 L
s
.57689 .94032 m
.54816 .9408 L
s
.43969 .9408 m
.41146 .94032 L
s
.28495 .75062 m
.24932 .76189 L
s
.24932 .76189 m
.27108 .81824 L
s
.73893 .76189 m
.71715 .81824 L
s
.73893 .76189 m
.70159 .75062 L
s
.30476 .81172 m
.27108 .81824 L
s
.71715 .81824 m
.68193 .81172 L
s
.29332 .56866 m
.31674 .52198 L
s
.37434 .50345 m
.31674 .52198 L
s
.67145 .52198 m
.69418 .56866 L
s
.67145 .52198 m
.61273 .50345 L
s
.59214 .47016 m
.61273 .50345 L
s
.59214 .47016 m
.5283 .46091 L
s
.4593 .46091 m
.39595 .47016 L
s
.39595 .47016 m
.37434 .50345 L
s
.57504 .90906 m
.54816 .9408 L
s
.54816 .9408 m
.51269 .94106 L
s
.47489 .94106 m
.43969 .9408 L
s
.41181 .90906 m
.43969 .9408 L
s
.37065 .90934 m
.33413 .86576 L
s
.41181 .90906 m
.37065 .90934 L
s
.65293 .86576 m
.61699 .90934 L
s
.61699 .90934 m
.57504 .90906 L
s
.5283 .46091 m
.53533 .493 L
s
.5283 .46091 m
.4593 .46091 L
s
.4593 .46091 m
.45111 .493 L
s
.52182 .90891 m
.51269 .94106 L
s
.51269 .94106 m
.47489 .94106 L
s
.46461 .90891 m
.47489 .94106 L
s
.27931 .62468 m
.29332 .56866 L
s
.69418 .56866 m
.70766 .62468 L
s
.69418 .56866 m
.62863 .55007 L
s
.35756 .55007 m
.29332 .56866 L
s
.61273 .50345 m
.62863 .55007 L
s
.61273 .50345 m
.53533 .493 L
s
.45111 .493 m
.37434 .50345 L
s
.37434 .50345 m
.35756 .55007 L
s
.33413 .86576 m
.30476 .81172 L
s
.68193 .81172 m
.65293 .86576 L
s
.65293 .86576 m
.59963 .86358 L
s
.38638 .86358 m
.33413 .86576 L
s
.2763 .68674 m
.27931 .62468 L
s
.34741 .60795 m
.27931 .62468 L
s
.70766 .62468 m
.71033 .68674 L
s
.70766 .62468 m
.63813 .60795 L
s
.59963 .86358 m
.57504 .90906 L
s
.57504 .90906 m
.52182 .90891 L
s
.46461 .90891 m
.41181 .90906 L
s
.38638 .86358 m
.41181 .90906 L
s
.30476 .81172 m
.28495 .75062 L
s
.70159 .75062 m
.68193 .81172 L
s
.68193 .81172 m
.61983 .80642 L
s
.36559 .80642 m
.30476 .81172 L
s
.28495 .75062 m
.2763 .68674 L
s
.71033 .68674 m
.70159 .75062 L
s
.71033 .68674 m
.63998 .67337 L
s
.34518 .67337 m
.2763 .68674 L
s
.3514 .74135 m
.28495 .75062 L
s
.70159 .75062 m
.63373 .74135 L
s
.53533 .493 m
.54077 .53949 L
s
.53533 .493 m
.45111 .493 L
s
.45111 .493 m
.44468 .53949 L
s
.53029 .86236 m
.52182 .90891 L
s
.52182 .90891 m
.46461 .90891 L
s
.45515 .86236 m
.46461 .90891 L
s
.62863 .55007 m
.63813 .60795 L
s
.62863 .55007 m
.54077 .53949 L
s
.44468 .53949 m
.35756 .55007 L
s
.35756 .55007 m
.34741 .60795 L
s
.05 w
.40751 .87586 Mdot
.5 Mabswid
.45515 .86236 m
.38638 .86358 L
s
.36559 .80642 m
.38638 .86358 L
s
.61983 .80642 m
.59963 .86358 L
s
.59963 .86358 m
.53029 .86236 L
s
.63813 .60795 m
.63998 .67337 L
s
.63813 .60795 m
.544 .59836 L
s
.44072 .59836 m
.34741 .60795 L
s
.34741 .60795 m
.34518 .67337 L
s
.54077 .53949 m
.544 .59836 L
s
.54077 .53949 m
.44468 .53949 L
s
.44468 .53949 m
.44072 .59836 L
s
.53734 .80342 m
.53029 .86236 L
s
.53029 .86236 m
.45515 .86236 L
s
.44737 .80342 m
.45515 .86236 L
s
.63373 .74135 m
.61983 .80642 L
s
.61983 .80642 m
.53734 .80342 L
s
.44737 .80342 m
.36559 .80642 L
s
.3514 .74135 m
.36559 .80642 L
s
.63998 .67337 m
.63373 .74135 L
s
.63998 .67337 m
.54455 .66569 L
s
.43978 .66569 m
.34518 .67337 L
s
.34518 .67337 m
.3514 .74135 L
s
.63373 .74135 m
.54226 .73605 L
s
.44206 .73605 m
.3514 .74135 L
s
.544 .59836 m
.54455 .66569 L
s
.544 .59836 m
.44072 .59836 L
s
.44072 .59836 m
.43978 .66569 L
s
.54226 .73605 m
.53734 .80342 L
s
.53734 .80342 m
.44737 .80342 L
s
.44206 .73605 m
.44737 .80342 L
s
.54455 .66569 m
.54226 .73605 L
s
.54455 .66569 m
.43978 .66569 L
s
.43978 .66569 m
.44206 .73605 L
s
.54226 .73605 m
.44206 .73605 L
s
gsave
.1914 .63751 -66.5938 -9.59375 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 19.1875 translate 1 -1 scale
63.000 12.500 moveto
%%IncludeResource: font Courier-Bold
%%IncludeFont: Courier-Bold
%%BeginResource: font Courier-Bold-MISO
%%BeginFont: Courier-Bold-MISO
/Courier-Bold /Courier-Bold-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.500 moveto
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
70.188 12.500 moveto
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.79367 .63751 -66.5938 -9.59375 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 19.1875 translate 1 -1 scale
63.000 12.500 moveto
%%IncludeResource: font Courier-Bold
%%IncludeFont: Courier-Bold
%%BeginResource: font Courier-Bold-MISO
%%BeginFont: Courier-Bold-MISO
/Courier-Bold /Courier-Bold-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.500 moveto
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
70.188 12.500 moveto
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
gsave
.49482 1.04656 -66.5938 -9.59375 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 19.1875 translate 1 -1 scale
63.000 12.500 moveto
%%IncludeResource: font Courier-Bold
%%IncludeFont: Courier-Bold
%%BeginResource: font Courier-Bold-MISO
%%BeginFont: Courier-Bold-MISO
/Courier-Bold /Courier-Bold-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 12.500 moveto
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(z) show
70.188 12.500 moveto
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
1 g
.58552 .52211 -12.875 -5.84375 Mabsadd m
.58552 .52211 12.875 -5.84375 Mabsadd L
.58552 .52211 12.875 5.84375 Mabsadd L
.58552 .52211 -12.875 5.84375 Mabsadd L
fill
0 g
gsave
.58552 .52211 -73.875 -9.84375 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 19.6875 translate 1 -1 scale
63.000 12.500 moveto
%%IncludeResource: font Courier-Bold
%%IncludeFont: Courier-Bold
%%BeginResource: font Courier-Bold-MISO
%%BeginFont: Courier-Bold-MISO
/Courier-Bold /Courier-Bold-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
%%IncludeResource: font Math2Mono-Bold
%%IncludeFont: Math2Mono-Bold
/Math2Mono-Bold findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
63.000 12.500 moveto
(\\310) show
70.188 12.500 moveto
%%IncludeResource: font Math1Mono-Bold
%%IncludeFont: Math1Mono-Bold
/Math1Mono-Bold findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(y) show
77.562 12.500 moveto
%%IncludeResource: font Math1Mono-Bold
%%IncludeFont: Math1Mono-Bold
/Math1Mono-Bold findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(>) show
84.750 12.500 moveto
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
1 g
.37197 .94649 -12.8125 -5.84375 Mabsadd m
.37197 .94649 12.8125 -5.84375 Mabsadd L
.37197 .94649 12.8125 5.84375 Mabsadd L
.37197 .94649 -12.8125 5.84375 Mabsadd L
fill
0 g
gsave
.37197 .94649 -73.8125 -9.84375 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 19.6875 translate 1 -1 scale
63.000 12.500 moveto
%%IncludeResource: font Courier-Bold
%%IncludeFont: Courier-Bold
%%BeginResource: font Courier-Bold-MISO
%%BeginFont: Courier-Bold-MISO
/Courier-Bold /Courier-Bold-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
%%IncludeResource: font Math2Mono-Bold
%%IncludeFont: Math2Mono-Bold
/Math2Mono-Bold findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
63.000 12.500 moveto
(\\310) show
70.188 12.500 moveto
%%IncludeResource: font Math1Mono-Bold
%%IncludeFont: Math1Mono-Bold
/Math1Mono-Bold findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
77.438 12.500 moveto
%%IncludeResource: font Math1Mono-Bold
%%IncludeFont: Math1Mono-Bold
/Math1Mono-Bold findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(>) show
84.625 12.500 moveto
%%IncludeResource: font Courier-Bold-MISO
%%IncludeFont: Courier-Bold-MISO
/Courier-Bold-MISO findfont 12.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{218.375, 265.625},
  ImageMargins->{{33.625, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000570001SQ000`40n007h00OooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8
ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00
Uooo5@00Vooo002?oolT002Dool008_oo`h000Coo`X000;oo`03003oo`0000/0093oo`00R?oo1@00
2_oo1P002_oo10002_oo1P00S?oo0026ool90008ool2000Aool20008ool9002:ool008?oo`H000Ko
o`P001Coo`T000Ooo`D008Ooo`00POoo1P002?oo1@000_oo00<00?ooool04_oo00@00?ooooooo`D0
00Soo`800003ool00000008008Coo`00Oooo0`000_oo0P0000?oo`0000001P000ooo7P0000Coo`00
ooooo`P00003ool0000000?oo`<008;oo`00OOoo0P000ooo1@001?oo20000_oo00<00?ooool05?oo
00@00?ooooooo`L000Goo`D000Coo`80083oo`00Nooo0P000_oo1@0000?oo`00000010001?oo0`00
0ooo00<00?ooool05?oo00<00?ooool00ooo0`000_oo1`000_oo10000ooo0P00O_oo001iool20000
1Ooo00000000ool01@001?oo1P000_oo00@00?ooool001Soo`03003ooooo00;oo`03003ooooo00D0
00Coo`D00004ool000000002ool00`00ooooo`1kool007Ooo`800004ool00000ool400000ooo003o
o`050007ool20002ool00`00ooooo`0Fool00`00ooooo`02ool20006ool600000ooo000000030000
0ooo00000003001kool007Koo`L000?oo`@000[oo`8000;oo`03003oo`0001Soo`05003oooooool0
0002ool2000:ool40002ool300000ooo00000002001iool007Coo`D000Goo`800003ool00?oo00So
o`<000Goo`8001Soo`04003ooooo0005ool30009ool40003ool6001hool007;oo`@000Koo`<000;o
o`03003ooooo00Goo`8000Koo`800003ool0000001Ooo`<00003ool0000000Koo`8000Ooo`04003o
o`00ool20005ool4001fool0077oo`800003ool00?oo00Coo`8000;oo`05003oooooool00003ool3
0006ool20002ool00`00ooooo`03000Bool40004ool20006ool30003ool01000ooooo`000ooo0P00
1Ooo0`00MOoo001`ool20002ool01000oooooooo0P000ooo00<00?ooool00_oo00@00?oo000000Oo
o`8000Coo`03003oo`0000?ooa8000?oo`03003ooooo00Coo`8000Ooo`800003ool00?oo00;oo`03
003ooooo00;oo`8000?oo`03003ooooo008007?oo`00K_oo0P000ooo00<00?ooool00P001Ooo00<0
0?ooool00_oo0P001ooo0P000ooo8P001Ooo0P001ooo0P000ooo00<00?ooool01?oo0P000_oo00<0
0?ooool00P00L_oo001]ool30003ool30006ool01000oooooooo0P0000?oo`00ool01?oo1`000_oo
0P001_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo00D00?ooooooo`0000;oo`L0
0003ool0000000Coo`800003ool0000000;oo`03003ooooo00Goo`<000;oo`03003oo`00077oo`00
K?oo0`000ooo0P001ooo00<00?ooool00P001?oo1P001ooo00<00?oo00001Ooo00<00?ooool00_oo
00<00?ooool01?oo00<00?ooool0;?oo00@00?oo000006ooo`00J_oo0P0000?oo`00ool00_oo0P00
2?oo2@000_oo00<00?ooool01ooo00@00?ooool000Coo`03003ooooo00?oo`03003ooooo00Coo`03
003ooooo02[oo`04003ooooo0002ool00`00ooooo`1/ool006Woo`800004ool00?ooool20002ool0
0`00ooooo`03ool50007ool01000ool000001Ooo0P000_oo00<00?ooool00ooo00<00?ooool00ooo
00<00?ooool01?oo00<00?ooool02ooo0P001ooo0P005Ooo0P0000Coo`00ooooo`8006coo`00J?oo
00@00?ooool000;oo`03003ooooo00;oo`03003ooooo00D000Woo`03003ooooo00;oo`8000;oo`03
003ooooo00;oo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00_oo`80
00Ooo`8001Ooo`03003oo`0000?oo`03003ooooo06Woo`00Iooo0P0000Goo`00ooooo`0000Goo`<0
0004ool000000009ool00`00ooooo`05ool30004ool01@00oooooooo00001ooo00<00?ooool01?oo
00<00?ooool02ooo0P002?oo0P001ooo00<00?ooool03Ooo00<00?oo00000_oo0P00J_oo001Vool2
0002ool30002ool400001?oo000000000_oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool0
0P000_oo00@00?ooool000Soo`03003ooooo00Coo`03003ooooo00_oo`8000Koo`H000Goo`<000Wo
o`8000?oo`8000?oo`8006Woo`00IOoo00@00?oo003oo`800004ool000000005ool01@00oooooooo
00001ooo0P002Ooo00<00?ooool00_oo0`000_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?oo
ool02_oo0P001Ooo20001?oo10002_oo0`000_oo00H00?ooool00?oo001Xool006Coo`04003oo`00
ool40007ool01P00ool00000ool000Koo`03003ooooo00Soo`03003ooooo00Goo`L000Coo`03003o
oooo00Koo`03003ooooo00[oo`8000Goo`800004ool00000ool20006ool4000;ool40002ool00`00
ool0001Wool006?oo`03003ooooo00<000Woo`03003ooooo00;oo`8000Goo`03003ooooo00Soo`03
003ooooo00Koo`03003oo`0000Cooa8000Soo`8000Goo`800004ool00000ool20008ool4000<ool4
0002ool00`00ooooo`1Tool006;oo`03003ooooo008000[oo`03003ooooo00?oo`8000Coo`03003o
oooo00Soo`03003ooooo00Ooo`8000Woo`03003ooooo00Koo`03003ooooo00[oo`8000Koo`800004
ool00000ool20005ool40006ool00`00ooooo`06ool400000ooo003oo`1Uool0067oo`03003ooooo
00<000Woo`03003ooooo00Coo`06003oo`00003oo`002ooo00<00?ooool01_oo00<00?oo00002Ooo
00<00?ooool01_oo00<00?ooool02_oo0P001_oo0P0000Coo`00003oo`8000?oo`@000Woo`03003o
oooo00Koo`03003oo`0000;oo`03003ooooo06?oo`00H?oo00<00?oo00000`002?oo0P001_oo00<0
0?ooool00_oo0P002_oo00<00?ooool01ooo0P002Ooo00<00?ooool02?oo00<00?ooool02Ooo0P00
1_oo0P0000Coo`00003oo`<000;oo`<000_oo`03003ooooo00Goo`03003oo`0000800003ool00?oo
06?oo`00Gooo0`0000?oo`00ool00P001_oo00<00?ooool01_oo00D00?ooooooo`0000;oo`8000Oo
o`03003ooooo00Soo`8000Woo`03003ooooo00Soo`03003ooooo00Woo`8000Goo`<00003ool00000
00;oo`8000;oo`03003ooooo00coo`8000Koo`03003ooooo00@006?oo`00G_oo0P000_oo00D00?oo
003oo`0000Goo`03003ooooo00Ooo`04003ooooo0005ool20004ool00`00ooooo`06oolM0008ool2
0006ool00`00ooooo`02000Gool00`00ooooo`03ool01P00oooooooo003oo`8006;oo`00GOoo0P00
0ooo00<00?oo00000_oo00D00?ooooooo`0000Woo`05003oooooool00007ool20002ool00`00ool0
00060003ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool2000:ool2000Gool01`00
oooooooo003oo`0000;oo`03003ooooo0080067oo`00G?oo00<00?ooool00_oo00<00?oo00000ooo
00@00?ooool000[oo`04003ooooo0006ool80009ool2000:ool00`00ooooo`08ool00`00ooooo`09
ool2000:ool2000Hool01P00ooooo`00ool000?oo`05003oooooool0001Pool005_oo`8000?oo`04
003ooooo0004ool2000:ool00`00ooooo`080004ool00`00ooooo`040006ool2000:ool00`00oooo
o`08ool00`00ooooo`0`ool20003ool01@00oooooooo00000_oo0P00Gooo001Kool20002ool01000
ooooo`001Ooo00<00?ooool02?oo1@002ooo00<00?ooool01?oo100000Coo`00ool000Woo`03003o
oooo00[oo`03003ooooo00Goo`05003oooooool00002ool40006ool00`00ooooo`06ool5000<ool0
1@00oooooooo00001?oo00@00?ooool005ooo`00F_oo0P000_oo00D00?ooooooo`0000Coo`04003o
o`000004ool400000ooo0000000<ool00`00ooooo`09ool70006ool00`00ooooo`0:ool00`00oooo
o`04ool8000;ool00`00ooooo`08ool00`00ool000030006ool200000ooo003oo`02ool00`00oooo
o`02ool01000ool00000G_oo001Jool200000ooo003oo`02ool00`00ooooo`03ool01000oooooooo
10001Ooo00<00?ooool02_oo00<00?ooool02ooo0P001?oo6P0000?oo`00ool00ooo00<00?ooool0
2_oo00<00?ooool02?oo00@00?ooool000;oo`<000;oo`05003oooooool00003ool00`00ooooo`03
ool3001Nool005Woo`05003oo`00ool00003ool00`00ooooo`02ool00`00ool000050006ool00`00
ooooo`0:ool00`00ooooo`0:ool00`00ool0000:ool00`00ooooo`0:ool00`00ooooo`05ool00`00
ool00006000:ool00`00ooooo`07ool01000ooooo`001Ooo10000ooo00D00?ooooooo`0000Coo`04
003oo`00001Mool005Woo`04003oo`000003ool00`00ooooo`02ool30006ool00`00ooooo`02ool0
0`00ooooo`0:ool00`00ooooo`0;ool00`00ool0000:ool00`00ooooo`0:ool00`00ooooo`05ool;
0008ool00`00ooooo`08ool01000ooooo`000ooo0P000ooo0`0000Goo`00ooooo`0000Goo`<005go
o`00F?oo00@00?oo000000Coo`H000Woo`800003ool0000000_oo`03003ooooo00coo`03003oo`00
00[oo`03003ooooo00[oo`03003ooooo00Coo`d000Soo`03003ooooo00Ooo`07003oooooool00?oo
00002?oo100000?oo`00ool01?oo0`00G?oo001Hool00`00ool00004ool20003ool00`00ooooo`0:
ool3000;ool00`00ooooo`0;ool01000ooooo`002_oo00<00?ooool02ooo00<00?ooool00ooo3@00
2Ooo00<00?ooool01_oo00@00?ooooooo`8000_oo`04003oo`000006ool2001Lool005Ooo`06003o
o`00ool00?oo0`000ooo00<00?ooool02_oo00<00?ooool00`002?oo00<00?ooool03?oo00@00?oo
ool000[oo`03003ooooo00_oo`03003ooooo00;oo`l000Soo`03003ooooo00Koo`@000;oo`03003o
oooo00Woo`03003oo`00008000?oo`04003oo`00001Kool005Ooo`04003oo`00ool20002ool01000
ooooo`003Ooo00@00?ooool000;oo`8000Goo`03003ooooo00goo`04003ooooo000:ool00`00oooo
o`0;ool00`00ooooo`02ool?0009ool00`00ooooo`03ool30005ool00`00ooooo`0:ool01000ool0
0?oo0P0000?oo`00ool00P00Fooo001Fool01000ool00?oo0`0000?oo`00ool00_oo00<00?ooool0
2_oo00D00?ooooooo`0000Coo`<000;oo`03003ooooo00coo`05003oooooool00009ool00`00oooo
o`0<ool00`00ooooo`02ool?0009ool00`00ooooo`030002ool00`00ooooo`04ool00`00ooooo`09
ool00`00ool00003ool01000oooooooo0P00F_oo001Fool01000ool000000ooo0P000_oo00<00?oo
ool02ooo00@00?ooool000Soo`@000goo`05003oooooool00009ool00`00ooooo`0<ool01000oooo
oooo40002?oo10001_oo00<00?ooool00ooo00<00?ooool02_oo0P000_oo00D00?oo00000?oo0080
05[oo`00EOoo00<00?oo00001_oo00<00?oo00003Ooo00D00?ooooooo`0000Woo`03003ooooo00D0
00Soo`05003oooooool00009ool00`00ooooo`0<ool00`00ooooo`02ool?0003ool50002ool00`00
ooooo`05ool00`00ooooo`04ool00`00ooooo`0:ool00`00ool00004ool4001Iool005Goo`<000Ko
o`03003oo`0000coo`03003ooooo00;oo`03003ooooo00Koo`03003ooooo00Koo`D000;oo`05003o
ooooool0000:ool00`00ooooo`0=ool01000oooooooo4P002?oo00<00?ooool01?oo00<00?ooool0
1Ooo00<00?ooool02Ooo0`001_oo0P00FOoo001Dool00`00ool00006ool00`00ool0000=ool00`00
ooooo`02ool00`00ooooo`06ool00`00ooooo`0;ool7000:ool00`00ooooo`0=ool01000oooooooo
3`002ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool02_oo00<00?ooool01_oo0P00F?oo
001Dool00`00ool00006ool00`00ool00002000:ool00`00ooooo`03ool00`00ooooo`06ool00`00
ooooo`0=ool01000oooooooo20000ooo00<00?ooool03Ooo00@00?ooooooo`h000goo`03003ooooo
00?oo`03003ooooo00Koo`03003ooooo00Ooo`<000Soo`8005Soo`00Dooo0`001ooo0P000ooo0P00
1ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool03Ooo00<00?ooool00_oo00<00?ooool0
1Ooo5`0000?oo`0000002`003Ooo00<00?ooool00ooo00<00?ooool01ooo00<00?ooool01Ooo00<0
0?ooool00P002?oo0P00Eooo001Cool00`00ool00007ool20005ool00`00ooooo`04ool00`00oooo
o`03ool00`00ooooo`06ool00`00ooooo`0=oola000?ool00`00ooooo`03ool00`00ooooo`06ool0
0`00ooooo`03ool20004ool00`00ooooo`06ool2001Gool005?oo`8000Soo`03003ooooo00Goo`80
00?oo`03003ooooo00Coo`03003ooooo00Koo`03003ooooo00?oo`/000Coo`03003ooooo00Soo`03
003ooooo00goo`03003ooooo00Cooa0000Woo`03003ooooo00?oo`03003ooooo00Ooo`05003ooooo
ool00006ool20008ool00`00ooooo`1Eool005?oo`8000Ooo`8000Woo`800003ool00?oo00Goo`03
003ooooo00;oo`X000[oo`03003ooooo00;oo`03003ooooo00Soo`03003ooooo00goo`03003ooooo
00?oo`@00003ool00?oo00?oo`03003ooooo00Coo`X000Goo`03003ooooo00Ooo`04003oo`000007
ool20008ool2001Fool005;oo`8000Soo`03003ooooo00[oo`8000Coo`L000?oo`03003ooooo00go
o`03003ooooo00?oo`03003ooooo00Soo`03003ooooo00goo`03003ooooo00?oo`<000Soo`03003o
oooo00goo`P000Ooo`8000[oo`8000Soo`03003ooooo05Coo`00D_oo0P002?oo00<00?ooool02_oo
00<00?oo00000`000_oo00<00?ooool01?oo00<00?ooool03_oo00<00?ooool00ooo00<00?ooool0
2?oo00<00?ooool03Ooo00<00?ooool00_oo0`002Ooo00<00?ooool03Ooo00<00?ooool00_oo00<0
0?ooool01`0000?oo`00ool02Ooo0P002?oo0P00EOoo001Bool00`00ooooo`06ool20009ool50003
ool200000ooo003oo`05ool00`00ooooo`0>ool00`00ooooo`03ool00`00ooooo`08ool00`00oooo
o`0=ool00`00ooooo`02ool30009ool00`00ooooo`0>ool01@00oooooooo00000ooo2@002Ooo00<0
0?oo00001ooo0P002ooo1P00A?oo001Bool20007ool00`00ooooo`04ool40002ool00`00ooooo`05
ool30005ool00`00ooooo`0=ool00`00ooooo`04ool00`00ooooo`07ool00`00ooooo`0?ool00`00
ooooo`04000:ool00`00ooooo`0=ool01P00oooooooo003oo`8000Soo`D000Goo`03003oo`0000Oo
o`8000_oo`H004Coo`00DOoo00@00?ooool000Goo`8000;oo`@000Goo`03003ooooo00Ooo`03003o
o`0000800003ool00?oo00ooo`03003ooooo00Coo`03003ooooo00Ooo`03003ooooo00ooo`03003o
o`0000@000[oo`03003ooooo00koo`03003oo`00008000[oo`03003ooooo00;oo`@000;oo`03003o
o`0000Goo`03003oo`0000goo`8004Koo`00DOoo00D00?ooooooo`0000Coo`@000Woo`03003ooooo
00Ooo`03003ooooo00;oo`@000koo`03003ooooo00Coo`03003ooooo00Ooo`03003ooooo00ooo`03
003oo`0000800003ool00?oo00Woo`03003ooooo00coo`@00003ool00?oo00_oo`03003ooooo00Go
o`<00003ool00?oo00?oo`03003ooooo008000goo`8004Goo`00>Ooo1@0000?oo`0000000`003Ooo
00D00?ooooooo`0000?oo`<000[oo`03003ooooo00Soo`03003ooooo00?oo`03003ooooo00D000Wo
o`03003ooooo00Coo`03003ooooo00Ooo`03003ooooo00ooo`D00003ool00?oo00Woo`03003ooooo
00Koo`H000;oo`04003ooooo000<ool00`00ooooo`07ool40003ool00`00ooooo`02000=ool20015
ool003Woo`D00003ool0000000<000coo`8000Coo`@00003ool00?oo00Woo`03003ooooo00Soo`03
003ooooo00;oo`03003ooooo00Koo`H000;oo`03003ooooo00Goo`03003ooooo00Ooo`03003ooooo
00ooo`@000;oo`03003ooooo00Woo`03003ooooo00D000Woo`03003oo`0000coo`03003ooooo00Oo
o`03003ooooo00800003ool00?oo00?oo`03003ooooo00_oo`<004Coo`00>ooo0P000ooo0P003_oo
00<00?ooool00_oo10000_oo00<00?ooool02?oo00<00?ooool02?oo00<00?ooool00_oo00<00?oo
ool03?oo2`002Ooo00<00?ooool03ooo10000_oo00<00?ooool030003_oo00<00?oo00003Ooo00<0
0?ooool01_oo00@00?ooooooo`8000Goo`03003ooooo00_oo`@004?oo`00??oo0P0000?oo`000000
3ooo00D00?ooooooo`0000?oo`800003ool00?oo00Soo`03003ooooo00Woo`03003ooooo00;oo`03
003ooooo00koo`03003ooooo00GoobD000Woo`03003ooooo00koo`03003oo`0000goo`03003ooooo
00Koo`03003ooooo00800003ool0000000?oo`8000_oo`800003ool0000004?oo`00?Ooo0`002?oo
00<00?ooool01Ooo00@00?oo000000Coo`04003oo`000009ool00`00ooooo`09ool00`00ooooo`02
ool00`00ooooo`0>ool00`00ooooo`05ool00`00ooooo`07ool00`00ooooo`0>ool40003ool00`00
ooooo`09ool00`00ooooo`0>ool00`00ool0000>ool00`00ooooo`05ool300000ooo003oo`02ool2
00000ooo00000005ool00`00ooooo`03ool20002ool20012ool003coo`800003ool0000000Koo`H0
00;oo`<000Koo`03003ooooo00<000Koo`03003ooooo00Woo`03003ooooo00;oo`03003ooooo00oo
o`03003ooooo00Goo`03003ooooo00Ooo`03003ooooo00koo`<000Coo`03003ooooo00Woo`03003o
oooo00ooo`8000koo`03003ooooo00Coo`8000?oo`03003ooooo00?oo`<00003ool0000000@000?o
o`8000Coo`80047oo`00>ooo0P000ooo0P001_oo2P001Ooo00D00?ooooooo`0000;oo`8000Coo`03
003ooooo00Woo`03003ooooo00;oo`03003ooooo00koo`03003ooooo00Koo`03003ooooo00Ooo`03
003ooooo00goo`@000Coo`03003ooooo00[oo`03003ooooo00koo`8000koo`03003ooooo00;oo`80
0003ool00?oo00?oo`04003oooooool:0004ool20004ool20011ool003Woo`D00003ool0000000<0
00Woo`/000?oo`03003ooooo00;oo`800003ool00?oo00_oo`03003ooooo00;oo`03003ooooo00ko
o`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00goo`<000Goo`03003ooooo00[oo`03003o
oooo00koo`8000ooo`04003oo`000003ool01000oooooooo2P002?oo10000_oo1000@?oo000iool5
00000ooo00000003000:ool20002ool:0006ool3000;ool01@00oooooooo00004Ooo00<00?ooool0
1_oo00<00?ooool01ooo00<00?ooool03?oo10001Ooo00<00?ooool02_oo00<00?ooool03ooo00<0
0?ooool03?oo0`001Ooo2@000ooo0P002?oo10000_oo1000@?oo001>ool20006ool:0003ool00`00
ool000020008ool01@00oooooooo00004Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool0
3?oo10001Ooo00<00?ooool02_oo00<00?ooool03ooo00<00?ooool02Ooo0`0000?oo`00ool02`00
1_oo0P00D_oo001>ool20006ool01000oooooooo2`000ooo0`001Ooo00D00?ooooooo`00017oo`03
003ooooo00Koo`03003ooooo00Ooo`03003ooooo00_oo`D000Goo`03003ooooo00[oo`03003ooooo
00ooo`03003ooooo00Koo`<000;oo`X000Goo`03003ooooo00?oo`8005;oo`00C_oo0P001_oo00D0
0?ooooooo`0000Goo`T000;oo`@00003ool00?oo00;oo`03003ooooo00ooo`03003ooooo00Koo`03
003ooooo00Koo`03003ooooo00coo`<000;oo`03003ooooo00;oo`03003ooooo00[oo`03003ooooo
00ooo`03003ooooo00?oo``000Goo`03003ooooo00?oo`03003ooooo00?oo`<0057oo`00COoo00<0
0?oo00001_oo00D00?ooooooo`0000Ooo`03003oo`0000T00006ool00000003oo`004_oo00<00?oo
ool01_oo00<00?ooool01_oo00<00?ooool02ooo10000_oo00<00?ooool00_oo00<00?ooool02_oo
00<00?ooool03ooo0P0000?oo`0000002@000ooo00<00?ooool00ooo00<00?ooool00ooo00<00?oo
ool00ooo00<00?oo0000DOoo001=ool00`00ool00006ool01@00oooooooo00001ooo00<00?ooool0
1?oo3@003ooo00<00?ooool0<@001_oo00<00?ooool03?oo2`002?oo00<00?ooool00ooo00<00?oo
ool00ooo00<00?ooool00ooo00<00?oo0000DOoo001=ool30005ool00`00ooooo`02ool00`00oooo
o`05ool00`00ooooo`09oolJ0007ool00`00ooooo`06ool00`00ooooo`0:ool30004ool00`00oooo
o`02ool00`00ooooo`05oolL000<ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`02ool3
001Aool004goo`03003oo`0000Goo`03003ooooo00;oo`03003ooooo00Goo`03003ooooo00Wooa@0
00?oo`03003ooooo00Ooo`03003ooooo00Koo`03003ooooo00[oo`<000Coo`03003ooooo00;oo`03
003ooooo00_oo`03003ooooo014000koo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00;o
o`03003oo`00057oo`00COoo0P0000?oo`0000000ooo00<00?ooool00_oo00<00?ooool01Ooo00<0
0?ooool00ooo1P000_oo00<00?oo00001?oo2P0000?oo`00000030002?oo00<00?ooool02Ooo0`00
1Ooo00<00?ooool00_oo00<00?ooool00_oo5@001Ooo0P0000?oo`00000010002?oo00<00?ooool0
0ooo00<00?ooool01?oo00H00?ooooooo`00ool2001Aool004goo`8000?oo`04003ooooo0004ool0
0`00ooooo`04ool00`00ool000040008ool00`00ool00009ool:0009oolW0007ool:0009ool00`00
ool00006ool60003ool00`00ooooo`02ool00`00ooooo`04ool01000ooooo`000_oo0P00DOoo001=
ool20004ool20005ool00`00ooooo`06000=ool00`00ool0000>ool90005ool00`00ooooo`06ool0
0`00ooooo`08ool30006ool00`00ooooo`02ool00`00ooooo`05ool9000>ool00`00ool0000<ool6
0002ool00`00ooooo`05ool20003ool2001Aool004goo`8000Goo`8000?oo`@000Coo`03003ooooo
00coo`03003oo`0001;oo`/000Soo`03003ooooo00Ooo`@000Koo`03003ooooo00;oo`03003ooooo
00T000;oo`03003ooooo00koo`03003oo`0000ooo`03003ooooo00@000Goo`8000Coo`80057oo`00
COoo0P001Ooo00<00?oo00000P0000?oo`00ool01Ooo00<00?ooool03?oo00<00?oo00004Ooo00<0
0?ooool00ooo2P001?oo00<00?ooool01ooo0`001ooo00<00?ooool02P001ooo00<00?ooool03Ooo
00<00?oo00003ooo00<00?ooool00_oo00D00?oo00000?oo00800003ool00?oo00?oo`80057oo`00
COoo0P001Ooo0`0000Coo`00000000Koo`03003ooooo00goo`03003ooooo013oo`03003ooooo00So
o`X000Soo`@000Koo`T000coo`03003ooooo00coo`04003ooooo000?ool00`00ooooo`02ool00`00
ool000030002ool00`00ooooo`02ool2001Aool004coo`03003oo`0000Coo`8000Goo`<000?oo`03
003ooooo00koo`03003ooooo013oo`03003ooooo00Soo`04003oooooool:0004ool30002ool:0000
0ooo003oo`0=ool00`00ooooo`0<ool01000ooooo`004?oo00@00?ooooooo`8000Coo`<000Coo`03
003oo`00053oo`00C?oo00<00?oo00000_oo0P0000?oo`00ool01Ooo00H00?oo00000?oo000@ool0
0`00ooooo`0@ool00`00ooooo`08ool00`00ooooo`06oolC00000ooo003oo`03ool00`00ooooo`0<
ool00`00ooooo`0<ool01000ooooo`004?oo00<00?oo00000P001ooo0P000ooo00<00?oo0000D?oo
001<ool00`00ool000020003ool00`00ooooo`04ool01000oooooooo10003_oo00<00?ooool04?oo
00<00?ooool02?oo00<00?ooool01_oo00<00?ooool00_oo2@001_oo00<00?ooool00_oo00<00?oo
ool03?oo00<00?ooool03?oo00D00?ooooooo`0000goo`@000;oo`03003ooooo00Goo`08003oo`00
003oo`00ool0053oo`00C?oo00<00?oo00001Ooo00<00?ooool01?oo00<00?ooool00_oo00<00?oo
ool01@002?oo0P004_oo00<00?ooool02Ooo00<00?ooool01Ooo00<00?ooool01?oo10002Ooo00<0
0?ooool00_oo00<00?ooool03?oo00<00?ooool03?oo00D00?ooooooo`0000Ooo`H000;oo`04003o
oooo0008ool01000oooooooo0P0000?oo`00ool0Cooo001<ool00`00ool00005ool00`00ooooo`04
ool00`00ooooo`02ool00`00ooooo`05ool60002ool2000Bool00`00ooooo`09ool00`00ooooo`05
ool00`00ooooo`04ool3000:ool00`00ooooo`02ool00`00ooooo`0<ool00`00ooooo`0<ool01@00
oooooooo00000_oo1@002?oo00@00?ooool000Soo`03003ooooo00;oo`03003oo`00053oo`00C?oo
0`001Ooo00<00?ooool01?oo00<00?ooool00_oo00<00?ooool02ooo1P003ooo00<00?ooool02_oo
00<00?ooool01Ooo00<00?ooool00ooo10002_oo00<00?ooool00_oo00<00?ooool03Ooo00<00?oo
ool02ooo1`003Ooo00@00?ooool000Soo`03003ooooo00;oo`<0053oo`00C?oo0`001?oo00<00?oo
ool01Ooo00<00?ooool00_oo00<00?ooool03Ooo0P000_oo1P002Ooo00<00?ooool02_oo00<00?oo
ool01Ooo00<00?ooool00ooo10002_oo00<00?ooool00_oo00<00?ooool03Ooo00<00?ooool01_oo
1P000ooo00<00?ooool03Ooo00@00?ooool000Woo`04003oooooool3001@ool004coo`04003ooooo
0003ool00`00ooooo`05ool00`00ooooo`02ool00`00ooooo`0=ool20008ool60003ool00`00oooo
o`0:ool00`00ooooo`05ool00`00ooooo`02ool5000:ool00`00ooooo`02ool00`00ooooo`0=ool0
0`00ooooo`060005ool01@00oooooooo00003ooo00@00?ooool000Woo`04003ooooo0002ool00`00
ooooo`1>ool004coo`04003ooooo0003ool00`00ooooo`06ool01@00oooooooo00003ooo0P003_oo
3P000_oo00<00?ooool01Ooo00<00?ooool00_oo1@002_oo00<00?ooool00_oo00<00?ooool00ooo
3@002ooo00D00?ooooooo`0000ooo`04003ooooo0009ool01000ooooo`000_oo00<00?ooool0C_oo
001<ool40003ool00`00ooooo`06ool01@00oooooooo00003ooo0P004Ooo00<00?ooool02?oo:@00
2_oo00<00?ooool02ooo00D00?ooooooo`0000ooo`04003ooooo0009ool01@00ooooo`00ool00P00
D?oo001<ool01000ooooo`000ooo00<00?ooool01_oo00@00?ooool0013oo`03003oo`00013oo`03
003ooooo00[oo`03003ooooo00Goo`04003oooooool300000ooo0000000:ool00`00ooooo`02ool0
0`00ooooo`0=ool00`00ooooo`0:ool00`00ooooo`02ool00`00ooooo`0>ool00`00ool00009ool0
0`00ooooo`0200000ooo003oo`1?ool004coo`03003ooooo008000;oo`03003ooooo00Koo`04003o
oooo000@ool00`00ool0000@ool00`00ooooo`0:ool00`00ooooo`05ool00`00ooooo`0400000ooo
0000000:ool00`00ooooo`02ool00`00ooooo`0=ool00`00ooooo`0:ool00`00ooooo`02ool00`00
ooooo`0>ool00`00ool00009ool01000ool000000_oo00<00?ooool0C_oo001=ool01000ool00?oo
0`002?oo00@00?ooool0013oo`03003oo`00013oo`03003ooooo00[oo`03003ooooo00Goo`03003o
oooo00<000;oo`8000[oo`03003ooooo00;oo`03003ooooo00goo`03003ooooo00[oo`03003ooooo
00;oo`03003ooooo00koo`03003oo`0000Woo`800004ool00?oo001Aool004goo`04003ooooo0002
ool20007ool01000ooooo`004?oo00<00?oo00004?oo00<00?ooool02_oo00<00?ooool01Ooo00<0
0?oo00000`000_oo0P002_oo00<00?ooool00_oo00<00?ooool03Ooo00<00?ooool02_oo00<00?oo
ool00ooo00<00?ooool03Ooo0P002Ooo0P0000Goo`00ooooo`00057oo`00COoo00@00?ooool000;o
o`03003oo`00008000Coo`04003ooooo000@ool00`00ool0000>oolo00000ooo003oo`0;ool00`00
ooooo`03ool00`00ooooo`0=ool20006ool300001?oo003oo`000_oo00<00?ooool0Cooo001=ool0
1000ooooo`000_oo00<00?ooool00_oo0`000_oo00<00?oo00003ooo4P000_oo00<00?ooool02_oo
00<00?ooool01Ooo1@000ooo0P002_oo00<00?ooool00_oo00<00?ooool03?oo4P000_oo00<00?oo
ool03Ooo0P000ooo0`001?oo00<00?oo00000_oo00<00?ooool0Cooo001=ool01000ooooo`000_oo
00<00?ooool01Ooo0`0000?oo`00ool01ooo1`0000Coo`00ool0013oo`03003ooooo00[oo`03003o
oooo00Goo`@000Coo`8000[oo`03003ooooo00;oo`03003ooooo00goo`03003ooooo00[oo`04003o
ooooool8000:ool50007ool00`00ool00002ool00`00ooooo`1?ool004goo`04003ooooo0002ool0
0`00ooooo`07ool;0008ool00`00ool0000@ool00`00ooooo`0:ool00`00ooooo`05ool40004ool2
000:ool00`00ooooo`02ool00`00ooooo`0=ool00`00ooooo`0:ool00`00ooooo`03ool00`00oooo
o`03ool<000:ool00`00ool00002ool00`00ooooo`1?ool004koo`03003oo`0000?oo`03003ooooo
00Goo`D00003ool0000000@000Soo`04003ooooo000?ool00`00ooooo`0:ool00`00ooooo`05ool3
0005ool2000:ool00`00ooooo`02ool00`00ooooo`0=ool00`00ooooo`09ool00`00ooooo`04ool0
0`00ooooo`05ool;0008ool01P00ooooo`00ool005;oo`00C_oo00@00?ooool000;oo`03003ooooo
00;oo`<00004ool00?oo0008ool60002ool01000ooooo`003ooo00<00?ooool02_oo00<00?ooool0
1?oo10001Ooo0P002_oo00<00?ooool00_oo00<00?ooool03Ooo00<00?ooool02Ooo00<00?ooool0
1?oo00<00?ooool01@002?oo0P0000Coo`00000000Goo`03003oo`0000;oo`03003ooooo053oo`00
C_oo00@00?ooool000;oo`03003oo`00008000Goo`8000koo`L000koo`03003ooooo00_oo`03003o
oooo00?oo`<000Koo`8000[oo`05003oooooool0000@ool00`00ooooo`09ool00`00ooooo`07000=
ool00`00ooooo`03ool30002ool00`00ool00002ool00`00ooooo`1@ool004koo`05003ooooo003o
o`030008ool00`00ool0000?ool01@00ooooo`00ool02P001?oo00<00?ooool02ooo00<00?ooool0
0_oo10001_oo0P002_oo00D00?ooooooo`00013oo`03003ooooo00;oo`X000Coo`03003ooooo00co
o`8000Soo`<00005ool00?ooool0001Bool004koo`03003ooooo00800003ool00?oo00Soo`03003o
o`0000ooo`05003oooooool0000:ool?0003ool00`00ooooo`02ool30007ool2000:ool01@00oooo
oooo00001_oo3`001ooo00<00?ooool01?oo00<00?ooool03?oo0P002_oo0`000_oo00<00?ooool0
D?oo001>ool01000ooooo`000_oo00<00?ooool01ooo00<00?oo00003ooo00D00?ooooooo`0000ko
o`03003ooooo00SoobT000[oo`03003ooooo00Woo`03003ooooo00Coo`03003ooooo00coo`03003o
oooo00Woo`03003oo`0000;oo`03003ooooo053oo`00Cooo0P0000Coo`00ool000[oo`8000ooo`05
003oooooool0000>ool00`00ooooo`0;ool01000oooooooo10001ooo0P002_oo00D00?ooooooo`00
013oo`03003ooooo00Soo`03003ooooo00Coo`03003ooooo00goo`03003ooooo00Woo`800003ool0
000005?oo`00Cooo0P0000Coo`00ool000[oo`8000ooo`05003oooooool0000>ool00`00ooooo`0;
ool01000oooooooo10001ooo0P002_oo00D00?ooooooo`00013oo`03003ooooo00Soo`03003ooooo
00Coo`03003ooooo00goo`03003ooooo00Woo`800003ool0000005?oo`00Cooo00L00?ooooooo`00
ool00009ool2000?ool01@00oooooooo00003_oo00<00?ooool02ooo00<00?ooool00`0000?oo`00
ool01_oo0P002_oo00D00?ooooooo`00013oo`03003ooooo00Soo`03003ooooo00Coo`03003ooooo
00coo`8000[oo`8000?oo`03003ooooo057oo`00Cooo0`0000Coo`00ool000[oo`03003ooooo00go
o`03003ooooo00;oo`03003ooooo00_oo`03003ooooo00_oo`03003ooooo00<00003ool00?oo00Ko
o`8000[oo`05003oooooool0000@ool00`00ooooo`08ool00`00ooooo`04ool00`00ooooo`0<ool2
000:ool200001?oo00000000Dooo001@ool01P00ool00000ool000[oo`03003ooooo00koo`05003o
ooooool0000=ool00`00ooooo`0;ool00`00ool000020002ool00`00ooooo`05ool2000:ool01@00
oooooooo00004?oo00<00?ooool02?oo00<00?ooool01?oo00<00?ooool03?oo0P002_oo0`0000?o
o`00ool0Dooo001@ool01000oooooooo0`002Ooo0P003ooo00D00?ooooooo`0000koo`03003ooooo
00[oo`03003oo`00008000;oo`03003ooooo00Goo`8000[oo`05003oooooool0000?ool00`00oooo
o`08ool00`00ooooo`05ool00`00ooooo`0;ool20009ool30003ool00`00ooooo`1Bool0057oo`03
003ooooo00800003ool00000008000Koo`03003ooooo00goo`05003oooooool0000>ool00`00oooo
o`0:ool40003ool00`00ooooo`05ool2000:ool01@00oooooooo00003ooo00<00?ooool02?oo00<0
0?ooool01Ooo00<00?ooool02ooo0P001Ooo10000_oo00@00?ooool005Goo`00DOoo00H00?ooool0
0?oo0004ool40002ool00`00ooooo`0=ool00`00ooooo`02ool00`00ooooo`0;ool00`00ooooo`0;
ool30003ool00`00ooooo`05ool2000:ool01000ooooo`004?oo00<00?ooool02?oo00<00?ooool0
1?oo00<00?ooool02ooo00@00?oo003oo`@000Goo`8000;oo`03003ooooo05?oo`00DOoo00@00?oo
ooooo`8000Soo`H000coo`03003ooooo00;oo`03003ooooo00_oo`03003ooooo00[oo`<000Coo`03
003ooooo00Goo`8000[oo`04003ooooo000@ool00`00ooooo`08ool00`00ooooo`04ool00`00oooo
o`09ool60009ool01@00oooooooo0000EOoo001Bool00`00ooooo`02000:ool01000oooooooo2000
1?oo00<00?ooool00_oo00<00?ooool02ooo00<00?ooool02_oo0`001?oo00<00?ooool01Ooo0P00
2_oo00@00?ooool0013oo`03003ooooo00Soo`03003ooooo00Coo`04003oooooool80002ool00`00
ool0000:ool01000ooooo`00E_oo001Bool01000oooooooo0P002_oo00<00?ooool02?oo3P002Ooo
00<00?ooool02Ooo0`001Ooo00<00?ooool01Ooo0P002_oo00@00?ooool0013oo`03003ooooo00Ko
o`d000Woo`03003oo`0000[oo`05003oooooool0001Fool005?oo`03003ooooo008000[oo`03003o
oooo00coo`03003ooooo00?oo`04003ooooooolC0002ool30005ool00`00ooooo`05ool2000:ool0
1000ooooo`001_oo4`0000?oo`00ool01_oo00<00?ooool02_oo00<00?oo00002_oo00@00?ooool0
05Ooo`00Dooo00H00?ooool00?oo0009ool00`00ooooo`0=ool00`00ooooo`02ool00`00ooooo`0:
ooli0003ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`0:ool00`00ool00009ool20002
ool00`00ooooo`1Eool005Coo`03003ooooo008000[oo`03003ooooo00OooaP000[oo`@000Goo`03
003ooooo00Goo`8000[oo`04003ooooo000=oolG000:ool00`00ool0000:ool01000ooooo`00F?oo
001Dool00`00ooooo`020008ool<0005ool00`00ooooo`02ool00`00ooooo`0:ool00`00ooooo`07
ool300000ooo003oo`04ool00`00ooooo`05ool2000:ool01000ooooo`004?oo00<00?ooool01ooo
00<00?ooool01?oo00<00?ooool03@002_oo00@00?ooool005Soo`00E?oo00@00?ooooooo`800003
ool0000000@000;oo`8000goo`03003ooooo00?oo`03003ooooo00[oo`03003ooooo00Koo`<00003
ool00?oo00Coo`03003ooooo00Goo`8000[oo`04003ooooo000?ool00`00ooooo`07ool00`00oooo
o`05ool00`00ooooo`0:ool00`00ool000060003ool01@00oooooooo0000F?oo001Eool00`00oooo
o`030009ool00`00ooooo`0<ool00`00ooooo`02ool00`00ooooo`0:ool00`00ooooo`05ool40000
0ooo003oo`04ool00`00ooooo`05ool2000:ool01000ooooo`003ooo00<00?ooool01ooo00<00?oo
ool01Ooo00<00?ooool02Ooo00<00?oo00001ooo10000_oo00<00?ooool0Eooo001Eool01@00ool0
0?oo00002_oo0P003Ooo00<00?ooool00ooo00<00?ooool02Ooo00<00?ooool01Ooo0`000ooo00<0
0?ooool00_oo00<00?ooool01Ooo0P002_oo00<00?oo00004?oo00<00?ooool01_oo00<00?ooool0
1_oo00<00?ooool02Ooo00<00?oo00002Ooo0`0000?oo`00ool0F?oo001Fool200000ooo0000000:
ool00`00ooooo`0;ool00`00ooooo`03ool00`00ooooo`09ool00`00ooooo`04ool40003ool00`00
ooooo`02ool00`00ooooo`05ool2000:ool00`00ool0000@ool00`00ooooo`06ool00`00ooooo`05
ool00`00ooooo`09ool00`00ool0000:ool01000ool00000F_oo001Fool20002ool00`00ooooo`08
ool2000=ool00`00ooooo`02ool00`00ooooo`09ool00`00ooooo`04ool30004ool00`00ooooo`02
ool00`00ooooo`05ool20009ool01000ooooo`004?oo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool02Ooo00<00?oo00002Ooo0P0000?oo`000000F_oo001Gool300000ooo003oo`08ool2000=
ool00`00ooooo`03ool00`00ooooo`08ool00`00ooooo`03ool40004ool00`00ooooo`02ool00`00
ooooo`05ool20009ool01000ooooo`004?oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool0
2?oo00@00?ooool000Soo`D005_oo`00Eooo00<00?ooool01`001Ooo0P003?oo00<00?ooool00ooo
00<00?ooool02Ooo00<00?ooool00_oo0`001Ooo00<00?ooool00_oo00<00?ooool01Ooo0P002Ooo
00@00?ooool000ooo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Soo`03003oo`0000Go
o`H000;oo`03003ooooo05Woo`00F?oo00D00?ooooooo`0000Coo`T000[oo`03003ooooo00?oo`03
003ooooo00Woo`04003oooooool40005ool00`00ooooo`02ool00`00ooooo`05ool20009ool01000
ooooo`003ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool02?oo2@000ooo0P000_oo00<0
0?ooool0F_oo001Iool00`00ooooo`020009ool200000ooo000000060003ool00`00ooooo`03ool0
0`00ooooo`08ool01000oooooooo0`001_oo00<00?ooool00_oo00<00?ooool01Ooo0P002Ooo00@0
0?ooool000ooo`03003ooooo00Goo`03003ooooo00Koo`04003oooooool800000ooo003oo`07ool0
1@00ool00?oo0000GOoo001Iool01@00oooooooo00002Ooo00<00?oo00002?oo7@001?oo00<00?oo
ool00_oo00<00?ooool01Ooo0P002Ooo00@00?ooool000Kooah000Koo`04003ooooo0008ool20002
ool00`00ooooo`1Kool005[oo`05003oooooool00009ool2000;ool00`00ooooo`04ool00`00oooo
o`07ool00`00ooooo`030003oolV0009ool00`00ooooo`04ool00`00ooooo`07ool00`00ooooo`07
ool00`00ool00008ool20002ool00`00ooooo`1Lool005_oo`04003ooooo0009ool00`00ool0000;
ool00`00ooooo`03ool00`00ooooo`07ool00`00ool000030007ool00`00ooooo`02ool00`00oooo
o`05ool20009ool01000ooooo`003_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo
00@00?ooool000Soo`04003ooooo001Oool005_oo`05003oooooool00009ool2000;ool00`00oooo
o`03ool00`00ooooo`08ool40009ool01@00oooooooo00001ooo0P002Ooo00<00?oo00003ooo00<0
0?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?oo00002?oo0P000_oo00<00?ooool0
GOoo001Lool00`00ooooo`020008ool00`00ool0000;ool00`00ooooo`03ool00`00ooooo`02ool9
0009ool01@00oooooooo00001ooo0P002Ooo00<00?oo00003ooo00<00?ooool01?oo00<00?ooool0
1_oo00<00?ooool01ooo00@00?ooool000Ooo`8000;oo`03003ooooo05koo`00GOoo00@00?ooool0
00Woo`8000_oo`03003ooooo00?oo`03003ooooo00/000Woo`05003oooooool00007ool20009ool0
0`00ool0000>ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`07ool00`00ool00008ool0
1000ooooo`00HOoo001Mool01@00oooooooo00002?oo00<00?oo00002_oo00<00?ooool01?oo3P00
2?oo00D00?ooooooo`0000Ooo`8000Woo`03003oo`0000koo`03003ooooo00Coo`03003ooooo00Ko
o`03003ooooo00Ooo`04003ooooo0007ool20002ool00`00ooooo`1Oool005koo`03003ooooo0080
00Soo`8000WooeL000Woo`03003oo`0000Ooo`8000;oo`03003ooooo063oo`00Gooo00@00?ooool0
00Goo`h000;oo`03003ooooo00?oo`l000Ooo`03003ooooo00;oo`03003ooooo00Coo`8000Soo`04
003ooooo000=ool00`00ooooo`04ool00`00ooooo`06ool?0006ool01000ooooo`00Hooo001Oool0
0`00ooooo`060004ool00`00ool0000:ool00`00ooooo`02ool?0007ool00`00ooooo`02ool00`00
ooooo`04ool20008ool01000ooooo`003Ooo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool0
1_oo00@00?ooool000;oo`L000;oo`03003ooooo067oo`00H?oo0P0000?oo`0000001ooo00@00?oo
ool000Woo`03003ooooo00;oo`l000Soo`05003oooooool00006ool20008ool00`00ool0000>ool0
0`00ooooo`03ool00`00ooooo`06ool00`00ooooo`06ool01@00oooooooo00001_oo1@00I?oo001Q
oolC0006ool00`00ooooo`0@0008ool01@00oooooooo00001_oo0P002?oo00<00?oo00003Ooo00<0
0?ooool01?oo00<00?ooool01_oo00<00?ooool00ooo400000?oo`00ool0I?oo001Rool00`00oooo
o`020007ool00`00ool00003oolM0004ool01@00oooooooo00001_oo0P002?oo00<00?oo00001Ooo
7P000_oo00@00?ooool000Koo`8000;oo`03003ooooo06Coo`00Hooo0P0000?oo`0000001ooo00<0
0?oo00002Ooo00<00?ooool03`001?oo8@002?oo00<00?ooool00ooo00<00?ooool01Ooo00<00?oo
ool01_oo00@00?ooool000Koo`8000;oo`03003ooooo06Goo`00IOoo00@00?oo000000Koo`04003o
oooo0008ool00`00ooooo`0?0008ool01@00oooooooo00001_oo0P002?oo00<00?oo00003?oo00<0
0?ooool00ooo00<00?ooool01_oo00<00?ooool01Ooo00D00?ooooooo`0000Goo`8000;oo`03003o
oooo06Koo`00I_oo00@00?oo000000Koo`04003ooooo0008ool00`00ooooo`0=000:ool01000oooo
o`001_oo0P002?oo0P003Ooo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01Ooo00D00?oo
ooooo`0000Goo`8000;oo`03003ooooo06Ooo`00Iooo00@00?oo000000Koo`04003ooooo0008ool0
0`00ool0000<000:ool01@00oooooooo00001Ooo0P002?oo0P003Ooo00<00?ooool00_oo00<00?oo
ool01Ooo00<00?ooool01Ooo00D00?ooooooo`0000Goo`800003ool0000006[oo`00J?oo00@00?oo
000000Koo`04003ooooo0007ool00`00ooooo`0<000:ool01@00oooooooo00001Ooo0P002?oo0P00
3?oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01?oo00D00?ooooooo`0000Goo`800003
ool00?oo06_oo`00JOoo00@00?oo000000Koo`04003ooooo0007ool01000oooooooo1`000_oo00<0
0?ooool02?oo00D00?ooooooo`0000Goo`8000Soo`8000coo`03003ooooo00;oo`03003ooooo00Go
o`03003ooooo00Coo`05003oooooool00005ool200000ooo003oo`1/ool006[oo`@000Koo`03003o
o`0000Ooo`03003ooooo00?oo`8000Koo`03003ooooo00Ooo`05003oooooool00005ool20007ool0
0`00ool0000;ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`04ool01000ooooo`001Ooo
0P0000?oo`00ool0KOoo001/ool30005ool01000ooooo`001ooo00<00?ooool00ooo00<00?ooool0
1?oo00<00?ooool02?oo00@00?ooool000Goo`8000Ooo`8000coo`03003ooooo00;oo`03003ooooo
00Goo`03003ooooo00Coo`05003oooooool00004ool200000ooo003oo`1^ool006goo`<000Goo`04
003ooooo0007ool00`00ooooo`02ool00`00ooooo`04ool00`00ooooo`08ool01@00oooooooo0000
1?oo0P001ooo0P003?oo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool01?oo00D00?oooooo
o`0000Coo`800003ool00?oo06ooo`00K_ooJ000LOoo001_oomV001bool0077oo`<000Coo`800003
ool0000000Goo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Ooo`04003ooooo0004ool2
0007ool00`00ooooo`09ool01@00oooooooo00001ooo00<00?ooool00_oo0P000_oo0P000_oo1@00
M?oo001bool40004ool01000ooooo`001Ooo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0
1ooo00@00?ooool000Coo`8000Ooo`03003ooooo00Woo`05003oooooool00006ool00`00ooooo`02
ool00`00ooooo`020003ool01000ool00000M_oo001dool40003ool200000ooo003oo`04ool00`00
ooooo`03ool00`00ooooo`02ool00`00ooooo`06ool01000ooooo`001?oo0P001_oo0P002_oo00D0
0?ooooooo`0000Koo`03003ooooo00;oo`04003ooooo0003ool5001gool0093oo`03003ooooo00Ko
o`03003oo`0000Coo`8000Koo`8000Woo`05003oooooool00005ool20003ool200000ooo00000003
ool4001iool006koo`8000Koo`H001Goo`03003ooooo00Goo`04003ooooo0003ool20006ool00`00
ooooo`07ool01@00oooooooo00001Ooo00<00?ooool00_oo00@00?ooool000?oo`D007[oo`00K_oo
0P001_oo1`003ooo00<00?ooool00_oo00<00?ooool01_oo00<00?oo00000ooo0P001_oo00<00?oo
ool01ooo00@00?ooool000Goo`03003ooooo00;oo`04003oo`000002ool5001lool006koo`8000_o
o`8000?oo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo00Goo`03003oo`0000?oo`8000Go
o`8000Soo`05003oooooool00004ool01000oooooooo10000ooo00@00?oo000007koo`00K_oo0P00
1ooo1P000ooo0`002Ooo=P0000?oo`0000000`00Oooo001^ool20006ool60004ool40008ool90006
ool20003ool20005ool00`00ooooo`03oolI0021ool006koo`8000Goo`8000_oo`@000Ooo`800003
ool0000000?ooaP00004ool00?ooool20002ool20002ool30002ool60023ool006koo`8000Goo`80
00goo`@000Ooo`800003ool00?oo00;oo`03003ooooo00?oo`8000;oo`8000Coo`03003ooooo00?o
o`8000;oo`03003ooooo008000;oo`<000;oo`D008Koo`00K_oo0P001_oo0P002_oo10001_oo0`00
0_oo10000_oo00<00?ooool00_oo00@00?oo003oo`8000?oo`8000Coo`03003ooooo00800004ool0
0000ool30002ool50029ool006koo`8000Ooo`H000?oo`@000Soo`D000;oo`<000;oo`03003ooooo
00;oo`800003ool0000000?oo`03003ooooo00;oo`05003ooooo003oo`0200000ooo00000007002<
ool006koo`8000Soo`D000?oo`<000Woob`008ooo`00K_oo0P001ooo0P001ooo00<00?ooool02ooo
9`00T_oo001^ool20007ool2000Wool2002Uool006koo`8000Soo`L0027oo`800:Goo`00K_oo0P00
2Ooo1@008_oo0P00YOoo002Pool2002Uool00:3oo`800:Goo`00X?oo0P00YOoo002Pool2002Uool0
0:3oo`800:Goo`00X?oo0P00YOoo002Pool2002Uool00:3oo`800:Goo`00X?oo0P00YOoo002Pool2
002Uool00:3oo`800:Goo`00X?oo0P00YOoo002Pool2002Uool00:3oo`800:Goo`00X?oo0P00YOoo
002Pool2002Uool00:3oo`800:Goo`00X?oo0P00YOoo003ooom8ool00?ooodSoo`00ooooB?oo003o
oom8ool009goo`P00:;oo`00WOoo2000X_oo002Nool20003ool2002Rool009ooo`800:Koo`00X?oo
0P00YOoo002Qool2002Tool009goo`8000?oo`800:?oo`00WOoo2000X_oo002Mool8002Rool00?oo
odSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo
003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?oo
odSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo
003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?oo
odSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo
003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo003ooom8ool00?oo
odSoo`00ooooB?oo003ooom8ool00?ooodSoo`00ooooB?oo0000\
\>"],
  ImageRangeCache->{{{116.438, 290.125}, {763.062, 551.562}} -> {-0.335735, \
2.25553, 0.00327227, 0.00327227}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for X",
ScreenRectangle->{{0, 1152}, {0, 864}},
WindowSize->{1120, 831},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PrintingMargins"->{{86.375, 54}, {72, 72}},
"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "Net", "hubble", \
"Users", "felix", "tmp"}, "print.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1},
Magnification->1.5
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 34, 0, 60, "Subsubsection"],
Cell[1764, 54, 45, 0, 46, "Text"],
Cell[1812, 56, 237, 4, 62, "Input"],
Cell[2052, 62, 784, 19, 84, "Text"],
Cell[2839, 83, 1384, 34, 238, "Input"],
Cell[4226, 119, 208, 7, 46, "Text"],

Cell[CellGroupData[{
Cell[4459, 130, 72, 1, 39, "Input"],
Cell[4534, 133, 35, 1, 39, "Output"]
}, Open  ]],
Cell[4584, 137, 209, 7, 46, "Text"],
Cell[4796, 146, 335, 6, 62, "Input"],
Cell[5134, 154, 52, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[5211, 159, 1002, 16, 246, "Input"],
Cell[6216, 177, 62685, 2425, 411, 26644, 1976, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

