(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     52654,       2341]*)
(*NotebookOutlinePosition[     53722,       2374]*)
(*  CellTagsIndexPosition[     53678,       2370]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Example 2", "Subsection"],

Cell[TextData[{
  "In this example we look at some plots of ",
  Cell[BoxData[
      FormBox["s", TraditionalForm]]],
  "-",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["v", "\[Prime]"], TraditionalForm]]],
  "-curves for different guesses and parameter ranges."
}], "Text"],

Cell[CellGroupData[{

Cell["Some definitions", "Subsubsection"],

Cell[TextData[{
  "Disturbance ",
  Cell[BoxData[
      FormBox["s", TraditionalForm]]],
  " and postestimation deviation ",
  Cell[BoxData[
      FormBox[
        RowBox[{"vp", "\[Congruent]", 
          SuperscriptBox["v", "\[Prime]"]}], TraditionalForm]]],
  " depending on ",
  Cell[BoxData[
      FormBox["\[Alpha]", TraditionalForm]]],
  " and ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"g1p", "[", "m_", "]"}], "\[Congruent]", 
          RowBox[{
            SubsuperscriptBox["g", "1", "\[Prime]"], "(", "m", ")"}]}], 
        TraditionalForm]]]
}], "Text"],

Cell[BoxData[{
    \(s[\[Alpha]_] = \(1\/6\) \((2  \[Alpha]\^2 - 2  \[Alpha] + 
            1)\); \), "\[IndentingNewLine]", 
    \(vp[\[Alpha]_, 
        g1p_] = \(1\/2\) \((g1p[0]\^2 + g1p[1]\^2 - 2  g1p[1] + 
              1)\) - \((g1p[0] - g1p[1] + 
              1)\) \[Alpha] + \[Alpha]\^2; \)}], "Input"],

Cell["Function that creates a list of tick marks for the plots:", "Text"],

Cell[BoxData[
    \(\(\(ticks[min_, max_] := 
      Chop[Range[Floor[min], Ceiling[max], 0.1]]; \)\(\ \)\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot of ",
  Cell[BoxData[
      \(TraditionalForm\`s\)]],
  "-",
  Cell[BoxData[
      \(TraditionalForm\`v\^\[Prime]\)]],
  "-curves of estimation procedures with different guesses and different \
ranges of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\)]]
}], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(g1p2a[m_, \[Alpha]_] := \[Alpha]\ m; \), "\[IndentingNewLine]", 
    \(g1p2b[m_, \[Alpha]_] := m; \), "\[IndentingNewLine]", 
    \(g1p2c[m_, \[Alpha]_] := \[Alpha]; \), "\[IndentingNewLine]", 
    \(\(pa = 
        ParametricPlot[{{s[\[Alpha]], 
              vp[\[Alpha], g1p2a[#, \[Alpha]] &]}}, {\[Alpha], 0.3, 0.8}, 
          DisplayFunction \[Rule] Identity];\)\), "\[IndentingNewLine]", 
    \(\(pb = 
        ParametricPlot[{s[\[Alpha]], 
            vp[\[Alpha], g1p2b[#, \[Alpha]] &]}, {\[Alpha], 0, 1}, 
          PlotStyle \[Rule] {Dashing[{0.02, 0.02}]}, 
          DisplayFunction \[Rule] Identity];\)\), "\[IndentingNewLine]", 
    \(\(pc = 
        ParametricPlot[{s[\[Alpha]], 
            vp[\[Alpha], g1p2c[#, \[Alpha]] &]}, {\[Alpha], 0, 0.6}, 
          PlotStyle \[Rule] Thickness[0.005], 
          DisplayFunction \[Rule] Identity];\)\), "\[IndentingNewLine]", 
    \(\(Show[pa, pb, pc, DisplayFunction \[Rule] $DisplayFunction, 
        PlotRange \[Rule] {{0, 0.2}, {0, 0.5}}, Ticks \[Rule] {ticks, ticks}, 
        AxesLabel \[Rule] {"\<s\>", \*"\"\<\!\(v\^\[Prime]\)\>\""}];\)\)}], \
"Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 5 0 1.23607 [
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[1 -0.0125 -9 -9 ]
[1 -0.0125 9 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 10 4.90625 ]
[-0.0125 .12361 -18 -4.5 ]
[-0.0125 .12361 0 4.5 ]
[-0.0125 .24721 -18 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .37082 -18 -4.5 ]
[-0.0125 .37082 0 4.5 ]
[-0.0125 .49443 -18 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .61803 -18 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[0 .64303 -7.03125 0 ]
[0 .64303 7.03125 10 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.5 0 m
.5 .00625 L
s
[(0.1)] .5 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(0.2)] 1 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
69.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .12361 m
.00625 .12361 L
s
[(0.1)] -0.0125 .12361 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(0.2)] -0.0125 .24721 1 0 Mshowa
0 .37082 m
.00625 .37082 L
s
[(0.3)] -0.0125 .37082 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(0.4)] -0.0125 .49443 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(0.5)] -0.0125 .61803 1 0 Mshowa
0 0 m
0 .61803 L
s
gsave
0 .64303 -68.0312 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 18 translate 1 -1 scale
63.000 11.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(v) show
69.000 7.625 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(\\242) show
73.062 11.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.48333 .15451 m
.4705 .14324 L
.45806 .13386 L
.4526 .13039 L
.44787 .1278 L
.44336 .12579 L
.44127 .12504 L
.43942 .12448 L
.43746 .12403 L
.43647 .12386 L
.43597 .12379 L
.43544 .12372 L
.435 .12368 L
.4346 .12365 L
.43441 .12364 L
.4342 .12363 L
.43401 .12362 L
.43383 .12361 L
.4336 .12361 L
.4334 .12361 L
.43318 .12361 L
.43306 .12361 L
.43295 .12361 L
.43276 .12362 L
.43255 .12362 L
.43234 .12363 L
.43215 .12365 L
.43171 .12368 L
.43131 .12373 L
.43043 .12386 L
.42968 .12403 L
.4289 .12424 L
.42762 .12472 L
.42628 .1254 L
.42498 .12627 L
.42388 .12722 L
.42269 .12852 L
.42168 .12991 L
.42003 .13299 L
.41929 .13486 L
.4186 .13705 L
.41804 .13934 L
.41754 .14201 L
.41734 .14331 L
.41717 .14473 L
.41703 .14611 L
.41692 .1474 L
.41682 .1488 L
.41676 .15017 L
.41673 .15093 L
Mistroke
.4167 .15164 L
.41669 .15244 L
.41668 .15289 L
.41667 .1533 L
.41667 .15367 L
.41667 .15406 L
.41667 .15439 L
.41667 .15476 L
.41667 .15516 L
.41667 .15559 L
.41668 .156 L
.41668 .15638 L
.4167 .15711 L
.41671 .15792 L
.41676 .15941 L
.41684 .16113 L
.41693 .16278 L
.41721 .16667 L
.41758 .17063 L
.41808 .1751 L
.42095 .19379 L
.4252 .21454 L
.43064 .23702 L
.43791 .26367 L
.44633 .29197 L
.45683 .32491 L
.46844 .35943 L
.481 .39522 L
.49586 .43607 L
.51163 .4781 L
.52814 .52102 L
.54717 .56941 L
.56667 .61803 L
Mfstroke
[ .02 .02 ] 0 setdash
.83333 0 m
.83128 0 L
.82941 1e-05 L
.82727 2e-05 L
.82524 3e-05 L
.82165 6e-05 L
.81777 .00011 L
.81352 .00018 L
.80904 .00027 L
.80107 .00048 L
.79292 .00076 L
.78537 .00109 L
.76846 .00203 L
.75163 .0033 L
.73658 .00473 L
.70397 .00889 L
.67705 .01356 L
.64917 .01978 L
.62214 .0274 L
.59907 .03538 L
.55658 .05465 L
.5189 .07871 L
.48821 .10598 L
.47474 .12136 L
.46157 .13944 L
.44175 .17599 L
.43407 .19562 L
.42739 .21781 L
.42473 .22904 L
.42226 .24157 L
.42049 .25267 L
.41892 .26521 L
.41834 .27114 L
.4178 .27761 L
.41742 .28327 L
.4171 .28952 L
.41695 .293 L
.41685 .29626 L
.41676 .29973 L
.41673 .30167 L
.4167 .30345 L
.41668 .30511 L
.41667 .30665 L
.41667 .30834 L
.41667 .30919 L
.41667 .31013 L
.41667 .31172 L
.41669 .31342 L
.41671 .31503 L
.41673 .31651 L
Mistroke
.41679 .31981 L
.41687 .32282 L
.41711 .32961 L
.41743 .33596 L
.41779 .34199 L
.41888 .35574 L
.42047 .37089 L
.42454 .39984 L
.43043 .43156 L
.44518 .49185 L
.46673 .56036 L
Mfstroke
.46673 .56036 m
.48844 .61803 L
s
.005 w
[ ] 0 setdash
.83333 .61803 m
.79375 .55933 L
.75284 .49864 L
.71655 .44482 L
.68361 .39596 L
.65065 .34707 L
.62098 .30305 L
.59164 .25953 L
.56553 .2208 L
.54239 .18649 L
.51993 .15316 L
.50038 .12417 L
.48352 .09916 L
.46765 .07562 L
.4544 .05598 L
.4425 .03832 L
.43316 .02447 L
.42615 .01406 L
.4208 .00614 L
.41772 .00157 L
.41693 .00039 L
.41673 9e-05 L
.41668 1e-05 L
.41667 1e-05 L
.41667 0 L
.41667 0 L
.41667 0 L
.41667 0 L
.41669 4e-05 L
.41676 .00013 L
.417 .0005 L
.41782 .00171 L
.421 .00643 L
.42599 .01383 L
.43333 .02472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{401.938, 248.438},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006A0000n1000`40n007h00OooooT_oo003ooonB
ool00<?oo`8000Coo`03003ooooo00;oo`@009ooo`8000Coo`03003ooooo00;oo`@0017oo`00`_oo
00@00?ooool000[oo`03003ooooo09goo`04003ooooo0008ool00`00ooooo`0Bool00<;oo`04003o
oooo000:ool00`00ooooo`2Mool01000ooooo`002Ooo00<00?ooool04Ooo0032ool01000ooooo`00
2_oo00<00?ooool0WOoo00@00?ooool000[oo`03003ooooo013oo`00`_oo00@00?ooool000[oo`03
003ooooo09goo`04003ooooo000;ool00`00ooooo`0?ool00<;oo`04003ooooo0008ool3002Oool0
1000ooooo`002?oo00@00?ooool0017oo`00`ooo0P002ooo00<00?ooool0W_oo0P002_oo0P004_oo
003ooonBool00?oooi;oo`00ooooT_oo003ooon5ool3000:ool001coool005h000koo`03003ooooo
00Ooo`007?oo00<00?ooool0SOoo0P007?oo00<00?ooool0G?oo20001_oo00<00?ooool0?_oo00<0
0?ooool02_oo0P002_oo000Lool00`00ooooo`2>ool2000Kool00`00ooooo`1>ool8001Eool00`00
ooooo`09ool00`00ooooo`0:ool001coo`03003ooooo08ooo`03003ooooo01Woo`03003ooooo04Ko
o`8006?oo`03003ooooo00[oo`<000Woo`007?oo00<00?ooool0T?oo00<00?ooool0Fooo1P00O_oo
000Lool00`00ooooo`2Aool00`00ooooo`1Bool2002:ool001coo`03003ooooo09;oo`03003ooooo
04goo`@008coo`007?oo00<00?ooool0T_oo0P00C?oo0P00T?oo000Lool00`00ooooo`2Cool00`00
ooooo`3Lool001coo`03003ooooo09Coo`03003ooooo047oo`<009Ooo`007?oo00<00?ooool0UOoo
00<00?ooool0??oo1000V_oo000Lool00`00ooooo`2Eool00`00ooooo`0kool00`00ooooo`2Lool0
01coo`03003ooooo09Koo`03003ooooo0=Woo`007?oo00<00?ooool0U_oo00<00?ooool0<_oo0P00
YOoo000Lool00`00ooooo`2Gool00`00ooooo`0_ool2002Wool001coo`03003ooooo09Soo`03003o
oooo02coo`800:Woo`007?oo00<00?ooool0V?oo00<00?ooool0:ooo00<00?ooool0ZOoo000Lool0
0`00ooooo`2Iool00`00ooooo`3Fool001coo`03003ooooo09[oo`03003ooooo0=Goo`007?oo00<0
0?ooool0V_oo00<00?ooool08_oo0P00/Ooo000Lool00`00ooooo`2Kool00`00ooooo`0Pool00`00
ooooo`2aool001coo`03003ooooo09coo`03003ooooo01goo`800;Coo`007?oo00<00?ooool0W?oo
0P007Ooo00<00?ooool0]?oo000Lool00`00ooooo`2Mool00`00ooooo`0Jool00`00ooooo`2eool0
01coo`03003ooooo09koo`03003ooooo0=7oo`007?oo00<00?ooool0W_oo0P00d_oo000Lool00`00
ooooo`2Oool00`00ooooo`0Bool00`00ooooo`2kool001coo`03003ooooo0:3oo`03003ooooo00oo
o`800;koo`007?oo00<00?ooool0XOoo00<00?ooool03Ooo00<00?ooool0__oo000Lool00`00oooo
o`2Qool00`00ooooo`0<ool00`00ooooo`2oool001coo`03003ooooo0:;oo`03003ooooo00[oo`03
003ooooo0<3oo`007?oo00<00?ooool0X_oo0P002_oo00<00?ooool0`Ooo000Lool00`00ooooo`2S
ool00`00ooooo`3<ool001coo`03003ooooo0:Coo`03003ooooo0<_oo`007?oo00<00?ooool0Y?oo
00<00?ooool0booo000Lool00`00ooooo`2Uool01@00oooooooo0000b?oo000Lool00`00ooooo`2V
ool00`00ool00039ool001coo`03003ooooo0:Koo`800<[oo`007?oo00<00?ooool0Y_oo0P00b_oo
000Lool00`00ooooo`2Uool01000ooooo`00bOoo000Lool00`00ooooo`2Tool01@00oooooooo0000
bOoo0007ool20004ool00`00ooooo`02ool40006ool00`00ooooo`2Yool00`00ooooo`36ool000Ko
o`04003ooooo000:ool00`00ooooo`05ool00`00ooooo`2Zool00`00ooooo`35ool000Koo`04003o
oooo000:ool00`00ooooo`05ool3002Cool2000Eool00`00ooooo`35ool000Koo`04003ooooo000:
ool00`00ooooo`05ool00`00ooooo`2Aool7000Cool00`00ooooo`34ool000Koo`04003ooooo000:
ool00`00ooooo`05ool00`00ooooo`2@ool00`00ooooo`05ool2000Bool00`00ooooo`33ool000Ko
o`04003ooooo0008ool30007ool00`00ooooo`2@ool00`00ooooo`07ool00`00ooooo`03ool00`00
ooooo`09ool20034ool000Ooo`8000_oo`03003ooooo00Goo`03003ooooo08ooo`03003ooooo00Wo
o`8000;oo`03003ooooo00_oo`03003ooooo0<;oo`007?oo00<00?ooool0S_oo00<00?ooool03?oo
0P003ooo00<00?ooool0`Ooo000Lool00`00ooooo`2>ool00`00ooooo`0<ool01000ool000003Ooo
0P00`_oo000Lool00`00ooooo`2>ool00`00ooooo`0<ool01@00oooooooo00003Ooo00<00?ooool0
`?oo000Lool00`00ooooo`2>ool00`00ooooo`0;ool00`00ooooo`03ool00`00ooooo`0;ool00`00
ooooo`2oool001coo`03003ooooo08koo`03003ooooo01;oo`03003ooooo00[oo`800<3oo`007?oo
00<00?ooool0S_oo00<00?ooool04ooo00<00?ooool02_oo00<00?ooool0__oo000Lool00`00oooo
o`2>ool00`00ooooo`0Dool00`00ooooo`0:ool00`00ooooo`2mool001coo`03003ooooo08koo`03
003ooooo02;oo`03003ooooo0;coo`007?oo00<00?ooool0S_oo00<00?ooool08_oo0P00_Ooo000L
ool00`00ooooo`2>ool00`00ooooo`0Sool00`00ooooo`2kool001coo`03003ooooo08koo`03003o
oooo00Soo`03003ooooo01Woo`03003ooooo0;[oo`007?oo00<00?ooool0S_oo00<00?ooool01ooo
00<00?ooool06ooo00<00?ooool0^Ooo000Lool00`00ooooo`2>ool00`00ooooo`07ool00`00oooo
o`0Kool00`00ooooo`2iool001coo`03003ooooo08koo`03003ooooo00Koo`03003ooooo01goo`03
003ooooo0;Soo`007?oo00<00?ooool0S_oo00<00?ooool01Ooo00<00?ooool07ooo00<00?ooool0
]ooo000Lool00`00ooooo`2>ool00`00ooooo`05ool00`00ooooo`0Oool2002hool001coo`03003o
oooo08koo`03003ooooo00Goo`03003ooooo023oo`03003ooooo0;Koo`007?oo00<00?ooool0Sooo
00<00?ooool0:?oo00<00?ooool0]Ooo000Lool00`00ooooo`2?ool00`00ooooo`0Yool00`00oooo
o`2dool001coo`03003ooooo08ooo`03003ooooo02Woo`03003ooooo0;Coo`007?oo00<00?ooool0
Sooo00<00?ooool0:_oo00<00?ooool0/ooo000Lool00`00ooooo`2?ool00`00ooooo`0[ool00`00
ooooo`2bool001coo`03003ooooo093oo`04003ooooo000Yool00`00ooooo`2bool001coo`03003o
oooo093oo`04003ooooo000Zool00`00ooooo`2aool001coo`03003ooooo093oo`03003oo`0002co
o`03003ooooo0;3oo`007?oo00<00?ooool0T?oo00<00?oo0000;?oo00<00?ooool0/?oo000Lool0
0`00ooooo`2Aool2000]ool00`00ooooo`2_ool001coo`03003ooooo097oo`03003ooooo02goo`03
003ooooo0:koo`007?oo00<00?ooool0TOoo00<00?ooool0;Ooo00<00?ooool0[_oo000Lool00`00
ooooo`2Aool00`00ooooo`0^ool00`00ooooo`2]ool001coo`03003ooooo09;oo`03003ooooo02ko
o`03003ooooo0:coo`007?oo00<00?ooool0T_oo00<00?ooool0;_oo00<00?ooool0[?oo000Lool0
0`00ooooo`2Bool00`00ooooo`0_ool00`00ooooo`2[ool001coo`03003ooooo09;oo`03003ooooo
02ooo`800:coo`007?oo00<00?ooool0Tooo00<00?ooool0;ooo00<00?ooool0Z_oo000Lool00`00
ooooo`2Cool00`00ooooo`0`ool00`00ooooo`2Yool001coo`03003ooooo093oo`04003ooooo000b
ool2002Zool000Ooo`8000Coo`03003ooooo00;oo`@000Koo`03003ooooo093oo`04003ooooo000c
ool00`00ooooo`2Xool000Koo`04003ooooo0008ool00`00ooooo`07ool00`00ooooo`2@ool01@00
oooooooo0000<ooo00<00?ooool0Yooo0006ool01000ooooo`002Ooo00<00?ooool01_oo0`00Sooo
00<00?ooool00_oo00<00?ooool0<Ooo0P00Z?oo0006ool01000ooooo`002_oo00<00?ooool01Ooo
00<00?ooool0Sooo00<00?ooool00_oo00<00?ooool0<_oo00<00?ooool0Y_oo0006ool01000oooo
o`002ooo00<00?ooool01?oo00<00?ooool0Sooo00<00?ooool00_oo00<00?ooool0<ooo00<00?oo
ool0YOoo0006ool01000ooooo`002?oo00@00?ooool000Koo`03003ooooo08ooo`03003ooooo00?o
o`03003ooooo03;oo`03003ooooo0:Goo`001ooo0P002_oo0P001ooo00<00?ooool0Sooo00<00?oo
ool00ooo00<00?ooool0<ooo00<00?ooool0Y?oo000Lool00`00ooooo`2Eool00`00ooooo`0dool0
0`00ooooo`2Sool001coo`03003ooooo09Goo`03003ooooo03Coo`03003ooooo0:?oo`007?oo00<0
0?ooool0U_oo00<00?ooool0=?oo00<00?ooool0X_oo000Lool00`00ooooo`2Fool00`00ooooo`0e
ool00`00ooooo`2Qool001coo`03003ooooo09Koo`03003ooooo03Goo`03003ooooo0:7oo`007?oo
00<00?ooool0U_oo00<00?ooool0=_oo00<00?ooool0X?oo000Lool00`00ooooo`2>ool00`00oooo
o`06ool00`00ooooo`0fool00`00ooooo`2Oool001coo`03003ooooo08koo`03003ooooo00Koo`03
003ooooo03Koo`03003ooooo09ooo`007?oo00<00?ooool0S_oo00<00?ooool01_oo00<00?ooool0
=ooo00<00?ooool0W_oo000Lool00`00ooooo`2>ool00`00ooooo`07ool00`00ooooo`0gool00`00
ooooo`2Mool001coo`03003ooooo08koo`03003ooooo00Ooo`03003ooooo03Ooo`03003ooooo09go
o`007?oo00<00?ooool0S_oo00<00?ooool01ooo00<00?ooool0>?oo00<00?ooool0W?oo000Lool0
0`00ooooo`2>ool00`00ooooo`08ool00`00ooooo`0hool00`00ooooo`2Kool001coo`03003ooooo
08koo`03003ooooo00Soo`03003ooooo03Soo`03003ooooo09_oo`007?oo00<00?ooool0VOoo00<0
0?ooool0>Ooo00<00?ooool0V_oo000Lool00`00ooooo`2Jool00`00ooooo`0iool00`00ooooo`2I
ool001coo`03003ooooo09[oo`03003ooooo03Woo`03003ooooo09Woo`007?oo00<00?ooool0V_oo
00<00?ooool0>_oo00<00?ooool0V?oo000Lool00`00ooooo`2Kool00`00ooooo`0jool00`00oooo
o`2Gool001coo`03003ooooo08koo`03003ooooo00[oo`03003ooooo03[oo`03003ooooo09Ooo`00
7?oo00<00?ooool0S_oo00<00?ooool02_oo00<00?ooool0>ooo00<00?ooool0U_oo000Lool00`00
ooooo`2>ool00`00ooooo`0;ool00`00ooooo`0kool00`00ooooo`2Eool001coo`03003ooooo08ko
o`03003ooooo00_oo`03003ooooo03_oo`03003ooooo09Goo`007?oo00<00?ooool0S_oo00<00?oo
ool02ooo00<00?ooool0??oo00<00?ooool0U?oo000Lool00`00ooooo`2>ool00`00ooooo`0<ool0
0`00ooooo`0lool00`00ooooo`2Cool001coo`03003ooooo08koo`03003ooooo00coo`03003ooooo
03coo`03003ooooo09?oo`007?oo00<00?ooool0S_oo00<00?ooool03?oo00<00?ooool0?Ooo00<0
0?ooool0T_oo000Lool00`00ooooo`2Nool00`00ooooo`0mool00`00ooooo`2Aool001coo`03003o
oooo09koo`03003ooooo03goo`03003ooooo097oo`007?oo00<00?ooool0W_oo00<00?ooool0?_oo
00<00?ooool0T?oo000Lool00`00ooooo`2Oool00`00ooooo`0nool00`00ooooo`2?ool001coo`03
003ooooo09ooo`03003ooooo03koo`80093oo`007?oo00<00?ooool0Wooo00<00?ooool0?ooo00<0
0?ooool0S_oo000Lool00`00ooooo`2?ool00`00ooooo`0>ool00`00ooooo`0oool00`00ooooo`2=
ool001coo`03003ooooo08ooo`03003ooooo00koo`03003ooooo03ooo`8008koo`007?oo00<00?oo
ool0Sooo00<00?ooool03_oo00<00?ooool0@?oo00<00?ooool0S?oo0007ool20004ool00`00oooo
o`03ool20007ool00`00ooooo`2?ool00`00ooooo`0?ool00`00ooooo`10ool00`00ooooo`2;ool0
00Koo`04003ooooo0008ool01000ooooo`001_oo00<00?ooool0Sooo00<00?ooool03ooo00<00?oo
ool0@Ooo00<00?ooool0R_oo0006ool01000ooooo`002ooo00<00?ooool01?oo0`00Sooo00<00?oo
ool03ooo00<00?ooool0@Ooo00<00?ooool0R_oo0006ool01000ooooo`002Ooo0P001ooo00<00?oo
ool0Sooo00<00?ooool04?oo00<00?ooool0@Ooo00<00?ooool0ROoo0006ool01000ooooo`002ooo
00<00?ooool01?oo00<00?ooool0Sooo00<00?ooool04?oo00<00?ooool0@_oo00<00?ooool0R?oo
0006ool01000ooooo`002?oo00@00?ooool000Koo`03003ooooo0:;oo`03003ooooo04;oo`8008Wo
o`001ooo0P002_oo0P001ooo00<00?ooool0Xooo00<00?ooool0@_oo00<00?ooool0Qooo000Lool0
0`00ooooo`2Sool00`00ooooo`13ool00`00ooooo`26ool001coo`03003ooooo0:?oo`03003ooooo
04?oo`8008Ooo`007?oo00<00?ooool0Y?oo00<00?ooool0@ooo00<00?ooool0QOoo000Lool00`00
ooooo`2Tool00`00ooooo`14ool00`00ooooo`24ool001coo`03003ooooo093oo`03003ooooo017o
o`03003ooooo04Goo`03003ooooo08?oo`007?oo00<00?ooool0T?oo00<00?ooool04_oo00<00?oo
ool0A?oo00<00?ooool0Pooo000Lool00`00ooooo`2@ool00`00ooooo`0Bool00`00ooooo`15ool0
0`00ooooo`22ool001coo`03003ooooo097oo`03003ooooo017oo`03003ooooo04Goo`8008?oo`00
7?oo00<00?ooool0TOoo00<00?ooool04_oo00<00?ooool0AOoo00<00?ooool0POoo000Lool00`00
ooooo`2Aool00`00ooooo`0Bool00`00ooooo`16ool00`00ooooo`20ool001coo`03003ooooo09;o
o`03003ooooo017oo`03003ooooo04Koo`80087oo`007?oo00<00?ooool0T_oo00<00?ooool04_oo
00<00?ooool0A_oo00<00?ooool0Oooo000Lool00`00ooooo`2Wool00`00ooooo`17ool00`00oooo
o`1nool001coo`03003ooooo0:Soo`03003ooooo04Koo`8007ooo`007?oo00<00?ooool0Z?oo00<0
0?ooool0Aooo00<00?ooool0OOoo000Lool00`00ooooo`2Xool00`00ooooo`18ool00`00ooooo`1l
ool001coo`03003ooooo0:Woo`03003ooooo04Ooo`8007goo`007?oo00<00?ooool0ZOoo00<00?oo
ool0B?oo00<00?ooool0Nooo000Lool00`00ooooo`2Cool00`00ooooo`0Cool00`00ooooo`19ool0
0`00ooooo`1jool001coo`03003ooooo09?oo`03003ooooo01Coo`03003ooooo04Soo`03003ooooo
07[oo`007?oo00<00?ooool0U?oo00<00?ooool04ooo00<00?ooool0BOoo00<00?ooool0NOoo000L
ool00`00ooooo`2Dool00`00ooooo`0Dool00`00ooooo`19ool00`00ooooo`1hool001coo`03003o
oooo09Coo`03003ooooo01Coo`03003ooooo04Woo`03003ooooo07Soo`007?oo00<00?ooool0UOoo
00<00?ooool04ooo00<00?ooool0B_oo00<00?ooool0Mooo000Lool00`00ooooo`2Eool00`00oooo
o`0Dool00`00ooooo`1:ool00`00ooooo`1fool001coo`03003ooooo0:coo`03003ooooo04[oo`80
07Ooo`007?oo00<00?ooool0[Ooo00<00?ooool0B_oo00<00?ooool0MOoo000Lool00`00ooooo`2]
ool00`00ooooo`1;ool00`00ooooo`1dool001coo`03003ooooo0:goo`03003ooooo04_oo`8007Go
o`007?oo00<00?ooool0[_oo00<00?ooool0Booo00<00?ooool0Looo000Lool00`00ooooo`2^ool0
0`00ooooo`1<ool00`00ooooo`1bool001coo`03003ooooo09Koo`03003ooooo01Koo`03003ooooo
04_oo`8007?oo`007?oo00<00?ooool0U_oo00<00?ooool05_oo00<00?ooool0C?oo00<00?ooool0
LOoo000Lool00`00ooooo`2Gool00`00ooooo`0Eool00`00ooooo`1=ool00`00ooooo`1`ool001co
o`03003ooooo09Ooo`03003ooooo01Koo`03003ooooo04coo`80077oo`007?oo00<00?ooool0Uooo
00<00?ooool05_oo00<00?ooool0COoo00<00?ooool0Kooo0007ool20004ool00`00ooooo`04ool0
0`00ooooo`05ool00`00ooooo`2Hool00`00ooooo`0Fool00`00ooooo`1=ool00`00ooooo`1^ool0
00Koo`04003ooooo000:ool00`00ooooo`05ool00`00ooooo`2Hool00`00ooooo`0Fool00`00oooo
o`1>ool00`00ooooo`1]ool000Koo`04003ooooo0007ool50006ool3002Hool00`00ooooo`0Fool0
0`00ooooo`1>ool00`00ooooo`1]ool000Koo`04003ooooo0007ool01000ooooo`001ooo00<00?oo
ool0/_oo00<00?ooool0C_oo00<00?ooool0K?oo0006ool01000ooooo`002?oo00<00?oo00001ooo
00<00?ooool0/_oo00<00?ooool0Cooo00<00?ooool0Jooo0006ool01000ooooo`002Ooo0P001ooo
00<00?ooool0/ooo00<00?ooool0C_oo00<00?ooool0Jooo0007ool2000;ool00`00ooooo`05ool0
0`00ooooo`2cool00`00ooooo`1?ool00`00ooooo`1Zool001coo`03003ooooo0;?oo`03003ooooo
053oo`03003ooooo06Woo`007?oo00<00?ooool0]?oo00<00?ooool0Cooo00<00?ooool0JOoo000L
ool00`00ooooo`2Jool00`00ooooo`0Gool00`00ooooo`1@ool00`00ooooo`1Xool001coo`03003o
oooo09[oo`03003ooooo01Soo`03003ooooo053oo`03003ooooo06Ooo`007?oo00<00?ooool0Vooo
00<00?ooool05ooo00<00?ooool0D?oo00<00?ooool0Iooo000Lool00`00ooooo`2Kool00`00oooo
o`0Gool00`00ooooo`1Aool00`00ooooo`1Vool001coo`03003ooooo09_oo`03003ooooo01Soo`03
003ooooo057oo`03003ooooo06Goo`007?oo00<00?ooool0W?oo00<00?ooool05ooo00<00?ooool0
DOoo00<00?ooool0IOoo000Lool00`00ooooo`2Lool00`00ooooo`0Gool00`00ooooo`1Bool00`00
ooooo`1Tool001coo`03003ooooo0;Ooo`03003ooooo05;oo`03003ooooo06?oo`007?oo00<00?oo
ool0]ooo00<00?ooool0D_oo00<00?ooool0Hooo000Lool00`00ooooo`2gool00`00ooooo`1Cool0
0`00ooooo`1Rool001coo`03003ooooo0;Soo`03003ooooo05?oo`03003ooooo067oo`007?oo00<0
0?ooool0^?oo00<00?ooool0Dooo00<00?ooool0HOoo000Lool00`00ooooo`2iool00`00ooooo`1C
ool00`00ooooo`1Pool001coo`03003ooooo09koo`03003ooooo01Soo`03003ooooo05Coo`03003o
oooo05ooo`007?oo00<00?ooool0W_oo00<00?ooool06?oo00<00?ooool0E?oo00<00?ooool0Gooo
000Lool00`00ooooo`2Oool00`00ooooo`0Hool00`00ooooo`1Dool00`00ooooo`1Nool001coo`03
003ooooo09ooo`03003ooooo01Soo`03003ooooo05Goo`03003ooooo05goo`007?oo00<00?ooool0
Wooo00<00?ooool06?oo00<00?ooool0EOoo00<00?ooool0GOoo000Lool00`00ooooo`2Pool00`00
ooooo`0Hool00`00ooooo`1Eool00`00ooooo`1Lool001coo`03003ooooo0:3oo`03003ooooo01So
o`03003ooooo05Koo`03003ooooo05_oo`007?oo00<00?ooool0XOoo00<00?ooool05ooo00<00?oo
ool0E_oo00<00?ooool0Fooo000Lool00`00ooooo`2Qool00`00ooooo`0Hool00`00ooooo`1Fool0
0`00ooooo`1Jool001coo`03003ooooo0:;oo`03003ooooo01Ooo`03003ooooo05Ooo`03003ooooo
05Woo`007?oo00<00?ooool0_Ooo00<00?ooool0E_oo00<00?ooool0FOoo000Lool00`00ooooo`2m
ool00`00ooooo`1Gool00`00ooooo`1Hool001coo`03003ooooo0;goo`03003ooooo05Soo`03003o
oooo05Ooo`007?oo00<00?ooool0__oo00<00?ooool0Eooo00<00?ooool0Eooo000Lool00`00oooo
o`2nool00`00ooooo`1Hool00`00ooooo`1Fool001coo`03003ooooo0;ooo`03003ooooo05Soo`03
003ooooo05Goo`007?oo00<00?ooool0Y?oo00<00?ooool06?oo00<00?ooool0F?oo00<00?ooool0
EOoo000Lool00`00ooooo`2Tool00`00ooooo`0Iool00`00ooooo`1Hool00`00ooooo`1Dool001co
o`03003ooooo0:Goo`03003ooooo01Soo`03003ooooo05Woo`03003ooooo05?oo`007?oo00<00?oo
ool0YOoo00<00?ooool06?oo00<00?ooool0FOoo00<00?ooool0Dooo000Lool00`00ooooo`2Vool0
0`00ooooo`0Hool00`00ooooo`1Iool00`00ooooo`1Bool000Ooo`8000Coo`03003ooooo00;oo`<0
00Ooo`03003ooooo0:Koo`03003ooooo01Soo`03003ooooo05[oo`03003ooooo057oo`001_oo00@0
0?ooool000_oo`03003ooooo00Coo`03003ooooo0:Ooo`03003ooooo01Soo`03003ooooo05Woo`03
003ooooo057oo`001_oo00@00?ooool000_oo`03003ooooo00Coo`<00<;oo`03003ooooo05[oo`03
003ooooo053oo`001_oo00@00?ooool000Soo`<00?oooggoo`001_oo00@00?ooool000Soo`03003o
oooo0?oooggoo`001_oo00@00?ooool000Soo`03003ooooo0?oooggoo`001ooo0P002Ooo1000oooo
O?oo003ooonBool00?oooi;oo`00ooooT_oo003ooonBool00?oooi;oo`00ooooT_oo003ooonBool0
01[oo`03003ooooo0?ooogGoo`006Ooo00<00?oo0000ooooM_oo000Iool00`00ool0003ooomfool0
01Soo`05003oooooool0003ooomeool001Ooo`8000;oo`800?ooogGoo`007Ooo00<00?ooool0oooo
L_oo000Nool00`00ooooo`3ooomaool00001\
\>"],
  ImageRangeCache->{{{122.5, 443}, {643.562, 445.812}} -> {-0.104119, \
1.26413, 0.00057456, 0.00232414}}]
}, Open  ]],

Cell[TextData[{
  "The solid thin curve shows a procedure with ",
  Cell[BoxData[
      \(TraditionalForm\`\(g\_1\%\[Prime]\)(
          m) = \[Alpha]\ m, \[Alpha] \[Element] \([0.3, 0.8]\)\)]],
  ".\nThe dashed curve shows a procedure with ",
  Cell[BoxData[
      \(TraditionalForm\`\(g\_1\%\[Prime]\)(m) = 
        m, \[Alpha] \[Element] \([0, 1]\)\)]],
  ".\nThe thick curve shows a procedure with ",
  Cell[BoxData[
      \(TraditionalForm\`\(g\_1\%\[Prime]\)(
          m) = \[Alpha], \[Alpha] \[Element] \([0, 0.6]\)\)]],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot of ",
  Cell[BoxData[
      FormBox["s", TraditionalForm]]],
  "-",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["v", "\[Prime]"], TraditionalForm]]],
  "-curve for guess with deviation ",
  Cell[BoxData[
      FormBox["0", TraditionalForm]]]
}], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"g1p", "[", 
          RowBox[{"m_", ",", "\[Alpha]_"}], "]"}], ":=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"m", "\[Equal]", "0"}], ",", "\[Alpha]", ",", 
            RowBox[{"1", "-", "\[Alpha]"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"ParametricPlot", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"s", "[", "\[Alpha]", "]"}], ",", 
              RowBox[{"vp", "[", 
                RowBox[{"\[Alpha]", ",", 
                  RowBox[{
                    RowBox[{"g1p", "[", 
                      RowBox[{"#", ",", "\[Alpha]"}], "]"}], "&"}]}], "]"}]}],
             "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"\[Alpha]", ",", "0", ",", "1"}], "}"}], ",", 
          RowBox[{"PlotStyle", "\[Rule]", 
            RowBox[{"Thickness", "[", "0.01", "]"}]}], ",", 
          RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"0", ",", "0.2"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", "0.5"}], "}"}]}], "}"}]}], ",", 
          RowBox[{"Ticks", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"ticks", ",", "ticks"}], "}"}]}], ",", 
          RowBox[{"AxesLabel", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"\"\<s\>\"", ",", "\"\<\!\(v\^\[Prime]\)\>\""}], 
              "}"}]}]}], "]"}], ";"}]}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 5 0 1.23607 [
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[1 -0.0125 -9 -9 ]
[1 -0.0125 9 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 10 4.90625 ]
[-0.0125 .12361 -18 -4.5 ]
[-0.0125 .12361 0 4.5 ]
[-0.0125 .24721 -18 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .37082 -18 -4.5 ]
[-0.0125 .37082 0 4.5 ]
[-0.0125 .49443 -18 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .61803 -18 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[0 .64303 -6.84375 0 ]
[0 .64303 6.84375 10 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.5 0 m
.5 .00625 L
s
[(0.1)] .5 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(0.2)] 1 -0.0125 0 1 Mshowa
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
69.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .12361 m
.00625 .12361 L
s
[(0.1)] -0.0125 .12361 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(0.2)] -0.0125 .24721 1 0 Mshowa
0 .37082 m
.00625 .37082 L
s
[(0.3)] -0.0125 .37082 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(0.4)] -0.0125 .49443 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(0.5)] -0.0125 .61803 1 0 Mshowa
0 0 m
0 .61803 L
s
gsave
0 .64303 -67.8438 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 18 translate 1 -1 scale
63.000 11.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(v) show
69.000 7.625 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(\\242) show
72.688 11.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.01 w
.83333 0 m
.83128 0 L
s
.83128 0 m
.83005 0 L
s
.82619 0 m
.82524 0 L
s
.82524 0 m
.82339 0 L
s
.80852 0 m
.80764 0 L
s
.80764 0 m
.80663 0 L
s
.80093 0 m
.79897 0 L
.79676 0 L
s
.79676 0 m
.79624 0 L
s
.79215 0 m
.79212 0 L
s
.79212 0 m
.79208 0 L
s
.78882 0 m
.78809 0 L
s
.78809 0 m
.78752 0 L
s
.78633 0 m
.78588 0 L
s
.78588 0 m
.78511 0 L
s
.78303 0 m
.78194 0 L
s
.78194 0 m
.7811 0 L
s
.77487 0 m
.77394 0 L
.77225 0 L
.77041 0 L
s
.77041 0 m
.76999 0 L
s
.76803 0 m
.76742 0 L
s
.76742 0 m
.76644 0 L
s
.76343 0 m
.76224 0 L
.76041 0 L
.75843 0 L
.75731 0 L
s
.75731 0 m
.7568 0 L
s
.75397 0 m
.753 0 L
.7519 0 L
s
.7519 0 m
.75114 0 L
s
.74881 0 m
.74813 0 L
.74631 0 L
s
.74631 0 m
.74545 0 L
s
.74231 0 m
.74162 0 L
s
.74162 0 m
.74144 0 L
s
.73841 0 m
.73685 0 L
.73577 0 L
.73475 0 L
s
.73475 0 m
.73323 0 L
s
.7313 0 m
.73079 0 L
.72971 0 L
.72857 0 L
.72669 0 L
.72493 0 L
.72301 0 L
s
.72301 0 m
.72199 0 L
s
.72189 0 m
.72094 0 L
.71994 0 L
.71887 0 L
s
.71887 0 m
.71782 0 L
s
.71555 0 m
.71467 0 L
s
.71467 0 m
.71425 0 L
s
.71312 0 m
.71261 0 L
s
.71261 0 m
.71219 0 L
s
.70634 0 m
.7061 0 L
s
.7061 0 m
.70554 0 L
s
.70464 0 m
.70397 0 L
.70232 0 L
.70055 0 L
s
.70055 0 m
.69998 0 L
s
.69668 0 m
.69567 0 L
.69383 0 L
s
.69383 0 m
.69363 0 L
s
.69242 0 m
.69192 0 L
.69014 0 L
s
.69014 0 m
.68932 0 L
s
.68875 0 m
.68827 0 L
s
.68827 0 m
.68756 0 L
s
.68711 0 m
.68624 0 L
s
.68624 0 m
.68559 0 L
s
.68304 0 m
.68258 0 L
.68087 0 L
s
.68087 0 m
.67966 0 L
s
.67576 0 m
.67435 0 L
s
.67435 0 m
.67307 0 L
s
.67252 0 m
.67131 0 L
.66995 0 L
s
.66995 0 m
.6696 0 L
s
.66141 0 m
.66118 0 L
.66033 0 L
s
.66033 0 m
.6596 0 L
s
.6563 0 m
.6556 0 L
s
.6556 0 m
.65486 0 L
s
.65012 0 m
.64952 0 L
.6479 0 L
.6462 0 L
s
.6462 0 m
.64559 0 L
s
.64348 0 m
.64317 0 L
s
.64317 0 m
.6428 0 L
s
.64089 0 m
.64031 0 L
.63887 0 L
.6373 0 L
s
.6373 0 m
.63659 0 L
s
.63307 0 m
.63271 0 L
.63106 0 L
.63014 0 L
.6293 0 L
.62768 0 L
s
.62768 0 m
.62702 0 L
s
.62554 0 m
.62471 0 L
.62342 0 L
.62193 0 L
.62054 0 L
.61925 0 L
s
.61925 0 m
.619 0 L
s
.61405 0 m
.61259 0 L
.61133 0 L
.60996 0 L
.60913 0 L
.60837 0 L
.60686 0 L
.6055 0 L
s
.6055 0 m
.60534 0 L
s
.60161 0 m
.60112 0 L
s
.60112 0 m
.60059 0 L
s
.59909 0 m
.59835 0 L
s
.59835 0 m
.59749 0 L
s
.5965 0 m
.59584 0 L
s
.59584 0 m
.59493 0 L
s
.5931 0 m
.5918 0 L
.59051 0 L
s
.59051 0 m
.58984 0 L
s
.589 0 m
.5877 0 L
.58622 0 L
s
.58622 0 m
.58503 0 L
s
.58015 0 m
.57927 0 L
.57794 0 L
s
.57794 0 m
.57766 0 L
s
.57692 0 m
.57648 0 L
.57522 0 L
.57406 0 L
.5728 0 L
.57147 0 L
s
.57147 0 m
.57103 0 L
s
.56992 0 m
.56918 0 L
.56807 0 L
s
.56807 0 m
.56773 0 L
s
.56524 0 m
.56461 0 L
s
.56461 0 m
.56358 0 L
s
.56286 0 m
.5623 0 L
s
.5623 0 m
.56214 0 L
s
.56016 0 m
.55979 0 L
s
.55979 0 m
.55943 0 L
s
.55799 0 m
.55751 0 L
s
.55751 0 m
.55705 0 L
s
.55588 0 m
.55552 0 L
s
.55552 0 m
.55517 0 L
s
.55339 0 m
.55245 0 L
s
.55245 0 m
.55196 0 L
s
.54715 0 m
.54698 0 L
.54575 0 L
.5446 0 L
s
.5446 0 m
.54413 0 L
s
.54286 0 m
.54231 0 L
.54169 0 L
.54101 0 L
.53981 0 L
s
.53981 0 m
.53897 0 L
s
.53851 0 m
.53805 0 L
.53738 0 L
s
.53738 0 m
.53666 0 L
s
.5337 0 m
.53352 0 L
.5323 0 L
.53117 0 L
s
.53117 0 m
.53038 0 L
s
.52644 0 m
.52619 0 L
s
.52619 0 m
.52596 0 L
s
.5224 0 m
.52214 0 L
.52099 0 L
.52035 0 L
.51976 0 L
s
.51976 0 m
.51911 0 L
s
.51526 0 m
.51497 0 L
.51434 0 L
.51374 0 L
s
.51374 0 m
.51337 0 L
s
.51155 0 m
.51106 0 L
s
.51106 0 m
.51074 0 L
s
.5077 0 m
.50753 0 L
.50692 0 L
s
.50692 0 m
.50657 0 L
s
.50614 0 m
.50584 0 L
s
.50584 0 m
.50556 0 L
s
.50371 0 m
.50311 0 L
s
.50311 0 m
.50294 0 L
s
.50204 0 m
.50103 0 L
s
.50103 0 m
.50065 0 L
s
.49538 0 m
0 0 rlineto s
.49222 0 m
0 0 rlineto s
.4871 0 m
.48671 0 L
.48622 0 L
.48532 0 L
.4844 0 L
.48387 0 L
s
.48166 0 m
.48158 0 L
.48076 0 L
s
.48076 0 m
.48027 0 L
s
.47958 0 m
.47904 0 L
.47824 0 L
s
.47824 0 m
.47786 0 L
s
.4763 0 m
.47607 0 L
.4753 0 L
.47457 0 L
.47378 0 L
s
.47177 0 m
0 0 rlineto s
.47063 0 m
.4704 0 L
.46962 0 L
.46886 0 L
s
.46622 0 m
.46595 0 L
s
.46595 0 m
.46552 0 L
s
.46441 0 m
.46392 0 L
.46316 0 L
s
.46316 0 m
.46273 0 L
s
.4618 0 m
.4612 0 L
.46055 0 L
s
.46055 0 m
.46041 0 L
s
.45858 0 m
.45846 0 L
.45778 0 L
s
.45778 0 m
.45753 0 L
s
.45482 0 m
.45463 0 L
s
.45463 0 m
.45449 0 L
s
.45343 0 m
.45299 0 L
.45242 0 L
.45179 0 L
s
.45179 0 m
.45156 0 L
s
.45133 0 m
.45112 0 L
s
.45112 0 m
.45094 0 L
s
.44962 0 m
.44945 0 L
.44914 0 L
s
.4484 0 m
.44817 0 L
s
.44817 0 m
.44803 0 L
s
.44728 0 m
.4471 0 L
s
.4471 0 m
.44695 0 L
s
.44645 0 m
0 0 rlineto s
.44285 0 m
.4427 0 L
.44239 0 L
.44185 0 L
.44135 0 L
.44081 0 L
.44036 0 L
.43986 0 L
.43788 0 L
.43762 0 L
s
.4372 0 m
.43706 0 L
.43679 0 L
s
.43679 0 m
.43663 0 L
s
.43458 0 m
.43447 0 L
.43408 0 L
s
.43408 0 m
.43389 0 L
s
.43293 0 m
.43259 0 L
.43221 0 L
s
.43125 0 m
.43105 0 L
.43065 0 L
s
.42994 0 m
.42984 0 L
.4295 0 L
s
.42904 0 m
.42879 0 L
s
.42879 0 m
.42857 0 L
s
.42816 0 m
.42804 0 L
.42767 0 L
s
.42767 0 m
.42758 0 L
s
.42738 0 m
.42728 0 L
.42692 0 L
s
.4264 0 m
.4263 0 L
s
.4263 0 m
.4262 0 L
s
.4258 0 m
.4257 0 L
.42544 0 L
s
.42544 0 m
.42529 0 L
s
.42425 0 m
.42395 0 L
s
.42395 0 m
.42383 0 L
s
.4237 0 m
.42362 0 L
.42334 0 L
.42306 0 L
s
.42257 0 m
.42232 0 L
.42206 0 L
.42193 0 L
s
.42193 0 m
.42187 0 L
s
.42167 0 m
.42164 0 L
s
.42164 0 m
.42159 0 L
s
.42108 0 m
.421 0 L
s
.421 0 m
.42085 0 L
s
.42035 0 m
.42024 0 L
.42013 0 L
s
.42013 0 m
.42005 0 L
s
.41978 0 m
.41973 0 L
s
.41973 0 m
.41965 0 L
s
.41938 0 m
.41922 0 L
.41906 0 L
s
.41906 0 m
.41896 0 L
s
.41868 0 m
.41862 0 L
.41847 0 L
.41833 0 L
s
.41822 0 m
.41817 0 L
.41805 0 L
.41798 0 L
.41792 0 L
.41781 0 L
s
.41743 0 m
0 0 rlineto s
.41732 0 m
.41728 0 L
s
.41728 0 m
.41724 0 L
s
.41699 0 m
.41695 0 L
.4169 0 L
.41686 0 L
.41682 0 L
.41678 0 L
s
.41674 0 m
.41672 0 L
.4167 0 L
.41669 0 L
s
.41669 0 m
.41668 0 L
s
.41667 0 m
0 0 rlineto s
.41667 0 m
.41668 0 L
s
.41668 0 m
.41668 0 L
s
.4167 0 m
.4167 0 L
.41672 0 L
.41674 0 L
.41677 0 L
.41681 0 L
s
.41681 0 m
.41683 0 L
s
.41707 0 m
.41709 0 L
.41716 0 L
s
.41716 0 m
.41722 0 L
s
.41748 0 m
0 0 rlineto s
.41768 0 m
0 0 rlineto s
.41785 0 m
.4179 0 L
s
.4179 0 m
.41795 0 L
s
.42637 0 m
.42661 0 L
s
.42661 0 m
.42675 0 L
s
.42689 0 m
.42698 0 L
.4273 0 L
.42765 0 L
.42785 0 L
s
.42785 0 m
.42794 0 L
s
.42846 0 m
0 0 rlineto s
.42876 0 m
.42886 0 L
.42922 0 L
.42958 0 L
s
.42958 0 m
.42977 0 L
s
.43014 0 m
.43032 0 L
s
.43032 0 m
.43049 0 L
s
.43074 0 m
.43106 0 L
.43129 0 L
.4315 0 L
.4319 0 L
s
.4319 0 m
.43211 0 L
s
.43252 0 m
.43273 0 L
.43317 0 L
.43356 0 L
.43399 0 L
s
.43399 0 m
.43412 0 L
s
.43449 0 m
.43473 0 L
s
.43473 0 m
.4349 0 L
s
.43591 0 m
0 0 rlineto s
.43624 0 m
.43645 0 L
s
.43645 0 m
.4367 0 L
s
.43724 0 m
.4375 0 L
.43779 0 L
.43811 0 L
s
.43811 0 m
.43825 0 L
s
.43853 0 m
.43868 0 L
.43897 0 L
.43923 0 L
.43974 0 L
.44028 0 L
s
.44134 0 m
.44187 0 L
s
.44298 0 m
.44333 0 L
.44386 0 L
.44436 0 L
.44492 0 L
s
.44492 0 m
.44537 0 L
s
.44577 0 m
.44603 0 L
.4466 0 L
.44722 0 L
.44757 0 L
s
.44757 0 m
.44763 0 L
s
.44929 0 m
.44963 0 L
.4502 0 L
s
.45138 0 m
.45191 0 L
s
.45191 0 m
.45216 0 L
s
.45254 0 m
.45295 0 L
.45354 0 L
.45413 0 L
.45477 0 L
s
.45557 0 m
.45594 0 L
.45661 0 L
s
.45716 0 m
.45733 0 L
.45798 0 L
.45867 0 L
.45934 0 L
.46008 0 L
s
.46008 0 m
.46061 0 L
s
.46211 0 m
.46271 0 L
.46341 0 L
.46407 0 L
.4648 0 L
.46558 0 L
.46625 0 L
s
.46625 0 m
.46643 0 L
s
.46806 0 m
.46824 0 L
s
.46824 0 m
.46845 0 L
s
.47057 0 m
.4709 0 L
.47163 0 L
s
.47314 0 m
.47382 0 L
s
.47382 0 m
.47414 0 L
s
.47515 0 m
.47588 0 L
s
.47588 0 m
.47614 0 L
s
.47803 0 m
.47822 0 L
s
.47822 0 m
.47858 0 L
s
.48 0 m
.48024 0 L
s
.48024 0 m
.48032 0 L
s
.48213 0 m
.48234 0 L
s
.48234 0 m
.48277 0 L
s
.48342 0 m
.48412 0 L
.485 0 L
s
.485 0 m
.48516 0 L
s
.48656 0 m
.48739 0 L
.48826 0 L
.48908 0 L
.49667 0 L
.49749 0 L
.49838 0 L
.49922 0 L
s
.50086 0 m
.50169 0 L
.50259 0 L
.50357 0 L
s
.50664 0 m
0 0 rlineto s
.50814 0 m
.50851 0 L
.50938 0 L
.51038 0 L
s
.51237 0 m
.51296 0 L
.51351 0 L
s
.51351 0 m
.51401 0 L
s
.5195 0 m
.52065 0 L
.5213 0 L
s
.52358 0 m
0 0 rlineto s
.52474 0 m
.52528 0 L
.52645 0 L
.52756 0 L
.52858 0 L
.53046 0 L
.53249 0 L
.53367 0 L
.53474 0 L
.53582 0 L
.53685 0 L
s
.53685 0 m
.53721 0 L
s
.53907 0 m
0 0 rlineto s
.54231 0 m
.54357 0 L
.54477 0 L
.54587 0 L
.54714 0 L
.54833 0 L
.55058 0 L
.55176 0 L
.55302 0 L
s
.5605 0 m
0 0 rlineto s
.56269 0 m
.56385 0 L
.56508 0 L
s
.56765 0 m
0 0 rlineto s
.57045 0 m
.57172 0 L
.57309 0 L
.57438 0 L
.57556 0 L
s
.57669 0 m
.57708 0 L
s
.57708 0 m
.57801 0 L
s
.57963 0 m
.58 0 L
.5808 0 L
.58167 0 L
s
.58167 0 m
.58192 0 L
s
.58323 0 m
.584 0 L
s
.584 0 m
.58424 0 L
s
.58891 0 m
.5902 0 L
s
.5902 0 m
.59088 0 L
s
.5923 0 m
.59304 0 L
s
.59304 0 m
.5939 0 L
s
.59688 0 m
0 0 rlineto s
.59865 0 m
.59959 0 L
s
.59959 0 m
.60034 0 L
s
.60239 0 m
0 0 rlineto s
.60458 0 m
.60535 0 L
.60621 0 L
.60701 0 L
.60855 0 L
.61002 0 L
.6151 0 L
.61645 0 L
s
.61645 0 m
.61718 0 L
s
.61884 0 m
.6193 0 L
.62058 0 L
s
.62291 0 m
.62377 0 L
.62527 0 L
.62668 0 L
.62812 0 L
.62944 0 L
s
.62944 0 m
.6302 0 L
s
.64361 0 m
.64671 0 L
s
.6496 0 m
0 0 rlineto s
.65518 0 m
.65599 0 L
s
.65599 0 m
.65628 0 L
s
.66125 0 m
.6622 0 L
.66323 0 L
s
.66502 0 m
0 0 rlineto s
.66729 0 m
.66839 0 L
s
.66839 0 m
.66902 0 L
s
.67127 0 m
.67171 0 L
.67331 0 L
s
.67518 0 m
0 0 rlineto s
.67839 0 m
.68002 0 L
.68178 0 L
s
.68178 0 m
.68287 0 L
s
.68515 0 m
.68613 0 L
s
.6887 0 m
.69047 0 L
.69206 0 L
.69355 0 L
.6952 0 L
s
.69791 0 m
.69885 0 L
s
.69885 0 m
.69957 0 L
s
.70092 0 m
.70274 0 L
s
.70274 0 m
.70308 0 L
s
.70469 0 m
.70645 0 L
.70837 0 L
.71004 0 L
.71185 0 L
s
.71469 0 m
.7156 0 L
.71671 0 L
.71776 0 L
.71871 0 L
.71974 0 L
.72074 0 L
s
.72074 0 m
.7213 0 L
s
.72302 0 m
.72411 0 L
.72607 0 L
s
.72768 0 m
.72819 0 L
s
.72819 0 m
.72911 0 L
s
.7306 0 m
.73174 0 L
.73363 0 L
.73469 0 L
.73565 0 L
s
.73565 0 m
.7363 0 L
s
.73871 0 m
0 0 rlineto s
.74114 0 m
.74161 0 L
.74361 0 L
s
.74361 0 m
.74455 0 L
s
.74652 0 m
.74754 0 L
.74949 0 L
s
.75219 0 m
.75309 0 L
.75473 0 L
s
.7584 0 m
.76036 0 L
.76243 0 L
.76421 0 L
.76616 0 L
.76827 0 L
.76946 0 L
.77054 0 L
s
.77054 0 m
.77159 0 L
s
.77398 0 m
.77465 0 L
s
.77465 0 m
.77522 0 L
s
.77825 0 m
0 0 rlineto s
.78208 0 m
.78382 0 L
s
.78382 0 m
.78462 0 L
s
.78902 0 m
.79092 0 L
.79294 0 L
.79486 0 L
s
.79486 0 m
.79544 0 L
s
.79866 0 m
.79982 0 L
.80089 0 L
s
.80497 0 m
.80683 0 L
s
.8102 0 m
.81194 0 L
.8156 0 L
.8195 0 L
.82113 0 L
.82289 0 L
.82483 0 L
.82665 0 L
.82834 0 L
.83013 0 L
.83166 0 L
s
.83166 0 m
.8325 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{420.75, 260.062},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006T000111000`40n007h00OooooYOoo003ooonU
ool00<coo`8000Coo`03003ooooo00;oo`@00:Soo`8000Coo`03003ooooo00;oo`@001;oo`00booo
00@00?ooool000[oo`03003ooooo0:Koo`04003ooooo0008ool00`00ooooo`0Cool00<_oo`04003o
oooo000:ool00`00ooooo`2Vool01000ooooo`002Ooo00<00?ooool04_oo003;ool01000ooooo`00
2_oo00<00?ooool0Y_oo00@00?ooool000[oo`03003ooooo017oo`00booo00@00?ooool000[oo`03
003ooooo0:Koo`04003ooooo000;ool00`00ooooo`0@ool00<_oo`04003ooooo0008ool3002Xool0
1000ooooo`002?oo00@00?ooool001;oo`00c?oo0P002ooo00<00?ooool0Yooo0P002_oo0P004ooo
003ooonUool00?ooojGoo`00ooooYOoo003ooonUool00;Cooi/004Ooo`<000_oo`007?ooo`00L000
3_oo00<00?ooool02?oo000Lool00`00ooooo`2EoonK000kool00`00ooooo`0:ool2000;ool001co
o`03003ooooo0;Coo`03003ooooo0;Coo`03003ooooo00Woo`03003ooooo00_oo`007?oo00<00?oo
ool0ooooNOoo0`002_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo
00<00?ooool0ooooQ_oo0007ool20004ool00`00ooooo`02ool40006ool00`00ooooo`3ooon6ool0
00Koo`04003ooooo000:ool00`00ooooo`05ool00`00ooooo`3ooon6ool000Koo`04003ooooo000:
ool00`00ooooo`05ool3003ooon6ool000Koo`04003ooooo000:ool00`00ooooo`05ool00`00oooo
o`3ooon6ool000Koo`04003ooooo000:ool00`00ooooo`05ool00`00ooooo`3ooon6ool000Koo`04
003ooooo0008ool30007ool00`00ooooo`3ooon6ool000Ooo`8000_oo`03003ooooo00Goo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003o
oooo0?ooohKoo`001ooo0P001?oo00<00?ooool00_oo10001_oo00<00?ooool0ooooQ_oo0006ool0
1000ooooo`002?oo00<00?ooool01ooo00<00?ooool0ooooQ_oo0006ool01000ooooo`002Ooo00<0
0?ooool01_oo0`00ooooQ_oo0006ool01000ooooo`002_oo00<00?ooool01Ooo00<00?ooool0oooo
Q_oo0006ool01000ooooo`002ooo00<00?ooool01?oo00<00?ooool0ooooQ_oo0006ool01000oooo
o`002?oo00@00?ooool000Koo`03003ooooo0?ooohKoo`001ooo0P002_oo0P001ooo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0
ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`001ooo0P001?oo00<0
0?ooool00ooo0P001ooo00<00?ooool0ooooQ_oo0006ool01000ooooo`002?oo00@00?ooool000Ko
o`03003ooooo0?ooohKoo`001_oo00@00?ooool000_oo`03003ooooo00Coo`<00?ooohKoo`001_oo
00@00?ooool000Woo`8000Ooo`03003ooooo0?ooohKoo`001_oo00@00?ooool000_oo`03003ooooo
00Coo`03003ooooo0?ooohKoo`001_oo00@00?ooool000Soo`04003ooooo0006ool00`00ooooo`3o
oon6ool000Ooo`8000[oo`8000Ooo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool001coo`03003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000L
ool00`00ooooo`3ooon6ool000Ooo`8000Coo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo
0?ooohKoo`001_oo00@00?ooool000[oo`03003ooooo00Goo`03003ooooo0?ooohKoo`001_oo00@0
0?ooool000Ooo`D000Koo`<00?ooohKoo`001_oo00@00?ooool000Ooo`04003ooooo0007ool00`00
ooooo`3ooon6ool000Koo`04003ooooo0008ool00`00ool00007ool00`00ooooo`3ooon6ool000Ko
o`04003ooooo0009ool20007ool00`00ooooo`3ooon6ool000Ooo`8000_oo`03003ooooo00Goo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool001coo`03
003ooooo0?ooohKoo`007?oo00<00?ooool0ooooQ_oo000Lool00`00ooooo`3ooon6ool000Ooo`80
00Coo`03003ooooo00;oo`<000Ooo`03003ooooo0?ooohKoo`001_oo00@00?ooool000_oo`03003o
oooo00Coo`03003ooooo0?ooohKoo`001_oo00@00?ooool000_oo`03003ooooo00Coo`<00?ooohKo
o`001_oo00@00?ooool000Soo`<00?oooi3oo`001_oo00@00?ooool000Soo`03003ooooo0?oooi3o
o`001_oo00@00?ooool000Soo`03003ooooo0?oooi3oo`001ooo0P002Ooo1000ooooSooo003ooonU
ool00?ooojGoo`00ooooYOoo003ooonUool00?ooojGoo`00ooooYOoo003ooonUool00?ooojGoo`00
6_oo00<00?ooool0ooooR?oo000Iool00`00ool0003ooon9ool001Woo`03003oo`000?ooohWoo`00
6?oo00D00?ooooooo`000?ooohSoo`005ooo0P000_oo0P00ooooR?oo000Mool00`00ooooo`3ooon5
ool001koo`03003ooooo0?ooohCoo`00\
\>"],
  ImageRangeCache->{{{122.5, 458.062}, {362.75, 155.75}} -> {-0.0993628, \
0.400635, 0.00054685, 0.00221205}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{992, 735},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PrintingMargins"->{{84.9375, 54}, {72, 72}},
"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "felix", \
"tmp"}, "mathematica.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 31, 0, 45, "Subsection"],
Cell[1773, 53, 284, 9, 32, "Text"],

Cell[CellGroupData[{
Cell[2082, 66, 41, 0, 42, "Subsubsection"],
Cell[2126, 68, 591, 20, 32, "Text"],
Cell[2720, 90, 312, 6, 83, "Input"],
Cell[3035, 98, 73, 0, 32, "Text"],
Cell[3111, 100, 119, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3267, 107, 305, 11, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[3597, 122, 1141, 21, 139, "Input"],
Cell[4741, 145, 22261, 582, 257, 6982, 389, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[27017, 730, 546, 14, 68, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27600, 749, 296, 11, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[27921, 764, 1547, 39, 59, "Input"],
Cell[29471, 805, 23143, 1531, 269, 14669, 1422, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

