(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    412297,      17193]*)
(*NotebookOutlinePosition[    413498,      17230]*)
(*  CellTagsIndexPosition[    413454,      17226]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Example 1", "Section"],

Cell[CellGroupData[{

Cell["Some definitions", "Subsection"],

Cell[TextData[{
  Cell[BoxData[
      FormBox[
        SuperscriptBox[
          RowBox[{"\[LeftBracketingBar]", 
            RowBox[{
              SubscriptBox["c", "1"], "(", 
              RowBox[{
                OverscriptBox["t", "~"], "+", "\[Tau]"}], ")"}], 
            "\[RightBracketingBar]"}], "2"], TraditionalForm]]],
  " according to 0310142307:"
}], "Text"],

Cell[BoxData[
    \(c12tt[\[Tau]_, \[Beta]p_, \[Omega]_] := 
      1 - \[Beta]p + \[Beta]p\ Cos[\[Omega]\ \[Tau]]; \)], "Input"],

Cell[TextData[{
  Cell[BoxData[
      \(TraditionalForm\`\(\[Omega]\&~\)\_aprx\)]],
  " according to 0310142307:"
}], "Text"],

Cell[BoxData[
    \(\(f[x_, y_] := 
        If[x \[GreaterEqual] 0 && x < 1 && y \[GreaterEqual] 4  x - 3 && 
            y \[LessEqual] 1, \(y - 1\)\/\(2 \((x - 1)\)\) - 1, 
          1];\)\)], "Input"],

Cell[BoxData[
    \(\(\(arcCosF[x_, y_] := 
      Block[{tmp = f[x, y]}, 
        If[tmp \[GreaterEqual] 1. , 0, 
          If[tmp \[LessEqual] \(-1. \), \[Pi], ArcCos[tmp]]]]; \)\( (*\ 
      The\ if' s\ are\ necessary\ to\ avoid\ numerical\ problems\ if\ tmp\ is\
\ slightly\ larger\ than\ 1\ which\ would\ cause\ ArcCos\ to\ return\ non - 
        real\ numbers\ *) \)\)\)], "Input"],

Cell[BoxData[
    \(\[Omega]TildeAprx[mmHalf\[CapitalDelta]t_, mm\[CapitalDelta]t_, 
        nn_, \[CapitalDelta]t_] := \(2\/\[CapitalDelta]t\) 
        arcCosF[mmHalf\[CapitalDelta]t\/nn, 
          mm\[CapitalDelta]t\/nn]\)], "Input"],

Cell[TextData[{
  "Mean-square deviation ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]\ \(\(\(\[Omega]\&~\)\^2\)(
          N, \[CapitalDelta]t)\)\)]],
  ":"
}], "Text"],

Cell[BoxData[
    \(p1[\[Tau]_, \[Beta]p_, \[Omega]_] := 
      c12tt[\[Tau], \[Beta]p, \[Omega]]; \)], "Input"],

Cell[BoxData[
    \(p[j_, nn_, \[Tau]_, \[Beta]p_, \[Omega]_] := 
      Block[{tmp = 
            p1[\[Tau], \[Beta]p, \[Omega]]}, \[IndentingNewLine]Binomial[nn, 
            j] If[\((tmp \[Equal] 0 \[And] 
                  j \[Equal] 0)\) \[Or] \((tmp \[Equal] 1 \[And] 
                  j \[Equal] nn)\), 
            1, \(tmp\^j\) \((1 - tmp)\)\^\(nn - j\)]]; \)], "Input"],

Cell[BoxData[
    \(\(\(\[Omega]Tilde[\[Beta]p_, \[Omega]_]\)\(:=\)\(If[\[Beta]p \[Equal] 
          0, 0, \[Omega]]\)\( (*See\ 0310142307\ and\ 0312150030*) \)\)\)], \
"Input"],

Cell[BoxData[
    \(\[CapitalDelta]\[Omega]TildeSq[\[CapitalDelta]t_, 
        nn_, \[Beta]p_, \[Omega]_] := \[Sum]\_\(j = 0\)\%nn p[j, 
            nn, \[CapitalDelta]t\/2, \[Beta]p, \[Omega]] \(\[Sum]\_\(k = \
0\)\%nn p[k, 
                nn, \[CapitalDelta]t, \[Beta]p, \[Omega]] \
\((\[Omega]TildeAprx[j, k, nn, \[CapitalDelta]t] - \[Omega]Tilde[\[Beta]p, \
\[Omega]])\)\^2\); \)], "Input"],

Cell[TextData[{
  "Maximal possible \[CapitalDelta]t (see 0310142307 to understand why this \
is the absolute maximum for ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]t\)]],
  "):"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"\[CapitalDelta]tMax", "[", "\[Omega]_", "]"}], "=", 
        FractionBox[
          RowBox[{"2", "\[Pi]"}], "\[Omega]"]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plots of ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta] \( \[Omega]\&~\)\^2\)]],
  " with fixed ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]t\)]]
}], "Subsection"],

Cell["\<\
Some default values (an A is appended to the variable names and \
functions to avoid naming conflicts):\
\>", "Text"],

Cell[BoxData[{
    \(\[Omega]A = 2  \[Pi] (*Hz*) ; \), "\[IndentingNewLine]", 
    \(maxNA = 51; \), "\[IndentingNewLine]", 
    \(\[Beta]pA = 1/3; \)}], "Input"],

Cell["\<\
Functions to create and display a list plot with the above default \
values:\
\>", "Text"],

Cell[BoxData[{
    \(createPlotA[\[CapitalDelta]t_ (*s*) ] := 
      ListPlot[Table[{nn, \[CapitalDelta]\[Omega]TildeSq[\[CapitalDelta]t, 
              nn, \[Beta]pA, \[Omega]A]}, {nn, 1, maxNA, 1}], 
        DisplayFunction \[Rule] Identity]; \), "\[IndentingNewLine]", 
    \(showPlotA[plot_] := 
      Show[plot, PlotRange \[Rule] {{1, maxNA}, {0, 150}}, 
        DisplayFunction \[Rule] $DisplayFunction, 
        AxesLabel \[Rule] {"\<N\>", \
\*"\"\<\[CapitalDelta]\!\(\[Omega]\&~\^2\)/\!\(Hz\^2\)\>\""}, 
        Ticks \[Rule] {{1, 25, 50}, {0, 50, 100, 150}}, 
        AxesOrigin \[Rule] {0.999, \(-0.01\)}]; \)}], "Input"],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot where ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]t\)]],
  " is ",
  Cell[BoxData[
      \(TraditionalForm\`0.18\)]],
  " times its maximum permissible value (see the definition of \
\[CapitalDelta]tMax above)"
}], "Subsubsection"],

Cell[BoxData[
    \(plotA1 = createPlotA[0.18  \[CapitalDelta]tMax[\[Omega]A]]; \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(showPlotA[plotA1]; \)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.02 0.02 -2.60209e-17 0.00412023 [
[0 -0.01254 -3 -9 ]
[0 -0.01254 3 0 ]
[.48 -0.01254 -6 -9 ]
[.48 -0.01254 6 0 ]
[.98 -0.01254 -6 -9 ]
[.98 -0.01254 6 0 ]
[1.025 -4e-05 0 -4.90625 ]
[1.025 -4e-05 10 4.90625 ]
[-0.01252 0 -6 -4.5 ]
[-0.01252 0 0 4.5 ]
[-0.01252 .20601 -12 -4.5 ]
[-0.01252 .20601 0 4.5 ]
[-0.01252 .41202 -18 -4.5 ]
[-0.01252 .41202 0 4.5 ]
[-0.01252 .61803 -18 -4.5 ]
[-0.01252 .61803 0 4.5 ]
[-2e-05 .64303 -22.3125 0 ]
[-2e-05 .64303 22.3125 13 ]
[ -2e-05 -4e-05 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
0 -4e-05 m
0 .00621 L
s
[(1)] 0 -0.01254 0 1 Mshowa
.48 -4e-05 m
.48 .00621 L
s
[(25)] .48 -0.01254 0 1 Mshowa
.98 -4e-05 m
.98 .00621 L
s
[(50)] .98 -0.01254 0 1 Mshowa
0 -4e-05 m
1 -4e-05 L
s
gsave
1.025 -4e-05 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(N) show
69.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
-2e-05 0 m
.00623 0 L
s
[(0)] -0.01252 0 1 0 Mshowa
-2e-05 .20601 m
.00623 .20601 L
s
[(50)] -0.01252 .20601 1 0 Mshowa
-2e-05 .41202 m
.00623 .41202 L
s
[(100)] -0.01252 .41202 1 0 Mshowa
-2e-05 .61803 m
.00623 .61803 L
s
[(150)] -0.01252 .61803 1 0 Mshowa
-2e-05 0 m
-2e-05 .61803 L
s
gsave
-2e-05 .64303 -83.3125 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
0 .30767 Mdot
.02 .40897 Mdot
.04 .47726 Mdot
.06 .52077 Mdot
.08 .54585 Mdot
.1 .55745 Mdot
.12 .5594 Mdot
.14 .55464 Mdot
.16 .54539 Mdot
.18 .53332 Mdot
.2 .51966 Mdot
.22 .5053 Mdot
.24 .49083 Mdot
.26 .47667 Mdot
.28 .46307 Mdot
.3 .45018 Mdot
.32 .43805 Mdot
.34 .4267 Mdot
.36 .41611 Mdot
.38 .40624 Mdot
.4 .39705 Mdot
.42 .38847 Mdot
.44 .38045 Mdot
.46 .37294 Mdot
.48 .3659 Mdot
.5 .35928 Mdot
.52 .35305 Mdot
.54 .34718 Mdot
.56 .34164 Mdot
.58 .33639 Mdot
.6 .33143 Mdot
.62 .32672 Mdot
.64 .32225 Mdot
.66 .318 Mdot
.68 .31396 Mdot
.7 .3101 Mdot
.72 .30642 Mdot
.74 .30289 Mdot
.76 .29952 Mdot
.78 .29628 Mdot
.8 .29317 Mdot
.82 .29018 Mdot
.84 .2873 Mdot
.86 .28452 Mdot
.88 .28184 Mdot
.9 .27925 Mdot
.92 .27674 Mdot
.94 .27431 Mdot
.96 .27196 Mdot
.98 .26967 Mdot
1 .26746 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001[oo`@006[oo`@000;oo`<006koo`<000Coo`8001Soo`007?oo00<00?ooool0JOoo00<00?oo
ool01_oo00<00?ooool0K_oo00@00?ooool000;oo`03003ooooo01Goo`007?oo00<00?ooool0J_oo
00<00?ooool01Ooo00<00?ooool0K_oo00@00?ooool000;oo`03003ooooo01Goo`007?oo00<00?oo
ool0Jooo00@00?ooooooo`<006koo`<000?oo`04003ooooo000Gool001coo`03003ooooo06coo`04
003ooooo001`ool00`00ooooo`03ool01000ooooo`005ooo000Jool3001[ool01000ooooo`000_oo
00<00?ooool0K_oo00<00?ooool00ooo00@00?ooool001Ooo`007?oo00<00?ooool0J_oo0P000ooo
1000KOoo10000ooo0P006?oo003ooolQool001Goo`800?Woo`8000;oo`03003ooooo00Woo`005?oo
00@00?ooool00?Woo`04003ooooo000;ool001Coo`04003ooooo0004ooo]0008ool01000ool00000
2ooo000Dool01000ooooo`001?oo00<00?ooool0K_oo00<00?ooool0M?oo00<00?ooool02_oo0P00
00?oo`00ool02_oo000Dool01000ooooo`001?oo00<00?ooool0l_oo00@00?ooool000_oo`005?oo
00@00?ooool000Coo`03003ooooo0?7oo`800003ool0000000_oo`005Ooo0P001Ooo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`003_oo0`001?oo0P00
1Ooo00<00?ooool0oooo0_oo000Aool01000ooooo`000_oo00<00?ooool00_oo00<00?ooool0oooo
0_oo000Aool01000ooooo`000_oo00<00?ooool00_oo0`00oooo0_oo000>ool30003ool01000oooo
o`001?oo00<00?ooool0oooo0_oo000>ool00`00ooooo`03ool01000ooooo`001?oo00<00?ooool0
oooo0_oo000>ool00`00ooooo`03ool01000ooooo`001?oo00<00?ooool0oooo0_oo000>ool40003
ool20005ool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo
0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo
0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo
0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0jOoo
00<00?ooool05Ooo000Lool00`00ooooo`3Oool00`00ooooo`02ool00`00ooooo`0Jool001coo`03
003ooooo0=Koo`03003ooooo00;oo`03003ooooo02?oo`007?oo00<00?ooool0dOoo00<00?ooool0
;Ooo000Lool00`00ooooo`38ool00`00ooooo`02ool00`00ooooo`0aool001coo`03003ooooo0<?o
o`03003ooooo03_oo`007?oo00<00?ooool0^_oo00D00?ooooooo`0004;oo`007?oo00<00?ooool0
]Ooo00<00?ooool0BOoo000Lool00`00ooooo`2`ool00`00ooooo`1>ool001coo`03003ooooo0:Oo
o`05003oooooool0001Eool001coo`03003ooooo0:;oo`03003ooooo05coo`007?oo00<00?ooool0
WOoo00<00?ooool0HOoo000Lool00`00ooooo`2Iool00`00ooooo`1Uool001coo`03003ooooo09Co
o`03003ooooo06[oo`007?oo00<00?ooool0Sooo00<00?ooool0Kooo000Lool00`00ooooo`2:ool0
0`00ooooo`1dool001coo`03003ooooo08Koo`03003ooooo07Soo`007?oo00<00?ooool0oooo0_oo
000Lool00`00ooooo`21ool00`00ooooo`1mool001coo`03003ooooo07coo`03003ooooo08;oo`00
7?oo00<00?ooool0Mooo00<00?ooool0Qooo000Lool00`00ooooo`3oool2ool001coo`03003ooooo
07?oo`03003ooooo08_oo`007?oo00<00?ooool0K_oo00<00?ooool0T?oo000Lool00`00ooooo`3o
ool2ool001coo`03003ooooo06Woo`03003ooooo09Goo`007?oo00<00?ooool0oooo0_oo000Lool0
0`00ooooo`1Uool00`00ooooo`2Iool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0H?oo
00<00?ooool0W_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo05_oo`03003ooooo0:?o
o`002?oo10000ooo0P001?oo0P001Ooo00<00?ooool0oooo0_oo000:ool01@00oooooooo00000_oo
00@00?ooool000;oo`03003ooooo00;oo`03003ooooo05Koo`03003ooooo0:Soo`002_oo00D00?oo
ooooo`0000;oo`04003ooooo0002ool00`00ooooo`02ool300000ooo003oo`3nool000[oo`05003o
ooooool00002ool01000ooooo`000_oo00<00?ooool00_oo00<00?ooool0D_oo00<00?ooool0[?oo
000:ool01@00oooooooo00000_oo00@00?ooool000;oo`03003ooooo00;oo`03003ooooo0?ooo`;o
o`002?oo0`000ooo00@00?ooool000;oo`04003ooooo0004ool00`00ooooo`3oool2ool000[oo`03
003ooooo00;oo`8000Coo`8000Goo`03003ooooo04goo`03003ooooo0;7oo`007?oo00<00?ooool0
oooo0_oo000Lool00`00ooooo`18ool00`00ooooo`2fool001coo`03003ooooo0?ooo`;oo`007?oo
00<00?ooool0oooo0_oo000Lool00`00ooooo`14ool00`00ooooo`2jool001coo`03003ooooo0?oo
o`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`0oool00`00ooooo`2oool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo00Koo`03003ooooo037oo`03003ooooo0<Coo`007?oo00<00?ooool0oooo0_oo000Lool0
0`00ooooo`3oool2ool001coo`03003ooooo03Goo`03003ooooo0<Woo`007?oo00<00?ooool0oooo
0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo037oo`03003ooooo0<goo`007?oo00<0
0?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<0
0?ooool02ooo00<00?ooool07_oo00<00?ooool0d_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool09ooo00<00?ooool0eooo000Lool00`00ooooo`3oool2
ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool04?oo00<00?ooool03ooo00<00?ooool0
g?oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo01koo`03003ooooo0>3oo`007?oo00<0
0?ooool05?oo00<00?ooool00_oo00<00?ooool0iOoo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`002?oo10000_oo0`001?oo0P001Ooo00<00?ooool0oooo0_oo000:ool00`00
ooooo`04ool01000ooooo`000_oo00<00?ooool00_oo00<00?ooool0oooo0_oo000:ool00`00oooo
o`04ool01000ooooo`000_oo00<00?ooool00_oo0`00oooo0_oo000:ool01000oooooooo0`000ooo
00@00?ooool00?ooo`Woo`002_oo00D00?ooooooo`0000Goo`04003ooooo003oool9ool000Soo`<0
00?oo`03003ooooo00?oo`04003ooooo003oool9ool000[oo`04003oooooool40003ool2003oool:
ool00?ooob7oo`00oooo8Ooo003ooolQool001[oo`03003ooooo0?ooo`Coo`002?oo1@000_oo1000
2?oo00@00?ooooooo`8000;oo`800003ool0000000800?Koo`002Ooo00<00?oo00000_oo00H00?oo
ool00?oo0007ool00`00ooooo`02ool01000ooooo`000_oo00@00?ooool00?Koo`002Ooo00<00?oo
00000_oo00H00?ooool00?oo0007ool00`00ooooo`02ool01000ooooo`000ooo00<00?ooool0m_oo
000:ool01@00oooooooo00001?oo00<00?ooool01_oo00@00?ooooooo`@000Coo`03003ooooo0?Go
o`002_oo00<00?ooool00_oo00<00?oo00000P002?oo00D00?ooooooo`0000;oo`03003ooooo00@0
0003ool0000000800?7oo`002_oo00<00?ooool00_oo0P0000?oo`00ool010001Ooo00@00?oo0000
00;oo`8000Ooo`800?;oo`005Ooo0P005ooo00<00?ooool0kooo000Gool00`00ooooo`0Bool3003a
ool001Goo`<00?ooo`Woo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {202.5, 61.1875}} -> {-39.8108, \
66.3912, 0.211968, 1.02891}}]
}, Open  ]],

Cell[TextData[{
  "We see that the convergence gets worse for the first couple of values of \
",
  Cell[BoxData[
      \(TraditionalForm\`N\)]],
  " and then gets better again. By incresing ",
  Cell[BoxData[
      \(TraditionalForm\`N\)]],
  ", however, it is possible to make ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta] \( \[Omega]\&~\)\^2\)]],
  " arbitrarily small because, as shown in 0312201200, ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Omega]\&~\)\_aprx\)]],
  " converges against ",
  Cell[BoxData[
      FormBox[
        RowBox[{\(\[Omega]\&~\), Cell[""]}], TraditionalForm]]],
  ". The weird behavior of the convergence for the first couple of values of \
",
  Cell[BoxData[
      \(TraditionalForm\`N\)]],
  " is probably related to the fact that for small ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]t\)]],
  " the possible parameters ",
  Cell[BoxData[
      \(TraditionalForm\`\((a\_aprx, b\_aprx)\)\)]],
  " of the function ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  " (see 0310142307) are with high probability close to the point ",
  Cell[BoxData[
      \(TraditionalForm\`\((1, 1)\)\)]],
  ", and in this point ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  " is discontinuous (see 0312031115)."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot where ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]t\)]],
  " is ",
  Cell[BoxData[
      \(TraditionalForm\`0.4\)]],
  " times its maximum permissible value (see the definition of \
\[CapitalDelta]tMax above):"
}], "Subsubsection"],

Cell[BoxData[
    \(plotA2 = createPlotA[0.4  \[CapitalDelta]tMax[\[Omega]A]]; \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(showPlotA[plotA2]; \)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.02 0.02 -2.60209e-17 0.00412023 [
[0 -0.01254 -3 -9 ]
[0 -0.01254 3 0 ]
[.48 -0.01254 -6 -9 ]
[.48 -0.01254 6 0 ]
[.98 -0.01254 -6 -9 ]
[.98 -0.01254 6 0 ]
[1.025 -4e-05 0 -4.90625 ]
[1.025 -4e-05 10 4.90625 ]
[-0.01252 0 -6 -4.5 ]
[-0.01252 0 0 4.5 ]
[-0.01252 .20601 -12 -4.5 ]
[-0.01252 .20601 0 4.5 ]
[-0.01252 .41202 -18 -4.5 ]
[-0.01252 .41202 0 4.5 ]
[-0.01252 .61803 -18 -4.5 ]
[-0.01252 .61803 0 4.5 ]
[-2e-05 .64303 -22.3125 0 ]
[-2e-05 .64303 22.3125 13 ]
[ -2e-05 -4e-05 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
0 -4e-05 m
0 .00621 L
s
[(1)] 0 -0.01254 0 1 Mshowa
.48 -4e-05 m
.48 .00621 L
s
[(25)] .48 -0.01254 0 1 Mshowa
.98 -4e-05 m
.98 .00621 L
s
[(50)] .98 -0.01254 0 1 Mshowa
0 -4e-05 m
1 -4e-05 L
s
gsave
1.025 -4e-05 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(N) show
69.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
-2e-05 0 m
.00623 0 L
s
[(0)] -0.01252 0 1 0 Mshowa
-2e-05 .20601 m
.00623 .20601 L
s
[(50)] -0.01252 .20601 1 0 Mshowa
-2e-05 .41202 m
.00623 .41202 L
s
[(100)] -0.01252 .41202 1 0 Mshowa
-2e-05 .61803 m
.00623 .61803 L
s
[(150)] -0.01252 .61803 1 0 Mshowa
-2e-05 0 m
-2e-05 .61803 L
s
gsave
-2e-05 .64303 -83.3125 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
0 .1687 Mdot
.02 .13838 Mdot
.04 .10824 Mdot
.06 .09375 Mdot
.08 .08952 Mdot
.1 .08716 Mdot
.12 .083 Mdot
.14 .07763 Mdot
.16 .07281 Mdot
.18 .06934 Mdot
.2 .06692 Mdot
.22 .06484 Mdot
.24 .06272 Mdot
.26 .06055 Mdot
.28 .0585 Mdot
.3 .05668 Mdot
.32 .05505 Mdot
.34 .05354 Mdot
.36 .0521 Mdot
.38 .05071 Mdot
.4 .04938 Mdot
.42 .04813 Mdot
.44 .04695 Mdot
.46 .04583 Mdot
.48 .04476 Mdot
.5 .04373 Mdot
.52 .04274 Mdot
.54 .04179 Mdot
.56 .04088 Mdot
.58 .04 Mdot
.6 .03916 Mdot
.62 .03835 Mdot
.64 .03756 Mdot
.66 .0368 Mdot
.68 .03607 Mdot
.7 .03536 Mdot
.72 .03468 Mdot
.74 .03402 Mdot
.76 .03337 Mdot
.78 .03275 Mdot
.8 .03215 Mdot
.82 .03156 Mdot
.84 .03099 Mdot
.86 .03043 Mdot
.88 .0299 Mdot
.9 .02937 Mdot
.92 .02886 Mdot
.94 .02837 Mdot
.96 .02788 Mdot
.98 .02741 Mdot
1 .02696 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool001[oo`@006[oo`@000;oo`<006koo`<000Coo`8001Soo`007?oo00<00?ooool0JOoo00<00?oo
ool01_oo00<00?ooool0K_oo00@00?ooool000;oo`03003ooooo01Goo`007?oo00<00?ooool0J_oo
00<00?ooool01Ooo00<00?ooool0K_oo00@00?ooool000;oo`03003ooooo01Goo`007?oo00<00?oo
ool0Jooo00@00?ooooooo`<006koo`<000?oo`04003ooooo000Gool001coo`03003ooooo06coo`04
003ooooo001`ool00`00ooooo`03ool01000ooooo`005ooo000Jool3001[ool01000ooooo`000_oo
00<00?ooool0K_oo00<00?ooool00ooo00@00?ooool001Ooo`007?oo00<00?ooool0J_oo0P000ooo
1000KOoo10000ooo0P006?oo003ooolQool001Goo`800?Woo`8000;oo`03003ooooo00Woo`005?oo
00@00?ooool00?Woo`04003ooooo000;ool001Coo`04003ooooo0004ooo]0008ool01000ool00000
2ooo000Dool01000ooooo`001?oo00<00?ooool0K_oo00<00?ooool0M?oo00<00?ooool02_oo0P00
00?oo`00ool02_oo000Dool01000ooooo`001?oo00<00?ooool0l_oo00@00?ooool000_oo`005?oo
00@00?ooool000Coo`03003ooooo0?7oo`800003ool0000000_oo`005Ooo0P001Ooo00<00?ooool0
oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0>Coo`03003ooooo00;oo`03003o
oooo01Goo`007?oo00<00?ooool0__oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00_oo
00D00?ooooooo`0000Coo`03003ooooo00;oo`05003oooooool0000Qool001coo`03003ooooo09go
o`03003ooooo00;oo`03003ooooo00;oo`05003oooooool00004ool00`00ooooo`02ool00`00oooo
o`02ool00`00ooooo`14ool001coo`03003ooooo08Koo`05003oooooool00004ool00`00ooooo`02
ool00`00ooooo`02ool00`00ooooo`1Uool001coo`03003ooooo07?oo`05003oooooool00004ool0
0`00ooooo`02ool00`00ooooo`1mool001coo`03003ooooo063oo`03003ooooo00;oo`05003ooooo
ool00004ool00`00ooooo`2@ool001coo`03003ooooo05;oo`05003oooooool00004ool00`00oooo
o`2Sool001coo`03003ooooo04Coo`05003oooooool00004ool00`00ooooo`2aool001coo`03003o
oooo03[oo`03003ooooo00;oo`03003ooooo0;ooo`007?oo00<00?ooool0<Ooo00D00?ooooooo`00
0<_oo`007?oo00<00?ooool09ooo00<00?ooool00_oo00<00?ooool0d_oo000Lool00`00ooooo`0R
ool00`00ooooo`3Lool001coo`03003ooooo01koo`03003ooooo0>3oo`007?oo00<00?ooool0oooo
0_oo000Lool00`00ooooo`0Iool00`00ooooo`3Uool001coo`03003ooooo013oo`05003oooooool0
003/ool001coo`03003ooooo00_oo`03003ooooo0??oo`007?oo00<00?ooool0oooo0_oo000Lool0
0`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool01_oo00<00?ooool0
n?oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo
0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo
0_oo000Lool01@00oooooooo0000oooo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?oo
o`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?oo
o`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?oo
o`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?oo
o`;oo`007?oo00<00?ooool0oooo0_oo000>ool30004ool20005ool00`00ooooo`3oool2ool0017o
o`04003ooooo0002ool00`00ooooo`02ool00`00ooooo`3oool2ool0017oo`04003ooooo0002ool0
0`00ooooo`02ool3003oool2ool000koo`<000?oo`04003ooooo0004ool00`00ooooo`3oool2ool0
00koo`03003ooooo00?oo`04003ooooo0004ool00`00ooooo`3oool2ool000koo`03003ooooo00?o
o`04003ooooo0004ool00`00ooooo`3oool2ool000koo`@000?oo`8000Goo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03003ooooo0?ooo`;o
o`002?oo10000ooo0P001?oo0P001Ooo00<00?ooool0oooo0_oo000:ool01@00oooooooo00000_oo
00@00?ooool000;oo`03003ooooo00;oo`03003ooooo0?ooo`;oo`002_oo00D00?ooooooo`0000;o
o`04003ooooo0002ool00`00ooooo`02ool3003oool2ool000[oo`05003oooooool00002ool01000
ooooo`000_oo00<00?ooool00_oo00<00?ooool0oooo0_oo000:ool01@00oooooooo00000_oo00@0
0?ooool000;oo`03003ooooo00;oo`03003ooooo0?ooo`;oo`002?oo0`000ooo00@00?ooool000;o
o`04003ooooo0004ool00`00ooooo`3oool2ool000[oo`03003ooooo00;oo`8000Coo`8000Goo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`007?oo00<00?ooool0oooo0_oo000Lool00`00ooooo`3oool2ool001coo`03
003ooooo0?ooo`;oo`002?oo10000_oo0`001?oo0P001Ooo00<00?ooool0oooo0_oo000:ool00`00
ooooo`04ool01000ooooo`000_oo00<00?ooool00_oo00<00?ooool0oooo0_oo000:ool00`00oooo
o`04ool01000ooooo`000_oo00<00?ooool00_oo0`00oooo0_oo000:ool01000oooooooo0`000ooo
00@00?ooool00?ooo`Woo`002_oo00D00?ooooooo`0000Goo`04003ooooo003oool9ool000Soo`<0
00?oo`03003ooooo00?oo`04003ooooo003oool9ool000[oo`04003oooooool40003ool2003oool:
ool00?ooob7oo`00oooo8Ooo003ooolQool001[oo`03003ooooo0?ooo`Coo`002?oo1@000_oo1000
2?oo00@00?ooooooo`8000;oo`800003ool0000000800?Koo`002Ooo00<00?oo00000_oo00H00?oo
ool00?oo0007ool00`00ooooo`02ool01000ooooo`000_oo00@00?ooool00?Koo`002Ooo00<00?oo
00000_oo00H00?ooool00?oo0007ool00`00ooooo`02ool01000ooooo`000ooo00<00?ooool0m_oo
000:ool01@00oooooooo00001?oo00<00?ooool01_oo00@00?ooooooo`@000Coo`03003ooooo0?Go
o`002_oo00<00?ooool00_oo00<00?oo00000P002?oo00D00?ooooooo`0000;oo`03003ooooo00@0
0003ool0000000800?7oo`002_oo00<00?ooool00_oo0P0000?oo`00ool010001Ooo00@00?oo0000
00;oo`8000Ooo`800?;oo`005Ooo0P005ooo00<00?ooool0kooo000Gool00`00ooooo`0Bool3003a
ool001Goo`<00?ooo`Woo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {501.25, 359.938}} -> {-39.8108, \
450.626, 0.211968, 1.02891}}]
}, Open  ]],

Cell[TextData[{
  "Probably because ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]t\)]],
  " is larger than in the previous plot, this plot doesn't show any weird \
behavior of ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta] \( \[Omega]\&~\)\^2\)]],
  " for the first couple of values of ",
  Cell[BoxData[
      \(TraditionalForm\`N\)]],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plots of ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta] \( \[Omega]\&~\)\^2\)]],
  " with variable ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalDelta]t\)]]
}], "Subsubsection"],

Cell["\<\
Some default values (a B is appended to the variable names and \
functions to avoid naming conflicts):\
\>", "Text"],

Cell[BoxData[{
    \(\[Omega]B = 2  \[Pi] (*Hz*) ; \), "\[IndentingNewLine]", 
    \(maxNB = 51; \)}], "Input"],

Cell["\<\
Functions to create and display a list plot with the above default \
values:\
\>", "Text"],

Cell[BoxData[{
    \(\(createPlotB[\[Beta]p_, nn_] := 
        ParametricPlot3D[{\(3\/2\) 
              nn\ \[CapitalDelta]t, \[CapitalDelta]t, \
\[CapitalDelta]\[Omega]TildeSq[\[CapitalDelta]t, 
              nn, \[Beta]p, \[Omega]B]}, {\[CapitalDelta]t, 
            0.1  \[CapitalDelta]tMax[\[Omega]B], \
\[CapitalDelta]tMax[\[Omega]B]}, 
          DisplayFunction \[Rule] Identity];\)\), "\[IndentingNewLine]", 
    \(createPlotsB[\[Beta]p_] := 
      Table[createPlotB[\[Beta]p, nn], {nn, 1, maxNB, 
          1}]\), "\[IndentingNewLine]", 
    \(\(showPlotsB[plots_] := 
        Show[plots, DisplayFunction \[Rule] $DisplayFunction, 
          AxesLabel \[Rule] {"\<T/s\>", "\<\[CapitalDelta]t/s\>", \*"\"\<\
\[CapitalDelta]\!\(\[Omega]\&~\^2\)/\!\(Hz\^2\)\>\""}, 
          BoxRatios \[Rule] {1, 1, 1}, 
          PlotRange \[Rule] {{0, 
                3/2*0.1*51}, {0.1  \[CapitalDelta]tMax[
                    2  \[Pi]], \[CapitalDelta]tMax[2  \[Pi]]}, {0, 500}}, 
          ViewPoint -> {2.507, \ 2.135, \ 0.779}, 
          Ticks \[Rule] {{{0.001, 0}, 3, 6}, {0.1, 0.4, 
                0.7, {0.999, 1. }}, {100, 200, 300, 400}}];\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot with ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Beta]\^'\) = \(0/6 = 0\)\)]]
}], "Subsubsection"],

Cell[BoxData[
    \(plotsB1 = createPlotsB[0. /6]; \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(showPlotsB[plotsB1]; \)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .91501 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0262702 1.06936 -2.13046e-17 1.06936 [
[.4752 .92837 -3.00977 0 ]
[.4752 .92837 2.99023 9 ]
[.31156 .91504 -3.05802 0 ]
[.31156 .91504 2.94198 9 ]
[.11966 .89942 -3.11475 0 ]
[.11966 .89942 2.88525 9 ]
[.26027 .97781 -11.2665 0 ]
[.26027 .97781 10.7335 9.8125 ]
[.03465 .18507 -8.54876 -9 ]
[.03465 .18507 9.45124 0 ]
[.18551 .12753 -8.69057 -9 ]
[.18551 .12753 9.30943 0 ]
[.35735 .06198 -8.85423 -9 ]
[.35735 .06198 9.14577 0 ]
[.55418 -0.0131 -6.02969 -9 ]
[.55418 -0.0131 5.97031 0 ]
[.27026 .02904 -13.6412 -9.8125 ]
[.27026 .02904 14.3588 0 ]
[.01476 .32278 -18 -5.85643 ]
[.01476 .32278 0 3.14357 ]
[.00798 .45516 -18 -5.61724 ]
[.00798 .45516 0 3.38276 ]
[.00104 .59125 -18 -5.3727 ]
[.00104 .59125 0 3.6273 ]
[-0.00606 .7312 -18 -5.12262 ]
[-0.00606 .7312 0 3.87738 ]
[-0.06073 .5083 -44.625 -7.93817 ]
[-0.06073 .5083 0 5.06183 ]
[ 0 0 0 0 ]
[ 1 .91501 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.4753 .91501 m
0 .87626 L
s
.47525 .915 m
.47527 .90832 L
s
[(0)] .4752 .92837 .00326 -1 Mshowa
.31182 .90168 m
.31195 .895 L
s
[(3)] .31156 .91504 .01934 -1 Mshowa
.12017 .88606 m
.12043 .87938 L
s
[(6)] .11966 .89942 .03825 -1 Mshowa
gsave
.26027 .97781 -72.2665 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(T) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.03399 .19842 m
.55496 0 L
s
.03399 .19842 m
.03365 .2051 L
s
[(0.1)] .03465 .18507 -0.05014 1 Mshowa
.18505 .14089 m
.18482 .14756 L
s
[(0.4)] .18551 .12753 -0.03438 1 Mshowa
.35714 .07534 m
.35703 .08203 L
s
[(0.7)] .35735 .06198 -0.0162 1 Mshowa
.55425 .00027 m
.55428 .00695 L
s
[(1.)] .55418 -0.0131 .00495 1 Mshowa
gsave
.27026 .02904 -74.6412 -13.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(t) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
75.000 11.250 moveto
(\\220) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
87.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.03399 .19842 m
0 .87626 L
s
.02756 .32664 m
.03396 .32857 L
s
[(100)] .01476 .32278 1 .30143 Mshowa
.02095 .45839 m
.02744 .46 L
s
[(200)] .00798 .45516 1 .24828 Mshowa
.01416 .5938 m
.02072 .59507 L
s
[(300)] .00104 .59125 1 .19393 Mshowa
.00718 .73304 m
.0138 .73395 L
s
[(400)] -0.00606 .7312 1 .13836 Mshowa
gsave
-0.06073 .5083 -105.625 -11.9382 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.47712 .35504 m
.4753 .91501 L
s
.4753 .91501 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
.96816 .22579 m
.47712 .35504 L
s
.03399 .19842 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.47712 .35504 m
.4753 .91501 L
s
.4753 .91501 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.03399 .19842 m
.47712 .35504 L
s
.96816 .22579 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
0 0 m
1 0 L
1 .91501 L
0 .91501 L
closepath
clip
newpath
.5 Mabswid
.46994 .3525 m
.47472 .3507 L
s
.47472 .3507 m
.47952 .34888 L
s
.46272 .34995 m
.46661 .34782 L
s
.47952 .34888 m
.48434 .34705 L
s
.46661 .34782 m
.47053 .34567 L
s
.45544 .34738 m
.45844 .34491 L
s
.48434 .34705 m
.48919 .34522 L
s
.47053 .34567 m
.47446 .34351 L
s
.48919 .34522 m
.49406 .34337 L
s
.45844 .34491 m
.46145 .34243 L
s
.44811 .34479 m
.4502 .34199 L
s
.47446 .34351 m
.47843 .34134 L
s
.49406 .34337 m
.49896 .34152 L
s
.46145 .34243 m
.46449 .33993 L
s
.44074 .34218 m
.4419 .33904 L
s
.4502 .34199 m
.4523 .33917 L
s
.49896 .34152 m
.50388 .33966 L
s
.47843 .34134 m
.48241 .33916 L
s
.46449 .33993 m
.46755 .33742 L
s
.50388 .33966 m
.50883 .33779 L
s
.4333 .33955 m
.43353 .33607 L
s
.4523 .33917 m
.45442 .33632 L
s
.48241 .33916 m
.48642 .33696 L
s
.4419 .33904 m
.44307 .33587 L
s
.50883 .33779 m
.5138 .33591 L
s
.46755 .33742 m
.47062 .33489 L
s
.48642 .33696 m
.49045 .33475 L
s
.42582 .33691 m
.42509 .33307 L
s
.45442 .33632 m
.45655 .33346 L
s
.43353 .33607 m
.43375 .33255 L
s
.5138 .33591 m
.51879 .33401 L
s
.44307 .33587 m
.44425 .33268 L
s
.47062 .33489 m
.47372 .33234 L
s
.49045 .33475 m
.49451 .33252 L
s
.41828 .33424 m
.41658 .33005 L
s
.51879 .33401 m
.52382 .33211 L
s
.45655 .33346 m
.4587 .33057 L
s
.42509 .33307 m
.42435 .32919 L
s
.44425 .33268 m
.44544 .32945 L
s
.43375 .33255 m
.43398 .32899 L
s
.49451 .33252 m
.49859 .33029 L
s
.47372 .33234 m
.47684 .32977 L
s
.52382 .33211 m
.52886 .3302 L
s
.41069 .33156 m
.40801 .327 L
s
.4587 .33057 m
.46087 .32766 L
s
.49859 .33029 m
.50269 .32804 L
s
.52886 .3302 m
.53393 .32828 L
s
.41658 .33005 m
.41487 .32581 L
s
.47684 .32977 m
.47998 .32719 L
s
.44544 .32945 m
.44664 .3262 L
s
.42435 .32919 m
.42361 .32527 L
s
.43398 .32899 m
.43421 .32541 L
s
.40304 .32886 m
.39937 .32394 L
s
.53393 .32828 m
.53903 .32636 L
s
.50269 .32804 m
.50682 .32578 L
s
.46087 .32766 m
.46305 .32473 L
s
.47998 .32719 m
.48315 .32459 L
s
.40801 .327 m
.4053 .32239 L
s
.53903 .32636 m
.54416 .32442 L
s
.44664 .3262 m
.44785 .32293 L
s
.39534 .32614 m
.39065 .32084 L
s
.41487 .32581 m
.41313 .32152 L
s
.50682 .32578 m
.51097 .3235 L
s
.43421 .32541 m
.43444 .32179 L
s
.42361 .32527 m
.42285 .32132 L
s
.46305 .32473 m
.46525 .32177 L
s
.48315 .32459 m
.48633 .32197 L
s
.54416 .32442 m
.54931 .32247 L
s
.39937 .32394 m
.39564 .31895 L
s
.51097 .3235 m
.51515 .32121 L
s
.38758 .32339 m
.38187 .31772 L
s
.44785 .32293 m
.44907 .31962 L
s
.54931 .32247 m
.55449 .32051 L
s
.4053 .32239 m
.40255 .31773 L
s
.48633 .32197 m
.48954 .31933 L
s
.43444 .32179 m
.43467 .31813 L
s
.46525 .32177 m
.46747 .3188 L
s
.41313 .32152 m
.41138 .31718 L
s
.42285 .32132 m
.42209 .31732 L
s
.51515 .32121 m
.51936 .3189 L
s
.39065 .32084 m
.3859 .31547 L
s
.37977 .32063 m
.37301 .31458 L
s
.55449 .32051 m
.55969 .31854 L
s
.44907 .31962 m
.45031 .31628 L
s
.48954 .31933 m
.49277 .31667 L
s
.39564 .31895 m
.39187 .31389 L
s
.51936 .3189 m
.52359 .31658 L
s
.46747 .3188 m
.4697 .3158 L
s
.55969 .31854 m
.56492 .31656 L
s
.43467 .31813 m
.43491 .31444 L
s
.3719 .31785 m
.36408 .3114 L
s
.40255 .31773 m
.39977 .313 L
s
.38187 .31772 m
.37607 .31196 L
s
.42209 .31732 m
.42132 .31327 L
s
.41138 .31718 m
.4096 .31279 L
s
.49277 .31667 m
.49602 .314 L
s
.52359 .31658 m
.52784 .31425 L
s
.56492 .31656 m
.57018 .31457 L
s
.45031 .31628 m
.45155 .31292 L
s
.4697 .3158 m
.47196 .31277 L
s
.3859 .31547 m
.38108 .31003 L
s
.36397 .31505 m
.35507 .30821 L
s
.37301 .31458 m
.36614 .30842 L
s
.57018 .31457 m
.57547 .31257 L
s
.43491 .31444 m
.43514 .31071 L
s
.52784 .31425 m
.53212 .31191 L
s
.49602 .314 m
.49929 .3113 L
s
.39187 .31389 m
.38805 .30877 L
s
.42132 .31327 m
.42055 .30919 L
s
.39977 .313 m
.39696 .30822 L
s
.45155 .31292 m
.4528 .30952 L
s
.4096 .31279 m
.4078 .30835 L
s
.47196 .31277 m
.47423 .30972 L
s
.57547 .31257 m
.58078 .31056 L
s
.35598 .31223 m
.34599 .30498 L
s
.37607 .31196 m
.37018 .30612 L
s
.53212 .31191 m
.53643 .30954 L
s
.36408 .3114 m
.35613 .30485 L
s
.49929 .3113 m
.50259 .30859 L
s
.43514 .31071 m
.43538 .30695 L
s
.58078 .31056 m
.58612 .30853 L
s
.38108 .31003 m
.37619 .3045 L
s
.47423 .30972 m
.47651 .30665 L
s
.53643 .30954 m
.54077 .30717 L
s
.4528 .30952 m
.45407 .3061 L
s
.34794 .30938 m
.33684 .30173 L
s
.42055 .30919 m
.41976 .30506 L
s
.38805 .30877 m
.38417 .30359 L
s
.50259 .30859 m
.50591 .30586 L
s
.58612 .30853 m
.59149 .3065 L
s
.36614 .30842 m
.35916 .30217 L
s
.4078 .30835 m
.40599 .30386 L
s
.39696 .30822 m
.39411 .30337 L
s
.35507 .30821 m
.34602 .30124 L
s
.54077 .30717 m
.54513 .30478 L
s
.43538 .30695 m
.43563 .30315 L
s
.47651 .30665 m
.47882 .30355 L
s
.33983 .30652 m
.32761 .29845 L
s
.59149 .3065 m
.59689 .30446 L
s
.37018 .30612 m
.3642 .30018 L
s
.45407 .3061 m
.45535 .30264 L
s
.50591 .30586 m
.50925 .30311 L
s
.41976 .30506 m
.41897 .30089 L
s
.34599 .30498 m
.33582 .2976 L
s
.35613 .30485 m
.34804 .29818 L
s
.54513 .30478 m
.54951 .30237 L
s
.37619 .3045 m
.37123 .2989 L
s
.59689 .30446 m
.60231 .30241 L
s
.40599 .30386 m
.40415 .29932 L
s
.33166 .30363 m
.3183 .29515 L
s
.38417 .30359 m
.38024 .29833 L
s
.47882 .30355 m
.48115 .30043 L
s
.39411 .30337 m
.39123 .29847 L
s
.43563 .30315 m
.43587 .29931 L
s
.50925 .30311 m
.51262 .30034 L
s
.45535 .30264 m
.45663 .29916 L
s
.60231 .30241 m
.60777 .30034 L
s
.54951 .30237 m
.55393 .29995 L
s
.35916 .30217 m
.35207 .29581 L
s
.33684 .30173 m
.32552 .29393 L
s
.34602 .30124 m
.3368 .29415 L
s
.41897 .30089 m
.41816 .29667 L
s
.32344 .30072 m
.30891 .29181 L
s
.48115 .30043 m
.48349 .29728 L
s
.60777 .30034 m
.61325 .29827 L
s
.51262 .30034 m
.51601 .29755 L
s
.3642 .30018 m
.35812 .29415 L
s
.55393 .29995 m
.55837 .29752 L
s
.40415 .29932 m
.40229 .29473 L
s
.43587 .29931 m
.43612 .29543 L
s
.45663 .29916 m
.45793 .29564 L
s
.37123 .2989 m
.36619 .29321 L
s
.39123 .29847 m
.38831 .2935 L
s
.32761 .29845 m
.31513 .29022 L
s
.38024 .29833 m
.37626 .293 L
s
.61325 .29827 m
.61876 .29618 L
s
.34804 .29818 m
.33981 .2914 L
s
.31515 .29779 m
.29944 .28845 L
s
.33582 .2976 m
.32545 .29008 L
s
.51601 .29755 m
.51943 .29474 L
s
.55837 .29752 m
.56284 .29507 L
s
.48349 .29728 m
.48585 .29411 L
s
.41816 .29667 m
.41735 .2924 L
s
.61876 .29618 m
.62431 .29408 L
s
.35207 .29581 m
.34486 .28935 L
s
.45793 .29564 m
.45925 .29209 L
s
.43612 .29543 m
.43637 .29152 L
s
.3183 .29515 m
.30463 .28647 L
s
.56284 .29507 m
.56734 .2926 L
s
.3068 .29484 m
.28989 .28506 L
s
.51943 .29474 m
.52287 .29191 L
s
.40229 .29473 m
.40041 .29008 L
s
.35812 .29415 m
.35196 .28803 L
s
.3368 .29415 m
.32741 .28693 L
s
.48585 .29411 m
.48823 .29091 L
s
.62431 .29408 m
.62988 .29197 L
s
.32552 .29393 m
.31397 .28596 L
s
.38831 .2935 m
.38535 .28847 L
s
.36619 .29321 m
.36109 .28745 L
s
.37626 .293 m
.37223 .28759 L
s
.56734 .2926 m
.57187 .29012 L
s
.41735 .2924 m
.41653 .28809 L
s
.45925 .29209 m
.46057 .28851 L
s
.62988 .29197 m
.63548 .28985 L
s
.52287 .29191 m
.52633 .28906 L
s
.29838 .29187 m
.28026 .28164 L
s
.30891 .29181 m
.29404 .28269 L
s
.43637 .29152 m
.43662 .28757 L
s
.33981 .2914 m
.33144 .28449 L
s
.48823 .29091 m
.49063 .28768 L
s
.31513 .29022 m
.30238 .28181 L
s
.57187 .29012 m
.57642 .28762 L
s
.32545 .29008 m
.31487 .28241 L
s
.40041 .29008 m
.3985 .28537 L
s
.63548 .28985 m
.64111 .28772 L
s
.34486 .28935 m
.33754 .28279 L
s
.52633 .28906 m
.52982 .28618 L
s
.28991 .28887 m
.27054 .27819 L
s
.46057 .28851 m
.46191 .2849 L
s
.38535 .28847 m
.38235 .28338 L
s
.29944 .28845 m
.28335 .27888 L
s
.41653 .28809 m
.4157 .28373 L
s
.35196 .28803 m
.3457 .28182 L
s
.64111 .28772 m
.64677 .28558 L
s
.49063 .28768 m
.49306 .28443 L
s
.57642 .28762 m
.58101 .28511 L
s
.37223 .28759 m
.36814 .28211 L
s
.43662 .28757 m
.43688 .28358 L
s
.36109 .28745 m
.35591 .28159 L
s
.32741 .28693 m
.31785 .27957 L
s
.30463 .28647 m
.29067 .27761 L
s
.52982 .28618 m
.53334 .28329 L
s
.31397 .28596 m
.30219 .27784 L
s
.28137 .28585 m
.26074 .27471 L
s
.64677 .28558 m
.65247 .28342 L
s
.3985 .28537 m
.39658 .28061 L
s
.58101 .28511 m
.58562 .28258 L
s
.28989 .28506 m
.27255 .27503 L
s
.46191 .2849 m
.46325 .28125 L
s
.33144 .28449 m
.32292 .27747 L
s
.49306 .28443 m
.4955 .28115 L
s
.4157 .28373 m
.41487 .27933 L
s
.43688 .28358 m
.43713 .27954 L
s
.65247 .28342 m
.65819 .28126 L
s
.38235 .28338 m
.37932 .27822 L
s
.53334 .28329 m
.53688 .28038 L
s
.27276 .28281 m
.25086 .2712 L
s
.33754 .28279 m
.33009 .27611 L
s
.29404 .28269 m
.27883 .27336 L
s
.58562 .28258 m
.59026 .28004 L
s
.31487 .28241 m
.3041 .27459 L
s
.36814 .28211 m
.36399 .27656 L
s
.3457 .28182 m
.33934 .27551 L
s
.30238 .28181 m
.28936 .27322 L
s
.28026 .28164 m
.26166 .27114 L
s
.35591 .28159 m
.35065 .27566 L
s
.65819 .28126 m
.66394 .27908 L
s
.46325 .28125 m
.46461 .27757 L
s
.4955 .28115 m
.49796 .27785 L
s
.39658 .28061 m
.39463 .27579 L
s
.53688 .28038 m
.54045 .27744 L
s
.59026 .28004 m
.59493 .27748 L
s
.26409 .27975 m
.24089 .26766 L
s
.31785 .27957 m
.30811 .27208 L
s
.43713 .27954 m
.43739 .27547 L
s
.41487 .27933 m
.41402 .27487 L
s
.66394 .27908 m
.66973 .27689 L
s
.28335 .27888 m
.26687 .26907 L
s
.37932 .27822 m
.37624 .27299 L
s
.27054 .27819 m
.25065 .26721 L
s
.49796 .27785 m
.50044 .27452 L
s
.30219 .27784 m
.29017 .26955 L
s
.29067 .27761 m
.27639 .26854 L
s
.46461 .27757 m
.46599 .27386 L
s
.59493 .27748 m
.59963 .2749 L
s
.32292 .27747 m
.31425 .27032 L
s
.54045 .27744 m
.54404 .27449 L
s
.66973 .27689 m
.67555 .27469 L
s
.25535 .27666 m
.23083 .26409 L
s
.36399 .27656 m
.35978 .27093 L
s
.33009 .27611 m
.32251 .26932 L
s
.39463 .27579 m
.39266 .27092 L
s
.35065 .27566 m
.34531 .26963 L
s
.33934 .27551 m
.33287 .26909 L
s
.43739 .27547 m
.43766 .27135 L
s
.27255 .27503 m
.25478 .26474 L
s
.59963 .2749 m
.60437 .27231 L
s
.41402 .27487 m
.41316 .27037 L
s
.26074 .27471 m
.23954 .26324 L
s
.67555 .27469 m
.6814 .27248 L
s
.3041 .27459 m
.2931 .26662 L
s
.50044 .27452 m
.50294 .27116 L
s
.54404 .27449 m
.54766 .27151 L
s
.46599 .27386 m
.46737 .27011 L
s
.24654 .27354 m
.22069 .26048 L
s
.27883 .27336 m
.26326 .26381 L
s
.28936 .27322 m
.27606 .26444 L
s
.37624 .27299 m
.37313 .26769 L
s
.6814 .27248 m
.68728 .27025 L
s
.60437 .27231 m
.60913 .2697 L
s
.30811 .27208 m
.29819 .26445 L
s
.54766 .27151 m
.55131 .26851 L
s
.43766 .27135 m
.43792 .26719 L
s
.25086 .2712 m
.22832 .25924 L
s
.50294 .27116 m
.50546 .26777 L
s
.26166 .27114 m
.24256 .26036 L
s
.35978 .27093 m
.35552 .26522 L
s
.39266 .27092 m
.39066 .26598 L
s
.23766 .27041 m
.21045 .25685 L
s
.41316 .27037 m
.4123 .26581 L
s
.31425 .27032 m
.30543 .26305 L
s
.68728 .27025 m
.69319 .26801 L
s
.46737 .27011 m
.46877 .26632 L
s
.60913 .2697 m
.61392 .26707 L
s
.34531 .26963 m
.33989 .26351 L
s
.29017 .26955 m
.27789 .26109 L
s
.32251 .26932 m
.31481 .26242 L
s
.33287 .26909 m
.32631 .26257 L
s
.26687 .26907 m
.24998 .25903 L
s
.27639 .26854 m
.26178 .25927 L
s
.55131 .26851 m
.55498 .26549 L
s
.69319 .26801 m
.69914 .26576 L
s
.50546 .26777 m
.50801 .26435 L
s
.37313 .26769 m
.36997 .26233 L
s
.24089 .26766 m
.21699 .2552 L
s
.22872 .26725 m
.20013 .25318 L
s
.25065 .26721 m
.23021 .25593 L
s
.43792 .26719 m
.43819 .26299 L
s
.61392 .26707 m
.61874 .26443 L
s
.2931 .26662 m
.28189 .25849 L
s
.46877 .26632 m
.47018 .2625 L
s
.39066 .26598 m
.38864 .26098 L
s
.4123 .26581 m
.41142 .26121 L
s
.69914 .26576 m
.70512 .2635 L
s
.55498 .26549 m
.55868 .26244 L
s
.35552 .26522 m
.3512 .25943 L
s
.25478 .26474 m
.23654 .25419 L
s
.29819 .26445 m
.28808 .25667 L
s
.27606 .26444 m
.26247 .25547 L
s
.61874 .26443 m
.6236 .26176 L
s
.50801 .26435 m
.51057 .26091 L
s
.23083 .26409 m
.20555 .25112 L
s
.21971 .26406 m
.18971 .24948 L
s
.26326 .26381 m
.24732 .25404 L
s
.33989 .26351 m
.33439 .2573 L
s
.70512 .2635 m
.71113 .26122 L
s
.23954 .26324 m
.21772 .25145 L
s
.30543 .26305 m
.29644 .25564 L
s
.43819 .26299 m
.43846 .25875 L
s
.32631 .26257 m
.31964 .25595 L
s
.47018 .2625 m
.47161 .25865 L
s
.55868 .26244 m
.56241 .25937 L
s
.31481 .26242 m
.30697 .2554 L
s
.36997 .26233 m
.36678 .25689 L
s
.6236 .26176 m
.62849 .25909 L
s
.71113 .26122 m
.71718 .25893 L
s
.41142 .26121 m
.41053 .25655 L
s
.27789 .26109 m
.26536 .25244 L
s
.38864 .26098 m
.38659 .25593 L
s
.51057 .26091 m
.51316 .25743 L
s
.21062 .26085 m
.1792 .24575 L
s
.22069 .26048 m
.19399 .24699 L
s
.24256 .26036 m
.22295 .24929 L
s
.3512 .25943 m
.34681 .25355 L
s
.56241 .25937 m
.56617 .25628 L
s
.26178 .25927 m
.24684 .24979 L
s
.22832 .25924 m
.2051 .24693 L
s
.62849 .25909 m
.63341 .25639 L
s
.24998 .25903 m
.23267 .24873 L
s
.71718 .25893 m
.72326 .25663 L
s
.43846 .25875 m
.43873 .25446 L
s
.47161 .25865 m
.47304 .25476 L
s
.28189 .25849 m
.27046 .2502 L
s
.20146 .25761 m
.16859 .24198 L
s
.51316 .25743 m
.51577 .25393 L
s
.33439 .2573 m
.32881 .25099 L
s
.36678 .25689 m
.36354 .25138 L
s
.21045 .25685 m
.18232 .24283 L
s
.28808 .25667 m
.27777 .24874 L
s
.72326 .25663 m
.72937 .25432 L
s
.41053 .25655 m
.40964 .25184 L
s
.63341 .25639 m
.63836 .25367 L
s
.56617 .25628 m
.56995 .25317 L
s
.31964 .25595 m
.31286 .24922 L
s
.23021 .25593 m
.20919 .24433 L
s
.38659 .25593 m
.38452 .25081 L
s
.29644 .25564 m
.2873 .2481 L
s
.26247 .25547 m
.24858 .24631 L
s
.30697 .2554 m
.299 .24826 L
s
.21699 .2552 m
.19234 .24235 L
s
.47304 .25476 m
.4745 .25083 L
s
.43873 .25446 m
.43901 .25013 L
s
.19223 .25435 m
.15789 .23818 L
s
.72937 .25432 m
.73552 .25199 L
s
.23654 .25419 m
.21783 .24336 L
s
.24732 .25404 m
.23099 .24402 L
s
.51577 .25393 m
.5184 .2504 L
s
.63836 .25367 m
.64334 .25094 L
s
.34681 .25355 m
.34236 .24759 L
s
.20013 .25318 m
.17053 .23862 L
s
.56995 .25317 m
.57377 .25003 L
s
.26536 .25244 m
.25256 .24362 L
s
.73552 .25199 m
.7417 .24965 L
s
.40964 .25184 m
.40873 .24707 L
s
.21772 .25145 m
.19527 .23932 L
s
.36354 .25138 m
.36025 .2458 L
s
.20555 .25112 m
.17945 .23773 L
s
.18293 .25106 m
.14709 .23435 L
s
.32881 .25099 m
.32314 .24459 L
s
.64334 .25094 m
.64836 .24819 L
s
.4745 .25083 m
.47596 .24687 L
s
.38452 .25081 m
.38242 .24562 L
s
.5184 .2504 m
.52105 .24683 L
s
.27046 .2502 m
.25879 .24173 L
s
.43901 .25013 m
.43929 .24575 L
s
.57377 .25003 m
.57761 .24687 L
s
.24684 .24979 m
.23155 .24008 L
s
.7417 .24965 m
.74792 .2473 L
s
.18971 .24948 m
.15863 .23437 L
s
.22295 .24929 m
.2028 .23792 L
s
.31286 .24922 m
.30597 .24238 L
s
.27777 .24874 m
.26727 .24066 L
s
.23267 .24873 m
.21492 .23817 L
s
.299 .24826 m
.29089 .24099 L
s
.64836 .24819 m
.65341 .24542 L
s
.2873 .2481 m
.27798 .24041 L
s
.17356 .24775 m
.13619 .23048 L
s
.34236 .24759 m
.33785 .24155 L
s
.74792 .2473 m
.75417 .24493 L
s
.40873 .24707 m
.40781 .24225 L
s
.19399 .24699 m
.1664 .23305 L
s
.2051 .24693 m
.18118 .23424 L
s
.57761 .24687 m
.58148 .24368 L
s
.47596 .24687 m
.47744 .24286 L
s
.52105 .24683 m
.52373 .24324 L
s
.24858 .24631 m
.23437 .23694 L
s
.36025 .2458 m
.35693 .24014 L
s
.43929 .24575 m
.43957 .24133 L
s
.1792 .24575 m
.1466 .23008 L
s
.38242 .24562 m
.3803 .24037 L
s
.65341 .24542 m
.65849 .24264 L
s
.75417 .24493 m
.76046 .24255 L
s
.32314 .24459 m
.31739 .23809 L
s
.16411 .24441 m
.1252 .22657 L
s
.20919 .24433 m
.18758 .23241 L
s
.23099 .24402 m
.21426 .23376 L
s
.58148 .24368 m
.58538 .24048 L
s
.25256 .24362 m
.23948 .2346 L
s
.21783 .24336 m
.19863 .23225 L
s
.52373 .24324 m
.52643 .23961 L
s
.47744 .24286 m
.47893 .23882 L
s
.18232 .24283 m
.15322 .22832 L
s
.65849 .24264 m
.66361 .23983 L
s
.76046 .24255 m
.76678 .24016 L
s
.30597 .24238 m
.29896 .23543 L
s
.19234 .24235 m
.16692 .2291 L
s
.40781 .24225 m
.40689 .23738 L
s
.16859 .24198 m
.13445 .22575 L
s
.25879 .24173 m
.24688 .2331 L
s
.33785 .24155 m
.33327 .23542 L
s
.43957 .24133 m
.43986 .23685 L
s
.15458 .24104 m
.1141 .22263 L
s
.29089 .24099 m
.28264 .23359 L
s
.26727 .24066 m
.25656 .23242 L
s
.58538 .24048 m
.58931 .23724 L
s
.27798 .24041 m
.26849 .23259 L
s
.3803 .24037 m
.37815 .23506 L
s
.76678 .24016 m
.77315 .23775 L
s
.35693 .24014 m
.35355 .2344 L
s
.23155 .24008 m
.2159 .23015 L
s
.66361 .23983 m
.66876 .23701 L
s
.52643 .23961 m
.52915 .23596 L
s
.19527 .23932 m
.17215 .22682 L
s
.47893 .23882 m
.48044 .23474 L
s
.17053 .23862 m
.13988 .22354 L
s
.15789 .23818 m
.12218 .22137 L
s
.21492 .23817 m
.19672 .22735 L
s
.31739 .23809 m
.31154 .23148 L
s
.2028 .23792 m
.1821 .22623 L
s
.77315 .23775 m
.77954 .23533 L
s
.17945 .23773 m
.15249 .2239 L
s
.14497 .23765 m
.1029 .21865 L
s
.40689 .23738 m
.40595 .23244 L
s
.58931 .23724 m
.59327 .23398 L
s
.66876 .23701 m
.67395 .23417 L
s
.23437 .23694 m
.21985 .22735 L
s
.43986 .23685 m
.44015 .23233 L
s
.52915 .23596 m
.5319 .23227 L
s
.29896 .23543 m
.29184 .22836 L
s
.33327 .23542 m
.32862 .22919 L
s
.77954 .23533 m
.78598 .2329 L
s
.37815 .23506 m
.37597 .22968 L
s
.48044 .23474 m
.48196 .23062 L
s
.23948 .2346 m
.22613 .22539 L
s
.35355 .2344 m
.35013 .22859 L
s
.15863 .23437 m
.1264 .21871 L
s
.14709 .23435 m
.10978 .21694 L
s
.18118 .23424 m
.15652 .22116 L
s
.13529 .23423 m
.09159 .21464 L
s
.67395 .23417 m
.67917 .2313 L
s
.59327 .23398 m
.59727 .2307 L
s
.21426 .23376 m
.19712 .22325 L
s
.28264 .23359 m
.27424 .22607 L
s
.24688 .2331 m
.23473 .22428 L
s
.1664 .23305 m
.13788 .21864 L
s
.78598 .2329 m
.79245 .23045 L
s
.26849 .23259 m
.25882 .22462 L
s
.40595 .23244 m
.405 .22745 L
s
.25656 .23242 m
.24564 .22402 L
s
.18758 .23241 m
.16534 .22013 L
s
.44015 .23233 m
.44044 .22777 L
s
.5319 .23227 m
.53467 .22855 L
s
.19863 .23225 m
.17891 .22083 L
s
.31154 .23148 m
.3056 .22478 L
s
.67917 .2313 m
.68443 .22842 L
s
.12553 .23078 m
.08018 .21059 L
s
.59727 .2307 m
.60129 .22739 L
s
.48196 .23062 m
.4835 .22646 L
s
.13619 .23048 m
.09726 .21247 L
s
.79245 .23045 m
.79895 .22798 L
s
.2159 .23015 m
.19988 .21998 L
s
.1466 .23008 m
.11276 .21382 L
s
.37597 .22968 m
.37376 .22422 L
s
.32862 .22919 m
.3239 .22288 L
s
.16692 .2291 m
.14069 .21543 L
s
.35013 .22859 m
.34667 .22269 L
s
.53467 .22855 m
.53746 .22479 L
s
.68443 .22842 m
.68972 .22552 L
s
.29184 .22836 m
.2846 .22117 L
s
.15322 .22832 m
.12309 .21331 L
s
.79895 .22798 m
.8055 .22551 L
s
.44044 .22777 m
.44073 .22315 L
s
.405 .22745 m
.40404 .2224 L
s
.60129 .22739 m
.60535 .22405 L
s
.21985 .22735 m
.20499 .21755 L
s
.19672 .22735 m
.17804 .21624 L
s
.11569 .2273 m
.06866 .2065 L
s
.17215 .22682 m
.14834 .21395 L
s
.1252 .22657 m
.0846 .20796 L
s
.4835 .22646 m
.48506 .22226 L
s
.1821 .22623 m
.16082 .21422 L
s
.27424 .22607 m
.26569 .21841 L
s
.13445 .22575 m
.09897 .20887 L
s
.68972 .22552 m
.69505 .2226 L
s
.8055 .22551 m
.81208 .22301 L
s
.22613 .22539 m
.21247 .21598 L
s
.53746 .22479 m
.54028 .22101 L
s
.3056 .22478 m
.29957 .21796 L
s
.25882 .22462 m
.24897 .2165 L
s
.23473 .22428 m
.22232 .21528 L
s
.37376 .22422 m
.37153 .2187 L
s
.60535 .22405 m
.60943 .22069 L
s
.24564 .22402 m
.2345 .21545 L
s
.15249 .2239 m
.12464 .20962 L
s
.10577 .22379 m
.05704 .20237 L
s
.13988 .22354 m
.10812 .20791 L
s
.19712 .22325 m
.17955 .21247 L
s
.44073 .22315 m
.44103 .21848 L
s
.81208 .22301 m
.81871 .22051 L
s
.3239 .22288 m
.31912 .21647 L
s
.34667 .22269 m
.34315 .21672 L
s
.1141 .22263 m
.07181 .20339 L
s
.69505 .2226 m
.70041 .21966 L
s
.40404 .2224 m
.40306 .21729 L
s
.48506 .22226 m
.48662 .21801 L
s
.12218 .22137 m
.08502 .20387 L
s
.2846 .22117 m
.27723 .21386 L
s
.15652 .22116 m
.1311 .20767 L
s
.54028 .22101 m
.54313 .21718 L
s
.17891 .22083 m
.15865 .20911 L
s
.60943 .22069 m
.61355 .2173 L
s
.81871 .22051 m
.82537 .21799 L
s
.09577 .22026 m
.0453 .1982 L
s
.16534 .22013 m
.14245 .2075 L
s
.19988 .21998 m
.18346 .20956 L
s
.70041 .21966 m
.70582 .2167 L
s
.1264 .21871 m
.09296 .20245 L
s
.37153 .2187 m
.36926 .21311 L
s
.1029 .21865 m
.05889 .19878 L
s
.13788 .21864 m
.10837 .20373 L
s
.44103 .21848 m
.44133 .21376 L
s
.26569 .21841 m
.25699 .21061 L
s
.48662 .21801 m
.48821 .21373 L
s
.82537 .21799 m
.83207 .21545 L
s
.29957 .21796 m
.29344 .21104 L
s
.20499 .21755 m
.18979 .20752 L
s
.61355 .2173 m
.6177 .21389 L
s
.40306 .21729 m
.40208 .21212 L
s
.54313 .21718 m
.546 .21333 L
s
.10978 .21694 m
.07091 .19881 L
s
.34315 .21672 m
.33959 .21066 L
s
.70582 .2167 m
.71125 .21372 L
s
.03914 .19646 m
.08569 .21669 L
s
.24897 .2165 m
.23894 .20822 L
s
.31912 .21647 m
.31426 .20996 L
s
.17804 .21624 m
.15888 .20484 L
s
.21247 .21598 m
.19852 .20636 L
s
.2345 .21545 m
.22313 .20671 L
s
.83207 .21545 m
.83881 .2129 L
s
.14069 .21543 m
.1136 .20131 L
s
.22232 .21528 m
.20965 .20609 L
s
.09159 .21464 m
.04583 .19412 L
s
.16082 .21422 m
.13893 .20187 L
s
.14834 .21395 m
.1238 .20068 L
s
.6177 .21389 m
.62189 .21044 L
s
.27723 .21386 m
.26974 .20643 L
s
.11276 .21382 m
.07761 .19692 L
s
.44133 .21376 m
.44164 .20899 L
s
.48821 .21373 m
.48981 .2094 L
s
.71125 .21372 m
.71673 .21072 L
s
.546 .21333 m
.5489 .20944 L
s
.12309 .21331 m
.09189 .19776 L
s
.36926 .21311 m
.36697 .20744 L
s
.03802 .19689 m
.07552 .2131 L
s
.83881 .2129 m
.84559 .21033 L
s
.09726 .21247 m
.05664 .19369 L
s
.17955 .21247 m
.16154 .20142 L
s
.40208 .21212 m
.40109 .20689 L
s
.29344 .21104 m
.28721 .204 L
s
.71673 .21072 m
.72225 .20769 L
s
.33959 .21066 m
.33598 .20451 L
s
.25699 .21061 m
.24813 .20267 L
s
.04416 .19455 m
.08018 .21059 L
s
.62189 .21044 m
.6261 .20697 L
s
.84559 .21033 m
.85241 .20775 L
s
.31426 .20996 m
.30933 .20336 L
s
.12464 .20962 m
.09584 .19485 L
s
.18346 .20956 m
.16665 .19889 L
s
.03694 .1973 m
.06526 .20948 L
s
.5489 .20944 m
.55182 .20552 L
s
.48981 .2094 m
.49142 .20503 L
s
.15865 .20911 m
.13783 .19706 L
s
.44164 .20899 m
.44194 .20417 L
s
.09897 .20887 m
.06206 .19132 L
s
.23894 .20822 m
.2287 .19979 L
s
.05027 .19222 m
.0846 .20796 L
s
.10812 .20791 m
.07517 .19171 L
s
.85241 .20775 m
.85926 .20516 L
s
.72225 .20769 m
.7278 .20465 L
s
.1311 .20767 m
.10486 .19376 L
s
.18979 .20752 m
.17423 .19725 L
s
.14245 .2075 m
.11889 .1945 L
s
.36697 .20744 m
.36465 .2017 L
s
.6261 .20697 m
.63035 .20348 L
s
.40109 .20689 m
.40008 .20159 L
s
.22313 .20671 m
.21154 .19779 L
s
.04281 .19506 m
.06866 .2065 L
s
.26974 .20643 m
.26212 .19886 L
s
.19852 .20636 m
.18425 .19652 L
s
.20965 .20609 m
.1967 .1967 L
s
.03592 .19769 m
.05492 .20582 L
s
.55182 .20552 m
.55476 .20156 L
s
.85926 .20516 m
.86617 .20255 L
s
.49142 .20503 m
.49305 .20061 L
s
.15888 .20484 m
.1392 .19313 L
s
.7278 .20465 m
.73339 .20158 L
s
.33598 .20451 m
.33231 .19828 L
s
.44194 .20417 m
.44225 .19929 L
s
.28721 .204 m
.28088 .19685 L
s
.0558 .19011 m
.08502 .20387 L
s
.10837 .20373 m
.07783 .18829 L
s
.63035 .20348 m
.63464 .19995 L
s
.04862 .19285 m
.07181 .20339 L
s
.30933 .20336 m
.30432 .19666 L
s
.24813 .20267 m
.23911 .19458 L
s
.86617 .20255 m
.87311 .19992 L
s
.0624 .1876 m
.09296 .20245 L
s
.04153 .19555 m
.05704 .20237 L
s
.03493 .19806 m
.0445 .20214 L
s
.13893 .20187 m
.11641 .18915 L
s
.36465 .2017 m
.36229 .19589 L
s
.40008 .20159 m
.39906 .19623 L
s
.73339 .20158 m
.73902 .1985 L
s
.55476 .20156 m
.55774 .19756 L
s
.16154 .20142 m
.14306 .19009 L
s
.1136 .20131 m
.08561 .18672 L
s
.1238 .20068 m
.0985 .18701 L
s
.49305 .20061 m
.4947 .19615 L
s
.63464 .19995 m
.63896 .1964 L
s
.87311 .19992 m
.88009 .19728 L
s
.2287 .19979 m
.21827 .19119 L
s
.44225 .19929 m
.44257 .19436 L
s
.16665 .19889 m
.14943 .18795 L
s
.26212 .19886 m
.25437 .19117 L
s
.05385 .19085 m
.07091 .19881 L
s
.04706 .19344 m
.05889 .19878 L
s
.73902 .1985 m
.74469 .19539 L
s
.33231 .19828 m
.32859 .19196 L
s
.04031 .19601 m
.0453 .1982 L
s
.21154 .19779 m
.1997 .18869 L
s
.06756 .18563 m
.09189 .19776 L
s
.55774 .19756 m
.56074 .19353 L
s
.88009 .19728 m
.88712 .19462 L
s
.17423 .19725 m
.1583 .18674 L
s
.13783 .19706 m
.11643 .18468 L
s
.06006 .18849 m
.07761 .19692 L
s
.28088 .19685 m
.27444 .18959 L
s
.1967 .1967 m
.18348 .18711 L
s
.30432 .19666 m
.29924 .18985 L
s
.18425 .19652 m
.16966 .18646 L
s
.63896 .1964 m
.64331 .19282 L
s
.39906 .19623 m
.39803 .1908 L
s
.4947 .19615 m
.49637 .19165 L
s
.36229 .19589 m
.35991 .19 L
s
.74469 .19539 m
.7504 .19226 L
s
.07348 .18338 m
.09584 .19485 L
s
.88712 .19462 m
.89418 .19194 L
s
.23911 .19458 m
.22992 .18635 L
s
.11889 .1945 m
.09461 .1811 L
s
.44257 .19436 m
.44289 .18938 L
s
.04557 .19401 m
.04583 .19412 L
s
.08035 .18076 m
.10486 .19376 L
s
.05201 .19156 m
.05664 .19369 L
s
.56074 .19353 m
.56377 .18946 L
s
.1392 .19313 m
.11899 .18111 L
s
.64331 .19282 m
.6477 .1892 L
s
.7504 .19226 m
.75615 .18911 L
s
.32859 .19196 m
.32482 .18555 L
s
.89418 .19194 m
.90129 .18925 L
s
.06489 .18665 m
.07517 .19171 L
s
.49637 .19165 m
.49805 .1871 L
s
.05787 .18933 m
.06206 .19132 L
s
.21827 .19119 m
.20764 .18242 L
s
.25437 .19117 m
.24648 .18334 L
s
.39803 .1908 m
.39698 .18531 L
s
.14306 .19009 m
.1241 .17846 L
s
.35991 .19 m
.35749 .18402 L
s
.29924 .18985 m
.29408 .18294 L
s
.27444 .18959 m
.2679 .1822 L
s
.56377 .18946 m
.56683 .18536 L
s
.44289 .18938 m
.44321 .18433 L
s
.6477 .1892 m
.65213 .18556 L
s
.11641 .18915 m
.09323 .17607 L
s
.75615 .18911 m
.76194 .18593 L
s
.1997 .18869 m
.18762 .1794 L
s
.07042 .18455 m
.07783 .18829 L
s
.14943 .18795 m
.13177 .17674 L
s
.18348 .18711 m
.16997 .17731 L
s
.49805 .1871 m
.49975 .1825 L
s
.08422 .17929 m
.0985 .18701 L
s
.1583 .18674 m
.14199 .17598 L
s
.07679 .18212 m
.08561 .18672 L
s
.16966 .18646 m
.15474 .17617 L
s
.22992 .18635 m
.22055 .17796 L
s
.76194 .18593 m
.76778 .18273 L
s
.65213 .18556 m
.65659 .18189 L
s
.32482 .18555 m
.321 .17904 L
s
.56683 .18536 m
.56991 .18121 L
s
.39698 .18531 m
.39592 .17975 L
s
.09803 .17403 m
.11643 .18468 L
s
.44321 .18433 m
.44353 .17923 L
s
.35749 .18402 m
.35504 .17797 L
s
.24648 .18334 m
.23845 .17537 L
s
.29408 .18294 m
.28884 .17592 L
s
.76778 .18273 m
.77365 .17951 L
s
.49975 .1825 m
.50146 .17786 L
s
.20764 .18242 m
.19679 .17347 L
s
.2679 .1822 m
.26126 .17469 L
s
.65659 .18189 m
.66109 .17819 L
s
.56991 .18121 m
.57303 .17703 L
s
.10355 .17193 m
.11899 .18111 L
s
.08842 .17769 m
.09461 .1811 L
s
.39592 .17975 m
.39485 .17412 L
s
.77365 .17951 m
.77957 .17627 L
s
.18762 .1794 m
.17529 .16991 L
s
.44353 .17923 m
.44386 .17408 L
s
.321 .17904 m
.31712 .17244 L
s
.10966 .1696 m
.1241 .17846 L
s
.66109 .17819 m
.66562 .17446 L
s
.35504 .17797 m
.35256 .17184 L
s
.22055 .17796 m
.21101 .16941 L
s
.50146 .17786 m
.5032 .17316 L
s
.16997 .17731 m
.15615 .16729 L
s
.57303 .17703 m
.57617 .17281 L
s
.11644 .16702 m
.13177 .17674 L
s
.77957 .17627 m
.78553 .173 L
s
.15474 .17617 m
.13946 .16564 L
s
.09301 .17594 m
.09323 .17607 L
s
.14199 .17598 m
.12528 .16496 L
s
.28884 .17592 m
.28351 .16879 L
s
.23845 .17537 m
.23028 .16726 L
s
.26126 .17469 m
.25449 .16705 L
s
.66562 .17446 m
.67019 .1707 L
s
.39485 .17412 m
.39377 .16841 L
s
.44386 .17408 m
.4442 .16886 L
s
.19679 .17347 m
.18572 .16435 L
s
.5032 .17316 m
.50495 .16842 L
s
.78553 .173 m
.79153 .16971 L
s
.57617 .17281 m
.57934 .16855 L
s
.31712 .17244 m
.31318 .16574 L
s
.35256 .17184 m
.35005 .16562 L
s
.67019 .1707 m
.67481 .16691 L
s
.17529 .16991 m
.16269 .16022 L
s
.79153 .16971 m
.79758 .1664 L
s
.21101 .16941 m
.20129 .1607 L
s
.4442 .16886 m
.44453 .16358 L
s
.28351 .16879 m
.2781 .16154 L
s
.57934 .16855 m
.58255 .16425 L
s
.50495 .16842 m
.50672 .16363 L
s
.39377 .16841 m
.39267 .16264 L
s
.14223 .15719 m
.15615 .16729 L
s
.23028 .16726 m
.22196 .159 L
s
.25449 .16705 m
.24762 .15929 L
s
.67481 .16691 m
.67946 .16308 L
s
.79758 .1664 m
.80367 .16306 L
s
.31318 .16574 m
.30918 .15895 L
s
.13256 .16088 m
.13946 .16564 L
s
.35005 .16562 m
.34749 .15931 L
s
.12403 .16413 m
.12528 .16496 L
s
.18572 .16435 m
.17443 .15504 L
s
.58255 .16425 m
.58578 .15991 L
s
.50672 .16363 m
.50851 .15879 L
s
.44453 .16358 m
.44487 .15824 L
s
.67946 .16308 m
.68414 .15922 L
s
.80367 .16306 m
.8098 .1597 L
s
.39267 .16264 m
.39156 .15679 L
s
.2781 .16154 m
.27261 .15418 L
s
.20129 .1607 m
.19138 .15182 L
s
.15329 .15298 m
.16269 .16022 L
s
.58578 .15991 m
.58904 .15552 L
s
.8098 .1597 m
.81598 .15631 L
s
.34749 .15931 m
.34491 .15292 L
s
.24762 .15929 m
.24063 .15139 L
s
.68414 .15922 m
.68887 .15533 L
s
.22196 .159 m
.21349 .15059 L
s
.30918 .15895 m
.30512 .15205 L
s
.50851 .15879 m
.51032 .15389 L
s
.44487 .15824 m
.44522 .15284 L
s
.39156 .15679 m
.39043 .15087 L
s
.81598 .15631 m
.82221 .1529 L
s
.58904 .15552 m
.59234 .1511 L
s
.68887 .15533 m
.69364 .15141 L
s
.16605 .14812 m
.17443 .15504 L
s
.27261 .15418 m
.26702 .1467 L
s
.51032 .15389 m
.51215 .14895 L
s
.34491 .15292 m
.34229 .14644 L
s
.82221 .1529 m
.82848 .14946 L
s
.44522 .15284 m
.44557 .14738 L
s
.30512 .15205 m
.30101 .14505 L
s
.19138 .15182 m
.18128 .14277 L
s
.69364 .15141 m
.69845 .14745 L
s
.24063 .15139 m
.23351 .14335 L
s
.59234 .1511 m
.59566 .14663 L
s
.39043 .15087 m
.38929 .14487 L
s
.21349 .15059 m
.20486 .14203 L
s
.51215 .14895 m
.514 .14395 L
s
.69845 .14745 m
.7033 .14346 L
s
.44557 .14738 m
.44592 .14185 L
s
.26702 .1467 m
.26135 .1391 L
s
.59566 .14663 m
.59902 .14212 L
s
.34229 .14644 m
.33963 .13987 L
s
.30101 .14505 m
.29683 .13794 L
s
.38929 .14487 m
.38813 .1388 L
s
.514 .14395 m
.51586 .13889 L
s
.7033 .14346 m
.70819 .13944 L
s
.23351 .14335 m
.22627 .13518 L
s
.18093 .14245 m
.18128 .14277 L
s
.59902 .14212 m
.60241 .13757 L
s
.19853 .13575 m
.20486 .14203 L
s
.44592 .14185 m
.44628 .13625 L
s
.33963 .13987 m
.33693 .13321 L
s
.70819 .13944 m
.71312 .13538 L
s
.26135 .1391 m
.25558 .13138 L
s
.51586 .13889 m
.51775 .13379 L
s
.38813 .1388 m
.38696 .13264 L
s
.29683 .13794 m
.29258 .13072 L
s
.60241 .13757 m
.60583 .13297 L
s
.44628 .13625 m
.44664 .13059 L
s
.71312 .13538 m
.7181 .13129 L
s
.21966 .1277 m
.22627 .13518 L
s
.51775 .13379 m
.51966 .12863 L
s
.33693 .13321 m
.3342 .12645 L
s
.60583 .13297 m
.60929 .12833 L
s
.38696 .13264 m
.38578 .12641 L
s
.25558 .13138 m
.24972 .12353 L
s
.7181 .13129 m
.72312 .12716 L
s
.29258 .13072 m
.28827 .12339 L
s
.44664 .13059 m
.447 .12486 L
s
.51966 .12863 m
.52159 .12341 L
s
.60929 .12833 m
.61278 .12364 L
s
.72312 .12716 m
.72818 .123 L
s
.3342 .12645 m
.33142 .1196 L
s
.38578 .12641 m
.38457 .12009 L
s
.447 .12486 m
.44737 .11906 L
s
.61278 .12364 m
.61631 .11891 L
s
.24549 .11787 m
.24972 .12353 L
s
.52159 .12341 m
.52354 .11813 L
s
.28827 .12339 m
.2839 .11595 L
s
.72818 .123 m
.73328 .11879 L
s
.38457 .12009 m
.38336 .11369 L
s
.33142 .1196 m
.32861 .11264 L
s
.44737 .11906 m
.44775 .11319 L
s
.61631 .11891 m
.61987 .11413 L
s
.73328 .11879 m
.73844 .11456 L
s
.52354 .11813 m
.52551 .11279 L
s
.2839 .11595 m
.27945 .10839 L
s
.73844 .11456 m
.74363 .11028 L
s
.61987 .11413 m
.62346 .1093 L
s
.38336 .11369 m
.38212 .1072 L
s
.44775 .11319 m
.44813 .10725 L
s
.52551 .11279 m
.5275 .1074 L
s
.32861 .11264 m
.32575 .10559 L
s
.74363 .11028 m
.74888 .10597 L
s
.62346 .1093 m
.6271 .10442 L
s
.27779 .10556 m
.27945 .10839 L
s
.5275 .1074 m
.52952 .10195 L
s
.44813 .10725 m
.44851 .10124 L
s
.38212 .1072 m
.38087 .10063 L
s
.32575 .10559 m
.32286 .09843 L
s
.6271 .10442 m
.63076 .09949 L
s
.52952 .10195 m
.53156 .09643 L
s
.44851 .10124 m
.4489 .09515 L
s
.38087 .10063 m
.3796 .09396 L
s
.63076 .09949 m
.63447 .09452 L
s
.32286 .09843 m
.31992 .09116 L
s
.53156 .09643 m
.53362 .09085 L
s
.4489 .09515 m
.44929 .08898 L
s
.63447 .09452 m
.63821 .08949 L
s
.3796 .09396 m
.37832 .08721 L
s
.31934 .08974 m
.31992 .09116 L
s
.53362 .09085 m
.5357 .08521 L
s
.63821 .08949 m
.64199 .08441 L
s
.44929 .08898 m
.44969 .08274 L
s
.37832 .08721 m
.37702 .08036 L
s
.5357 .08521 m
.53781 .0795 L
s
.64199 .08441 m
.64581 .07928 L
s
.44969 .08274 m
.4501 .07642 L
s
.37702 .08036 m
.3757 .07342 L
s
.53781 .0795 m
.53995 .07373 L
s
.64581 .07928 m
.64967 .0741 L
s
.4501 .07642 m
.4505 .07002 L
s
.64967 .0741 m
.65357 .06886 L
s
.53995 .07373 m
.5421 .06789 L
s
.37478 .06862 m
.3757 .07342 L
s
.4505 .07002 m
.45092 .06353 L
s
.65357 .06886 m
.65752 .06357 L
s
.5421 .06789 m
.54429 .06199 L
s
.65752 .06357 m
.6615 .05822 L
s
.45092 .06353 m
.45134 .05696 L
s
.54429 .06199 m
.54649 .05601 L
s
.45134 .05696 m
.45176 .05031 L
s
.54649 .05601 m
.54873 .04997 L
s
.45176 .05031 m
.45219 .04357 L
s
.54873 .04997 m
.55099 .04385 L
s
.55099 .04385 m
.55328 .03766 L
s
.45248 .03903 m
.45219 .04357 L
s
.55328 .03766 m
.55559 .03139 L
s
.55559 .03139 m
.55794 .02505 L
s
.55794 .02505 m
.56031 .01864 L
s
.56031 .01864 m
.56271 .01214 L
s
.56271 .01214 m
.56514 .00556 L
s
.25 Mabswid
.03399 .19842 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.96816 .22579 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
.03399 .19842 m
0 .87626 L
s
.02756 .32664 m
.03396 .32857 L
s
[(100)] .01476 .32278 1 .30143 Mshowa
.02095 .45839 m
.02744 .46 L
s
[(200)] .00798 .45516 1 .24828 Mshowa
.01416 .5938 m
.02072 .59507 L
s
[(300)] .00104 .59125 1 .19393 Mshowa
.00718 .73304 m
.0138 .73395 L
s
[(400)] -0.00606 .7312 1 .13836 Mshowa
gsave
-0.06073 .5083 -105.625 -11.9382 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 263.5},
  ImageMargins->{{44, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P00011a000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00;Coo`@000?oo`03003ooooo06;oo`00]_oo00<00?ooool0Iooo002fool0
0`00ooooo`1Wool00;Koo`03003ooooo06Ooo`00]_oo00<00?ooool0Iooo002dool3001Yool00;Ko
o`03003ooooo06Ooo`00oooo8Ooo003ooolQool007Woo`03003ooooo0:Coo`00K?oo1@001?oo0P00
0ooo00<00?ooool00_oo0`00=_oo0P00I_oo001]ool00`00ool00004ool01000ooooo`000_oo00<0
0?ooool01Ooo00<00?ooool0<Ooo0P0000Coo`00000006Coo`00KOoo00<00?oo00001?oo00<00?oo
ool00ooo00<00?ooool00ooo0P00<Ooo0`000ooo00<00?oo00000P00H_oo001^ool00`00ooooo`03
ool00`00ooooo`04ool01@00oooooooo0000<?oo0`001_oo00<00?oo00000_oo0P00H?oo001^ool0
0`00ooooo`02ool40004ool00`00ooooo`02ool3000[ool20009ool20005ool2001Nool006koo`03
003ooooo00?oo`03003ooooo00Goo`03003ooooo02coo`<000_oo`03003ooooo00Koo`8005coo`00
M?oo00<00?ooool0<Ooo0`003_oo00<00?ooool02?oo00<00?ooool0FOoo0025ool20004ool00`00
ooooo`03ool00`00ooooo`0Bool2000Aool00`00ooooo`09ool2001Iool008Coo`04003ooooo0009
ool00`00ooooo`0?ool3000Bool2000=ool2001Gool008Coo`04003ooooo000:ool00`00ooooo`0;
ool3000Eool2000?ool2001Eool008Coo`04003ooooo000:ool00`00ooooo`09ool200000ooo003o
o`0Eool2000Aool2001Cool008Coo`04003ooooo000;ool00`00ooooo`05ool30003ool00`00oooo
o`0Cool00`00ool0000Cool2001Aool008Coo`04003ooooo0008ool01000ooooo`001?oo0`001_oo
00<00?ooool04ooo00<00?oo00005Ooo0P00Cooo0025ool20009ool40002ool20009ool00`00oooo
o`0Cool00`00ool0000Gool00`00ooooo`1<ool009?oo`<000_oo`03003ooooo01;oo`04003ooooo
000Fool01000ool00000C?oo002@ool3000>ool00`00ooooo`0Bool01000ooooo`005Ooo00<00?oo
ool00_oo0P00B_oo002>ool3000@ool00`00ooooo`0Aool01@00oooooooo00005Ooo00<00?ooool0
1?oo0P00B?oo002;ool30002ool00`00ooooo`0>ool00`00ooooo`0Aool01@00oooooooo00005?oo
00<00?ooool01ooo0P00A_oo0029ool200000ooo003oo`02ool00`00ooooo`0>ool00`00ooooo`0A
ool01@00oooooooo00005?oo00<00?ooool02Ooo0P00A?oo0026ool30007ool00`00ooooo`0>ool0
0`00ooooo`0@ool00`00ooooo`02ool00`00ooooo`0Aool00`00ooooo`0<ool00`00ooooo`11ool0
08?oo`<000[oo`03003ooooo00koo`03003ooooo013oo`03003ooooo00;oo`03003ooooo013oo`03
003ooooo00koo`80047oo`00POoo0`003Ooo00<00?ooool03Ooo00<00?ooool04?oo00<00?ooool0
0_oo00<00?ooool03ooo00<00?ooool04Ooo0P00?ooo001Nool20004ool00`00ooooo`04ool00`00
ooooo`0@ool30003ool00`00ooooo`0:ool00`00ooooo`0=ool00`00ooooo`0?ool00`00ooooo`03
ool00`00ooooo`0>ool00`00ooooo`0Dool2000mool005goo`04003ooooo000:ool00`00ooooo`0=
ool30006ool00`00ooooo`0:ool00`00ooooo`0<ool00`00ooooo`0@ool00`00ooooo`03ool00`00
ooooo`0=ool00`00ooooo`0Gool2000kool005goo`04003ooooo0007ool5000<ool20009ool00`00
ooooo`0:ool00`00ooooo`0<ool00`00ooooo`0@ool00`00ooooo`03ool00`00ooooo`0=ool00`00
ooooo`0Iool2000iool005goo`04003ooooo0007ool01000ooooo`002_oo0`0000?oo`00ool02Ooo
00<00?ooool02Ooo00<00?ooool03?oo00<00?ooool03ooo00<00?ooool01?oo00<00?ooool03?oo
00<00?ooool06Ooo00@00?ooool003Soo`00GOoo00@00?ooool000Soo`03003oo`0000Ooo`<000Go
o`03003ooooo00Ooo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00ooo`03003ooooo00Co
o`03003ooooo00coo`03003ooooo01Soo`03003ooooo00;oo`8003Koo`00GOoo00@00?ooool000Wo
o`8000Goo`8000Soo`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00_oo`03003ooooo00oo
o`03003ooooo00Coo`03003ooooo00_oo`03003ooooo01Ooo`8000Ooo`8003Coo`00G_oo0P002ooo
00<00?ooool00`000_oo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00<00?ooool0
2ooo00<00?ooool03_oo00<00?ooool01Ooo00<00?ooool02_oo00<00?ooool05ooo00<00?ooool0
2Ooo0P00<_oo001[ool30005ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`09ool00`00
ooooo`0;ool00`00ooooo`0>ool00`00ooooo`05ool00`00ooooo`09ool00`00ooooo`0Gool00`00
ooooo`0<ool2000`ool006Woo`8000;oo`03003ooooo00Coo`03003ooooo00Koo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00_oo`03003ooooo00goo`03003ooooo00Koo`03003ooooo00So
o`03003ooooo01Ooo`03003ooooo00ooo`8002koo`00I_oo0`001Ooo00<00?ooool01?oo0P001ooo
00<00?ooool01_oo00<00?ooool02Ooo00<00?ooool02_oo00<00?ooool03Ooo00<00?ooool01_oo
00<00?ooool02?oo00<00?ooool05Ooo0P005?oo0P00;?oo001Sool30002ool20005ool00`00oooo
o`04ool00`00ooooo`05ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`0:ool00`00oooo
o`0=ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`0Eool00`00ooooo`0Fool00`00oooo
o`0Yool0063oo`H000Coo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo
00Koo`03003ooooo00Soo`03003ooooo00[oo`03003ooooo00coo`03003ooooo00Ooo`03003ooooo
00Koo`03003ooooo01Coo`8001[oo`8002Woo`00G_oo10001?oo00<00?ooool00_oo00<00?ooool0
0ooo00<00?ooool00ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool02?oo00<00?ooool0
2_oo00<00?ooool03?oo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool05?oo00<00?ooool0
7?oo0P009ooo000kool20004ool00`00ooooo`02ool4000Bool20004ool00`00ooooo`02ool00`00
ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`06ool00`00
ooooo`07ool00`00ooooo`0:ool00`00ooooo`0<ool00`00ooooo`07ool00`00ooooo`05ool00`00
ooooo`0Cool00`00ooooo`0Jool30002ool2000Uool003[oo`04003ooooo000:ool00`00ooooo`0=
ool40002ool20003ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03
ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`07ool00`00ooooo`0:ool00`00ooooo`0;
ool00`00ooooo`08ool00`00ooooo`04ool00`00ooooo`0Cool00`00ooooo`0Jool00`00ooooo`05
ool2000Sool003[oo`04003ooooo000:ool00`00ooooo`0;ool500000ooo00000002ool20002ool0
0`00ooooo`02ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`04ool00`00ooooo`05ool0
0`00ooooo`05ool00`00ooooo`07ool00`00ooooo`09ool00`00ooooo`0;ool00`00ooooo`08ool0
0`00ooooo`03ool00`00ooooo`0Cool00`00ooooo`0Iool2000:ool2000Qool003[oo`04003ooooo
000:ool00`00ooooo`08ool30002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`8000?oo`03
003ooooo00;oo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Goo`03
003ooooo00Ooo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00;oo`03
003ooooo01?oo`03003ooooo01Ooo`<000koo`03003ooooo01koo`00>_oo00@00?ooool000[oo`03
003ooooo00Goo`P00005ool00?ooool00002ool20002ool01@00oooooooo00000ooo00D00?oooooo
o`0000Goo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Koo`03003ooooo00Koo`03003o
oooo00Soo`03003ooooo00_oo`03003ooooo00Woo`03003ooooo00;oo`03003ooooo017oo`8001Wo
o`03003ooooo013oo`8001koo`00>_oo00@00?ooool000Soo`<000Goo`L00006ool00?oo003oo`00
0_oo00D00?ooooooo`0000;oo`05003oooooool00003ool01@00oooooooo00001Ooo00<00?ooool0
0ooo00<00?ooool00ooo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool02?oo00<00?ooool0
2ooo00<00?ooool02Ooo00D00?ooooooo`0001?oo`03003ooooo01Soo`03003ooooo01?oo`8001co
o`00>ooo0P002ooo00@00?ooooooo`H00005ool00000003oo`0300001?oo00000?oo0P000_oo0P00
00?oo`00ool00_oo00@00?ooooooo`8000;oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo
00?oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo00[oo`03003ooooo
00[oo`04003ooooo000Cool00`00ooooo`0Fool3000Hool2000Jool004Soo``00005ool00000003o
o`0300001?oo00000?oo0P000_oo00@00?oo000000;oo`8000?oo`04003ooooo0005ool00`00oooo
o`02ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`07ool00`00oooo
o`0:ool00`00ooooo`0:ool01000ooooo`004_oo00<00?ooool05_oo00<00?ooool06ooo0P006?oo
0016ool;00000ooo0000000300001?oo00000?oo0P000_oo00@00?ooool000;oo`04003ooooo0003
ool01@00oooooooo00000_oo0P001?oo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool01?oo
00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00<00?ooool02ooo00<00?oo00004_oo
00<00?ooool05Ooo0P007_oo10005_oo0013ool@00000ooo0000000300000ooo003oo`0300001Ooo
00000?oo00000_oo00@00?ooool000?oo`05003oooooool00003ool01@00oooooooo00001Ooo00<0
0?ooool00ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool02Ooo00<0
0?ooool02ooo0P004_oo00<00?ooool05?oo0P007_oo0P001?oo0P005?oo0012oolC00000ooo0000
000900001?oo003oo`000_oo00@00?ooool000?oo`05003oooooool00003ool01@00oooooooo0000
1Ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool0
2Ooo00<00?ooool02ooo00<00?ooool03ooo0P005_oo00<00?ooool06ooo0`002?oo00<00?ooool0
4Ooo0012oolN00000ooo003oo`0400001?oo00000?oo0P000_oo0P000_oo00D00?ooooooo`0000?o
o`8000Coo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00Ooo`03003o
oooo00Soo`03003ooooo00coo`03003ooooo00koo`03003ooooo01?oo`<001coo`8000coo`80017o
o`00@_oo00@00?oooooooah00004ool00?oo0002ool01000ooooo`000ooo00@00?ooool000?oo`05
003oooooool00004ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`08ool00`00ooooo`0:ool3000?ool00`00ooooo`0Cool00`00ooooo`0Jool3000@
ool2000?ool004;oo`03003ooooo00Gooa/00003ool0000000800004ool00000ool20002ool01000
ooooo`000ooo00D00?ooooooo`0000Coo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Soo`03003ooooo00[oo`03003oo`0000goo`8001?oo`<001[o
o`<001Goo`8000goo`00@_oo00<00?ooool01ooo6`0000?oo`0000000P0000Goo`00ooooo`0000;o
o`04003ooooo0003ool01@00oooooooo00001?oo00<00?ooool00_oo00<00?ooool00ooo00<00?oo
ool01Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02_oo00@00?ooool000coo`03003o
oooo01;oo`03003ooooo01Soo`<001[oo`8000_oo`00@_oo00<00?ooool02_oo6P0000?oo`00ool0
100000Goo`00ooooo`0000;oo`05003oooooool00003ool01@00oooooooo00001?oo00<00?ooool0
1?oo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00D00?oooooo
o`0000_oo`03003ooooo017oo`8001[oo`03003ooooo01goo`8000Woo`00@_oo00<00?ooool03?oo
5`0000Coo`00000000;oo`05003oo`00ool00002ool00`00ooooo`020002ool01@00oooooooo0000
0ooo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool0
1_oo00<00?ooool02_oo00D00?ooooooo`0000[oo`03003ooooo013oo`8001Woo`<0027oo`8000So
o`00@_oo00<00?ooool03_oo5`0000?oo`0000000_oo0`0000Goo`00003oo`0000?oo`04003ooooo
0002ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`04ool00`00oooo
o`05ool00`00ooooo`06ool00`00ooooo`09ool00`00ooooo`02ool00`00ooooo`06ool2000Aool2
000Hool3000Pool400000ooo003oo`07ool004;oo`03003ooooo01?ooaL000;oo`8000;oo`03003o
o`0000?oo`04003ooooo0002ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`02ool00`00
ooooo`04ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`09ool00`00ooooo`02ool00`00
ooooo`06ool00`00ooooo`0?ool00`00ooooo`0Eool3000Oool40005ool00`00ooooo`06ool004;o
o`03003ooooo01GooaL00007ool00000003oo`00ool00P0000?oo`00ool00_oo00<00?ooool00P00
0_oo00<00?ooool00ooo00D00?ooooooo`0000Koo`03003ooooo00Goo`03003ooooo00Goo`03003o
oooo00Soo`8000Goo`03003ooooo00Goo`03003ooooo00goo`<001Koo`8001koo`@000Woo`03003o
oooo00Koo`00@Ooo00<00?ooool06Ooo5P0000Goo`00003oo`0000;oo`03003oo`0000?oo`04003o
oooo0003ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`04ool00`00
ooooo`06ool00`00ooooo`08ool00`00ooooo`04ool00`00ooooo`03ool2000?ool00`00ooooo`0D
ool3000Mool3000=ool00`00ooooo`06ool0047oo`03003ooooo01cooa800003ool00?oo00D00003
ool0000000800003ool00?oo00;oo`05003oooooool00003ool00`00ooooo`02ool00`00ooooo`03
ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`05ool00`00ooooo`02
ool00`00ooooo`0=ool2000Dool3000Lool4000@ool00`00ooooo`06ool0047oo`03003ooooo01oo
oa000004ool00000ool200002Ooo00000?oo003oo`00ool00003ool01000ooooo`001?oo00<00?oo
ool00_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01ooo00<00?oo
ool01_oo00<00?ooool00P003ooo00<00?ooool04_oo0P006ooo10005?oo00<00?ooool01_oo0011
ool00`00ooooo`0Qool@00000ooo003oo`020002ool01P00ool00?oo003oo`8000;oo`04003ooooo
0004ool01@00oooooooo00001Ooo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01_oo0P00
2?oo00<00?oo00003ooo0P004_oo0`006Ooo10006?oo00<00?ooool01_oo0011ool00`00ooooo`0T
oolA00002Ooo003ooooo003oo`00ool00002ool01000ooooo`000_oo00D00?ooooooo`0000Coo`03
003ooooo00;oo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Ooo`03
003oo`0000coo`<0017oo`<001Woo`<001coo`03003ooooo00Koo`00@Ooo00<00?ooool09ooo3`00
00Woo`00003oo`00ool00?oo00000_oo00@00?ooool000;oo`05003oooooool00003ool00`00oooo
o`02ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`08ool2000<ool0
0`00ooooo`0?ool3000Hool4000Oool00`00ooooo`06ool0047oo`03003ooooo02[ooa000003ool0
0?oo00@00007ool00?ooool00?oo00000ooo00D00?ooooooo`0000Coo`03003ooooo00?oo`03003o
oooo00Coo`03003ooooo00Koo`03003ooooo00Ooo`8000_oo`80013oo`8001Ooo`@002Coo`03003o
oooo00Goo`00@Ooo00<00?ooool0;Ooo40000_oo0P000_oo00<00?oo00000_oo00D00?ooooooo`00
00?oo`03003ooooo00;oo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00Goo`8000Soo`80
0003ool00?oo00Soo`8000ooo`<001Koo`<002Soo`03003ooooo00Goo`00@Ooo00<00?ooool0<?oo
3`0000?oo`0000000_oo00<00?oo00000_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`03003o
oooo00;oo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00Koo`05003oooooool00008ool0
0`00ooooo`0<ool3000Eool4000[ool00`00ooooo`05ool0047oo`03003ooooo03;oo`h00008ool0
0000003oo`00ool000;oo`04003ooooo0003ool01@00oooooooo00001?oo00<00?ooool00ooo00<0
0?ooool01?oo00<00?ooool01_oo0P000ooo00<00?ooool01?oo0P003Ooo0P005?oo1000;ooo00<0
0?ooool01Ooo0011ool00`00ooooo`0eool>00001Ooo003oo`00ool00P0000Coo`00ool000?oo`05
003oooooool00004ool00`00ooooo`03ool00`00ooooo`03ool20007ool00`00ooooo`03ool00`00
ooooo`02ool2000<ool3000Bool4000cool00`00ooooo`05ool0047oo`03003ooooo03Soo`<00003
ool0000000<00003ool0000000800006ool00?oo003oo`000_oo00@00?ooool000?oo`03003ooooo
00;oo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo00Coo`8000Koo`03003ooooo008000co
o`8001;oo`<003Ooo`03003ooooo00Goo`00@Ooo00<00?ooool0>ooo2@0000Woo`00003oo`00ool0
0?oo00000_oo00@00?ooool000?oo`05003oooooool00004ool00`00ooooo`03ool00`00ooooo`05
ool00`00ooooo`05ool00`00ool0000;ool3000@ool4000jool00`00ooooo`05ool0047oo`03003o
oooo03goo`/00008ool00?oo003oo`00ool000;oo`05003oooooool00003ool01@00oooooooo0000
1Ooo00<00?ooool01?oo0P002?oo0P002_oo0P003ooo1000?_oo00<00?ooool01Ooo0011ool00`00
ooooo`10ool900002?oo003oo`00ool00?oo0002ool01000ooooo`000ooo00D00?ooooooo`0000Co
o`03003ooooo00Coo`03003ooooo00Koo`<000Soo`<000goo`@004;oo`03003ooooo00Goo`00;?oo
10000ooo0P001?oo0P001_oo00<00?ooool0@ooo2@0000Soo`00ool00?oo003oo`000_oo00@00?oo
ool000?oo`8000?oo`8000Goo`8000Koo`8000;oo`03003ooooo00Coo`8000goo`<004Koo`03003o
oooo00Goo`00;_oo00D00?ooooooo`0000;oo`04003ooooo0002ool00`00ooooo`03ool00`00oooo
o`16ool500000ooo00000002ool20002ool00`00ool00002ool01@00oooooooo00001?oo00<00?oo
ool00_oo0P001_oo0P001?oo00@00?ooooooo`<000_oo`@004Woo`03003ooooo00Goo`00;_oo00D0
0?ooooooo`0000;oo`04003ooooo0002ool00`00ooooo`03ool00`00ooooo`19ool300002ooo0000
0?oo003oo`00ool00?oo00000_oo00D00?ooooooo`0000?oo`03003ooooo00?oo`03003ooooo00?o
o`8000Koo`@000[oo`@004goo`03003ooooo00Goo`00;_oo00D00?ooooooo`0000;oo`04003ooooo
0002ool00`00ooooo`03ool00`00ooooo`1<ool50002ool20002ool00`00ool00002ool20002ool0
0`00ooooo`02ool20005ool00`00ooooo`04ool30009ool4001Aool00`00ooooo`05ool002koo`05
003oooooool00002ool01000ooooo`000_oo00<00?ooool00_oo0P00D?oo100000Soo`00ool00?oo
003oo`000_oo00<00?ooool00P000ooo00<00?ooool00ooo0P001Ooo0P000_oo00<00?ooool01?oo
0`00EOoo00<00?ooool01Ooo000/ool30003ool01000ooooo`000_oo00@00?ooool000Coo`<005;o
o`D00004ool00?oo0002ool01000ooooo`000ooo00<00?ooool00_oo0P001Ooo0P001?oo00<00?oo
ool01000F?oo00<00?ooool01Ooo000^ool00`00ooooo`02ool20004ool20005ool00`00ooooo`1E
ool400000ooo003oo`0200000ooo003oo`02ool01000oooooooo0P000ooo10001Ooo1000G?oo00<0
0?ooool01Ooo0010ool00`00ooooo`1Hool300001?oo003oo`000_oo0P000_oo0P000ooo0P001_oo
0`0000?oo`00ool0GOoo00<00?ooool01Ooo0010ool00`00ooooo`1Kool400001?oo003ooooo0P00
0ooo0P001?oo10001?oo00<00?ooool0G?oo00<00?ooool01Ooo0010ool00`00ooooo`1Mool30002
ool60002ool40008ool00`00ooooo`1Lool00`00ooooo`05ool0043oo`03003ooooo063oo`@000;o
o`@000coo`03003ooooo05goo`03003ooooo00Coo`00@?oo00<00?ooool0Hooo0`004?oo00<00?oo
ool0GOoo00<00?ooool01?oo0010ool00`00ooooo`1Tool00`00ooooo`0?ool00`00ooooo`1Mool0
0`00ooooo`04ool0043oo`03003ooooo06Coo`03003ooooo00ooo`03003ooooo05goo`03003ooooo
00Coo`00@?oo00<00?ooool0I?oo00<00?ooool03ooo00<00?ooool0GOoo00<00?ooool01?oo0010
ool00`00ooooo`1Tool00`00ooooo`0?ool00`00ooooo`1Mool00`00ooooo`04ool0043oo`03003o
oooo06Coo`03003ooooo00ooo`03003ooooo05goo`03003ooooo00Coo`00@?oo00<00?ooool0I?oo
00<00?ooool03ooo00<00?ooool0GOoo00<00?ooool01?oo0010ool00`00ooooo`1Tool00`00oooo
o`0?ool00`00ooooo`1Mool00`00ooooo`04ool0043oo`03003ooooo06Coo`03003ooooo00ooo`03
003ooooo05goo`03003ooooo00Coo`00@?oo00<00?ooool0I?oo00<00?ooool03ooo00<00?ooool0
GOoo00<00?ooool01?oo0010ool00`00ooooo`1Tool00`00ooooo`0?ool00`00ooooo`1Mool00`00
ooooo`04ool0043oo`03003ooooo06Coo`03003ooooo00ooo`03003ooooo05goo`03003ooooo00Co
o`00@?oo00<00?ooool0I?oo00<00?ooool03ooo00<00?ooool0GOoo00<00?ooool01?oo000oool0
0`00ooooo`1Uool00`00ooooo`0?ool00`00ooooo`1Mool00`00ooooo`04ool003ooo`03003ooooo
06Goo`03003ooooo013oo`03003ooooo05coo`03003ooooo00Coo`00?ooo00<00?ooool0IOoo00<0
0?ooool04?oo00<00?ooool0G?oo00<00?ooool01?oo000oool00`00ooooo`1Uool00`00ooooo`0@
ool00`00ooooo`1Lool00`00ooooo`04ool003ooo`03003ooooo06Goo`03003ooooo013oo`03003o
oooo05goo`03003ooooo00?oo`00?ooo00<00?ooool0IOoo00<00?ooool04?oo00<00?ooool0GOoo
00<00?ooool00ooo000[ool40003ool20004ool20005ool00`00ooooo`1Uool00`00ooooo`0@ool0
0`00ooooo`1Mool00`00ooooo`03ool002_oo`03003ooooo00?oo`04003ooooo0002ool01000oooo
o`001?oo00<00?ooool0IOoo00<00?ooool04?oo00<00?ooool0GOoo00<00?ooool00ooo000/ool0
0`00ooooo`02ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo06Goo`03003ooooo013o
o`03003ooooo05goo`03003ooooo00?oo`00;Ooo00D00?ooooooo`0000;oo`04003ooooo0002ool0
0`00ooooo`02ool00`00ooooo`1Uool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03ool0
02koo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool00_oo00<00?ooool0IOoo00<00?oo
ool04?oo00<00?ooool0GOoo00<00?ooool00ooo000[ool01000ooooo`000_oo00@00?ooool000;o
o`04003ooooo0004ool2001Vool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03ool002co
o`8000Coo`8000Coo`8000Goo`03003ooooo06Goo`03003ooooo013oo`03003ooooo05goo`03003o
oooo00?oo`00?ooo00<00?ooool0IOoo00<00?ooool04?oo00<00?ooool0GOoo00<00?ooool00ooo
000oool00`00ooooo`1Uool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03ool003ooo`03
003ooooo06Goo`03003ooooo013oo`03003ooooo05goo`03003ooooo00?oo`005?oo00<00?ooool0
:?oo00<00?ooool0IOoo00<00?ooool04?oo00<00?ooool0GOoo00<00?ooool00ooo0002ool50002
ool40008ool01000oooooooo0P000_oo0P0000?oo`0000000P006ooo00<00?ooool0IOoo00<00?oo
ool04?oo00<00?ooool0GOoo00<00?ooool00ooo0003ool00`00ool00002ool01P00ooooo`00ool0
00Ooo`03003ooooo00;oo`04003ooooo0002ool01000ooooo`006ooo00<00?ooool0IOoo00<00?oo
ool04?oo00<00?ooool0GOoo00<00?ooool00ooo0003ool00`00ool00002ool01P00ooooo`00ool0
00Ooo`03003ooooo00;oo`04003ooooo0003ool00`00ooooo`0Jool00`00ooooo`1Vool00`00oooo
o`0@ool00`00ooooo`1Mool00`00ooooo`03ool000Coo`05003oooooool00004ool00`00ooooo`06
ool01000oooooooo10001?oo00<00?ooool06Ooo00<00?ooool0I_oo00<00?ooool04?oo00<00?oo
ool0GOoo00<00?ooool00ooo0004ool00`00ooooo`02ool01@00oooooooo00002?oo00D00?oooooo
o`0000;oo`03003ooooo00@00003ool00000008001Goo`03003ooooo06Koo`03003ooooo013oo`03
003ooooo05goo`03003ooooo00?oo`001?oo00<00?ooool00_oo00D00?oo00000?oo00@000Goo`04
003oo`000002ool20007ool2000Fool00`00ooooo`1Vool00`00ooooo`0@ool00`00ooooo`1Mool0
0`00ooooo`03ool000Woo`800004ool00?ooool2000Gool00`00ooooo`0Cool00`00ooooo`1Vool0
0`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool0017oo`03003ooooo01;oo`<001Goo`03
003ooooo06Koo`03003ooooo013oo`03003ooooo05koo`03003ooooo00;oo`003ooo0`00;?oo00<0
0?ooool0I_oo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000nool00`00ooooo`1V
ool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool003koo`03003ooooo06Koo`03003o
oooo013oo`03003ooooo05koo`03003ooooo00;oo`00?_oo00<00?ooool0I_oo00<00?ooool04?oo
00<00?ooool0G_oo00<00?ooool00_oo000nool00`00ooooo`1Vool00`00ooooo`0@ool00`00oooo
o`1Nool00`00ooooo`02ool003koo`03003ooooo06Koo`03003ooooo013oo`03003ooooo05koo`03
003ooooo00;oo`00?_oo00<00?ooool0I_oo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool0
0_oo000nool00`00ooooo`1Vool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool003ko
o`03003ooooo06Koo`03003ooooo013oo`03003ooooo05koo`03003ooooo00;oo`00?_oo00<00?oo
ool0I_oo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000nool00`00ooooo`1Vool0
0`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool003koo`03003ooooo06Koo`03003ooooo
013oo`03003ooooo05koo`03003ooooo00;oo`00?_oo00<00?ooool0I_oo00<00?ooool04?oo00<0
0?ooool0G_oo00<00?ooool00_oo000Zool20004ool20004ool20006ool00`00ooooo`1Vool00`00
ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool002Woo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000Coo`03003ooooo06Ooo`03003ooooo013oo`03003ooooo05koo`03003ooooo
00;oo`00;?oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`02ool00`00ooooo`1Wool0
0`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool002[oo`8000?oo`04003ooooo0002ool0
1000ooooo`001?oo00<00?ooool0Iooo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo
000/ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo00;oo`<006Ooo`03003ooooo013o
o`03003ooooo05ooo`03003ooooo007oo`00:Ooo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`001?oo00<00?ooool0Iooo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000Z
ool20004ool20004ool20005ool00`00ooooo`1Wool00`00ooooo`0@ool00`00ooooo`1Oool00`00
ooooo`01ool003goo`03003ooooo06Ooo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007o
o`00?Ooo00<00?ooool0Iooo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool0
0`00ooooo`1Wool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool003goo`03003ooooo
06Ooo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00?Ooo00<00?ooool0Iooo00<0
0?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool00`00ooooo`1Wool00`00ooooo`0@
ool00`00ooooo`1Oool00`00ooooo`01ool003goo`03003ooooo06Ooo`03003ooooo013oo`03003o
oooo05ooo`03003ooooo007oo`00?Ooo00<00?ooool0Iooo00<00?ooool04?oo00<00?ooool0Gooo
00<00?ooool00Ooo000mool00`00ooooo`1Wool00`00ooooo`0@ool00`00ooooo`1Oool00`00oooo
o`01ool003goo`03003ooooo06Ooo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00
?Ooo00<00?ooool0Iooo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool00`00
ooooo`1Wool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool003goo`03003ooooo06Oo
o`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00??oo00<00?ooool0J?oo00<00?oo
ool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000lool00`00ooooo`1Xool00`00ooooo`0@ool0
0`00ooooo`1Oool00`00ooooo`01ool003coo`03003ooooo06Soo`03003ooooo013oo`03003ooooo
05ooo`03003ooooo007oo`00??oo00<00?ooool0J?oo00<00?ooool04?oo00<00?ooool0H?oo0@00
0Ooo0Ooo000lool00`00ooooo`1Xool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool003co
o`03003ooooo06Soo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00??oo00<00?ooool0
J?oo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool00`00ooooo`1Xool00`00oooo
o`0@ool00`00ooooo`1Pool10001ool1ool003coo`03003ooooo06Soo`03003ooooo013oo`03003o
oooo063oo`40007oo`7oo`00??oo00<00?ooool0J?oo00<00?ooool04?oo00<00?ooool0H?oo0@00
0Ooo0Ooo000lool00`00ooooo`1Xool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool003co
o`03003ooooo06Soo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00:_oo00<00?ooool0
0_oo0P001?oo0P001Ooo00<00?ooool0J?oo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo
000Zool01@00oooooooo00000_oo00@00?ooool000;oo`03003ooooo00;oo`03003ooooo06Soo`03
003ooooo013oo`03003ooooo063oo`40007oo`7oo`009ooo1@000_oo00@00?ooool000;oo`04003o
oooo0004ool00`00ooooo`1Xool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool002Ooo`04
003ooooo0003ool01000ooooo`000_oo00@00?ooool000Coo`8006Woo`03003ooooo013oo`03003o
oooo063oo`40007oo`7oo`00:?oo00<00?oo00000ooo00@00?ooool000;oo`04003ooooo0004ool0
0`00ooooo`1Xool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool002Woo`8000?oo`04003o
oooo0002ool01000ooooo`001?oo00<00?ooool0J?oo00<00?ooool04?oo00<00?ooool0H?oo0@00
0Ooo0Ooo000Zool00`00ooooo`02ool20004ool20005ool00`00ooooo`1Xool00`00ooooo`0@ool0
0`00ooooo`1Pool10001ool1ool003_oo`03003ooooo06Woo`03003ooooo013oo`03003ooooo063o
o`40007oo`7oo`00>ooo00<00?ooool0JOoo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo
000kool00`00ooooo`1Yool00`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03003ooooo
06Woo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo00<00?ooool0JOoo00<00?ooool0
4?oo00<00?ooool0HOoo0@000Ooo000kool00`00ooooo`1Yool00`00ooooo`0@ool00`00ooooo`1Q
ool10001ool003_oo`03003ooooo06Woo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo
00<00?ooool0JOoo00<00?ooool04?oo00<00?ooool0HOoo0@000Ooo000kool00`00ooooo`1Yool0
0`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03003ooooo06Woo`03003ooooo013oo`03
003ooooo067oo`40007oo`00>ooo00<00?ooool0JOoo00<00?ooool04?oo00<00?ooool0HOoo0@00
0Ooo000kool00`00ooooo`1Yool00`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03003o
oooo06Woo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo00<00?ooool0JOoo00<00?oo
ool04?oo00<00?ooool0HOoo0@000Ooo000kool00`00ooooo`1Yool00`00ooooo`0@ool00`00oooo
o`1Qool10001ool003_oo`03003ooooo06Woo`03003ooooo013oo`03003ooooo067oo`40007oo`00
>ooo00<00?ooool0JOoo00<00?ooool04?oo00<00?ooool0HOoo0@000Ooo000kool00`00ooooo`1Y
ool00`00ooooo`0;ool9001Pool10001ool003_oo`03003ooooo06Woo`03003ooooo00/000Woo`P0
05Soo`40007oo`00>ooo00<00?ooool0H_oo2P007?oo2000D?oo0@000Ooo000jool00`00ooooo`1H
ool;0007ool00`00ooooo`0Tool80018ool10001ool003[oo`03003ooooo04goo`/001;oo`03003o
oooo02coo`L004;oo`400000>_oo00<00?ooool0@ooo2P007Ooo00<00?ooool0<ooo2000>_oo0@00
000jool00`00ooooo`0hool;000Wool00`00ooooo`0kool8000bool1000003[oo`03003ooooo02go
o`/003;oo`03003ooooo04?oo`P002[oo`400000>_oo00<00?ooool08ooo2P00?Ooo00<00?ooool0
Booo20008_oo0@00000jool00`00ooooo`0Hool;0017ool00`00ooooo`1Cool7000Kool1000003[o
o`03003ooooo00goo`/005;oo`03003ooooo05[oo`P001?oo`400000>_oo00<00?ooool00ooo2P00
GOoo00<00?ooool0H_oo20002ooo0@00000jool7001Vool00`00ooooo`1Zool80003ool10000047o
o``000Woo`03003ooooo04koo`03003ooooo06koo`P00000COoo3000C_oo00<00?ooool0Gooo3`00
2?oo001Iool<0012ool00`00ooooo`1@ool?000Gool006Goo``0013oo`03003ooooo02?oo`03003o
oooo047oo`l002Koo`00LOoo30001?oo00<00?ooool08ooo00<00?ooool0<_oo3`00=Ooo001mool<
000Nool00`00ooooo`0Sool?0014ool008Woo``001;oo`03003ooooo01Coo`l005?oo`00E?oo0P00
?ooo30001_oo00<00?ooool01Ooo3`00H_oo001Cool01000ooooo`00B_oo3P00LOoo001Cool01000
ooooo`00bOoo001Cool3003:ool005?oo`03003ooooo02[oo`8009koo`00Dooo00<00?ooool0:Ooo
00@00?ooool009goo`00E?oo0`00:ooo00<00?ooool0Vooo0020ool2000Tool2001hool008;oo`03
003ooooo023oo`04003ooooo001gool007ooo`04003ooooo000Rool01000ooooo`00Mooo0020ool2
000Sool01000ooooo`00Mooo002Uool01000ooooo`00Mooo002Uool01000ooooo`00Mooo002Vool2
001hool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00Looo00<00?ooool0Z_oo001]ool3
0004ool00`00ooooo`02ool3002Tool006koo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo
0:7oo`00K_oo00<00?ooool00ooo00<00?ooool00ooo0P00Y?oo001^ool00`00ooooo`04ool01@00
oooooooo0000Y_oo001/ool01@00ool00?oo00001?oo00<00?ooool00_oo0`00Xooo001/ool50005
ool00`00ooooo`2Wool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{131.938, 361.312}, {326.062, 116.312}} -> {-0.890814, \
0.511313, 0.00409734, 0.00409734}}]
}, Open  ]],

Cell[TextData[{
  "As expected by looking at 0310142307, ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Omega]\&~\)\_aprx\)]],
  " converges immediately because in step 3c) of the procedure the \
probability to measure 1 is always 1. ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " is the total measurement time ",
  Cell[BoxData[
      \(TraditionalForm\`N\ \[CapitalDelta]t\)]],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot with ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Beta]\^'\) = 1/6\)]]
}], "Subsubsection"],

Cell[BoxData[
    \(plotsB2 = createPlotsB[1. /6]; \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(showPlotsB[plotsB2]; \)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .91501 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0262702 1.06936 -2.13046e-17 1.06936 [
[.4752 .92837 -3.00977 0 ]
[.4752 .92837 2.99023 9 ]
[.31156 .91504 -3.05802 0 ]
[.31156 .91504 2.94198 9 ]
[.11966 .89942 -3.11475 0 ]
[.11966 .89942 2.88525 9 ]
[.26027 .97781 -11.2665 0 ]
[.26027 .97781 10.7335 9.8125 ]
[.03465 .18507 -8.54876 -9 ]
[.03465 .18507 9.45124 0 ]
[.18551 .12753 -8.69057 -9 ]
[.18551 .12753 9.30943 0 ]
[.35735 .06198 -8.85423 -9 ]
[.35735 .06198 9.14577 0 ]
[.55418 -0.0131 -6.02969 -9 ]
[.55418 -0.0131 5.97031 0 ]
[.27026 .02904 -13.6412 -9.8125 ]
[.27026 .02904 14.3588 0 ]
[.01476 .32278 -18 -5.85643 ]
[.01476 .32278 0 3.14357 ]
[.00798 .45516 -18 -5.61724 ]
[.00798 .45516 0 3.38276 ]
[.00104 .59125 -18 -5.3727 ]
[.00104 .59125 0 3.6273 ]
[-0.00606 .7312 -18 -5.12262 ]
[-0.00606 .7312 0 3.87738 ]
[-0.06073 .5083 -44.625 -7.93817 ]
[-0.06073 .5083 0 5.06183 ]
[ 0 0 0 0 ]
[ 1 .91501 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.4753 .91501 m
0 .87626 L
s
.47525 .915 m
.47527 .90832 L
s
[(0)] .4752 .92837 .00326 -1 Mshowa
.31182 .90168 m
.31195 .895 L
s
[(3)] .31156 .91504 .01934 -1 Mshowa
.12017 .88606 m
.12043 .87938 L
s
[(6)] .11966 .89942 .03825 -1 Mshowa
gsave
.26027 .97781 -72.2665 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(T) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.03399 .19842 m
.55496 0 L
s
.03399 .19842 m
.03365 .2051 L
s
[(0.1)] .03465 .18507 -0.05014 1 Mshowa
.18505 .14089 m
.18482 .14756 L
s
[(0.4)] .18551 .12753 -0.03438 1 Mshowa
.35714 .07534 m
.35703 .08203 L
s
[(0.7)] .35735 .06198 -0.0162 1 Mshowa
.55425 .00027 m
.55428 .00695 L
s
[(1.)] .55418 -0.0131 .00495 1 Mshowa
gsave
.27026 .02904 -74.6412 -13.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(t) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
75.000 11.250 moveto
(\\220) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
87.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.03399 .19842 m
0 .87626 L
s
.02756 .32664 m
.03396 .32857 L
s
[(100)] .01476 .32278 1 .30143 Mshowa
.02095 .45839 m
.02744 .46 L
s
[(200)] .00798 .45516 1 .24828 Mshowa
.01416 .5938 m
.02072 .59507 L
s
[(300)] .00104 .59125 1 .19393 Mshowa
.00718 .73304 m
.0138 .73395 L
s
[(400)] -0.00606 .7312 1 .13836 Mshowa
gsave
-0.06073 .5083 -105.625 -11.9382 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.47712 .35504 m
.4753 .91501 L
s
.4753 .91501 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
.96816 .22579 m
.47712 .35504 L
s
.03399 .19842 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.47712 .35504 m
.4753 .91501 L
s
.4753 .91501 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.03399 .19842 m
.47712 .35504 L
s
.96816 .22579 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
0 0 m
1 0 L
1 .91501 L
0 .91501 L
closepath
clip
newpath
.5 Mabswid
.46967 .42173 m
.47448 .41906 L
s
.47448 .41906 m
.47932 .41637 L
s
.47932 .41637 m
.48418 .41365 L
s
.46226 .4455 m
.4662 .44128 L
s
.48418 .41365 m
.48906 .41092 L
s
.4662 .44128 m
.47017 .43701 L
s
.48906 .41092 m
.49397 .40817 L
s
.47017 .43701 m
.47415 .4327 L
s
.49397 .40817 m
.4989 .40541 L
s
.45478 .46832 m
.45783 .4623 L
s
.47415 .4327 m
.47815 .42835 L
s
.4989 .40541 m
.50385 .40263 L
s
.45783 .4623 m
.4609 .45619 L
s
.50385 .40263 m
.50883 .39984 L
s
.47815 .42835 m
.48218 .42398 L
s
.4609 .45619 m
.46399 .45002 L
s
.4472 .49022 m
.44935 .48216 L
s
.50883 .39984 m
.51383 .39705 L
s
.48218 .42398 m
.48623 .41959 L
s
.46399 .45002 m
.46709 .44379 L
s
.44935 .48216 m
.45151 .47397 L
s
.51383 .39705 m
.51886 .39425 L
s
.48623 .41959 m
.4903 .41518 L
s
.51886 .39425 m
.52391 .39145 L
s
.46709 .44379 m
.47022 .43754 L
s
.43954 .51122 m
.44078 .5009 L
s
.45151 .47397 m
.45369 .4657 L
s
.4903 .41518 m
.49439 .41078 L
s
.52391 .39145 m
.52899 .38864 L
s
.47022 .43754 m
.47336 .43129 L
s
.44078 .5009 m
.44201 .49042 L
s
.49439 .41078 m
.49851 .40638 L
s
.45369 .4657 m
.45588 .45739 L
s
.52899 .38864 m
.53409 .38583 L
s
.4318 .53135 m
.4321 .51856 L
s
.47336 .43129 m
.47653 .42504 L
s
.49851 .40638 m
.50265 .40199 L
s
.44201 .49042 m
.44326 .47986 L
s
.45588 .45739 m
.45809 .44906 L
s
.53409 .38583 m
.53921 .38302 L
s
.47653 .42504 m
.47971 .41882 L
s
.50265 .40199 m
.50681 .39762 L
s
.53921 .38302 m
.54437 .38022 L
s
.4321 .51856 m
.4324 .5056 L
s
.45809 .44906 m
.46031 .44076 L
s
.44326 .47986 m
.44452 .46927 L
s
.42398 .55063 m
.42333 .53518 L
s
.50681 .39762 m
.51099 .39327 L
s
.54437 .38022 m
.54954 .37742 L
s
.47971 .41882 m
.48291 .41264 L
s
.4324 .5056 m
.43271 .49257 L
s
.46031 .44076 m
.46254 .43251 L
s
.44452 .46927 m
.44579 .45871 L
s
.54954 .37742 m
.55475 .37462 L
s
.51099 .39327 m
.5152 .38895 L
s
.48291 .41264 m
.48613 .40651 L
s
.42333 .53518 m
.42268 .51957 L
s
.46254 .43251 m
.46479 .42435 L
s
.55475 .37462 m
.55998 .37183 L
s
.43271 .49257 m
.43301 .47956 L
s
.41608 .56909 m
.41447 .55081 L
s
.5152 .38895 m
.51943 .38467 L
s
.44579 .45871 m
.44706 .44823 L
s
.48613 .40651 m
.48938 .40046 L
s
.55998 .37183 m
.56523 .36904 L
s
.42268 .51957 m
.42203 .50393 L
s
.51943 .38467 m
.52369 .38042 L
s
.46479 .42435 m
.46705 .41629 L
s
.43301 .47956 m
.43332 .46665 L
s
.44706 .44823 m
.44834 .43789 L
s
.48938 .40046 m
.49264 .39448 L
s
.41447 .55081 m
.41285 .5324 L
s
.56523 .36904 m
.57051 .36626 L
s
.40809 .58674 m
.40551 .56548 L
s
.52369 .38042 m
.52797 .37621 L
s
.42203 .50393 m
.42136 .48838 L
s
.46705 .41629 m
.46933 .40837 L
s
.49264 .39448 m
.49592 .38859 L
s
.57051 .36626 m
.57582 .36349 L
s
.43332 .46665 m
.43363 .45391 L
s
.44834 .43789 m
.44963 .42771 L
s
.41285 .5324 m
.41122 .51402 L
s
.52797 .37621 m
.53227 .37204 L
s
.57582 .36349 m
.58116 .36073 L
s
.46933 .40837 m
.47162 .40059 L
s
.40551 .56548 m
.40291 .54413 L
s
.42136 .48838 m
.42069 .47303 L
s
.49592 .38859 m
.49922 .38279 L
s
.44963 .42771 m
.45093 .41774 L
s
.43363 .45391 m
.43393 .44141 L
s
.40002 .60361 m
.39646 .57922 L
s
.53227 .37204 m
.5366 .36792 L
s
.58116 .36073 m
.58652 .35797 L
s
.41122 .51402 m
.40957 .49584 L
s
.47162 .40059 m
.47393 .39297 L
s
.49922 .38279 m
.50255 .37709 L
s
.42069 .47303 m
.42001 .45798 L
s
.5366 .36792 m
.54096 .36385 L
s
.40291 .54413 m
.40029 .52292 L
s
.58652 .35797 m
.59191 .35523 L
s
.45093 .41774 m
.45223 .408 L
s
.43393 .44141 m
.43424 .4292 L
s
.39646 .57922 m
.39287 .55483 L
s
.50255 .37709 m
.50589 .37149 L
s
.47393 .39297 m
.47625 .38552 L
s
.39188 .61972 m
.38732 .59208 L
s
.40957 .49584 m
.4079 .478 L
s
.54096 .36385 m
.54534 .35982 L
s
.59191 .35523 m
.59733 .3525 L
s
.42001 .45798 m
.41931 .44331 L
s
.45223 .408 m
.45354 .39852 L
s
.40029 .52292 m
.39763 .50205 L
s
.43424 .4292 m
.43454 .41732 L
s
.59733 .3525 m
.60278 .34977 L
s
.50589 .37149 m
.50926 .366 L
s
.54534 .35982 m
.54974 .35584 L
s
.47625 .38552 m
.47859 .37825 L
s
.39287 .55483 m
.38923 .5307 L
s
.4079 .478 m
.4062 .46061 L
s
.38732 .59208 m
.38271 .56454 L
s
.38365 .6351 m
.37809 .60408 L
s
.45354 .39852 m
.45487 .38931 L
s
.60278 .34977 m
.60825 .34705 L
s
.41931 .44331 m
.41861 .42908 L
s
.54974 .35584 m
.55418 .35191 L
s
.43454 .41732 m
.43485 .40581 L
s
.50926 .366 m
.51265 .36062 L
s
.39763 .50205 m
.39494 .48168 L
s
.47859 .37825 m
.48094 .37117 L
s
.60825 .34705 m
.61376 .34435 L
s
.38923 .5307 m
.38555 .50709 L
s
.4062 .46061 m
.40449 .44378 L
s
.55418 .35191 m
.55864 .34803 L
s
.45487 .38931 m
.4562 .38038 L
s
.38271 .56454 m
.37805 .53744 L
s
.51265 .36062 m
.51606 .35534 L
s
.41861 .42908 m
.4179 .41534 L
s
.37809 .60408 m
.37245 .57332 L
s
.43485 .40581 m
.43515 .39469 L
s
.48094 .37117 m
.48331 .36427 L
s
.61376 .34435 m
.61929 .34165 L
s
.37535 .64975 m
.36876 .61526 L
s
.39494 .48168 m
.39222 .46196 L
s
.55864 .34803 m
.56312 .34419 L
s
.51606 .35534 m
.51949 .35016 L
s
.38555 .50709 m
.38182 .48419 L
s
.4562 .38038 m
.45753 .37174 L
s
.40449 .44378 m
.40275 .42757 L
s
.61929 .34165 m
.62485 .33896 L
s
.48331 .36427 m
.4857 .35756 L
s
.37805 .53744 m
.37333 .51107 L
s
.4179 .41534 m
.41717 .40212 L
s
.43515 .39469 m
.43545 .38398 L
s
.56312 .34419 m
.56764 .3404 L
s
.37245 .57332 m
.36675 .5432 L
s
.36876 .61526 m
.36209 .5812 L
s
.39222 .46196 m
.38946 .443 L
s
.62485 .33896 m
.63044 .33627 L
s
.36697 .66371 m
.35934 .62565 L
s
.51949 .35016 m
.52295 .34509 L
s
.45753 .37174 m
.45888 .36339 L
s
.38182 .48419 m
.37805 .46217 L
s
.4857 .35756 m
.48811 .35104 L
s
.40275 .42757 m
.40099 .41204 L
s
.56764 .3404 m
.57218 .33665 L
s
.63044 .33627 m
.63606 .3336 L
s
.43545 .38398 m
.43575 .37368 L
s
.41717 .40212 m
.41643 .38946 L
s
.37333 .51107 m
.36855 .48565 L
s
.52295 .34509 m
.52643 .34012 L
s
.36675 .5432 m
.36097 .51405 L
s
.38946 .443 m
.38666 .42488 L
s
.36209 .5812 m
.35533 .54804 L
s
.57218 .33665 m
.57674 .33295 L
s
.63606 .3336 m
.64171 .33093 L
s
.45888 .36339 m
.46024 .35533 L
s
.35934 .62565 m
.35162 .58825 L
s
.35851 .67699 m
.34983 .63529 L
s
.48811 .35104 m
.49053 .34469 L
s
.40099 .41204 m
.3992 .39722 L
s
.37805 .46217 m
.37421 .44114 L
s
.52643 .34012 m
.52993 .33525 L
s
.43575 .37368 m
.43605 .36379 L
s
.41643 .38946 m
.41567 .37736 L
s
.64171 .33093 m
.64739 .32827 L
s
.57674 .33295 m
.58134 .32928 L
s
.36855 .48565 m
.36369 .46136 L
s
.46024 .35533 m
.4616 .34754 L
s
.49053 .34469 m
.49297 .33852 L
s
.36097 .51405 m
.35511 .48614 L
s
.38666 .42488 m
.38381 .40764 L
s
.35533 .54804 m
.34847 .51613 L
s
.52993 .33525 m
.53346 .33047 L
s
.64739 .32827 m
.6531 .32562 L
s
.34997 .68962 m
.34024 .6442 L
s
.35162 .58825 m
.34378 .55202 L
s
.34983 .63529 m
.34104 .5945 L
s
.58134 .32928 m
.58597 .32565 L
s
.43605 .36379 m
.43635 .35431 L
s
.3992 .39722 m
.39738 .38312 L
s
.37421 .44114 m
.37032 .42118 L
s
.41567 .37736 m
.41491 .36582 L
s
.49297 .33852 m
.49543 .33251 L
s
.6531 .32562 m
.65884 .32297 L
s
.4616 .34754 m
.46298 .34002 L
s
.53346 .33047 m
.53702 .32578 L
s
.36369 .46136 m
.35876 .43833 L
s
.58597 .32565 m
.59062 .32206 L
s
.38381 .40764 m
.38093 .39132 L
s
.35511 .48614 m
.34915 .45964 L
s
.65884 .32297 m
.66462 .32033 L
s
.34136 .70161 m
.33055 .65241 L
s
.34847 .51613 m
.3415 .48576 L
s
.43635 .35431 m
.43664 .34523 L
s
.34024 .6442 m
.33036 .6 L
s
.34378 .55202 m
.33582 .51737 L
s
.34104 .5945 m
.33211 .55521 L
s
.39738 .38312 m
.39553 .36976 L
s
.41491 .36582 m
.41413 .35482 L
s
.49543 .33251 m
.4979 .32666 L
s
.53702 .32578 m
.54059 .32117 L
s
.59062 .32206 m
.5953 .3185 L
s
.37032 .42118 m
.36637 .40234 L
s
.46298 .34002 m
.46436 .33277 L
s
.66462 .32033 m
.67042 .31769 L
s
.35876 .43833 m
.35375 .41663 L
s
.38093 .39132 m
.378 .37592 L
s
.5953 .3185 m
.60002 .31497 L
s
.43664 .34523 m
.43694 .33652 L
s
.34915 .45964 m
.34309 .43469 L
s
.54059 .32117 m
.5442 .31663 L
s
.4979 .32666 m
.5004 .32095 L
s
.33267 .71298 m
.32077 .65996 L
s
.67042 .31769 m
.67625 .31505 L
s
.39553 .36976 m
.39366 .35711 L
s
.33055 .65241 m
.31957 .60477 L
s
.3415 .48576 m
.33442 .45713 L
s
.41413 .35482 m
.41333 .34436 L
s
.46436 .33277 m
.46576 .32575 L
s
.33036 .6 m
.32032 .55764 L
s
.33582 .51737 m
.32773 .4846 L
s
.33211 .55521 m
.32304 .51785 L
s
.36637 .40234 m
.36235 .38463 L
s
.60002 .31497 m
.60476 .31147 L
s
.67625 .31505 m
.68212 .31242 L
s
.5442 .31663 m
.54783 .31217 L
s
.5004 .32095 m
.50292 .31539 L
s
.43694 .33652 m
.43723 .32816 L
s
.35375 .41663 m
.34865 .39629 L
s
.378 .37592 m
.37502 .36142 L
s
.46576 .32575 m
.46716 .31896 L
s
.3239 .72375 m
.3109 .66686 L
s
.68212 .31242 m
.68802 .30979 L
s
.34309 .43469 m
.33693 .41135 L
s
.41333 .34436 m
.41252 .33439 L
s
.60476 .31147 m
.60953 .308 L
s
.39366 .35711 m
.39175 .34514 L
s
.32077 .65996 m
.30868 .60887 L
s
.31957 .60477 m
.30841 .55938 L
s
.33442 .45713 m
.3272 .43035 L
s
.54783 .31217 m
.55148 .30777 L
s
.32032 .55764 m
.31011 .51761 L
s
.32773 .4846 m
.31949 .45391 L
s
.32304 .51785 m
.3138 .48273 L
s
.36235 .38463 m
.35827 .36805 L
s
.50292 .31539 m
.50545 .30995 L
s
.68802 .30979 m
.69395 .30716 L
s
.43723 .32816 m
.43753 .32015 L
s
.60953 .308 m
.61434 .30454 L
s
.46716 .31896 m
.46858 .31238 L
s
.37502 .36142 m
.37199 .34779 L
s
.34865 .39629 m
.34347 .37732 L
s
.31506 .73394 m
.30095 .67315 L
s
.55148 .30777 m
.55517 .30344 L
s
.41252 .33439 m
.4117 .32491 L
s
.69395 .30716 m
.69991 .30453 L
s
.39175 .34514 m
.38982 .33383 L
s
.3109 .66686 m
.29768 .61233 L
s
.50545 .30995 m
.50801 .30463 L
s
.33693 .41135 m
.33065 .38963 L
s
.61434 .30454 m
.61917 .30111 L
s
.30868 .60887 m
.29637 .56046 L
s
.30841 .55938 m
.29703 .51673 L
s
.3272 .43035 m
.31986 .40547 L
s
.31011 .51761 m
.29969 .48022 L
s
.43753 .32015 m
.43782 .31243 L
s
.35827 .36805 m
.35412 .35254 L
s
.31949 .45391 m
.3111 .42539 L
s
.3138 .48273 m
.30437 .45006 L
s
.69991 .30453 m
.70591 .3019 L
s
.46858 .31238 m
.47001 .306 L
s
.55517 .30344 m
.55888 .29916 L
s
.61917 .30111 m
.62403 .2977 L
s
.30614 .74356 m
.2909 .67886 L
s
.50801 .30463 m
.51058 .29941 L
s
.37199 .34779 m
.36892 .33499 L
s
.4117 .32491 m
.41086 .31586 L
s
.34347 .37732 m
.33819 .35967 L
s
.30095 .67315 m
.28658 .61518 L
s
.70591 .3019 m
.71194 .29927 L
s
.38982 .33383 m
.38785 .32312 L
s
.29768 .61233 m
.2842 .56094 L
s
.55888 .29916 m
.56261 .29493 L
s
.33065 .38963 m
.32426 .36951 L
s
.47001 .306 m
.47145 .29979 L
s
.43782 .31243 m
.43812 .30501 L
s
.62403 .2977 m
.62893 .29431 L
s
.29637 .56046 m
.2838 .51525 L
s
.71194 .29927 m
.718 .29665 L
s
.35412 .35254 m
.3499 .33806 L
s
.31986 .40547 m
.31237 .38248 L
s
.29703 .51673 m
.28541 .47714 L
s
.51058 .29941 m
.51318 .29429 L
s
.29969 .48022 m
.28906 .44565 L
s
.3111 .42539 m
.30253 .39908 L
s
.29714 .75264 m
.28076 .68399 L
s
.30437 .45006 m
.29476 .4199 L
s
.41086 .31586 m
.41001 .30721 L
s
.56261 .29493 m
.56638 .29074 L
s
.36892 .33499 m
.36579 .32295 L
s
.62893 .29431 m
.63386 .29093 L
s
.2909 .67886 m
.27536 .61746 L
s
.718 .29665 m
.7241 .29402 L
s
.38785 .32312 m
.38586 .31298 L
s
.33819 .35967 m
.33282 .34327 L
s
.47145 .29979 m
.4729 .29374 L
s
.28658 .61518 m
.27191 .56085 L
s
.43812 .30501 m
.43842 .29783 L
s
.51318 .29429 m
.5158 .28925 L
s
.2842 .56094 m
.27043 .51322 L
s
.63386 .29093 m
.63882 .28756 L
s
.7241 .29402 m
.73023 .29139 L
s
.32426 .36951 m
.31775 .3509 L
s
.56638 .29074 m
.57017 .2866 L
s
.28807 .76119 m
.27053 .68859 L
s
.2838 .51525 m
.27096 .47354 L
s
.3499 .33806 m
.3456 .32453 L
s
.41001 .30721 m
.40915 .29892 L
s
.31237 .38248 m
.30473 .3613 L
s
.28541 .47714 m
.27354 .44077 L
s
.28076 .68399 m
.26404 .61919 L
s
.4729 .29374 m
.47436 .28782 L
s
.30253 .39908 m
.29379 .37491 L
s
.28906 .44565 m
.2782 .41396 L
s
.36579 .32295 m
.36262 .31161 L
s
.29476 .4199 m
.28494 .39226 L
s
.73023 .29139 m
.7364 .28876 L
s
.38586 .31298 m
.38383 .30334 L
s
.5158 .28925 m
.51844 .2843 L
s
.63882 .28756 m
.64381 .2842 L
s
.43842 .29783 m
.43871 .29088 L
s
.27536 .61746 m
.25949 .56024 L
s
.57017 .2866 m
.57399 .28249 L
s
.33282 .34327 m
.32736 .32804 L
s
.27191 .56085 m
.25691 .5107 L
s
.27892 .76923 m
.26021 .69267 L
s
.7364 .28876 m
.7426 .28613 L
s
.64381 .2842 m
.64884 .28085 L
s
.40915 .29892 m
.40827 .29095 L
s
.31775 .3509 m
.31112 .33371 L
s
.27043 .51322 m
.25634 .46947 L
s
.47436 .28782 m
.47584 .28204 L
s
.27053 .68859 m
.25262 .62041 L
s
.51844 .2843 m
.5211 .2794 L
s
.3456 .32453 m
.34124 .31186 L
s
.57399 .28249 m
.57784 .2784 L
s
.43871 .29088 m
.43901 .28413 L
s
.27096 .47354 m
.25782 .43545 L
s
.7426 .28613 m
.74883 .2835 L
s
.38383 .30334 m
.38178 .29416 L
s
.30473 .3613 m
.29694 .34184 L
s
.36262 .31161 m
.3594 .30091 L
s
.26404 .61919 m
.24694 .55913 L
s
.27354 .44077 m
.2614 .40763 L
s
.64884 .28085 m
.6539 .2775 L
s
.29379 .37491 m
.28486 .3528 L
s
.2782 .41396 m
.26708 .38508 L
s
.28494 .39226 m
.27491 .36702 L
s
.26969 .77677 m
.2498 .69626 L
s
.32736 .32804 m
.32179 .31387 L
s
.25949 .56024 m
.24323 .50772 L
s
.74883 .2835 m
.7551 .28086 L
s
.57784 .2784 m
.58172 .27435 L
s
.5211 .2794 m
.52378 .27457 L
s
.47584 .28204 m
.47733 .27635 L
s
.26021 .69267 m
.24108 .62115 L
s
.40827 .29095 m
.40738 .28327 L
s
.6539 .2775 m
.65899 .27416 L
s
.25691 .5107 m
.24153 .46499 L
s
.43901 .28413 m
.43931 .27756 L
s
.34124 .31186 m
.3368 .29998 L
s
.31112 .33371 m
.30436 .31782 L
s
.7551 .28086 m
.76141 .27823 L
s
.25262 .62041 m
.23426 .55757 L
s
.38178 .29416 m
.37969 .28538 L
s
.25634 .46947 m
.24189 .42977 L
s
.58172 .27435 m
.58562 .27032 L
s
.3594 .30091 m
.35612 .29077 L
s
.26039 .78383 m
.2393 .69937 L
s
.52378 .27457 m
.52649 .26978 L
s
.29694 .34184 m
.28899 .32395 L
s
.47733 .27635 m
.47883 .27076 L
s
.65899 .27416 m
.66412 .27082 L
s
.25782 .43545 m
.24435 .40096 L
s
.24694 .55913 m
.2294 .50432 L
s
.76141 .27823 m
.76775 .27559 L
s
.2498 .69626 m
.22943 .62143 L
s
.28486 .3528 m
.27575 .33257 L
s
.2614 .40763 m
.24895 .37761 L
s
.40738 .28327 m
.40647 .27583 L
s
.32179 .31387 m
.31613 .30066 L
s
.26708 .38508 m
.25571 .35887 L
s
.27491 .36702 m
.26465 .34405 L
s
.43931 .27756 m
.43961 .27113 L
s
.58562 .27032 m
.58956 .2663 L
s
.24323 .50772 m
.22654 .46014 L
s
.66412 .27082 m
.66928 .26748 L
s
.76775 .27559 m
.77413 .27295 L
s
.52649 .26978 m
.52922 .26504 L
s
.24108 .62115 m
.22145 .55558 L
s
.25101 .79042 m
.2287 .70203 L
s
.37969 .28538 m
.37757 .27694 L
s
.3368 .29998 m
.33228 .28878 L
s
.47883 .27076 m
.48035 .26524 L
s
.30436 .31782 m
.29747 .30309 L
s
.24153 .46499 m
.22575 .42375 L
s
.35612 .29077 m
.35279 .28111 L
s
.2393 .69937 m
.21767 .62127 L
s
.23426 .55757 m
.21541 .50054 L
s
.58956 .2663 m
.59353 .2623 L
s
.77413 .27295 m
.78054 .27032 L
s
.66928 .26748 m
.67447 .26414 L
s
.24189 .42977 m
.22707 .39401 L
s
.40647 .27583 m
.40556 .2686 L
s
.43961 .27113 m
.43991 .26483 L
s
.28899 .32395 m
.28088 .30747 L
s
.52922 .26504 m
.53197 .26033 L
s
.24156 .79656 m
.21801 .70425 L
s
.24435 .40096 m
.23054 .36989 L
s
.31613 .30066 m
.31036 .28828 L
s
.22943 .62143 m
.20851 .5532 L
s
.2294 .50432 m
.21137 .45495 L
s
.27575 .33257 m
.26643 .31405 L
s
.78054 .27032 m
.78699 .26768 L
s
.48035 .26524 m
.48188 .25979 L
s
.24895 .37761 m
.2362 .3505 L
s
.26465 .34405 m
.25415 .32313 L
s
.67447 .26414 m
.6797 .2608 L
s
.59353 .2623 m
.59753 .25831 L
s
.25571 .35887 m
.24407 .33512 L
s
.37757 .27694 m
.37542 .26879 L
s
.33228 .28878 m
.32769 .27817 L
s
.2287 .70203 m
.2058 .62071 L
s
.35279 .28111 m
.34941 .27187 L
s
.22654 .46014 m
.20938 .41745 L
s
.22145 .55558 m
.20126 .49642 L
s
.78699 .26768 m
.79348 .26504 L
s
.53197 .26033 m
.53474 .25564 L
s
.43991 .26483 m
.44022 .25863 L
s
.29747 .30309 m
.29045 .28937 L
s
.40556 .2686 m
.40463 .26155 L
s
.23202 .80226 m
.20723 .70606 L
s
.6797 .2608 m
.68497 .25746 L
s
.59753 .25831 m
.60155 .25432 L
s
.22575 .42375 m
.20952 .38682 L
s
.48188 .25979 m
.48343 .25439 L
s
.21767 .62127 m
.19542 .55045 L
s
.79348 .26504 m
.80001 .2624 L
s
.21541 .50054 m
.196 .44946 L
s
.28088 .30747 m
.27261 .29222 L
s
.22707 .39401 m
.21184 .36196 L
s
.31036 .28828 m
.30449 .27662 L
s
.21801 .70425 m
.19381 .61977 L
s
.37542 .26879 m
.37324 .26089 L
s
.53474 .25564 m
.53754 .25099 L
s
.68497 .25746 m
.69027 .25411 L
s
.26643 .31405 m
.25692 .29702 L
s
.23054 .36989 m
.21636 .34196 L
s
.20851 .5532 m
.18695 .49197 L
s
.44022 .25863 m
.44052 .25251 L
s
.60155 .25432 m
.60561 .25034 L
s
.80001 .2624 m
.80657 .25976 L
s
.32769 .27817 m
.32303 .26806 L
s
.34941 .27187 m
.34598 .26298 L
s
.21137 .45495 m
.1928 .4109 L
s
.22241 .80755 m
.19635 .70748 L
s
.40463 .26155 m
.40368 .25463 L
s
.25415 .32313 m
.24342 .30403 L
s
.2362 .3505 m
.22313 .32604 L
s
.24407 .33512 m
.23215 .31357 L
s
.48343 .25439 m
.48499 .24904 L
s
.2058 .62071 m
.1822 .54736 L
s
.69027 .25411 m
.6956 .25076 L
s
.29045 .28937 m
.28329 .27653 L
s
.53754 .25099 m
.54037 .24634 L
s
.80657 .25976 m
.81318 .25712 L
s
.20938 .41745 m
.19173 .37941 L
s
.20126 .49642 m
.18044 .44371 L
s
.20723 .70606 m
.18171 .61846 L
s
.60561 .25034 m
.60971 .24637 L
s
.37324 .26089 m
.37103 .25319 L
s
.21272 .81242 m
.18538 .70852 L
s
.44052 .25251 m
.44083 .24646 L
s
.19542 .55045 m
.17247 .48724 L
s
.27261 .29222 m
.26417 .27804 L
s
.30449 .27662 m
.29852 .26556 L
s
.20952 .38682 m
.19284 .35386 L
s
.6956 .25076 m
.70097 .24741 L
s
.81318 .25712 m
.81982 .25448 L
s
.48499 .24904 m
.48656 .24371 L
s
.40368 .25463 m
.40273 .24783 L
s
.196 .44946 m
.17599 .40412 L
s
.34598 .26298 m
.3425 .25437 L
s
.19381 .61977 m
.16884 .54396 L
s
.32303 .26806 m
.31828 .25837 L
s
.54037 .24634 m
.54322 .24171 L
s
.60971 .24637 m
.61383 .24239 L
s
.21184 .36196 m
.19619 .33328 L
s
.25692 .29702 m
.2472 .28129 L
s
.19635 .70748 m
.16949 .61682 L
s
.81982 .25448 m
.8265 .25184 L
s
.70097 .24741 m
.70638 .24405 L
s
.21636 .34196 m
.20181 .31684 L
s
.18695 .49197 m
.16468 .43771 L
s
.24342 .30403 m
.23244 .2865 L
s
.20295 .8169 m
.17431 .70919 L
s
.22313 .32604 m
.20973 .3039 L
s
.1928 .4109 m
.17366 .37183 L
s
.23215 .31357 m
.21994 .29393 L
s
.28329 .27653 m
.276 .26442 L
s
.44083 .24646 m
.44114 .24046 L
s
.48656 .24371 m
.48815 .23841 L
s
.37103 .25319 m
.36879 .24565 L
s
.1822 .54736 m
.15782 .48224 L
s
.61383 .24239 m
.61798 .23841 L
s
.54322 .24171 m
.54609 .23709 L
s
.8265 .25184 m
.83322 .2492 L
s
.40273 .24783 m
.40176 .24111 L
s
.18171 .61846 m
.15534 .54027 L
s
.70638 .24405 m
.71183 .24069 L
s
.19173 .37941 m
.17353 .34562 L
s
.29852 .26556 m
.29244 .255 L
s
.18044 .44371 m
.15894 .39715 L
s
.18538 .70852 m
.15716 .61485 L
s
.3425 .25437 m
.33896 .246 L
s
.26417 .27804 m
.25556 .26473 L
s
.31828 .25837 m
.31346 .24902 L
s
.19311 .821 m
.16314 .70953 L
s
.83322 .2492 m
.83998 .24657 L
s
.61798 .23841 m
.62217 .23442 L
s
.17247 .48724 m
.14872 .4315 L
s
.48815 .23841 m
.48976 .23312 L
s
.19284 .35386 m
.17567 .32447 L
s
.44114 .24046 m
.44146 .23449 L
s
.71183 .24069 m
.71732 .23732 L
s
.54609 .23709 m
.54899 .23248 L
s
.2472 .28129 m
.23727 .26664 L
s
.16884 .54396 m
.143 .477 L
s
.17599 .40412 m
.15532 .36409 L
s
.36879 .24565 m
.36651 .23823 L
s
.83998 .24657 m
.84678 .24394 L
s
.16949 .61682 m
.14169 .5363 L
s
.19619 .33328 m
.1801 .30754 L
s
.40176 .24111 m
.40077 .23446 L
s
.276 .26442 m
.26857 .2529 L
s
.17431 .70919 m
.1447 .61258 L
s
.23244 .2865 m
.22122 .27029 L
s
.20181 .31684 m
.18687 .29414 L
s
.18318 .82473 m
.15188 .70953 L
s
.71732 .23732 m
.72284 .23394 L
s
.62217 .23442 m
.62639 .23043 L
s
.21994 .29393 m
.20744 .2759 L
s
.20973 .3039 m
.19599 .28374 L
s
.16468 .43771 m
.14165 .39001 L
s
.33896 .246 m
.33537 .2378 L
s
.54899 .23248 m
.55192 .22786 L
s
.84678 .24394 m
.85362 .24131 L
s
.48976 .23312 m
.49138 .22784 L
s
.17366 .37183 m
.15392 .33725 L
s
.29244 .255 m
.28625 .24483 L
s
.44146 .23449 m
.44177 .22854 L
s
.31346 .24902 m
.30856 .23993 L
s
.15782 .48224 m
.13255 .4251 L
s
.25556 .26473 m
.24677 .25215 L
s
.15534 .54027 m
.128 .47155 L
s
.72284 .23394 m
.7284 .23056 L
s
.15716 .61485 m
.1279 .53208 L
s
.16314 .70953 m
.13212 .61002 L
s
.62639 .23043 m
.63065 .22643 L
s
.85362 .24131 m
.86051 .23868 L
s
.36651 .23823 m
.3642 .23089 L
s
.17353 .34562 m
.15479 .31556 L
s
.17318 .8281 m
.14052 .70921 L
s
.40077 .23446 m
.39978 .22784 L
s
.15894 .39715 m
.13671 .35622 L
s
.55192 .22786 m
.55487 .22324 L
s
.49138 .22784 m
.49302 .22256 L
s
.23727 .26664 m
.22713 .25286 L
s
.7284 .23056 m
.734 .22717 L
s
.17567 .32447 m
.15799 .29815 L
s
.26857 .2529 m
.261 .24185 L
s
.86051 .23868 m
.86743 .23605 L
s
.14872 .4315 m
.12412 .38272 L
s
.44177 .22854 m
.44209 .22261 L
s
.33537 .2378 m
.33173 .22973 L
s
.63065 .22643 m
.63494 .22241 L
s
.143 .477 m
.11618 .41853 L
s
.22122 .27029 m
.20974 .25515 L
s
.15188 .70953 m
.11942 .6072 L
s
.14169 .5363 m
.11282 .46589 L
s
.1447 .61258 m
.11395 .52763 L
s
.1801 .30754 m
.16357 .2843 L
s
.15532 .36409 m
.13398 .32877 L
s
.16309 .83113 m
.12905 .7086 L
s
.28625 .24483 m
.27996 .23497 L
s
.30856 .23993 m
.30358 .23103 L
s
.20744 .2759 m
.19464 .2592 L
s
.55487 .22324 m
.55785 .21862 L
s
.18687 .29414 m
.17154 .27347 L
s
.86743 .23605 m
.8744 .23343 L
s
.734 .22717 m
.73964 .22378 L
s
.19599 .28374 m
.1819 .26522 L
s
.39978 .22784 m
.39877 .22125 L
s
.3642 .23089 m
.36187 .22362 L
s
.49302 .22256 m
.49467 .21729 L
s
.24677 .25215 m
.23781 .24014 L
s
.63494 .22241 m
.63926 .21839 L
s
.14165 .39001 m
.11781 .34822 L
s
.15392 .33725 m
.13354 .30656 L
s
.44209 .22261 m
.44241 .21668 L
s
.8744 .23343 m
.88141 .23081 L
s
.14052 .70921 m
.1066 .60413 L
s
.15292 .83383 m
.11749 .7077 L
s
.13255 .4251 m
.10634 .3753 L
s
.73964 .22378 m
.74532 .22039 L
s
.13212 .61002 m
.09986 .52296 L
s
.55785 .21862 m
.56085 .21399 L
s
.1279 .53208 m
.09746 .46005 L
s
.128 .47155 m
.09958 .4118 L
s
.33173 .22973 m
.32803 .22176 L
s
.261 .24185 m
.25329 .23116 L
s
.22713 .25286 m
.21678 .23977 L
s
.15479 .31556 m
.13547 .28867 L
s
.63926 .21839 m
.64362 .21435 L
s
.49467 .21729 m
.49634 .21201 L
s
.88141 .23081 m
.88846 .2282 L
s
.39877 .22125 m
.39775 .21467 L
s
.13671 .35622 m
.11371 .3202 L
s
.30358 .23103 m
.29852 .22229 L
s
.36187 .22362 m
.3595 .21638 L
s
.27996 .23497 m
.27356 .22534 L
s
.74532 .22039 m
.75103 .21699 L
s
.20974 .25515 m
.198 .24087 L
s
.15799 .29815 m
.1398 .27438 L
s
.44241 .21668 m
.44274 .21074 L
s
.14268 .8362 m
.10582 .70653 L
s
.12905 .7086 m
.09364 .60081 L
s
.56085 .21399 m
.56389 .20934 L
s
.12412 .38272 m
.09861 .34013 L
s
.88846 .2282 m
.89556 .22559 L
s
.11942 .6072 m
.0856 .5181 L
s
.16357 .2843 m
.14657 .26312 L
s
.19464 .2592 m
.18154 .24357 L
s
.64362 .21435 m
.64801 .2103 L
s
.11395 .52763 m
.08192 .45404 L
s
.11618 .41853 m
.0883 .36776 L
s
.23781 .24014 m
.22867 .22856 L
s
.11282 .46589 m
.08277 .40493 L
s
.17154 .27347 m
.15579 .25445 L
s
.1819 .26522 m
.16745 .24802 L
s
.13398 .32877 m
.11192 .29747 L
s
.75103 .21699 m
.75679 .21359 L
s
.49634 .21201 m
.49803 .20672 L
s
.32803 .22176 m
.32427 .21383 L
s
.89556 .22559 m
.90269 .22298 L
s
.39775 .21467 m
.39671 .20809 L
s
.13235 .83826 m
.09405 .7051 L
s
.11749 .7077 m
.08056 .59728 L
s
.56389 .20934 m
.56695 .20469 L
s
.3595 .21638 m
.35709 .20915 L
s
.25329 .23116 m
.24544 .22075 L
s
.44274 .21074 m
.44307 .20479 L
s
.64801 .2103 m
.65244 .20624 L
s
.11781 .34822 m
.0931 .31153 L
s
.13354 .30656 m
.11252 .27911 L
s
.75679 .21359 m
.76259 .21018 L
s
.29852 .22229 m
.29338 .21363 L
s
.21678 .23977 m
.20621 .22722 L
s
.1066 .60413 m
.07119 .51304 L
s
.27356 .22534 m
.26705 .21587 L
s
.09986 .52296 m
.06618 .44788 L
s
.49803 .20672 m
.49973 .20142 L
s
.10634 .3753 m
.07912 .33193 L
s
.09746 .46005 m
.06573 .39794 L
s
.09958 .4118 m
.07 .36011 L
s
.13547 .28867 m
.11556 .26438 L
s
.12193 .84002 m
.08217 .70342 L
s
.198 .24087 m
.186 .22725 L
s
.76259 .21018 m
.76844 .20678 L
s
.65244 .20624 m
.65691 .20216 L
s
.56695 .20469 m
.57004 .20002 L
s
.10582 .70653 m
.06735 .59353 L
s
.39671 .20809 m
.39566 .20149 L
s
.32427 .21383 m
.32046 .20594 L
s
.11371 .3202 m
.08991 .2883 L
s
.44307 .20479 m
.4434 .19882 L
s
.22867 .22856 m
.21935 .2173 L
s
.1398 .27438 m
.12108 .25268 L
s
.09364 .60081 m
.05662 .50782 L
s
.18154 .24357 m
.16812 .22874 L
s
.35709 .20915 m
.35466 .20192 L
s
.14657 .26312 m
.1291 .24358 L
s
.49973 .20142 m
.50145 .1961 L
s
.16745 .24802 m
.15265 .23185 L
s
.76844 .20678 m
.77432 .20337 L
s
.0856 .5181 m
.05025 .44158 L
s
.15579 .25445 m
.13963 .23672 L
s
.29338 .21363 m
.28816 .20503 L
s
.07466 .30634 m
.09861 .34013 L
s
.65691 .20216 m
.66141 .19806 L
s
.11144 .8415 m
.07018 .7015 L
s
.24544 .22075 m
.23745 .21052 L
s
.57004 .20002 m
.57315 .19533 L
s
.05245 .39839 m
.08192 .45404 L
s
.26705 .21587 m
.26042 .20651 L
s
.11192 .29747 m
.08913 .26947 L
s
.09405 .7051 m
.054 .58958 L
s
.06652 .33462 m
.0883 .36776 L
s
.05918 .36535 m
.08277 .40493 L
s
.20621 .22722 m
.19542 .21506 L
s
.39566 .20149 m
.3946 .19487 L
s
.4434 .19882 m
.44374 .19284 L
s
.77432 .20337 m
.78025 .19996 L
s
.08056 .59728 m
.04189 .50243 L
s
.32046 .20594 m
.3166 .19804 L
s
.66141 .19806 m
.66595 .19395 L
s
.50145 .1961 m
.50319 .19077 L
s
.10086 .84269 m
.05809 .69936 L
s
.09436 .25833 m
.11252 .27911 L
s
.35466 .20192 m
.35218 .19466 L
s
.57315 .19533 m
.5763 .19062 L
s
.07908 .29374 m
.0931 .31153 L
s
.04316 .45415 m
.07119 .51304 L
s
.186 .22725 m
.17373 .2141 L
s
.08217 .70342 m
.04052 .58545 L
s
.21935 .2173 m
.20985 .20626 L
s
.78025 .19996 m
.78622 .19656 L
s
.04923 .41696 m
.06618 .44788 L
s
.1003 .24783 m
.11556 .26438 L
s
.28816 .20503 m
.28285 .19645 L
s
.07047 .32033 m
.07912 .33193 L
s
.66595 .19395 m
.67052 .18982 L
s
.16812 .22874 m
.15439 .21453 L
s
.03463 .51519 m
.06735 .59353 L
s
.44374 .19284 m
.44407 .18683 L
s
.05573 .38183 m
.06573 .39794 L
s
.3946 .19487 m
.39353 .18822 L
s
.06277 .34958 m
.07 .36011 L
s
.23745 .21052 m
.2293 .2004 L
s
.09019 .84362 m
.04589 .697 L
s
.26042 .20651 m
.25368 .19719 L
s
.50319 .19077 m
.50495 .18541 L
s
.10678 .23776 m
.12108 .25268 L
s
.5763 .19062 m
.57947 .18589 L
s
.08382 .28124 m
.08991 .2883 L
s
.15265 .23185 m
.13747 .21644 L
s
.78622 .19656 m
.79223 .19316 L
s
.1139 .22809 m
.1291 .24358 L
s
.13963 .23672 m
.12305 .21997 L
s
.07018 .7015 m
.0269 .58115 L
s
.3166 .19804 m
.31267 .19013 L
s
.04022 .47437 m
.05662 .50782 L
s
.35218 .19466 m
.34968 .18737 L
s
.19542 .21506 m
.1844 .20317 L
s
.67052 .18982 m
.67514 .18568 L
s
.04619 .43442 m
.05025 .44158 L
s
.07944 .84428 m
.03358 .69444 L
s
.03185 .53782 m
.054 .58958 L
s
.79223 .19316 m
.79829 .18976 L
s
.44407 .18683 m
.44442 .18079 L
s
.0889 .26923 m
.08913 .26947 L
s
.57947 .18589 m
.58268 .18113 L
s
.50495 .18541 m
.50672 .18004 L
s
.39353 .18822 m
.39244 .18154 L
s
.28285 .19645 m
.27746 .18785 L
s
.20985 .20626 m
.20016 .19536 L
s
.17373 .2141 m
.16119 .2013 L
s
.02396 .6062 m
.05809 .69936 L
s
.67514 .18568 m
.67979 .18152 L
s
.03743 .49361 m
.04189 .50243 L
s
.79829 .18976 m
.8044 .18636 L
s
.25368 .19719 m
.24683 .1879 L
s
.2293 .2004 m
.22101 .19035 L
s
.34968 .18737 m
.34714 .18004 L
s
.0686 .84469 m
.02115 .69168 L
s
.31267 .19013 m
.30869 .18217 L
s
.14053 .20093 m
.15439 .21453 L
s
.0292 .55963 m
.04052 .58545 L
s
.58268 .18113 m
.58591 .17636 L
s
.50672 .18004 m
.50851 .17464 L
s
.44442 .18079 m
.44476 .17473 L
s
.67979 .18152 m
.68448 .17734 L
s
.02134 .63126 m
.04589 .697 L
s
.39244 .18154 m
.39133 .17481 L
s
.8044 .18636 m
.81055 .18297 L
s
.13061 .20987 m
.13747 .21644 L
s
.1844 .20317 m
.17314 .19144 L
s
.1218 .21879 m
.12305 .21997 L
s
.01388 .70551 m
.05768 .84486 L
s
.27746 .18785 m
.27198 .17923 L
s
.58591 .17636 m
.58918 .17155 L
s
.02666 .58061 m
.0269 .58115 L
s
.68448 .17734 m
.68921 .17315 L
s
.81055 .18297 m
.81674 .17958 L
s
.20016 .19536 m
.19028 .18453 L
s
.34714 .18004 m
.34457 .17267 L
s
.50851 .17464 m
.51032 .16922 L
s
.30869 .18217 m
.30465 .17417 L
s
.44476 .17473 m
.44511 .16864 L
s
.01882 .65566 m
.03358 .69444 L
s
.24683 .1879 m
.23985 .17858 L
s
.15181 .19209 m
.16119 .2013 L
s
.22101 .19035 m
.21256 .1803 L
s
.01132 .73384 m
.04666 .8448 L
s
.39133 .17481 m
.39021 .16804 L
s
.81674 .17958 m
.82298 .1762 L
s
.58918 .17155 m
.59248 .16672 L
s
.68921 .17315 m
.69398 .16895 L
s
.51032 .16922 m
.51215 .16377 L
s
.01639 .6794 m
.02115 .69168 L
s
.27198 .17923 m
.26641 .17055 L
s
.16478 .18297 m
.17314 .19144 L
s
.44511 .16864 m
.44547 .16251 L
s
.34457 .17267 m
.34196 .16523 L
s
.00882 .76171 m
.03556 .84451 L
s
.82298 .1762 m
.82927 .17282 L
s
.30465 .17417 m
.30054 .1661 L
s
.69398 .16895 m
.6988 .16474 L
s
.59248 .16672 m
.5958 .16186 L
s
.39021 .16804 m
.38908 .16123 L
s
.23985 .17858 m
.23276 .16921 L
s
.19028 .18453 m
.18021 .17372 L
s
.21256 .1803 m
.20395 .17023 L
s
.51215 .16377 m
.514 .15829 L
s
.00638 .7891 m
.02436 .84401 L
s
.44547 .16251 m
.44582 .15635 L
s
.6988 .16474 m
.70365 .16051 L
s
.5958 .16186 m
.59916 .15697 L
s
.34196 .16523 m
.33931 .15773 L
s
.26641 .17055 m
.26075 .1618 L
s
.38908 .16123 m
.38793 .15436 L
s
.30054 .1661 m
.29637 .15797 L
s
.514 .15829 m
.51587 .15278 L
s
.00401 .81599 m
.01308 .84329 L
s
.70365 .16051 m
.70855 .15627 L
s
.23276 .16921 m
.22554 .15978 L
s
.59916 .15697 m
.60256 .15204 L
s
.44582 .15635 m
.44619 .15016 L
s
.17986 .17335 m
.18021 .17372 L
s
.19764 .16293 m
.20395 .17023 L
s
.33931 .15773 m
.33662 .15017 L
s
.70855 .15627 m
.71348 .15203 L
s
.51587 .15278 m
.51776 .14723 L
s
.38793 .15436 m
.38677 .14745 L
s
.26075 .1618 m
.255 .15297 L
s
.60256 .15204 m
.60598 .14709 L
s
.29637 .15797 m
.29214 .14975 L
s
.44619 .15016 m
.44655 .14393 L
s
.71348 .15203 m
.71846 .14777 L
s
.21894 .15123 m
.22554 .15978 L
s
.51776 .14723 m
.51967 .14164 L
s
.60598 .14709 m
.60944 .14211 L
s
.33662 .15017 m
.3339 .14254 L
s
.38677 .14745 m
.38559 .14049 L
s
.71846 .14777 m
.72349 .14351 L
s
.44655 .14393 m
.44692 .13766 L
s
.255 .15297 m
.24915 .14405 L
s
.29214 .14975 m
.28785 .14145 L
s
.51967 .14164 m
.52161 .13601 L
s
.60944 .14211 m
.61293 .13709 L
s
.72349 .14351 m
.72856 .13925 L
s
.38559 .14049 m
.3844 .13348 L
s
.3339 .14254 m
.33114 .13484 L
s
.44692 .13766 m
.44729 .13135 L
s
.61293 .13709 m
.61646 .13204 L
s
.52161 .13601 m
.52356 .13035 L
s
.28785 .14145 m
.28349 .13306 L
s
.24493 .13765 m
.24915 .14405 L
s
.72856 .13925 m
.73367 .13499 L
s
.3844 .13348 m
.38319 .12642 L
s
.33114 .13484 m
.32833 .12707 L
s
.44729 .13135 m
.44767 .12499 L
s
.61646 .13204 m
.62002 .12697 L
s
.73367 .13499 m
.73883 .13072 L
s
.52356 .13035 m
.52553 .12463 L
s
.28349 .13306 m
.27906 .12458 L
s
.73883 .13072 m
.74404 .12645 L
s
.62002 .12697 m
.62362 .12186 L
s
.38319 .12642 m
.38196 .11931 L
s
.44767 .12499 m
.44806 .11859 L
s
.52553 .12463 m
.52753 .11887 L
s
.32833 .12707 m
.32549 .11922 L
s
.74404 .12645 m
.74929 .12218 L
s
.62362 .12186 m
.62725 .11672 L
s
.2774 .12142 m
.27906 .12458 L
s
.52753 .11887 m
.52954 .11307 L
s
.44806 .11859 m
.44844 .11214 L
s
.38196 .11931 m
.38072 .11214 L
s
.32549 .11922 m
.3226 .11131 L
s
.62725 .11672 m
.63092 .11156 L
s
.52954 .11307 m
.53158 .10721 L
s
.44844 .11214 m
.44883 .10564 L
s
.38072 .11214 m
.37945 .10492 L
s
.63092 .11156 m
.63463 .10637 L
s
.3226 .11131 m
.31967 .10332 L
s
.53158 .10721 m
.53365 .1013 L
s
.44883 .10564 m
.44923 .09908 L
s
.63463 .10637 m
.63837 .10116 L
s
.37945 .10492 m
.37817 .09764 L
s
.3191 .10176 m
.31967 .10332 L
s
.53365 .1013 m
.53573 .09534 L
s
.63837 .10116 m
.64216 .09593 L
s
.44923 .09908 m
.44963 .09247 L
s
.37817 .09764 m
.37688 .0903 L
s
.53573 .09534 m
.53784 .08933 L
s
.64216 .09593 m
.64598 .09067 L
s
.44963 .09247 m
.45004 .08579 L
s
.37688 .0903 m
.37556 .08289 L
s
.53784 .08933 m
.53998 .08327 L
s
.64598 .09067 m
.64985 .0854 L
s
.45004 .08579 m
.45045 .07904 L
s
.64985 .0854 m
.65375 .08011 L
s
.53998 .08327 m
.54214 .07715 L
s
.37465 .0778 m
.37556 .08289 L
s
.45045 .07904 m
.45086 .07223 L
s
.65375 .08011 m
.65769 .07481 L
s
.54214 .07715 m
.54432 .07098 L
s
.45086 .07223 m
.45129 .06534 L
s
.65769 .07481 m
.66168 .06949 L
s
.54432 .07098 m
.54653 .06477 L
s
.45129 .06534 m
.45171 .05838 L
s
.54653 .06477 m
.54877 .0585 L
s
.45171 .05838 m
.45215 .05134 L
s
.54877 .0585 m
.55103 .05219 L
s
.55103 .05219 m
.55332 .04583 L
s
.45244 .04661 m
.45215 .05134 L
s
.55332 .04583 m
.55563 .03942 L
s
.55563 .03942 m
.55798 .03298 L
s
.55798 .03298 m
.56035 .0265 L
s
.56035 .0265 m
.56275 .01997 L
s
.56275 .01997 m
.56519 .01342 L
s
.25 Mabswid
.03399 .19842 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.96816 .22579 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
.03399 .19842 m
0 .87626 L
s
.02756 .32664 m
.03396 .32857 L
s
[(100)] .01476 .32278 1 .30143 Mshowa
.02095 .45839 m
.02744 .46 L
s
[(200)] .00798 .45516 1 .24828 Mshowa
.01416 .5938 m
.02072 .59507 L
s
[(300)] .00104 .59125 1 .19393 Mshowa
.00718 .73304 m
.0138 .73395 L
s
[(400)] -0.00606 .7312 1 .13836 Mshowa
gsave
-0.06073 .5083 -105.625 -11.9382 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 263.5},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P00011a000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00;Coo`@000?oo`03003ooooo06;oo`00]_oo00<00?ooool0Iooo002fool0
0`00ooooo`1Wool00;Koo`03003ooooo06Ooo`00]_oo00<00?ooool0Iooo002dool3001Yool00;Ko
o`03003ooooo06Ooo`00oooo8Ooo003ooolQool007Woo`03003ooooo0:Coo`00K?oo1@001?oo0P00
0ooo00<00?ooool00_oo0`00=_oo0P00I_oo001]ool00`00ool00004ool01000ooooo`000_oo00<0
0?ooool01Ooo00<00?ooool0<Ooo0P0000Coo`00000006Coo`00KOoo00<00?oo00001?oo00<00?oo
ool00ooo00<00?ooool00ooo0P00<Ooo0`000ooo00<00?ooool00P00H_oo001^ool00`00ooooo`03
ool00`00ooooo`04ool01@00oooooooo0000<?oo0`001_oo00<00?oo00000_oo0P00H?oo001^ool0
0`00ooooo`02ool40004ool00`00ooooo`02ool3000[ool20009ool00`00ool00004ool2001Nool0
06koo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo02coo`<000_oo`03003oo`0000Koo`80
05coo`00M?oo00<00?ooool0<Ooo0`003_oo0P002Ooo00<00?ooool0FOoo0025ool20004ool00`00
ooooo`03ool00`00ooooo`0Bool2000Aool2000:ool2001Iool008Coo`04003ooooo0009ool00`00
ooooo`0?ool3000Cool2000<ool2001Gool008Coo`04003ooooo000:ool00`00ooooo`0;ool3000F
ool00`00ooooo`0=ool2001Eool008Coo`04003ooooo000:ool00`00ooooo`09ool200000ooo003o
o`0Eool2000Aool2001Cool008Coo`04003ooooo000;ool00`00ooooo`05ool30003ool00`00oooo
o`0Dool2000Cool2001Aool008Coo`04003ooooo0008ool01000ooooo`001?oo0`001_oo00<00?oo
ool05?oo0P005Ooo0P00Cooo0025ool20009ool40002ool20009ool00`00ooooo`0Cool00`00ool0
000Gool00`00ooooo`1<ool009?oo`<000_oo`03003ooooo01?oo`03003oo`0001Soo`8004coo`00
TOoo0P003_oo00<00?ooool04ooo00<00?oo00006_oo0P00B_oo002>ool3000@ool00`00ooooo`0B
ool01000ooooo`005ooo00<00?ooool00_oo0P00B?oo002;ool30002ool00`00ooooo`0>ool00`00
ooooo`0Bool01000ooooo`005_oo00<00?ooool01Ooo0P00A_oo0029ool200000ooo003oo`02ool0
0`00ooooo`0>ool00`00ooooo`0Bool01000ooooo`005Ooo00<00?ooool02?oo0P00A?oo0026ool3
0007ool00`00ooooo`0>ool00`00ooooo`0Aool01@00oooooooo00005?oo00<00?ooool02ooo00<0
0?ooool0@Ooo0023ool3000:ool00`00ooooo`0>ool00`00ooooo`0Aool01@00oooooooo00005?oo
00<00?ooool03?oo0P00@Ooo0021ool2000=ool00`00ooooo`0>ool00`00ooooo`0@ool00`00oooo
o`02ool00`00ooooo`0Bool00`00ooooo`0>ool2000oool005koo`8000Coo`03003ooooo00Coo`03
003ooooo013oo`<000ooo`03003ooooo00koo`03003ooooo013oo`03003ooooo00;oo`03003ooooo
017oo`03003ooooo017oo`8003goo`00GOoo00@00?ooool000[oo`03003ooooo00goo`<000Goo`03
003ooooo00[oo`03003ooooo00koo`03003ooooo013oo`03003ooooo00;oo`03003ooooo013oo`03
003ooooo01Coo`8003_oo`00GOoo00@00?ooool000Ooo`D000coo`8000Soo`03003ooooo00_oo`03
003ooooo00goo`03003ooooo00ooo`03003ooooo00?oo`03003ooooo00ooo`03003ooooo01Ooo`80
03Woo`00GOoo00@00?ooool000Ooo`04003ooooo000:ool3000;ool00`00ooooo`0:ool00`00oooo
o`0<ool00`00ooooo`0@ool00`00ooooo`03ool00`00ooooo`0>ool00`00ooooo`0Jool00`00oooo
o`0fool005goo`04003ooooo0008ool00`00ool00007ool3000>ool00`00ooooo`0:ool00`00oooo
o`0<ool00`00ooooo`0@ool00`00ooooo`03ool00`00ooooo`0=ool00`00ooooo`0Lool2000fool0
05goo`04003ooooo0009ool20005ool20006ool00`00ooooo`08ool00`00ooooo`0:ool00`00oooo
o`0<ool00`00ooooo`0@ool00`00ooooo`03ool00`00ooooo`0<ool00`00ooooo`0Oool2000dool0
05koo`8000_oo`03003ooooo00<000Woo`03003ooooo00Ooo`03003ooooo00[oo`03003ooooo00co
o`03003ooooo00ooo`03003ooooo00Coo`03003ooooo00coo`03003ooooo01[oo`03003ooooo00Co
o`8003;oo`00Jooo0`003?oo00<00?ooool02?oo00<00?ooool02_oo00<00?ooool02ooo00<00?oo
ool03ooo00<00?ooool01?oo00<00?ooool03?oo00<00?ooool06?oo0P002Ooo0P00<?oo001Yool2
000@ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`0;ool00`00ooooo`0?ool00`00oooo
o`04ool00`00ooooo`0;ool00`00ooooo`0Gool2000=ool2000^ool006Koo`<000Woo`03003ooooo
00Koo`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00_oo`03003ooooo00koo`03003ooooo
00Goo`03003ooooo00[oo`03003ooooo01Ooo`03003ooooo00ooo`8002coo`00I?oo0P003?oo00<0
0?ooool01ooo00<00?ooool01ooo00<00?ooool02Ooo00<00?ooool02ooo00<00?ooool03_oo00<0
0?ooool01Ooo00<00?ooool02Ooo00<00?ooool05ooo00<00?ooool04_oo00<00?ooool0:Ooo001Q
ool4000>ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`09ool00`00ooooo`0;ool00`00
ooooo`0=ool00`00ooooo`06ool00`00ooooo`08ool00`00ooooo`0Gool00`00ooooo`0Dool2000Y
ool005koo`<000_oo`03003ooooo00Coo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00So
o`03003ooooo00_oo`03003ooooo00goo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo01Ko
o`03003ooooo01Ooo`8002Ooo`00>ooo0P001?oo00<00?ooool00_oo10004_oo0P003ooo00<00?oo
ool01?oo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00<00?ooool02_oo00<00?oo
ool03Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool05_oo00<00?ooool06_oo0P009Ooo
000jool01000ooooo`002_oo00<00?ooool03_oo0`004_oo00<00?ooool01?oo00<00?ooool01_oo
00<00?ooool01ooo00<00?ooool02?oo00<00?ooool02_oo00<00?ooool03?oo00<00?ooool01ooo
00<00?ooool01ooo00<00?ooool05Ooo00<00?ooool07Ooo0P008ooo000jool01000ooooo`002_oo
00<00?ooool02ooo0`003_oo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?oo
ool01ooo00<00?ooool02?oo00<00?ooool02_oo00<00?ooool03?oo00<00?ooool01ooo00<00?oo
ool01_oo00<00?ooool05Ooo00<00?ooool08?oo0P008Ooo000jool01000ooooo`002_oo00<00?oo
ool02Ooo0P004_oo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01_oo
00<00?ooool02?oo00<00?ooool02_oo00<00?ooool03?oo00<00?ooool01ooo00<00?ooool01Ooo
00<00?ooool05?oo0P009Ooo00<00?ooool07_oo000jool01000ooooo`002_oo00<00?ooool01_oo
0`003ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool01_oo00<00?oo
ool01_oo00<00?ooool01ooo00<00?ooool02_oo00<00?ooool03?oo00<00?ooool01ooo00<00?oo
ool01Ooo00<00?ooool04ooo00<00?ooool07Ooo00<00?ooool01_oo0P007_oo000jool01000oooo
o`002?oo0`001Ooo0`004ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool00ooo00<00?oo
ool01_oo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02_oo00<00?ooool02ooo00<00?oo
ool02?oo00<00?ooool01?oo00<00?ooool04ooo00<00?ooool07Ooo00<00?ooool02Ooo0P007?oo
000kool2000;ool01000oooooooo0P004_oo00D00?ooooooo`0000Koo`03003ooooo00?oo`03003o
oooo00Coo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Soo`03003ooooo00Woo`03003o
oooo00_oo`03003ooooo00Soo`03003ooooo00?oo`03003ooooo01?oo`03003ooooo01_oo`<000ko
o`8001[oo`00BOoo0`005Ooo00D00?ooooooo`0000Koo`03003ooooo00?oo`03003ooooo00?oo`03
003ooooo00Koo`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00_oo`03
003ooooo00Woo`03003ooooo00;oo`03003ooooo01?oo`03003ooooo01[oo`8001?oo`8001Soo`00
A_oo0`006Ooo00@00?ooooooo`8000Goo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00_oo`03003ooooo00Wo
o`03003ooooo00;oo`03003ooooo01;oo`03003ooooo01[oo`03003ooooo01Goo`8001Koo`00A?oo
0P005ooo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo00<00?oo
ool01?oo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02?oo00<00?oo
ool02ooo00<00?ooool02Ooo00D00?ooooooo`0001Coo`03003ooooo01[oo`03003ooooo01Soo`80
01Coo`00@_oo0P006_oo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool0
0ooo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool0
2?oo00<00?ooool02ooo00<00?ooool02Ooo00@00?ooool001Coo`03003ooooo01Soo`<001goo`03
003ooooo017oo`00@_oo00<00?oo00000P004ooo00<00?ooool00_oo00<00?ooool00_oo00<00?oo
ool00_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool00ooo00<00?oo
ool01_oo00<00?ooool01_oo00<00?ooool02?oo00<00?ooool02_oo00<00?ooool02_oo00@00?oo
ool001?oo`03003ooooo01Soo`03003ooooo01ooo`80017oo`00@_oo00<00?ooool00_oo0`004Ooo
00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool01ooo
00<00?ooool02_oo00<00?ooool02_oo00<00?oo00004ooo00<00?ooool05ooo0P009?oo0P003ooo
0012ool00`00ooooo`05ool2000<ool01@00oooooooo00001?oo00<00?ooool00_oo00<00?ooool0
0_oo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool0
1Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00<00?ooool02ooo00<00?oo0000
4_oo00<00?ooool05_oo0P00:?oo0P003Ooo0012ool00`00ooooo`07ool3000:ool01@00oooooooo
00001?oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo00<00?oo
ool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?oo
ool02Ooo00<00?ooool02ooo0P004_oo00<00?ooool05Ooo0P00;?oo0P002ooo0012ool00`00oooo
o`0:ool30004ool01@00oooooooo00000ooo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool0
0_oo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool0
1?oo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00<00?ooool02ooo00<00?ooool0
3ooo0P005ooo00<00?ooool0;_oo0P002Ooo0012ool00`00ooooo`0=ool30002ool01@00oooooooo
00000ooo00<00?ooool00_oo00D00?ooooooo`0000Coo`03003ooooo00?oo`03003ooooo00?oo`03
003ooooo00;oo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Ooo`03
003ooooo00Woo`03003ooooo00_oo`03003ooooo00koo`03003ooooo01Koo`03003ooooo023oo`<0
00goo`8000Soo`00@_oo00<00?ooool03ooo1@000ooo00D00?ooooooo`0000Coo`05003oooooool0
0004ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`04ool00`00oooo
o`04ool00`00ooooo`05ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`0;ool2000?ool0
0`00ooooo`0Dool3000Pool3000<ool400000ooo003oo`07ool004;oo`03003ooooo013oo`03003o
oooo008000?oo`05003oooooool00004ool01000oooooooo0P000ooo00<00?ooool00ooo00<00?oo
ool00_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01_oo00<00?oo
ool01_oo00<00?ooool02?oo00<00?ooool02_oo00<00?oo00003_oo00<00?ooool05?oo00<00?oo
ool07ooo0P002ooo10001Ooo00<00?ooool01_oo0012ool00`00ooooo`0=ool01000oooooooo0P00
0_oo0`0000?oo`00ool00_oo00D00?ooooooo`0000Goo`05003oooooool00004ool00`00ooooo`02
ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`06
ool00`00ooooo`08ool00`00ooooo`0:ool00`00ool0000=ool00`00ooooo`0Dool00`00ooooo`0M
ool30009ool40009ool00`00ooooo`06ool0047oo`03003ooooo00ooo`03003ooooo00;oo`05003o
oooo003oo`030003ool01@00oooooooo00001Ooo00D00?ooooooo`0000Coo`03003ooooo00;oo`03
003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Koo`03
003ooooo00Ooo`03003ooooo00[oo`04003ooooo000<ool00`00ooooo`0Bool3000Mool30009ool3
000=ool00`00ooooo`06ool0047oo`03003ooooo00goo`03003ooooo008000?oo`04003ooooo0003
ool300000ooo003oo`02ool00`00ooooo`02ool01@00oooooooo00001?oo00<00?ooool00_oo00<0
0?ooool01?oo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01_oo00<0
0?ooool01ooo00<00?ooool02Ooo00D00?ooooooo`0000_oo`03003ooooo01;oo`03003ooooo01co
o`8000Soo`@0013oo`03003ooooo00Koo`00@Ooo00<00?ooool03_oo00D00?ooooooo`0000;oo`05
003oooooool00003ool00`00ool000020003ool00`00ooooo`02ool01@00oooooooo00001?oo00<0
0?ooool00_oo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool01Ooo00<00?ooool01?oo00<0
0?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00D00?ooooooo`0000[oo`03003ooooo01;o
o`03003ooooo01[oo`<000Koo`@001Coo`03003ooooo00Koo`00@Ooo00<00?ooool03_oo00<00?oo
ool00_oo00@00?ooool000?oo`05003oooooool00002ool30002ool01@00oooooooo00001?oo00<0
0?ooool00_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool01?oo00<0
0?ooool01?oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool02Ooo00<00?ooool00_oo00<0
0?ooool01ooo00<00?ooool04?oo0`006_oo0`001Ooo10006?oo00<00?ooool01_oo0011ool00`00
ooooo`0<ool01000ooooo`001?oo00<00?oo00001?oo00D00?ooooooo`0000;oo`03003oo`0000<0
00?oo`05003oooooool00004ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`02ool00`00
ooooo`04ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`09ool00`00
ooooo`02ool00`00ooooo`07ool00`00ooooo`0?ool00`00ooooo`0Jool00`00ooooo`03ool3000L
ool00`00ooooo`06ool0047oo`03003ooooo00goo`04003ooooo0003ool01000ooooo`000ooo00<0
0?ooool00_oo00@00?ooool000?oo`<00003ool00?oo00?oo`03003ooooo00;oo`03003ooooo00;o
o`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Ko
o`03003ooooo00Koo`03003ooooo00Soo`03003ooooo00?oo`03003ooooo00Goo`80017oo`03003o
oooo01Soo`<000;oo`@001ooo`03003ooooo00Koo`00@Ooo00<00?ooool02_oo00D00?ooooooo`00
00;oo`05003oooooool00002ool01@00oooooooo00001?oo00<00?oo00001Ooo10001?oo00<00?oo
ool00_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool01?oo00<00?oo
ool01?oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool02?oo00<00?ooool01?oo00<00?oo
ool01?oo00<00?ooool04?oo00<00?ooool05_oo0`0000?oo`0000000P009?oo00<00?ooool01Ooo
0011ool00`00ooooo`0;ool01@00oooooooo00000_oo00D00?ooooooo`0000;oo`05003oooooool0
0003ool01000ooooo`001?oo00@00?ooooooo`<00003ool00?oo00?oo`03003ooooo00;oo`03003o
oooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Coo`03003o
oooo00Koo`03003ooooo00Soo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo00koo`<001Oo
o`800004ool00000000Xool00`00ooooo`05ool0047oo`03003ooooo00coo`07003oooooool00?oo
00001?oo00<00?oo00001?oo00D00?ooooooo`0000;oo`03003ooooo00;oo`04003ooooo0002ool3
0004ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`03ool00`00oooo
o`04ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`05ool00`00oooo
o`02ool00`00ooooo`0>ool00`00ooooo`0Fool5000[ool00`00ooooo`05ool0047oo`03003ooooo
00Woo`05003oooooool00003ool00`00ool00003ool01000ooooo`001?oo00D00?ooooooo`0000;o
o`05003oooooool00003ool01000oooooooo0`000_oo00<00?ooool00_oo00<00?ooool00_oo00<0
0?ooool00ooo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool01_oo00<0
0?ooool01ooo00<00?ooool01Ooo00D00?ooooooo`00013oo`03003ooooo01Coo`@002ooo`03003o
oooo00Goo`00@Ooo00<00?ooool02_oo00@00?ooool000?oo`04003ooooo0003ool01000ooooo`00
0ooo00<00?ooool00_oo00<00?oo00001?oo00D00?ooooooo`0000?oo`03003oo`0000<000Coo`03
003ooooo00;oo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03
003ooooo00Goo`03003ooooo00Ooo`03003ooooo00Koo`04003ooooo000?ool2000Cool4000cool0
0`00ooooo`05ool0047oo`03003ooooo00_oo`04003ooooo0003ool01000ooooo`000ooo00@00?oo
ool000?oo`05003oooooool00002ool00`00ooooo`02ool01@00oooooooo00000ooo00@00?oooooo
o`<000;oo`03003ooooo00;oo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03003o
oooo00Coo`03003ooooo00Goo`03003ooooo00Ooo`03003ooooo00Koo`03003oo`0000ooo`03003o
oooo013oo`D003Goo`03003ooooo00Goo`00@Ooo00<00?ooool02?oo00D00?ooooooo`0000;oo`05
003oooooool00002ool01000ooooo`000ooo00D00?ooooooo`0000?oo`04003ooooo0003ool00`00
ooooo`02ool01000ooooo`001?oo00<00?oo00000`000ooo00<00?ooool00_oo00<00?ooool00ooo
00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo
00<00?oo00003_oo00<00?ooool03Ooo1P00>?oo00<00?ooool01Ooo0011ool00`00ooooo`09ool0
1000ooooo`000ooo00L00?ooooooo`00ool00003ool01000ooooo`001?oo00@00?ooool000?oo`05
003oooooool00003ool01@00oooooooo00000ooo00<00?ooool00_oo0P000ooo00<00?ooool00_oo
00<00?ooool00_oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01ooo0P003?oo0`003?oo10000_oo0P00>_oo00<00?ooool01Ooo0011ool00`00oooo
o`09ool01@00oooooooo00000ooo00@00?ooool000;oo`05003oooooool00002ool01@00oooooooo
00000ooo00@00?ooool000Coo`05003oooooool00002ool00`00ooooo`02ool00`00ooooo`02ool5
0004ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`04ool00`00ooooo`05ool00`00oooo
o`05ool00`00ooooo`08ool00`00ooooo`0:ool00`00ooooo`09ool40004ool2000lool00`00oooo
o`05ool002coo`@000?oo`8000Coo`8000Koo`03003ooooo00Ooo`04003ooooo0003ool01000oooo
o`000ooo00@00?ooool000;oo`05003oooooool00003ool01@00oooooooo00000_oo00D00?oooooo
o`0000?oo`05003oooooool00004ool01@00oooooooo00000ooo0`000ooo00<00?ooool00_oo00<0
0?ooool00ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool02?oo0P00
2ooo00<00?ooool01ooo0`001Ooo0`00?_oo00<00?ooool01Ooo000^ool01@00oooooooo00000_oo
00@00?ooool000;oo`03003ooooo00?oo`03003ooooo00Soo`04003ooooo0003ool01000ooooo`00
0ooo00<00?oo00000ooo00D00?ooooooo`0000;oo`05003oooooool00003ool01@00oooooooo0000
0ooo00@00?ooool000Coo`03003ooooo00;oo`06003oooooool00?oo10001?oo00<00?ooool00ooo
00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01ooo00<00?oo00002_oo
00<00?ooool01?oo10001ooo00<00?ooool0?ooo00<00?ooool01Ooo000^ool01@00oooooooo0000
0_oo00@00?ooool000;oo`03003ooooo00?oo`03003ooooo00Woo`04003ooooo0002ool01@00oooo
oooo00000_oo00@00?ooool000?oo`04003ooooo0003ool01@00oooooooo00000ooo00@00?ooool0
00?oo`05003oooooool00004ool01@00oooooooo00000ooo00<00?ooool00_oo0`000_oo00<00?oo
ool00ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01_oo00@00?oo
ool000Soo`8000?oo`@000Soo`<004;oo`03003ooooo00Goo`00;_oo00D00?ooooooo`0000;oo`04
003ooooo0002ool00`00ooooo`03ool00`00ooooo`06ool01`00oooooooo003oo`0000?oo`05003o
ooooool00002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`05003oooooool00003ool01@00
oooooooo00000ooo00@00?ooool000Coo`05003oooooool00004ool00`00ooooo`02ool01P00ool0
0000ool000Coo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Ooo`04
003ooooo0007ool00`00ool000030009ool30015ool00`00ooooo`05ool002koo`05003oooooool0
0002ool01000ooooo`000_oo00<00?ooool00_oo0P002Ooo00@00?ooool000;oo`05003oooooool0
0002ool01000ooooo`000_oo00D00?ooooooo`0000?oo`04003ooooo0003ool01@00oooooooo0000
0_oo00D00?ooooooo`0000?oo`05003oooooool00004ool01@00oooooooo00001?oo00@00?oooooo
o`<000?oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00Koo`05003o
ooooool00006ool3000<ool00`00ooooo`16ool00`00ooooo`05ool002coo`<000?oo`04003ooooo
0002ool01000ooooo`001?oo0`002Ooo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`00
0_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01@00oooooooo00000ooo00D00?ooooooo`00
00;oo`03003ooooo00;oo`05003oooooool00003ool00`00ooooo`02ool00`00ooooo`02ool00`00
ool000030005ool00`00ooooo`03ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`02ool0
0`00ooooo`04000<ool30019ool00`00ooooo`05ool002koo`03003ooooo00;oo`8000Coo`8000Go
o`03003ooooo00Koo`05003oooooool00002ool01000ooooo`000_oo00@00?ooool000;oo`05003o
ooooool00002ool01@00oooooooo00000_oo00D00?ooooooo`0000?oo`04003ooooo0003ool01@00
oooooooo00000ooo00D00?ooooooo`0000Coo`05003oooooool00004ool00`00ooooo`02ool30003
ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`02ool40002ool00`00
ooooo`09ool2001<ool00`00ooooo`05ool0043oo`03003ooooo00Ooo`04003ooooo0003ool00`00
ool00002ool01@00oooooooo00000_oo00D00?ooooooo`0000;oo`04003ooooo0002ool01@00oooo
oooo00000ooo00D00?ooooooo`0000;oo`05003oooooool00003ool00`00ooooo`02ool01@00oooo
oooo00000ooo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool010001Ooo00<00?ooool01?oo
00<00?ooool01?oo00<00?ooool00`0000Coo`00ooooo`8000_oo`03003ooooo04coo`03003ooooo
00Goo`00@?oo00<00?ooool01ooo00D00?ooooooo`0000;oo`04003ooooo0002ool01000ooooo`00
0ooo00@00?ooool000?oo`04003ooooo0002ool01@00oooooooo00000ooo00@00?ooool000?oo`05
003oooooool00003ool01@00oooooooo00000ooo00<00?ooool00_oo00<00?ooool00_oo00D00?oo
ooooo`0000Goo`8000Coo`03003ooooo00?oo`03003ooooo00Coo`@000Coo`03003oo`0000[oo`<0
04ooo`03003ooooo00Goo`00@?oo00<00?ooool02?oo00@00?ooool000?oo`04003ooooo0002ool0
1000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01000ooooo`000ooo00D00?oo
ooooo`0000;oo`05003oooooool00003ool01@00oooooooo00001?oo00D00?ooooooo`0000Coo`05
003oooooool00005ool00`00ool000030006ool00`00ooooo`050007ool2000:ool00`00ooooo`1@
ool00`00ooooo`05ool0043oo`03003ooooo00Goo`04003ooooo0003ool01000ooooo`000_oo00@0
0?ooool000;oo`05003oooooool00003ool01000ooooo`000_oo00@00?ooool000?oo`04003ooooo
0003ool01@00oooooooo00000ooo00D00?ooooooo`0000;oo`03003ooooo00;oo`05003oooooool0
0004ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`02ool30003ool40004ool00`00oooo
o`05ool00`00ooooo`06ool3001Dool00`00ooooo`04ool0043oo`03003ooooo00Koo`04003ooooo
0003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000_oo00D0
0?ooooooo`0000;oo`05003oooooool00003ool01000ooooo`000ooo00D00?ooooooo`0000?oo`05
003oooooool00004ool01@00oooooooo00001?oo00<00?ooool00_oo00<00?ooool00_oo00<00?oo
ool010001_oo00<00?ooool01Ooo0P001_oo0P00Eooo00<00?ooool01?oo0010ool00`00ooooo`07
ool01000ooooo`000_oo00@00?ooool000;oo`07003oooooool00?oo00000ooo00D00?ooooooo`00
00;oo`04003ooooo0003ool01000ooooo`000_oo00D00?ooooooo`0000?oo`05003oooooool00002
ool01@00oooooooo00000ooo00<00?ooool00_oo00D00?ooooooo`0000Coo`03003ooooo00;oo`03
003ooooo00;oo`07003oooooool00?oo00001Ooo00<00?ooool01Ooo00<00?oo00001Ooo00<00?oo
ool0Eooo00<00?ooool01?oo0010ool00`00ooooo`05ool01P00ooooo`00ool000?oo`04003ooooo
0002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo0002ool01@00oooooooo0000
0_oo00@00?ooool000Coo`04003ooooo0003ool01@00oooooooo00000_oo00<00?ooool00_oo00D0
0?ooooooo`0000?oo`03003ooooo00;oo`03003ooooo00;oo`03003ooooo00;oo`04003oooooool2
0005ool00`00ooooo`05ool01000ooooo`000_oo0`00F_oo00<00?ooool01?oo0010ool00`00oooo
o`06ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000?oo`04003ooooo0002ool01@00
oooooooo00000_oo00@00?ooool000?oo`04003ooooo0002ool01@00oooooooo00000ooo00@00?oo
ool000?oo`04003ooooo0004ool01@00oooooooo00000ooo00<00?ooool00_oo00<00?ooool00_oo
00<00?ooool00_oo00@00?ooooooo`8000Goo`03003ooooo00Coo`07003oooooool00?oo0000GOoo
00<00?ooool01?oo0010ool00`00ooooo`06ool01000ooooo`000_oo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`000ooo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000_oo
00<00?ooool00_oo00@00?ooool000?oo`04003ooooo0003ool01@00oooooooo00000ooo00D00?oo
ooooo`0000Coo`03003ooooo00;oo`03003ooooo00;oo`04003oooooool20005ool00`00ooooo`03
ool00`00ooooo`02ool2001Nool00`00ooooo`04ool0043oo`03003ooooo00Ooo`03003oo`0000;o
o`04003ooooo0002ool01000ooooo`000ooo00@00?ooool000;oo`05003oooooool00002ool01000
ooooo`000ooo00@00?ooool000;oo`05003oooooool00002ool01@00oooooooo00000ooo00@00?oo
ool000?oo`05003oooooool00004ool01@00oooooooo00001?oo00<00?ooool00_oo00D00?oooooo
o`0000Goo`03003ooooo00?oo`04003oooooool3001Oool00`00ooooo`04ool0043oo`03003ooooo
00Coo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00
oooooooo00000_oo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`000ooo00D00?oo
ooooo`0000;oo`05003oooooool00002ool00`00ooooo`02ool01000ooooo`001?oo00D00?oooooo
o`0000Coo`05003oooooool00004ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`020002
ool00`00ooooo`1Mool00`00ooooo`04ool0043oo`03003ooooo00Coo`07003oooooool00?oo0000
0ooo00<00?oo00000ooo00<00?oo00000ooo00@00?ooool000;oo`05003oooooool00002ool01000
ooooo`000ooo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`001?oo00@00?ooool0
00?oo`05003oooooool00003ool01@00oooooooo00001?oo00D00?ooooooo`0000Coo`05003ooooo
ool00004ool200000ooo00000004ool00`00ooooo`1Mool00`00ooooo`04ool0043oo`03003ooooo
00Goo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00
oooooooo00000_oo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`000ooo00@00?oo
ool000?oo`05003oooooool00002ool01@00oooooooo00000ooo00D00?ooooooo`0000?oo`05003o
ooooool00004ool01000oooooooo0P000ooo00D00?ooooooo`0000;oo`03003ooooo00Coo`03003o
oooo05goo`03003ooooo00Coo`00@?oo00<00?ooool01Ooo00D00?ooooooo`0000;oo`06003ooooo
003oo`000ooo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?ooool0
00?oo`04003ooooo0002ool01000ooooo`000ooo00@00?ooool000?oo`05003oooooool00002ool0
1@00oooooooo00000ooo00D00?ooooooo`0000?oo`03003ooooo00;oo`04003oooooool20003ool0
0`00ooooo`040007ool00`00ooooo`1Mool00`00ooooo`04ool0043oo`03003ooooo00?oo`04003o
oooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo
00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000_oo00D00?ooooooo`0000;oo`05003o
ooooool00002ool01@00oooooooo00000ooo00@00?ooool000?oo`05003oooooool00003ool00`00
ooooo`02ool01000oooooooo0P000_oo00<00?ooool00P002_oo00<00?ooool0GOoo00<00?ooool0
1?oo0010ool00`00ooooo`04ool00`00ool00003ool01000ooooo`000_oo00<00?oo00000ooo00@0
0?ooool000;oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?ooool000;oo`05003ooooo
ool00002ool01000ooooo`000_oo00D00?ooooooo`0000?oo`04003ooooo0003ool01@00oooooooo
00000_oo00D00?ooooooo`0000?oo`03003ooooo00;oo`04003oooooool20002ool01000ool00000
2ooo00<00?ooool0GOoo00<00?ooool01?oo0010ool00`00ooooo`04ool01000ooooo`000_oo00@0
0?ooool000?oo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000ooo00@00?ooool000;o
o`04003ooooo0003ool01000ooooo`000_oo00@00?ooool000?oo`04003ooooo0003ool01@00oooo
oooo00000_oo00D00?ooooooo`0000?oo`04003ooooo0003ool01@00oooooooo00000ooo00L00?oo
003oo`00ool0000=ool00`00ooooo`1Mool00`00ooooo`04ool003ooo`03003ooooo00Koo`03003o
o`0000?oo`04003ooooo0002ool00`00ool00003ool01000ooooo`000_oo00@00?ooool000;oo`05
003oooooool00002ool00`00ool00003ool01000ooooo`000ooo00<00?oo00000ooo00D00?oooooo
o`0000;oo`05003oooooool00002ool01@00oooooooo00000ooo00@00?ooool000?oo`05003ooooo
ool00003ool00`00ool00003000>ool00`00ooooo`1Mool00`00ooooo`04ool003ooo`03003ooooo
00?oo`04003ooooo0002ool01000ooooo`000_oo00L00?ooooooo`00ool00002ool01000ooooo`00
0_oo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`000_oo00D00?ooooooo`0000;o
o`04003ooooo0002ool01@00oooooooo00000_oo00D00?ooooooo`0000?oo`04003ooooo0003ool0
1000ooooo`000ooo00D00?ooooooo`0000?oo`@00003ool00?oo00koo`03003ooooo05coo`03003o
oooo00Coo`00?ooo00<00?ooool01?oo00H00?ooool00?oo0003ool01000ooooo`000_oo00<00?oo
00000ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?oo00000ooo00@0
0?ooool000?oo`03003oo`0000?oo`04003ooooo0002ool01@00oooooooo00000ooo00@00?ooool0
00?oo`04003ooooo0003ool01@00oooooooo00000ooo00<00?oo00000P004?oo00<00?ooool0G?oo
00<00?ooool01?oo000oool00`00ooooo`04ool01000ooooo`000_oo00@00?ooool000;oo`07003o
ooooool00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo00000_oo00@0
0?ooool000;oo`05003oooooool00002ool00`00ool00003ool01@00oooooooo00000_oo00@00?oo
ool000?oo`04003ooooo0003ool01@00oooooooo00000_oo00D00?ooooooo`0000;oo`03003ooooo
0080017oo`03003ooooo05coo`03003ooooo00Coo`00?ooo00<00?ooool01Ooo00H00?ooool00?oo
0003ool01000ooooo`000_oo00<00?oo00000ooo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`000ooo00<00?oo00000ooo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo0000
0_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01@00oooooooo00000_oo00D00?ooooooo`00
00;oo`04003oo`00000Bool00`00ooooo`1Mool00`00ooooo`03ool003ooo`03003ooooo00?oo`03
003oo`0000?oo`03003oo`0000;oo`04003ooooo0003ool00`00ool00002ool01000ooooo`000_oo
00@00?ooool000;oo`05003oooooool00002ool00`00ool00003ool01`00oooooooo003oo`0000;o
o`05003oooooool00002ool01000ooooo`000ooo00L00?ooooooo`00ool00003ool01@00oooooooo
00000_oo00@00?ooool000?oo`@001;oo`03003ooooo05goo`03003ooooo00?oo`00:ooo10000ooo
0P001?oo0P001Ooo00<00?ooool00ooo00@00?ooool000;oo`03003oo`0000?oo`04003ooooo0002
ool00`00ool00003ool01000ooooo`000_oo00H00?ooool00?oo0003ool01000ooooo`000_oo00@0
0?ooool000?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`05003oooooool0
0002ool01000ooooo`000ooo00@00?ooool000;oo`08003ooooo003oo`00ool001;oo`03003ooooo
05goo`03003ooooo00?oo`00:ooo00<00?ooool00ooo00@00?ooool000;oo`04003ooooo0004ool0
0`00ooooo`03ool01000ooooo`000ooo00<00?oo00000_oo00@00?ooool000?oo`03003oo`0000;o
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000ooo00@00?ooool000;oo`05
003oooooool00002ool01000ooooo`000_oo0`0000?oo`00ool04Ooo00<00?ooool0GOoo00<00?oo
ool00ooo000/ool00`00ooooo`02ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo00Co
o`04003ooooo0002ool00`00ool00003ool01000ooooo`000_oo00<00?oo00000ooo00H00?ooool0
0?oo0003ool00`00ool00003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool0
1000ooooo`000_oo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`000ooo00@00?oo
ool000;oo`03003ooooo008000;oo`03003ooooo013oo`03003ooooo05goo`03003ooooo00?oo`00
;Ooo00D00?ooooooo`0000;oo`04003ooooo0002ool00`00ooooo`02ool00`00ooooo`02ool00`00
ool00002ool01`00oooooooo003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?oo
ool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`07003oooooool00?oo
00000ooo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000_oo00@00?ooool000?oo`04
003ooooo0002ool00`00ooooo`020002ool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03
ool002koo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool00_oo00<00?ooool00_oo00@0
0?ooool000;oo`06003ooooo003oo`000ooo00@00?ooool000;oo`03003oo`0000;oo`07003ooooo
ool00?oo00000ooo00<00?oo00000ooo00<00?oo00000ooo00@00?ooool000;oo`04003ooooo0002
ool01000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01000ooooo`000_oo00@0
0?ooool000;oo`04003oo`000003ool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03ool0
02_oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000Coo`8000Coo`03003oo`0000;o
o`07003oooooool00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00H00?ooool00?oo0003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool01P00
ooooo`00ool000?oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?ooool000;oo`05003o
oooo003oo`020003ool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03ool002coo`8000Co
o`8000Coo`8000Goo`03003ooooo00?oo`04003ooooo0002ool01P00ooooo`00ool000?oo`04003o
oooo0002ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool0
1`00oooooooo003oo`0000;oo`04003ooooo0003ool01000ooooo`000_oo00@00?ooool000;oo`04
003ooooo0003ool01000ooooo`000_oo00@00?ooool000;oo`04003oo`000003ool00`00ooooo`0@
ool00`00ooooo`1Mool00`00ooooo`03ool003ooo`03003ooooo00Coo`03003oo`0000;oo`07003o
ooooool00?oo00000_oo00@00?ooool000;oo`06003ooooo003oo`000ooo00<00?oo00000ooo00<0
0?oo00000ooo00<00?oo00000ooo00@00?ooool000;oo`03003oo`0000?oo`04003ooooo0002ool0
1000ooooo`000_oo00@00?ooool000?oo`04003ooooo0002ool01000ooooo`000_oo00<00?oo0000
1?oo00<00?ooool04?oo00<00?ooool0GOoo00<00?ooool00ooo000oool01@00oooooooo00000_oo
00@00?ooool000;oo`06003ooooo003oo`000ooo00<00?oo00000ooo00<00?oo00000_oo00@00?oo
ool000;oo`06003ooooo003oo`000ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?ooool000;oo`04
003ooooo0002ool20005ool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03ool003ooo`05
003oooooool00003ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo
0002ool00`00ool00003ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000?oo`03003o
o`0000;oo`04003ooooo0003ool01P00ooooo`00ool000?oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`05003oo`00ool00005ool00`00ooooo`0@ool00`00ooooo`1Mool00`00
ooooo`03ool001Coo`03003ooooo02Soo`03003ooooo00;oo`04003ooooo0002ool01000ooooo`00
0_oo00<00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00H00?oo
ool00?oo0003ool01000ooooo`000_oo00H00?ooool00?oo0002ool01@00oooooooo00000_oo00@0
0?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`05003oo`00ool0
0005ool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03ool000;oo`D000;oo`@000Soo`04
003oooooool20002ool200000ooo00000002000Kool00`00ooooo`02ool01000ooooo`000_oo00@0
0?ooool000;oo`04003ooooo0002ool00`00ool00003ool00`00ool00002ool01`00oooooooo003o
o`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool0
1000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00P00?ooool0
0?oo003oo`001Ooo00<00?ooool04?oo00<00?ooool0GOoo00<00?ooool00ooo0003ool00`00ool0
0002ool01P00ooooo`00ool000Ooo`03003ooooo00;oo`04003ooooo0002ool01000ooooo`006ooo
00L00?ooooooo`00ool00002ool01000ooooo`000_oo00H00?ooool00?oo0002ool01000ooooo`00
0_oo00H00?ooool00?oo0003ool01P00ooooo`00ool000;oo`07003oooooool00?oo00000_oo00@0
0?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002
ool01`00oooooooo003oo`0000;oo`04003oo`000006ool00`00ooooo`0@ool00`00ooooo`1Mool0
0`00ooooo`03ool000?oo`03003oo`0000;oo`06003ooooo003oo`001ooo00<00?ooool00_oo00@0
0?ooool000?oo`03003ooooo01[oo`03003ooooo00;oo`03003oo`0000;oo`04003ooooo0002ool0
1P00ooooo`00ool000?oo`03003oo`0000?oo`03003oo`0000;oo`04003ooooo0002ool01P00oooo
o`00ool000?oo`04003ooooo0002ool01P00ooooo`00ool000;oo`05003oooooool00002ool01P00
ooooo`00ool000;oo`05003oooooool00002ool01P00ooooo`00ool000;oo`03003oo`0000Ooo`03
003ooooo013oo`03003ooooo05goo`03003ooooo00?oo`001?oo00D00?ooooooo`0000Coo`03003o
oooo00Koo`04003oooooool40004ool00`00ooooo`0Iool00`00ooooo`03ool00`00ool00002ool0
1000ooooo`000_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00H00?ooool00?oo0003ool0
0`00ool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`03003oo`0000?oo`04003ooooo0002ool01P00ooooo`00ool000;oo`03
003oo`0000Ooo`03003ooooo013oo`03003ooooo05goo`03003ooooo00?oo`001?oo00<00?ooool0
0_oo00D00?ooooooo`0000Soo`05003oooooool00002ool00`00ooooo`0400000ooo00000002000E
ool00`00ooooo`03ool00`00ool00002ool01000ooooo`000_oo00H00?ooool00?oo0002ool01000
ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool00`00ool00002ool01@00oooooooo0000
0_oo00<00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?oo
ool000;oo`04003ooooo0002ool00`00ool00002ool00`00ool00007ool00`00ooooo`0@ool00`00
ooooo`1Mool00`00ooooo`03ool000Coo`03003ooooo00;oo`05003oo`00003oo`040005ool01000
ool000000_oo0P001ooo0P005_oo00<00?ooool01?oo00<00?oo00000_oo00@00?ooool000;oo`03
003oo`0000;oo`06003ooooo003oo`000ooo00<00?oo00000_oo00@00?ooool000;oo`06003ooooo
003oo`000ooo00@00?ooool000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`04003ooooo
0002ool01000ooooo`000_oo00@00?ooool000;oo`03003oo`0000;oo`8000Soo`03003ooooo013o
o`03003ooooo05goo`03003ooooo00?oo`002Ooo0P0000Coo`00ooooo`8001Ooo`03003ooooo01?o
o`05003oooooool00002ool00`00ool00002ool01000ooooo`000_oo00H00?ooool00?oo0002ool0
1000ooooo`000_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool00`00ool00002ool0
1000ooooo`000_oo00@00?ooool000;oo`03003oo`0000Woo`03003ooooo013oo`03003ooooo05ko
o`03003ooooo00;oo`004Ooo00<00?ooool04_oo0`005Ooo00D00?ooooooo`0000;oo`06003ooooo
003oo`000ooo00H00?ooool00?oo0002ool00`00ool00002ool01`00oooooooo003oo`0000;oo`04
003ooooo0002ool01P00ooooo`00ool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?oo
ool000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000;oo`03
003oo`0000Woo`03003ooooo013oo`03003ooooo05koo`03003ooooo00;oo`003ooo0`00;?oo00<0
0?ooool00_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00H00?ooool00?oo0002ool00`00
ool00003ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00003ool01000ooooo`00
0_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`03003oo`0000?oo`03
003oo`0000;oo`06003ooooo003oo`002Ooo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool0
0_oo000nool00`00ooooo`02ool01000ooooo`000_oo00<00?oo00000ooo00<00?oo00000_oo00H0
0?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo
0002ool01000ooooo`000_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool000;o
o`04003ooooo0002ool00`00ool00002ool01P00ooooo`00ool000Woo`03003ooooo013oo`03003o
oooo05koo`03003ooooo00;oo`00?_oo00<00?ooool00ooo00H00?ooool00?oo0002ool01000oooo
o`000_oo00<00?oo00000_oo00H00?ooool00?oo0003ool00`00ool00002ool01000ooooo`000_oo
00H00?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01P00
ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool01P00ooooo`00ool000Woo`03003o
oooo013oo`03003ooooo05koo`03003ooooo00;oo`00?_oo00<00?ooool00ooo00H00?ooool00?oo
0002ool01000ooooo`000_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool000;o
o`06003ooooo003oo`000_oo00<00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool0
0`00ooooo`09ool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool003koo`04003ooooo
0002ool01000ooooo`000_oo00<00?oo00000ooo00<00?oo00000_oo00H00?ooool00?oo0002ool0
1`00oooooooo003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00@0
0?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00<00?oo00000_oo
00H00?ooool00?oo0002ool00`00ooooo`09ool00`00ooooo`0@ool00`00ooooo`1Nool00`00oooo
o`02ool003koo`04003ooooo0003ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool0
0002ool01P00ooooo`00ool000?oo`04003ooooo0002ool00`00ool00002ool01P00ooooo`00ool0
00;oo`04003ooooo0002ool01P00ooooo`00ool000;oo`04003ooooo0002ool01000ooooo`000_oo
00<00?oo00000ooo00<00?oo00000_oo00<00?oo00002ooo00<00?ooool04?oo00<00?ooool0G_oo
00<00?ooool00_oo000nool01@00oooooooo00000_oo00H00?ooool00?oo0002ool01000ooooo`00
0_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`06003ooooo003oo`00
0_oo00<00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00H00?oo
ool00?oo0002ool00`00ool00003ool00`00ool00002ool00`00ool0000;ool00`00ooooo`0@ool0
0`00ooooo`1Nool00`00ooooo`02ool003koo`05003oooooool00002ool01000ooooo`000_oo00<0
0?oo00000ooo00<00?oo00000_oo00H00?ooool00?oo0002ool01`00oooooooo003oo`0000;oo`04
003ooooo0002ool02000ooooo`00ool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`04
003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00<00?ooool02ooo
00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000nool01P00ooooo`00ool000;oo`06
003ooooo003oo`000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01000oooo
o`000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`04003ooooo0002ool01P00oooo
o`00ool000;oo`04003ooooo0002ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool0
000=ool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool003koo`06003ooooo003oo`00
0_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01P00ooooo`00ool000;oo`04
003ooooo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`06
003ooooo003oo`000_oo00@00?ooool000;oo`04003ooooo0002ool00`00ool00002ool01P00oooo
o`00ool000goo`03003ooooo013oo`03003ooooo05koo`03003ooooo00;oo`00?_oo00L00?oooooo
o`00ool00002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00@00?ooool000;o
o`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00P00?ooool00?oo003oo`000_oo
00@00?ooool000;oo`06003ooooo003oo`000ooo00<00?oo00000_oo00H00?ooool00?oo0002ool0
1P00ooooo`00ool000goo`03003ooooo013oo`03003ooooo05koo`03003ooooo00;oo`00?_oo00L0
0?ooooooo`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool0
1000ooooo`000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002
ool00`00ool00002ool01`00oooooooo003oo`0000;oo`06003ooooo003oo`000_oo00H00?ooool0
0?oo0003ool01@00ool00?oo00003Ooo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo
000nool01@00oooooooo00000_oo00<00?oo00000_oo00H00?ooool00?oo0003ool00`00ool00002
ool01P00ooooo`00ool000;oo`04003ooooo0002ool01P00ooooo`00ool000;oo`06003ooooo003o
o`000_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`06003ooooo003o
o`000_oo00H00?ooool00?oo000?ool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool0
02[oo`8000Coo`8000Coo`8000Koo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`00
0_oo00@00?ooool000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`03
003oo`0000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00@0
0?ooool000;oo`06003ooooo003oo`000_oo00H00?ooool00?oo000?ool00`00ooooo`0@ool00`00
ooooo`1Nool00`00ooooo`02ool002Woo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool0
00Coo`04003ooooo0002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`04003ooooo0002ool0
1P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00H00?oo
ool00?oo0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000ooo00<00?oo00000_oo
00<00?oo00000_oo00H00?ooool00?oo000?ool00`00ooooo`0@ool00`00ooooo`1Nool00`00oooo
o`02ool002coo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool00_oo00L00?ooooooo`00
ool00002ool01P00ooooo`00ool000;oo`03003oo`0000?oo`03003oo`0000;oo`06003ooooo003o
o`000_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00H00?ooool00?oo0002ool00`00ool0
0002ool02@00oooooooo003oo`00ool00003ool00`00ool00002ool00`00ool00003ool01@00ool0
0?oo00003ooo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000Zool20003ool01000
ooooo`000_oo00@00?ooool000Coo`05003oooooool00002ool00`00ool00002ool01P00ooooo`00
ool000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00H0
0?ooool00?oo0002ool01P00ooooo`00ool000;oo`04003ooooo0002ool01P00ooooo`00ool000;o
o`06003ooooo003oo`000_oo00@00?ooool000;oo`03003oo`00017oo`03003ooooo013oo`03003o
oooo05koo`03003ooooo00;oo`00;?oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`02
ool300001Ooo003ooooo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool01000ooooo`00
0_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`09003oooooool0
0?oo003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01P00ooooo`00ool000;oo`04003o
oooo0002ool00`00ool0000Aool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool002Wo
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000Coo`03003ooooo00;oo`03003oo`00
00;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool0
0`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool00`00ool0
0002ool01000ooooo`000_oo00<00?oo00000_oo00@00?ooool000;oo`03003oo`0000;oo`03003o
o`00017oo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00:_oo0P001?oo0P001?oo
0P001Ooo00<00?oo00000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`04003ooooo
0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool0
1P00ooooo`00ool000;oo`03003oo`0000;oo`09003oooooool00?oo003oo`0000?oo`03003oo`00
00;oo`03003oo`0000;oo`03003oo`00017oo`03003ooooo013oo`03003ooooo05ooo`03003ooooo
007oo`00?Ooo00<00?oo00000_oo00@00?ooool000;oo`03003oo`0000;oo`06003ooooo003oo`00
0_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00H00?ooool00?oo0002
ool01P00ooooo`00ool000;oo`04003ooooo0002ool01P00ooooo`00ool000;oo`06003ooooo003o
o`000_oo00<00?oo00000ooo00<00?ooool04?oo00<00?ooool04?oo00<00?ooool0Gooo00<00?oo
ool00Ooo000mool01000ooooo`000_oo00H00?ooool00?oo0002ool00`00ool00002ool01000oooo
o`000_oo00<00?oo00000_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00D00?oo003oo`00
00;oo`07003oooooool00?oo00000_oo00<00?oo00000_oo00@00?ooool000;oo`03003oo`0000;o
o`03003oo`0000;oo`04003ooooo0002ool00`00ooooo`0@ool00`00ooooo`0@ool00`00ooooo`1O
ool00`00ooooo`01ool003goo`04003ooooo0002ool01P00ooooo`00ool000;oo`06003ooooo003o
o`000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01P00
ooooo`00ool000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00@00?ooool000;oo`03003o
o`0000;oo`06003ooooo003oo`000_oo00@00?ooool001;oo`03003ooooo013oo`03003ooooo05oo
o`03003ooooo007oo`00?Ooo00D00?ooooooo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`06
003ooooo003oo`000_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00H0
0?ooool00?oo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000ooo00D00?oo003oo`00
00;oo`06003ooooo003oo`000_oo00<00?ooool04ooo00<00?ooool04?oo00<00?ooool0Gooo00<0
0?ooool00Ooo000mool01@00oooooooo00000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool0
00;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`03003oo`0000;oo`08003ooooo003oo`00
ool000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00H0
0?ooool00?oo0002ool00`00ool0000Eool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01
ool003goo`05003oo`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00@0
0?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool02000ooooo`00ool0
0?oo0003ool00`00ool00002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00<0
0?oo00000_oo00@00?ooool001Coo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00
?Ooo00<00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00ool000;o
o`04003ooooo0002ool00`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo
00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00<00?oo0000
0_oo00@00?ooool001Coo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00?Ooo00H0
0?ooool00?oo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00H00?ooool00?oo
0002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00H00?ooool0
0?oo0002ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool00`00ool0000G
ool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool003goo`06003ooooo003oo`000_oo
00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00@00?ooool0
00;oo`03003oo`0000;oo`08003ooooo003oo`00ool000;oo`04003ooooo0002ool00`00ool00002
ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00@00?ooool001Koo`03003ooooo013o
o`03003ooooo05ooo`03003ooooo007oo`00?Ooo00@00?ooool000;oo`04003ooooo0002ool01@00
ool00?oo00000_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00<00?oo00000_oo00H00?oo
ool00?oo0002ool02000ooooo`00ool00?oo0003ool01@00ool00?oo00000_oo00@00?ooool000;o
o`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?ooool05?oo00<00?ooool04?oo00<00?oo
ool0Gooo00<00?ooool00Ooo000mool20002ool00`00ool00002ool01P00ooooo`00ool000;oo`06
003ooooo003oo`000_oo00H00?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00ool000;o
o`06003ooooo003oo`000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`06003ooooo
003oo`000_oo00<00?oo00006Ooo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000m
ool20002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00H00?oo
ool00?oo0002ool01000ooooo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo
00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool0
0`00ooooo`0Fool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool003goo`05003oo`00
ool00002ool01000ooooo`000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`04003ooooo0002
ool00`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo0002ool01@00ool00?oo00000_oo
00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00<00?ooool0
5_oo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000lool01000ooooo`000_oo00<0
0?oo00000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo
00<00?oo00000_oo00H00?ooool00?oo0002ool02000ooooo`00ool00?oo0003ool01@00ool00?oo
00000ooo00D00?oo003oo`0000?oo`03003oo`0000;oo`03003ooooo01Soo`03003ooooo013oo`03
003ooooo05ooo`03003ooooo007oo`00??oo00L00?ooooooo`00ool00002ool00`00ool00002ool0
0`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`03
003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00H00?ooool0
0?oo0002ool01P00ooooo`00ool000;oo`03003ooooo01Soo`03003ooooo013oo`03003ooooo05oo
o`03003ooooo007oo`00??oo00L00?ooooooo`00ool00002ool01000ooooo`000_oo00<00?oo0000
0_oo00D00?oo003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01P00ooooo`00ool000;o
o`03003oo`0000?oo`05003oo`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`06
003ooooo003oo`000_oo00<00?ooool06?oo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool0
0Ooo000lool00`00ool00002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool01P00oooo
o`00ool000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00@00?ooool0
00;oo`05003oo`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00@00?oo
ool001coo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00??oo00<00?oo00000_oo00<0
0?oo00000_oo00@00?ooool000;oo`8000;oo`06003ooooo003oo`000_oo00H00?ooool00?oo0002
ool00`00ool00002ool01P00ooooo`00ool000;oo`08003ooooo003oo`00ool000?oo`05003oo`00
ool00003ool01@00ool00?oo00000_oo00@00?ooool001coo`03003ooooo013oo`03003ooooo063o
o`40007oo`7oo`00??oo00@00?ooool000;oo`8000;oo`04003ooooo0002ool00`00ool00002ool0
0`00ooooo`020002ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool00`00
ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00<00?oo
00000_oo00<00?ooool06_oo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool01000
ooooo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002
ool01P00ooooo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000?oo`05003o
o`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo000Nool00`00oooo
o`0@ool00`00ooooo`1Pool10001ool1ool003coo`04003ooooo0002ool00`00ool00002ool01000
ooooo`000_oo0P000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`03003oo`0000;o
o`03003oo`0000;oo`04003ooooo0002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool0
1P00ooooo`00ool000;oo`03003ooooo01coo`03003ooooo013oo`03003ooooo063oo`40007oo`7o
o`00??oo00D00?ooooooo`0000;oo`8000;oo`04003ooooo0002ool00`00ool00002ool00`00oooo
o`020002ool01P00ooooo`00ool000;oo`03003oo`0000?oo`03003oo`0000;oo`03003oo`0000;o
o`03003oo`0000;oo`08003ooooo003oo`00ool000;oo`06003ooooo003oo`000_oo00<00?ooool0
7?oo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool20002ool01P00ooooo`00ool0
00;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00H00?ooool00?oo0002
ool00`00ool00002ool00`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`03003o
o`0000;oo`06003ooooo003oo`000_oo00<00?ooool07_oo00<00?ooool04?oo00<00?ooool0H?oo
0@000Ooo0Ooo000lool20002ool01P00ooooo`00ool000;oo`04003ooooo0002ool20002ool01P00
ooooo`00ool000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo
00000ooo00D00?oo003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?ooool0
7_oo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool00`00ool00002ool01@00ool0
0?oo00000_oo00@00?ooool000;oo`03003oo`0000;oo`03003ooooo008000;oo`06003ooooo003o
o`000_oo00<00?oo00000ooo00D00?oo003oo`0000;oo`08003ooooo003oo`00ool000;oo`06003o
oooo003oo`000_oo00H00?ooool00?oo000Pool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1
ool002[oo`03003ooooo00;oo`8000Coo`8000Goo`03003oo`0000;oo`06003ooooo003oo`000_oo
00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool02000ooooo`00ool00?oo0002
ool01P00ooooo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`08003ooooo003oo`00ool0
00;oo`04003ooooo000Rool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool002[oo`05003o
ooooool00002ool01000ooooo`000_oo00<00?ooool00_oo00<00?oo00000_oo00H00?ooool00?oo
0002ool00`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`00
0_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool00`00
ool00002ool01P00ooooo`00ool002;oo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00
9ooo1@000_oo00@00?ooool000;oo`04003ooooo0004ool01000ooooo`000_oo00D00?oo003oo`00
00;oo`04003ooooo0002ool20002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00<0
0?oo00000_oo00<00?oo00000_oo00<00?oo00000ooo00D00?oo003oo`0000;oo`03003oo`0000;o
o`04003ooooo000Tool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool002Ooo`04003ooooo
0003ool01000ooooo`000_oo00@00?ooool000Coo`800005ool00?ooool00002ool01@00ool00?oo
00000_oo00H00?ooool00?oo0002ool00`00ooooo`020002ool01P00ooooo`00ool000;oo`04003o
oooo0002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool0
0002ool00`00ooooo`0Rool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool002Soo`03003o
o`0000?oo`04003ooooo0002ool01000ooooo`001?oo0P0000Goo`00ooooo`0000;oo`03003oo`00
00;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00P00?ooool00?oo003o
o`000_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool02000ooooo`00ool0
0?oo000Wool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool002Woo`8000?oo`04003ooooo
0002ool01000ooooo`001?oo00@00?oo000000;oo`06003ooooo003oo`000_oo00@00?ooool000;o
o`8000;oo`06003ooooo003oo`000_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool0
0002ool00`00ool00002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`03003ooooo02Coo`03
003ooooo013oo`03003ooooo063oo`40007oo`7oo`00:_oo00<00?ooool00_oo0P001?oo0P001Ooo
00D00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00D0
0?oo003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo
00000ooo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003ooooo02Coo`03003ooooo013oo`03
003ooooo063oo`40007oo`7oo`00>ooo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`03
003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo0000
0_oo00P00?ooool00?oo003oo`000_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00oooo
o`0Vool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool003_oo`04003oooooool20002ool0
1P00ooooo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00P0
0?ooool00?oo003oo`000_oo00H00?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00ool0
00;oo`03003oo`0002Soo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00>ooo00L00?oo
ooooo`00ool00002ool00`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`05003o
o`00ool00002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`00
00;oo`06003ooooo003oo`000_oo00<00?ooool0:?oo00<00?ooool04?oo00<00?ooool0HOoo0@00
0Ooo000kool20002ool00`00ool00002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`03003o
o`0000;oo`03003ooooo008000?oo`05003oo`00ool00002ool01P00ooooo`00ool000;oo`05003o
o`00ool00003ool00`00ool00002ool00`00ool0000Zool00`00ooooo`0@ool00`00ooooo`1Qool1
0001ool003_oo`8000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00<0
0?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool02000ooooo`00
ool00?oo0002ool01P00ooooo`00ool000;oo`03003ooooo02[oo`03003ooooo013oo`03003ooooo
067oo`40007oo`00>ooo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00D00?oo003o
o`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00P00?ooool00?oo003oo`000_oo00H0
0?ooool00?oo0002ool00`00ool00002ool01000ooooo`00;_oo00<00?ooool04?oo00<00?ooool0
HOoo0@000Ooo000kool00`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool00002
ool01P00ooooo`00ool000;oo`05003oo`00ool00002ool01P00ooooo`00ool000;oo`08003ooooo
003oo`00ool000;oo`03003oo`0000;oo`04003ooooo000^ool00`00ooooo`0@ool00`00ooooo`1Q
ool10001ool003_oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo
00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool01@00ool00?oo00000_oo00H00?ooool0
0?oo0002ool01@00ool00?oo00000ooo00<00?ooool0;_oo00<00?ooool04?oo00<00?ooool0HOoo
0@000Ooo000kool01000ooooo`000_oo00<00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`00
00;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00P00?ooool00?oo003oo`000_oo00P00?oo
ool00?oo003oo`000_oo00@00?ooool0033oo`03003ooooo013oo`03003ooooo067oo`40007oo`00
>ooo00@00?ooool000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo
003oo`000_oo00D00?oo003oo`0000;oo`08003ooooo003oo`00ool000;oo`06003ooooo003oo`00
0_oo00<00?oo0000<ooo00<00?ooool04?oo00<00?ooool0HOoo0@000Ooo000kool01000ooooo`00
0_oo00<00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool00`00
ooooo`020003ool01@00ool00?oo00000_oo00P00?ooool00?oo003oo`000_oo00<00?ooool0<ooo
00<00?ooool04?oo00<00?ooool0HOoo0@000Ooo000kool20002ool01P00ooooo`00ool000;oo`03
003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`03003o
o`0000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0003Goo`03003ooooo013oo`03003o
oooo067oo`40007oo`00>ooo0P000_oo00H00?ooool00?oo0002ool00`00ool00002ool01@00ool0
0?oo00000_oo00@00?ooool000;oo`05003oo`00ool00002ool02000ooooo`00ool00?oo0002ool0
1P00ooooo`00ool003Ooo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo00D00?oo003o
o`0000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo
00D00?oo003oo`0000?oo`07003oo`00ool00?oo00000ooo00<00?oo0000>Ooo00<00?ooool04?oo
00<00?ooool0HOoo0@000Ooo000kool01@00ool00?oo00000ooo0P000_oo00@00?ooool000;oo`05
003oo`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`05003oo`00ool00002ool0
1000ooooo`00>ooo00<00?ooool04?oo00<00?ooool0HOoo0@000Ooo000kool00`00ool00002ool0
1P00ooooo`00ool000;oo`03003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool01P00
ooooo`00ool000;oo`03003oo`0000;oo`03003oo`0003koo`03003ooooo013oo`03003ooooo067o
o`40007oo`00>ooo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool00`00ool0
0002ool01P00ooooo`00ool000;oo`05003oo`00ool00002ool01P00ooooo`00ool0043oo`03003o
oooo013oo`03003ooooo067oo`40007oo`00>ooo0P0000Coo`00ool000;oo`03003oo`0000;oo`04
003ooooo0002ool01@00ool00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000?oo`03003o
o`0004;oo`03003ooooo00_oo`T0063oo`40007oo`00>ooo0P000_oo00<00?oo00000_oo00<00?oo
00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`00
0_oo00<00?ooool0@_oo00<00?ooool02`002Ooo2000F?oo0@000Ooo000kool20002ool00`00ool0
0002ool00`00ool00002ool00`00ool00002ool01@00ool00?oo00000_oo00@00?ooool000;oo`05
003oo`00ool00010ool:000Lool8001@ool10001ool003[oo`04003ooooo0002ool20002ool00`00
ool00002ool00`00ool00003ool01@00ool00?oo00000_oo00<00?oo00000_oo00<00?ooool0=ooo
2`001ooo00<00?ooool09?oo2000B?oo0@000Ooo000jool01000ooooo`000_oo00<00?oo00000_oo
00<00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003ooooo037oo`/001;oo`03
003ooooo02coo`L004;oo`400000>_oo00<00?ooool01Ooo00H00?ooool00?oo0002ool00`00ool0
000cool:000Mool00`00ooooo`0cool8000jool1000003[oo`03003ooooo03Soo`/002Ooo`03003o
oooo03_oo`P003;oo`400000>_oo00<00?ooool0;Ooo2`00<_oo00<00?ooool0@ooo2000:_oo0@00
000jool00`00ooooo`0Sool:000mool00`00ooooo`1;ool8000Rool1000003[oo`03003ooooo01So
o`/004Ooo`03003ooooo05?oo`L001_oo`400000>_oo00<00?ooool03Ooo2`00D_oo00<00?ooool0
F_oo20004ooo0@00000jool00`00ooooo`03ool:001Mool00`00ooooo`1Rool8000;ool1000003[o
o`L006Koo`03003ooooo06[oo`P000?oo`400000@Ooo30002Ooo00<00?ooool0C_oo00<00?ooool0
K_oo2000001=ool<001>ool00`00ooooo`1Oool?0008ool005Woo``004;oo`03003ooooo053oo`l0
01Ooo`00IOoo30004?oo00<00?ooool08ooo00<00?ooool0@Ooo3`009_oo001aool<0004ool00`00
ooooo`0Sool00`00ooooo`0bool?000eool007goo``001koo`03003ooooo02?oo`l004Coo`00ROoo
30004_oo00<00?ooool05?oo3`00Dooo001Dool2000oool<0006ool00`00ooooo`05ool?001Rool0
05?oo`04003ooooo001:ool>001aool005?oo`04003ooooo0039ool005?oo`<00<[oo`00Dooo00<0
0?ooool0:_oo0P00W_oo001Cool00`00ooooo`0Yool01000ooooo`00WOoo001Dool3000[ool00`00
ooooo`2Kool0083oo`8002Coo`8007Soo`00P_oo00<00?ooool08?oo00@00?ooool007Ooo`00Oooo
00@00?ooool002;oo`04003ooooo001gool0083oo`8002?oo`04003ooooo001gool00:Goo`04003o
oooo001gool00:Goo`04003ooooo001gool00:Koo`8007Soo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo001cool00`00ooooo`2Zool006goo`<000Coo`03003ooooo00;oo`<00:Coo`00K_oo
00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0XOoo001^ool00`00ooooo`03ool00`00oooo
o`03ool2002Tool006koo`03003ooooo00Coo`05003oooooool0002Vool006coo`05003oo`00ool0
0004ool00`00ooooo`02ool3002Sool006coo`D000Goo`03003ooooo0:Ooo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {656.188, 446.438}} -> {-0.886652, \
2.2021, 0.00409734, 0.00409734}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot with ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Beta]\^'\) = \(2/6 = 1/3\)\)]]
}], "Subsubsection"],

Cell[BoxData[
    \(plotsB3 = createPlotsB[2. /6]; \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(showPlotsB[plotsB3]; \)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .91501 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0262702 1.06936 -2.13046e-17 1.06936 [
[.4752 .92837 -3.00977 0 ]
[.4752 .92837 2.99023 9 ]
[.31156 .91504 -3.05802 0 ]
[.31156 .91504 2.94198 9 ]
[.11966 .89942 -3.11475 0 ]
[.11966 .89942 2.88525 9 ]
[.26027 .97781 -11.2665 0 ]
[.26027 .97781 10.7335 9.8125 ]
[.03465 .18507 -8.54876 -9 ]
[.03465 .18507 9.45124 0 ]
[.18551 .12753 -8.69057 -9 ]
[.18551 .12753 9.30943 0 ]
[.35735 .06198 -8.85423 -9 ]
[.35735 .06198 9.14577 0 ]
[.55418 -0.0131 -6.02969 -9 ]
[.55418 -0.0131 5.97031 0 ]
[.27026 .02904 -13.6412 -9.8125 ]
[.27026 .02904 14.3588 0 ]
[.01476 .32278 -18 -5.85643 ]
[.01476 .32278 0 3.14357 ]
[.00798 .45516 -18 -5.61724 ]
[.00798 .45516 0 3.38276 ]
[.00104 .59125 -18 -5.3727 ]
[.00104 .59125 0 3.6273 ]
[-0.00606 .7312 -18 -5.12262 ]
[-0.00606 .7312 0 3.87738 ]
[-0.06073 .5083 -44.625 -7.93817 ]
[-0.06073 .5083 0 5.06183 ]
[ 0 0 0 0 ]
[ 1 .91501 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.4753 .91501 m
0 .87626 L
s
.47525 .915 m
.47527 .90832 L
s
[(0)] .4752 .92837 .00326 -1 Mshowa
.31182 .90168 m
.31195 .895 L
s
[(3)] .31156 .91504 .01934 -1 Mshowa
.12017 .88606 m
.12043 .87938 L
s
[(6)] .11966 .89942 .03825 -1 Mshowa
gsave
.26027 .97781 -72.2665 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(T) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.03399 .19842 m
.55496 0 L
s
.03399 .19842 m
.03365 .2051 L
s
[(0.1)] .03465 .18507 -0.05014 1 Mshowa
.18505 .14089 m
.18482 .14756 L
s
[(0.4)] .18551 .12753 -0.03438 1 Mshowa
.35714 .07534 m
.35703 .08203 L
s
[(0.7)] .35735 .06198 -0.0162 1 Mshowa
.55425 .00027 m
.55428 .00695 L
s
[(1.)] .55418 -0.0131 .00495 1 Mshowa
gsave
.27026 .02904 -74.6412 -13.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(t) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
75.000 11.250 moveto
(\\220) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
87.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.03399 .19842 m
0 .87626 L
s
.02756 .32664 m
.03396 .32857 L
s
[(100)] .01476 .32278 1 .30143 Mshowa
.02095 .45839 m
.02744 .46 L
s
[(200)] .00798 .45516 1 .24828 Mshowa
.01416 .5938 m
.02072 .59507 L
s
[(300)] .00104 .59125 1 .19393 Mshowa
.00718 .73304 m
.0138 .73395 L
s
[(400)] -0.00606 .7312 1 .13836 Mshowa
gsave
-0.06073 .5083 -105.625 -11.9382 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.47712 .35504 m
.4753 .91501 L
s
.4753 .91501 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
.96816 .22579 m
.47712 .35504 L
s
.03399 .19842 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.47712 .35504 m
.4753 .91501 L
s
.4753 .91501 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.03399 .19842 m
.47712 .35504 L
s
.96816 .22579 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
0 0 m
1 0 L
1 .91501 L
0 .91501 L
closepath
clip
newpath
.5 Mabswid
.46956 .44793 m
.47439 .44404 L
s
.47439 .44404 m
.47925 .4401 L
s
.47925 .4401 m
.48412 .43612 L
s
.48412 .43612 m
.48902 .4321 L
s
.46203 .49475 m
.466 .4873 L
s
.48902 .4321 m
.49394 .42805 L
s
.466 .4873 m
.47 .47973 L
s
.49394 .42805 m
.49888 .42398 L
s
.47 .47973 m
.47401 .47208 L
s
.49888 .42398 m
.50384 .41989 L
s
.47401 .47208 m
.47804 .46438 L
s
.50384 .41989 m
.50883 .4158 L
s
.45439 .53767 m
.4575 .52577 L
s
.47804 .46438 m
.48209 .45666 L
s
.50883 .4158 m
.51384 .4117 L
s
.4575 .52577 m
.46062 .5137 L
s
.48209 .45666 m
.48616 .44896 L
s
.51384 .4117 m
.51887 .40761 L
s
.46062 .5137 m
.46375 .50157 L
s
.51887 .40761 m
.52393 .40354 L
s
.48616 .44896 m
.49025 .4413 L
s
.46375 .50157 m
.4669 .48947 L
s
.44666 .57692 m
.44888 .55981 L
s
.52393 .40354 m
.52901 .39948 L
s
.49025 .4413 m
.49436 .43373 L
s
.4669 .48947 m
.47006 .47746 L
s
.44888 .55981 m
.45111 .54259 L
s
.52901 .39948 m
.53411 .39545 L
s
.49436 .43373 m
.49849 .42625 L
s
.47006 .47746 m
.47324 .46562 L
s
.45111 .54259 m
.45335 .52542 L
s
.53411 .39545 m
.53924 .39144 L
s
.49849 .42625 m
.50264 .4189 L
s
.47324 .46562 m
.47643 .45402 L
s
.45335 .52542 m
.45561 .50847 L
s
.43883 .61272 m
.44016 .5898 L
s
.53924 .39144 m
.54439 .38747 L
s
.50264 .4189 m
.50681 .4117 L
s
.47643 .45402 m
.47964 .44272 L
s
.45561 .50847 m
.45787 .49187 L
s
.44016 .5898 m
.44149 .56692 L
s
.54439 .38747 m
.54957 .38354 L
s
.50681 .4117 m
.511 .40466 L
s
.47964 .44272 m
.48286 .43176 L
s
.45787 .49187 m
.46013 .47575 L
s
.44149 .56692 m
.44283 .54437 L
s
.54957 .38354 m
.55477 .37964 L
s
.511 .40466 m
.51521 .39779 L
s
.44283 .54437 m
.44417 .52236 L
s
.48286 .43176 m
.4861 .42117 L
s
.46013 .47575 m
.46241 .4602 L
s
.43091 .64527 m
.43134 .61606 L
s
.55477 .37964 m
.56 .37579 L
s
.51521 .39779 m
.51944 .39111 L
s
.44417 .52236 m
.44551 .50112 L
s
.43134 .61606 m
.43177 .58719 L
s
.4861 .42117 m
.48936 .41099 L
s
.46241 .4602 m
.46469 .4453 L
s
.56 .37579 m
.56525 .37199 L
s
.51944 .39111 m
.5237 .38462 L
s
.43177 .58719 m
.43219 .55905 L
s
.44551 .50112 m
.44685 .4808 L
s
.56525 .37199 m
.57053 .36824 L
s
.48936 .41099 m
.49263 .40124 L
s
.46469 .4453 m
.46699 .43111 L
s
.5237 .38462 m
.52797 .37833 L
s
.43219 .55905 m
.43261 .53197 L
s
.42291 .67478 m
.42243 .63891 L
s
.44685 .4808 m
.44819 .46152 L
s
.57053 .36824 m
.57583 .36454 L
s
.49263 .40124 m
.49592 .39191 L
s
.46699 .43111 m
.46929 .41766 L
s
.42243 .63891 m
.42194 .60383 L
s
.52797 .37833 m
.53227 .37225 L
s
.43261 .53197 m
.43301 .5062 L
s
.57583 .36454 m
.58116 .36089 L
s
.44819 .46152 m
.44954 .44337 L
s
.49592 .39191 m
.49922 .38303 L
s
.46929 .41766 m
.4716 .40499 L
s
.42194 .60383 m
.42144 .57006 L
s
.43301 .5062 m
.43341 .48193 L
s
.53227 .37225 m
.5366 .36636 L
s
.58116 .36089 m
.58652 .3573 L
s
.44954 .44337 m
.45088 .42641 L
s
.42144 .57006 m
.42092 .538 L
s
.41483 .70142 m
.41344 .65863 L
s
.49922 .38303 m
.50255 .37458 L
s
.4716 .40499 m
.47393 .39309 L
s
.43341 .48193 m
.43379 .4593 L
s
.5366 .36636 m
.54095 .36067 L
s
.58652 .3573 m
.5919 .35376 L
s
.41344 .65863 m
.41202 .61725 L
s
.42092 .538 m
.42037 .50795 L
s
.45088 .42641 m
.45222 .41065 L
s
.50255 .37458 m
.50589 .36655 L
s
.47393 .39309 m
.47626 .38195 L
s
.54095 .36067 m
.54532 .35518 L
s
.43379 .4593 m
.43417 .43837 L
s
.41202 .61725 m
.41058 .57792 L
s
.5919 .35376 m
.59731 .35027 L
s
.42037 .50795 m
.41981 .4801 L
s
.45222 .41065 m
.45356 .39608 L
s
.41058 .57792 m
.40911 .54108 L
s
.50589 .36655 m
.50926 .35894 L
s
.59731 .35027 m
.60275 .34683 L
s
.40668 .7254 m
.40436 .67551 L
s
.54532 .35518 m
.54972 .34987 L
s
.47626 .38195 m
.47861 .37156 L
s
.43417 .43837 m
.43453 .41918 L
s
.41981 .4801 m
.41921 .45456 L
s
.40436 .67551 m
.40201 .62782 L
s
.40911 .54108 m
.4076 .50707 L
s
.45356 .39608 m
.4549 .38269 L
s
.60275 .34683 m
.60822 .34345 L
s
.50926 .35894 m
.51264 .35171 L
s
.54972 .34987 m
.55414 .34474 L
s
.40201 .62782 m
.39962 .58306 L
s
.47861 .37156 m
.48097 .36187 L
s
.43453 .41918 m
.43488 .40168 L
s
.41921 .45456 m
.41859 .43137 L
s
.4076 .50707 m
.40604 .47605 L
s
.60822 .34345 m
.61371 .34011 L
s
.39962 .58306 m
.39718 .54173 L
s
.55414 .34474 m
.55859 .33979 L
s
.51264 .35171 m
.51605 .34486 L
s
.4549 .38269 m
.45625 .37041 L
s
.39845 .74688 m
.39521 .68979 L
s
.48097 .36187 m
.48334 .35287 L
s
.43488 .40168 m
.43522 .38583 L
s
.39521 .68979 m
.39191 .63586 L
s
.61371 .34011 m
.61923 .33683 L
s
.41859 .43137 m
.41794 .41047 L
s
.40604 .47605 m
.40444 .44809 L
s
.39718 .54173 m
.39468 .50414 L
s
.55859 .33979 m
.56307 .33499 L
s
.39191 .63586 m
.38856 .58589 L
s
.51605 .34486 m
.51948 .33835 L
s
.45625 .37041 m
.4576 .35918 L
s
.61923 .33683 m
.62478 .33359 L
s
.48334 .35287 m
.48573 .34449 L
s
.39468 .50414 m
.39212 .4704 L
s
.43522 .38583 m
.43554 .37152 L
s
.40444 .44809 m
.4028 .42312 L
s
.38856 .58589 m
.38513 .5404 L
s
.41794 .41047 m
.41727 .39178 L
s
.56307 .33499 m
.56757 .33036 L
s
.39015 .76603 m
.38597 .70171 L
s
.51948 .33835 m
.52293 .33217 L
s
.62478 .33359 m
.63037 .33039 L
s
.38597 .70171 m
.38173 .64167 L
s
.4576 .35918 m
.45895 .34892 L
s
.38513 .5404 m
.38162 .49963 L
s
.48573 .34449 m
.48814 .3367 L
s
.39212 .4704 m
.38949 .44047 L
s
.56757 .33036 m
.5721 .32586 L
s
.38173 .64167 m
.3774 .58677 L
s
.4028 .42312 m
.40111 .40102 L
s
.43554 .37152 m
.43586 .35865 L
s
.41727 .39178 m
.41656 .37514 L
s
.63037 .33039 m
.63598 .32723 L
s
.52293 .33217 m
.5264 .32628 L
s
.3774 .58677 m
.37297 .53747 L
s
.38162 .49963 m
.37803 .4636 L
s
.45895 .34892 m
.46031 .33955 L
s
.5721 .32586 m
.57666 .32149 L
s
.48814 .3367 m
.49056 .32943 L
s
.38179 .783 m
.37667 .71149 L
s
.38949 .44047 m
.38679 .41418 L
s
.63598 .32723 m
.64162 .32412 L
s
.37667 .71149 m
.37146 .64553 L
s
.5264 .32628 m
.5299 .32066 L
s
.40111 .40102 m
.39937 .38157 L
s
.43586 .35865 m
.43617 .34708 L
s
.41656 .37514 m
.41584 .36037 L
s
.37297 .53747 m
.36843 .49393 L
s
.37146 .64553 m
.36613 .58599 L
s
.57666 .32149 m
.58125 .31725 L
s
.37803 .4636 m
.37434 .43213 L
s
.64162 .32412 m
.64729 .32103 L
s
.46031 .33955 m
.46168 .33097 L
s
.49056 .32943 m
.49299 .32264 L
s
.38679 .41418 m
.38402 .39127 L
s
.36613 .58599 m
.36068 .53326 L
s
.5299 .32066 m
.53342 .31529 L
s
.36843 .49393 m
.36377 .45602 L
s
.37336 .79796 m
.36729 .71933 L
s
.64729 .32103 m
.65299 .31798 L
s
.43617 .34708 m
.43648 .33667 L
s
.39937 .38157 m
.39759 .36452 L
s
.58125 .31725 m
.58587 .31312 L
s
.36729 .71933 m
.3611 .64768 L
s
.41584 .36037 m
.41509 .34728 L
s
.37434 .43213 m
.37055 .40491 L
s
.49299 .32264 m
.49545 .31627 L
s
.36068 .53326 m
.35509 .48734 L
s
.46168 .33097 m
.46306 .3231 L
s
.3611 .64768 m
.35477 .58382 L
s
.53342 .31529 m
.53697 .31014 L
s
.65299 .31798 m
.65872 .31496 L
s
.58587 .31312 m
.59051 .30909 L
s
.38402 .39127 m
.38119 .3714 L
s
.36377 .45602 m
.35899 .42339 L
s
.35477 .58382 m
.34827 .52803 L
s
.43648 .33667 m
.43678 .32726 L
s
.39759 .36452 m
.39577 .34957 L
s
.36487 .81103 m
.35784 .72542 L
s
.65872 .31496 m
.66448 .31197 L
s
.41509 .34728 m
.41431 .33564 L
s
.35509 .48734 m
.34934 .44793 L
s
.49545 .31627 m
.49792 .31026 L
s
.35784 .72542 m
.35066 .64834 L
s
.53697 .31014 m
.54054 .30518 L
s
.37055 .40491 m
.36668 .38151 L
s
.46306 .3231 m
.46444 .31583 L
s
.59051 .30909 m
.59518 .30515 L
s
.35066 .64834 m
.3433 .5805 L
s
.34827 .52803 m
.3416 .48011 L
s
.66448 .31197 m
.67027 .309 L
s
.35899 .42339 m
.35408 .39555 L
s
.38119 .3714 m
.37829 .35417 L
s
.43678 .32726 m
.43707 .31871 L
s
.59518 .30515 m
.59989 .3013 L
s
.49792 .31026 m
.50041 .30458 L
s
.54054 .30518 m
.54414 .30039 L
s
.3433 .5805 m
.33574 .52202 L
s
.39577 .34957 m
.39391 .33644 L
s
.34934 .44793 m
.34344 .41446 L
s
.41431 .33564 m
.41352 .32523 L
s
.35632 .82235 m
.34832 .72994 L
s
.46444 .31583 m
.46583 .30908 L
s
.67027 .309 m
.6761 .30605 L
s
.34832 .72994 m
.34014 .64769 L
s
.36668 .38151 m
.36271 .36143 L
s
.3416 .48011 m
.33473 .43953 L
s
.34014 .64769 m
.33174 .57623 L
s
.59989 .3013 m
.60462 .29752 L
s
.54414 .30039 m
.54776 .29575 L
s
.33574 .52202 m
.32796 .47245 L
s
.6761 .30605 m
.68195 .30312 L
s
.35408 .39555 m
.34906 .37187 L
s
.50041 .30458 m
.50292 .29915 L
s
.37829 .35417 m
.37533 .33919 L
s
.43707 .31871 m
.43736 .31088 L
s
.33174 .57623 m
.32308 .5154 L
s
.46583 .30908 m
.46723 .30275 L
s
.34344 .41446 m
.33737 .38623 L
s
.39391 .33644 m
.392 .3248 L
s
.41352 .32523 m
.41271 .31585 L
s
.34771 .83206 m
.33873 .73303 L
s
.33873 .73303 m
.32953 .64593 L
s
.60462 .29752 m
.60938 .2938 L
s
.68195 .30312 m
.68784 .3002 L
s
.33473 .43953 m
.32767 .40549 L
s
.54776 .29575 m
.55141 .29124 L
s
.36271 .36143 m
.35864 .34415 L
s
.32953 .64593 m
.32006 .57117 L
s
.32796 .47245 m
.31994 .43099 L
s
.50292 .29915 m
.50546 .29395 L
s
.43736 .31088 m
.43765 .30363 L
s
.32308 .5154 m
.31416 .46449 L
s
.68784 .3002 m
.69376 .29729 L
s
.60938 .2938 m
.61418 .29015 L
s
.34906 .37187 m
.34391 .35171 L
s
.37533 .33919 m
.3723 .32604 L
s
.46723 .30275 m
.46864 .29677 L
s
.32006 .57117 m
.31029 .50834 L
s
.55141 .29124 m
.55509 .28685 L
s
.41271 .31585 m
.41188 .30729 L
s
.33904 .84027 m
.32907 .73484 L
s
.32907 .73484 m
.31884 .6432 L
s
.392 .3248 m
.39006 .31438 L
s
.33737 .38623 m
.33115 .36243 L
s
.50546 .29395 m
.50801 .28893 L
s
.69376 .29729 m
.69971 .29439 L
s
.32767 .40549 m
.32041 .37704 L
s
.61418 .29015 m
.619 .28654 L
s
.31884 .6432 m
.30828 .56549 L
s
.35864 .34415 m
.35449 .32914 L
s
.31994 .43099 m
.31168 .39656 L
s
.43765 .30363 m
.43794 .29684 L
s
.31416 .46449 m
.30496 .42239 L
s
.46864 .29677 m
.47007 .29106 L
s
.55509 .28685 m
.55879 .28255 L
s
.31029 .50834 m
.3002 .45636 L
s
.69971 .29439 m
.7057 .2915 L
s
.3723 .32604 m
.36922 .31435 L
s
.33031 .8471 m
.31934 .73552 L
s
.30828 .56549 m
.29737 .50094 L
s
.34391 .35171 m
.33864 .3344 L
s
.31934 .73552 m
.30806 .63964 L
s
.619 .28654 m
.62386 .28298 L
s
.41188 .30729 m
.41103 .29937 L
s
.50801 .28893 m
.51058 .28405 L
s
.39006 .31438 m
.38808 .30491 L
s
.30806 .63964 m
.29639 .5593 L
s
.33115 .36243 m
.32479 .34223 L
s
.7057 .2915 m
.71171 .28862 L
s
.55879 .28255 m
.56252 .27833 L
s
.32041 .37704 m
.31296 .35318 L
s
.43794 .29684 m
.43823 .29039 L
s
.47007 .29106 m
.4715 .28556 L
s
.62386 .28298 m
.62874 .27946 L
s
.35449 .32914 m
.35026 .31593 L
s
.31168 .39656 m
.30319 .368 L
s
.30496 .42239 m
.29547 .38774 L
s
.3002 .45636 m
.28978 .41381 L
s
.29737 .50094 m
.28608 .44815 L
s
.51058 .28405 m
.51317 .27927 L
s
.32152 .85265 m
.30955 .73517 L
s
.30955 .73517 m
.29719 .63537 L
s
.71171 .28862 m
.71776 .28573 L
s
.29639 .5593 m
.28431 .49333 L
s
.41103 .29937 m
.41017 .29193 L
s
.36922 .31435 m
.36608 .30379 L
s
.56252 .27833 m
.56628 .27419 L
s
.33864 .3344 m
.33326 .31933 L
s
.29719 .63537 m
.28439 .55271 L
s
.62874 .27946 m
.63366 .27597 L
s
.38808 .30491 m
.38607 .29616 L
s
.71776 .28573 m
.72385 .28285 L
s
.4715 .28556 m
.47295 .28021 L
s
.43823 .29039 m
.43852 .2842 L
s
.32479 .34223 m
.31827 .32487 L
s
.51317 .27927 m
.51578 .27457 L
s
.31268 .85702 m
.29968 .73392 L
s
.29968 .73392 m
.28623 .63051 L
s
.56628 .27419 m
.57006 .27009 L
s
.63366 .27597 m
.63861 .27252 L
s
.31296 .35318 m
.30533 .33294 L
s
.35026 .31593 m
.34595 .30408 L
s
.28431 .49333 m
.27178 .43991 L
s
.28439 .55271 m
.2711 .48557 L
s
.72385 .28285 m
.72997 .27997 L
s
.28608 .44815 m
.2744 .4053 L
s
.30319 .368 m
.29447 .3441 L
s
.28978 .41381 m
.27903 .37903 L
s
.29547 .38774 m
.28571 .35912 L
s
.28623 .63051 m
.27227 .54581 L
s
.41017 .29193 m
.40929 .28484 L
s
.36608 .30379 m
.36289 .29407 L
s
.47295 .28021 m
.47441 .27496 L
s
.38607 .29616 m
.38403 .28794 L
s
.51578 .27457 m
.51842 .26992 L
s
.72997 .27997 m
.73612 .27708 L
s
.63861 .27252 m
.6436 .26908 L
s
.33326 .31933 m
.32777 .30596 L
s
.43852 .2842 m
.43881 .27819 L
s
.57006 .27009 m
.57388 .26605 L
s
.30378 .8603 m
.28974 .73186 L
s
.28974 .73186 m
.27518 .62515 L
s
.27227 .54581 m
.25776 .47772 L
s
.2711 .48557 m
.25732 .4317 L
s
.31827 .32487 m
.31161 .30965 L
s
.73612 .27708 m
.7423 .2742 L
s
.27518 .62515 m
.26004 .53868 L
s
.27178 .43991 m
.25882 .39687 L
s
.6436 .26908 m
.64862 .26567 L
s
.34595 .30408 m
.34156 .29325 L
s
.40929 .28484 m
.40841 .278 L
s
.47441 .27496 m
.47588 .26978 L
s
.51842 .26992 m
.52108 .2653 L
s
.30533 .33294 m
.29752 .31546 L
s
.2744 .4053 m
.26234 .37045 L
s
.57388 .26605 m
.57772 .26205 L
s
.27903 .37903 m
.26795 .35038 L
s
.29482 .86259 m
.27973 .72909 L
s
.27973 .72909 m
.26404 .61937 L
s
.29447 .3441 m
.28554 .32379 L
s
.28571 .35912 m
.27568 .33517 L
s
.43881 .27819 m
.4391 .2723 L
s
.36289 .29407 m
.35964 .28496 L
s
.38403 .28794 m
.38196 .28009 L
s
.7423 .2742 m
.74852 .27131 L
s
.64862 .26567 m
.65367 .26227 L
s
.32777 .30596 m
.32218 .29384 L
s
.26004 .53868 m
.24426 .46984 L
s
.26404 .61937 m
.24768 .53139 L
s
.57772 .26205 m
.58159 .25807 L
s
.25776 .47772 m
.24267 .42353 L
s
.52108 .2653 m
.52376 .2607 L
s
.74852 .27131 m
.75478 .26842 L
s
.47588 .26978 m
.47737 .26462 L
s
.28581 .86396 m
.26964 .72569 L
s
.25732 .4317 m
.24302 .38855 L
s
.26964 .72569 m
.2528 .61325 L
s
.40841 .278 m
.40751 .27132 L
s
.65367 .26227 m
.65875 .25888 L
s
.31161 .30965 m
.30482 .29602 L
s
.34156 .29325 m
.3371 .28314 L
s
.4391 .2723 m
.43939 .26648 L
s
.25882 .39687 m
.24542 .36199 L
s
.75478 .26842 m
.76107 .26552 L
s
.38196 .28009 m
.37986 .27249 L
s
.58159 .25807 m
.58549 .25413 L
s
.29752 .31546 m
.28954 .30002 L
s
.35964 .28496 m
.35635 .27627 L
s
.26234 .37045 m
.24992 .34175 L
s
.2528 .61325 m
.2352 .52398 L
s
.24768 .53139 m
.23061 .46196 L
s
.52376 .2607 m
.52646 .25611 L
s
.28554 .32379 m
.27639 .30612 L
s
.26795 .35038 m
.25657 .32635 L
s
.65875 .25888 m
.66387 .2555 L
s
.27568 .33517 m
.26539 .31472 L
s
.27674 .8645 m
.25949 .72174 L
s
.47737 .26462 m
.47887 .25948 L
s
.24426 .46984 m
.22784 .41544 L
s
.25949 .72174 m
.24147 .60685 L
s
.76107 .26552 m
.7674 .26263 L
s
.32218 .29384 m
.31649 .28262 L
s
.40751 .27132 m
.40659 .26473 L
s
.24267 .42353 m
.22701 .38033 L
s
.58549 .25413 m
.58942 .2502 L
s
.43939 .26648 m
.43969 .26068 L
s
.66387 .2555 m
.66902 .25213 L
s
.7674 .26263 m
.77376 .25973 L
s
.52646 .25611 m
.52919 .25151 L
s
.24147 .60685 m
.22259 .51649 L
s
.3371 .28314 m
.33256 .27355 L
s
.30482 .29602 m
.2979 .28353 L
s
.24302 .38855 m
.22824 .35363 L
s
.2352 .52398 m
.2168 .45411 L
s
.26761 .86427 m
.24925 .71731 L
s
.37986 .27249 m
.37773 .26507 L
s
.24925 .71731 m
.23003 .60022 L
s
.47887 .25948 m
.48038 .25432 L
s
.35635 .27627 m
.35301 .26788 L
s
.24542 .36199 m
.2316 .33322 L
s
.23061 .46196 m
.21282 .40743 L
s
.58942 .2502 m
.59338 .24629 L
s
.77376 .25973 m
.78016 .25683 L
s
.28954 .30002 m
.28139 .28605 L
s
.66902 .25213 m
.6742 .24876 L
s
.24992 .34175 m
.23713 .3176 L
s
.40659 .26473 m
.40567 .2582 L
s
.43969 .26068 m
.43998 .2549 L
s
.27639 .30612 m
.26703 .29038 L
s
.52919 .25151 m
.53194 .2469 L
s
.23003 .60022 m
.20986 .50897 L
s
.25657 .32635 m
.24488 .3057 L
s
.25843 .86334 m
.23894 .71247 L
s
.31649 .28262 m
.3107 .27204 L
s
.22784 .41544 m
.21078 .3722 L
s
.26539 .31472 m
.25485 .2968 L
s
.23894 .71247 m
.21849 .59342 L
s
.78016 .25683 m
.78659 .25393 L
s
.22259 .51649 m
.20284 .44631 L
s
.48038 .25432 m
.48191 .24915 L
s
.59338 .24629 m
.59738 .2424 L
s
.6742 .24876 m
.67942 .24539 L
s
.37773 .26507 m
.37558 .25774 L
s
.33256 .27355 m
.32795 .26432 L
s
.22701 .38033 m
.21081 .34537 L
s
.35301 .26788 m
.34962 .25969 L
s
.2168 .45411 m
.19761 .3995 L
s
.2979 .28353 m
.29085 .27186 L
s
.78659 .25393 m
.79306 .25103 L
s
.53194 .2469 m
.53471 .24229 L
s
.24919 .86178 m
.22855 .70725 L
s
.21849 .59342 m
.19699 .50144 L
s
.43998 .2549 m
.44028 .2491 L
s
.40567 .2582 m
.40473 .2517 L
s
.22855 .70725 m
.20685 .58649 L
s
.67942 .24539 m
.68468 .24201 L
s
.22824 .35363 m
.21298 .32476 L
s
.59738 .2424 m
.6014 .23851 L
s
.48191 .24915 m
.48346 .24395 L
s
.20986 .50897 m
.18871 .43856 L
s
.28139 .28605 m
.27308 .27313 L
s
.21282 .40743 m
.19432 .36417 L
s
.79306 .25103 m
.79957 .24813 L
s
.2316 .33322 m
.21737 .30889 L
s
.3107 .27204 m
.30481 .26193 L
s
.26703 .29038 m
.25747 .27601 L
s
.23988 .85964 m
.21809 .70173 L
s
.53471 .24229 m
.5375 .23767 L
s
.68468 .24201 m
.68997 .23864 L
s
.23713 .3176 m
.22401 .2967 L
s
.37558 .25774 m
.37339 .25049 L
s
.25485 .2968 m
.24407 .2807 L
s
.24488 .3057 m
.23291 .28747 L
s
.20284 .44631 m
.1822 .39165 L
s
.21809 .70173 m
.1951 .57946 L
s
.20685 .58649 m
.18398 .49392 L
s
.6014 .23851 m
.60545 .23463 L
s
.21078 .3722 m
.19311 .33718 L
s
.79957 .24813 m
.80612 .24524 L
s
.32795 .26432 m
.32327 .25536 L
s
.44028 .2491 m
.44059 .24329 L
s
.34962 .25969 m
.34617 .25164 L
s
.40473 .2517 m
.40379 .2452 L
s
.48346 .24395 m
.48501 .23872 L
s
.19699 .50144 m
.17441 .43088 L
s
.29085 .27186 m
.28366 .26078 L
s
.68997 .23864 m
.6953 .23525 L
s
.23052 .85697 m
.20753 .69595 L
s
.21081 .34537 m
.19407 .31635 L
s
.80612 .24524 m
.8127 .24235 L
s
.5375 .23767 m
.54032 .23304 L
s
.19761 .3995 m
.17763 .35621 L
s
.60545 .23463 m
.60954 .23075 L
s
.20753 .69595 m
.18324 .57236 L
s
.1951 .57946 m
.17084 .48643 L
s
.37339 .25049 m
.37117 .24327 L
s
.27308 .27313 m
.2646 .26098 L
s
.21298 .32476 m
.19727 .30019 L
s
.18871 .43856 m
.16659 .38389 L
s
.6953 .23525 m
.70066 .23185 L
s
.30481 .26193 m
.29881 .25215 L
s
.44059 .24329 m
.4409 .23744 L
s
.8127 .24235 m
.81933 .23947 L
s
.48501 .23872 m
.48659 .23345 L
s
.2211 .85384 m
.1969 .68994 L
s
.19432 .36417 m
.17515 .32904 L
s
.40379 .2452 m
.40283 .23869 L
s
.18398 .49392 m
.15994 .42326 L
s
.34617 .25164 m
.34268 .24368 L
s
.21737 .30889 m
.20275 .28768 L
s
.25747 .27601 m
.24771 .26263 L
s
.32327 .25536 m
.31852 .24658 L
s
.54032 .23304 m
.54317 .22841 L
s
.60954 .23075 m
.61366 .22688 L
s
.1969 .68994 m
.17126 .56522 L
s
.22401 .2967 m
.21054 .27811 L
s
.24407 .2807 m
.23304 .2659 L
s
.81933 .23947 m
.82599 .2366 L
s
.23291 .28747 m
.22064 .27097 L
s
.70066 .23185 m
.70606 .22845 L
s
.18324 .57236 m
.15755 .47898 L
s
.1822 .39165 m
.16071 .34831 L
s
.28366 .26078 m
.27635 .25014 L
s
.19311 .33718 m
.17485 .30796 L
s
.21161 .85027 m
.18618 .68374 L
s
.4409 .23744 m
.44121 .23157 L
s
.17441 .43088 m
.15077 .37619 L
s
.48659 .23345 m
.48818 .22816 L
s
.37117 .24327 m
.36893 .23607 L
s
.61366 .22688 m
.61781 .22301 L
s
.82599 .2366 m
.83269 .23374 L
s
.54317 .22841 m
.54604 .22377 L
s
.18618 .68374 m
.15918 .55805 L
s
.70606 .22845 m
.7115 .22503 L
s
.17084 .48643 m
.14529 .41572 L
s
.40283 .23869 m
.40186 .23216 L
s
.29881 .25215 m
.29272 .24263 L
s
.19407 .31635 m
.17681 .2915 L
s
.2646 .26098 m
.25596 .24939 L
s
.17763 .35621 m
.15691 .32094 L
s
.17126 .56522 m
.14412 .47157 L
s
.34268 .24368 m
.33914 .23579 L
s
.20206 .84632 m
.17538 .67739 L
s
.83269 .23374 m
.83943 .23089 L
s
.31852 .24658 m
.31369 .23794 L
s
.61781 .22301 m
.62199 .21913 L
s
.7115 .22503 m
.71697 .22161 L
s
.19727 .30019 m
.18109 .27864 L
s
.16659 .38389 m
.14353 .34047 L
s
.48818 .22816 m
.48978 .22284 L
s
.54604 .22377 m
.54894 .21914 L
s
.24771 .26263 m
.23775 .24999 L
s
.44121 .23157 m
.44152 .22566 L
s
.17538 .67739 m
.14697 .55088 L
s
.15994 .42326 m
.13475 .36855 L
s
.83943 .23089 m
.84622 .22806 L
s
.20275 .28768 m
.18773 .26869 L
s
.15755 .47898 m
.13047 .40824 L
s
.36893 .23607 m
.36665 .22888 L
s
.17515 .32904 m
.15531 .29958 L
s
.23304 .2659 m
.22177 .25204 L
s
.19245 .84203 m
.16448 .67091 L
s
.27635 .25014 m
.2689 .23983 L
s
.21054 .27811 m
.19673 .26116 L
s
.40186 .23216 m
.40087 .22562 L
s
.15918 .55805 m
.13053 .46421 L
s
.22064 .27097 m
.20808 .2557 L
s
.71697 .22161 m
.72248 .21817 L
s
.62199 .21913 m
.62621 .21525 L
s
.84622 .22806 m
.85304 .22524 L
s
.54894 .21914 m
.55186 .21452 L
s
.16071 .34831 m
.13838 .31286 L
s
.33914 .23579 m
.33555 .22795 L
s
.16448 .67091 m
.13465 .54371 L
s
.48978 .22284 m
.4914 .21749 L
s
.29272 .24263 m
.28652 .23329 L
s
.17485 .30796 m
.156 .28279 L
s
.44152 .22566 m
.44183 .21973 L
s
.31369 .23794 m
.30878 .2294 L
s
.18277 .83743 m
.1535 .66434 L
s
.15077 .37619 m
.12612 .33266 L
s
.25596 .24939 m
.24715 .23822 L
s
.72248 .21817 m
.72803 .21471 L
s
.14529 .41572 m
.11851 .36097 L
s
.14412 .47157 m
.11547 .40082 L
s
.62621 .21525 m
.63045 .21137 L
s
.85304 .22524 m
.85991 .22244 L
s
.14697 .55088 m
.1168 .45691 L
s
.36665 .22888 m
.36434 .22169 L
s
.17681 .2915 m
.15904 .26956 L
s
.40087 .22562 m
.39988 .21906 L
s
.55186 .21452 m
.55481 .2099 L
s
.1535 .66434 m
.12221 .53657 L
s
.15691 .32094 m
.13544 .2912 L
s
.23775 .24999 m
.22758 .23788 L
s
.17303 .83256 m
.14242 .65768 L
s
.4914 .21749 m
.49304 .21213 L
s
.72803 .21471 m
.73362 .21125 L
s
.85991 .22244 m
.86681 .21966 L
s
.2689 .23983 m
.26132 .22976 L
s
.18109 .27864 m
.16447 .25922 L
s
.63045 .21137 m
.63474 .20748 L
s
.44183 .21973 m
.44215 .21377 L
s
.22177 .25204 m
.21025 .23888 L
s
.14353 .34047 m
.11957 .30479 L
s
.33555 .22795 m
.3319 .22013 L
s
.13465 .54371 m
.10292 .44967 L
s
.18773 .26869 m
.17232 .25129 L
s
.13053 .46421 m
.10028 .39346 L
s
.20808 .2557 m
.19523 .24134 L
s
.13475 .36855 m
.10844 .32488 L
s
.13047 .40824 m
.10205 .35343 L
s
.19673 .26116 m
.18258 .24541 L
s
.28652 .23329 m
.28022 .22409 L
s
.86681 .21966 m
.87376 .2169 L
s
.30878 .2294 m
.3038 .22092 L
s
.14242 .65768 m
.10964 .52945 L
s
.55481 .2099 m
.55778 .20528 L
s
.15531 .29958 m
.13482 .27405 L
s
.73362 .21125 m
.73925 .20777 L
s
.16322 .82745 m
.13124 .65097 L
s
.39988 .21906 m
.39887 .21248 L
s
.36434 .22169 m
.36201 .21449 L
s
.49304 .21213 m
.49469 .20675 L
s
.24715 .23822 m
.23817 .22737 L
s
.63474 .20748 m
.63906 .20357 L
s
.87376 .2169 m
.88075 .21416 L
s
.44215 .21377 m
.44248 .20778 L
s
.13838 .31286 m
.11525 .2828 L
s
.12221 .53657 m
.08887 .44248 L
s
.73925 .20777 m
.74491 .20429 L
s
.156 .28279 m
.13657 .26043 L
s
.15334 .82212 m
.11998 .64421 L
s
.13124 .65097 m
.09694 .52237 L
s
.1168 .45691 m
.0849 .38615 L
s
.55778 .20528 m
.56078 .20068 L
s
.3319 .22013 m
.32821 .21232 L
s
.22758 .23788 m
.2172 .22617 L
s
.12612 .33266 m
.10046 .29672 L
s
.11547 .40082 m
.08536 .34593 L
s
.26132 .22976 m
.2536 .21986 L
s
.63906 .20357 m
.64341 .19966 L
s
.88075 .21416 m
.88779 .21144 L
s
.11851 .36097 m
.09051 .31712 L
s
.49469 .20675 m
.49635 .20136 L
s
.15904 .26956 m
.14076 .24968 L
s
.39887 .21248 m
.39785 .20588 L
s
.74491 .20429 m
.75062 .20079 L
s
.3038 .22092 m
.29874 .21249 L
s
.28022 .22409 m
.27382 .21498 L
s
.36201 .21449 m
.35964 .20728 L
s
.21025 .23888 m
.19848 .22623 L
s
.14339 .8166 m
.10861 .63743 L
s
.13544 .2912 m
.11326 .26526 L
s
.10964 .52945 m
.07467 .43534 L
s
.11998 .64421 m
.08412 .51533 L
s
.16447 .25922 m
.1474 .24132 L
s
.44248 .20778 m
.4428 .20176 L
s
.88779 .21144 m
.89487 .20875 L
s
.56078 .20068 m
.56381 .19609 L
s
.19523 .24134 m
.18209 .22764 L
s
.17232 .25129 m
.15651 .23502 L
s
.10292 .44967 m
.06932 .37888 L
s
.64341 .19966 m
.6478 .19572 L
s
.18258 .24541 m
.16808 .23053 L
s
.23817 .22737 m
.22902 .21673 L
s
.75062 .20079 m
.75637 .19728 L
s
.11957 .30479 m
.09471 .27437 L
s
.10028 .39346 m
.06845 .33845 L
s
.49635 .20136 m
.49804 .19597 L
s
.13336 .81092 m
.09714 .63063 L
s
.89487 .20875 m
.90199 .20609 L
s
.32821 .21232 m
.32445 .2045 L
s
.13482 .27405 m
.11367 .25123 L
s
.10205 .35343 m
.07231 .30936 L
s
.10844 .32488 m
.08105 .28864 L
s
.10861 .63743 m
.07116 .50833 L
s
.39785 .20588 m
.39682 .19926 L
s
.09694 .52237 m
.0603 .42825 L
s
.56381 .19609 m
.56687 .1915 L
s
.6478 .19572 m
.65222 .19177 L
s
.2536 .21986 m
.24574 .21008 L
s
.35964 .20728 m
.35724 .20005 L
s
.75637 .19728 m
.76215 .19377 L
s
.4428 .20176 m
.44313 .19573 L
s
.2172 .22617 m
.20661 .21474 L
s
.29874 .21249 m
.2936 .20407 L
s
.08887 .44248 m
.05355 .37165 L
s
.27382 .21498 m
.2673 .20592 L
s
.12327 .8051 m
.08557 .62383 L
s
.13657 .26043 m
.11656 .24005 L
s
.49804 .19597 m
.49974 .19058 L
s
.11525 .2828 m
.09131 .25643 L
s
.09714 .63063 m
.05807 .50137 L
s
.05532 .33759 m
.0849 .38615 L
s
.76215 .19377 m
.76798 .19025 L
s
.19848 .22623 m
.18645 .21394 L
s
.65222 .19177 m
.65668 .18781 L
s
.56687 .1915 m
.56995 .18692 L
s
.08412 .51533 m
.04577 .42121 L
s
.14076 .24968 m
.12195 .23125 L
s
.07636 .26884 m
.10046 .29672 L
s
.39682 .19926 m
.39577 .19261 L
s
.06164 .31256 m
.08536 .34593 L
s
.32445 .2045 m
.32065 .19667 L
s
.1131 .79916 m
.0739 .61704 L
s
.22902 .21673 m
.21969 .20623 L
s
.06859 .28964 m
.09051 .31712 L
s
.18209 .22764 m
.16864 .21441 L
s
.44313 .19573 m
.44346 .18967 L
s
.1474 .24132 m
.12986 .22451 L
s
.35724 .20005 m
.3548 .19279 L
s
.04663 .38175 m
.07467 .43534 L
s
.16808 .23053 m
.15323 .21625 L
s
.15651 .23502 m
.1403 .21957 L
s
.76798 .19025 m
.77385 .18673 L
s
.11326 .26526 m
.09034 .24196 L
s
.49974 .19058 m
.50146 .18519 L
s
.08557 .62383 m
.04484 .49446 L
s
.65668 .18781 m
.66118 .18381 L
s
.2936 .20407 m
.28838 .19564 L
s
.24574 .21008 m
.23774 .20035 L
s
.56995 .18692 m
.57306 .18234 L
s
.10286 .79312 m
.06212 .61026 L
s
.03865 .43204 m
.07116 .50833 L
s
.2673 .20592 m
.26067 .19686 L
s
.0523 .35227 m
.06932 .37888 L
s
.20661 .21474 m
.1958 .20348 L
s
.77385 .18673 m
.77977 .18321 L
s
.08061 .25967 m
.09471 .27437 L
s
.39577 .19261 m
.39471 .18594 L
s
.09543 .23374 m
.11367 .25123 L
s
.44346 .18967 m
.4438 .1836 L
s
.0584 .32501 m
.06845 .33845 L
s
.0739 .61704 m
.03148 .4876 L
s
.66118 .18381 m
.66571 .1798 L
s
.04388 .3983 m
.0603 .42825 L
s
.32065 .19667 m
.31678 .1888 L
s
.50146 .18519 m
.5032 .1798 L
s
.07236 .27909 m
.08105 .28864 L
s
.09254 .78699 m
.05023 .6035 L
s
.06503 .30069 m
.07231 .30936 L
s
.3548 .19279 m
.35234 .18551 L
s
.57306 .18234 m
.57621 .17776 L
s
.18645 .21394 m
.17416 .20188 L
s
.03603 .45185 m
.05807 .50137 L
s
.77977 .18321 m
.78572 .17968 L
s
.10123 .22593 m
.11656 .24005 L
s
.21969 .20623 m
.21018 .19579 L
s
.04947 .36557 m
.05355 .37165 L
s
.28838 .19564 m
.28307 .18717 L
s
.66571 .1798 m
.67028 .17576 L
s
.08518 .25053 m
.09131 .25643 L
s
.02858 .51192 m
.06212 .61026 L
s
.16864 .21441 m
.15488 .20149 L
s
.1076 .21834 m
.12195 .23125 L
s
.08214 .78079 m
.03823 .59677 L
s
.4438 .1836 m
.44414 .17751 L
s
.39471 .18594 m
.39364 .17925 L
s
.23774 .20035 m
.22959 .19062 L
s
.26067 .19686 m
.25394 .18778 L
s
.5032 .1798 m
.50495 .17443 L
s
.78572 .17968 m
.79172 .17617 L
s
.57621 .17776 m
.57938 .17318 L
s
.15323 .21625 m
.13802 .20239 L
s
.0413 .41343 m
.04577 .42121 L
s
.11462 .21092 m
.12986 .22451 L
s
.1403 .21957 m
.12367 .20468 L
s
.03356 .47017 m
.04484 .49446 L
s
.31678 .1888 m
.31286 .18089 L
s
.1958 .20348 m
.18477 .1923 L
s
.35234 .18551 m
.34983 .17819 L
s
.67028 .17576 m
.67489 .17169 L
s
.07166 .77453 m
.02612 .59006 L
s
.02607 .53535 m
.05023 .6035 L
s
.09011 .24175 m
.09034 .24196 L
s
.79172 .17617 m
.79777 .17266 L
s
.57938 .17318 m
.58258 .16859 L
s
.44414 .17751 m
.44448 .17141 L
s
.50495 .17443 m
.50672 .16906 L
s
.39364 .17925 m
.39255 .17254 L
s
.28307 .18717 m
.27768 .17865 L
s
.21018 .19579 m
.20049 .18535 L
s
.17416 .20188 m
.16161 .18992 L
s
.03123 .4871 m
.03148 .4876 L
s
.67489 .17169 m
.67954 .1676 L
s
.01896 .6032 m
.06109 .76822 L
s
.79777 .17266 m
.80386 .16916 L
s
.02368 .55727 m
.03823 .59677 L
s
.25394 .18778 m
.24708 .17864 L
s
.22959 .19062 m
.2213 .18084 L
s
.34983 .17819 m
.3473 .17083 L
s
.31286 .18089 m
.30888 .17292 L
s
.141 .1889 m
.15488 .20149 L
s
.58258 .16859 m
.58581 .16398 L
s
.50672 .16906 m
.50851 .16369 L
s
.44448 .17141 m
.44483 .1653 L
s
.01635 .63273 m
.05045 .76188 L
s
.67954 .1676 m
.68422 .16348 L
s
.80386 .16916 m
.80999 .16567 L
s
.13114 .19637 m
.13802 .20239 L
s
.39255 .17254 m
.39144 .1658 L
s
.18477 .1923 m
.17351 .18113 L
s
.02142 .57776 m
.02612 .59006 L
s
.12242 .20361 m
.12367 .20468 L
s
.27768 .17865 m
.27221 .17006 L
s
.58581 .16398 m
.58907 .15936 L
s
.01385 .66076 m
.03972 .75552 L
s
.80999 .16567 m
.81617 .16219 L
s
.68422 .16348 m
.68895 .15933 L
s
.50851 .16369 m
.51032 .15833 L
s
.20049 .18535 m
.19061 .17486 L
s
.3473 .17083 m
.34473 .16344 L
s
.30888 .17292 m
.30485 .1649 L
s
.44483 .1653 m
.44518 .15919 L
s
.24708 .17864 m
.24011 .16941 L
s
.15222 .1812 m
.16161 .18992 L
s
.2213 .18084 m
.21285 .17099 L
s
.39144 .1658 m
.39033 .15903 L
s
.81617 .16219 m
.8224 .15873 L
s
.01146 .68734 m
.02891 .74913 L
s
.58907 .15936 m
.59237 .15471 L
s
.68895 .15933 m
.69371 .15515 L
s
.51032 .15833 m
.51215 .15298 L
s
.27221 .17006 m
.26664 .16138 L
s
.16514 .17295 m
.17351 .18113 L
s
.8224 .15873 m
.82868 .15529 L
s
.44518 .15919 m
.44553 .15306 L
s
.34473 .16344 m
.34212 .15601 L
s
.00919 .71252 m
.01801 .74274 L
s
.69371 .15515 m
.69852 .15095 L
s
.30485 .1649 m
.30075 .15682 L
s
.59237 .15471 m
.59569 .15004 L
s
.39033 .15903 m
.3892 .15224 L
s
.24011 .16941 m
.23302 .1601 L
s
.19061 .17486 m
.18053 .16427 L
s
.21285 .17099 m
.20425 .16103 L
s
.51215 .15298 m
.514 .14762 L
s
.69852 .15095 m
.70336 .14672 L
s
.44553 .15306 m
.44589 .14692 L
s
.59569 .15004 m
.59905 .14533 L
s
.34212 .15601 m
.33947 .14854 L
s
.26664 .16138 m
.26099 .15262 L
s
.3892 .15224 m
.38805 .14543 L
s
.30075 .15682 m
.29658 .14867 L
s
.514 .14762 m
.51587 .14225 L
s
.70336 .14672 m
.70825 .14247 L
s
.59905 .14533 m
.60244 .14059 L
s
.23302 .1601 m
.22581 .15067 L
s
.44589 .14692 m
.44625 .14077 L
s
.18019 .16391 m
.18053 .16427 L
s
.19794 .15377 m
.20425 .16103 L
s
.70825 .14247 m
.71318 .1382 L
s
.33947 .14854 m
.33679 .14103 L
s
.51587 .14225 m
.51775 .13687 L
s
.38805 .14543 m
.38689 .13858 L
s
.26099 .15262 m
.25525 .14377 L
s
.60244 .14059 m
.60586 .13581 L
s
.29658 .14867 m
.29236 .14046 L
s
.44625 .14077 m
.44661 .13462 L
s
.71318 .1382 m
.71816 .13391 L
s
.21922 .1421 m
.22581 .15067 L
s
.51775 .13687 m
.51966 .13147 L
s
.60586 .13581 m
.60932 .13099 L
s
.33679 .14103 m
.33407 .13348 L
s
.38689 .13858 m
.38571 .13171 L
s
.71816 .13391 m
.72317 .12961 L
s
.44661 .13462 m
.44698 .12845 L
s
.25525 .14377 m
.2494 .13482 L
s
.29236 .14046 m
.28807 .13218 L
s
.51966 .13147 m
.52159 .12605 L
s
.60932 .13099 m
.61281 .12612 L
s
.72317 .12961 m
.72823 .12529 L
s
.38571 .13171 m
.38451 .12481 L
s
.33407 .13348 m
.3313 .12589 L
s
.44698 .12845 m
.44735 .12227 L
s
.61281 .12612 m
.61633 .12121 L
s
.52159 .12605 m
.52354 .1206 L
s
.72823 .12529 m
.73334 .12097 L
s
.28807 .13218 m
.28371 .12385 L
s
.24519 .12841 m
.2494 .13482 L
s
.38451 .12481 m
.3833 .11787 L
s
.3313 .12589 m
.3285 .11826 L
s
.44735 .12227 m
.44773 .11606 L
s
.61633 .12121 m
.61989 .11624 L
s
.73334 .12097 m
.73849 .11664 L
s
.52354 .1206 m
.52551 .11511 L
s
.28371 .12385 m
.27928 .11545 L
s
.73849 .11664 m
.74368 .11232 L
s
.61989 .11624 m
.62349 .11123 L
s
.3833 .11787 m
.38207 .11091 L
s
.44773 .11606 m
.44811 .10984 L
s
.52551 .11511 m
.52751 .10957 L
s
.3285 .11826 m
.32566 .11058 L
s
.74368 .11232 m
.74893 .10799 L
s
.62349 .11123 m
.62712 .10618 L
s
.27762 .11234 m
.27928 .11545 L
s
.52751 .10957 m
.52952 .10399 L
s
.44811 .10984 m
.4485 .10358 L
s
.38207 .11091 m
.38083 .1039 L
s
.32566 .11058 m
.32277 .10285 L
s
.62712 .10618 m
.63078 .10107 L
s
.52952 .10399 m
.53156 .09835 L
s
.4485 .10358 m
.44889 .09728 L
s
.38083 .1039 m
.37956 .09686 L
s
.63078 .10107 m
.63449 .09593 L
s
.32277 .10285 m
.31984 .09507 L
s
.53156 .09835 m
.53362 .09265 L
s
.44889 .09728 m
.44928 .09095 L
s
.63449 .09593 m
.63823 .09074 L
s
.37956 .09686 m
.37828 .08977 L
s
.31926 .09356 m
.31984 .09507 L
s
.53362 .09265 m
.53571 .08689 L
s
.63823 .09074 m
.64201 .08551 L
s
.44928 .09095 m
.44968 .08456 L
s
.37828 .08977 m
.37698 .08264 L
s
.53571 .08689 m
.53782 .08106 L
s
.64201 .08551 m
.64583 .08025 L
s
.44968 .08456 m
.45009 .07812 L
s
.37698 .08264 m
.37567 .07546 L
s
.53782 .08106 m
.53995 .07517 L
s
.64583 .08025 m
.64969 .07495 L
s
.45009 .07812 m
.45049 .07162 L
s
.64969 .07495 m
.65359 .06963 L
s
.53995 .07517 m
.54211 .0692 L
s
.37476 .07052 m
.37567 .07546 L
s
.45049 .07162 m
.45091 .06505 L
s
.65359 .06963 m
.65753 .06429 L
s
.54211 .0692 m
.54429 .06315 L
s
.65753 .06429 m
.66151 .05892 L
s
.45091 .06505 m
.45133 .0584 L
s
.54429 .06315 m
.5465 .05704 L
s
.45133 .0584 m
.45175 .05167 L
s
.5465 .05704 m
.54873 .05085 L
s
.45175 .05167 m
.45218 .04485 L
s
.54873 .05085 m
.55099 .0446 L
s
.55099 .0446 m
.55328 .03828 L
s
.45247 .04026 m
.45218 .04485 L
s
.55328 .03828 m
.5556 .03189 L
s
.5556 .03189 m
.55794 .02545 L
s
.55794 .02545 m
.56031 .01895 L
s
.56031 .01895 m
.56271 .0124 L
s
.56271 .0124 m
.56514 .00581 L
s
.25 Mabswid
.03399 .19842 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.96816 .22579 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
.03399 .19842 m
0 .87626 L
s
.02756 .32664 m
.03396 .32857 L
s
[(100)] .01476 .32278 1 .30143 Mshowa
.02095 .45839 m
.02744 .46 L
s
[(200)] .00798 .45516 1 .24828 Mshowa
.01416 .5938 m
.02072 .59507 L
s
[(300)] .00104 .59125 1 .19393 Mshowa
.00718 .73304 m
.0138 .73395 L
s
[(400)] -0.00606 .7312 1 .13836 Mshowa
gsave
-0.06073 .5083 -105.625 -11.9382 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 263.5},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P00011a000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00;Coo`@000?oo`03003ooooo06;oo`00]_oo00<00?ooool0Iooo002fool0
0`00ooooo`1Wool00;Koo`03003ooooo06Ooo`00]_oo00<00?ooool0Iooo002dool3001Yool00;Ko
o`03003ooooo06Ooo`00oooo8Ooo003ooolQool007Woo`03003ooooo0:Coo`00K?oo1@001?oo0P00
0ooo00<00?ooool00_oo0`00=_oo0P00I_oo001]ool00`00ool00004ool01000ooooo`000_oo00<0
0?ooool01Ooo00<00?ooool0<Ooo0P0000Coo`00000006Coo`00KOoo00<00?oo00001?oo00<00?oo
ool00ooo00<00?ooool00ooo0P00<Ooo0`000ooo00<00?oo00000P00H_oo001^ool00`00ooooo`03
ool00`00ooooo`04ool01@00oooooooo0000<?oo0`001_oo00<00?oo00000_oo0P00H?oo001^ool0
0`00ooooo`02ool40004ool00`00ooooo`02ool3000[ool20009ool20005ool2001Nool006koo`03
003ooooo00?oo`03003ooooo00Goo`03003ooooo02coo`<000_oo`03003ooooo00Koo`8005coo`00
M?oo00<00?ooool0<Ooo0`003_oo00<00?ooool02?oo00<00?ooool0FOoo0025ool20004ool00`00
ooooo`03ool00`00ooooo`0Bool2000Aool00`00ooooo`09ool2001Iool008Coo`04003ooooo0009
ool00`00ooooo`0?ool3000Bool2000=ool2001Gool008Coo`04003ooooo000:ool00`00ooooo`0;
ool3000Eool2000?ool2001Eool008Coo`04003ooooo000:ool00`00ooooo`09ool200000ooo003o
o`0Eool2000Aool2001Cool008Coo`04003ooooo000;ool00`00ooooo`05ool30003ool00`00oooo
o`0Cool00`00ool0000Cool2001Aool008Coo`04003ooooo0008ool01000ooooo`001?oo0`001_oo
00<00?ooool04ooo00<00?oo00005Ooo0P00Cooo0025ool20009ool40002ool20009ool00`00oooo
o`0Cool00`00ool0000Gool00`00ooooo`1<ool009?oo`<000_oo`03003ooooo01;oo`04003ooooo
000Fool01000ool00000C?oo002Aool2000>ool00`00ooooo`0Bool01000ooooo`005_oo00@00?oo
ooooo`8004[oo`00S_oo0`004?oo00<00?ooool04_oo00@00?ooool001Goo`03003ooooo00Coo`80
04Soo`00Rooo0`000_oo00<00?ooool03_oo00<00?ooool04Ooo00D00?ooooooo`0001Coo`03003o
oooo00Ooo`8004Koo`00ROoo0P0000?oo`00ool00_oo00<00?ooool03_oo00<00?ooool04Ooo00D0
0?ooooooo`0001Coo`03003ooooo00Woo`8004Coo`00Q_oo0`001ooo00<00?ooool03_oo00<00?oo
ool04?oo00<00?ooool00_oo00<00?ooool04Ooo00<00?ooool03?oo00<00?ooool0@Ooo0023ool3
000:ool00`00ooooo`0>ool00`00ooooo`0@ool00`00ooooo`02ool00`00ooooo`0@ool00`00oooo
o`0>ool20011ool0087oo`<000coo`03003ooooo00koo`03003ooooo013oo`03003ooooo00;oo`03
003ooooo013oo`03003ooooo013oo`8003ooo`00G_oo0P001?oo00<00?ooool01?oo00<00?ooool0
4?oo0`000_oo00<00?ooool02ooo00<00?ooool03Ooo00<00?ooool03ooo00<00?ooool00ooo00<0
0?ooool03ooo00<00?ooool04ooo0P00?Ooo001Mool01000ooooo`002_oo00<00?ooool03Ooo0`00
1_oo00<00?ooool02_oo00<00?ooool03?oo00<00?ooool04?oo00<00?ooool00ooo00<00?ooool0
3_oo00<00?ooool05_oo0P00>ooo001Mool01000ooooo`001ooo1@003?oo0P002Ooo00<00?ooool0
2_oo00<00?ooool03?oo00<00?ooool04?oo00<00?ooool00ooo00<00?ooool03Ooo00<00?ooool0
6Ooo0P00>Ooo001Mool01000ooooo`001ooo00@00?ooool000[oo`@000[oo`03003ooooo00[oo`03
003ooooo00coo`03003ooooo013oo`03003ooooo00?oo`03003ooooo00coo`03003ooooo01Woo`04
003ooooo000hool005goo`04003ooooo0008ool00`00ool00007ool30004ool00`00ooooo`07ool0
0`00ooooo`0:ool00`00ooooo`0<ool00`00ooooo`0?ool00`00ooooo`04ool00`00ooooo`0<ool0
0`00ooooo`0Hool00`00ooooo`02ool2000fool005goo`04003ooooo0009ool20005ool20007ool0
0`00ooooo`08ool00`00ooooo`0:ool00`00ooooo`0;ool00`00ooooo`0?ool00`00ooooo`04ool0
0`00ooooo`0;ool00`00ooooo`0Gool20007ool2000dool005koo`8000_oo`03003ooooo00<000[o
o`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00_oo`03003ooooo00koo`03003ooooo00Go
o`03003ooooo00[oo`03003ooooo01Ooo`03003ooooo00Woo`8003;oo`00Jooo0`001?oo00<00?oo
ool01_oo00<00?ooool02?oo00<00?ooool02Ooo00<00?ooool02ooo00<00?ooool03_oo00<00?oo
ool01Ooo00<00?ooool02_oo00<00?ooool05_oo00<00?ooool03?oo0P00<?oo001Yool20008ool0
0`00ooooo`06ool00`00ooooo`07ool00`00ooooo`09ool00`00ooooo`0;ool00`00ooooo`0>ool0
0`00ooooo`05ool00`00ooooo`09ool00`00ooooo`0Fool00`00ooooo`0?ool2000^ool006Koo`<0
00[oo`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00Woo`03003ooooo00_oo`03003ooooo
00goo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo01Koo`03003ooooo01;oo`8002coo`00
I?oo0P001_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02?oo00<0
0?ooool02ooo00<00?ooool03Ooo00<00?ooool01_oo00<00?ooool02?oo00<00?ooool05Ooo00<0
0?ooool05Ooo00<00?ooool0:Ooo001Qool40008ool00`00ooooo`05ool00`00ooooo`05ool00`00
ooooo`07ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0<ool00`00ooooo`07ool00`00
ooooo`07ool00`00ooooo`0Eool00`00ooooo`0Gool2000Yool005koo`<000goo`03003ooooo00Go
o`03003ooooo00Goo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00[oo`03003ooooo00co
o`03003ooooo00Ooo`03003ooooo00Koo`03003ooooo01Goo`03003ooooo01[oo`8002Ooo`00>ooo
0P001?oo00<00?ooool00_oo10004_oo0P002Ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?oo
ool01_oo00<00?ooool01_oo00<00?ooool02?oo00<00?ooool02_oo00<00?ooool03?oo00<00?oo
ool01ooo00<00?ooool01Ooo00<00?ooool05?oo0P007Ooo00@00?oo000002Goo`00>_oo00@00?oo
ool000[oo`03003ooooo00koo`<000coo`8000Goo`03003ooooo00Goo`03003ooooo00Goo`03003o
oooo00Koo`03003ooooo00Soo`03003ooooo00[oo`03003ooooo00coo`03003ooooo00Ooo`03003o
oooo00Goo`03003ooooo01?oo`03003ooooo01_oo`8000Coo`8002?oo`00>_oo00@00?ooool000[o
o`03003ooooo00_oo`<000[oo`03003ooooo00Coo`03003ooooo00?oo`03003ooooo00Coo`03003o
oooo00Koo`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00[oo`03003ooooo00_oo`03003o
oooo00Soo`03003ooooo00Coo`03003ooooo01?oo`03003ooooo01[oo`8000Soo`80027oo`00>_oo
00@00?ooool000[oo`03003ooooo00Woo`8000koo`03003ooooo00?oo`03003ooooo00Coo`03003o
oooo00Coo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00[oo`03003o
oooo00_oo`03003ooooo00Soo`03003ooooo00?oo`03003ooooo01?oo`03003ooooo01Woo`8000co
o`03003ooooo01koo`00>_oo00@00?ooool000[oo`03003ooooo00Koo`<000coo`03003ooooo00;o
o`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Koo`03003ooooo00Go
o`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00;o
o`03003ooooo01?oo`03003ooooo01Woo`03003ooooo00goo`8001koo`00>_oo00@00?ooool000So
o`<000Goo`<0013oo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Co
o`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00_o
o`03003ooooo00Woo`03003ooooo00;oo`03003ooooo01;oo`03003ooooo01Woo`03003ooooo013o
o`8001coo`00>ooo0P002ooo00@00?ooooooo`8000ooo`03003ooooo00;oo`03003ooooo00;oo`03
003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Koo`03
003ooooo00Ooo`03003ooooo00Soo`03003ooooo00_oo`03003ooooo00Woo`05003oooooool0000D
ool00`00ooooo`0Gool3000Eool2000Jool004Woo`<001;oo`03003ooooo00;oo`03003ooooo00;o
o`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00?oo`03003ooooo00Oo
o`03003ooooo00Koo`03003ooooo00Soo`03003ooooo00_oo`03003ooooo00Woo`04003ooooo000D
ool00`00ooooo`0Fool2000Jool2000Hool004Koo`<0017oo`03003ooooo00;oo`05003oooooool0
0005ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`04ool00`00oooo
o`06ool00`00ooooo`06ool00`00ooooo`08ool00`00ooooo`0:ool00`00ooooo`0:ool01000oooo
o`004_oo0P006?oo00<00?ooool07?oo0P005_oo0014ool2000Bool01000ooooo`001?oo00D00?oo
ooooo`0000Goo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00?oo`03
003ooooo00Koo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo00[oo`03003ooooo00[oo`03
003oo`0001;oo`03003ooooo01Ooo`03003ooooo01ooo`8001Coo`00@_oo0P005Ooo00@00?ooool0
00Coo`04003oooooool20004ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`04ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`0:ool00`00
ooooo`0:ool2000Bool00`00ooooo`0Gool00`00ooooo`0Rool00`00ooooo`0Aool004;oo`03003o
o`00008000ooo`05003oooooool00002ool00`00ooooo`02ool00`00ooooo`02ool01@00oooooooo
00001Ooo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool01Ooo00<00?oo
ool01_oo00<00?ooool01ooo00<00?ooool02Ooo00<00?ooool02ooo0P004Ooo00<00?ooool05Ooo
0`009_oo0P004Ooo0012ool00`00ooooo`02ool3000=ool01@00oooooooo00000_oo00<00?ooool0
0_oo00<00?ooool00_oo00D00?ooooooo`0000Coo`03003ooooo00?oo`03003ooooo00?oo`03003o
oooo00Coo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00Woo`03003o
oooo00_oo`03003ooooo00ooo`03003ooooo01Goo`03003ooooo023oo`<000Koo`8000ooo`00@_oo
00<00?ooool01Ooo0P002?oo00D00?ooooooo`0000?oo`03003ooooo008000?oo`05003oooooool0
0004ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`04ool00`00oooo
o`04ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`09ool00`00ooooo`0;ool00`00oooo
o`0>ool00`00ooooo`0Dool2000Qool2000;ool2000=ool004;oo`03003ooooo00Ooo`<000Koo`05
003oooooool00003ool01@00oooooooo00000ooo00D00?ooooooo`0000Coo`03003ooooo00;oo`03
003ooooo00?oo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Goo`03
003ooooo00Ooo`03003ooooo00Soo`03003ooooo00_oo`8000ooo`03003ooooo01?oo`80023oo`<0
00ooo`8000_oo`00@_oo00<00?ooool02_oo0`0000Coo`00ooooo`8000;oo`05003oooooool00003
ool01@00oooooooo00000ooo0P000ooo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo
00<00?ooool01?oo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool02?oo
00<00?ooool02_oo00<00?oo00003_oo00<00?ooool04ooo00<00?ooool07Ooo0`005?oo0P002Ooo
0012ool00`00ooooo`0<ool50002ool01000ooooo`000ooo00D00?ooooooo`0000?oo`03003ooooo
00;oo`04003ooooo0005ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00oooo
o`04ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`08ool00`00ooooo`0:ool00`00ool0
000<ool2000Dool2000Nool2000Hool20008ool004;oo`03003ooooo00goo`03003ooooo00<00004
ool00?ooool20002ool01@00oooooooo00000ooo00<00?ooool00_oo00@00?ooool000Goo`05003o
ooooool00006ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`08ool00`00ooooo`09ool01000ooooo`002ooo00<00?ooool04ooo00<00?ooool06ooo
0`005_oo100000?oo`00ool01ooo0012ool00`00ooooo`0;ool00`00ooooo`020002ool40003ool0
1000ooooo`000ooo00D00?ooooooo`0000?oo`05003oooooool00004ool00`00ooooo`02ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`06ool00`00
ooooo`07ool00`00ooooo`09ool01@00oooooooo00002_oo00<00?ooool04_oo0P007Ooo00<00?oo
ool04ooo10001Ooo00<00?ooool01_oo0012ool00`00ooooo`0<ool01@00oooooooo00000_oo00<0
0?oo00000P000_oo00@00?ooool000?oo`05003oooooool00003ool01@00oooooooo00001?oo00<0
0?ooool00_oo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool01Ooo00<0
0?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00D00?ooooooo`0000Woo`03003ooooo017o
o`8001coo`<001;oo`@000Woo`03003ooooo00Koo`00@Ooo00<00?ooool02ooo00@00?ooool000?o
o`03003oo`0000?oo`@000?oo`05003oooooool00003ool01@00oooooooo00000ooo00<00?ooool0
0_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool0
1?oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool02Ooo00<00?ooool00_oo00<00?ooool0
1ooo00<00?ooool04?oo00<00?ooool06_oo0P004_oo0`003Ooo00<00?ooool01_oo0011ool00`00
ooooo`0<ool01000ooooo`000ooo00<00?oo00000ooo00<00?ooool00`0000?oo`00ool00_oo00D0
0?ooooooo`0000;oo`05003oooooool00004ool00`00ooooo`02ool00`00ooooo`03ool00`00oooo
o`02ool00`00ooooo`04ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`06ool00`00oooo
o`09ool00`00ooooo`02ool00`00ooooo`06ool00`00ooooo`0?ool2000Jool3000@ool4000@ool0
0`00ooooo`06ool0047oo`03003ooooo00Woo`05003oooooool00002ool01000ooooo`000_oo00D0
0?ooooooo`0000;oo`03003oo`00008000?oo`05003oooooool00002ool01@00oooooooo00001?oo
00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool01?oo
00<00?ooool01_oo00<00?ooool01_oo00<00?ooool02?oo00<00?ooool00ooo00<00?ooool01Ooo
00<00?ooool03ooo00<00?ooool06Ooo00<00?ooool03Ooo10005?oo00<00?ooool01_oo0011ool0
0`00ooooo`0:ool20002ool01000ooooo`000_oo00@00?ooool000?oo`04003ooooo0003ool20002
ool01@00oooooooo00000_oo00D00?ooooooo`0000Coo`03003ooooo00;oo`03003ooooo00;oo`03
003ooooo00;oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Koo`03003ooooo00Koo`03
003ooooo00Ooo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00ooo`03003ooooo01Ooo`<0
00coo`@001Soo`03003ooooo00Koo`00@Ooo00<00?ooool02?oo00D00?ooooooo`0000;oo`04003o
oooo0002ool01000ooooo`000ooo00@00?ooool000?oo`@000?oo`05003oooooool00003ool00`00
ooooo`02ool00`00ooooo`02ool01@00oooooooo00001Ooo00<00?ooool00ooo00<00?ooool01?oo
00<00?ooool01?oo00<00?ooool01_oo00<00?ooool02?oo00<00?ooool01?oo00<00?ooool00ooo
00<00?ooool03_oo0P006?oo0P003?oo0`007?oo00<00?ooool01_oo0011ool00`00ooooo`09ool0
1@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000ooo00<00?oo
ool00`0000?oo`00ool00_oo00D00?ooooooo`0000?oo`03003ooooo00;oo`03003ooooo00;oo`03
003ooooo00;oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Koo`03
003ooooo00Ooo`03003ooooo00Goo`04003oooooool2000?ool2000Gool3000:ool4000Oool00`00
ooooo`06ool0047oo`03003ooooo00Ooo`03003ooooo00800003ool00?oo00;oo`04003ooooo0002
ool01000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo0002ool30003ool01000ooooo`00
1?oo00<00?ooool00_oo00D00?ooooooo`0000Goo`03003ooooo00;oo`03003ooooo00Coo`03003o
oooo00Coo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Koo`04003ooooo000@ool00`00
ooooo`0Fool00`00ooooo`07ool4000Tool00`00ooooo`05ool0047oo`03003ooooo00Soo`07003o
ooooool00?oo00000ooo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo00000_oo00@0
0?ooool000?oo`D000?oo`05003oooooool00004ool00`00ooooo`02ool00`00ooooo`02ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`07ool00`00
ooooo`06ool00`00ool0000@ool00`00ooooo`0Dool30007ool3000Xool00`00ooooo`05ool0047o
o`03003ooooo00Ooo`03003oo`0000?oo`03003oo`0000;oo`05003oooooool00002ool01000oooo
o`000_oo00@00?ooool000?oo`04003ooooo0003ool00`00ooooo`020003ool01@00oooooooo0000
0ooo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool0
1?oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?oo00003_oo0P005_oo00<0
0?ooool01?oo1000:ooo00<00?ooool01Ooo0011ool00`00ooooo`08ool00`00ool00002ool01000
ooooo`000_oo00@00?ooool000?oo`03003oo`0000?oo`04003ooooo0003ool01000ooooo`000ooo
00@00?ooooooo`@000Coo`05003oooooool00004ool01@00oooooooo00001Ooo00<00?ooool00ooo
00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo0P003Ooo0P005Ooo
0`000ooo1000;ooo00<00?ooool01Ooo0011ool00`00ooooo`08ool01000ooooo`000_oo00@00?oo
ool000;oo`04003ooooo0003ool00`00ool00003ool01000ooooo`000_oo00D00?ooooooo`0000?o
o`04003ooooo0002ool30003ool01@00oooooooo00000ooo00<00?ooool00_oo00<00?ooool00ooo
00<00?ooool00ooo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo
00<00?ooool02ooo00<00?ooool05?oo00<00?ooool01000<ooo00<00?ooool01Ooo0011ool00`00
ooooo`06ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000?oo`04003ooooo0003ool01000ooooo`000ooo00@00?ooooooo`D000?oo`03003o
oooo00;oo`05003oooooool00005ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`04ool0
0`00ooooo`06ool00`00ooooo`07ool2000<ool00`00ooooo`0Bool6000gool00`00ooooo`05ool0
047oo`03003ooooo00Ooo`04003ooooo0002ool01000ooooo`000_oo00H00?ooool00?oo0003ool0
1000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01000ooooo`000ooo00@00?oo
ooooo`<000;oo`05003oooooool00004ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03
ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`06ool00`00ool0000;ool00`00ooooo`0B
ool4000jool00`00ooooo`05ool0047oo`03003ooooo00Goo`06003ooooo003oo`000ooo00<00?oo
00000_oo00@00?ooool000;oo`07003oooooool00?oo00000ooo00@00?ooool000?oo`04003ooooo
0003ool01000ooooo`000ooo00H00?ooooooo`00ool30003ool00`00ooooo`02ool00`00ooooo`02
ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`06
ool01000ooooo`002Ooo0P004Ooo1000?_oo00<00?ooool01Ooo0011ool00`00ooooo`06ool00`00
ool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo
00@00?ooool000;oo`05003oooooool00002ool01000ooooo`000ooo00<00?ooool00_oo00<00?oo
ool00`000_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool00ooo00<0
0?ooool01?oo00<00?ooool01?oo00<00?ooool01ooo00@00?ooool000Soo`03003ooooo00goo`D0
047oo`03003ooooo00Goo`00;?oo10000ooo0P001?oo0P001_oo00<00?ooool01ooo00<00?oo0000
0_oo00@00?ooool000;oo`04003ooooo0002ool00`00ool00003ool01000ooooo`000_oo00D00?oo
ooooo`0000;oo`05003oooooool00002ool01000ooooo`000ooo00D00?ooooooo`0000?oo`03003o
o`00008000Coo`03003ooooo00;oo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03
003ooooo00Goo`03003ooooo00Koo`05003oooooool00007ool00`00ooooo`0;ool300000ooo0000
0013ool00`00ooooo`05ool002koo`05003oooooool00002ool01000ooooo`000_oo00<00?ooool0
0ooo00<00?ooool01?oo00L00?ooooooo`00ool00002ool01P00ooooo`00ool000?oo`06003ooooo
003oo`000ooo00<00?oo00000ooo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`00
0ooo00D00?ooooooo`0000?oo`03003ooooo00;oo`<00003ool00?oo00?oo`03003ooooo00;oo`03
003ooooo00?oo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00Koo`05003oooooool00006
ool00`00ooooo`08ool40003ool00`00ooooo`13ool00`00ooooo`05ool002koo`05003oooooool0
0002ool01000ooooo`000_oo00<00?ooool00ooo00<00?ooool01Ooo00@00?ooool000;oo`03003o
o`0000;oo`04003ooooo0003ool00`00ool00002ool01000ooooo`000_oo00D00?ooooooo`0000;o
o`04003ooooo0003ool01000ooooo`000_oo00D00?ooooooo`0000?oo`05003oooooool00003ool0
0`00ooooo`030003ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`04
ool00`00ooooo`06ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`05ool40004ool30016
ool00`00ooooo`05ool002koo`05003oooooool00002ool01000ooooo`000_oo00<00?ooool00ooo
00<00?ooool01Ooo00D00?ooooooo`0000;oo`03003oo`0000;oo`04003ooooo0002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00@00?ooool000;oo`05
003oooooool00003ool01@00oooooooo00000ooo00<00?ooool00_oo0`000_oo00D00?ooooooo`00
00Goo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00?oo`03003ooooo
00;oo`03003ooooo00;oo`@000Ooo`03003ooooo04Ooo`03003ooooo00Goo`00;_oo00D00?oooooo
o`0000;oo`04003ooooo0002ool00`00ooooo`02ool20005ool01000ooooo`000_oo00@00?ooool0
00;oo`04003ooooo0002ool01000ooooo`000_oo00<00?oo00000_oo00@00?ooool000?oo`04003o
oooo0002ool01@00oooooooo00000_oo00@00?ooool000?oo`05003oooooool00002ool00`00oooo
o`02ool01@00oooooooo00000_oo0`000ooo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool0
1?oo00<00?ooool01?oo00<00?ooool01?oo00D00?ooooooo`0000;oo`<000Soo`<004[oo`03003o
oooo00Goo`00;?oo0`000ooo00@00?ooool000;oo`04003ooooo0004ool30005ool01000ooooo`00
0_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?oo
ool000;oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?ooool000?oo`05003oooooool0
0003ool01@00oooooooo00000ooo00<00?ooool00_oo1@001Ooo00<00?ooool00_oo00<00?ooool0
1?oo00<00?ooool01Ooo00<00?ooool01?oo00<00?ooool010002_oo00<00?ooool0Booo00<00?oo
ool01Ooo000^ool00`00ooooo`02ool20004ool20005ool00`00ooooo`06ool01000ooooo`000_oo
00<00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`00
0ooo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`000ooo00@00?ooool000?oo`05
003oooooool00004ool01@00oooooooo00000ooo0`000ooo00<00?ooool00_oo00<00?ooool01?oo
00<00?ooool01?oo00<00?ooool01?oo10003?oo0P00C_oo00<00?ooool01Ooo0010ool00`00oooo
o`04ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00003ool01P00ooooo`00ool0
00;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`04003ooooo0002ool01@00
oooooooo00000ooo00@00?ooool000Coo`05003oooooool00003ool00`00ooooo`02ool00`00ool0
00030004ool00`00ooooo`04ool00`00ooooo`04ool01000oooooooo0`0000?oo`0000002ooo0P00
D?oo00<00?ooool01Ooo0010ool00`00ooooo`05ool01P00ooooo`00ool000;oo`06003ooooo003o
o`000_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00@00?ooool000?oo`04003ooooo0002
ool01000ooooo`000ooo00@00?ooool000?oo`04003ooooo0003ool01@00oooooooo00000ooo00D0
0?ooooooo`0000Coo`03003ooooo00;oo`8000?oo`03003ooooo00?oo`03003ooooo00Coo`D000Co
o`03003ooooo00Woo`03003ooooo053oo`03003ooooo00Goo`00@?oo00<00?ooool01Ooo00@00?oo
ool000;oo`03003oo`0000;oo`06003ooooo003oo`000ooo00@00?ooool000;oo`03003oo`0000;o
o`04003ooooo0002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01000
ooooo`000ooo00D00?ooooooo`0000?oo`05003oooooool00004ool00`00ooooo`02ool00`00ool0
00030005ool01000oooooooo10002?oo00<00?ooool01ooo0P00Dooo00<00?ooool01Ooo0010ool0
0`00ooooo`03ool01000ooooo`000_oo00<00?oo00000_oo00@00?ooool000;oo`03003oo`0000;o
o`04003ooooo0002ool00`00ool00002ool01000ooooo`000ooo00@00?ooool000;oo`04003ooooo
0002ool01@00oooooooo00000ooo00@00?ooool000?oo`04003ooooo0003ool01@00oooooooo0000
1?oo00<00?ooool00_oo00<00?ooool00_oo0`000_oo1@000_oo00<00?ooool01_oo0P001ooo0P00
E_oo00<00?ooool01?oo0010ool00`00ooooo`04ool00`00ool00002ool01000ooooo`000_oo00<0
0?oo00000_oo00<00?oo00000ooo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool0
1000ooooo`000_oo00@00?ooool000?oo`04003ooooo0003ool01000ooooo`000ooo00D00?oooooo
o`0000;oo`05003oooooool00004ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`030006
ool00`00ooooo`05ool00`00ool00006ool00`00ooooo`1Fool00`00ooooo`04ool0043oo`03003o
oooo00Coo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00@0
0?ooool000;oo`03003oo`0000;oo`04003ooooo0003ool01000ooooo`000_oo00@00?ooool000;o
o`04003ooooo0003ool01@00oooooooo00000_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`05
003oooooool00004ool00`00ooooo`02ool01000oooooooo0P001Ooo00<00?ooool01Ooo00@00?oo
ool000Goo`03003ooooo05Ooo`03003ooooo00Coo`00@?oo00<00?ooool00_oo00H00?ooool00?oo
0002ool01000ooooo`000_oo00<00?oo00000_oo00<00?oo00000ooo00@00?ooool000;oo`03003o
o`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`05003oooooool00002
ool01@00oooooooo00000ooo00@00?ooool000?oo`05003oooooool00003ool00`00ooooo`02ool0
0`00ooooo`02ool01000oooooooo0P001Ooo00<00?ooool01?oo00D00?ooooooo`0000?oo`8005[o
o`03003ooooo00Coo`00@?oo00<00?ooool00_oo00L00?ooooooo`00ool00002ool00`00ool00002
ool01000ooooo`000_oo00<00?oo00000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo
0003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000ooo00@0
0?ooool000?oo`05003oooooool00003ool00`00ooooo`02ool01@00oooooooo00001?oo00<00?oo
ool01?oo00<00?ooool01?oo00D00?ooooooo`0000;oo`03003ooooo05[oo`03003ooooo00Coo`00
@?oo00<00?ooool00ooo00H00?ooool00?oo0002ool01000ooooo`000_oo00<00?oo00000_oo00<0
0?oo00000ooo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000ooo00D00?ooooooo`0000;oo`05003o
ooooool00003ool00`00ooooo`02ool01@00oooooooo00001?oo00<00?ooool00ooo00<00?ooool0
1?oo00<00?ooool00_oo0`00GOoo00<00?ooool01?oo0010ool00`00ooooo`03ool01`00oooooooo
003oo`0000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo
00<00?oo00000_oo00@00?ooool000?oo`03003oo`0000?oo`04003ooooo0002ool01000ooooo`00
0_oo00D00?ooooooo`0000;oo`05003oooooool00002ool01@00oooooooo00000ooo00<00?ooool0
0_oo00D00?ooooooo`0000Coo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00?oo`03003o
oooo05goo`03003ooooo00Coo`00@?oo00<00?ooool00_oo00<00?oo00000_oo00<00?oo00000_oo
00@00?ooool000;oo`03003oo`0000;oo`03003oo`0000?oo`06003ooooo003oo`000_oo00@00?oo
ool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool0
1000ooooo`000ooo00D00?ooooooo`0000;oo`05003oooooool00003ool00`00ooooo`02ool00`00
ooooo`02ool00`00ooooo`02ool00`00ooooo`04ool01000oooooooo0`00Gooo00<00?ooool01?oo
0010ool00`00ooooo`02ool00`00ool00003ool00`00ool00002ool00`00ool00002ool01000oooo
o`000_oo00<00?oo00000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0003ool00`00
ool00002ool01000ooooo`000ooo00<00?oo00000ooo00@00?ooool000?oo`04003ooooo0003ool0
1000ooooo`000ooo00D00?ooooooo`0000Coo`03003ooooo00;oo`03003ooooo00;oo`03003ooooo
00?oo`05003oooooool00002ool00`00ooooo`1Mool00`00ooooo`04ool0043oo`03003ooooo00?o
o`03003oo`0000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool00`00ool00003
ool01P00ooooo`00ool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04
003ooooo0002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`05003oooooool00002ool01@00
oooooooo00000ooo00<00?ooool00_oo00@00?ooooooo`8000?oo`03003ooooo00?oo`05003ooooo
ool00003ool00`00ooooo`1Mool00`00ooooo`04ool0043oo`03003ooooo00?oo`03003oo`0000?o
o`03003oo`0000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`00
0_oo00<00?oo00000_oo00@00?ooool000?oo`03003oo`0000;oo`04003ooooo0002ool01000oooo
o`000_oo00@00?ooool000?oo`05003oooooool00002ool01@00oooooooo00000_oo00D00?oooooo
o`0000Coo`04003oooooool20003ool00`00ooooo`03ool01000ooooo`001?oo00<00?ooool0GOoo
00<00?ooool01?oo0010ool01@00oooooooo00000_oo00<00?oo00000_oo00<00?oo00000_oo00@0
0?ooool000;oo`03003oo`0000;oo`03003oo`0000?oo`06003ooooo003oo`000_oo00<00?oo0000
0_oo00L00?ooooooo`00ool00003ool00`00ool00003ool00`00ool00003ool00`00ool00003ool0
1@00oooooooo00000_oo00D00?ooooooo`0000;oo`05003oooooool00004ool01000oooooooo0P00
0ooo00<00?ooool00_oo00@00?ooool000Goo`03003ooooo05goo`03003ooooo00Coo`00@?oo00<0
0?ooool00_oo00D00?oo003oo`0000?oo`03003oo`0000;oo`03003oo`0000;oo`04003ooooo0002
ool00`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo0002ool01000ooooo`000_oo00H0
0?ooool00?oo0003ool00`00ool00003ool01000ooooo`000_oo00D00?ooooooo`0000;oo`05003o
ooooool00003ool01000ooooo`000ooo00<00?ooool00_oo0P000ooo00D00?ooooooo`0000;oo`03
003ooooo00Coo`03003ooooo05goo`03003ooooo00Coo`00@?oo00<00?ooool00_oo00H00?ooool0
0?oo0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00<00?oo00000_oo00@0
0?ooool000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000ooo00D00?ooooooo`0000;o
o`05003oooooool00004ool20002ool00`00ooooo`02ool30007ool00`00ooooo`1Mool00`00oooo
o`04ool003ooo`05003oooooool00002ool01@00ool00?oo00000ooo00<00?oo00000_oo00<00?oo
00000_oo00@00?ooool000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00H00?ooool00?oo
0003ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool01000
ooooo`000_oo00D00?ooooooo`0000?oo`04003ooooo0003ool01`00oooooooo003oo`0000;oo`04
003oooooool20009ool00`00ooooo`1Mool00`00ooooo`04ool003ooo`05003oooooool00003ool0
1@00ool00?oo00000_oo00<00?oo00000_oo00@00?ooool000;oo`03003oo`0000;oo`03003oo`00
00;oo`04003ooooo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000ooo00<00?oo0000
0ooo00<00?oo00000_oo00@00?ooool000?oo`04003ooooo0002ool01@00oooooooo00000ooo00@0
0?ooool000?oo`07003oooooool00?oo00000_oo00<00?ooool00P002ooo00<00?ooool0G?oo00<0
0?ooool01?oo000oool00`00ooooo`02ool02000ooooo`00ool00?oo0003ool00`00ool00002ool0
0`00ool00002ool00`00ool00003ool01@00ool00?oo00000_oo00@00?ooool000;oo`03003oo`00
00;oo`04003ooooo0002ool01000ooooo`000_oo00H00?ooool00?oo0002ool01000ooooo`000ooo
00L00?ooooooo`00ool00003ool01@00oooooooo00000_oo00D00?ooooooo`0000?oo`05003oo`00
ool00003ool00`00ooooo`0:ool00`00ooooo`1Lool00`00ooooo`04ool003ooo`03003ooooo00;o
o`09003oooooool00?oo003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00H00?oo
ool00?oo0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000ooo00<00?oo00000_oo
00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00L00?ooooooo`00ool00003ool0
1@00oooooooo00000_oo00D00?ooooooo`0000?oo`06003oo`00ool00?oo0P003Ooo00<00?ooool0
G?oo00<00?ooool01?oo000oool01`00oooooooo003oo`0000;oo`05003oo`00ool00003ool01@00
ool00?oo00000_oo00H00?ooool00?oo0003ool01@00ool00?oo00000_oo00@00?ooool000;oo`03
003oo`0000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000oooo
o`000_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01000ooooo`000ooo00H00?ooooooo`00
ool200001?oo003oo`003Ooo00<00?ooool0GOoo00<00?ooool00ooo000oool01`00oooooooo003o
o`0000?oo`05003oo`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool00003ool0
0`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool0
0002ool01`00oooooooo003oo`0000;oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?oo
ool000?oo`04003ooooo0003ool01P00oooooooo003oo`<00003ool00?oo00goo`03003ooooo05go
o`03003ooooo00?oo`00:ooo10000ooo0P001?oo0P001Ooo00<00?ooool00_oo00<00?oo00000_oo
00D00?oo003oo`0000?oo`05003oo`00ool00002ool01P00ooooo`00ool000;oo`08003ooooo003o
o`00ool000?oo`03003oo`0000;oo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo
00<00?oo00000_oo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`000ooo00@00?oo
ool000?oo`04003ooooo0002ool200000ooo003oo`0>ool00`00ooooo`1Mool00`00ooooo`03ool0
02_oo`03003ooooo00?oo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool00_oo00<00?oo
00000_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool00`00ool00003ool0
1@00ool00?oo00000_oo00@00?ooool000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00@0
0?ooool000;oo`06003ooooo003oo`000ooo00@00?ooool000;oo`04003ooooo0002ool01@00oooo
oooo00000_oo00D00?ooooooo`0000;oo`03003oo`0000800003ool00?oo00koo`03003ooooo05go
o`03003ooooo00?oo`00;?oo00<00?ooool00_oo00@00?ooool000;oo`04003ooooo0004ool00`00
ooooo`03ool00`00ool00002ool01@00ool00?oo00000ooo00D00?oo003oo`0000;oo`03003oo`00
00;oo`06003ooooo003oo`000_oo00<00?oo00000ooo00D00?oo003oo`0000;oo`06003ooooo003o
o`000_oo00@00?ooool000;oo`06003ooooo003oo`000ooo00@00?ooool000;oo`04003ooooo0002
ool01@00oooooooo00000_oo00D00?ooooooo`0000;oo`800004ool00000000@ool00`00ooooo`1M
ool00`00ooooo`03ool002goo`05003oooooool00002ool01000ooooo`000_oo00<00?ooool00_oo
00@00?ooool000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo
00000_oo00<00?oo00000_oo00P00?ooool00?oo003oo`000ooo00<00?oo00000_oo00<00?oo0000
0_oo00H00?ooool00?oo0003ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000;oo`04
003ooooo0002ool01@00oooooooo00000_oo00D00?ooooooo`0000;oo`06003ooooo003oo`004?oo
00<00?ooool0GOoo00<00?ooool00ooo000^ool01000ooooo`000_oo00@00?ooool000;oo`03003o
oooo00;oo`04003ooooo0002ool00`00ool00003ool01@00ool00?oo00000ooo00D00?oo003oo`00
00;oo`05003oo`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo0002ool01@00ool00?oo
00000_oo00H00?ooool00?oo0003ool01P00ooooo`00ool000;oo`04003ooooo0002ool01000oooo
o`000ooo00<00?oo00000ooo00@00?ooool000;oo`05003oooooool00002ool2000Aool00`00oooo
o`1Mool00`00ooooo`03ool002_oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000Co
o`8000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool01@00ool0
0?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00ool000;oo`06003o
oooo003oo`000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`00
0ooo00<00?oo00000ooo00@00?ooool000;oo`06003oooooool00?oo0`004Ooo00<00?ooool0GOoo
00<00?ooool00ooo000/ool20004ool20004ool20005ool01@00oooooooo00000_oo00<00?oo0000
0_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`05003oo`00ool00002ool00`00ool0
0002ool00`00ool00003ool00`00ool00002ool00`00ool00002ool00`00ool00002ool01000oooo
o`000_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo
0002ool01000ooooo`000ooo00@00?oo000001;oo`03003ooooo05goo`03003ooooo00?oo`00?ooo
00P00?ooool00?oo003oo`000_oo00P00?ooool00?oo003oo`000ooo00D00?oo003oo`0000;oo`05
003oo`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo
00H00?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`03003oo`0000;oo`05003o
ooooool00002ool01000ooooo`000_oo00@00?ooool000;oo`03003ooooo008001;oo`03003ooooo
05goo`03003ooooo00?oo`00?ooo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002
ool01P00ooooo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool0
1000ooooo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000ooo00<00?oo00000_oo
00@00?ooool000;oo`03003oo`0000;oo`05003oooooool00002ool01000ooooo`000_oo00@00?oo
ool000;oo`03003oo`00008001;oo`03003ooooo05goo`03003ooooo00?oo`00?ooo00T00?oooooo
o`00ool00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool01@00
ool00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool01P00oooo
o`00ool000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000ooo
00@00?ooool000;oo`04003ooooo0002ool02000ooooo`00ool00?oo000Bool00`00ooooo`1Mool0
0`00ooooo`03ool001Coo`03003ooooo02Soo`07003oooooool00?oo00000_oo00<00?oo00000_oo
00D00?oo003oo`0000?oo`05003oo`00ool00002ool00`00ool00002ool00`00ool00002ool01@00
ool00?oo00000ooo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00@00?ooool000;o
o`03003oo`0000;oo`04003ooooo0002ool00`00ool00003ool01000ooooo`000_oo00@00?ooool0
00;oo`08003ooooo003oo`00ool001;oo`03003ooooo05goo`03003ooooo00?oo`000_oo1@000_oo
10002?oo00@00?ooooooo`8000;oo`800003ool00000008001_oo`05003oo`00ool00002ool01@00
ool00?oo00000_oo00<00?oo00000_oo00P00?ooool00?oo003oo`000_oo00H00?ooool00?oo0002
ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00ool0
00;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`000_oo0`0000?oo`00ool04Ooo00<00?ooool0GOoo00<00?ooool00ooo
0003ool00`00ool00002ool01P00ooooo`00ool000Ooo`03003ooooo00;oo`04003ooooo0002ool0
1000ooooo`006ooo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`00
00;oo`08003ooooo003oo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;o
o`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00@00?ooool000;oo`06003o
oooo003oo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool0
0`0000?oo`00ool04Ooo00<00?ooool0GOoo00<00?ooool00ooo0003ool00`00ool00002ool01P00
ooooo`00ool000Ooo`03003ooooo00;oo`04003ooooo0003ool00`00ooooo`0Jool01`00oooooooo
003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool00002
ool00`00ool00002ool00`00ool00002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool0
0`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`020002ool00`00ooooo`0@ool0
0`00ooooo`1Mool00`00ooooo`03ool000Coo`05003oooooool00004ool00`00ooooo`06ool01000
oooooooo10001?oo00<00?ooool06Ooo00L00?ooooooo`00ool00002ool00`00ool00002ool01@00
ool00?oo00000_oo00P00?ooool00?oo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<0
0?oo00000_oo00P00?ooool00?oo003oo`000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool0
00;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00<00?ooool00P000_oo00<00?ooool04?oo00<00?ooool0GOoo00<00?ooool00ooo0004ool00`00
ooooo`02ool01@00oooooooo00002?oo00D00?ooooooo`0000;oo`03003ooooo00@00003ool00000
008001Goo`05003oooooool00002ool00`00ool00002ool00`00ool00002ool01@00ool00?oo0000
0_oo00P00?ooool00?oo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo
00D00?oo003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01P00ooooo`00ool000;oo`03
003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`05003oo`00ool0
0002ool00`00ooooo`0@ool00`00ooooo`1Mool00`00ooooo`03ool000Coo`03003ooooo00;oo`05
003oo`00003oo`040005ool01000ool000000_oo0P001ooo0P005_oo00<00?ooool00_oo00<00?oo
00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002
ool00`00ool00002ool00`00ool00002ool00`00ool00002ool02000ooooo`00ool00?oo0002ool0
1P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00@00?ooool000;oo`04
003ooooo0002ool01000ooooo`000_oo00@00?oo000000?oo`03003ooooo013oo`03003ooooo05go
o`03003ooooo00?oo`002Ooo0P0000Coo`00ooooo`8001Ooo`03003ooooo01?oo`08003ooooo003o
o`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002
ool00`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool00002
ool00`00ool00002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00<00?oo0000
0_oo00@00?ooool000;oo`04003ooooo0002ool00`00ool00002ool20003ool00`00ooooo`0@ool0
0`00ooooo`1Nool00`00ooooo`02ool0017oo`03003ooooo01;oo`<001Goo`06003ooooo003oo`00
0_oo00<00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003ooooo008000;oo`08
003ooooo003oo`00ool000;oo`03003oo`0000;oo`08003ooooo003oo`00ool000;oo`03003oo`00
00;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`03003oo`0000;oo`03003oo`0000;oo`04
003ooooo0002ool01P00ooooo`00ool000;oo`8000?oo`03003ooooo013oo`03003ooooo05koo`03
003ooooo00;oo`003ooo0`00;?oo00/00?ooooooo`00ool00?oo003oo`0000;oo`08003ooooo003o
o`00ool000;oo`03003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool00`00ool00002
ool00`00ool00002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool0
1000ooooo`000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00<00?oo
00000_oo0P000ooo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000nool02@00oooo
oooo003oo`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool00002ool01@00ool0
0?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`03003ooooo0080
00;oo`04003ooooo0002ool00`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`04
003ooooo0002ool00`00ool00002ool01P00ooooo`00ool000;oo`8000?oo`03003ooooo013oo`03
003ooooo05koo`03003ooooo00;oo`00?_oo00L00?ooooooo`00ool00002ool00`00ool00002ool0
0`00ool00002ool00`00ool00002ool00`00ool00002ool20002ool00`00ool00002ool00`00ool0
0002ool00`00ool00002ool00`00ool00002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002
ool01P00ooooo`00ool000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01P00
ooooo`00ool000;oo`8000?oo`03003ooooo013oo`03003ooooo05koo`03003ooooo00;oo`00?_oo
00<00?oo00000_oo00L00?oo003oo`00ool00002ool00`00ool00002ool00`00ool00002ool00`00
ool00002ool01@00ool00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`08003ooooo
003oo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo
00000_oo00@00?ooool000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?ooool00_oo
00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000nool00`00ool00002ool01@00ool0
0?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00L00?oo003oo`00ool0
0002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool01@00ool00?oo00000_oo00<00?oo
00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool01000ooooo`000_oo
00<00?oo00000_oo00P00?ooool00?oo003oo`001Ooo00<00?ooool04?oo00<00?ooool0G_oo00<0
0?ooool00_oo000nool02000ooooo`00ool00?oo0002ool01@00ool00?oo00000_oo00<00?oo0000
0_oo00<00?oo00000_oo00<00?ooool00P000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03
003oo`0000;oo`05003oo`00ool00002ool02000ooooo`00ool00?oo0002ool01P00ooooo`00ool0
00;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool02000ooooo`00ool00?oo0005
ool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool003koo`04003ooooo0002ool01`00
ool00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`8000;oo`08
003ooooo003oo`00ool000;oo`05003oo`00ool00002ool00`00ooooo`020002ool01000ooooo`00
0_oo0P000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo
00P00?ooool00?oo003oo`001Ooo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000n
ool01000ooooo`000_oo00D00?oo003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool0
0`00ool00002ool01@00ool00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`03003o
o`0000;oo`05003oo`00ool00002ool02000ooooo`00ool00?oo0002ool01P00ooooo`00ool000;o
o`03003oo`0000;oo`06003ooooo003oo`000_oo00P00?ooool00?oo003oo`001Ooo00<00?ooool0
4?oo00<00?ooool0G_oo00<00?ooool00_oo000nool20002ool01@00ool00?oo00000_oo00D00?oo
003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`07003oo`00ool00?oo00000_oo00<00?oo
ool00P000_oo00H00?ooool00?oo0002ool01`00ool00?oo003oo`0000;oo`06003ooooo003oo`00
0_oo00<00?oo00000_oo00P00?ooool00?oo003oo`000_oo00H00?ooool00?oo0002ool02000oooo
o`00ool00?oo0005ool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool003koo`8000;o
o`03003oo`0000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00<00?oo00000_oo00<0
0?ooool00P000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00
ool00002ool00`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`05003oo`00ool0
0002ool01000ooooo`000_oo00<00?oo00000_oo00D00?oo003oo`0000Goo`03003ooooo013oo`03
003ooooo05koo`03003ooooo00;oo`00?_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`8000;o
o`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`8000;oo`08003ooooo003oo`00ool0
00;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo
00H00?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`05
003oo`00ool00005ool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool003koo`05003o
o`00ool00002ool00`00ool00002ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`03003o
o`0000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00D00?oo003oo`0000;oo`05003o
o`00ool00002ool02000ooooo`00ool00?oo0002ool01P00ooooo`00ool000;oo`03003oo`0000;o
o`06003ooooo003oo`000_oo00<00?oo00000_oo00@00?oo000000Koo`03003ooooo013oo`03003o
oooo05koo`03003ooooo00;oo`00?_oo00P00?ooool00?oo003oo`000_oo00D00?oo003oo`0000;o
o`05003oo`00ool00002ool00`00ool00002ool01`00ool00?oo003oo`0000;oo`03003ooooo0080
00;oo`0:003ooooo003oo`00ool00?oo0002ool00`00ool00002ool01@00ool00?oo00000_oo00H0
0?ooool00?oo0002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`04003o
o`000006ool00`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool002[oo`8000Coo`8000Co
o`8000Koo`800004ool00?oo0002ool00`00ool00002ool20002ool00`00ool00002ool01@00ool0
0?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool01@00ool0
0?oo00000_oo00D00?oo003oo`0000;oo`03003ooooo008000;oo`06003ooooo003oo`000_oo00H0
0?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool00`00ool00007ool00`00ooooo`0@
ool00`00ooooo`1Nool00`00ooooo`02ool002Woo`04003ooooo0002ool01000ooooo`000_oo00@0
0?ooool000Coo`03003oo`0000;oo`8000;oo`03003oo`0000;oo`07003oo`00ool00?oo00000_oo
00<00?oo00000_oo00<00?oo00000_oo0P000_oo00P00?ooool00?oo003oo`000_oo00D00?oo003o
o`0000;oo`07003oo`00ool00?oo00000_oo00P00?ooool00?oo003oo`000_oo00<00?oo00000_oo
00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool00`00ool00007ool00`00oooo
o`0@ool00`00ooooo`1Nool00`00ooooo`02ool002coo`04003ooooo0002ool01000ooooo`000_oo
00<00?ooool00_oo00X00?ooool00?oo003oo`00ool000;oo`05003oo`00ool00002ool01@00ool0
0?oo00000_oo00D00?oo003oo`0000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00D0
0?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00P00?ooool00?oo003oo`000_oo00<00?oo
00000_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool00`00ool00007ool0
0`00ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool002[oo`8000?oo`04003ooooo0002ool0
1000ooooo`001?oo00@00?ooool000;oo`8000;oo`03003oo`0000;oo`8000;oo`05003oo`00ool0
0002ool00`00ool00002ool01`00ool00?oo003oo`0000;oo`03003ooooo008000;oo`03003ooooo
008000;oo`03003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool01@00ool00?oo0000
0_oo00H00?ooool00?oo0002ool00`00ool00002ool00`00ool00002ool00`00ool00007ool00`00
ooooo`0@ool00`00ooooo`1Nool00`00ooooo`02ool002coo`04003ooooo0002ool01000ooooo`00
0_oo00<00?ooool00_oo10000_oo0P000_oo00<00?oo00000_oo00L00?oo003oo`00ool00002ool0
1@00ool00?oo00000_oo00L00?oo003oo`00ool00002ool00`00ooooo`020002ool02P00ooooo`00
ool00?oo003oo`000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003ooooo008000;oo`06
003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00001ooo00<00?ooool0
4?oo00<00?ooool0Gooo00<00?ooool00Ooo000Yool01000ooooo`000_oo00@00?ooool000;oo`04
003ooooo0004ool20002ool01`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool20002ool0
0`00ool00002ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool0
1`00ool00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool01P00ooooo`00ool0
00;oo`05003oo`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool00007ool00`00
ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool002[oo`8000Coo`8000Coo`8000Goo`8000;o
o`05003oo`00ool00002ool00`00ool00002ool20002ool01@00ool00?oo00000_oo00D00?oo003o
o`0000;oo`07003oo`00ool00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`07003o
o`00ool00?oo00000_oo00D00?oo003oo`0000;oo`06003ooooo003oo`000_oo00D00?oo003oo`00
00;oo`03003oo`0000;oo`03003oo`0000;oo`8000Soo`03003ooooo013oo`03003ooooo05ooo`03
003ooooo007oo`00?Ooo00D00?oo003oo`0000;oo`8000;oo`03003oo`0000;oo`8000;oo`03003o
o`0000;oo`05003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool0
1@00ool00?oo00000_oo00L00?oo003oo`00ool00002ool00`00ool00002ool00`00ool00002ool0
0`00ool00002ool01@00ool00?oo00000_oo00P00?ooool00?oo003oo`000_oo0P002?oo00<00?oo
ool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool01@00ool00?oo00000_oo0P000_oo00`0
0?ooool00?oo003oo`00ool00?oo0002ool01@00ool00?oo00000_oo00L00?oo003oo`00ool00002
ool01@00ool00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`8000;oo`03003oo`00
00;oo`03003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00003ool01`00ool00?oo003oo`00
00;oo`8000Soo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00?Ooo00<00?oo0000
0_oo00L00?oo003oo`00ool00002ool00`00ooooo`020002ool20002ool03000ooooo`00ool00?oo
003oo`00ool000;oo`03003oo`0000;oo`8000;oo`03003ooooo008000;oo`03003oo`0000;oo`07
003oo`00ool00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool0
1`00ool00?oo003oo`0000;oo`03003ooooo00Ooo`03003ooooo013oo`03003ooooo05ooo`03003o
oooo007oo`00?Ooo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`8000;oo`05
003oo`00ool00002ool01@00ool00?oo00000_oo00L00?oo003oo`00ool00002ool20002ool02P00
ooooo`00ool00?oo003oo`000_oo00<00?oo00000_oo00L00?oo003oo`00ool00002ool00`00ool0
0002ool01P00ooooo`00ool000;oo`07003oo`00ool00?oo00000_oo00<00?ooool01ooo00<00?oo
ool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool01P00ooooo`00ool000;oo`8000;oo`03
003oo`0000;oo`07003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`09003oo`00ool00?oo
003oo`0000;oo`05003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`05
003oo`00ool00002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool01@00ool00?oo0000
0_oo00<00?oo00002Ooo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool20000
1?oo003oo`000_oo0P000_oo00P00?ooool00?oo003oo`000_oo0P000_oo00D00?oo003oo`0000;o
o`09003oo`00ool00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool20002ool0
0`00ool00002ool01@00ool00?oo00000_oo00<00?ooool00P000_oo00H00?ooool00?oo0002ool0
1@00ool00?oo00000_oo0P002_oo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000m
ool200002ooo003ooooo003oo`00ool00?oo00000_oo00<00?ooool00P000_oo0P000_oo00h00?oo
ool00?oo003oo`00ool00?oo003oo`000_oo00D00?oo003oo`0000;oo`05003oo`00ool00002ool0
1`00ool00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`05003o
o`00ool00002ool01@00ool00?oo00000_oo0P002_oo00<00?ooool04?oo00<00?ooool0Gooo00<0
0?ooool00Ooo000mool01000ool000000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`8000;o
o`05003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`09003oo`00ool00?oo003oo`0000;o
o`05003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;o
o`03003oo`0000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo0P002_oo00<00?ooool0
4?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool01`00ool00?oo003oo`0000;oo`8000;oo`03
003oo`0000;oo`07003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`09003oo`00ool00?oo
003oo`0000;oo`8000;oo`05003oo`00ool00002ool00`00ool00002ool20002ool00`00ool00002
ool00`00ool00002ool00`00ool00002ool01@00ool00?oo00000_oo00D00?oo003oo`0000;oo`80
00[oo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00?Ooo00L00?oo003oo`00ool0
0002ool20002ool02000ooooo`00ool00?oo0002ool20002ool01@00ool00?oo00000_oo00T00?oo
003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`8000;oo`03003oo`0000;oo`07003oo`00
ool00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`05003oo`00
ool00002ool00`00ooooo`09ool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool003go
o`07003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`05003oo`00ool00002ool20002ool0
4000ooooo`00ool00?oo003oo`00ool00?oo003oo`000_oo00<00?oo00000_oo00T00?oo003oo`00
ool00?oo00000_oo00<00?oo00000_oo0P000_oo00<00?oo00000_oo00<00?oo00000_oo00D00?oo
003oo`0000;oo`05003oo`00ool00002ool00`00ooooo`09ool00`00ooooo`0@ool00`00ooooo`1O
ool00`00ooooo`01ool003goo`03003ooooo008000;oo`05003oo`00ool00002ool00`00ool00002
ool20002ool01@00ool00?oo00000_oo00L00?oo003oo`00ool00002ool200001ooo003ooooo003o
o`0000;oo`05003oo`00ool00002ool20002ool00`00ool00002ool01`00ool00?oo003oo`0000;o
o`03003oo`0000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00<00?ooool02Ooo00<0
0?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool02000ooooo`00ool00?oo0002ool2
0002ool00`00ool00002ool20002ool00`00ool00002ool01@00ool00?oo00000_oo0P000_oo00T0
0?oo003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00L0
0?oo003oo`00ool00002ool00`00ool00002ool01@00ool00?oo00000_oo00D00?oo003oo`0000;o
o`03003ooooo00Woo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00??oo0P000_oo
00D00?oo003oo`0000;oo`8000;oo`03003oo`0000;oo`07003oo`00ool00?oo00000_oo00D00?oo
003oo`0000;oo`09003oo`00ool00?oo003oo`0000;oo`8000;oo`05003oo`00ool00002ool01`00
ool00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`00
00;oo`05003oo`00ool00002ool00`00ooooo`09ool00`00ooooo`0@ool00`00ooooo`1Oool00`00
ooooo`01ool003coo`09003oo`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool01@00ool0
0?oo00000_oo0P000_oo00<00?ooool00P000_oo00T00?oo003oo`00ool00?oo00000_oo00T00?oo
003oo`00ool00?oo00000_oo00L00?oo003oo`00ool00002ool00`00ool00002ool00`00ool00002
ool02@00ool00?oo003oo`00ool00002ool01@00ool00?oo00000_oo00<00?ooool02Ooo00<00?oo
ool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000lool00`00ool00002ool20002ool01@00ool0
0?oo00000_oo00L00?oo003oo`00ool00002ool20002ool02P00ooooo`00ool00?oo003oo`000_oo
0P0000Ooo`00ooooo`00ool00002ool01000ool000000_oo00L00?oo003oo`00ool00002ool00`00
ool00002ool00`00ool00002ool01`00ool00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo
00<00?ooool02Ooo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000lool00`00ool0
0002ool01@00ool00?oo00000_oo0P000_oo00L00?oo003oo`00ool00002ool01@00ool00?oo0000
0_oo00L00?oo003oo`00ool00002ool01000ool000000_oo00<00?oo00000_oo00T00?oo003oo`00
ool00?oo00000_oo0P000_oo00<00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`09
003oo`00ool00?oo003oo`0000koo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00??oo
00<00?oo00000_oo00D00?oo003oo`0000;oo`8000;oo`03003oo`0000;oo`8000;oo`05003oo`00
ool00002ool01`00ool00?oo003oo`0000;oo`04003oo`000002ool00`00ool00002ool01@00ool0
0?oo00000_oo0P000_oo00L00?oo003oo`00ool00002ool00`00ool00002ool01@00ool00?oo0000
0_oo00L00?oo003oo`00ool00002ool00`00ooooo`0;ool00`00ooooo`0@ool00`00ooooo`1Pool1
0001ool1ool003coo`800008ool00?oo003oo`00ool000;oo`05003oo`00ool00002ool02@00ool0
0?oo003oo`00ool00002ool01@00ool00?oo00000_oo0P000_oo00T00?oo003oo`00ool00?oo0000
0_oo00D00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo0P000_oo00<00?oo00000_oo00<0
0?oo00000_oo00T00?oo003oo`00ool00?oo00000_oo00<00?ooool02ooo00<00?ooool04?oo00<0
0?ooool0H?oo0@000Ooo0Ooo000lool200003Ooo003ooooo003oo`00ool00?oo003oo`0000;oo`07
003oo`00ool00?oo00000_oo0P000_oo00D00?oo003oo`0000;oo`09003oo`00ool00?oo003oo`00
00;oo`8000;oo`05003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`07003oo`00ool00?oo
00000_oo00<00?oo00000_oo00T00?oo003oo`00ool00?oo00000_oo00<00?ooool02ooo00<00?oo
ool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool01000ool000000_oo00T00?oo003oo`00ool0
0?oo00000_oo00L00?oo003oo`00ool00002ool20002ool00`00ooooo`020002ool02@00ool00?oo
003oo`00ool00002ool02@00ool00?oo003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`07
003oo`00ool00?oo00000_oo00<00?oo00000_oo00T00?oo003oo`00ool00?oo00000_oo00<00?oo
ool02ooo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool02`00ool00?oo003oo`00
ool00?oo00000_oo0P000_oo00L00?oo003oo`00ool00002ool01@00ool00?oo00000_oo00L00?oo
003oo`00ool00002ool200001ooo003ooooo003oo`0000;oo`80000;ool00?ooool00?oo003oo`00
ool00002ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`07003oo`00ool00?oo00000_oo
00<00?oo00003Ooo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool01@00ool00?oo
00000_oo0P0000_oo`00ooooo`00ool00?oo003oo`0000;oo`8000;oo`05003oo`00ool00002ool0
1`00ool00?oo003oo`0000;oo`04003oo`000002ool00`00ool00002ool01000ool000000_oo00L0
0?oo003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`0<003ooooo003oo`00ool00?oo003o
o`000_oo00<00?ooool03Ooo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool01@00
ool00?oo00000_oo00/00?oo003oo`00ool00?oo003oo`0000;oo`0;003oo`00ool00?oo003oo`00
ool00002ool01`00ool00?oo003oo`0000;oo`04003oo`000002ool00`00ool00002ool01@00ool0
0?oo00000_oo0P0000Coo`00ool000;oo`07003oo`00ool00?oo00000ooo00/00?oo003oo`00ool0
0?oo003oo`0000;oo`03003ooooo00goo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00
??oo00<00?ooool00P000_oo00T00?oo003oo`00ool00?oo00000_oo00L00?oo003oo`00ool00002
ool20002ool01@00ool00?oo00000_oo0P000_oo00T00?oo003oo`00ool00?oo00000_oo00D00?oo
003oo`0000;oo`04003oo`00ool20002ool00`00ool00002ool00`00ool00002ool02`00ool00?oo
003oo`00ool00?oo00000_oo00<00?ooool03Ooo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo
0Ooo000Zool00`00ooooo`02ool20004ool20005ool200002_oo003oo`00ool00?oo003oo`000_oo
0P000_oo00L00?oo003oo`00ool00002ool20002ool01@00ool00?oo00000_oo00T00?oo003oo`00
ool00?oo00000_oo0P000_oo00D00?oo003oo`0000;oo`0;003oo`00ool00?oo003oo`00ool00002
ool00`00ool00002ool02@00ool00?oo003oo`00ool00002ool00`00ool0000?ool00`00ooooo`0@
ool00`00ooooo`1Pool10001ool1ool002[oo`05003oooooool00002ool01000ooooo`000_oo00<0
0?ooool00_oo0P0000Coo`00ool000;oo`80000?ool00?ooool00?oo003oo`00ool00?oo003oo`00
00;oo`05003oo`00ool00002ool20002ool02@00ool00?oo003oo`00ool00002ool02@00ool00?oo
003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00T00?oo
003oo`00ool00?oo00000_oo00<00?oo00003ooo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo
0Ooo000Wool50002ool01000ooooo`000_oo00@00?ooool000Coo`8000;oo`8000;oo`04003oo`00
0002ool01`00ool00?oo003oo`0000;oo`8000;oo`05003oo`00ool00002ool03@00ool00?oo003o
o`00ool00?oo003oo`0000;oo`09003oo`00ool00?oo003oo`0000;oo`07003oo`00ool00?oo0000
0_oo00L00?oo003oo`00ool00002ool02@00ool00?oo003oo`00ool00002ool00`00ooooo`0?ool0
0`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool002Ooo`04003ooooo0003ool01000ooooo`00
0_oo00@00?ooool000Coo`8000;oo`0=003oo`00ool00?oo003oo`00ool00?oo00000_oo00@00?oo
003oo`8000;oo`05003oo`00ool00002ool02@00ool00?oo003oo`00ool00002ool20002ool00`00
ool00002ool200002ooo003ooooo003oo`00ool00?oo00000_oo00L00?oo003oo`00ool00002ool0
2@00ool00?oo003oo`00ool00002ool00`00ooooo`0?ool00`00ooooo`0@ool00`00ooooo`1Pool1
0001ool1ool002Soo`03003oo`0000?oo`04003ooooo0002ool01000ooooo`001?oo00d00?oo003o
o`00ool00?oo003oo`00ool00002ool20002ool03@00ool00?oo003oo`00ool00?oo003oo`0000;o
o`07003oo`00ool00?oo00000_oo00@00?oo000000;oo`03003oo`0000;oo`80000;ool00?ooool0
0?oo003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00<0
0?oo00004Ooo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000Yool20003ool01000oooo
o`000_oo00@00?ooool000Coo`07003oo`00ool00?oo00000_oo0P0000ooo`00ooooo`00ool00?oo
003oo`00ool00?oo00000_oo0P000_oo00T00?oo003oo`00ool00?oo00000_oo00T00?oo003oo`00
ool00?oo00000_oo00@00?oo000000;oo`07003oo`00ool00?oo00000_oo00L00?oo003oo`00ool0
0002ool01`00ool00?oo003oo`0000;oo`03003oo`00017oo`03003ooooo013oo`03003ooooo063o
o`40007oo`7oo`00:_oo00<00?ooool00_oo0P001?oo0P001Ooo00<00?ooool00_oo0P000_oo00@0
0?oo000000;oo`0;003oo`00ool00?oo003oo`00ool00002ool02@00ool00?oo003oo`00ool00002
ool20002ool02@00ool00?oo003oo`00ool00002ool03@00ool00?oo003oo`00ool00?oo003oo`00
00;oo`07003oo`00ool00?oo00000_oo00L00?oo003oo`00ool00002ool00`00ool0000Aool00`00
ooooo`0@ool00`00ooooo`1Pool10001ool1ool003_oo`03003ooooo00?oo`0=003oo`00ool00?oo
003oo`00ool00?oo00000_oo00@00?oo003oo`8000;oo`05003oo`00ool00002ool20002ool02@00
ool00?oo003oo`00ool00002ool20002ool01@00ool00?oo00000_oo0P0000Coo`00ool000;oo`07
003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0001?oo`03003ooooo013oo`03
003ooooo063oo`40007oo`7oo`00>ooo00<00?ooool01Ooo00L00?oo003oo`00ool00002ool20002
ool01000ool00?oo0P000_oo00D00?oo003oo`0000;oo`0=003oo`00ool00?oo003oo`00ool00?oo
00000_oo00T00?oo003oo`00ool00?oo00000_oo00l00?oo003oo`00ool00?oo003oo`00ool00?oo
00000_oo00D00?oo003oo`0000;oo`03003oo`0001?oo`03003ooooo013oo`03003ooooo063oo`40
007oo`7oo`00>ooo00<00?ooool02?oo0P0001Goo`00ooooo`00ool00?oo003oo`00ool00?oo003o
o`00ool00?oo00000_oo00T00?oo003oo`00ool00?oo00000_oo0P000_oo00T00?oo003oo`00ool0
0?oo00000_oo00d00?oo003oo`00ool00?oo003oo`00ool00002ool01`00ool00?oo003oo`0000;o
o`03003oo`0001?oo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo00<00?ooool02?oo
00@00?oo000000;oo`0;003oo`00ool00?oo003oo`00ool00002ool200003Ooo003ooooo003oo`00
ool00?oo003oo`0000;oo`8000;oo`09003oo`00ool00?oo003oo`0000;oo`0=003oo`00ool00?oo
003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`05003oo`00ool0000Cool00`00ooooo`0@
ool00`00ooooo`1Qool10001ool003_oo`03003ooooo00[oo`07003oo`00ool00?oo00000_oo0P00
00Coo`00ool000;oo`0?003oo`00ool00?oo003oo`00ool00?oo003oo`0000;oo`0=003oo`00ool0
0?oo003oo`00ool00?oo00000_oo00L00?oo003oo`00ool00002ool02`00ool00?oo003oo`00ool0
0?oo00000_oo00D00?oo003oo`0001?oo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo
00<00?ooool03?oo00<00?ooool00P000_oo00H00?oo00000?oo0002ool01@00ool00?oo00000_oo
0P000_oo0P000_oo00T00?oo003oo`00ool00?oo00000_oo0P0000_oo`00ooooo`00ool00?oo003o
o`0000;oo`09003oo`00ool00?oo003oo`0000;oo`05003oo`00ool0000Eool00`00ooooo`0@ool0
0`00ooooo`1Qool10001ool003_oo`03003ooooo00coo`0;003ooooo003oo`00ool00?oo003oo`02
0002ool01@00ool00?oo00000_oo01400?oo003oo`00ool00?oo003oo`00ool00?oo003oo`0000;o
o`04003oo`000002ool01`00ool00?oo003oo`0000;oo`09003oo`00ool00?oo003oo`0000;oo`05
003oo`00ool0000Eool00`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03003ooooo00oo
o`0A003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00002ool04@00ool00?oo003oo`00ool0
0?oo003oo`00ool00?oo00000_oo00d00?oo003oo`00ool00?oo003oo`00ool00002ool01`00ool0
0?oo003oo`0000;oo`07003oo`00ool00?oo00005Ooo00<00?ooool04?oo00<00?ooool0HOoo0@00
0Ooo000kool00`00ooooo`0Aool00`00ooooo`0200001?oo003oo`000_oo0P00017oo`00ooooo`00
ool00?oo003oo`00ool00?oo003oo`0000;oo`8000;oo`0=003oo`00ool00?oo003oo`00ool00?oo
00000_oo00L00?oo003oo`00ool00002ool01`00ool00?oo003oo`0001Goo`03003ooooo013oo`03
003ooooo067oo`40007oo`00>ooo00<00?ooool05?oo00H00?oo00000?oo0002ool01000ool00000
0_oo00T00?oo003oo`00ool00?oo00000_oo0P000_oo01400?oo003oo`00ool00?oo003oo`00ool0
0?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00D00?oo003oo`0001Ooo`03003ooooo013o
o`03003ooooo067oo`40007oo`00>ooo00<00?ooool05?oo00H00?oo00000?oo0002ool03`00ool0
0?oo003oo`00ool00?oo003oo`00ool00002ool05@00ool00?oo003oo`00ool00?oo003oo`00ool0
0?oo003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`05003oo`00ool0000Gool00`00oooo
o`0@ool00`00ooooo`1Qool10001ool003_oo`03003ooooo01Koo`04003oo`000002ool01@00ool0
0?oo00000_oo0P0000Koo`00ool00?oo0002ool03@00ool00?oo003oo`00ool00?oo003oo`0000;o
o`0?003oo`00ool00?oo003oo`00ool00?oo003oo`0000;oo`05003oo`00ool0000Gool00`00oooo
o`0@ool00`00ooooo`1Qool10001ool003_oo`03003ooooo01Soo`09003oo`00ool00?oo003oo`00
00;oo`04003oo`00ool20002ool03`00ool00?oo003oo`00ool00?oo003oo`00ool00002ool03`00
ool00?oo003oo`00ool00?oo003oo`00ool00002ool00`00ool0000Iool00`00ooooo`0@ool00`00
ooooo`1Qool10001ool003_oo`03003ooooo01Soo`03003oo`0000;oo`80000Eool00?ooool00?oo
003oo`00ool00?oo003oo`00ool00?oo003oo`0000;oo`80000Cool00?ooool00?oo003oo`00ool0
0?oo003oo`00ool00?oo00000_oo00<00?oo00006Ooo00<00?ooool04?oo00<00?ooool0HOoo0@00
0Ooo000kool00`00ooooo`0Jool01@00ooooo`00ool00P000_oo01400?oo003oo`00ool00?oo003o
o`00ool00?oo003oo`0000;oo`04003oo`000002ool03@00ool00?oo003oo`00ool00?oo003oo`00
00;oo`05003oo`00ool0000Iool00`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03003o
oooo01goo`04003oo`000002ool02@00ool00?oo003oo`00ool00002ool20002ool20002ool04`00
ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`0000;oo`03003oo`0001_oo`03003ooooo
013oo`03003ooooo067oo`40007oo`00>ooo00<00?ooool07ooo00d00?oo003oo`00ool00?oo003o
o`00ool00002ool05`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo0000
0_oo00L00?oo003oo`00ool0000Kool00`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03
003ooooo027oo`03003ooooo00800006ool00?oo003oo`000_oo01D00?oo003oo`00ool00?oo003o
o`00ool00?oo003oo`00ool00?oo00000_oo00T00?oo003oo`00ool00?oo00006ooo00<00?ooool0
2ooo2@00H?oo0@000Ooo000kool00`00ooooo`0Qool01`00ooooo`00ool00?oo0080000Iool00?oo
ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo00000_oo00L00?oo003oo`00
ool0000Mool00`00ooooo`0;0009ool8001Hool10001ool003_oo`03003ooooo02Coo`06003oo`00
ool00?oo0P000_oo01D00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo00000_oo
00L00?oo003oo`00ool0000Fool:000Lool8001@ool10001ool003[oo`03003ooooo02Ooo`0K003o
o`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`0000;oo`07003o
o`00ool00?oo00003Ooo2`001ooo00<00?ooool09?oo2000B?oo0@000Ooo000jool00`00ooooo`0Y
ool01@00ool00?oo00000_oo0P0000[oo`00ool00?oo003oo`00ool000;oo`0=003oo`00ool00?oo
003oo`00ool00?oo00000_oo2`004_oo00<00?ooool0;?oo1`00@_oo0@00000jool00`00ooooo`0[
ool00`00ool00002ool01000ool00?oo0P000_oo00@00?oo000000;oo`04003oo`00ool;000Mool0
0`00ooooo`0cool8000jool1000003[oo`03003ooooo02goo`0;003ooooo003oo`00ool00?oo003o
o`0;00001?oo003oo`008ooo00<00?ooool0>ooo2000<_oo0@00000jool00`00ooooo`0]ool<0000
3?oo003oo`00ool00?oo003oo`00ool002Goo`03003ooooo04?oo`P002[oo`400000>_oo00<00?oo
ool08ooo2P001ooo00l00?oo003oo`00ool00?oo003oo`00ool00?oo00009ooo00<00?ooool0Booo
20008_oo0@00000jool00`00ooooo`0Hool;000Cool02`00ool00?oo003oo`00ool00?oo0000:Ooo
00<00?ooool0Dooo1`006ooo0@00000jool00`00ooooo`0=ool;001Bool00`00ooooo`1Jool8000C
ool1000003[oo`03003ooooo00?oo`X005goo`03003ooooo06;oo`P000_oo`400000>_oo1`00I_oo
00<00?ooool0J_oo20000ooo0@000011ool<0009ool00`00ooooo`1>ool00`00ooooo`1^ool80000
04goo``004koo`03003ooooo05ooo`l000Soo`00FOoo3000@_oo00<00?ooool0D?oo3`005ooo001U
ool<000@ool00`00ooooo`0Sool00`00ooooo`11ool?000Vool0077oo``000Coo`03003ooooo02?o
o`03003ooooo03;oo`l003Goo`00OOoo30007_oo00<00?ooool08ooo3`00A?oo0029ool<000Bool0
0`00ooooo`0Dool?001Cool005Coo`8003ooo``000Koo`03003ooooo00Goo`l006;oo`00Dooo00@0
0?ooool004[oo`h0077oo`00Dooo00@00?ooool00<Woo`00Dooo0`00b_oo001Cool00`00ooooo`0Z
ool2002Nool005?oo`03003ooooo02Woo`04003ooooo002Mool005Coo`<002_oo`03003ooooo09_o
o`00P?oo0P009?oo0P00N?oo0022ool00`00ooooo`0Pool01000ooooo`00Mooo001oool01000oooo
o`008_oo00@00?ooool007Ooo`00P?oo0P008ooo00@00?ooool007Ooo`00YOoo00@00?ooool007Oo
o`00YOoo00@00?ooool007Ooo`00Y_oo0P00N?oo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQ
ool007?oo`03003ooooo0:[oo`00KOoo0`001?oo00<00?ooool00_oo0`00Y?oo001^ool00`00oooo
o`03ool00`00ooooo`05ool00`00ooooo`2Qool006koo`03003ooooo00?oo`03003ooooo00?oo`80
0:Coo`00K_oo00<00?ooool01?oo00D00?ooooooo`000:Koo`00K?oo00D00?oo003oo`0000Coo`03
003ooooo00;oo`<00:?oo`00K?oo1@001Ooo00<00?ooool0Yooo003ooolQool00?ooob7oo`00oooo
8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {270.938, 61.1875}} -> {-0.886652, \
0.22898, 0.00409734, 0.00409734}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot with ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Beta]\^'\) = \(3/6 = 1/2\)\)]]
}], "Subsubsection"],

Cell[BoxData[
    \(plotsB4 = createPlotsB[3. /6]; \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(showPlotsB[plotsB4]; \)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .91501 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0262702 1.06936 -2.13046e-17 1.06936 [
[.4752 .92837 -3.00977 0 ]
[.4752 .92837 2.99023 9 ]
[.31156 .91504 -3.05802 0 ]
[.31156 .91504 2.94198 9 ]
[.11966 .89942 -3.11475 0 ]
[.11966 .89942 2.88525 9 ]
[.26027 .97781 -11.2665 0 ]
[.26027 .97781 10.7335 9.8125 ]
[.03465 .18507 -8.54876 -9 ]
[.03465 .18507 9.45124 0 ]
[.18551 .12753 -8.69057 -9 ]
[.18551 .12753 9.30943 0 ]
[.35735 .06198 -8.85423 -9 ]
[.35735 .06198 9.14577 0 ]
[.55418 -0.0131 -6.02969 -9 ]
[.55418 -0.0131 5.97031 0 ]
[.27026 .02904 -13.6412 -9.8125 ]
[.27026 .02904 14.3588 0 ]
[.01476 .32278 -18 -5.85643 ]
[.01476 .32278 0 3.14357 ]
[.00798 .45516 -18 -5.61724 ]
[.00798 .45516 0 3.38276 ]
[.00104 .59125 -18 -5.3727 ]
[.00104 .59125 0 3.6273 ]
[-0.00606 .7312 -18 -5.12262 ]
[-0.00606 .7312 0 3.87738 ]
[-0.06073 .5083 -44.625 -7.93817 ]
[-0.06073 .5083 0 5.06183 ]
[ 0 0 0 0 ]
[ 1 .91501 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.4753 .91501 m
0 .87626 L
s
.47525 .915 m
.47527 .90832 L
s
[(0)] .4752 .92837 .00326 -1 Mshowa
.31182 .90168 m
.31195 .895 L
s
[(3)] .31156 .91504 .01934 -1 Mshowa
.12017 .88606 m
.12043 .87938 L
s
[(6)] .11966 .89942 .03825 -1 Mshowa
gsave
.26027 .97781 -72.2665 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(T) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.03399 .19842 m
.55496 0 L
s
.03399 .19842 m
.03365 .2051 L
s
[(0.1)] .03465 .18507 -0.05014 1 Mshowa
.18505 .14089 m
.18482 .14756 L
s
[(0.4)] .18551 .12753 -0.03438 1 Mshowa
.35714 .07534 m
.35703 .08203 L
s
[(0.7)] .35735 .06198 -0.0162 1 Mshowa
.55425 .00027 m
.55428 .00695 L
s
[(1.)] .55418 -0.0131 .00495 1 Mshowa
gsave
.27026 .02904 -74.6412 -13.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(t) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
75.000 11.250 moveto
(\\220) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
87.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.03399 .19842 m
0 .87626 L
s
.02756 .32664 m
.03396 .32857 L
s
[(100)] .01476 .32278 1 .30143 Mshowa
.02095 .45839 m
.02744 .46 L
s
[(200)] .00798 .45516 1 .24828 Mshowa
.01416 .5938 m
.02072 .59507 L
s
[(300)] .00104 .59125 1 .19393 Mshowa
.00718 .73304 m
.0138 .73395 L
s
[(400)] -0.00606 .7312 1 .13836 Mshowa
gsave
-0.06073 .5083 -105.625 -11.9382 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.47712 .35504 m
.4753 .91501 L
s
.4753 .91501 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
.96816 .22579 m
.47712 .35504 L
s
.03399 .19842 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.47712 .35504 m
.4753 .91501 L
s
.4753 .91501 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.03399 .19842 m
.47712 .35504 L
s
.96816 .22579 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
0 0 m
1 0 L
1 .91501 L
0 .91501 L
closepath
clip
newpath
.5 Mabswid
.46946 .47319 m
.47431 .46782 L
s
.47431 .46782 m
.47918 .46236 L
s
.47918 .46236 m
.48407 .45682 L
s
.48407 .45682 m
.48898 .45121 L
s
.48898 .45121 m
.49391 .44557 L
s
.49391 .44557 m
.49886 .43989 L
s
.46182 .54 m
.46583 .52839 L
s
.46583 .52839 m
.46985 .51659 L
s
.49886 .43989 m
.50384 .43419 L
s
.46985 .51659 m
.4739 .50469 L
s
.50384 .43419 m
.50883 .42849 L
s
.4739 .50469 m
.47795 .49278 L
s
.50883 .42849 m
.51385 .42281 L
s
.47795 .49278 m
.48203 .48094 L
s
.51385 .42281 m
.51888 .41716 L
s
.45406 .59828 m
.45722 .57869 L
s
.48203 .48094 m
.48612 .46924 L
s
.51888 .41716 m
.52394 .41155 L
s
.45722 .57869 m
.4604 .55902 L
s
.48612 .46924 m
.49022 .45775 L
s
.52394 .41155 m
.52902 .40599 L
s
.4604 .55902 m
.46358 .53948 L
s
.49022 .45775 m
.49434 .44653 L
s
.46358 .53948 m
.46677 .52026 L
s
.52902 .40599 m
.53413 .40049 L
s
.49434 .44653 m
.49848 .43563 L
s
.46677 .52026 m
.46997 .50155 L
s
.53413 .40049 m
.53925 .39508 L
s
.49848 .43563 m
.50263 .4251 L
s
.46997 .50155 m
.47318 .4835 L
s
.44621 .64877 m
.44852 .62 L
s
.53925 .39508 m
.5444 .38974 L
s
.44852 .62 m
.45083 .59153 L
s
.47318 .4835 m
.47639 .46621 L
s
.50263 .4251 m
.50681 .41496 L
s
.45083 .59153 m
.45314 .56376 L
s
.5444 .38974 m
.54957 .3845 L
s
.47639 .46621 m
.47962 .4498 L
s
.50681 .41496 m
.511 .40525 L
s
.45314 .56376 m
.45545 .53702 L
s
.45545 .53702 m
.45776 .51158 L
s
.54957 .3845 m
.55477 .37937 L
s
.47962 .4498 m
.48286 .43432 L
s
.511 .40525 m
.51521 .39599 L
s
.45776 .51158 m
.46007 .48763 L
s
.48286 .43432 m
.4861 .41981 L
s
.55477 .37937 m
.55999 .37434 L
s
.51521 .39599 m
.51944 .38718 L
s
.43827 .69223 m
.43972 .65346 L
s
.46007 .48763 m
.46239 .46534 L
s
.43972 .65346 m
.44116 .61576 L
s
.4861 .41981 m
.48936 .4063 L
s
.44116 .61576 m
.4426 .57969 L
s
.55999 .37434 m
.56523 .36942 L
s
.46239 .46534 m
.4647 .44477 L
s
.51944 .38718 m
.52369 .37882 L
s
.4426 .57969 m
.44402 .54572 L
s
.44402 .54572 m
.44543 .51416 L
s
.48936 .4063 m
.49264 .39376 L
s
.56523 .36942 m
.5705 .36462 L
s
.4647 .44477 m
.46701 .42596 L
s
.52369 .37882 m
.52796 .37092 L
s
.44543 .51416 m
.44683 .48522 L
s
.5705 .36462 m
.5758 .35994 L
s
.49264 .39376 m
.49593 .38219 L
s
.46701 .42596 m
.46933 .40891 L
s
.44683 .48522 m
.44821 .45901 L
s
.52796 .37092 m
.53225 .36346 L
s
.43026 .72932 m
.43084 .6801 L
s
.43084 .6801 m
.43141 .63309 L
s
.43141 .63309 m
.43196 .58903 L
s
.5758 .35994 m
.58112 .35539 L
s
.43196 .58903 m
.43248 .54844 L
s
.49593 .38219 m
.49923 .37154 L
s
.44821 .45901 m
.44958 .43555 L
s
.46933 .40891 m
.47165 .39356 L
s
.43248 .54844 m
.43297 .51164 L
s
.53225 .36346 m
.53657 .35642 L
s
.58112 .35539 m
.58646 .35095 L
s
.44958 .43555 m
.45094 .41476 L
s
.43297 .51164 m
.43343 .47877 L
s
.49923 .37154 m
.50256 .36177 L
s
.47165 .39356 m
.47398 .37984 L
s
.53657 .35642 m
.54091 .34979 L
s
.43343 .47877 m
.43387 .44981 L
s
.58646 .35095 m
.59184 .34664 L
s
.45094 .41476 m
.4523 .39652 L
s
.42159 .64474 m
.42124 .59323 L
s
.4219 .70084 m
.42159 .64474 L
s
.42217 .76069 m
.4219 .70084 L
s
.50256 .36177 m
.5059 .3528 L
s
.42124 .59323 m
.42084 .54683 L
s
.47398 .37984 m
.47631 .36764 L
s
.54091 .34979 m
.54528 .34354 L
s
.43387 .44981 m
.43427 .42463 L
s
.42084 .54683 m
.42039 .50577 L
s
.59184 .34664 m
.59724 .34245 L
s
.4523 .39652 m
.45365 .38065 L
s
.42039 .50577 m
.4199 .47002 L
s
.5059 .3528 m
.50926 .34459 L
s
.47631 .36764 m
.47866 .35684 L
s
.54528 .34354 m
.54967 .33765 L
s
.43427 .42463 m
.43465 .40297 L
s
.59724 .34245 m
.60267 .33837 L
s
.4199 .47002 m
.41935 .43935 L
s
.41169 .65174 m
.41043 .59348 L
s
.41288 .71651 m
.41169 .65174 L
s
.45365 .38065 m
.45499 .36693 L
s
.41043 .59348 m
.4091 .54216 L
s
.50926 .34459 m
.51264 .33705 L
s
.41403 .78693 m
.41288 .71651 L
s
.54967 .33765 m
.55408 .33209 L
s
.60267 .33837 m
.60812 .33442 L
s
.47866 .35684 m
.48101 .34732 L
s
.4091 .54216 m
.40769 .4978 L
s
.43465 .40297 m
.43501 .38452 L
s
.41935 .43935 m
.41876 .41339 L
s
.40769 .4978 m
.40621 .46012 L
s
.60812 .33442 m
.61361 .33057 L
s
.45499 .36693 m
.45633 .35511 L
s
.51264 .33705 m
.51604 .33011 L
s
.55408 .33209 m
.55853 .32684 L
s
.48101 .34732 m
.48338 .33894 L
s
.43501 .38452 m
.43534 .36889 L
s
.41876 .41339 m
.41812 .39162 L
s
.40621 .46012 m
.40465 .42858 L
s
.40172 .65495 m
.39954 .59073 L
s
.40381 .72783 m
.40172 .65495 L
s
.39954 .59073 m
.39725 .53538 L
s
.61361 .33057 m
.61912 .32683 L
s
.40583 .80857 m
.40381 .72783 L
s
.39725 .53538 m
.39486 .48863 L
s
.55853 .32684 m
.56299 .32185 L
s
.51604 .33011 m
.51946 .32372 L
s
.45633 .35511 m
.45767 .34495 L
s
.48338 .33894 m
.48576 .33156 L
s
.40465 .42858 m
.40303 .40249 L
s
.61912 .32683 m
.62466 .32319 L
s
.39486 .48863 m
.39236 .44981 L
s
.43534 .36889 m
.43566 .35567 L
s
.41812 .39162 m
.41744 .37347 L
s
.56299 .32185 m
.56749 .31711 L
s
.51946 .32372 m
.52291 .31779 L
s
.39169 .65511 m
.38856 .58576 L
s
.39469 .73543 m
.39169 .65511 L
s
.38856 .58576 m
.3853 .52724 L
s
.62466 .32319 m
.63023 .31965 L
s
.45767 .34495 m
.45902 .33615 L
s
.39236 .44981 m
.38976 .41801 L
s
.39758 .82611 m
.39469 .73543 L
s
.3853 .52724 m
.38189 .47887 L
s
.48576 .33156 m
.48816 .32502 L
s
.40303 .40249 m
.40133 .38101 L
s
.56749 .31711 m
.57202 .31258 L
s
.43566 .35567 m
.43597 .34442 L
s
.41744 .37347 m
.41672 .35831 L
s
.52291 .31779 m
.52638 .31228 L
s
.63023 .31965 m
.63583 .3162 L
s
.38189 .47887 m
.37835 .43953 L
s
.38976 .41801 m
.38707 .39215 L
s
.45902 .33615 m
.46037 .32848 L
s
.38158 .65286 m
.3775 .57918 L
s
.3855 .7399 m
.38158 .65286 L
s
.3775 .57918 m
.37323 .51827 L
s
.57202 .31258 m
.57657 .30824 L
s
.48816 .32502 m
.49058 .3192 L
s
.63583 .3162 m
.64146 .31283 L
s
.40133 .38101 m
.39958 .36329 L
s
.38928 .84002 m
.3855 .7399 L
s
.43597 .34442 m
.43627 .33475 L
s
.52638 .31228 m
.52987 .3071 L
s
.37323 .51827 m
.36879 .46893 L
s
.41672 .35831 m
.41598 .34554 L
s
.37835 .43953 m
.37467 .40785 L
s
.57657 .30824 m
.58115 .30407 L
s
.64146 .31283 m
.64712 .30953 L
s
.38707 .39215 m
.38428 .37108 L
s
.46037 .32848 m
.46173 .32166 L
s
.49058 .3192 m
.49301 .31395 L
s
.36879 .46893 m
.36416 .4295 L
s
.37141 .64871 m
.36634 .57147 L
s
.37627 .74172 m
.37141 .64871 L
s
.36634 .57147 m
.36105 .50887 L
s
.52987 .3071 m
.53339 .30223 L
s
.39958 .36329 m
.39778 .3485 L
s
.38094 .8507 m
.37627 .74172 L
s
.43627 .33475 m
.43656 .32625 L
s
.64712 .30953 m
.65281 .30631 L
s
.58115 .30407 m
.58576 .30005 L
s
.37467 .40785 m
.37088 .38238 L
s
.41598 .34554 m
.41521 .33456 L
s
.36105 .50887 m
.35553 .45906 L
s
.49301 .31395 m
.49546 .30914 L
s
.36416 .4295 m
.35938 .39816 L
s
.46173 .32166 m
.46309 .31547 L
s
.53339 .30223 m
.53693 .29759 L
s
.38428 .37108 m
.38142 .35371 L
s
.65281 .30631 m
.65853 .30314 L
s
.36117 .64309 m
.35509 .56303 L
s
.36698 .74133 m
.36117 .64309 L
s
.58576 .30005 m
.5904 .29615 L
s
.35509 .56303 m
.34874 .4993 L
s
.35553 .45906 m
.3498 .41983 L
s
.37255 .85854 m
.36698 .74133 L
s
.39778 .3485 m
.39593 .33589 L
s
.43656 .32625 m
.43684 .31859 L
s
.37088 .38238 m
.36697 .36167 L
s
.41521 .33456 m
.41442 .32489 L
s
.65853 .30314 m
.66428 .30003 L
s
.34874 .4993 m
.34212 .4494 L
s
.53693 .29759 m
.5405 .29316 L
s
.49546 .30914 m
.49793 .30465 L
s
.5904 .29615 m
.59507 .29235 L
s
.46309 .31547 m
.46447 .30971 L
s
.35938 .39816 m
.35444 .37308 L
s
.35085 .63636 m
.34375 .55412 L
s
.35763 .73911 m
.35085 .63636 L
s
.38142 .35371 m
.3785 .33906 L
s
.66428 .30003 m
.67007 .29696 L
s
.34375 .55412 m
.33631 .48976 L
s
.3498 .41983 m
.34387 .3889 L
s
.36412 .86389 m
.35763 .73911 L
s
.43684 .31859 m
.43713 .31148 L
s
.59507 .29235 m
.59976 .28865 L
s
.5405 .29316 m
.54409 .28889 L
s
.34212 .4494 m
.33525 .41054 L
s
.39593 .33589 m
.39404 .32481 L
s
.49793 .30465 m
.50042 .30038 L
s
.41442 .32489 m
.41361 .3161 L
s
.36697 .36167 m
.36296 .34448 L
s
.67007 .29696 m
.67588 .29393 L
s
.33631 .48976 m
.32856 .44004 L
s
.46447 .30971 m
.46585 .3042 L
s
.34046 .62881 m
.3323 .54498 L
s
.34823 .73537 m
.34046 .62881 L
s
.35444 .37308 m
.34937 .35262 L
s
.59976 .28865 m
.60449 .28502 L
s
.35565 .86705 m
.34823 .73537 L
s
.3323 .54498 m
.32375 .48036 L
s
.54409 .28889 m
.54772 .28475 L
s
.3785 .33906 m
.3755 .32634 L
s
.67588 .29393 m
.68173 .29094 L
s
.34387 .3889 m
.33776 .36414 L
s
.43713 .31148 m
.43741 .3047 L
s
.50042 .30038 m
.50293 .29622 L
s
.33525 .41054 m
.32814 .38001 L
s
.39404 .32481 m
.39212 .31478 L
s
.32856 .44004 m
.32051 .40161 L
s
.41361 .3161 m
.41279 .30789 L
s
.46585 .3042 m
.46725 .29881 L
s
.60449 .28502 m
.60925 .28147 L
s
.32999 .62069 m
.32074 .53575 L
s
.33877 .73041 m
.32999 .62069 L
s
.68173 .29094 m
.68761 .28796 L
s
.36296 .34448 m
.35887 .32977 L
s
.32375 .48036 m
.31483 .43099 L
s
.34713 .8683 m
.33877 .73041 L
s
.54772 .28475 m
.55136 .2807 L
s
.32074 .53575 m
.31105 .47117 L
s
.34937 .35262 m
.34418 .33543 L
s
.50293 .29622 m
.50546 .2921 L
s
.68761 .28796 m
.69352 .28501 L
s
.60925 .28147 m
.61404 .27796 L
s
.43741 .3047 m
.43769 .29811 L
s
.3755 .32634 m
.37246 .31494 L
s
.33776 .36414 m
.3315 .34379 L
s
.46725 .29881 m
.46866 .29344 L
s
.31943 .61218 m
.30908 .52656 L
s
.32926 .72446 m
.31943 .61218 L
s
.55136 .2807 m
.55504 .27674 L
s
.32814 .38001 m
.32083 .35547 L
s
.33857 .86789 m
.32926 .72446 L
s
.39212 .31478 m
.39017 .30546 L
s
.41279 .30789 m
.41195 .30002 L
s
.32051 .40161 m
.31219 .37142 L
s
.31483 .43099 m
.30557 .39299 L
s
.31105 .47117 m
.30093 .42224 L
s
.69352 .28501 m
.69946 .28207 L
s
.61404 .27796 m
.61886 .27451 L
s
.35887 .32977 m
.35469 .31679 L
s
.30908 .52656 m
.29821 .46224 L
s
.50546 .2921 m
.50801 .28793 L
s
.43769 .29811 m
.43798 .29159 L
s
.55504 .27674 m
.55874 .27284 L
s
.69946 .28207 m
.70544 .27913 L
s
.30879 .60343 m
.29729 .51749 L
s
.31968 .71774 m
.30879 .60343 L
s
.32997 .86606 m
.31968 .71774 L
s
.34418 .33543 m
.33889 .32054 L
s
.46866 .29344 m
.47008 .28803 L
s
.37246 .31494 m
.36936 .30447 L
s
.61886 .27451 m
.62371 .27111 L
s
.41195 .30002 m
.4111 .29238 L
s
.3315 .34379 m
.32509 .32651 L
s
.39017 .30546 m
.38819 .29663 L
s
.50801 .28793 m
.51058 .28368 L
s
.29821 .46224 m
.28686 .41378 L
s
.70544 .27913 m
.71145 .2762 L
s
.30093 .42224 m
.29044 .38464 L
s
.29729 .51749 m
.28522 .45355 L
s
.32083 .35547 m
.31333 .33512 L
s
.30557 .39299 m
.29601 .36305 L
s
.31219 .37142 m
.30364 .347 L
s
.55874 .27284 m
.56247 .26897 L
s
.62371 .27111 m
.6286 .26774 L
s
.31004 .71041 m
.29806 .59457 L
s
.32133 .86301 m
.31004 .71041 L
s
.35469 .31679 m
.35044 .30504 L
s
.29806 .59457 m
.2854 .50859 L
s
.43798 .29159 m
.43827 .2851 L
s
.47008 .28803 m
.47151 .28256 L
s
.71145 .2762 m
.71749 .27326 L
s
.51058 .28368 m
.51317 .2793 L
s
.36936 .30447 m
.36621 .2947 L
s
.33889 .32054 m
.33348 .30728 L
s
.56247 .26897 m
.56622 .26514 L
s
.4111 .29238 m
.41024 .28489 L
s
.6286 .26774 m
.63351 .26441 L
s
.2854 .50859 m
.27209 .44513 L
s
.38819 .29663 m
.38618 .28818 L
s
.28522 .45355 m
.27262 .40557 L
s
.30033 .70263 m
.28724 .58569 L
s
.31264 .85891 m
.30033 .70263 L
s
.71749 .27326 m
.72356 .27031 L
s
.28724 .58569 m
.27338 .49988 L
s
.28686 .41378 m
.2751 .37651 L
s
.32509 .32651 m
.31854 .31139 L
s
.47151 .28256 m
.47296 .27701 L
s
.29044 .38464 m
.2796 .35488 L
s
.43827 .2851 m
.43856 .27863 L
s
.31333 .33512 m
.30565 .31768 L
s
.29601 .36305 m
.28618 .33867 L
s
.30364 .347 m
.29486 .32656 L
s
.63351 .26441 m
.63846 .2611 L
s
.51317 .2793 m
.51578 .27478 L
s
.56622 .26514 m
.57001 .26133 L
s
.35044 .30504 m
.34612 .29423 L
s
.72356 .27031 m
.72967 .26736 L
s
.29056 .69453 m
.27632 .57685 L
s
.30391 .85393 m
.29056 .69453 L
s
.27338 .49988 m
.25881 .43694 L
s
.41024 .28489 m
.40937 .27753 L
s
.27632 .57685 m
.26124 .49138 L
s
.36621 .2947 m
.36302 .2855 L
s
.27209 .44513 m
.2582 .39758 L
s
.72967 .26736 m
.73581 .26439 L
s
.38618 .28818 m
.38414 .28007 L
s
.63846 .2611 m
.64344 .25783 L
s
.33348 .30728 m
.32798 .29524 L
s
.47296 .27701 m
.47442 .27141 L
s
.57001 .26133 m
.57382 .25754 L
s
.43856 .27863 m
.43885 .27219 L
s
.27262 .40557 m
.25955 .36856 L
s
.51578 .27478 m
.51842 .27011 L
s
.28071 .68619 m
.26531 .5681 L
s
.29513 .84821 m
.28071 .68619 L
s
.31854 .31139 m
.31186 .29787 L
s
.2751 .37651 m
.26295 .34684 L
s
.73581 .26439 m
.74199 .26141 L
s
.64344 .25783 m
.64845 .25458 L
s
.26531 .5681 m
.24897 .48309 L
s
.2796 .35488 m
.26845 .33045 L
s
.26124 .49138 m
.24537 .42897 L
s
.34612 .29423 m
.34172 .28417 L
s
.30565 .31768 m
.29781 .30233 L
s
.40937 .27753 m
.40849 .2703 L
s
.28618 .33867 m
.27609 .31811 L
s
.29486 .32656 m
.28588 .30893 L
s
.57382 .25754 m
.57766 .25375 L
s
.47442 .27141 m
.4759 .26576 L
s
.25881 .43694 m
.24359 .38978 L
s
.51842 .27011 m
.52108 .26529 L
s
.74199 .26141 m
.7482 .25841 L
s
.36302 .2855 m
.35977 .2768 L
s
.27079 .67772 m
.25419 .55949 L
s
.2863 .84188 m
.27079 .67772 L
s
.38414 .28007 m
.38207 .27227 L
s
.43885 .27219 m
.43914 .26578 L
s
.64845 .25458 m
.6535 .25136 L
s
.2582 .39758 m
.24379 .36076 L
s
.32798 .29524 m
.32237 .28417 L
s
.25419 .55949 m
.23657 .47502 L
s
.7482 .25841 m
.75444 .25539 L
s
.57766 .25375 m
.58153 .24997 L
s
.24897 .48309 m
.23177 .42121 L
s
.25955 .36856 m
.24605 .33892 L
s
.52108 .26529 m
.52376 .26035 L
s
.2608 .66918 m
.24298 .55103 L
s
.27742 .83505 m
.2608 .66918 L
s
.4759 .26576 m
.47738 .26011 L
s
.40849 .2703 m
.40759 .26322 L
s
.31186 .29787 m
.30505 .28557 L
s
.6535 .25136 m
.65858 .24815 L
s
.34172 .28417 m
.33725 .27476 L
s
.24537 .42897 m
.2288 .38213 L
s
.26295 .34684 m
.25044 .32233 L
s
.75444 .25539 m
.76072 .25236 L
s
.43914 .26578 m
.43944 .25944 L
s
.29781 .30233 m
.2898 .28856 L
s
.38207 .27227 m
.37997 .26479 L
s
.26845 .33045 m
.257 .30973 L
s
.58153 .24997 m
.58543 .2462 L
s
.35977 .2768 m
.35647 .26855 L
s
.24298 .55103 m
.22404 .46714 L
s
.28588 .30893 m
.2767 .29335 L
s
.27609 .31811 m
.26576 .30027 L
s
.24359 .38978 m
.22781 .35308 L
s
.65858 .24815 m
.66369 .24496 L
s
.25072 .66063 m
.23165 .54274 L
s
.26848 .82782 m
.25072 .66063 L
s
.23657 .47502 m
.21802 .41361 L
s
.52376 .26035 m
.52646 .25531 L
s
.76072 .25236 m
.76703 .24931 L
s
.47738 .26011 m
.47889 .25448 L
s
.32237 .28417 m
.31667 .27387 L
s
.40759 .26322 m
.40668 .25628 L
s
.24379 .36076 m
.2289 .33109 L
s
.58543 .2462 m
.58936 .24244 L
s
.43944 .25944 m
.43974 .25319 L
s
.23177 .42121 m
.21382 .37463 L
s
.66369 .24496 m
.66884 .24179 L
s
.76703 .24931 m
.77338 .24624 L
s
.23165 .54274 m
.21137 .45946 L
s
.24057 .65211 m
.22022 .53463 L
s
.25949 .82026 m
.24057 .65211 L
s
.33725 .27476 m
.3327 .26587 L
s
.30505 .28557 m
.2981 .27423 L
s
.52646 .25531 m
.52918 .2502 L
s
.37997 .26479 m
.37784 .25761 L
s
.24605 .33892 m
.23214 .31427 L
s
.35647 .26855 m
.35313 .26067 L
s
.22404 .46714 m
.2041 .40617 L
s
.47889 .25448 m
.4804 .24889 L
s
.2288 .38213 m
.21161 .3455 L
s
.77338 .24624 m
.77977 .24316 L
s
.58936 .24244 m
.59332 .23868 L
s
.2898 .28856 m
.28163 .27598 L
s
.25044 .32233 m
.23759 .30141 L
s
.66884 .24179 m
.67403 .23862 L
s
.2767 .29335 m
.26731 .2793 L
s
.257 .30973 m
.24527 .29165 L
s
.40668 .25628 m
.40576 .24949 L
s
.23034 .64366 m
.20868 .52669 L
s
.25045 .81246 m
.23034 .64366 L
s
.22022 .53463 m
.19857 .45194 L
s
.43974 .25319 m
.44004 .24701 L
s
.26576 .30027 m
.25518 .28442 L
s
.31667 .27387 m
.31086 .26418 L
s
.21802 .41361 m
.19865 .36723 L
s
.52918 .2502 m
.53193 .24504 L
s
.77977 .24316 m
.78619 .24007 L
s
.22781 .35308 m
.21149 .32333 L
s
.67403 .23862 m
.67924 .23547 L
s
.59332 .23868 m
.59731 .23493 L
s
.4804 .24889 m
.48193 .24335 L
s
.21137 .45946 m
.19001 .39887 L
s
.37784 .25761 m
.37567 .25068 L
s
.3327 .26587 m
.32809 .25736 L
s
.78619 .24007 m
.79264 .23697 L
s
.22002 .6353 m
.19702 .51893 L
s
.24135 .80449 m
.22002 .6353 L
s
.35313 .26067 m
.34973 .25309 L
s
.2289 .33109 m
.21355 .30627 L
s
.2981 .27423 m
.29103 .2636 L
s
.21382 .37463 m
.19518 .33799 L
s
.20868 .52669 m
.18562 .44459 L
s
.53193 .24504 m
.5347 .23988 L
s
.44004 .24701 m
.44034 .24093 L
s
.67924 .23547 m
.6845 .23232 L
s
.40576 .24949 m
.40483 .24282 L
s
.59731 .23493 m
.60133 .23119 L
s
.2041 .40617 m
.18327 .35993 L
s
.23214 .31427 m
.21785 .29312 L
s
.79264 .23697 m
.79913 .23387 L
s
.28163 .27598 m
.2733 .26427 L
s
.48193 .24335 m
.48347 .23788 L
s
.23218 .79638 m
.20962 .62706 L
s
.19857 .45194 m
.17576 .39168 L
s
.20962 .62706 m
.18525 .51134 L
s
.23759 .30141 m
.22441 .28305 L
s
.21161 .3455 m
.19382 .31562 L
s
.31086 .26418 m
.30496 .25488 L
s
.26731 .2793 m
.25773 .26636 L
s
.6845 .23232 m
.68979 .22917 L
s
.24527 .29165 m
.23325 .27548 L
s
.37567 .25068 m
.37348 .24394 L
s
.25518 .28442 m
.24436 .27002 L
s
.19702 .51893 m
.17253 .43737 L
s
.5347 .23988 m
.5375 .23473 L
s
.79913 .23387 m
.80566 .23076 L
s
.60133 .23119 m
.60538 .22746 L
s
.32809 .25736 m
.32339 .2491 L
s
.44034 .24093 m
.44065 .23492 L
s
.19865 .36723 m
.17852 .33055 L
s
.34973 .25309 m
.34628 .24568 L
s
.22296 .7882 m
.19912 .61894 L
s
.40483 .24282 m
.40388 .23627 L
s
.21149 .32333 m
.19466 .2983 L
s
.48347 .23788 m
.48503 .23246 L
s
.19912 .61894 m
.17337 .5039 L
s
.68979 .22917 m
.69512 .22603 L
s
.29103 .2636 m
.28383 .25342 L
s
.19001 .39887 m
.16769 .3527 L
s
.80566 .23076 m
.81223 .22765 L
s
.18562 .44459 m
.16133 .38459 L
s
.5375 .23473 m
.54031 .22963 L
s
.18525 .51134 m
.1593 .43029 L
s
.60538 .22746 m
.60946 .22375 L
s
.21367 .77998 m
.18854 .61097 L
s
.21355 .30627 m
.19777 .28483 L
s
.19518 .33799 m
.17588 .30794 L
s
.2733 .26427 m
.26481 .25315 L
s
.81223 .22765 m
.81884 .22455 L
s
.37348 .24394 m
.37126 .23732 L
s
.69512 .22603 m
.70048 .22287 L
s
.44065 .23492 m
.44096 .22899 L
s
.30496 .25488 m
.29895 .2458 L
s
.18854 .61097 m
.16136 .49662 L
s
.21785 .29312 m
.20317 .27443 L
s
.48503 .23246 m
.4866 .2271 L
s
.40388 .23627 m
.40293 .22979 L
s
.18327 .35993 m
.16162 .32315 L
s
.25773 .26636 m
.24794 .25417 L
s
.34628 .24568 m
.34278 .23834 L
s
.32339 .2491 m
.31863 .24092 L
s
.20432 .77176 m
.17786 .60313 L
s
.60946 .22375 m
.61358 .22004 L
s
.54031 .22963 m
.54316 .22459 L
s
.22441 .28305 m
.2109 .2665 L
s
.17576 .39168 m
.15191 .34554 L
s
.81884 .22455 m
.82548 .22146 L
s
.17253 .43737 m
.14673 .37758 L
s
.24436 .27002 m
.23331 .25662 L
s
.17337 .5039 m
.14592 .42331 L
s
.23325 .27548 m
.22095 .26066 L
s
.19382 .31562 m
.17546 .29033 L
s
.70048 .22287 m
.70588 .21971 L
s
.17786 .60313 m
.14923 .48948 L
s
.28383 .25342 m
.27651 .2435 L
s
.82548 .22146 m
.83217 .21838 L
s
.44096 .22899 m
.44126 .22311 L
s
.1949 .76356 m
.16709 .59544 L
s
.4866 .2271 m
.48819 .22178 L
s
.61358 .22004 m
.61773 .21635 L
s
.37126 .23732 m
.369 .23074 L
s
.17852 .33055 m
.15766 .30028 L
s
.54316 .22459 m
.54602 .21963 L
s
.19466 .2983 m
.17733 .27653 L
s
.70588 .21971 m
.71131 .21654 L
s
.40293 .22979 m
.40195 .22338 L
s
.16136 .49662 m
.13239 .41643 L
s
.16769 .3527 m
.14447 .31578 L
s
.29895 .2458 m
.29285 .23676 L
s
.1593 .43029 m
.13194 .37065 L
s
.26481 .25315 m
.25615 .24235 L
s
.83217 .21838 m
.83889 .21532 L
s
.16133 .38459 m
.13592 .33842 L
s
.16709 .59544 m
.13698 .48246 L
s
.34278 .23834 m
.33923 .23095 L
s
.18542 .75542 m
.15623 .5879 L
s
.31863 .24092 m
.31379 .23271 L
s
.19777 .28483 m
.18154 .26575 L
s
.61773 .21635 m
.62191 .21267 L
s
.71131 .21654 m
.71679 .21335 L
s
.24794 .25417 m
.23796 .24245 L
s
.48819 .22178 m
.48979 .21651 L
s
.17588 .30794 m
.15594 .28235 L
s
.44126 .22311 m
.44158 .21729 L
s
.54602 .21963 m
.54892 .21475 L
s
.83889 .21532 m
.84566 .21229 L
s
.20317 .27443 m
.18811 .25744 L
s
.23331 .25662 m
.22201 .24388 L
s
.14923 .48948 m
.11871 .40964 L
s
.369 .23074 m
.36672 .22412 L
s
.17586 .74733 m
.14526 .5805 L
s
.15623 .5879 m
.12461 .47557 L
s
.2109 .2665 m
.19705 .25119 L
s
.27651 .2435 m
.26905 .23365 L
s
.22095 .26066 m
.20836 .24674 L
s
.40195 .22338 m
.40097 .21701 L
s
.14592 .42331 m
.11697 .36377 L
s
.16162 .32315 m
.13916 .29263 L
s
.71679 .21335 m
.7223 .21015 L
s
.62191 .21267 m
.62613 .20899 L
s
.84566 .21229 m
.85246 .20927 L
s
.14673 .37758 m
.11971 .33133 L
s
.15191 .34554 m
.12708 .30843 L
s
.17546 .29033 m
.15654 .26818 L
s
.54892 .21475 m
.55184 .20996 L
s
.33923 .23095 m
.33563 .22343 L
s
.48979 .21651 m
.49141 .21127 L
s
.29285 .23676 m
.28666 .22767 L
s
.16623 .73933 m
.1342 .57323 L
s
.44158 .21729 m
.44189 .2115 L
s
.31379 .23271 m
.30889 .22437 L
s
.14526 .5805 m
.11211 .46878 L
s
.25615 .24235 m
.24733 .23172 L
s
.85246 .20927 m
.85931 .20629 L
s
.7223 .21015 m
.72785 .20693 L
s
.13698 .48246 m
.10487 .40292 L
s
.62613 .20899 m
.63038 .20532 L
s
.17733 .27653 m
.1595 .257 L
s
.15766 .30028 m
.1361 .27434 L
s
.13239 .41643 m
.10182 .35693 L
s
.40097 .21701 m
.39997 .21065 L
s
.36672 .22412 m
.36441 .2174 L
s
.15653 .73143 m
.12304 .5661 L
s
.55184 .20996 m
.55478 .20525 L
s
.23796 .24245 m
.22778 .231 L
s
.85931 .20629 m
.8662 .20335 L
s
.13194 .37065 m
.10328 .32427 L
s
.14447 .31578 m
.12038 .28495 L
s
.49141 .21127 m
.49304 .20605 L
s
.72785 .20693 m
.73344 .20369 L
s
.18154 .26575 m
.16487 .24827 L
s
.13592 .33842 m
.10944 .30108 L
s
.1342 .57323 m
.09947 .46209 L
s
.26905 .23365 m
.26147 .22381 L
s
.44189 .2115 m
.44221 .20574 L
s
.63038 .20532 m
.63466 .20166 L
s
.22201 .24388 m
.21048 .23153 L
s
.12461 .47557 m
.09087 .39627 L
s
.18811 .25744 m
.17266 .24159 L
s
.33563 .22343 m
.33199 .21576 L
s
.8662 .20335 m
.87313 .20044 L
s
.15594 .28235 m
.13538 .25978 L
s
.20836 .24674 m
.19549 .23339 L
s
.19705 .25119 m
.18288 .2367 L
s
.14675 .72362 m
.11177 .55909 L
s
.28666 .22767 m
.28036 .21847 L
s
.30889 .22437 m
.30391 .21589 L
s
.73344 .20369 m
.73907 .20043 L
s
.11871 .40964 m
.08648 .35013 L
s
.55478 .20525 m
.55776 .20062 L
s
.39997 .21065 m
.39896 .20431 L
s
.12304 .5661 m
.08671 .45549 L
s
.36441 .2174 m
.36207 .21055 L
s
.49304 .20605 m
.4947 .20086 L
s
.24733 .23172 m
.23835 .22118 L
s
.87313 .20044 m
.8801 .19757 L
s
.63466 .20166 m
.63898 .19799 L
s
.13916 .29263 m
.11593 .26628 L
s
.11697 .36377 m
.08663 .31722 L
s
.1369 .71593 m
.1004 .55221 L
s
.15654 .26818 m
.13705 .24817 L
s
.11211 .46878 m
.07671 .38967 L
s
.44221 .20574 m
.44253 .20001 L
s
.11971 .33133 m
.09153 .29372 L
s
.12708 .30843 m
.10129 .27725 L
s
.73907 .20043 m
.74474 .19715 L
s
.55776 .20062 m
.56076 .19606 L
s
.8801 .19757 m
.88712 .19475 L
s
.22778 .231 m
.2174 .21973 L
s
.33199 .21576 m
.32829 .20792 L
s
.11177 .55909 m
.07382 .44896 L
s
.26147 .22381 m
.25376 .21396 L
s
.10487 .40292 m
.07094 .34336 L
s
.63898 .19799 m
.64333 .19432 L
s
.1595 .257 m
.14117 .23898 L
s
.12697 .70835 m
.08892 .54543 L
s
.4947 .20086 m
.49636 .19569 L
s
.39896 .20431 m
.39794 .19796 L
s
.74474 .19715 m
.75044 .19385 L
s
.1361 .27434 m
.11384 .25129 L
s
.30391 .21589 m
.29885 .20728 L
s
.28036 .21847 m
.27396 .20917 L
s
.21048 .23153 m
.1987 .21946 L
s
.88712 .19475 m
.89418 .19198 L
s
.09947 .46209 m
.06239 .38312 L
s
.36207 .21055 m
.3597 .20353 L
s
.16487 .24827 m
.14776 .23184 L
s
.10182 .35693 m
.06975 .31017 L
s
.44253 .20001 m
.44285 .19429 L
s
.56076 .19606 m
.56378 .19156 L
s
.19549 .23339 m
.18233 .22043 L
s
.17266 .24159 m
.15683 .2265 L
s
.12038 .28495 m
.09542 .25817 L
s
.18288 .2367 m
.16836 .22274 L
s
.1004 .55221 m
.06078 .44251 L
s
.64333 .19432 m
.64772 .19064 L
s
.10328 .32427 m
.07336 .28634 L
s
.11696 .70088 m
.07734 .53876 L
s
.75044 .19385 m
.75619 .19052 L
s
.23835 .22118 m
.2292 .21071 L
s
.10944 .30108 m
.08191 .26951 L
s
.89418 .19198 m
.90129 .18925 L
s
.09087 .39627 m
.0552 .33661 L
s
.49636 .19569 m
.49805 .19054 L
s
.13538 .25978 m
.11417 .23922 L
s
.32829 .20792 m
.32454 .19994 L
s
.08671 .45549 m
.0479 .3766 L
s
.39794 .19796 m
.39691 .19161 L
s
.56378 .19156 m
.56684 .18711 L
s
.10687 .69354 m
.06565 .53219 L
s
.64772 .19064 m
.65215 .18695 L
s
.25376 .21396 m
.2459 .20411 L
s
.75619 .19052 m
.76198 .18717 L
s
.44285 .19429 m
.44318 .18859 L
s
.08892 .54543 m
.04761 .43612 L
s
.3597 .20353 m
.3573 .19635 L
s
.05668 .30875 m
.08648 .35013 L
s
.2174 .21973 m
.2068 .20859 L
s
.29885 .20728 m
.29372 .19856 L
s
.13705 .24817 m
.11699 .22956 L
s
.11593 .26628 m
.09191 .24272 L
s
.27396 .20917 m
.26746 .1998 L
s
.49805 .19054 m
.49975 .1854 L
s
.04841 .34447 m
.07671 .38967 L
s
.06277 .2884 m
.08663 .31722 L
s
.0967 .68631 m
.05384 .52571 L
s
.0771 .25258 m
.10129 .27725 L
s
.76198 .18717 m
.76781 .18381 L
s
.1987 .21946 m
.18667 .20758 L
s
.04093 .38506 m
.07382 .44896 L
s
.06951 .26968 m
.09153 .29372 L
s
.14117 .23898 m
.12233 .22195 L
s
.65215 .18695 m
.65661 .18324 L
s
.56684 .18711 m
.56992 .1827 L
s
.07734 .53876 m
.0343 .42978 L
s
.39691 .19161 m
.39586 .18525 L
s
.32454 .19994 m
.32074 .19186 L
s
.2292 .21071 m
.21987 .20032 L
s
.18233 .22043 m
.16887 .20775 L
s
.44318 .18859 m
.44351 .1829 L
s
.14776 .23184 m
.1302 .21614 L
s
.11384 .25129 m
.09085 .23017 L
s
.0538 .32062 m
.07094 .34336 L
s
.16836 .22274 m
.1535 .20916 L
s
.15683 .2265 m
.14059 .21192 L
s
.3573 .19635 m
.35486 .18902 L
s
.76781 .18381 m
.77368 .18042 L
s
.08645 .67919 m
.04193 .51932 L
s
.49975 .1854 m
.50146 .18029 L
s
.65661 .18324 m
.66111 .1795 L
s
.04583 .35781 m
.06239 .38312 L
s
.29372 .19856 m
.28851 .18979 L
s
.2459 .20411 m
.23791 .19432 L
s
.0385 .40105 m
.06078 .44251 L
s
.56992 .1827 m
.57304 .17831 L
s
.03164 .44967 m
.06565 .53219 L
s
.05964 .29852 m
.06975 .31017 L
s
.26746 .1998 m
.26084 .1904 L
s
.08127 .24496 m
.09542 .25817 L
s
.2068 .20859 m
.19599 .19757 L
s
.09589 .22316 m
.11417 .23922 L
s
.77368 .18042 m
.77959 .17702 L
s
.39586 .18525 m
.39479 .17888 L
s
.07611 .67219 m
.0299 .513 L
s
.06605 .27872 m
.07336 .28634 L
s
.44351 .1829 m
.44384 .17721 L
s
.07318 .26102 m
.08191 .26951 L
s
.66111 .1795 m
.66564 .17573 L
s
.32074 .19186 m
.31689 .18372 L
s
.50146 .18029 m
.5032 .17519 L
s
.0511 .33139 m
.0552 .33661 L
s
.02935 .46868 m
.05384 .52571 L
s
.35486 .18902 m
.3524 .18158 L
s
.57304 .17831 m
.57618 .17394 L
s
.18667 .20758 m
.17437 .19585 L
s
.03621 .41577 m
.04761 .43612 L
s
.0434 .37001 m
.0479 .3766 L
s
.10164 .21644 m
.11699 .22956 L
s
.77959 .17702 m
.78555 .1736 L
s
.21987 .20032 m
.21036 .19001 L
s
.0229 .52379 m
.06568 .6653 L
s
.08577 .23735 m
.09191 .24272 L
s
.28851 .18979 m
.28322 .181 L
s
.16887 .20775 m
.15511 .19523 L
s
.66564 .17573 m
.67022 .17193 L
s
.10796 .20982 m
.12233 .22195 L
s
.44384 .17721 m
.44418 .17153 L
s
.39479 .17888 m
.39372 .17251 L
s
.23791 .19432 m
.22977 .18459 L
s
.26084 .1904 m
.25412 .18101 L
s
.1535 .20916 m
.13828 .19582 L
s
.5032 .17519 m
.50495 .17011 L
s
.11494 .20325 m
.1302 .21614 L
s
.78555 .1736 m
.79154 .17017 L
s
.02718 .48633 m
.04193 .51932 L
s
.57618 .17394 m
.57935 .16957 L
s
.14059 .21192 m
.12395 .19768 L
s
.02054 .54842 m
.05517 .65852 L
s
.03406 .42936 m
.0343 .42978 L
s
.31689 .18372 m
.31297 .17558 L
s
.19599 .19757 m
.18496 .18661 L
s
.3524 .18158 m
.3499 .17407 L
s
.09063 .22998 m
.09085 .23017 L
s
.67022 .17193 m
.67483 .1681 L
s
.79154 .17017 m
.79759 .16673 L
s
.02513 .50275 m
.0299 .513 L
s
.0183 .5715 m
.04457 .65184 L
s
.44418 .17153 m
.44452 .16584 L
s
.57935 .16957 m
.58255 .16519 L
s
.50495 .17011 m
.50672 .16504 L
s
.39372 .17251 m
.39263 .16614 L
s
.28322 .181 m
.27784 .17223 L
s
.21036 .19001 m
.20067 .17974 L
s
.17437 .19585 m
.16182 .18415 L
s
.67483 .1681 m
.67948 .16422 L
s
.79759 .16673 m
.80367 .16328 L
s
.25412 .18101 m
.24728 .17164 L
s
.22977 .18459 m
.22148 .17491 L
s
.01617 .59312 m
.03388 .64526 L
s
.3499 .17407 m
.34737 .16652 L
s
.31297 .17558 m
.309 .16745 L
s
.14123 .18295 m
.15511 .19523 L
s
.58255 .16519 m
.58579 .16081 L
s
.50672 .16504 m
.50851 .15997 L
s
.44452 .16584 m
.44486 .16015 L
s
.80367 .16328 m
.80981 .15983 L
s
.67948 .16422 m
.68416 .16031 L
s
.1314 .18997 m
.13828 .19582 L
s
.39263 .16614 m
.39152 .15975 L
s
.18496 .18661 m
.1737 .17562 L
s
.1227 .19665 m
.12395 .19768 L
s
.01415 .61338 m
.0231 .63878 L
s
.27784 .17223 m
.27237 .16348 L
s
.58579 .16081 m
.58905 .1564 L
s
.80981 .15983 m
.81598 .15637 L
s
.68416 .16031 m
.68889 .15635 L
s
.20067 .17974 m
.19079 .16943 L
s
.50851 .15997 m
.51032 .1549 L
s
.34737 .16652 m
.3448 .15896 L
s
.309 .16745 m
.30497 .15935 L
s
.24728 .17164 m
.24032 .16229 L
s
.44486 .16015 m
.44521 .15444 L
s
.15244 .17555 m
.16182 .18415 L
s
.22148 .17491 m
.21303 .16523 L
s
.39152 .15975 m
.3904 .15336 L
s
.81598 .15637 m
.82221 .15292 L
s
.58905 .1564 m
.59234 .15197 L
s
.68889 .15635 m
.69366 .15235 L
s
.51032 .1549 m
.51215 .14981 L
s
.27237 .16348 m
.26682 .15474 L
s
.16534 .16753 m
.1737 .17562 L
s
.82221 .15292 m
.82848 .14946 L
s
.44521 .15444 m
.44556 .14872 L
s
.3448 .15896 m
.3422 .15143 L
s
.30497 .15935 m
.30087 .15127 L
s
.69366 .15235 m
.69846 .1483 L
s
.59234 .15197 m
.59567 .1475 L
s
.3904 .15336 m
.38926 .14696 L
s
.24032 .16229 m
.23323 .15293 L
s
.19079 .16943 m
.18072 .15899 L
s
.21303 .16523 m
.20443 .15551 L
s
.51215 .14981 m
.514 .14471 L
s
.69846 .1483 m
.70331 .14421 L
s
.44556 .14872 m
.44591 .14298 L
s
.59567 .1475 m
.59903 .14299 L
s
.3422 .15143 m
.33955 .14393 L
s
.26682 .15474 m
.26116 .14602 L
s
.38926 .14696 m
.38811 .14053 L
s
.30087 .15127 m
.29671 .14321 L
s
.514 .14471 m
.51586 .13959 L
s
.70331 .14421 m
.7082 .14008 L
s
.23323 .15293 m
.22602 .14355 L
s
.59903 .14299 m
.60242 .13844 L
s
.44591 .14298 m
.44627 .13721 L
s
.18037 .15863 m
.18072 .15899 L
s
.19812 .14841 m
.20443 .15551 L
s
.33955 .14393 m
.33687 .13648 L
s
.7082 .14008 m
.71313 .13591 L
s
.51586 .13959 m
.51775 .13444 L
s
.38811 .14053 m
.38694 .13409 L
s
.26116 .14602 m
.25542 .13729 L
s
.60242 .13844 m
.60584 .13385 L
s
.29671 .14321 m
.29248 .13516 L
s
.44627 .13721 m
.44663 .13141 L
s
.71313 .13591 m
.71811 .13171 L
s
.21943 .13509 m
.22602 .14355 L
s
.51775 .13444 m
.51966 .12925 L
s
.60584 .13385 m
.6093 .1292 L
s
.33687 .13648 m
.33415 .12906 L
s
.38694 .13409 m
.38576 .12761 L
s
.71811 .13171 m
.72312 .12747 L
s
.25542 .13729 m
.24958 .12855 L
s
.44663 .13141 m
.447 .12557 L
s
.29248 .13516 m
.28819 .1271 L
s
.51966 .12925 m
.52159 .12402 L
s
.6093 .1292 m
.61279 .12449 L
s
.72312 .12747 m
.72818 .12321 L
s
.38576 .12761 m
.38456 .1211 L
s
.33415 .12906 m
.33138 .12167 L
s
.447 .12557 m
.44737 .11969 L
s
.61279 .12449 m
.61632 .11973 L
s
.52159 .12402 m
.52354 .11875 L
s
.24536 .12233 m
.24958 .12855 L
s
.28819 .1271 m
.28382 .11902 L
s
.72818 .12321 m
.73329 .11892 L
s
.38456 .1211 m
.38335 .11454 L
s
.33138 .12167 m
.32858 .11429 L
s
.44737 .11969 m
.44775 .11375 L
s
.61632 .11973 m
.61988 .1149 L
s
.73329 .11892 m
.73844 .11461 L
s
.52354 .11875 m
.52551 .11342 L
s
.28382 .11902 m
.27939 .11091 L
s
.73844 .11461 m
.74363 .1103 L
s
.61988 .1149 m
.62347 .11002 L
s
.38335 .11454 m
.38211 .10792 L
s
.44775 .11375 m
.44812 .10777 L
s
.52551 .11342 m
.5275 .10804 L
s
.32858 .11429 m
.32573 .1069 L
s
.74363 .1103 m
.74888 .10597 L
s
.62347 .11002 m
.6271 .10507 L
s
.27773 .10791 m
.27939 .11091 L
s
.5275 .10804 m
.52952 .10261 L
s
.44812 .10777 m
.44851 .10173 L
s
.38211 .10792 m
.38086 .10125 L
s
.32573 .1069 m
.32284 .09948 L
s
.6271 .10507 m
.63077 .10007 L
s
.52952 .10261 m
.53156 .09711 L
s
.44851 .10173 m
.4489 .09562 L
s
.38086 .10125 m
.3796 .09451 L
s
.63077 .10007 m
.63448 .095 L
s
.32284 .09948 m
.3199 .09202 L
s
.53156 .09711 m
.53362 .09154 L
s
.4489 .09562 m
.44929 .08945 L
s
.63448 .095 m
.63822 .08988 L
s
.3796 .09451 m
.37831 .0877 L
s
.31932 .09057 m
.3199 .09202 L
s
.53362 .09154 m
.53571 .0859 L
s
.63822 .08988 m
.642 .08471 L
s
.44929 .08945 m
.44969 .08322 L
s
.37831 .0877 m
.37701 .08081 L
s
.53571 .0859 m
.53781 .08019 L
s
.642 .08471 m
.64582 .07949 L
s
.44969 .08322 m
.45009 .07691 L
s
.37701 .08081 m
.37569 .07383 L
s
.53781 .08019 m
.53995 .0744 L
s
.64582 .07949 m
.64968 .07422 L
s
.45009 .07691 m
.4505 .07052 L
s
.64968 .07422 m
.65358 .06892 L
s
.53995 .0744 m
.54211 .06853 L
s
.37478 .06903 m
.37569 .07383 L
s
.4505 .07052 m
.45092 .06406 L
s
.65358 .06892 m
.65752 .06358 L
s
.54211 .06853 m
.54429 .06258 L
s
.65752 .06358 m
.6615 .05822 L
s
.45092 .06406 m
.45133 .05751 L
s
.54429 .06258 m
.5465 .05654 L
s
.45133 .05751 m
.45176 .05087 L
s
.5465 .05654 m
.54873 .05042 L
s
.45176 .05087 m
.45219 .04415 L
s
.54873 .05042 m
.55099 .04422 L
s
.55099 .04422 m
.55328 .03795 L
s
.45248 .03961 m
.45219 .04415 L
s
.55328 .03795 m
.55559 .0316 L
s
.55559 .0316 m
.55794 .02518 L
s
.55794 .02518 m
.56031 .0187 L
s
.56031 .0187 m
.56271 .01216 L
s
.56271 .01216 m
.56514 .00556 L
s
.25 Mabswid
.03399 .19842 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
0 .87626 L
s
0 .87626 m
.03399 .19842 L
s
.96816 .22579 m
.55496 0 L
s
.55496 0 m
.5591 .82574 L
s
.5591 .82574 m
1 .8831 L
s
1 .8831 m
.96816 .22579 L
s
.03399 .19842 m
0 .87626 L
s
.02756 .32664 m
.03396 .32857 L
s
[(100)] .01476 .32278 1 .30143 Mshowa
.02095 .45839 m
.02744 .46 L
s
[(200)] .00798 .45516 1 .24828 Mshowa
.01416 .5938 m
.02072 .59507 L
s
[(300)] .00104 .59125 1 .19393 Mshowa
.00718 .73304 m
.0138 .73395 L
s
[(400)] -0.00606 .7312 1 .13836 Mshowa
gsave
-0.06073 .5083 -105.625 -11.9382 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 21 translate 1 -1 scale
63.000 14.438 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.438 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
69.000 14.438 moveto
(w) show
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 10.250 moveto
(\\216) show
75.000 9.375 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.312 14.438 moveto
(\\220) show
86.312 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(Hz) show
98.312 10.625 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
103.625 14.438 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 263.5},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P00011a000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00;Coo`@000?oo`03003ooooo06;oo`00]_oo00<00?ooool0Iooo002fool0
0`00ooooo`1Wool00;Koo`03003ooooo06Ooo`00]_oo00<00?ooool0Iooo002dool3001Yool00;Ko
o`03003ooooo06Ooo`00oooo8Ooo003ooolQool007Woo`03003ooooo0:Coo`00K?oo1@001?oo0P00
0ooo00<00?ooool00_oo0`00=_oo0P00I_oo001]ool00`00ool00004ool01000ooooo`000_oo00<0
0?ooool01Ooo00<00?ooool0<Ooo0P0000Coo`00000006Coo`00KOoo00<00?oo00001?oo00<00?oo
ool00ooo00<00?ooool00ooo0P00<Ooo0`000ooo00<00?oo00000P00H_oo001^ool00`00ooooo`03
ool00`00ooooo`04ool01@00oooooooo0000<?oo0`001_oo00<00?oo00000_oo0P00H?oo001^ool0
0`00ooooo`02ool40004ool00`00ooooo`02ool3000[ool20009ool20005ool2001Nool006koo`03
003ooooo00?oo`03003ooooo00Goo`03003ooooo02coo`<000_oo`03003ooooo00Koo`8005coo`00
M?oo00<00?ooool0<Ooo0`003_oo00<00?ooool02?oo00<00?ooool0FOoo0025ool20004ool00`00
ooooo`03ool00`00ooooo`0Bool2000Aool00`00ooooo`09ool2001Iool008Coo`04003ooooo0009
ool00`00ooooo`0?ool3000Bool2000=ool2001Gool008Coo`04003ooooo000:ool00`00ooooo`0;
ool3000Eool2000?ool2001Eool008Coo`04003ooooo000:ool00`00ooooo`09ool200000ooo003o
o`0Eool2000Aool2001Cool008Coo`04003ooooo000;ool00`00ooooo`05ool30003ool00`00oooo
o`0Cool00`00ool0000Cool2001Aool008Coo`04003ooooo0008ool01000ooooo`001?oo0`001_oo
00<00?ooool04ooo00<00?oo00005Ooo0P00Cooo0025ool20009ool40002ool20009ool00`00oooo
o`0Cool00`00ool0000Gool00`00ooooo`1<ool009?oo`<000_oo`03003ooooo01;oo`04003ooooo
000Fool01000ool00000C?oo002@ool3000>ool00`00ooooo`0Bool01000ooooo`005Ooo00<00?oo
ool00_oo0P00B_oo002>ool3000@ool00`00ooooo`0Bool01000ooooo`005Ooo00<00?ooool01?oo
0P00B?oo002;ool30002ool00`00ooooo`0>ool00`00ooooo`0Aool01@00oooooooo00005?oo00<0
0?ooool01ooo0P00A_oo0029ool200000ooo003oo`02ool00`00ooooo`0>ool00`00ooooo`0Aool0
1@00oooooooo00005?oo00<00?ooool02Ooo0P00A?oo0026ool30007ool00`00ooooo`0>ool00`00
ooooo`0@ool00`00ooooo`02ool00`00ooooo`0Aool00`00ooooo`0<ool00`00ooooo`11ool008?o
o`<000[oo`03003ooooo00koo`03003ooooo013oo`03003ooooo00;oo`03003ooooo013oo`03003o
oooo00koo`80047oo`00POoo0`003Ooo00<00?ooool03Ooo00<00?ooool04?oo00<00?ooool00_oo
00<00?ooool03ooo00<00?ooool04Ooo0P00?ooo001Nool20004ool00`00ooooo`04ool00`00oooo
o`0@ool30003ool00`00ooooo`0:ool00`00ooooo`0=ool00`00ooooo`0?ool00`00ooooo`03ool0
0`00ooooo`0>ool00`00ooooo`0Dool2000mool005goo`04003ooooo000:ool00`00ooooo`0=ool3
0006ool00`00ooooo`0:ool00`00ooooo`0<ool00`00ooooo`0@ool00`00ooooo`03ool00`00oooo
o`0>ool00`00ooooo`0Fool2000kool005goo`04003ooooo0007ool5000<ool20009ool00`00oooo
o`0:ool00`00ooooo`0<ool00`00ooooo`0@ool00`00ooooo`03ool00`00ooooo`0=ool00`00oooo
o`0Iool2000iool005goo`04003ooooo0007ool01000ooooo`002_oo0`0000?oo`00ool02?oo00<0
0?ooool02_oo00<00?ooool03?oo00<00?ooool04?oo00<00?ooool00ooo00<00?ooool03?oo00<0
0?ooool06Ooo00@00?ooool003Soo`00GOoo00@00?ooool000Soo`03003oo`0000Ooo`<000Coo`03
003ooooo00Soo`03003ooooo00[oo`03003ooooo00_oo`03003ooooo00ooo`03003ooooo00Coo`03
003ooooo00coo`03003ooooo01Soo`03003ooooo00;oo`8003Koo`00GOoo00@00?ooool000Woo`80
00Goo`8000Soo`03003ooooo00Soo`03003ooooo00Woo`03003ooooo00_oo`03003ooooo00ooo`03
003ooooo00Coo`03003ooooo00_oo`03003ooooo01Ooo`8000Ooo`8003Coo`00G_oo0P002ooo00<0
0?ooool00`0000?oo`00ool01ooo00<00?ooool02?oo00<00?ooool02Ooo00<00?ooool02ooo00<0
0?ooool03_oo00<00?ooool01Ooo00<00?ooool02_oo00<00?ooool05ooo00<00?ooool02Ooo0P00
<_oo001[ool30005ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`09ool00`00ooooo`0;
ool00`00ooooo`0>ool00`00ooooo`05ool00`00ooooo`09ool00`00ooooo`0Gool00`00ooooo`0<
ool2000`ool006Woo`800003ool00?oo00Goo`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo
00Woo`03003ooooo00_oo`03003ooooo00goo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo
01Ooo`03003ooooo00ooo`8002koo`00I_oo0`001?oo00<00?ooool01?oo00<00?ooool01_oo00<0
0?ooool01ooo00<00?ooool02?oo00<00?ooool02ooo00<00?ooool03Ooo00<00?ooool01_oo00<0
0?ooool02?oo00<00?ooool05_oo00<00?ooool04_oo0P00;?oo001Tool20008ool00`00ooooo`04
ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`0=
ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`0Fool00`00ooooo`0Eool00`00ooooo`0Y
ool0067oo`@000Woo`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Ooo`03003ooooo00So
o`03003ooooo00[oo`03003ooooo00coo`03003ooooo00Ooo`03003ooooo00Koo`03003ooooo01Ko
o`03003ooooo01Ooo`8002Woo`00G_oo0`001_oo00<00?ooool01Ooo00<00?ooool01?oo00<00?oo
ool01_oo00<00?ooool01_oo00<00?ooool02?oo00<00?ooool02_oo00<00?ooool03?oo00<00?oo
ool01ooo00<00?ooool01Ooo00<00?ooool05?oo0`007?oo0P009ooo000kool20004ool00`00oooo
o`02ool4000Bool2000:ool20005ool00`00ooooo`05ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`08ool00`00ooooo`0:ool00`00ooooo`0<ool00`00ooooo`07ool00`00ooooo`05ool0
0`00ooooo`0Cool00`00ooooo`0Jool30002ool2000Uool003[oo`04003ooooo000:ool00`00oooo
o`0>ool30007ool00`00ooooo`04ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`07ool00`00ooooo`0:ool00`00ooooo`0;ool00`00ooooo`08ool0
0`00ooooo`04ool00`00ooooo`0Cool00`00ooooo`0Jool00`00ooooo`05ool2000Sool003[oo`04
003ooooo000:ool00`00ooooo`0;ool3000;ool20005ool00`00ooooo`03ool00`00ooooo`04ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`0:ool00`00ooooo`0;ool0
0`00ooooo`08ool00`00ooooo`03ool00`00ooooo`0Cool00`00ooooo`0Iool2000:ool2000Qool0
03[oo`04003ooooo000:ool00`00ooooo`09ool2000:ool00`00ooooo`03ool00`00ooooo`03ool0
0`00ooooo`03ool00`00ooooo`03ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`08ool0
0`00ooooo`09ool00`00ooooo`0:ool00`00ooooo`09ool00`00ooooo`02ool00`00ooooo`0Cool0
0`00ooooo`0Iool00`00ooooo`0<ool00`00ooooo`0Nool003[oo`04003ooooo000:ool00`00oooo
o`06ool3000=ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`04ool00`00ooooo`03ool0
0`00ooooo`05ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`0;ool0
0`00ooooo`09ool00`00ooooo`02ool00`00ooooo`0Bool00`00ooooo`0Gool3000@ool2000Nool0
03[oo`04003ooooo0008ool30005ool3000<ool00`00ooooo`02ool00`00ooooo`02ool00`00oooo
o`03ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`05ool00`00oooo
o`07ool00`00ooooo`08ool00`00ooooo`0;ool00`00ooooo`09ool01@00oooooooo00005?oo00<0
0?ooool05ooo00<00?ooool04ooo0P007?oo000kool2000;ool01000oooooooo0P003?oo00D00?oo
ooooo`0000Coo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03
003ooooo00Coo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo00_oo`03
003ooooo00Woo`04003ooooo000Cool2000Hool2000Hool2000Jool004Woo`<000ooo`05003ooooo
ool00004ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`04ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`08ool00`00ooooo`0:ool00`00
ooooo`0:ool01000ooooo`004_oo00<00?ooool05_oo0P007?oo0P006?oo0016ool3000?ool01@00
oooooooo00000ooo0P000ooo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool01?oo00<00?oo
ool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01ooo00<00?ooool01ooo00<00?oo
ool02_oo00<00?ooool02_oo00<00?oo00004_oo00<00?ooool05Ooo0P007_oo10005_oo0014ool2
000Cool20002ool00`00ooooo`02ool01@00oooooooo00001?oo00<00?ooool00_oo00<00?ooool0
0ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01_oo00<00?ooool0
1ooo00<00?ooool02Ooo00<00?ooool02ooo0P004_oo00<00?ooool05Ooo00<00?ooool07?oo0P00
1?oo0P005?oo0012ool2000Bool00`00ooooo`02ool01000ooooo`001?oo00D00?ooooooo`0000Co
o`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Go
o`03003ooooo00Koo`03003ooooo00Ooo`03003ooooo00Woo`03003ooooo00_oo`03003ooooo013o
o`03003ooooo01Coo`8001coo`<000Soo`03003ooooo017oo`00@_oo00<00?oo00000P003Ooo00@0
0?ooool000Coo`03003ooooo008000?oo`05003oooooool00004ool00`00ooooo`02ool00`00oooo
o`03ool00`00ooooo`02ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`06ool00`00oooo
o`07ool00`00ooooo`09ool00`00ooooo`0;ool00`00ooooo`0?ool00`00ooooo`0Cool2000Lool2
000<ool2000Aool004;oo`03003ooooo00;oo`<000_oo`03003ooooo008000?oo`05003oooooool0
0003ool01@00oooooooo00000ooo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool00_oo00<0
0?ooool01?oo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo00<0
0?ooool02ooo00<00?ooool03Ooo0P005Ooo00<00?ooool06Ooo0`004?oo0P003ooo0012ool00`00
ooooo`05ool20007ool00`00ooooo`020002ool01@00oooooooo00000ooo00D00?ooooooo`0000?o
o`05003oooooool00004ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`04ool00`00oooo
o`05ool00`00ooooo`05ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`0;ool2000>ool0
0`00ooooo`0Cool2000Kool00`00ooooo`0Cool2000=ool004;oo`03003ooooo00Ooo`<000;oo`03
003ooooo008000;oo`04003ooooo0002ool00`00ooooo`02ool01000ooooo`001?oo00D00?oooooo
o`0000Coo`03003ooooo00?oo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo00Coo`03003o
oooo00Koo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo00[oo`03003oo`0000goo`03003o
oooo01;oo`8001_oo`8001Soo`8000_oo`00@_oo00<00?ooool02_oo10000ooo00@00?ooool000;o
o`800003ool00?oo00?oo`04003ooooo0004ool01@00oooooooo00001?oo00<00?ooool00_oo00<0
0?ooool00ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01_oo00<0
0?ooool02?oo00<00?ooool02_oo00<00?oo00003?oo00<00?ooool04_oo00<00?ooool06?oo0`00
7?oo0P002Ooo0012ool00`00ooooo`0;ool00`00ool000020002ool01000ooooo`000ooo00<00?oo
00001?oo00@00?ooool000Coo`05003oooooool00004ool00`00ooooo`02ool00`00ooooo`03ool0
0`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`08ool0
0`00ooooo`09ool01000ooooo`002ooo00<00?ooool04Ooo0P006_oo00<00?ooool07_oo0P002?oo
0012ool00`00ooooo`0:ool00`00ool00003ool40002ool01`00oooooooo003oo`0000Coo`04003o
oooo0004ool01@00oooooooo00000ooo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool01?oo
00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01ooo00<00?ooool02Ooo
00D00?ooooooo`0000Woo`8001;oo`8001Woo`<001goo`@00003ool00?oo00Ooo`00@_oo00<00?oo
ool02ooo00@00?oo000000;oo`04003oo`000002ool01000ooooo`000_oo0P000ooo00@00?ooool0
00Coo`05003oooooool00003ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`03ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`09ool01@00
oooooooo00002?oo00<00?ooool04?oo0P006Ooo0P007?oo10001Ooo00<00?ooool01_oo0012ool0
0`00ooooo`09ool01000ooooo`000_oo00@00?ooool000;oo`@000;oo`05003oooooool00003ool0
0`00ooooo`020002ool00`00ooooo`02ool01@00oooooooo00001?oo00<00?ooool00_oo00<00?oo
ool00ooo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool01_oo00<00?ooool01_oo00<00?oo
ool02Ooo00<00?ooool00_oo00<00?ooool01_oo00<00?ooool03ooo00<00?ooool05_oo0`006_oo
10002Ooo00<00?ooool01_oo0011ool00`00ooooo`0;ool01000ooooo`000_oo00@00?ooool000;o
o`03003oo`0000800003ool00?oo00;oo`05003oooooool00003ool01000ooooo`001?oo00D00?oo
ooooo`0000?oo`03003ooooo00?oo`03003ooooo00;oo`03003ooooo00Coo`03003ooooo00Goo`03
003ooooo00Goo`03003ooooo00Koo`03003ooooo00Woo`03003ooooo00;oo`03003ooooo00Goo`03
003ooooo00ooo`03003ooooo01Koo`03003ooooo01Soo`<000goo`03003ooooo00Koo`00@Ooo00<0
0?ooool02Ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo0`000ooo00D00?oo
ooooo`0000?oo`04003ooooo0004ool01@00oooooooo00000ooo00<00?ooool00_oo00<00?ooool0
0ooo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool0
2Ooo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool03Ooo0`005ooo0P005ooo10004?oo00<0
0?ooool01_oo0011ool00`00ooooo`08ool00`00ool00002ool01000ooooo`000_oo00@00?ooool0
00;oo`06003oooooool00?oo0`0000?oo`00ool00_oo00@00?ooool000?oo`03003ooooo00;oo`04
003ooooo0004ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`04ool0
0`00ooooo`04ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`03ool00`00ooooo`03ool0
0`00ooooo`0=ool00`00ooooo`0Eool3000Eool4000Dool00`00ooooo`06ool0047oo`03003ooooo
00Woo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`@000;o
o`05003oooooool00003ool01@00oooooooo00000ooo00<00?ooool00_oo00<00?ooool00_oo00<0
0?ooool00_oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01?oo00<00?ooool01_oo00<0
0?ooool02?oo00<00?ooool01?oo00@00?ooooooo`8000ooo`03003ooooo01Goo`03003ooooo01;o
o`@001Soo`03003ooooo00Koo`00@Ooo00<00?ooool01ooo00H00?ooool00?oo0002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00ool0000200000ooo003oo`02ool01@00
oooooooo00000ooo00D00?ooooooo`0000?oo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo
00;oo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Koo`03003ooooo00Soo`03003ooooo
00Coo`04003ooooo000?ool2000Eool3000Bool3000Lool00`00ooooo`06ool0047oo`03003ooooo
00Soo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003o
oooo0002ool00`00ool000020003ool01@00oooooooo00000_oo00<00?ooool00_oo00D00?oooooo
o`0000Coo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Coo`03003o
oooo00Koo`03003ooooo00Ooo`03003ooooo00Goo`03003oo`0000ooo`03003ooooo01Coo`03003o
oooo00ooo`@001ooo`03003ooooo00Koo`00@Ooo00<00?ooool01_oo00H00?ooool00?oo0002ool0
1000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool0
100000?oo`00ool00_oo00@00?ooool000?oo`05003oooooool00004ool00`00ooooo`02ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`06ool00`00
ooooo`06ool00`00ool0000>ool00`00ooooo`0Bool3000>ool4000Tool00`00ooooo`05ool0047o
o`03003ooooo00Ooo`06003ooooo003oo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool30002ool01000ooooo`001?oo00D00?oo
ooooo`0000Coo`05003oooooool00005ool00`00ooooo`03ool00`00ooooo`04ool00`00ooooo`05
ool00`00ooooo`07ool00`00ooooo`06ool2000<ool3000Dool00`00ooooo`0<ool3000Xool00`00
ooooo`05ool0047oo`03003ooooo00Goo`04003ooooo0002ool00`00ool00002ool01000ooooo`00
0_oo00@00?ooool000;oo`06003ooooo003oo`000ooo00@00?ooool000;oo`04003oooooool30000
1Ooo003ooooo00001?oo00D00?ooooooo`0000?oo`03003ooooo00;oo`03003ooooo00;oo`03003o
oooo00Coo`03003ooooo00?oo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Ooo`03003o
oooo00[oo`03003ooooo01;oo`<000_oo`@002_oo`03003ooooo00Goo`00@Ooo00<00?ooool01_oo
00<00?oo00000ooo00<00?oo00000_oo00@00?ooool000;oo`06003ooooo003oo`000ooo00<00?oo
00000ooo00@00?ooool000;oo`04003ooooo0002ool30002ool01@00oooooooo00000ooo00<00?oo
ool00_oo00D00?ooooooo`0000Goo`03003ooooo00?oo`03003ooooo00?oo`03003ooooo00Goo`03
003ooooo00Koo`03003ooooo00Koo`8000_oo`03003ooooo017oo`8000[oo`@002ooo`03003ooooo
00Goo`00@Ooo00<00?ooool01ooo00<00?oo00000_oo00@00?ooool000;oo`06003ooooo003oo`00
0ooo00<00?oo00000_oo00@00?ooool000;oo`05003oooooool00002ool01000ooooo`000ooo1000
1?oo00D00?ooooooo`0000?oo`03003ooooo00;oo`03003ooooo00;oo`03003ooooo00?oo`03003o
oooo00?oo`03003ooooo00Coo`03003ooooo00Koo`03003ooooo00Koo`03003oo`0000[oo`03003o
oooo013oo`8000Soo`@003?oo`03003ooooo00Goo`00@Ooo00<00?ooool01?oo00L00?ooooooo`00
ool00002ool01000ooooo`000_oo00H00?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool0
00;oo`04003ooooo0003ool01000ooooo`000_oo00@00?ooooooo`<00003ool00?oo00;oo`05003o
ooooool00004ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`04ool0
0`00ooooo`06ool00`00ooooo`05ool01000ooooo`002?oo0P004Ooo0P001ooo0`00=ooo00<00?oo
ool01Ooo0011ool00`00ooooo`05ool01`00oooooooo003oo`0000;oo`03003oo`0000;oo`07003o
ooooool00?oo00000_oo00H00?ooool00?oo0003ool01000ooooo`000_oo00D00?ooooooo`0000;o
o`04003ooooo0003ool00`00ool000020003ool01@00oooooooo00001?oo00<00?ooool00_oo00<0
0?ooool00_oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?ooool01_oo00@0
0?ooool000Ooo`03003ooooo00ooo`8000Goo`@003[oo`03003ooooo00Goo`00@Ooo00<00?ooool0
1?oo00<00?oo00000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01P00oooo
o`00ool000;oo`04003ooooo0003ool01000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo
0003ool300000ooo003oo`02ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`02ool00`00
ooooo`03ool00`00ooooo`04ool00`00ooooo`04ool00`00ooooo`06ool01@00oooooooo00001_oo
00<00?ooool03_oo0P000ooo1000?_oo00<00?ooool01Ooo0011ool00`00ooooo`05ool00`00ool0
0002ool00`00ool00003ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo00@00?oo
ool000;oo`04003ooooo0003ool01000ooooo`000_oo00@00?ooool000?oo`06003oooooool00?oo
0`000ooo00D00?ooooooo`0000Coo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00?oo`03
003ooooo00Goo`03003ooooo00Goo`03003ooooo00;oo`03003ooooo00?oo`03003ooooo00koo`03
003oo`0000<004;oo`03003ooooo00Goo`00;?oo10000ooo0P001?oo0P001_oo00<00?ooool00ooo
00H00?ooool00?oo0002ool00`00ool00002ool01000ooooo`000_oo00H00?ooool00?oo0002ool0
1P00ooooo`00ool000?oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?ooool000?oo`04
003ooooo0003ool50003ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`03ool00`00oooo
o`03ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`03ool00`00ooooo`02ool00`00oooo
o`0=ool40016ool00`00ooooo`05ool002koo`05003oooooool00002ool01000ooooo`000_oo00<0
0?ooool00ooo00<00?ooool01?oo00H00?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00
ool000?oo`03003oo`0000;oo`03003oo`0000;oo`05003oooooool00002ool01000ooooo`000_oo
00@00?ooool000;oo`05003oooooool00003ool01000ooooo`000ooo0`000_oo00<00?ooool00_oo
00D00?ooooooo`0000Goo`03003ooooo00?oo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo
00Coo`05003oooooool0000=ool40019ool00`00ooooo`05ool002koo`05003oooooool00002ool0
1000ooooo`000_oo00<00?ooool00ooo00<00?ooool01?oo00@00?ooool000;oo`03003oo`0000;o
o`06003ooooo003oo`000_oo00@00?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01000
ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01000ooooo`000ooo00H00?oooooo
o`00ool30004ool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`04ool0
0`00ooooo`03ool00`00ooooo`05ool01000ooooo`002_oo1@00C?oo00<00?ooool01Ooo000^ool0
1@00oooooooo00000_oo00@00?ooool000;oo`03003ooooo00?oo`03003ooooo00;oo`04003ooooo
0002ool02000ooooo`00ool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`03003oo`00
00;oo`03003oo`0000?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`05003o
ooooool00002ool01@00oooooooo00000ooo0`000_oo00<00?ooool00_oo00<00?ooool00_oo00<0
0?ooool00ooo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool01Ooo00<00?oo00001ooo1000
00Coo`00000004goo`03003ooooo00Goo`00;_oo00D00?ooooooo`0000;oo`04003ooooo0002ool0
0`00ooooo`02ool20005ool01000ooooo`000_oo00<00?oo00000_oo00<00?oo00000_oo00H00?oo
ool00?oo0003ool00`00ool00002ool00`00ool00002ool01000ooooo`000_oo00@00?ooool000?o
o`04003ooooo0002ool01@00oooooooo00000_oo00D00?ooooooo`0000?oo`06003oooooool00?oo
0`001?oo00<00?ooool00_oo00<00?ooool00ooo00<00?ooool00ooo00<00?ooool01?oo00<00?oo
ool01Ooo0P001Ooo0`001?oo00<00?ooool0C_oo00<00?ooool01Ooo000/ool30003ool01000oooo
o`000_oo00@00?ooool000Coo`<000?oo`05003oo`00ool00002ool01000ooooo`000_oo00<00?oo
00000_oo00<00?oo00000_oo00@00?ooool000;oo`06003ooooo003oo`000_oo00@00?ooool000;o
o`04003ooooo0002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`05003oooooool00003ool0
1@00oooooooo00000ooo100000?oo`00ool00ooo00<00?ooool00_oo00<00?ooool00ooo00<00?oo
ool01?oo00<00?ooool01_oo0P0000?oo`0000000P001_oo00<00?ooool0Cooo00<00?ooool01Ooo
000^ool00`00ooooo`02ool20004ool20005ool00`00ooooo`03ool00`00ool00002ool01000oooo
o`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000ooo00<00?oo00000_oo00<00?oo
00000_oo00L00?ooooooo`00ool00002ool01000ooooo`000ooo00@00?ooool000?oo`04003ooooo
0003ool01000ooooo`000ooo00D00?ooooooo`0000?oo`<000?oo`03003ooooo00;oo`03003ooooo
00?oo`03003ooooo00Coo`03003ooooo00Goo`@000Woo`03003ooooo053oo`03003ooooo00Goo`00
@?oo00<00?ooool01?oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool01P00
ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00@00?ooool0
00;oo`04003ooooo0002ool01@00oooooooo00000_oo00D00?ooooooo`0000?oo`04003ooooo0003
ool00`00ooooo`02ool00`00ool000030004ool00`00ooooo`02ool00`00ooooo`04ool00`00oooo
o`03ool50009ool2001Cool00`00ooooo`05ool0043oo`03003ooooo00;oo`06003ooooo003oo`00
0_oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool01P00ooooo`00ool000;o
o`03003oo`0000?oo`06003ooooo003oo`000_oo00@00?ooool000;oo`05003oooooool00002ool0
1@00oooooooo00000_oo00D00?ooooooo`0000;oo`03003ooooo00;oo`05003oooooool00004ool2
0003ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`040002ool00`00ool00008ool00`00
ooooo`1Cool00`00ooooo`05ool0043oo`03003ooooo00?oo`03003oo`0000;oo`03003oo`0000;o
o`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00@00?ooool000;oo`03003o
o`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`05003oooooool00002
ool01@00oooooooo00000_oo00D00?ooooooo`0000?oo`05003oooooool00004ool00`00ool00003
0004ool00`00ooooo`02ool40006ool00`00ool00007ool00`00ooooo`1Dool00`00ooooo`05ool0
043oo`03003ooooo00?oo`06003ooooo003oo`000_oo00<00?oo00000_oo00H00?ooool00?oo0002
ool00`00ool00002ool01P00ooooo`00ool000;oo`03003oo`0000?oo`06003ooooo003oo`000_oo
00@00?ooool000;oo`04003ooooo0003ool01000ooooo`000ooo00@00?ooool000?oo`05003ooooo
ool00003ool00`00ooooo`02ool01000oooooooo10000_oo1@0000?oo`00ool01_oo00@00?ooool0
00Koo`03003ooooo05Koo`03003ooooo00Coo`00@?oo00D00?ooooooo`0000;oo`03003oo`0000;o
o`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00@00?oo
ool000;oo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04
003ooooo0003ool01000ooooo`000ooo00@00?ooool000?oo`05003oooooool00004ool01@00oooo
oooo00000ooo0`001Ooo00<00?ooool01?oo00D00?ooooooo`0000Goo`03003ooooo05Ooo`03003o
oooo00Coo`00@?oo00<00?ooool00_oo00P00?ooool00?oo003oo`000_oo00<00?oo00000_oo00H0
0?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00ool000;oo`03003oo`0000?oo`06003o
oooo003oo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo00000_oo00D00?oo
ooooo`0000;oo`05003oooooool00003ool00`00ooooo`02ool01@00oooooooo00001?oo00<00?oo
ool00ooo00<00?ooool01Ooo00D00?ooooooo`0000?oo`8005[oo`03003ooooo00Coo`00@?oo00<0
0?ooool00_oo00@00?ooool000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`03
003oo`0000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo
00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`05003o
ooooool00002ool01@00oooooooo00000ooo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool0
0ooo00<00?ooool01?oo00<00?ooool00_oo00@00?ooool005coo`03003ooooo00Coo`00@?oo00L0
0?ooooooo`00ool00002ool01@00ool00?oo00000_oo00<00?oo00000_oo00@00?ooool000;oo`05
003oo`00ool00002ool01P00ooooo`00ool000;oo`03003oo`0000?oo`06003ooooo003oo`000_oo
00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00@00?ooool000?oo`04003ooooo
0003ool01@00oooooooo00001?oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool01?oo00<0
0?ooool00ooo00<00?oo0000GOoo00<00?ooool01?oo0010ool01@00oooooooo00000_oo00<00?oo
00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo
00000_oo00<00?oo00000ooo00<00?oo00000_oo00H00?ooool00?oo0003ool00`00ool00003ool0
0`00ool00003ool01000ooooo`000_oo00D00?ooooooo`0000;oo`04003ooooo0003ool01@00oooo
oooo00001?oo00<00?ooool00_oo00<00?ooool00_oo00<00?ooool01?oo00<00?ooool00ooo0P00
G_oo00<00?ooool01?oo0010ool00`00ooooo`02ool00`00ool00002ool02P00ooooo`00ool00?oo
003oo`000_oo00@00?ooool000;oo`05003oo`00ool00002ool01P00ooooo`00ool000;oo`03003o
o`0000?oo`03003oo`0000;oo`06003ooooo003oo`000ooo00@00?ooool000;oo`04003ooooo0002
ool01@00oooooooo00000_oo00D00?ooooooo`0000;oo`05003oooooool00004ool01000oooooooo
0P001?oo00<00?ooool00ooo00<00?ooool01?oo00<00?ooool0GOoo00<00?ooool01?oo0010ool0
0`00ooooo`02ool01000ooooo`000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`00
00;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000?oo`03003oo`0000;oo`06003ooooo
003oo`000_oo00@00?ooool000;oo`04003ooooo0002ool01`00oooooooo003oo`0000?oo`04003o
oooo0003ool01000ooooo`000ooo00<00?ooool00_oo00@00?ooooooo`8000?oo`03003ooooo00Co
o`03003ooooo00?oo`8005ooo`03003ooooo00Coo`00@?oo00@00?ooool000;oo`03003oo`0000;o
o`0:003ooooo003oo`00ool00?oo0002ool01000ooooo`000_oo00D00?oo003oo`0000;oo`06003o
oooo003oo`000_oo00<00?oo00000ooo00D00?oo003oo`0000?oo`03003oo`0000?oo`03003oo`00
00?oo`06003ooooo003oo`000ooo00@00?ooool000?oo`05003oooooool00002ool00`00ooooo`02
ool01000oooooooo0P000ooo00<00?ooool00ooo00<00?ooool00ooo00<00?oo0000Gooo00<00?oo
ool01?oo0010ool01`00oooooooo003oo`0000;oo`08003ooooo003oo`00ool000;oo`03003oo`00
00;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`04003ooooo
0002ool01P00ooooo`00ool000;oo`03003oo`0000?oo`03003oo`0000?oo`04003ooooo0002ool0
1000ooooo`000_oo00D00?ooooooo`0000?oo`04003ooooo0003ool00`00ooooo`02ool20003ool0
0`00ooooo`03ool00`00ooooo`02ool01000ooooo`00Gooo00<00?ooool01?oo0010ool01@00oooo
oooo00000_oo00<00?oo00000_oo00X00?ooool00?oo003oo`00ool000;oo`06003ooooo003oo`00
0_oo00<00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool01P00ooooo`00ool000;o
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooo
oooo00000ooo00@00?ooool000?oo`03003ooooo00;oo`8000?oo`03003ooooo00;oo`03003ooooo
00;oo`05003oooooool0001Oool00`00ooooo`04ool0043oo`08003ooooo003oo`00ool000;oo`08
003ooooo003oo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`03003o
o`0000;oo`03003oo`0000;oo`03003oo`0000?oo`03003oo`0000;oo`06003ooooo003oo`000_oo
00@00?ooool000;oo`04003ooooo0002ool01`00oooooooo003oo`0000?oo`05003oooooool00002
ool01@00oooooooo00001?oo0P000ooo00<00?ooool00_oo00D00?ooooooo`0000Coo`03003ooooo
05goo`03003ooooo00Coo`00@?oo00H00?ooool00?oo0002ool00`00ool00002ool02P00ooooo`00
ool00?oo003oo`000_oo00<00?oo00000_oo00P00?ooool00?oo003oo`000ooo00<00?oo00000_oo
00<00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00003ool00`00ool00003ool00`00ool0
0002ool01`00oooooooo003oo`0000?oo`05003oooooool00002ool01@00oooooooo00000ooo00<0
0?oo00000_oo00<00?ooool00_oo00D00?ooooooo`0000Goo`03003ooooo05goo`03003ooooo00Co
o`00@?oo00T00?ooooooo`00ool00?oo00000_oo00P00?ooool00?oo003oo`000_oo00<00?oo0000
0_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo
00H00?ooool00?oo0003ool00`00ool00002ool00`00ool00003ool01P00ooooo`00ool000?oo`04
003ooooo0002ool01@00oooooooo00000_oo00D00?ooooooo`0000?oo`03003oo`0000;oo`03003o
oooo00;oo`05003oooooool00005ool00`00ooooo`1Mool00`00ooooo`04ool003ooo`03003ooooo
00;oo`03003oo`0000;oo`03003oo`0000;oo`0:003ooooo003oo`00ool00?oo0002ool00`00ool0
0002ool00`00ool00002ool00`00ool00003ool01@00ool00?oo00000_oo00<00?oo00000_oo00@0
0?ooool000;oo`03003oo`0000;oo`04003ooooo0002ool01P00ooooo`00ool000?oo`04003ooooo
0002ool01@00oooooooo00000_oo00D00?ooooooo`0000?oo`03003oo`0000;oo`05003oooooool0
0003ool00`00ooooo`04ool00`00ooooo`1Mool00`00ooooo`04ool003ooo`04003ooooo0002ool0
1@00ool00?oo00000_oo00P00?ooool00?oo003oo`000_oo00<00?oo00000_oo00D00?oo003oo`00
00;oo`05003oo`00ool00003ool00`00ool00002ool00`00ool00002ool00`00ool00002ool01000
ooooo`000_oo00<00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00D00?ooooooo`0000;oo`05003oooooool00003ool01@00ool00?oo00001?oo00@00?oo000000So
o`03003ooooo05coo`03003ooooo00Coo`00?ooo00L00?ooooooo`00ool00002ool01@00ool00?oo
00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002
ool01000ooooo`000_oo00D00?oo003oo`0000;oo`06003ooooo003oo`000_oo00@00?ooool000;o
o`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`04003ooooo
0003ool02@00oooooooo003oo`00ool00003ool00`00ool0000:ool00`00ooooo`1Lool00`00oooo
o`04ool003ooo`05003oooooool00002ool01@00ool00?oo00000_oo00P00?ooool00?oo003oo`00
0_oo00D00?oo003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool01P00ooooo`00ool0
00;oo`03003oo`0000;oo`03003oo`0000?oo`03003oo`0000;oo`03003oo`0000;oo`04003ooooo
0002ool01000ooooo`000_oo00@00?ooool000?oo`04003ooooo0003ool02@00oooooooo003oo`00
ool00003ool00`00ool0000:ool00`00ooooo`1Lool00`00ooooo`04ool003ooo`08003ooooo003o
o`00ool000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00<00?oo00000_oo00D00?oo
003oo`0000;oo`05003oo`00ool00003ool00`00ool00002ool00`00ool00002ool00`00ool00002
ool01P00ooooo`00ool000;oo`04003ooooo0002ool00`00ool00002ool01000ooooo`000_oo00@0
0?ooool000?oo`04003ooooo0003ool01P00oooooooo003oo`8000Coo`8000_oo`03003ooooo05go
o`03003ooooo00?oo`00?ooo00@00?ooool000;oo`07003oo`00ool00?oo00000_oo00D00?oo003o
o`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool00`00ool00002ool0
0`00ool00002ool00`00ool00002ool00`00ool00002ool01000ooooo`000_oo00<00?oo00000_oo
00H00?ooool00?oo0002ool01000ooooo`000ooo00<00?oo00000ooo00@00?ooool000?oo`06003o
ooooool00?oo0P000ooo0P003?oo00<00?ooool0GOoo00<00?ooool00ooo000[ool40003ool20004
ool20005ool02@00oooooooo003oo`00ool00002ool01@00ool00?oo00000_oo00D00?oo003oo`00
00;oo`05003oo`00ool00002ool00`00ool00002ool01@00ool00?oo00000_oo00H00?ooool00?oo
0002ool00`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`00
0_oo00@00?ooool000?oo`03003oo`0000?oo`04003ooooo0003ool01000ooooo`000_oo0P000ooo
00<00?ooool02ooo00<00?ooool0GOoo00<00?ooool00ooo000[ool00`00ooooo`03ool01000oooo
o`000_oo00@00?ooool000Coo`05003oooooool00002ool01`00ool00?oo003oo`0000;oo`07003o
o`00ool00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`08003o
oooo003oo`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`04003ooooo0002ool00`00ool0
0002ool01P00ooooo`00ool000;oo`07003oooooool00?oo00000ooo00@00?ooool000?oo`04003o
oooo0002ool20002ool00`00ooooo`0<ool00`00ooooo`1Mool00`00ooooo`03ool002coo`03003o
oooo00;oo`04003ooooo0002ool01000ooooo`001?oo00<00?oo00000_oo00D00?oo003oo`0000;o
o`05003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool00`00ool0
0002ool00`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool0
0002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00D00?ooooooo`0000;oo`04003o
oooo0002ool01@00oooooooo00000_oo00<00?ooool00P000_oo00<00?ooool03?oo00<00?ooool0
GOoo00<00?ooool00ooo000]ool01@00oooooooo00000_oo00@00?ooool000;oo`03003ooooo00;o
o`03003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool01`00ool00?oo003oo`0000;o
o`03003oo`0000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00P00?ooool00?oo003o
o`000_oo00<00?oo00000_oo00H00?ooool00?oo0002ool00`00ool00002ool01000ooooo`000_oo
00<00?oo00000ooo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo00000_oo0P0000?o
o`0000003_oo00<00?ooool0GOoo00<00?ooool00ooo000^ool01000ooooo`000_oo00@00?ooool0
00;oo`03003ooooo00;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00D00?oo003oo`0000;o
o`8000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00<00?oo00000_oo00<00?oo0000
0_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool0
0?oo0002ool01P00ooooo`00ool000?oo`04003ooooo0002ool01000ooooo`000_oo00D00?oooooo
o`0000;oo`@000ooo`03003ooooo05goo`03003ooooo00?oo`00:ooo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`001?oo0P0000Woo`00ooooo`00ool00?oo00000_oo00<00?oo00000_oo
00L00?oo003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`00
00;oo`03003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;o
o`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo
0003ool01000ooooo`000_oo00<00?ooool03Ooo00<00?ooool0GOoo00<00?ooool00ooo000/ool2
0004ool20004ool20005ool01`00oooooooo003oo`0000;oo`05003oo`00ool00002ool00`00ool0
0002ool200001ooo003ooooo003oo`0000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo
00<00?oo00000_oo00D00?oo003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00H0
0?ooool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooo
oooo00000_oo00<00?oo00004?oo00<00?ooool0GOoo00<00?ooool00ooo000oool01@00ool00?oo
00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`09003oo`00ool00?oo003oo`0000;oo`07
003oo`00ool00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00D00?oo
003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00@00?ooool0
00;oo`04003ooooo0002ool01000ooooo`000_oo00H00?ooooooo`00ool200000ooo003oo`0?ool0
0`00ooooo`1Mool00`00ooooo`03ool003ooo`03003oo`0000;oo`05003oo`00ool00002ool01@00
ool00?oo00000_oo00L00?oo003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`05003oo`00
ool00002ool00`00ool00002ool00`00ool00002ool01@00ool00?oo00000_oo00H00?ooool00?oo
0002ool00`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000?oo`800004ool00?oo000@ool00`00ooooo`1Mool00`00ooooo`03ool003ooo`08
003ooooo003oo`00ool000;oo`03003oo`0000;oo`05003oo`00ool00002ool20002ool01@00ool0
0?oo00000_oo00D00?oo003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool01@00ool0
0?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00H00?ooool00?oo0002
ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool00`00ooooo`03000Aool00`00
ooooo`1Mool00`00ooooo`03ool001Coo`03003ooooo02Soo`06003ooooo003oo`000_oo00D00?oo
003oo`0000;oo`03003oo`0000;oo`07003oo`00ool00?oo00000_oo00T00?oo003oo`00ool00?oo
00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool00`00ool00002
ool00`00ool00002ool01P00ooooo`00ool000;oo`03003oo`0000;oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000;oo`03003ooooo00<0017oo`03003ooooo05goo`03003ooooo00?o
o`000_oo1@000_oo10002?oo00@00?ooooooo`8000;oo`800003ool00000008001_oo`8000;oo`07
003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00<00?oo
00000_oo0P000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`8000;oo`06
003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00@00?ooool0
00;oo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool00`004Ooo00<00?ooool0GOoo00<0
0?ooool00ooo0003ool00`00ool00002ool01P00ooooo`00ool000Ooo`03003ooooo00;oo`04003o
oooo0002ool01000ooooo`006ooo00L00?oo003oo`00ool00002ool01@00ool00?oo00000_oo00T0
0?oo003oo`00ool00?oo00000_oo00<00?oo00000_oo00/00?oo003oo`00ool00?oo003oo`0000;o
o`03003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool00`00ool0
0002ool02000ooooo`00ool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo
0002ool00`00ooooo`02000Bool00`00ooooo`1Mool00`00ooooo`03ool000?oo`03003oo`0000;o
o`06003ooooo003oo`001ooo00<00?ooool00_oo00@00?ooool000?oo`03003ooooo01[oo`06003o
oooo003oo`000_oo00L00?oo003oo`00ool00002ool01@00ool00?oo00000_oo00D00?oo003oo`00
00;oo`09003oo`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool01`00
ool00?oo003oo`0000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?oo00000_oo00<0
0?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool02000ooooo`00ool00?oo000Bool00`00
ooooo`1Mool00`00ooooo`03ool000Coo`05003oooooool00004ool00`00ooooo`06ool01000oooo
oooo10001?oo00<00?ooool06Ooo00d00?ooooooo`00ool00?oo003oo`00ool00002ool01@00ool0
0?oo00000_oo00L00?oo003oo`00ool00002ool00`00ool00002ool20002ool01`00ool00?oo003o
o`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool00`00ool00002ool0
0`00ool00002ool02000ooooo`00ool00?oo0002ool01000ooooo`000_oo00@00?ooool000;oo`05
003ooooo003oo`03000Bool00`00ooooo`1Mool00`00ooooo`03ool000Coo`03003ooooo00;oo`05
003oooooool00008ool01@00oooooooo00000_oo00<00?ooool0100000?oo`0000000P005Ooo00T0
0?oo003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`0;003oo`00ool00?oo003oo`00ool0
0002ool00`00ool00002ool02@00ool00?oo003oo`00ool00002ool01@00ool00?oo00000_oo00D0
0?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003o
o`000_oo00<00?oo00000_oo00@00?ooool000;oo`04003ooooo0002ool00`00ool00002000Bool0
0`00ooooo`1Mool00`00ooooo`03ool000Coo`03003ooooo00;oo`05003oo`00003oo`040005ool0
1000ool000000_oo0P001ooo0P005_oo00<00?oo00000_oo0P000_oo00L00?oo003oo`00ool00002
ool01@00ool00?oo00000_oo0P000_oo00<00?oo00000_oo00L00?oo003oo`00ool00002ool20002
ool00`00ool00002ool00`00ool00002ool20002ool02000ooooo`00ool00?oo0002ool01P00oooo
o`00ool000;oo`03003oo`0000;oo`03003oo`0000;oo`04003ooooo0002ool01000ooooo`000_oo
00<00?oo00000P004_oo00<00?ooool0GOoo00<00?ooool00ooo0009ool200001?oo003ooooo0P00
5ooo00<00?ooool04ooo00X00?ooool00?oo003oo`00ool000;oo`8000;oo`05003oo`00ool00002
ool01`00ool00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00L00?oo003oo`00ool00002
ool01@00ool00?oo00000_oo00D00?oo003oo`0000;oo`05003oo`00ool00002ool01P00ooooo`00
ool000;oo`03003oo`0000;oo`06003ooooo003oo`000_oo00@00?ooool000;oo`08003ooooo003o
o`00ool001;oo`03003ooooo05koo`03003ooooo00;oo`004Ooo00<00?ooool04_oo0`005Ooo00@0
0?ooool000;oo`8000;oo`07003oo`00ool00?oo00000_oo0P000_oo00L00?oo003oo`00ool00002
ool00`00ool00002ool20002ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`07003oo`00
ool00?oo00000_oo00<00?ooool00P000_oo00H00?ooool00?oo0002ool00`00ool00002ool01P00
ooooo`00ool000;oo`04003ooooo0002ool02000ooooo`00ool00?oo000Bool00`00ooooo`1Nool0
0`00ooooo`02ool000ooo`<002coo`8000;oo`0=003oo`00ool00?oo003oo`00ool00?oo00000_oo
00/00?oo003oo`00ool00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool20002
ool01@00ool00?oo00000_oo00L00?oo003oo`00ool00002ool02000ooooo`00ool00?oo0002ool0
0`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`04003ooooo0002ool02000oooo
o`00ool00?oo000Bool00`00ooooo`1Nool00`00ooooo`02ool003koo`8000;oo`03003ooooo0080
00;oo`0;003oo`00ool00?oo003oo`00ool00002ool02@00ool00?oo003oo`00ool00002ool01`00
ool00?oo003oo`0000;oo`8000;oo`03003oo`0000;oo`07003oo`00ool00?oo00000_oo00<00?oo
00000_oo00D00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`06003ooooo003oo`00
0_oo00@00?ooool000;oo`04003oo`00ool200000ooo003oo`0Aool00`00ooooo`1Nool00`00oooo
o`02ool003koo`05003oo`00ool00002ool20002ool01`00ool00?oo003oo`0000;oo`8000;oo`07
003oo`00ool00?oo00000_oo00L00?oo003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`09
003oo`00ool00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002
ool00`00ool00002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00H00?oo00000?oo
000Bool00`00ooooo`1Nool00`00ooooo`02ool003koo`05003oo`00ool00002ool01@00ool00?oo
00000_oo0P000_oo00d00?oo003oo`00ool00?oo003oo`00ool00002ool00`00ool00002ool20002
ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`07003oo`00ool00?oo00000_oo00D00?oo
003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool01P00ooooo`00ool000;oo`06003o
oooo003oo`000_oo00<00?oo00000_oo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo
000nool00`00ool00002ool01`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool20002ool2
0002ool20002ool00`00ool00002ool01@00ool00?oo00000_oo0P000_oo00D00?oo003oo`0000;o
o`07003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`0:003ooooo003oo`00ool00?oo0002
ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?ooool04?oo
00<00?ooool0G_oo00<00?ooool00_oo000nool01P00ooooo`00ool000;oo`8000;oo`07003oo`00
ool00?oo00000_oo0P000_oo00T00?oo003oo`00ool00?oo00000_oo00L00?oo003oo`00ool00002
ool20002ool01@00ool00?oo00000_oo00L00?oo003oo`00ool00002ool01@00ool00?oo00000_oo
00H00?ooool00?oo0002ool20002ool01P00ooooo`00ool000;oo`06003ooooo003oo`000_oo00<0
0?oo00000_oo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000nool200001?oo003o
o`000_oo0P000_oo00D00?oo003oo`0000;oo`0=003oo`00ool00?oo003oo`00ool00?oo00000_oo
00T00?oo003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`0;003oo`00ool00?oo003oo`00
ool00002ool01@00ool00?oo00000_oo00H00?ooool00?oo0002ool20002ool01P00ooooo`00ool0
00;oo`06003ooooo003oo`000_oo00<00?oo00000_oo00<00?ooool04?oo00<00?ooool0G_oo00<0
0?ooool00_oo000nool200001?oo003oo`000_oo00D00?oo003oo`0000;oo`8000;oo`03003ooooo
00800004ool00?oo0002ool20002ool00`00ool00002ool200002Ooo003ooooo003oo`00ool00002
ool01`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool20002ool01P00ooooo`00ool000;o
o`05003oo`00ool00002ool00`00ool00002ool01P00ooooo`00ool000;oo`8000?oo`03003ooooo
013oo`03003ooooo05koo`03003ooooo00;oo`00?_oo00@00?oo000000;oo`07003oo`00ool00?oo
00000_oo00D00?oo003oo`0000;oo`8000;oo`0;003oo`00ool00?oo003oo`00ool00002ool20002
ool01`00ool00?oo003oo`0000;oo`03003oo`0000;oo`8000;oo`05003oo`00ool00002ool01@00
ool00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`06003ooooo
003oo`000_oo0P000ooo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000nool01`00
ool00?oo003oo`0000;oo`8000;oo`07003oo`00ool00?oo00000_oo00/00?oo003oo`00ool00?oo
003oo`0000;oo`8000;oo`09003oo`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool02@00
ool00?oo003oo`00ool00002ool01@00ool00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo
00D00?oo003oo`0000;oo`06003ooooo003oo`000_oo0P000ooo00<00?ooool04?oo00<00?ooool0
G_oo00<00?ooool00_oo000nool01`00ool00?oo003oo`0000;oo`8000;oo`05003oo`00ool00002
ool03@00ool00?oo003oo`00ool00?oo003oo`0000;oo`09003oo`00ool00?oo003oo`0000;oo`80
00;oo`05003oo`00ool00002ool01`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool01@00
ool00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`06003ooooo003oo`000_oo0P00
0ooo00<00?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000nool00`00ooooo`020002ool0
1`00ool00?oo003oo`0000;oo`8000;oo`03003ooooo008000;oo`8000;oo`8000;oo`09003oo`00
ool00?oo003oo`0000;oo`8000;oo`03003ooooo008000;oo`07003oo`00ool00?oo00000_oo00D0
0?oo003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool00`00ool00002ool00`00ool0
0002ool00`00ool00002ool00`00ooooo`02ool00`00ooooo`0@ool00`00ooooo`1Nool00`00oooo
o`02ool003koo`08003ooooo003oo`00ool000;oo`800009ool00?ooool00?oo003oo`0000;oo`80
00;oo`0;003oo`00ool00?oo003oo`00ool00002ool200002Ooo003ooooo003oo`00ool00002ool0
1`00ool00?oo003oo`0000;oo`8000;oo`05003oo`00ool00002ool01@00ool00?oo00000_oo00<0
0?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?oo00000_oo00<00?ooool00_oo00<0
0?ooool04?oo00<00?ooool0G_oo00<00?ooool00_oo000Zool20004ool20004ool20006ool20000
1_oo003oo`00ool000;oo`8000;oo`07003oo`00ool00?oo00000_oo00/00?oo003oo`00ool00?oo
003oo`0000;oo`8000;oo`0;003oo`00ool00?oo003oo`00ool00002ool01`00ool00?oo003oo`00
00;oo`09003oo`00ool00?oo003oo`0000;oo`05003oo`00ool00002ool00`00ool00002ool00`00
ool00002ool00`00ool00002ool01000ool00?oo0P001?oo00<00?ooool04?oo00<00?ooool0G_oo
00<00?ooool00_oo000Yool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0004ool00`00
ool00002ool200003Ooo003ooooo003oo`00ool00?oo003oo`0000;oo`09003oo`00ool00?oo003o
o`0000;oo`8000;oo`0?003oo`00ool00?oo003oo`00ool00?oo003oo`0000;oo`07003oo`00ool0
0?oo00000_oo00T00?oo003oo`00ool00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;o
o`03003oo`0000;oo`03003oo`0000;oo`04003oo`00ool20004ool00`00ooooo`0@ool00`00oooo
o`1Nool00`00ooooo`02ool002coo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool00_oo
00<00?oo00000_oo00/00?oo003oo`00ool00?oo003oo`0000;oo`8000;oo`05003oo`00ool00002
ool20002ool20002ool02@00ool00?oo003oo`00ool00002ool200001ooo003ooooo003oo`0000;o
o`8000;oo`09003oo`00ool00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool0
0`00ool00002ool00`00ool00002ool01000ool00?oo0P001?oo00<00?ooool04?oo00<00?ooool0
G_oo00<00?ooool00_oo000Zool20003ool01000ooooo`000_oo00@00?ooool000Coo`800008ool0
0?oo003oo`00ool000;oo`800009ool00?ooool00?oo003oo`0000;oo`8000;oo`0;003oo`00ool0
0?oo003oo`00ool00002ool200001?oo003ooooo0P000_oo00D00?oo003oo`0000;oo`8000;oo`03
003oo`0000;oo`800007ool00?ooool00?oo00000_oo00D00?oo003oo`0000;oo`03003oo`0000;o
o`03003oo`0000;oo`04003oo`00ool20004ool00`00ooooo`0@ool00`00ooooo`1Nool00`00oooo
o`02ool002coo`04003ooooo0002ool01000ooooo`000_oo00<00?ooool00_oo10000_oo0P0000Co
o`00ooooo`8000;oo`07003oo`00ool00?oo00000_oo0P000_oo00L00?oo003oo`00ool00002ool2
0002ool200003Ooo003ooooo003oo`00ool00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo
00@00?oo000000;oo`0:003ooooo003oo`00ool00?oo0002ool00`00ool00002ool00`00ool00002
ool01000ool00?oo0P001?oo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000Yool0
1000ooooo`000_oo00@00?ooool000;oo`04003ooooo0004ool01000ool000000_oo0P000_oo00d0
0?oo003oo`00ool00?oo003oo`00ool00002ool02`00ool00?oo003oo`00ool00?oo00000_oo00l0
0?oo003oo`00ool00?oo003oo`00ool00?oo00000_oo0P000_oo00L00?oo003oo`00ool00002ool0
1`00ool00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo00<00?oo00000_oo00<00?oo0000
0_oo00D00?oo003oo`0000Goo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00:_oo
0P001?oo0P001?oo0P001Ooo01400?oo003oo`00ool00?oo003oo`00ool00?oo003oo`0000;oo`80
00;oo`09003oo`00ool00?oo003oo`0000;oo`8000;oo`0?003oo`00ool00?oo003oo`00ool00?oo
003oo`0000;oo`07003oo`00ool00?oo00000_oo0P000_oo00L00?oo003oo`00ool00002ool01`00
ool00?oo003oo`0000;oo`03003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00005ool00`00
ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool003goo`0;003oo`00ool00?oo003oo`00ool0
0002ool200002Ooo003ooooo003oo`00ool00002ool200001?oo003oo`000_oo00L00?oo003oo`00
ool00002ool200001?oo003ooooo0P0000_oo`00ooooo`00ool00?oo003oo`0000;oo`8000;oo`05
003oo`00ool00002ool02@00ool00?oo003oo`00ool00002ool00`00ool00002ool00`00ool00002
ool01000ool000001_oo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool00`00
ooooo`020002ool200003ooo003ooooo003oo`00ool00?oo003oo`00ool00002ool20002ool20002
ool01`00ool00?oo003oo`0000;oo`800004ool00?ooool200002ooo003ooooo003oo`00ool00?oo
00000_oo00T00?oo003oo`00ool00?oo00000_oo00D00?oo003oo`0000;oo`8000;oo`03003oo`00
00;oo`03003oo`0000;oo`04003oo`000006ool00`00ooooo`0@ool00`00ooooo`1Oool00`00oooo
o`01ool003goo`03003ooooo008000;oo`0A003oo`00ool00?oo003oo`00ool00?oo003oo`00ool0
0002ool02`00ool00?oo003oo`00ool00?oo00000_oo0P000_oo00@00?oo000000;oo`0=003oo`00
ool00?oo003oo`00ool00?oo00000_oo00T00?oo003oo`00ool00?oo00000_oo00D00?oo003oo`00
00;oo`07003oo`00ool00?oo00000_oo00<00?oo00000_oo00@00?oo000000Koo`03003ooooo013o
o`03003ooooo05ooo`03003ooooo007oo`00?Ooo0P00013oo`00ool00?oo003oo`00ool00?oo003o
o`00ool000;oo`80000?ool00?ooool00?oo003oo`00ool00?oo003oo`0000;oo`8000;oo`04003o
o`000002ool03@00ool00?oo003oo`00ool00?oo003oo`0000;oo`09003oo`00ool00?oo003oo`00
00;oo`05003oo`00ool00002ool00`00ool00002ool20002ool00`00ool00002ool01000ool00000
1_oo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo000mool20002ool20002ool20000
1?oo003ooooo0P0001Goo`00ooooo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo00000_oo
00l00?oo003oo`00ool00?oo003oo`00ool00?oo00000_oo0P000_oo0P000_oo00T00?oo003oo`00
ool00?oo00000_oo00<00?oo00000_oo00D00?oo003oo`0000;oo`07003oo`00ool00?oo00000_oo
00@00?oo000000Koo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00?Ooo0P000_oo
0P000_oo00@00?oo000000;oo`0;003oo`00ool00?oo003oo`00ool00002ool200001?oo003oo`00
0_oo0P000_oo00l00?oo003oo`00ool00?oo003oo`00ool00?oo00000_oo00@00?oo003oo`8000;o
o`09003oo`00ool00?oo003oo`0000;oo`03003oo`0000;oo`05003oo`00ool00002ool01`00ool0
0?oo003oo`0000;oo`04003oo`000006ool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01
ool003goo`0I003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo0000
0_oo0P000_oo0P000_oo00L00?oo003oo`00ool00002ool200001?oo003ooooo0P0000ooo`00oooo
o`00ool00?oo003oo`00ool00?oo00000_oo0P0000Ooo`00ooooo`00ool00002ool01@00ool00?oo
00000_oo00L00?oo003oo`00ool00002ool200000ooo003oo`05ool00`00ooooo`0@ool00`00oooo
o`1Oool00`00ooooo`01ool003goo`03003oo`0000;oo`8000;oo`800004ool00?ooool200001?oo
003ooooo0P0001?oo`00ooooo`00ool00?oo003oo`00ool00?oo003oo`00ool00002ool200001?oo
003ooooo0P0000ooo`00ooooo`00ool00?oo003oo`00ool00?oo00000_oo0P0000Ooo`00ooooo`00
ool00002ool01@00ool00?oo00000_oo00L00?oo003oo`00ool00002ool200000ooo003oo`05ool0
0`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool003goo`03003ooooo00;oo`8000;oo`80
0004ool00?ooool200005Ooo003ooooo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00002
ool20002ool01000ool000000_oo00@00?oo000000;oo`07003oo`00ool00?oo00000_oo0P000_oo
00@00?oo000000;oo`03003oo`0000;oo`05003oo`00ool00002ool01`00ool00?oo003oo`0000;o
o`800003ool00?oo00Goo`03003ooooo013oo`03003ooooo05ooo`03003ooooo007oo`00?Ooo00<0
0?ooool01?oo01l00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool0
0?oo003oo`00ool00002ool20002ool01000ool000000_oo00d00?oo003oo`00ool00?oo003oo`00
ool00002ool20002ool02@00ool00?oo003oo`00ool00002ool01@00ool00?oo00000_oo00L00?oo
003oo`00ool00002ool200000ooo003oo`05ool00`00ooooo`0@ool00`00ooooo`1Oool00`00oooo
o`01ool003goo`03003ooooo00Coo`03003ooooo0080000>ool00?oo003oo`00ool00?oo003oo`00
ool000;oo`800008ool00?oo003oo`00ool000;oo`0G003oo`00ool00?oo003oo`00ool00?oo003o
o`00ool00?oo003oo`00ool00002ool03@00ool00?oo003oo`00ool00?oo003oo`0000;oo`05003o
o`00ool00002ool01`00ool00?oo003oo`0000;oo`8000Soo`03003ooooo013oo`03003ooooo05oo
o`03003ooooo007oo`00?Ooo00<00?ooool01ooo0P0000Coo`00ooooo`800004ool00?ooool20000
1?oo003ooooo0P0000Coo`00ool000;oo`0C003oo`00ool00?oo003oo`00ool00?oo003oo`00ool0
0?oo00000_oo0P0000Coo`00ool000;oo`09003oo`00ool00?oo003oo`0000;oo`8000;oo`05003o
o`00ool00002ool02`00ool00?oo003oo`00ool00?oo00002?oo00<00?ooool04?oo00<00?ooool0
Gooo00<00?ooool00Ooo000mool00`00ooooo`09ool20002ool01000ool000000_oo00@00?oo0000
00;oo`04003oo`00ool20002ool04`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00
00;oo`06003oo`00003oo`000_oo00T00?oo003oo`00ool00?oo00000_oo0P000_oo00D00?oo003o
o`0000;oo`0;003oo`00ool00?oo003oo`00ool00008ool00`00ooooo`0@ool00`00ooooo`1Oool0
0`00ooooo`01ool003coo`03003ooooo00[oo`0U003oo`00ool00?oo003oo`00ool00?oo003oo`00
ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo00000_oo0P0000Soo`00oooo
o`00ool00?oo0P000_oo00T00?oo003oo`00ool00?oo00000_oo00T00?oo003oo`00ool00?oo0000
0_oo00/00?oo003oo`00ool00?oo003oo`0000Soo`03003ooooo013oo`03003ooooo05ooo`03003o
oooo007oo`00??oo00<00?ooool03?oo01d00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00
ool00?oo003oo`00ool00?oo003oo`0000;oo`800004ool00?ooool200002?oo003ooooo003oo`00
ool20002ool02@00ool00?oo003oo`00ool00002ool02@00ool00?oo003oo`00ool00002ool02`00
ool00?oo003oo`00ool00?oo00002?oo00<00?ooool04?oo00<00?ooool0Gooo00<00?ooool00Ooo
000lool00`00ooooo`0?ool200001?oo003ooooo0P0000Coo`00ooooo`800008ool00?oo003oo`00
ool000;oo`8000;oo`800006ool00?ooool00?oo0P000_oo01400?oo003oo`00ool00?oo003oo`00
ool00?oo003oo`0000;oo`09003oo`00ool00?oo003oo`0000;oo`0;003oo`00ool00?oo003oo`00
ool00008ool00`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`01ool003coo`03003ooooo00oo
o`04003oo`000002ool01000ool000000_oo0P0000Soo`00ool00?oo003oo`000_oo0P000_oo00@0
0?oo000000;oo`04003oo`000002ool04@00ool00?oo003oo`00ool00?oo003oo`00ool00?oo0000
0_oo00T00?oo003oo`00ool00?oo00000_oo00/00?oo003oo`00ool00?oo003oo`0000Soo`03003o
oooo013oo`03003ooooo063oo`40007oo`7oo`00??oo00<00?ooool04Ooo00H00?oo003oo`00ool2
0002ool01000ool00?oo0P0000coo`00ooooo`00ool00?oo003oo`00ool20002ool03@00ool00?oo
003oo`00ool00?oo003oo`0000;oo`800006ool00?oo003oo`000_oo00T00?oo003oo`00ool00?oo
00000_oo00/00?oo003oo`00ool00?oo003oo`0000Soo`03003ooooo013oo`03003ooooo063oo`40
007oo`7oo`00??oo00<00?ooool04ooo02/00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00
ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00002ool20000
1_oo003oo`00ool000;oo`09003oo`00ool00?oo003oo`0000;oo`0;003oo`00ool00?oo003oo`00
ool00008ool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool003coo`03003ooooo01?oo`03
003ooooo00800004ool00?ooool200008?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool0
0?oo003oo`00ool00?oo003oo`00ool00?oo0002ool01000ool00?oo0P0000goo`00ooooo`00ool0
0?oo003oo`00ool00002ool02`00ool00?oo003oo`00ool00?oo00002?oo00<00?ooool04?oo00<0
0?ooool0H?oo0@000Ooo0Ooo000lool00`00ooooo`0Fool01000ool000000_oo0P00023oo`00ool0
0?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`000_oo
00H00?oo003oo`00ool20002ool00`00ool00002ool01000ool000000_oo00T00?oo003oo`00ool0
0?oo00002_oo00<00?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000lool00`00ooooo`0Fool0
1000ool000000_oo00X00?oo00000?oo003oo`00ool000;oo`80000<ool00?oo003oo`00ool00?oo
003oo`000_oo0P0000Coo`00ool000;oo`06003oo`00ool00?oo0P000_oo00<00?oo00000_oo00l0
0?oo003oo`00ool00?oo003oo`00ool00?oo00002_oo00<00?ooool04?oo00<00?ooool0H?oo0@00
0Ooo0Ooo000lool00`00ooooo`0Hool02000ool00?oo003oo`00ool200001_oo003ooooo003oo`80
00;oo`800008ool00?oo003oo`00ool000;oo`06003oo`00003oo`000_oo00d00?oo003oo`00ool0
0?oo003oo`00ool00002ool03`00ool00?oo003oo`00ool00?oo003oo`00ool0000:ool00`00oooo
o`0@ool00`00ooooo`1Pool10001ool1ool003coo`03003ooooo01[oo`08003oo`00ool00?oo003o
o`8000;oo`06003oo`00ool00?oo0P0000Coo`00ooooo`800006ool00?ooool00?oo0P0000goo`00
ooooo`00ool00?oo003oo`00ool00002ool20002ool03`00ool00?oo003oo`00ool00?oo003oo`00
ool0000:ool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool003coo`03003ooooo01[oo`03
003ooooo0080000?ool00?oo003oo`00ool00?oo003oo`00ool00?oo008000;oo`800008ool00?oo
ool00?oo003oo`8000;oo`09003oo`00ool00?oo003oo`0000;oo`8000;oo`0?003oo`00ool00?oo
003oo`00ool00?oo003oo`0000[oo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00:_oo
00<00?ooool00_oo0P001?oo0P001Ooo00<00?ooool07Ooo0P0000ooo`00ool00?oo003oo`00ool0
0?oo003oo`00ool00P000_oo00@00?oo000000;oo`04003oo`00ool20002ool02@00ool00?oo003o
o`00ool00002ool04`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`0000[oo`03003o
oooo013oo`03003ooooo063oo`40007oo`7oo`00:_oo00D00?ooooooo`0000;oo`04003ooooo0002
ool00`00ooooo`02ool00`00ooooo`0Mool05`00ool00000ool00?oo003oo`00ool00?oo003oo`00
ool00?oo003oo`00ool00P000_oo01400?oo003oo`00ool00?oo003oo`00ool00?oo003oo`0000;o
o`0C003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo00002_oo00<00?ooool04?oo00<0
0?ooool0H?oo0@000Ooo0Ooo000Wool50002ool01000ooooo`000_oo00@00?ooool000Coo`03003o
oooo01ooo`800004ool00?oo0002ool200008?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00
ool00?oo003oo`00ool00?oo003oo`00ool00?oo0002ool04`00ool00?oo003oo`00ool00?oo003o
o`00ool00?oo003oo`0000[oo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`009ooo00@0
0?ooool000?oo`04003ooooo0002ool01000ooooo`001?oo0P008?oo00H00?oo00000?oo0002ool0
8P00ool00000ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool0
0?oo003oo`000_oo01400?oo003oo`00ool00?oo003oo`00ool00?oo003oo`0000coo`03003ooooo
013oo`03003ooooo063oo`40007oo`7oo`00:?oo00<00?oo00000ooo00@00?ooool000;oo`04003o
oooo0004ool00`00ooooo`0Qool01000ool000000_oo02800?oo00000?oo003oo`00ool00?oo003o
o`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool000;oo`0A003oo`00ool00?oo
003oo`00ool00?oo003oo`00ool0000<ool00`00ooooo`0@ool00`00ooooo`1Pool10001ool1ool0
02Woo`8000?oo`04003ooooo0002ool01000ooooo`001?oo00<00?ooool08Ooo00`00?oo003oo`00
ool00?oo003oo`00ool200006_oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003o
o`00ool00?oo0002ool04@00ool00?oo003oo`00ool00?oo003oo`00ool00?oo00003?oo00<00?oo
ool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000Zool00`00ooooo`02ool20004ool20005ool00`00
ooooo`0Sool02P00ool00?oo003oo`00ool00?oo0P00013oo`00ool00?oo003oo`00ool00?oo003o
o`00ool000;oo`800006ool00?oo003oo`000_oo01400?oo003oo`00ool00?oo003oo`00ool00?oo
003oo`0000coo`03003ooooo013oo`03003ooooo063oo`40007oo`7oo`00>ooo00<00?ooool09_oo
00X00?oo003oo`00ool00?oo003oo`8000;oo`80000:ool00?oo003oo`00ool00?oo0002ool02000
ool00000ool00?oo0002ool04@00ool00?oo003oo`00ool00?oo003oo`00ool00?oo00003?oo00<0
0?ooool04?oo00<00?ooool0H?oo0@000Ooo0Ooo000kool00`00ooooo`0Vool00`00ooooo`020000
2Ooo003oo`00ool00?oo003oo`0200001?oo003ooooo0P0000Coo`00ool000;oo`08003oo`00003o
o`00ool000;oo`0=003oo`00ool00?oo003oo`00ool00?oo00000_oo0P003?oo00<00?ooool04?oo
00<00?ooool0H?oo0@000Ooo0Ooo000kool00`00ooooo`0Yool03@00ool00000ool00?oo003oo`00
ool00?oo008000;oo`800004ool00?oo0002ool01000ool00?oo0P00017oo`00ooooo`00ool00?oo
003oo`00ool00?oo003oo`0000;oo`8000coo`03003ooooo013oo`03003ooooo067oo`40007oo`00
>ooo00<00?ooool0:Ooo00d00?oo00000?oo003oo`00ool00?oo003oo`020002ool01P00ool00000
ool000;oo`06003oo`00ool00?oo0P000_oo00d00?oo003oo`00ool00?oo003oo`00ool00002ool0
0`00ooooo`0;ool00`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03003ooooo02_oo`0C
003oo`00003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00P000_oo01L00?oo003oo`00ool0
0?oo003oo`00ool00?oo003oo`00ool00?oo003oo`0000;oo`03003ooooo00_oo`03003ooooo013o
o`03003ooooo067oo`40007oo`00>ooo00<00?ooool0:ooo01<00?oo00000?oo003oo`00ool00?oo
003oo`00ool00?oo003oo`020002ool05@00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003o
o`00ool00002ool00`00ool0000=ool00`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03
003ooooo02goo`0Z003oo`00003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00
ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`000_oo00<00?oo00003Ooo00<00?ooool0
4?oo00<00?ooool0HOoo0@000Ooo000kool00`00ooooo`0]ool0:P00ool00000ool00?oo003oo`00
ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00
ool000;oo`03003oo`0000goo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo00<00?oo
ool0;ooo02H00?oo00000?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00
ool00?oo003oo`00ool00?oo003oo`000_oo00D00?oo003oo`0000goo`03003ooooo013oo`03003o
oooo067oo`40007oo`00>ooo00<00?ooool0;ooo00@00?oo003oo`80000Pool00?oo003oo`00ool0
0?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool000;oo`03003oo`00
00ooo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo00<00?ooool0<Ooo00@00?oo003o
o`80000Lool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo
0002ool01@00ool00?oo00003ooo00<00?ooool04?oo00<00?ooool0HOoo0@000Ooo000kool00`00
ooooo`0aool01000ool00?oo0P0001[oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003o
o`00ool00?oo003oo`000_oo00L00?oo003oo`00ool0000?ool00`00ooooo`0@ool00`00ooooo`1Q
ool10001ool003_oo`03003ooooo03?oo`04003oo`00ool200005_oo003oo`00ool00?oo003oo`00
ool00?oo003oo`00ool00?oo003oo`000_oo00T00?oo003oo`00ool00?oo00003ooo00<00?ooool0
4?oo00<00?ooool0HOoo0@000Ooo000kool00`00ooooo`0eool01000ool000000_oo0P0000koo`00
ool00?oo003oo`00ool00?oo003oo`000_oo00d00?oo003oo`00ool00?oo003oo`00ool0000?ool0
0`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03003ooooo03Goo`06003oo`00ool00?oo
0P0000koo`00ool00?oo003oo`00ool00?oo003oo`000_oo00/00?oo003oo`00ool00?oo003oo`00
017oo`03003ooooo013oo`03003ooooo067oo`40007oo`00>ooo00<00?ooool0=ooo00H00?oo003o
o`00ool200003?oo003oo`00ool00?oo003oo`00ool000;oo`0;003oo`00ool00?oo003oo`00ool0
000Aool00`00ooooo`0@ool00`00ooooo`1Qool10001ool003_oo`03003ooooo03Ooo`08003oo`00
ool00?oo003oo`80000:ool00?oo003oo`00ool00?oo0002ool02`00ool00?oo003oo`00ool00?oo
00004Ooo00<00?ooool04?oo00<00?ooool0HOoo0@000Ooo000kool00`00ooooo`0iool01P00ool0
0?oo003oo`8000;oo`800006ool00?oo003oo`000_oo00/00?oo003oo`00ool00?oo003oo`00017o
o`03003ooooo00_oo`T0063oo`40007oo`00>ooo00<00?ooool0>Ooo00`00?oo003oo`00ool00?oo
003oo`00ool200001?oo003oo`000_oo00T00?oo003oo`00ool00?oo00004ooo00<00?ooool02`00
2Ooo2000F?oo0@000Ooo000kool00`00ooooo`0kool03P00ool00?oo003oo`00ool00?oo003oo`00
ool20002ool02@00ool00?oo003oo`00ool0000<ool:000Lool8001@ool10001ool003[oo`03003o
oooo03koo`0J003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`00ool00?oo003o
o`/000Ooo`03003ooooo02Coo`P004Soo`40007oo`00>_oo00<00?ooool0?_oo00h00?oo003oo`00
ool00?oo003oo`00ool00?oo30004_oo00<00?ooool0;?oo1`00@_oo0@00000jool00`00ooooo`10
ool00`00ool0000:00002?oo003oo`00ool00?oo000Eool00`00ooooo`0cool8000jool1000003[o
o`03003ooooo03Soo`/0000Bool00?oo003oo`00ool00?oo003oo`00ool00?oo003oo`005Ooo00<0
0?ooool0>ooo2000<_oo0@00000jool00`00ooooo`0]ool;000:ool04@00ool00?oo003oo`00ool0
0?oo003oo`00ool00?oo00005ooo00<00?ooool0@ooo2000:_oo0@00000jool00`00ooooo`0Sool:
000Gool03`00ool00?oo003oo`00ool00?oo003oo`00ool0000Gool00`00ooooo`1;ool8000Rool1
000003[oo`03003ooooo01Soo`/002?oo`0;003oo`00ool00?oo003oo`00ool0000Iool00`00oooo
o`1Cool7000Kool1000003[oo`03003ooooo00goo`/0033oo`07003oo`00ool00?oo00006ooo00<0
0?ooool0F_oo20004ooo0@00000jool00`00ooooo`03ool:001Mool00`00ooooo`1Rool8000;ool1
000003[oo`L006Koo`03003ooooo06[oo`P000?oo`400000@Ooo30002Ooo00<00?ooool0C_oo00<0
0?ooool0K_oo2000001=ool<001>ool00`00ooooo`1Oool?0008ool005Woo``004;oo`03003ooooo
053oo`l001Ooo`00IOoo30004?oo00<00?ooool08ooo00<00?ooool0@Ooo3`009_oo001aool<0004
ool00`00ooooo`0Sool00`00ooooo`0bool?000eool007goo``001koo`03003ooooo02?oo`l004Co
o`00ROoo30004_oo00<00?ooool05?oo3`00Dooo001Dool2000oool<0006ool00`00ooooo`05ool?
001Rool005?oo`04003ooooo001:ool>001aool005?oo`04003ooooo0039ool005?oo`<00<[oo`00
Dooo00<00?ooool0:_oo0P00W_oo001Cool00`00ooooo`0Yool01000ooooo`00WOoo001Dool3000[
ool00`00ooooo`2Kool0083oo`8002Coo`8007Soo`00P_oo00<00?ooool08?oo00@00?ooool007Oo
o`00Oooo00@00?ooool002;oo`04003ooooo001gool0083oo`8002?oo`04003ooooo001gool00:Go
o`04003ooooo001gool00:Goo`04003ooooo001gool00:Koo`8007Soo`00oooo8Ooo003ooolQool0
0?ooob7oo`00oooo8Ooo001cool00`00ooooo`2Zool006goo`<000Coo`03003ooooo00;oo`<00:Co
o`00K_oo00<00?ooool00ooo00<00?ooool01Ooo00<00?ooool0XOoo001^ool00`00ooooo`03ool0
0`00ooooo`03ool2002Tool006koo`03003ooooo00Coo`05003oooooool0002Vool006coo`05003o
o`00ool00004ool00`00ooooo`02ool3002Sool006coo`D000Goo`03003ooooo0:Ooo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?oo
ob7oo`00\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {565.188, 355.438}} -> {-0.886652, \
1.73603, 0.00409734, 0.00409734}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
Evaluator->"hubble",
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingPageRange->{2, 4},
PageHeaders->{{Inherited, Inherited, Inherited}, {None, Inherited, None}},
PageHeaderLines->{False, False},
PrintingOptions->{"PrintingMargins"->{{93.5625, 54}, {36, 36}},
"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "felix", \
"tmp"}, "omegaerrs.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1},
Magnification->1
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 28, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[1792, 55, 38, 0, 45, "Subsection"],
Cell[1833, 57, 374, 11, 32, "Text"],
Cell[2210, 70, 128, 2, 27, "Input"],
Cell[2341, 74, 125, 4, 32, "Text"],
Cell[2469, 80, 203, 4, 45, "Input"],
Cell[2675, 86, 386, 7, 59, "Input"],
Cell[3064, 95, 236, 4, 43, "Input"],
Cell[3303, 101, 183, 6, 32, "Text"],
Cell[3489, 109, 112, 2, 27, "Input"],
Cell[3604, 113, 379, 7, 45, "Input"],
Cell[3986, 122, 177, 3, 27, "Input"],
Cell[4166, 127, 395, 7, 42, "Input"],
Cell[4564, 136, 206, 6, 32, "Text"],
Cell[4773, 144, 198, 5, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5008, 154, 210, 7, 45, "Subsection"],
Cell[5221, 163, 127, 3, 32, "Text"],
Cell[5351, 168, 162, 3, 59, "Input"],
Cell[5516, 173, 100, 3, 32, "Text"],
Cell[5619, 178, 631, 11, 66, "Input"],

Cell[CellGroupData[{
Cell[6275, 193, 270, 9, 42, "Subsubsection"],
Cell[6548, 204, 93, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[6666, 209, 52, 1, 27, "Input"],
Cell[6721, 212, 12931, 373, 186, 5760, 280, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[19667, 588, 1279, 38, 68, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20983, 631, 270, 9, 42, "Subsubsection"],
Cell[21256, 642, 92, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[21373, 647, 52, 1, 27, "Input"],
Cell[21428, 650, 12930, 373, 186, 5758, 280, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[34373, 1026, 381, 12, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34791, 1043, 216, 7, 42, "Subsubsection"],
Cell[35010, 1052, 126, 3, 32, "Text"],
Cell[35139, 1057, 111, 2, 43, "Input"],
Cell[35253, 1061, 100, 3, 32, "Text"],
Cell[35356, 1066, 1160, 22, 137, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36553, 1093, 126, 4, 42, "Subsubsection"],
Cell[36682, 1099, 64, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[36771, 1104, 54, 1, 27, "Input"],
Cell[36828, 1107, 69197, 3701, 272, 43059, 3374, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]],
Cell[106040, 4811, 406, 12, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[106483, 4828, 118, 4, 42, "Subsubsection"],
Cell[106604, 4834, 64, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[106693, 4839, 54, 1, 27, "Input"],
Cell[106750, 4842, 99397, 4075, 272, 43009, 3374, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[206196, 8923, 128, 4, 42, "Subsubsection"],
Cell[206327, 8929, 64, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[206416, 8934, 54, 1, 27, "Input"],
Cell[206473, 8937, 103726, 4127, 272, 43053, 3374, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[310248, 13070, 128, 4, 42, "Subsubsection"],
Cell[310379, 13076, 64, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[310468, 13081, 54, 1, 27, "Input"],
Cell[310525, 13084, 101720, 4103, 272, 43055, 3374, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

