(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     12653,        410]*)
(*NotebookOutlinePosition[     13291,        433]*)
(*  CellTagsIndexPosition[     13247,        429]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Calculations used to create input data", "Subsection"],

Cell[CellGroupData[{

Cell["Tools", "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"innerProd", "[", 
        RowBox[{"\[Psi]_", ",", "\[CurlyPhi]_"}], "]"}], ":=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{"adj", "[", "\[Psi]", "]"}], ".", "\[CurlyPhi]"}], ")"}], 
        "\[LeftDoubleBracket]", 
        RowBox[{"1", ",", "1"}], 
        "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"normalize", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{"\[Psi]", "/", 
          SqrtBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"adj", "[", "\[Psi]", "]"}], ".", "\[Psi]"}], ")"}],
               "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
        RowBox[{"NumberForm", "[", 
          RowBox[{
            RowBox[{"MatrixForm", "[", 
              RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], 
      " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ket0", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", "1", "}"}], ",", 
            RowBox[{"{", "0", "}"}]}], "}"}]}], ";", 
      RowBox[{"ket1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", "0", "}"}], ",", 
            RowBox[{"{", "1", "}"}]}], "}"}]}], ";"}]], "Input"],

Cell["Conversion of stuff to strings:", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"complexToString", "[", "z_", "]"}], ":=", 
        RowBox[{"\"\<(\>\"", "<>", 
          RowBox[{"ToString", "[", 
            RowBox[{"N", "[", 
              RowBox[{"Re", "[", "z", "]"}], "]"}], "]"}], "<>", "\"\<,\>\"", 
          "<>", 
          RowBox[{"ToString", "[", 
            RowBox[{"N", "[", 
              RowBox[{"Im", "[", "z", "]"}], "]"}], "]"}], "<>", 
          "\"\<)\>\""}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"stateToString", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{
          RowBox[{"complexToString", "[", 
            RowBox[{"\[Psi]", "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
          "<>", "\"\< \>\"", "<>", 
          RowBox[{"complexToString", "[", 
            RowBox[{"\[Psi]", "\[LeftDoubleBracket]", 
              RowBox[{"2", ",", "1"}], "\[RightDoubleBracket]"}], "]"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"opToString", "[", "a_", "]"}], ":=", 
      RowBox[{"StringJoin", "[", 
        RowBox[{"Flatten", "[", 
          RowBox[{"Table", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"complexToString", "[", 
                  RowBox[{"a", "\[LeftDoubleBracket]", 
                    RowBox[{"j", ",", "k"}], "\[RightDoubleBracket]"}], "]"}],
                 "<>", "\"\< \>\""}], ",", 
              RowBox[{"{", 
                RowBox[{"j", ",", "1", ",", "2"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"k", ",", "1", ",", "2"}], "}"}]}], "]"}], "]"}], 
        "]"}]}]}], "Input"],

Cell[TextData[{
  "Measurement operators with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          OverscriptBox["p", "_"], "=", 
          FractionBox["1", "2"]}], TraditionalForm]]],
  ":"
}], "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"m0", "[", "\[CapitalDelta]p_", "]"}], ":=", 
        RowBox[{
          RowBox[{
            SqrtBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "-", "\[CapitalDelta]p"}], ")"}], "/", "2"}]], 
            
            RowBox[{"ket0", ".", 
              RowBox[{"adj", "[", "ket0", "]"}]}]}], "+", 
          RowBox[{
            SqrtBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "+", "\[CapitalDelta]p"}], ")"}], "/", "2"}]], 
            
            RowBox[{"ket1", ".", 
              RowBox[{"adj", "[", "ket1", "]"}]}]}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"m1", "[", "\[CapitalDelta]p_", "]"}], ":=", 
        RowBox[{
          RowBox[{
            SqrtBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "+", "\[CapitalDelta]p"}], ")"}], "/", "2"}]], 
            
            RowBox[{"ket0", ".", 
              RowBox[{"adj", "[", "ket0", "]"}]}]}], "+", 
          RowBox[{
            SqrtBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "-", "\[CapitalDelta]p"}], ")"}], "/", "2"}]], 
            
            RowBox[{"ket1", ".", 
              RowBox[{"adj", "[", "ket1", "]"}]}]}]}]}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Miscellaneous", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.0", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.0", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.707107,0.) (0.,0.) (0.,0.) (0.707107,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.707107,0.) (0.,0.) (0.,0.) (0.707107,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.2", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.2", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.632456,0.) (0.,0.) (0.,0.) (0.774597,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.774597,0.) (0.,0.) (0.,0.) (0.632456,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.4", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.4", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.547723,0.) (0.,0.) (0.,0.) (0.83666,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.83666,0.) (0.,0.) (0.,0.) (0.547723,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.6", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.6", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.447214,0.) (0.,0.) (0.,0.) (0.894427,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.894427,0.) (0.,0.) (0.,0.) (0.447214,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.8", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.8", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.316228,0.) (0.,0.) (0.,0.) (0.948683,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.948683,0.) (0.,0.) (0.,0.) (0.316228,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "1.0", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "1.0", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.,0.) (0.,0.) (0.,0.) (1.,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(1.,0.) (0.,0.) (0.,0.) (0.,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"ee0", "=", 
        RowBox[{"-", "0.196349"}]}], ";", 
      RowBox[{"ee1", "=", "0.196349"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"\[CapitalOmega]R", "=", 
      RowBox[{"ee1", "-", "ee0"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"ttR", "=", 
        RowBox[{"2", 
          RowBox[{"\[Pi]", "/", "\[CapitalOmega]R"}]}]}], 
      RowBox[{"(*", 
        RowBox[{"dim", 
          RowBox[{"(", "t", ")"}]}], "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"\[Tau]", "=", "1"}], ";"}], 
      RowBox[{"(*", 
        RowBox[{"dim", 
          RowBox[{"(", "t", ")"}]}], "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{"\[Tau]", "/", "ttR"}], "\[IndentingNewLine]", 
    RowBox[{"1", "/", "16."}]}], "Input"],

Cell[BoxData["0.392698`"], "Output"],

Cell[BoxData["16.000044072492315`"], "Output"],

Cell[BoxData["0.062499827842301105`"], "Output"],

Cell[BoxData["0.0625`"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"\[Tau]", "=", 
      RowBox[{"1.25", "/", 
        RowBox[{"(", 
          RowBox[{"1", "/", "16."}], ")"}]}]}], "\[IndentingNewLine]", 
    RowBox[{"\[Tau]", "/", "ttR"}]}], "Input"],

Cell[BoxData["20.`"], "Output"],

Cell[BoxData["1.249996556846022`"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"nrecpoints", "=", 
      RowBox[{"1.25", "/", 
        RowBox[{"(", 
          RowBox[{"1", "/", "256"}], ")"}]}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"2", "^", "20"}], "/", "nrecpoints"}], "\[IndentingNewLine]", 
    RowBox[{"2", "^", "12"}]}], "Input"],

Cell[BoxData["320.`"], "Output"],

Cell[BoxData["3276.8`"], "Output"],

Cell[BoxData["4096"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"2", "^", "20"}], "/", "20."}], "\[IndentingNewLine]", 
    RowBox[{"2", "^", "16"}]}], "Input"],

Cell[BoxData["52428.8`"], "Output"],

Cell[BoxData["65536"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"ee0", "=", "0.020944"}], ";", 
      RowBox[{"ee1", "=", 
        RowBox[{"-", "0.020944"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"\[CapitalOmega]R", "=", 
      RowBox[{"ee1", "-", "ee0"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"ttR", "=", 
        RowBox[{"2", 
          RowBox[{"\[Pi]", "/", "\[CapitalOmega]R"}]}]}], 
      RowBox[{"(*", 
        RowBox[{"dim", 
          RowBox[{"(", "t", ")"}]}], "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"\[Tau]", "=", "9.375"}], 
      RowBox[{"(*", 
        RowBox[{"dim", 
          RowBox[{"(", "t", ")"}]}], "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{"\[Tau]", "/", "ttR"}], "\[IndentingNewLine]", 
    RowBox[{"1", "/", "16."}]}], "Input"],

Cell[BoxData[
    RowBox[{"-", "0.041888`"}]], "Output"],

Cell[BoxData[
    RowBox[{"-", "149.99964923557073`"}]], "Output"],

Cell[BoxData["9.375`"], "Output"],

Cell[BoxData[
    RowBox[{"-", "0.0625001461521873`"}]], "Output"],

Cell[BoxData["0.0625`"], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 60, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[1824, 55, 30, 0, 42, "Subsubsection"],
Cell[1857, 57, 1456, 40, 105, "Input"],
Cell[3316, 99, 369, 11, 27, "Input"],
Cell[3688, 112, 47, 0, 32, "Text"],
Cell[3738, 114, 1682, 40, 59, "Input"],
Cell[5423, 156, 210, 8, 34, "Text"],
Cell[5636, 166, 1392, 41, 59, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7065, 212, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[7128, 216, 205, 4, 43, "Input"],
Cell[7336, 222, 79, 0, 27, "Output"],
Cell[7418, 224, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7534, 229, 205, 4, 43, "Input"],
Cell[7742, 235, 79, 0, 27, "Output"],
Cell[7824, 237, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7940, 242, 205, 4, 43, "Input"],
Cell[8148, 248, 78, 0, 27, "Output"],
Cell[8229, 250, 78, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8344, 255, 205, 4, 43, "Input"],
Cell[8552, 261, 79, 0, 27, "Output"],
Cell[8634, 263, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8750, 268, 205, 4, 43, "Input"],
Cell[8958, 274, 79, 0, 27, "Output"],
Cell[9040, 276, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9156, 281, 205, 4, 43, "Input"],
Cell[9364, 287, 67, 0, 27, "Output"],
Cell[9434, 289, 67, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9538, 294, 796, 21, 107, "Input"],
Cell[10337, 317, 36, 0, 27, "Output"],
Cell[10376, 319, 46, 0, 27, "Output"],
Cell[10425, 321, 48, 0, 27, "Output"],
Cell[10476, 323, 34, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10547, 328, 211, 5, 43, "Input"],
Cell[10761, 335, 31, 0, 27, "Output"],
Cell[10795, 337, 45, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10877, 342, 299, 7, 59, "Input"],
Cell[11179, 351, 32, 0, 27, "Output"],
Cell[11214, 353, 34, 0, 27, "Output"],
Cell[11251, 355, 31, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11319, 360, 138, 3, 43, "Input"],
Cell[11460, 365, 35, 0, 27, "Output"],
Cell[11498, 367, 32, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11567, 372, 776, 20, 107, "Input"],
Cell[12346, 394, 56, 1, 27, "Output"],
Cell[12405, 397, 66, 1, 27, "Output"],
Cell[12474, 400, 33, 0, 27, "Output"],
Cell[12510, 402, 66, 1, 27, "Output"],
Cell[12579, 405, 34, 0, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

