(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     89709,       2867]*)
(*NotebookOutlinePosition[     91011,       2906]*)
(*  CellTagsIndexPosition[     90967,       2902]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Example", "Section"],

Cell["\<\
Below are shown power spectra of some functions. Note the symmetry \
in power spectra of real valued functions.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Tools", "Subsection"],

Cell[BoxData[
    RowBox[{"<<", "Miscellaneous`Units`"}]], "Input"],

Cell["Fourier transform:", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"gg", "[", 
        RowBox[{"n_", ",", "nn_", ",", "\[Tau]_", ",", "g_"}], "]"}], ":=", 
      RowBox[{
        FractionBox["\[Tau]", 
          SqrtBox[
            RowBox[{"2", "\[Pi]"}]]], 
        RowBox[{"Sum", "[", 
          RowBox[{
            RowBox[{"Conjugate", "[", 
              RowBox[{
                RowBox[{"Exp", "[", 
                  RowBox[{
                    RowBox[{"-", "2"}], "\[Pi]", " ", "I", " ", "k", " ", 
                    RowBox[{"n", "/", "nn"}]}], "]"}], 
                RowBox[{"g", "[", 
                  RowBox[{"k", " ", "\[Tau]"}], "]"}]}], "]"}], ",", 
            RowBox[{"{", 
              RowBox[{"k", ",", "0", ",", 
                RowBox[{"nn", "-", "1"}]}], "}"}]}], "]"}]}]}]], "Input"],

Cell["Power specturm:", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"powspec", "[", 
        RowBox[{"nn_", ",", "\[Tau]_", ",", "g_"}], "]"}], ":=", 
      RowBox[{"Table", "[", 
        RowBox[{
          SuperscriptBox[
            RowBox[{"Abs", "[", 
              RowBox[{"gg", "[", 
                RowBox[{"n", ",", "nn", ",", "\[Tau]", ",", "g"}], "]"}], 
              "]"}], "2"], ",", 
          RowBox[{"{", 
            RowBox[{"n", ",", "0", ",", 
              RowBox[{"nn", "-", "1"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Power spectra", "Subsection"],

Cell[CellGroupData[{

Cell["Real valued function", "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"g1", "[", "t_", "]"}], ":=", 
        RowBox[{
          RowBox[{"Exp", "[", 
            RowBox[{"-", 
              SuperscriptBox[
                RowBox[{"(", 
                  RowBox[{"3", "-", 
                    RowBox[{"t", "/", "Second"}]}], ")"}], "2"]}], "]"}], 
          RowBox[{"Sin", "[", 
            RowBox[{"t", "/", "Second"}], "]"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"\[Tau]", "=", 
        RowBox[{"0.06", "Second"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"nn", "=", "100"}], ";"}]}], "Input"],

Cell["Time spectrum:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"Plot", "[", 
        RowBox[{
          RowBox[{"g1", "[", 
            RowBox[{"t", " ", "Second"}], "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"t", ",", "0", ",", 
              RowBox[{"nn", " ", 
                RowBox[{"\[Tau]", "/", "Second"}]}]}], "}"}], ",", 
          RowBox[{"AxesLabel", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"\"\<t/s\>\"", ",", "\"\<\>\""}], "}"}]}]}], "]"}], 
      ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.15873 0.255116 0.737886 [
[.18254 .24262 -3 -9 ]
[.18254 .24262 3 0 ]
[.34127 .24262 -3 -9 ]
[.34127 .24262 3 0 ]
[.5 .24262 -3 -9 ]
[.5 .24262 3 0 ]
[.65873 .24262 -3 -9 ]
[.65873 .24262 3 0 ]
[.81746 .24262 -3 -9 ]
[.81746 .24262 3 0 ]
[.97619 .24262 -3 -9 ]
[.97619 .24262 3 0 ]
[1.025 .25512 0 -4.90625 ]
[1.025 .25512 22 4.90625 ]
[.01131 .10754 -24 -4.5 ]
[.01131 .10754 0 4.5 ]
[.01131 .40269 -18 -4.5 ]
[.01131 .40269 0 4.5 ]
[.01131 .55027 -18 -4.5 ]
[.01131 .55027 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .25512 m
.18254 .26137 L
s
[(1)] .18254 .24262 0 1 Mshowa
.34127 .25512 m
.34127 .26137 L
s
[(2)] .34127 .24262 0 1 Mshowa
.5 .25512 m
.5 .26137 L
s
[(3)] .5 .24262 0 1 Mshowa
.65873 .25512 m
.65873 .26137 L
s
[(4)] .65873 .24262 0 1 Mshowa
.81746 .25512 m
.81746 .26137 L
s
[(5)] .81746 .24262 0 1 Mshowa
.97619 .25512 m
.97619 .26137 L
s
[(6)] .97619 .24262 0 1 Mshowa
.125 Mabswid
.05556 .25512 m
.05556 .25887 L
s
.0873 .25512 m
.0873 .25887 L
s
.11905 .25512 m
.11905 .25887 L
s
.15079 .25512 m
.15079 .25887 L
s
.21429 .25512 m
.21429 .25887 L
s
.24603 .25512 m
.24603 .25887 L
s
.27778 .25512 m
.27778 .25887 L
s
.30952 .25512 m
.30952 .25887 L
s
.37302 .25512 m
.37302 .25887 L
s
.40476 .25512 m
.40476 .25887 L
s
.43651 .25512 m
.43651 .25887 L
s
.46825 .25512 m
.46825 .25887 L
s
.53175 .25512 m
.53175 .25887 L
s
.56349 .25512 m
.56349 .25887 L
s
.59524 .25512 m
.59524 .25887 L
s
.62698 .25512 m
.62698 .25887 L
s
.69048 .25512 m
.69048 .25887 L
s
.72222 .25512 m
.72222 .25887 L
s
.75397 .25512 m
.75397 .25887 L
s
.78571 .25512 m
.78571 .25887 L
s
.84921 .25512 m
.84921 .25887 L
s
.88095 .25512 m
.88095 .25887 L
s
.9127 .25512 m
.9127 .25887 L
s
.94444 .25512 m
.94444 .25887 L
s
.25 Mabswid
0 .25512 m
1 .25512 L
s
gsave
1.025 .25512 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(t) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .10754 m
.03006 .10754 L
s
[(-0.2)] .01131 .10754 1 0 Mshowa
.02381 .40269 m
.03006 .40269 L
s
[(0.2)] .01131 .40269 1 0 Mshowa
.02381 .55027 m
.03006 .55027 L
s
[(0.4)] .01131 .55027 1 0 Mshowa
.125 Mabswid
.02381 .14443 m
.02756 .14443 L
s
.02381 .18133 m
.02756 .18133 L
s
.02381 .21822 m
.02756 .21822 L
s
.02381 .29201 m
.02756 .29201 L
s
.02381 .32891 m
.02756 .32891 L
s
.02381 .3658 m
.02756 .3658 L
s
.02381 .43959 m
.02756 .43959 L
s
.02381 .47648 m
.02756 .47648 L
s
.02381 .51338 m
.02756 .51338 L
s
.02381 .07064 m
.02756 .07064 L
s
.02381 .03375 m
.02756 .03375 L
s
.02381 .58717 m
.02756 .58717 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .25512 m
.02846 .25512 L
.03279 .25512 L
.03754 .25513 L
.04262 .25514 L
.04795 .25515 L
.053 .25516 L
.05753 .25518 L
.06244 .25521 L
.06751 .25524 L
.07308 .25528 L
.0789 .25534 L
.08435 .25541 L
.0941 .25557 L
.09957 .2557 L
.10458 .25584 L
.11442 .25621 L
.12357 .25668 L
.1336 .2574 L
.14429 .25846 L
.15457 .25987 L
.1657 .26193 L
.17621 .26454 L
.18587 .26764 L
.2047 .27616 L
.21538 .2828 L
.22536 .29042 L
.24571 .31095 L
.26427 .33629 L
.30411 .41297 L
.34244 .50475 L
.36183 .54838 L
.37212 .56813 L
.37753 .57713 L
.38322 .58538 L
.38827 .59154 L
.39379 .5969 L
.39622 .59877 L
.39881 .60043 L
.40125 .60167 L
.40348 .6025 L
.40466 .60282 L
.40592 .60308 L
.40699 .60323 L
.40817 .60332 L
.40944 .60332 L
.41015 .60328 L
.4108 .60321 L
.41206 .60301 L
.41326 .60273 L
Mistroke
.41437 .60239 L
.41555 .60195 L
.41767 .60093 L
.4202 .59935 L
.42249 .59757 L
.42738 .59263 L
.43254 .58573 L
.44178 .56909 L
.45171 .54515 L
.46261 .5122 L
.50309 .345 L
.54205 .17052 L
.55225 .13166 L
.56333 .09494 L
.5729 .06845 L
.58346 .04535 L
.58838 .03681 L
.59368 .02922 L
.59869 .02355 L
.60327 .01963 L
.60581 .01796 L
.60817 .01672 L
.61076 .01572 L
.61221 .0153 L
.61352 .01503 L
.61419 .01492 L
.61491 .01483 L
.61618 .01473 L
.61689 .01472 L
.61756 .01472 L
.61828 .01475 L
.61906 .01481 L
.61974 .01489 L
.62047 .015 L
.6218 .01526 L
.62309 .01559 L
.62429 .01597 L
.62892 .01802 L
.63152 .01958 L
.6339 .02124 L
.64293 .02944 L
.65296 .04155 L
.66353 .05695 L
.7016 .12224 L
.72283 .15715 L
.74213 .18436 L
.76242 .20723 L
.78115 .22314 L
.80031 .23487 L
.8104 .23948 L
Mistroke
.8211 .24338 L
.83052 .24611 L
.84064 .24842 L
.85021 .25012 L
.85895 .25133 L
.86881 .25238 L
.87795 .25312 L
.88799 .25372 L
.89356 .25398 L
.89868 .25418 L
.90897 .25449 L
.91439 .25461 L
.92011 .25472 L
.92973 .25485 L
.93519 .25491 L
.94028 .25495 L
.9449 .25498 L
.94993 .25501 L
.95436 .25503 L
.95907 .25505 L
.9633 .25506 L
.96796 .25508 L
.97619 .25509 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`006_oo00<00?ooool0
oooo1?oo000Jool00`00ooooo`3oool4ool001[oo`03003ooooo0?ooo`Coo`006_oo00<00?ooool0
ROoo1000M_oo000Jool00`00ooooo`27ool20004ool2001dool001[oo`03003ooooo08Koo`03003o
oooo00Koo`03003ooooo077oo`006_oo00<00?ooool0Q_oo00<00?ooool01ooo00<00?ooool0L?oo
000Jool00`00ooooo`25ool00`00ooooo`09ool00`00ooooo`1_ool001[oo`8008Goo`03003ooooo
00_oo`03003ooooo06koo`006_oo00<00?ooool0Q?oo00<00?ooool03?oo00<00?ooool0KOoo000J
ool00`00ooooo`23ool00`00ooooo`0>ool00`00ooooo`1/ool001[oo`03003ooooo08?oo`03003o
oooo00koo`03003ooooo06coo`006_oo00<00?ooool0P_oo00<00?ooool04?oo00<00?ooool0Jooo
000Jool00`00ooooo`22ool00`00ooooo`0Aool00`00ooooo`1Zool001[oo`03003ooooo087oo`03
003ooooo01?oo`03003ooooo06Woo`006_oo00<00?ooool0POoo00<00?ooool04ooo00<00?ooool0
JOoo000Jool20021ool00`00ooooo`0Eool00`00ooooo`1Xool001[oo`03003ooooo083oo`03003o
oooo01Goo`03003ooooo06Soo`006_oo00<00?ooool0Oooo00<00?ooool05ooo00<00?ooool0Iooo
000Jool00`00ooooo`1oool00`00ooooo`0Gool00`00ooooo`1Wool001[oo`03003ooooo07ooo`03
003ooooo01Soo`03003ooooo06Koo`006_oo00<00?ooool0O_oo00<00?ooool06Ooo00<00?ooool0
I_oo000Jool00`00ooooo`1nool00`00ooooo`0Jool00`00ooooo`1Uool000Soo`8000Coo`03003o
oooo00;oo`@000?oo`03003ooooo07koo`03003ooooo01_oo`03003ooooo06Coo`001ooo00@00?oo
ool000Soo`03003ooooo00Coo`03003ooooo07goo`03003ooooo01coo`03003ooooo06Coo`001ooo
00@00?ooool000Woo`03003ooooo00?oo`<007goo`03003ooooo01goo`03003ooooo06?oo`0000?o
o`0000000P000_oo00@00?ooool000[oo`03003ooooo00;oo`03003ooooo07goo`03003ooooo01go
o`03003ooooo06?oo`001ooo00@00?ooool000_oo`05003oooooool0001nool00`00ooooo`0Oool0
0`00ooooo`1Rool000Ooo`04003ooooo0008ool01000ooooo`000ooo00<00?ooool0O?oo00<00?oo
ool07ooo00<00?ooool0H_oo0008ool2000:ool20004ool00`00ooooo`1lool00`00ooooo`0Pool0
0`00ooooo`1Qool001[oo`03003ooooo07_oo`03003ooooo02;oo`03003ooooo063oo`006_oo00<0
0?ooool0Nooo00<00?ooool08_oo00<00?ooool0H?oo000Jool00`00ooooo`1kool00`00ooooo`0S
ool00`00ooooo`1Oool001[oo`03003ooooo07_oo`03003ooooo02?oo`03003ooooo05ooo`006_oo
0P00Nooo00<00?ooool09Ooo00<00?ooool0G_oo000Jool00`00ooooo`1jool00`00ooooo`0Vool0
0`00ooooo`1Mool001[oo`03003ooooo07[oo`03003ooooo02Koo`03003ooooo05goo`006_oo00<0
0?ooool0N_oo00<00?ooool09ooo00<00?ooool0G?oo000Jool00`00ooooo`1iool00`00ooooo`0Y
ool00`00ooooo`1Kool001[oo`03003ooooo07Woo`03003ooooo02Woo`03003ooooo05_oo`006_oo
00<00?ooool0NOoo00<00?ooool0:_oo00<00?ooool0F_oo000Jool00`00ooooo`1iool00`00oooo
o`0[ool00`00ooooo`1Iool001[oo`03003ooooo07Woo`03003ooooo02_oo`03003ooooo05Woo`00
6_oo0P00NOoo00<00?ooool0;Ooo00<00?ooool0F?oo000Jool00`00ooooo`1hool00`00ooooo`0^
ool00`00ooooo`1Gool001[oo`03003ooooo07Soo`03003ooooo02ooo`03003ooooo05Koo`006_oo
00<00?ooool0N?oo00<00?ooool0;ooo00<00?ooool0E_oo000Jool00`00ooooo`1gool00`00oooo
o`0aool00`00ooooo`1Eool001[oo`03003ooooo07Ooo`03003ooooo03;oo`03003ooooo05Coo`00
6_oo00<00?ooool0Mooo00<00?ooool0<ooo00<00?ooool0Dooo000Jool00`00ooooo`0Qool4000R
ool4000Sool20007ool00`00ooooo`0Jool00`00ooooo`0Gool00`00ooooo`07ool3000Tool2000R
ool001[oo`8002Coo`03003ooooo027oo`03003ooooo02?oo`04003ooooo0005ool00`00ooooo`0K
ool00`00ooooo`0Hool2000:ool00`00ooooo`0Pool01000ooooo`008Ooo000Jool00`00ooooo`0S
ool00`00ooooo`0Rool00`00ooooo`0Uool00`00ooooo`03ool00`00ooooo`0Hool5000Kool00`00
ooooo`07ool00`00ooooo`0Pool01000ooooo`008Ooo000Jool00`00ooooo`0Sool00`00ooooo`0S
ool00`00ooooo`0Rool20006ool00`00ooooo`0Hool01000ooooo`007Ooo00<00?ooool00ooo0`00
8ooo0`008_oo000Jool00`00ooooo`0Sool00`00ooooo`0Tool00`00ooooo`0Sool00`00ooooo`03
ool00`00ooooo`0Iool00`00ool0000Nool20003ool00`00ooooo`0Sool00`00ooooo`0Rool001[o
o`03003ooooo027oo`<002?oo`04003ooooo000Rool01000ooooo`001?oo00<00?ooool06ooo0P00
8?oo0P0000?oo`00ool09?oo00<00?ooool08_oo000Jool00`00ooooo`0Sool00`00ooooo`0Rool2
000Tool20005ool00`00ooooo`0Lool00`00ooooo`0Pool5000Sool3000Qool001[oo`03003ooooo
07Goo`03003ooooo04;oo`8004Ooo`006_oo00<00?ooool0MOoo00<00?ooool0A?oo0`00=_oo00<0
0?ooool02ooo000Jool00`00ooooo`1dool00`00ooooo`18ool6000/ool20003ool00`00ooooo`02
ool30005ool001Goonl000Woo`04003ooooo0002ool00`00ooooo`05ool00`00ooooo`02ool001[o
o`03003ooooo00Goo`03003ooooo00Goo`03003ooooo00Coo`X000Goo`03003ooooo00Goo`03003o
oooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003o
oooo00Goo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00;oo`03003o
o`0000Ooo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003o
oooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003o
oooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Goo`03003ooooo00goo`03003o
oooo00?oo`03003ooooo00?oo`8000Goo`006_oo00<00?ooool07_oo1P009Ooo00<00?ooool08ooo
00<00?ooool00_oo00<00?ooool07Ooo00<00?ooool08ooo00<00?ooool08ooo00<00?ooool03Ooo
00<00?ooool01?oo00D00?ooooooo`0000Ooo`006_oo00<00?ooool08ooo0P00Cooo00<00?ooool0
N?oo10001?oo00<00?ooool00_oo0`001?oo000Jool00`00ooooo`0Uool2001<ool00`00ooooo`1j
ool00`00ooooo`05ool00`00ooooo`08ool001[oo`03003ooooo02Ooo`8004[oo`03003ooooo07[o
o`03003ooooo013oo`006_oo00<00?ooool0:Ooo0P00B?oo00<00?ooool0SOoo000Jool00`00oooo
o`0[ool00`00ooooo`15ool00`00ooooo`2=ool001[oo`03003ooooo02coo`03003ooooo04?oo`03
003ooooo08koo`006_oo0P00;_oo00<00?ooool0@_oo00<00?ooool0S_oo000Jool00`00ooooo`0^
ool00`00ooooo`11ool00`00ooooo`2>ool001[oo`03003ooooo02ooo`80047oo`03003ooooo08ko
o`006_oo00<00?ooool0<Ooo00<00?ooool0?Ooo00<00?ooool0Sooo000Jool00`00ooooo`0bool0
0`00ooooo`0lool00`00ooooo`2?ool001[oo`03003ooooo03?oo`03003ooooo03_oo`03003ooooo
08ooo`006_oo00<00?ooool0<ooo00<00?ooool0>ooo00<00?ooool0Sooo000Jool00`00ooooo`0d
ool00`00ooooo`0iool00`00ooooo`2@ool001[oo`03003ooooo03Goo`03003ooooo03Soo`03003o
oooo093oo`006_oo0P00=ooo00<00?ooool0=ooo00<00?ooool0T?oo000Jool00`00ooooo`0fool0
0`00ooooo`0gool00`00ooooo`2@ool001[oo`03003ooooo03Ooo`03003ooooo03Goo`03003ooooo
097oo`006_oo00<00?ooool0=ooo00<00?ooool0=Ooo00<00?ooool0TOoo000Jool00`00ooooo`0h
ool00`00ooooo`0dool00`00ooooo`2Aool001[oo`03003ooooo03Soo`03003ooooo03Coo`03003o
oooo097oo`006_oo00<00?ooool0>Ooo00<00?ooool0<ooo00<00?ooool0TOoo000Jool00`00oooo
o`0iool00`00ooooo`0bool00`00ooooo`2Bool001[oo`03003ooooo03[oo`03003ooooo037oo`03
003ooooo09;oo`006_oo0P00>ooo00<00?ooool0<Ooo00<00?ooool0T_oo000Jool00`00ooooo`0k
ool00`00ooooo`0`ool00`00ooooo`2Bool001[oo`03003ooooo03_oo`03003ooooo02ooo`03003o
oooo09?oo`006_oo00<00?ooool0??oo00<00?ooool0;_oo00<00?ooool0Tooo000Jool00`00oooo
o`0lool00`00ooooo`0^ool00`00ooooo`2Cool001[oo`03003ooooo03goo`03003ooooo02goo`03
003ooooo09?oo`002?oo0P001?oo00<00?ooool00_oo10000ooo00<00?ooool0?Ooo00<00?ooool0
;?oo00<00?ooool0U?oo0007ool01000ooooo`002?oo00<00?ooool01?oo00<00?ooool0?_oo00<0
0?ooool0:ooo00<00?ooool0U?oo0007ool01000ooooo`002Ooo00<00?ooool00ooo0`00?_oo00<0
0?ooool0:ooo00<00?ooool0U?oo0007ool01000ooooo`002_oo00<00?ooool00_oo00<00?ooool0
?ooo00<00?ooool0:_oo00<00?ooool0U?oo0007ool01000ooooo`002ooo00D00?ooooooo`00047o
o`03003ooooo02[oo`03003ooooo09Coo`001ooo00@00?ooool000Soo`04003ooooo0003ool00`00
ooooo`10ool00`00ooooo`0Xool00`00ooooo`2Eool000Soo`8000[oo`8000Coo`03003ooooo043o
o`03003ooooo02Soo`03003ooooo09Goo`006_oo00<00?ooool0@Ooo00<00?ooool09ooo00<00?oo
ool0UOoo000Jool00`00ooooo`11ool00`00ooooo`0Wool00`00ooooo`2Eool001[oo`03003ooooo
04;oo`03003ooooo02Goo`03003ooooo09Koo`006_oo00<00?ooool0@_oo00<00?ooool09Ooo00<0
0?ooool0U_oo000Jool20013ool00`00ooooo`0Uool00`00ooooo`2Fool001[oo`03003ooooo04?o
o`03003ooooo02Coo`03003ooooo09Koo`006_oo00<00?ooool0@ooo00<00?ooool08ooo00<00?oo
ool0Uooo000Jool00`00ooooo`14ool00`00ooooo`0Rool00`00ooooo`2Gool001[oo`03003ooooo
04Coo`03003ooooo02;oo`03003ooooo09Ooo`006_oo00<00?ooool0A?oo00<00?ooool08_oo00<0
0?ooool0Uooo000Jool00`00ooooo`15ool00`00ooooo`0Qool00`00ooooo`2Gool001[oo`03003o
oooo04Goo`03003ooooo023oo`03003ooooo09Soo`006_oo00<00?ooool0A_oo00<00?ooool07ooo
00<00?ooool0V?oo000Jool20017ool00`00ooooo`0Oool00`00ooooo`2Hool001[oo`03003ooooo
04Ooo`03003ooooo01koo`03003ooooo09Soo`006_oo00<00?ooool0Aooo00<00?ooool07Ooo00<0
0?ooool0VOoo000Jool00`00ooooo`17ool00`00ooooo`0Mool00`00ooooo`2Iool001[oo`03003o
oooo04Soo`03003ooooo01coo`03003ooooo09Woo`006_oo00<00?ooool0B?oo00<00?ooool07?oo
00<00?ooool0VOoo000Jool00`00ooooo`19ool00`00ooooo`0Jool00`00ooooo`2Jool001[oo`03
003ooooo04Woo`03003ooooo01[oo`03003ooooo09[oo`006_oo00<00?ooool0BOoo00<00?ooool0
6_oo00<00?ooool0V_oo000Jool2001;ool00`00ooooo`0Iool00`00ooooo`2Jool001[oo`03003o
oooo04[oo`03003ooooo01Soo`03003ooooo09_oo`006_oo00<00?ooool0Booo00<00?ooool05ooo
00<00?ooool0Vooo000Jool00`00ooooo`1;ool00`00ooooo`0Gool00`00ooooo`2Kool001[oo`03
003ooooo04coo`03003ooooo01Goo`03003ooooo09coo`006_oo00<00?ooool0C?oo00<00?ooool0
5Ooo00<00?ooool0W?oo0008ool20004ool00`00ooooo`04ool00`00ooooo`02ool00`00ooooo`1=
ool00`00ooooo`0Cool00`00ooooo`2Mool000Ooo`04003ooooo000:ool00`00ooooo`02ool00`00
ooooo`1=ool00`00ooooo`0Cool00`00ooooo`2Mool000Ooo`04003ooooo0007ool50003ool3001>
ool00`00ooooo`0Bool00`00ooooo`2Mool000Ooo`04003ooooo0007ool01000ooooo`001?oo00<0
0?ooool0C_oo00<00?ooool04Ooo00<00?ooool0W_oo0007ool01000ooooo`002?oo00<00?oo0000
1?oo00<00?ooool0Cooo00<00?ooool04?oo00<00?ooool0W_oo0007ool01000ooooo`002Ooo0P00
1?oo00<00?ooool0Cooo00<00?ooool04?oo00<00?ooool0W_oo0008ool2000;ool00`00ooooo`02
ool00`00ooooo`1@ool00`00ooooo`0>ool00`00ooooo`2Oool001[oo`03003ooooo053oo`03003o
oooo00koo`03003ooooo09ooo`006_oo00<00?ooool0DOoo00<00?ooool03Ooo00<00?ooool0Wooo
000Jool00`00ooooo`1Bool00`00ooooo`0;ool00`00ooooo`2Pool001[oo`03003ooooo05;oo`03
003ooooo00_oo`03003ooooo0:3oo`006_oo0P00E?oo00<00?ooool02?oo0P00Xooo000Jool00`00
ooooo`1Dool00`00ooooo`06ool00`00ooooo`2Sool001[oo`03003ooooo05Goo`03003ooooo00Go
o`03003ooooo0:?oo`006_oo00<00?ooool0E_oo0P000ooo0P00Y_oo000Jool00`00ooooo`1Gool5
002Wool001[oo`03003ooooo0?ooo`Coo`006_oo00<00?ooool0oooo1?oo000Jool00`00ooooo`3o
ool4ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003o
oolQool00001\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {584.5, 443.188}} -> {-5.06397, \
2.7306, 0.0265302, 0.00570703}}]
}, Open  ]],

Cell["Power spectrum:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ListPlot", "[", 
        RowBox[{"Simplify", "[", 
          RowBox[{
            RowBox[{"powspec", "[", 
              RowBox[{"nn", ",", "\[Tau]", ",", "g1"}], "]"}], "/", 
            SuperscriptBox[
              RowBox[{"Abs", "[", "Second", "]"}], "2"]}], "]"}], "]"}], 
      ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00952381 0.0147151 1.99791e+11 [
[.21429 .00222 -6 -9 ]
[.21429 .00222 6 0 ]
[.40476 .00222 -6 -9 ]
[.40476 .00222 6 0 ]
[.59524 .00222 -6 -9 ]
[.59524 .00222 6 0 ]
[.78571 .00222 -6 -9 ]
[.78571 .00222 6 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .11461 -37.5 -6.25 ]
[.01131 .11461 0 6.25 ]
[.01131 .21451 -37.5 -6.25 ]
[.01131 .21451 0 6.25 ]
[.01131 .3144 -49.5 -6.25 ]
[.01131 .3144 0 6.25 ]
[.01131 .4143 -37.5 -6.25 ]
[.01131 .4143 0 6.25 ]
[.01131 .51419 -49.5 -6.25 ]
[.01131 .51419 0 6.25 ]
[.01131 .61409 -37.5 -6.25 ]
[.01131 .61409 0 6.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .01472 m
.21429 .02097 L
s
[(20)] .21429 .00222 0 1 Mshowa
.40476 .01472 m
.40476 .02097 L
s
[(40)] .40476 .00222 0 1 Mshowa
.59524 .01472 m
.59524 .02097 L
s
[(60)] .59524 .00222 0 1 Mshowa
.78571 .01472 m
.78571 .02097 L
s
[(80)] .78571 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(100)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.2619 .01472 m
.2619 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.5 .01472 m
.5 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.7381 .01472 m
.7381 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .11461 m
.03006 .11461 L
s
gsave
.01131 .11461 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-13) show
grestore
.02381 .21451 m
.03006 .21451 L
s
gsave
.01131 .21451 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-12) show
grestore
.02381 .3144 m
.03006 .3144 L
s
gsave
.01131 .3144 -49.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(1.5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-12) show
grestore
.02381 .4143 m
.03006 .4143 L
s
gsave
.01131 .4143 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-12) show
grestore
.02381 .51419 m
.03006 .51419 L
s
gsave
.01131 .51419 -49.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(2.5) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-12) show
grestore
.02381 .61409 m
.03006 .61409 L
s
gsave
.01131 .61409 -37.5 -4.25 Mabsadd m
1 1 Mabs scale
/Courier findfont 10 scalefont setfont
(3) show
/Math1Mono findfont 10 scalefont setfont
(\\264) show
/Courier findfont 10 scalefont setfont
(10) show
0 5.25 rmoveto
/Courier findfont 7.5 scalefont setfont
(-12) show
grestore
.125 Mabswid
.02381 .03469 m
.02756 .03469 L
s
.02381 .05467 m
.02756 .05467 L
s
.02381 .07465 m
.02756 .07465 L
s
.02381 .09463 m
.02756 .09463 L
s
.02381 .13459 m
.02756 .13459 L
s
.02381 .15457 m
.02756 .15457 L
s
.02381 .17455 m
.02756 .17455 L
s
.02381 .19453 m
.02756 .19453 L
s
.02381 .23448 m
.02756 .23448 L
s
.02381 .25446 m
.02756 .25446 L
s
.02381 .27444 m
.02756 .27444 L
s
.02381 .29442 m
.02756 .29442 L
s
.02381 .33438 m
.02756 .33438 L
s
.02381 .35436 m
.02756 .35436 L
s
.02381 .37434 m
.02756 .37434 L
s
.02381 .39432 m
.02756 .39432 L
s
.02381 .43428 m
.02756 .43428 L
s
.02381 .45425 m
.02756 .45425 L
s
.02381 .47423 m
.02756 .47423 L
s
.02381 .49421 m
.02756 .49421 L
s
.02381 .53417 m
.02756 .53417 L
s
.02381 .55415 m
.02756 .55415 L
s
.02381 .57413 m
.02756 .57413 L
s
.02381 .59411 m
.02756 .59411 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.10952 .37229 Mdot
.11905 .28037 Mdot
.12857 .24719 Mdot
.1381 .22349 Mdot
.14762 .20513 Mdot
.15714 .19002 Mdot
.16667 .17706 Mdot
.17619 .16564 Mdot
.18571 .15544 Mdot
.19524 .14625 Mdot
.20476 .13795 Mdot
.21429 .13043 Mdot
.22381 .12361 Mdot
.23333 .11742 Mdot
.24286 .11181 Mdot
.25238 .10671 Mdot
.2619 .10208 Mdot
.27143 .09788 Mdot
.28095 .09405 Mdot
.29048 .09057 Mdot
.3 .08741 Mdot
.30952 .08453 Mdot
.31905 .0819 Mdot
.32857 .07952 Mdot
.3381 .07735 Mdot
.34762 .07537 Mdot
.35714 .07358 Mdot
.36667 .07195 Mdot
.37619 .07048 Mdot
.38571 .06915 Mdot
.39524 .06795 Mdot
.40476 .06688 Mdot
.41429 .06592 Mdot
.42381 .06507 Mdot
.43333 .06433 Mdot
.44286 .06368 Mdot
.45238 .06313 Mdot
.4619 .06267 Mdot
.47143 .0623 Mdot
.48095 .06201 Mdot
.49048 .06181 Mdot
.5 .06169 Mdot
.50952 .06165 Mdot
.51905 .06169 Mdot
.52857 .06181 Mdot
.5381 .06201 Mdot
.54762 .0623 Mdot
.55714 .06267 Mdot
.56667 .06313 Mdot
.57619 .06368 Mdot
.58571 .06433 Mdot
.59524 .06507 Mdot
.60476 .06592 Mdot
.61429 .06688 Mdot
.62381 .06795 Mdot
.63333 .06915 Mdot
.64286 .07048 Mdot
.65238 .07195 Mdot
.6619 .07358 Mdot
.67143 .07537 Mdot
.68095 .07735 Mdot
.69048 .07952 Mdot
.7 .0819 Mdot
.70952 .08453 Mdot
.71905 .08741 Mdot
.72857 .09057 Mdot
.7381 .09405 Mdot
.74762 .09788 Mdot
.75714 .10208 Mdot
.76667 .10671 Mdot
.77619 .11181 Mdot
.78571 .11742 Mdot
.79524 .12361 Mdot
.80476 .13043 Mdot
.81429 .13795 Mdot
.82381 .14625 Mdot
.83333 .15544 Mdot
.84286 .16564 Mdot
.85238 .17706 Mdot
.8619 .19002 Mdot
.87143 .20513 Mdot
.88095 .22349 Mdot
.89048 .24719 Mdot
.9 .28037 Mdot
.90952 .37229 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo001Lool40003ool2000Vool00`00ooooo`02ool2000Uool20004ool2000Uool20004ool2
000Rool40003ool20004ool20003ool005coo`03003ooooo00?oo`04003ooooo000Uool01@00oooo
oooo00000_oo00<00?ooool08Ooo00@00?ooool000;oo`04003ooooo000Sool01000ooooo`000_oo
00@00?ooool002?oo`05003oooooool00002ool01000ooooo`000_oo0@000Ooo0Ooo001Mool00`00
ooooo`02ool01000ooooo`008_oo1@000_oo00@00?ooool002?oo`04003ooooo0002ool01000oooo
o`008ooo00@00?ooool000;oo`04003ooooo000Sool01@00oooooooo00000_oo00@00?ooool000;o
o`40007oo`7oo`00G_oo00D00?ooooooo`0000;oo`03003ooooo023oo`04003ooooo0003ool01000
ooooo`008ooo0`000ooo00@00?ooool002Coo`8000?oo`04003ooooo000Sool01@00oooooooo0000
0_oo00@00?ooool000;oo`40007oo`7oo`00Gooo00@00?ooool000;oo`03003ooooo027oo`03003o
o`0000?oo`04003ooooo000Sool00`00ooooo`03ool01000ooooo`008ooo00@00?ooool000;oo`04
003ooooo000Sool01@00oooooooo00000_oo00@00?ooool000;oo`40007oo`7oo`00G?oo00@00?oo
ool000;oo`04003ooooo000Tool20003ool01000ooooo`008ooo00<00?ooool00ooo00@00?ooool0
02?oo`04003ooooo0002ool01000ooooo`008Ooo0`000ooo00@00?ooool000;oo`04003ooooo0002
ool003Coo`03003ooooo02Koo`8000Coo`8002Koo`03003ooooo00;oo`8002Goo`<000?oo`8002Go
o`8000Coo`8002Coo`03003ooooo00;oo`8000Coo`8000?oo`00=?oo00<00?ooool0jOoo000dool0
0`00ooooo`3Yool003Coo`03003ooooo0>Woo`00;_ookP001?oo000dool00`00ooooo`08ool00`00
ooooo`09ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00
ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`08ool00`00
ooooo`08ool00`00ooooo`09ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`08ool00`00
ooooo`09ool00`00ooooo`08ool00`00ooooo`08ool00`00ooooo`09ool00`00ooooo`07ool003Co
o`03003ooooo0>Woo`00=?oo00<00?ooool0jOoo000dool00`00ooooo`3Yool003Coo`800>[oo`00
=?oo00<00?ooool0jOoo000dool00`00ooooo`3Yool003Coo`03003ooooo0>Woo`00=?oo0P00j_oo
000dool00`00ooooo`3Yool003Coo`03003ooooo06goo`06003ooooo003oo`00M_oo000dool00`00
ooooo`1Iool00`00ool00002ool01`00ool00?oo003oo`0000;oo`05003oo`00ool00008ool00`00
ool00002ool01`00ool00?oo003oo`0000;oo`05003oo`00ool0001Rool003Coo`03003ooooo05;o
o`03003oo`0000;oo`03003ooooo02koo`06003ooooo003oo`00Fooo000dool2001<ool01P00oooo
o`00ool003koo`03003oo`0000;oo`03003ooooo05;oo`00=?oo00<00?ooool0Aooo00<00?oo0000
C?oo00<00?oo0000D?oo000dool00`00ooooo`12ool01000ooooo`00E?oo00@00?ooool004_oo`00
=?oo00<00?ooool0@?oo00<00?ooool0G?oo00<00?ooool0Aooo000dool00`00ooooo`0lool00`00
ool0001Rool00`00ool00015ool003Coo`8003[oo`03003ooooo06[oo`03003ooooo043oo`003Ooo
0`000ooo00H00?ooooooo`00ool40003ool2000Bool00`00ooooo`0gool00`00ooooo`1^ool00`00
ooooo`0nool0013oo`07003oooooool00?oo00001?oo00D00?ooooooo`0000;oo`03003ooooo00oo
o`03003ooooo03Goo`03003ooooo07;oo`03003ooooo03coo`004?oo00<00?ooool00_oo00<00?oo
ool00ooo00D00?ooooooo`0000;oo`03003ooooo00ooo`03003ooooo03?oo`03003ooooo07Koo`03
003ooooo03[oo`003Ooo0`001?oo00<00?oo00001?oo00D00?ooooooo`0000;oo`03003ooooo00oo
o`03003ooooo033oo`03003ooooo07coo`03003ooooo03Ooo`003Ooo00<00?ooool00ooo00D00?oo
ooooo`0000?oo`05003oooooool00002ool00`00ooooo`0?ool2003Zool000goo`03003ooooo00Wo
o`<000?oo`04003ooooo000Aool00`00ooooo`0^ool00`00ooooo`20ool00`00ooooo`0eool000go
o`@000[oo`03003ooooo00;oo`8000Soo`<000?oo`05003oooooool0000^ool00`00ooooo`24ool0
0`00ooooo`0cool002_oo`07003oooooool00?oo00000_oo00<00?ooool0jOoo000Uool30003ool0
0`00ooooo`03ool00`00ooooo`02000[ool00`00ooooo`28ool00`00ooooo`0aool002_oo`03003o
oooo00;oo`8000;oo`03003ooooo0>Woo`00:_oo0P000ooo00<00?oo00000_oo00<00?ooool09ooo
00<00?ooool0S_oo00<00?ooool0;_oo000[ool00`00ooooo`02ool01@00oooooooo0000jooo000d
ool00`00ooooo`0Uool00`00ooooo`2Bool00`00ooooo`0/ool003Coo`800>[oo`00=?oo00<00?oo
ool08ooo00<00?ooool0U_oo00<00?ooool0:_oo000dool00`00ooooo`3Yool003Coo`03003ooooo
027oo`03003ooooo09[oo`03003ooooo02Soo`00=?oo00<00?ooool0jOoo000dool2003Zool003Co
o`03003ooooo01koo`03003ooooo0:3oo`03003ooooo02Goo`00=?oo00<00?ooool0jOoo000dool0
0`00ooooo`3Yool003Coo`8001goo`03003ooooo0:Coo`03003ooooo02?oo`00=?oo00<00?ooool0
jOoo000=ool40002ool01P00oooooooo003oo`@000?oo`8001;oo`03003ooooo0>Woo`003ooo00<0
0?ooool00_oo00<00?oo00001?oo00D00?ooooooo`0000;oo`03003ooooo00ooo`03003ooooo01[o
o`03003ooooo0:Soo`03003ooooo027oo`003ooo00<00?ooool00ooo00<00?ooool00ooo00D00?oo
ooooo`0000;oo`03003ooooo00ooo`03003ooooo0>Woo`003ooo00<00?ooool00_oo00<00?oo0000
1?oo00D00?ooooooo`0000;oo`03003ooooo00ooo`800>[oo`003ooo00D00?ooooooo`0000?oo`05
003oooooool00003ool01000ooooo`004Ooo00<00?ooool0jOoo000=ool30009ool30003ool01000
ooooo`001ooo0`000_oo0`000_oo00<00?ooool0jOoo000?ool00`00ooooo`09ool00`00ooooo`02
ool20009ool01@00oooooooo00001?oo00<00?ooool06?oo00<00?ooool0[Ooo00<00?ooool07_oo
000Uool30003ool00`00ooooo`02ool01@00oooooooo0000jooo000[ool00`00ooooo`03ool00`00
ooooo`02003Zool002[oo`8000?oo`03003oo`0000;oo`03003ooooo0>Woo`00:ooo00<00?ooool0
0_oo00D00?ooooooo`000>_oo`00=?oo00<00?ooool05Ooo00<00?ooool0/_oo00<00?ooool07?oo
000dool00`00ooooo`3Yool003Coo`800>[oo`00=?oo00<00?ooool0jOoo000dool00`00ooooo`3Y
ool003Coo`03003ooooo0>Woo`00=?oo0P00j_oo000dool00`00ooooo`3Yool003Coo`03003ooooo
01?oo`03003ooooo0;Koo`03003ooooo01[oo`00=?oo00<00?ooool0jOoo000dool00`00ooooo`3Y
ool003Coo`800>[oo`00=?oo00<00?ooool0jOoo00000ooo000000020003ool00`00ooooo`02ool3
0003ool01P00oooooooo003oo`@000?oo`8001;oo`03003ooooo0>Woo`000ooo00<00?ooool02_oo
00L00?ooooooo`00ool00004ool01@00oooooooo00000_oo00<00?ooool03ooo00<00?ooool0jOoo
0003ool00`00ooooo`0:ool00`00ooooo`02ool00`00ooooo`03ool01@00oooooooo00000_oo00<0
0?ooool03ooo00<00?ooool0jOoo0003ool00`00ooooo`07ool30004ool00`00ool00004ool01@00
oooooooo00000_oo00<00?ooool03ooo0P00j_oo0003ool00`00ooooo`07ool00`00ooooo`03ool0
1@00oooooooo00000ooo00D00?ooooooo`0000;oo`03003ooooo00ooo`03003ooooo0>Woo`0000Co
o`00000000Woo`03003ooooo00Woo`<000?oo`04003ooooo0007ool30002ool30002ool00`00oooo
o`3Yool000?oo`03003ooooo00Ooo`@000[oo`03003ooooo00;oo`8000Woo`05003oooooool00004
ool00`00ooooo`3Yool002Goo`<000?oo`03003ooooo00;oo`05003oooooool0003[ool002_oo`03
003ooooo00?oo`03003ooooo00800>[oo`00:_oo0P000ooo00<00?oo00000_oo00<00?ooool0jOoo
000[ool00`00ooooo`02ool01@00oooooooo0000jooo000dool00`00ooooo`3Yool003Coo`800>[o
o`00=?oo00<00?ooool0jOoo000dool00`00ooooo`3Yool003Coo`03003ooooo0>Woo`00=?oo00<0
0?ooool0jOoo000dool2000Bool00`00ooooo`2jool00`00ooooo`0Hool003Coo`03003ooooo0>Wo
o`00=?oo00<00?ooool0jOoo000dool00`00ooooo`3Yool003Coo`03003ooooo0>Woo`00=?oo0P00
j_oo000=ool40002ool01P00oooooooo003oo`@000?oo`8001;oo`03003ooooo0>Woo`003Ooo00<0
0?ooool01?oo00<00?oo00001?oo00D00?ooooooo`0000;oo`03003ooooo00ooo`03003ooooo0>Wo
o`003_oo00<00?ooool01?oo00<00?ooool00ooo00D00?ooooooo`0000;oo`03003ooooo00ooo`03
003ooooo0>Woo`003ooo00<00?ooool00_oo00<00?oo00001?oo00D00?ooooooo`0000;oo`03003o
oooo00ooo`03003ooooo0>Woo`004?oo00@00?ooool000?oo`05003oooooool00003ool01000oooo
o`004Ooo0P00j_oo000=ool01000ooooo`002?oo0`000ooo00@00?ooool0017oo`03003ooooo0>Wo
o`003_oo0P002ooo00<00?ooool00_oo0P002?oo0`000_oo0`000_oo00<00?ooool0jOoo000[ool0
1@00oooooooo00001?oo00<00?ooool0jOoo000Uool30003ool00`00ooooo`02ool01000oooooooo
0P00j_oo000[ool00`00ooooo`03ool01000ooooo`00jooo000Zool20003ool00`00ool00002ool0
0`00ooooo`3Yool002_oo`03003ooooo00;oo`05003oooooool0003[ool003Coo`03003ooooo0>Wo
o`00=?oo0P00j_oo000dool00`00ooooo`3Yool003Coo`03003ooooo0>Woo`00=?oo00<00?ooool0
jOoo000dool00`00ooooo`3Yool003Coo`800>[oo`00=?oo00<00?ooool0jOoo000dool00`00oooo
o`3Yool003Coo`03003ooooo0>Woo`00=?oo0P00j_oo000dool00`00ooooo`3Yool00003ool00000
008000?oo`03003ooooo00;oo`<000?oo`06003oooooool00?oo10000ooo0P004_oo00<00?ooool0
jOoo00000ooo003oo`0=ool01`00oooooooo003oo`0000Coo`05003oooooool00002ool00`00oooo
o`0?ool00`00ooooo`3Yool000;oo`03003ooooo00_oo`03003ooooo00;oo`03003ooooo00?oo`05
003oooooool00002ool00`00ooooo`0?ool00`00ooooo`3Yool000?oo`03003ooooo00Ooo`<000Co
o`03003oo`0000Coo`05003oooooool00002ool00`00ooooo`0?ool2003Zool000Coo`03003ooooo
00Koo`03003ooooo00?oo`05003oooooool00003ool01@00oooooooo00000_oo00<00?ooool03ooo
00<00?ooool0jOoo00001Ooo003ooooo00002?oo00<00?ooool02Ooo0`000ooo00@00?ooool000Oo
o`<000;oo`<000;oo`03003ooooo0>Woo`000_oo0P002Ooo10002_oo00<00?ooool00_oo0P002Ooo
00D00?ooooooo`0000Coo`03003ooooo0>Woo`009Ooo0`000ooo00<00?ooool00_oo00D00?oooooo
o`000>_oo`00:ooo00<00?ooool00ooo00<00?ooool00P00j_oo000Zool20003ool00`00ool00002
ool00`00ooooo`3Yool002_oo`03003ooooo00;oo`05003oooooool0003[ool003Coo`03003ooooo
0>Woo`00=?oo00<00?ooool0jOoo000dool2003Zool003Coo`03003ooooo0>Woo`00=?oo00<00?oo
ool0jOoo000dool00`00ooooo`3Yool003Coo`800>[oo`00=?oo00<00?ooool0jOoo000dool00`00
ooooo`3Yool003Coo`03003ooooo0>Woo`00=?oo00<00?ooool0jOoo000dool2003Zool003Coo`03
003ooooo0>Woo`003_oo0P000ooo00H00?ooooooo`00ool40003ool2000Bool00`00ooooo`3Yool0
00goo`04003ooooo0003ool00`00ool00004ool01@00oooooooo00000_oo00<00?ooool03ooo00<0
0?ooool0jOoo000@ool00`00ooooo`02ool00`00ooooo`03ool01@00oooooooo00000_oo00<00?oo
ool03ooo00<00?ooool0jOoo000>ool20004ool00`00ool00004ool01@00oooooooo00000_oo00<0
0?ooool03ooo0P00j_oo000@ool01000ooooo`000ooo00D00?ooooooo`0000?oo`04003ooooo000A
ool00`00ooooo`3Yool000goo`04003ooooo0008ool30003ool01000ooooo`001ooo0`000_oo0`00
k_oo000>ool2000;ool00`00ooooo`02ool20009ool01@00oooooooo0000l?oo000Uool30003ool0
0`00ooooo`02ool00`00ooooo`3]ool002_oo`03003ooooo00?oo`03003ooooo0>coo`00:_oo0P00
0ooo00<00?oo0000k_oo000[ool00`00ooooo`02ool00`00ooooo`3]ool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {779.188, 637.875}} -> {-96.0506, \
1.64438*^-11, 0.443931, 2.11617*^-14}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Complex valued function", "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"g2", "[", "t_", "]"}], ":=", 
        RowBox[{"Exp", "[", 
          RowBox[{"I", " ", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"t", "/", "Second"}], "-", "2"}], ")"}], "2"]}], 
          "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"\[Tau]", "=", 
        RowBox[{"0.3", "Second"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"nn", "=", "20"}], ";"}]}], "Input"],

Cell["Time spectrum:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ParametricPlot3D", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"t", ",", 
              RowBox[{"Re", "[", 
                RowBox[{"g2", "[", 
                  RowBox[{"t", " ", "Second"}], "]"}], "]"}], ",", 
              RowBox[{"Im", "[", 
                RowBox[{"g2", "[", 
                  RowBox[{"t", " ", "Second"}], "]"}], "]"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"t", ",", "0", ",", 
              RowBox[{"nn", " ", 
                RowBox[{"\[Tau]", "/", "Second"}]}]}], "}"}], ",", 
          RowBox[{"AxesLabel", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"\"\<t/s\>\"", ",", "\"\<re\>\"", ",", "\"\<im\>\""}], 
              "}"}]}], ",", 
          RowBox[{"BoxRatios", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"2", ",", "1", ",", "1"}], "}"}]}]}], "]"}], 
      ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81195 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.00456621 1.00457 -0.0697153 1.00457 [
[.04837 .26216 -6 -8.80895 ]
[.04837 .26216 0 .19105 ]
[.26993 .18003 -5.74471 -9 ]
[.26993 .18003 .25529 0 ]
[.50799 .09182 -5.35641 -9 ]
[.50799 .09182 .64359 0 ]
[.76445 -0.00313 -4.96811 -9 ]
[.76445 -0.00313 1.03189 0 ]
[.34612 .08889 -20.3521 -9.8125 ]
[.34612 .08889 1.64794 0 ]
[.80735 .00226 0 -6.15551 ]
[.80735 .00226 12 2.84449 ]
[.85014 .06853 0 -6.10264 ]
[.85014 .06853 24 2.89736 ]
[.89077 .13145 0 -6.05304 ]
[.89077 .13145 6 2.94696 ]
[.9294 .19126 0 -6.00642 ]
[.9294 .19126 18 2.99358 ]
[.96616 .24819 0 -5.96251 ]
[.96616 .24819 6 3.03749 ]
[.95019 .11106 0 -6.59941 ]
[.95019 .11106 16 3.21309 ]
[.97044 .25951 0 -5.95445 ]
[.97044 .25951 12 3.04555 ]
[.97996 .33747 0 -5.90317 ]
[.97996 .33747 24 3.09683 ]
[.98988 .4187 0 -5.84987 ]
[.98988 .4187 6 3.15013 ]
[1.00022 .50342 0 -5.79445 ]
[1.00022 .50342 18 3.20555 ]
[1.01102 .59186 0 -5.73676 ]
[1.01102 .59186 6 3.26324 ]
[1.05014 .40162 0 -6.5804 ]
[1.05014 .40162 16 3.5446 ]
[ 0 0 0 0 ]
[ 1 .81195 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.04148 .27674 m
.7913 0 L
s
.05744 .27085 m
.06197 .27519 L
s
[(0)] .04837 .26216 1 .95754 Mshowa
.27841 .18929 m
.28264 .19393 L
s
[(2)] .26993 .18003 .9149 1 Mshowa
.51574 .1017 m
.51962 .10664 L
s
[(4)] .50799 .09182 .78547 1 Mshowa
.77133 .00737 m
.77478 .01262 L
s
[(6)] .76445 -0.00313 .65604 1 Mshowa
.125 Mabswid
.11124 .25099 m
.11392 .25364 L
s
.16598 .23079 m
.16862 .23348 L
s
.2217 .21022 m
.22429 .21296 L
s
.33614 .16799 m
.33863 .17081 L
s
.39492 .14629 m
.39736 .14916 L
s
.45478 .1242 m
.45716 .12712 L
s
.57785 .07878 m
.58012 .08179 L
s
.64113 .05542 m
.64333 .05848 L
s
.70561 .03162 m
.70775 .03473 L
s
gsave
.34612 .08889 -81.3521 -13.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(t) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.25 Mabswid
.7913 0 m
.95779 .2576 L
s
.79556 .00659 m
.78967 .00876 L
s
[(-1)] .80735 .00226 -1 .36789 Mshowa
.83831 .07274 m
.8324 .07485 L
s
[(-0.5)] .85014 .06853 -1 .35614 Mshowa
.8789 .13554 m
.87297 .13759 L
s
[(0)] .89077 .13145 -1 .34512 Mshowa
.91749 .19524 m
.91154 .19724 L
s
[(0.5)] .9294 .19126 -1 .33476 Mshowa
.95422 .25207 m
.94825 .25401 L
s
[(1)] .96616 .24819 -1 .325 Mshowa
.125 Mabswid
.80429 .0201 m
.80076 .0214 L
s
.81293 .03347 m
.80939 .03476 L
s
.82148 .0467 m
.81794 .04798 L
s
.82994 .05979 m
.8264 .06106 L
s
.8466 .08556 m
.84305 .08682 L
s
.8548 .09825 m
.85125 .0995 L
s
.86292 .11081 m
.85936 .11205 L
s
.87095 .12324 m
.86739 .12447 L
s
.88678 .14772 m
.88321 .14895 L
s
.89457 .15978 m
.89101 .161 L
s
.90229 .17172 m
.89872 .17293 L
s
.90993 .18354 m
.90636 .18474 L
s
.92498 .20683 m
.92141 .20802 L
s
.9324 .21831 m
.92882 .21949 L
s
.93974 .22967 m
.93616 .23085 L
s
.94702 .24092 m
.94343 .24209 L
s
gsave
.95019 .11106 -61 -10.5994 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(re) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.25 Mabswid
.95779 .2576 m
1 .60412 L
s
.95849 .26337 m
.95252 .2653 L
s
[(-1)] .97044 .25951 -1 .32321 Mshowa
.96798 .34121 m
.96198 .34308 L
s
[(-0.5)] .97996 .33747 -1 .31181 Mshowa
.97785 .42231 m
.97184 .42411 L
s
[(0)] .98988 .4187 -1 .29997 Mshowa
.98816 .50689 m
.98212 .50863 L
s
[(0.5)] 1.00022 .50342 -1 .28765 Mshowa
.99891 .59519 m
.99286 .59685 L
s
[(1)] 1.01102 .59186 -1 .27484 Mshowa
.125 Mabswid
.96036 .27869 m
.95677 .27984 L
s
.96224 .29413 m
.95865 .29527 L
s
.96414 .30969 m
.96055 .31083 L
s
.96605 .32538 m
.96245 .32651 L
s
.96992 .35716 m
.96632 .35827 L
s
.97188 .37324 m
.96828 .37435 L
s
.97385 .38946 m
.97025 .39056 L
s
.97585 .40582 m
.97224 .40691 L
s
.97988 .43894 m
.97627 .44002 L
s
.98192 .45571 m
.97831 .45678 L
s
.98398 .47263 m
.98037 .47369 L
s
.98606 .48969 m
.98244 .49074 L
s
.99027 .52425 m
.98665 .52528 L
s
.9924 .54175 m
.98878 .54278 L
s
.99455 .55941 m
.99093 .56043 L
s
.99672 .57722 m
.99309 .57823 L
s
gsave
1.05014 .40162 -61 -10.5804 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 18.125 translate 1 -1 scale
63.000 11.562 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.562 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(im) show
75.000 11.562 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.25 Mabswid
.04148 .27674 m
0 .62185 L
s
0 .62185 m
.24023 .81195 L
s
.24023 .81195 m
.25974 .48381 L
s
.25974 .48381 m
.04148 .27674 L
s
.7913 0 m
.95779 .2576 L
s
.95779 .2576 m
1 .60412 L
s
1 .60412 m
.82157 .36284 L
s
.82157 .36284 m
.7913 0 L
s
.04148 .27674 m
0 .62185 L
s
0 .62185 m
.82157 .36284 L
s
.82157 .36284 m
.7913 0 L
s
.7913 0 m
.04148 .27674 L
s
.25974 .48381 m
.95779 .2576 L
s
.95779 .2576 m
1 .60412 L
s
1 .60412 m
.24023 .81195 L
s
.24023 .81195 m
.25974 .48381 L
s
0 0 m
1 0 L
1 .81195 L
0 .81195 L
closepath
clip
newpath
.5 Mabswid
.45255 .58122 m
.46168 .57405 L
s
.44323 .59032 m
.45255 .58122 L
s
.46168 .57405 m
.47073 .5689 L
s
.43353 .60119 m
.44323 .59032 L
s
.47073 .5689 m
.47976 .56583 L
s
.42324 .61357 m
.43353 .60119 L
s
.47976 .56583 m
.48881 .56485 L
s
.4121 .62716 m
.42324 .61357 L
s
.48881 .56485 m
.49785 .56594 L
s
.39978 .64151 m
.4121 .62716 L
s
.49785 .56594 m
.5068 .56901 L
s
.5068 .56901 m
.51555 .57392 L
s
.38594 .65605 m
.39978 .64151 L
s
.51555 .57392 m
.52392 .58047 L
s
.3702 .67002 m
.38594 .65605 L
s
.09716 .35593 m
.07892 .36918 L
s
.52392 .58047 m
.53173 .58835 L
s
.3522 .68252 m
.3702 .67002 L
s
.53173 .58835 m
.5387 .59715 L
s
.3316 .69245 m
.3522 .68252 L
s
.73153 .37024 m
.75806 .44018 L
s
.69687 .30404 m
.73153 .37024 L
s
.5387 .59715 m
.54456 .60633 L
s
.65747 .24926 m
.69687 .30404 L
s
.30818 .69853 m
.3316 .69245 L
s
.54456 .60633 m
.54899 .61515 L
s
.75806 .44018 m
.77335 .50418 L
s
.07892 .36918 m
.06963 .39877 L
s
.61677 .21148 m
.65747 .24926 L
s
.28186 .69936 m
.30818 .69853 L
s
.54899 .61515 m
.55167 .62273 L
s
.84942 .29948 m
.88762 .40284 L
s
.79815 .20791 m
.84942 .29948 L
s
.2528 .69352 m
.28186 .69936 L
s
.77335 .50418 m
.77505 .55084 L
s
.55167 .62273 m
.55227 .62795 L
s
.57801 .19407 m
.61677 .21148 L
s
.88762 .40284 m
.90443 .48995 L
s
.06963 .39877 m
.0704 .44031 L
s
.74386 .151 m
.79815 .20791 L
s
.22153 .67973 m
.2528 .69352 L
s
.55227 .62795 m
.55054 .62952 L
s
.0704 .44031 m
.08108 .48859 L
s
.189 .65705 m
.22153 .67973 L
s
.54403 .19809 m
.57801 .19407 L
s
.55054 .62952 m
.54632 .62599 L
s
.15664 .62522 m
.189 .65705 L
s
.77505 .55084 m
.76235 .56823 L
s
.08108 .48859 m
.1004 .53832 L
s
.90443 .48995 m
.8951 .53046 L
s
.6973 .14404 m
.74386 .151 L
s
.12636 .58499 m
.15664 .62522 L
s
.1004 .53832 m
.12636 .58499 L
s
.54632 .62599 m
.53959 .61583 L
s
.51704 .22223 m
.54403 .19809 L
s
.53959 .61583 m
.53061 .59759 L
s
.76235 .56823 m
.73721 .54663 L
s
.49832 .26304 m
.51704 .22223 L
s
.53061 .59759 m
.51995 .57013 L
s
.66805 .19075 m
.6973 .14404 L
s
.8951 .53046 m
.86208 .50005 L
s
.51995 .57013 m
.50856 .53291 L
s
.48811 .31546 m
.49832 .26304 L
s
.50856 .53291 m
.49785 .48636 L
s
.73721 .54663 m
.70546 .48323 L
s
.48569 .37371 m
.48811 .31546 L
s
.49785 .48636 m
.48957 .43222 L
s
.48957 .43222 m
.48569 .37371 L
s
.66158 .2799 m
.66805 .19075 L
s
.78806 .25713 m
.78909 .14163 L
s
.70546 .48323 m
.67664 .387 L
s
.86208 .50005 m
.81889 .3971 L
s
.67664 .387 m
.66158 .2799 L
s
.81889 .3971 m
.78806 .25713 L
s
.25 Mabswid
.7913 0 m
.95779 .2576 L
s
.95779 .2576 m
1 .60412 L
s
1 .60412 m
.82157 .36284 L
s
.82157 .36284 m
.7913 0 L
s
.04148 .27674 m
0 .62185 L
s
0 .62185 m
.82157 .36284 L
s
.82157 .36284 m
.7913 0 L
s
.7913 0 m
.04148 .27674 L
s
.04148 .27674 m
.7913 0 L
s
.05744 .27085 m
.06197 .27519 L
s
[(0)] .04837 .26216 1 .95754 Mshowa
.27841 .18929 m
.28264 .19393 L
s
[(2)] .26993 .18003 .9149 1 Mshowa
.51574 .1017 m
.51962 .10664 L
s
[(4)] .50799 .09182 .78547 1 Mshowa
.77133 .00737 m
.77478 .01262 L
s
[(6)] .76445 -0.00313 .65604 1 Mshowa
.125 Mabswid
.11124 .25099 m
.11392 .25364 L
s
.16598 .23079 m
.16862 .23348 L
s
.2217 .21022 m
.22429 .21296 L
s
.33614 .16799 m
.33863 .17081 L
s
.39492 .14629 m
.39736 .14916 L
s
.45478 .1242 m
.45716 .12712 L
s
.57785 .07878 m
.58012 .08179 L
s
.64113 .05542 m
.64333 .05848 L
s
.70561 .03162 m
.70775 .03473 L
s
gsave
.34612 .08889 -81.3521 -13.8125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(t) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
81.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.25 Mabswid
.95779 .2576 m
1 .60412 L
s
.95849 .26337 m
.95252 .2653 L
s
[(-1)] .97044 .25951 -1 .32321 Mshowa
.96798 .34121 m
.96198 .34308 L
s
[(-0.5)] .97996 .33747 -1 .31181 Mshowa
.97785 .42231 m
.97184 .42411 L
s
[(0)] .98988 .4187 -1 .29997 Mshowa
.98816 .50689 m
.98212 .50863 L
s
[(0.5)] 1.00022 .50342 -1 .28765 Mshowa
.99891 .59519 m
.99286 .59685 L
s
[(1)] 1.01102 .59186 -1 .27484 Mshowa
.125 Mabswid
.96036 .27869 m
.95677 .27984 L
s
.96224 .29413 m
.95865 .29527 L
s
.96414 .30969 m
.96055 .31083 L
s
.96605 .32538 m
.96245 .32651 L
s
.96992 .35716 m
.96632 .35827 L
s
.97188 .37324 m
.96828 .37435 L
s
.97385 .38946 m
.97025 .39056 L
s
.97585 .40582 m
.97224 .40691 L
s
.97988 .43894 m
.97627 .44002 L
s
.98192 .45571 m
.97831 .45678 L
s
.98398 .47263 m
.98037 .47369 L
s
.98606 .48969 m
.98244 .49074 L
s
.99027 .52425 m
.98665 .52528 L
s
.9924 .54175 m
.98878 .54278 L
s
.99455 .55941 m
.99093 .56043 L
s
.99672 .57722 m
.99309 .57823 L
s
gsave
1.05014 .40162 -61 -10.5804 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 18.125 translate 1 -1 scale
63.000 11.562 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.562 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(im) show
75.000 11.562 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 233.812},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000jA000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00<;oo`8005co
o`00`Ooo00@00?ooool005_oo`00`Ooo00@00?ooool005_oo`00`Ooo0`00G?oo0031ool00`00oooo
o`1Lool00<7oo`03003ooooo01?oo`@004Goo`00`_oo0`005?oo00<00?ooool0A?oo003Iool00`00
ooooo`14ool00<_oo`8000Coo`@000Coo`03003ooooo04Coo`00b?oo0`0000?oo`0000002ooo00<0
0?ooool0A?oo0036ool20003ool30009ool30016ool00<?oo`<00003ool00?oo00?oo`03003oo`00
00[oo`03003ooooo04Coo`00`?oo0`002Ooo00@00?ooool0053oo`00__oo0P003?oo00@00?ooool0
053oo`00^ooo0`003ooo0P0000?oo`00ool0C_oo002hool3000Bool01000ooooo`00Cooo002fool2
000Eool01@00oooooooo0000C_oo002cool4000Fool00`00ooooo`03001=ool00;3oo`<001[oo`03
003ooooo00;oo`03003ooooo04_oo`00[_oo0P007Ooo00<00?ooool00ooo00<00?ooool0B_oo002[
ool3000Oool00`00ooooo`03ool2001;ool00:Soo`<002;oo`03003ooooo00?oo`8004_oo`00YOoo
0`009Ooo00<00?ooool01Ooo00<00?ooool02ooo0P001?oo00<00?ooool00_oo0`00;ooo002Sool4
000Vool00`00ooooo`06ool00`00ooooo`09ool01000ooooo`002ooo00<00?ooool0;?oo001=ool0
0`00ooooo`1@ool3000Zool00`00ooooo`05ool2000;ool01000ooooo`002ooo00<00?ooool0;?oo
0019ool20003ool00`00ooooo`02ool3000Zool00`00ooooo`0Jool3000]ool00`00ooooo`07ool0
0`00ooooo`02ool40002ool01000ooooo`002?oo0`00;ooo0018ool01000ooooo`000_oo00<00?oo
ool01Ooo00<00?ooool09ooo00<00?ooool06?oo0P00<Ooo00<00?ooool01ooo00<00?ooool01ooo
00@00?ooool000Soo`03003ooooo02ooo`00B?oo00<00?ooool00ooo00<00?ooool00ooo0P009ooo
1@005_oo0`00<ooo00<00?ooool01_oo0P002Ooo00@00?ooool000Soo`03003ooooo02ooo`00B?oo
00<00?ooool01?oo00D00?ooooooo`0002Woo`04003ooooo000Dool3000fool00`00ooooo`05ool0
1000ooooo`002Ooo0P002Ooo1000;_oo0017ool40004ool00`00ooooo`02ool3000Wool00`00ool0
000Bool2000iool00`00ooooo`08ool00`00ooooo`14ool004Soo`03003ooooo00Goo`03003ooooo
02coo`8000ooo`<003_oo`03003ooooo00Woo`03003ooooo04?oo`00B?oo00<00?ooool0=Ooo00<0
0?ooool02_oo0`00?_oo00<00?ooool02?oo00<00?oo0000A?oo002:ool30011ool00`00ooooo`0:
ool00`00ooooo`12ool008Soo`8004Coo`03003ooooo00_oo`03003ooooo047oo`00QOoo0`00A_oo
00<00?ooool02_oo0`006_oo10000ooo0`007_oo0022ool40018ool00`00ooooo`0<ool00`00oooo
o`0Iool00`00ooooo`02ool00`00ooooo`0Oool0083oo`8000Coo`03003ooooo04Goo`03003ooooo
00goo`03003ooooo01Soo`03003ooooo00;oo`<001ooo`00OOoo0`00C_oo00<00?ooool03?oo0`00
6Ooo00<00?ooool00_oo00@00?ooool001koo`00N_oo0`00D_oo00<00?ooool03Ooo00<00?ooool0
1Ooo0P003ooo00<00?oo00000P000_oo0P007ooo001hool2001Eool00`00ooooo`0>ool00`00oooo
o`03ool01000ooooo`00=_oo001eool3001Gool00`00ooooo`0=ool30004ool01000ooooo`00=_oo
001bool4001Iool00`00ooooo`0?ool00`00ooooo`02ool01000ooooo`00=_oo001`ool2001Mool0
0`00ooooo`0@ool01@00oooooooo00000_oo00<00?ooool0=?oo001]ool3001Oool00`00ooooo`0@
ool01@00oooooooo00000_oo00<00?ooool0=?oo001Zool3001Rool00`00ooooo`0?ool00`00ool0
0003ool2000gool006Ooo`<004Woo`<001Goo`05003oooooool0000Dool00`00ooooo`0iool006Go
o`8004_oo`04003oooooool6000?ool01@00oooooooo00005?oo00<00?ooool0>Ooo001Rool30000
0ooo003oo`1:ool00`00ooooo`07ool4000;ool01@00oooooooo00004ooo00<00?oo0000>_oo0010
ool4000Kool3001?ool00`00ooooo`0<ool00`00ooooo`08ool01@00oooooooo00005_oo00<00?oo
ool0=ooo0010ool00`00ooooo`0Jool2001Bool00`00ooooo`0=ool00`00ooooo`07ool01@00oooo
oooo00005_oo00<00?ooool0=ooo0011ool00`00ooooo`0Fool3001Cool00`00ooooo`0?ool00`00
ooooo`06ool00`00ooooo`02ool00`00ooooo`0Bool00`00ool0000hool004;oo`03003ooooo01;o
o`<005Koo`03003ooooo013oo`03003ooooo00Goo`03003ooooo00;oo`03003ooooo01Goo`03003o
oooo03Goo`00@ooo00<00?ooool03ooo0`00Eooo00<00?ooool04_oo00<00?ooool01?oo00<00?oo
ool00_oo00<00?ooool05Ooo00<00?ooool0=Ooo0010ool01000ooooo`003_oo0`00FOoo00<00?oo
ool05?oo00<00?ooool00ooo00<00?ooool00_oo00<00?ooool05?oo0`00=_oo0011ool2000<ool3
001Lool00`00ooooo`0Eool00`00ooooo`02ool00`00ooooo`02ool00`00ooooo`0Gool00`00oooo
o`04ool20004ool00`00ooooo`02ool3000Qool004goo`8005koo`03003ooooo01Koo`03003ooooo
00;oo`03003ooooo00;oo`03003ooooo01Ooo`03003ooooo00?oo`04003ooooo000;ool00`00oooo
o`0Nool004[oo`<0063oo`03003ooooo01Ooo`05003oooooool00004ool00`00ooooo`0Fool30004
ool01000ooooo`002ooo00<00?ooool07_oo0017ool3001Rool00`00ooooo`0Iool01000ooooo`00
1?oo00<00?ooool06?oo00<00?ooool00_oo00@00?ooool000Soo`<0027oo`00A?oo0`0000?oo`00
ool0Aooo1@005_oo00<00?ooool06_oo00<00?oo00001?oo00<00?ooool06Ooo00D00?ooooooo`00
00;oo`03003ooooo00Koo`03003ooooo027oo`00@_oo0P00B?oo1@001Ooo0P005?oo00<00?ooool0
6ooo0P001?oo00<00?ooool06?oo0`000_oo00@00?ooool000Soo`03003ooooo027oo`00?ooo0`00
BOoo00<00?ooool02_oo0P004_oo00<00?ooool07?oo00<00?ooool00_oo00<00?ooool06_oo00@0
0?ooooooo`8000Woo`@0023oo`00??oo0`00Booo00<00?ooool03Ooo0P004?oo00<00?ooool07?oo
0P001?oo00<00?ooool06_oo00<00?ooool0;ooo000iool3001=ool00`00ooooo`0@ool2000>ool0
0`00ooooo`0Lool00`00ool00003ool00`00ooooo`0Iool3000`ool003Ooo`<004koo`03003ooooo
01?oo`8000_oo`03003ooooo01goo`04003ooooo0002ool00`00ooooo`0Kool00`00ooooo`0^ool0
03Coo`<0053oo`03003ooooo01Koo`03003ooooo00Soo`03003ooooo01goo`04003ooooo0002ool0
0`00ooooo`0Lool00`00ooooo`0]ool003;oo`8005;oo`03003ooooo01Soo`03003ooooo00Ooo`03
003ooooo01goo`07003oooooool00?oo00007Ooo0`00;_oo000_ool3001Cool00`00ooooo`0Jool0
0`00ooooo`06ool00`00ooooo`0Mool01`00oooooooo003oo`0001ooo`03003ooooo02coo`00;?oo
0`00E_oo00<00?ooool06ooo00<00?ooool01Ooo00<00?ooool07Ooo00<00?ooool00_oo0P008?oo
00<00?ooool0:ooo000Yool3001Hool00`00ooooo`0Mool00`00ooooo`04ool00`00ooooo`0Mool0
0`00ooooo`02ool2000Oool00`00ool0000/ool000Soo`8001goo`8005_oo`03003ooooo01koo`03
003ooooo00?oo`03003ooooo01goo`03003ooooo00?oo`03003ooooo01ooo`03003ooooo00Coo`@0
02;oo`001ooo00@00?ooool001Woo`<005coo`03003ooooo023oo`03003ooooo00;oo`03003ooooo
01goo`03003ooooo00?oo`80027oo`03003ooooo00Goo`03003ooooo027oo`001ooo00@00?ooool0
01Koo`<005ooo`03003ooooo027oo`05003oooooool0000Oool00`00ooooo`03ool2000Pool20007
ool00`00ooooo`0Qool000Ooo`04003ooooo000Dool2001Qool00`00ooooo`0Sool01000ooooo`00
7ooo00<00?ooool00ooo00<00?oo00008Ooo00<00?ooool01?oo00<00?ooool00_oo10006ooo0007
ool01000ooooo`004Ooo0`00Hooo00<00?ooool09?oo00<00?oo00007ooo00<00?ooool01?oo0P00
8Ooo0P001Ooo00<00?ooool01?oo00<00?ooool06_oo0007ool01000ooooo`003_oo0`0000?oo`00
ool0H_oo00<00?ooool09_oo0P007ooo00<00?ooool01?oo00<00?oo00007ooo00<00?oo00000ooo
0`001_oo00<00?ooool06_oo0008ool2000=ool2001Xool00`00ooooo`0Vool00`00ooooo`0Nool0
0`00ooooo`04ool01000ooooo`008?oo0P000ooo10001?oo00<00?ooool06_oo000Dool3001Yool0
0`00ooooo`0Wool2000Oool00`00ooooo`04ool01000ooooo`007Ooo1@002ooo00<00?ooool06_oo
000Aool3001/ool00`00ooooo`0Wool00`00ool0000Nool00`00ooooo`04ool01@00oooooooo0000
6Ooo0`000_oo00<00?oo00002Ooo0`007?oo000>ool3001_ool00`00ooooo`0Wool01000ooooo`00
7_oo00<00?ooool00ooo00D00?ooooooo`0001Koo`<000Ooo`03003ooooo00Woo`03003ooooo01[o
o`003?oo1000Kooo00<00?ooool0:?oo00@00?ooool001koo`03003ooooo00?oo`03003ooooo00;o
o`03003ooooo013oo`<000[oo`03003ooooo02Koo`002_oo0P00Looo00<00?ooool0:?oo00D00?oo
ooooo`0001goo`03003ooooo00?oo`03003ooooo00;oo`03003ooooo00coo`@000coo`<002Ooo`00
2_oo0P00Looo00<00?ooool0:?oo00<00?ooool00_oo00<00?ooool06_oo00<00?ooool00ooo00<0
0?ooool00ooo00<00?ooool02?oo0`004_oo00<00?ooool09Ooo000:ool00`00ool0001bool00`00
ooooo`0Xool00`00ooooo`02ool00`00ooooo`0Kool00`00ooooo`02ool00`00ooooo`04ool00`00
ooooo`04ool3000Eool00`00ooooo`0Uool000[oo`04003ooooo001aool00`00ooooo`0Xool00`00
ooooo`03ool00`00ooooo`0Jool00`00ooooo`02ool00`00ooooo`04ool01000oooooooo0`006?oo
00<00?ooool09Ooo000:ool01@00oooooooo0000Kooo00<00?ooool0:_oo00<00?ooool00ooo00<0
0?ooool06Ooo00<00?ooool00_oo00<00?ooool01Ooo0`006_oo0P009ooo0009ool00`00ooooo`03
ool00`00ooooo`1/ool00`00ooooo`0Zool00`00ooooo`04ool00`00ooooo`0Hool00`00ooooo`03
ool01000oooooooo10007Ooo00<00?ooool09Ooo0009ool00`00ooooo`04ool00`00ooooo`1[ool0
0`00ooooo`0Zool00`00ooooo`04ool00`00ooooo`0Hool00`00ooooo`03ool40004ool00`00oooo
o`0Jool00`00ooooo`0Uool000Woo`03003ooooo00Goo`8006_oo`03003ooooo02[oo`03003ooooo
00Goo`03003ooooo01Soo`03003ooooo00<000Ooo`03003ooooo01[oo`03003ooooo02Goo`002Ooo
00<00?ooool01ooo00<00?ooool0Iooo00<00?ooool0:ooo00<00?ooool01_oo00<00?ooool05ooo
0`000_oo00<00?ooool01_oo00<00?ooool06?oo0`009_oo0009ool00`00ooooo`08ool00`00oooo
o`1Vool00`00ooooo`0[ool00`00ooooo`06ool00`00ooooo`0Cool50004ool00`00ooooo`06ool0
0`00ooooo`0Jool00`00ooooo`0Tool000Woo`03003ooooo00Woo`03003ooooo06Goo`03003ooooo
02_oo`03003ooooo00Ooo`03003ooooo00ooo`<000Coo`03003ooooo00;oo`03003ooooo00Ooo`03
003ooooo01Woo`03003ooooo02Coo`002Ooo00<00?ooool02_oo00<00?ooool0I?oo00<00?ooool0
;?oo00<00?ooool01_oo00<00?ooool03?oo0`002?oo00D00?ooooooo`0000Woo`03003ooooo01Wo
o`03003ooooo02Coo`002Ooo00<00?ooool02ooo00<00?ooool0Hooo00<00?ooool0;?oo00<00?oo
ool01ooo00<00?ooool02?oo0`002ooo00D00?ooooooo`0000Woo`03003ooooo01Soo`8000[oo`80
00Coo`03003ooooo00;oo`<000koo`002Ooo00<00?ooool03?oo00<00?ooool0H_oo00<00?ooool0
;?oo00<00?ooool01ooo00<00?ooool01Ooo0`003_oo00D00?ooooooo`0000[oo`03003ooooo01So
o`03003ooooo00Ooo`04003ooooo000;ool00`00ooooo`0;ool000Soo`03003ooooo00koo`03003o
oooo067oo`03003ooooo02coo`03003ooooo00Soo`04003oooooool3000Aool01@00oooooooo0000
2_oo00<00?ooool06?oo00<00?ooool01ooo00@00?ooool000_oo`03003ooooo00_oo`002?oo00<0
0?ooool03ooo00<00?ooool0H?oo00<00?ooool0;?oo00<00?ooool02?oo10005?oo00D00?oooooo
o`0000[oo`03003ooooo01Soo`04003oooooool40002ool01000ooooo`002?oo0`003_oo0008ool0
0`00ooooo`0@ool00`00ooooo`1Oool00`00ooooo`0/ool00`00ooooo`06ool4000Gool01000oooo
o`002ooo00<00?ooool05ooo0P002?oo00@00?ooool000Soo`03003ooooo00koo`002?oo00<00?oo
ool04Ooo00<00?ooool0G_oo00<00?ooool0;?oo00<00?ooool00ooo0`000ooo00<00?ooool05Ooo
00D00?ooooooo`0000[oo`03003ooooo01Koo`03003oo`0000Soo`04003ooooo0008ool00`00oooo
o`0>ool000Soo`03003ooooo01;oo`03003ooooo05goo`03003ooooo02goo`03003oo`00008000Oo
o`03003ooooo01Coo`05003oooooool0000;ool00`00ooooo`0Gool00`00ooooo`07ool20009ool4
000=ool000Soo`03003ooooo01?oo`03003ooooo05coo`03003ooooo02_oo`@000[oo`03003ooooo
01Coo`05003oooooool0000;ool00`00ooooo`0Gool00`00ooooo`0Sool000Soo`03003ooooo00ko
o`03003ooooo00?oo`8005coo`03003ooooo02Soo`<000;oo`03003ooooo00[oo`03003ooooo01Co
o`04003ooooo000;ool00`00ooooo`0Gool00`00ooooo`0Sool000Soo`03003ooooo00goo`03003o
oooo00Koo`03003ooooo05Woo`03003ooooo02Goo`<000Goo`03003ooooo00[oo`03003ooooo01Co
o`04003ooooo000<ool00`00ooooo`0Eool2000Uool000Soo`03003ooooo00_oo`8000[oo`03003o
oooo05Soo`03003ooooo02;oo`<000Soo`03003ooooo00_oo`03003ooooo01?oo`04003oo`00000<
ool00`00ooooo`0Fool00`00ooooo`0Sool000Ooo`03003ooooo00_oo`03003ooooo00_oo`03003o
oooo05Ooo`03003ooooo01ooo`<000_oo`03003ooooo00_oo`03003ooooo01;oo`<000;oo`03003o
oooo00[oo`03003ooooo01Goo`03003ooooo02?oo`001ooo00<00?ooool02_oo00<00?ooool03Ooo
00<00?ooool0E_oo00<00?ooool07?oo0`003_oo00<00?ooool03?oo00<00?ooool03_oo0`0000?o
o`00ool00_oo00<00?ooool02_oo00<00?ooool05_oo00<00?ooool08_oo0007ool00`00ooooo`0:
ool00`00ooooo`0>ool00`00ooooo`1Eool00`00ooooo`0Iool3000Bool00`00ooooo`0;ool00`00
ooooo`0:ool40005ool01@00oooooooo00002ooo00<00?ooool05?oo0`009?oo0007ool00`00oooo
o`09ool00`00ooooo`0@ool00`00ooooo`1Dool00`00ooooo`0Fool3000Eool00`00ooooo`0<ool0
0`00ooooo`06ool30009ool00`00ooooo`02ool00`00ooooo`09ool00`00ooooo`0Eool00`00oooo
o`0Rool000Ooo`03003ooooo00Woo`03003ooooo017oo`03003ooooo05?oo`03003ooooo01?oo`<0
01Soo`03003ooooo00coo`03003ooooo00?oo`<000coo`03003ooooo00?oo`03003ooooo00Soo`03
003ooooo01Goo`03003ooooo02;oo`001ooo00<00?ooool02?oo00<00?ooool04ooo00<00?ooool0
D_oo00<00?ooool04?oo0`006ooo00<00?ooool03Ooo00<00?oo00000P003ooo00<00?ooool00ooo
00<00?ooool02?oo00<00?ooool05Ooo00<00?ooool08_oo0007ool00`00ooooo`08ool00`00oooo
o`0Dool00`00ooooo`1Aool00`00ooooo`0<ool4000Nool00`00ooooo`0<ool3000Cool00`00oooo
o`03ool00`00ooooo`08ool00`00ooooo`0Cool2000Fool600000ooo003oo`020003ool000Ooo`03
003ooooo00Ooo`03003ooooo01Koo`03003ooooo053oo`03003ooooo00Woo`<002?oo`03003ooooo
00Soo`<00003ool00?oo01?oo`03003ooooo00Coo`03003ooooo00Ooo`03003ooooo01Coo`03003o
oooo01Koo`08003ooooo003oo`00ool000Coo`001ooo00<00?ooool01ooo00<00?ooool05ooo00<0
0?ooool0Cooo00<00?ooool01_oo0`009_oo00<00?ooool01?oo10001Ooo00<00?ooool04Ooo00<0
0?ooool01Ooo00<00?ooool01ooo00<00?ooool04ooo00<00?ooool05_oo00P00?ooool00?oo003o
o`001?oo0006ool00`00ooooo`08ool00`00ooooo`0Hool00`00ooooo`1>ool00`00ooooo`03ool3
000Yool01000oooooooo0`002Ooo00<00?ooool04Ooo00<00?ooool01Ooo00<00?ooool01ooo00<0
0?ooool05?oo00<00?ooool05Ooo00P00?ooool00?oo003oo`001?oo0006ool00`00ooooo`08ool0
0`00ooooo`0Iool2001?ool00`00ool00002000]ool3000<ool00`00ooooo`0Bool00`00ooooo`05
ool00`00ooooo`06ool00`00ooooo`0Bool30004ool2000@ool20002ool200000ooo003oo`04ool0
00Koo`03003ooooo00Soo`03003ooooo01_oo`03003ooooo04_oo`<002goo`@000ooo`03003ooooo
017oo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo01Coo`05003oooooool00002ool00`00
ooooo`0Jool000Koo`03003ooooo00Soo`03003ooooo01coo`03003ooooo04Ooo`<00003ool00?oo
02[oo`<000?oo`03003ooooo00goo`03003ooooo01;oo`03003ooooo00Koo`03003ooooo00Goo`03
003ooooo01?oo`05003oooooool00002ool00`00ooooo`0>ool00`00ooooo`09ool000Koo`03003o
oooo00Soo`03003ooooo01goo`03003ooooo04?oo`<000Coo`03003ooooo02Koo`<000Ooo`03003o
oooo00goo`03003ooooo017oo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo01?oo`05003o
ooooool00002ool00`00ooooo`0Jool000Koo`03003ooooo00Woo`03003ooooo01goo`03003ooooo
03ooo`<000Ooo`03003ooooo02;oo`@000[oo`03003ooooo00goo`03003ooooo01;oo`03003ooooo
00Koo`03003ooooo00Coo`03003ooooo01;oo`8000?oo`04003ooooo000Lool000Koo`03003ooooo
00Woo`03003ooooo01koo`03003ooooo03[oo`@000[oo`03003ooooo01ooo`<000koo`03003ooooo
00goo`03003ooooo01;oo`03003ooooo00Ooo`03003ooooo00?oo`03003ooooo017oo`03003oo`00
00?oo`04003ooooo000Lool000Koo`03003ooooo00Woo`03003ooooo01ooo`03003ooooo03Koo`<0
00koo`03003ooooo01coo`<001;oo`03003ooooo00goo`03003ooooo017oo`03003ooooo00Soo`03
003ooooo00;oo`03003ooooo01?oo`03003ooooo00;oo`8001goo`001_oo00<00?ooool02Ooo00<0
0?ooool08?oo00<00?ooool0<_oo0`004Ooo00<00?ooool06Ooo0`005Ooo00<00?ooool03Ooo00<0
0?ooool04_oo00<00?ooool01ooo00<00?ooool00_oo00<00?ooool05?oo00<00?ooool08?oo0005
ool00`00ooooo`0:ool00`00ooooo`0Qool00`00ooooo`0^ool3000Dool00`00ooooo`0Fool3000H
ool00`00ooooo`0>ool00`00ooooo`0Aool00`00ooooo`08ool00`00ooooo`02ool00`00ooooo`0C
ool00`00ooooo`0Pool000Goo`03003ooooo00[oo`03003ooooo02;oo`03003ooooo02[oo`<001Oo
o`03003ooooo01?oo`<001_oo`03003ooooo00koo`03003ooooo01;oo`03003ooooo00Soo`05003o
ooooool0000Cool3000Rool000Goo`03003ooooo00[oo`03003ooooo02?oo`03003ooooo02Koo`<0
01[oo`03003ooooo00ooo`@001ooo`03003ooooo00goo`03003ooooo01;oo`03003ooooo00Woo`04
003ooooo000Eool00`00ooooo`0Pool000Goo`03003ooooo00[oo`03003ooooo02Coo`03003ooooo
02;oo`<001koo`03003ooooo00_oo`<002?oo`03003ooooo00goo`03003ooooo01;oo`03003ooooo
00Woo`04003ooooo000Eool00`00ooooo`0Pool000Goo`03003ooooo00[oo`03003ooooo02Goo`03
003ooooo01koo`<0027oo`03003ooooo00Soo`<002Koo`03003ooooo00koo`03003ooooo01;oo`03
003ooooo00Woo`03003oo`0001Goo`03003ooooo023oo`001Ooo00<00?ooool02ooo00<00?ooool0
9Ooo0P006ooo0`009?oo00<00?ooool01Ooo0`00:_oo00<00?ooool03Ooo00<00?ooool04_oo00<0
0?ooool02_oo0P005?oo0P008_oo0005ool00`00ooooo`0;ool00`00ooooo`0Wool00`00ooooo`0E
ool3000Wool00`00ooooo`02ool3000]ool00`00ooooo`0=ool00`00ooooo`0Cool00`00ooooo`0:
ool2000Dool00`00ooooo`0Pool000Goo`03003ooooo00_oo`03003ooooo02Soo`03003ooooo013o
o`@002[oo`03003oo`000080033oo`03003ooooo00goo`03003ooooo01?oo`03003ooooo00[oo`80
01Goo`03003ooooo01ooo`001?oo00<00?ooool03?oo00<00?ooool0:Ooo00<00?ooool03?oo0`00
;?oo1000=?oo00<00?ooool03Ooo00<00?ooool04ooo00<00?ooool02_oo00<00?ooool04ooo00<0
0?ooool07ooo0004ool00`00ooooo`0<ool00`00ooooo`0Zool00`00ooooo`08ool3000/ool30002
ool00`00ooooo`0cool00`00ooooo`0=ool00`00ooooo`0Cool00`00ooooo`0:ool2000Bool3000Q
ool000Coo`03003ooooo00coo`03003ooooo02_oo`03003ooooo00Coo`<002coo`<000Koo`03003o
oooo03;oo`03003ooooo00goo`03003ooooo01?oo`03003ooooo00[oo`03003oo`0001?oo`03003o
oooo01ooo`001?oo00<00?ooool03Ooo00<00?ooool0:ooo00<00?ooool00`00;?oo0`002Ooo00<0
0?ooool0<ooo00<00?ooool03?oo00<00?ooool05?oo00<00?ooool02Ooo00<00?oo00004ooo00<0
0?ooool07ooo0004ool00`00ooooo`0=ool00`00ooooo`0/ool2000/ool3000<ool00`00ooooo`0c
ool00`00ooooo`0=ool00`00ooooo`0Cool00`00ooooo`0:ool00`00ool0000Bool00`00ooooo`0O
ool000Coo`03003ooooo00goo`03003ooooo02coo`03003ooooo02Soo`<000ooo`03003ooooo03?o
o`03003ooooo00goo`03003ooooo01Coo`03003ooooo00Woo`04003ooooo000Aool00`00ooooo`0O
ool000Coo`03003ooooo00goo`03003ooooo02coo`03003ooooo02Coo`@001;oo`03003ooooo03Co
o`03003ooooo00coo`03003ooooo01Coo`03003ooooo00Woo`04003ooooo000@ool20005ool20004
ool00`00ooooo`02ool3000>ool000Coo`03003ooooo00koo`03003ooooo02_oo`03003ooooo027o
o`<001Koo`03003ooooo03Coo`03003ooooo00coo`03003ooooo01Goo`03003ooooo00Soo`05003o
ooooool0000Aool01@00oooooooo00000_oo00<00?ooool02Ooo00<00?ooool02ooo0004ool00`00
ooooo`0>ool00`00ooooo`0[ool00`00ooooo`0Nool3000Jool00`00ooooo`0dool00`00ooooo`0<
ool00`00ooooo`0Dool00`00ooooo`07ool00`00ooooo`03ool00`00ooooo`0>ool01@00oooooooo
00000_oo00<00?ooool02Ooo00<00?ooool02ooo0003ool00`00ooooo`0@ool00`00ooooo`0Zool0
0`00ooooo`0Kool3000Mool00`00ooooo`0dool00`00ooooo`0<ool00`00ooooo`0Eool00`00oooo
o`06ool00`00ooooo`04ool00`00ooooo`0=ool01@00oooooooo00000_oo00<00?ooool01_oo0`00
3_oo0003ool00`00ooooo`0@ool00`00ooooo`0Zool00`00ooooo`0Hool3000Pool00`00ooooo`0e
ool00`00ooooo`0;ool00`00ooooo`0Fool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`0;
ool30003ool01000ooooo`002?oo00<00?ooool03_oo0003ool00`00ooooo`0@ool00`00ooooo`0Z
ool00`00ooooo`0Eool3000Sool00`00ooooo`0eool00`00ooooo`0;ool00`00ooooo`0Gool00`00
ooooo`04ool00`00ooooo`05ool00`00ooooo`0<ool01@00oooooooo00000_oo00<00?ooool01_oo
00<00?ooool03_oo0003ool00`00ooooo`0Aool00`00ooooo`0Xool00`00ooooo`0Cool3000Wool0
0`00ooooo`0eool00`00ooooo`0:ool00`00ooooo`0Hool00`00ooooo`02ool00`00ooooo`07ool0
0`00ooooo`0;ool00`00ooooo`02ool20009ool4000=ool000?oo`03003ooooo017oo`03003ooooo
02Soo`03003ooooo00ooo`@002[oo`03003ooooo03Goo`03003ooooo00[oo`03003ooooo01Woo`05
003oooooool0000:ool00`00ooooo`0:ool00`00ooooo`0Nool000?oo`03003ooooo017oo`03003o
oooo02Soo`03003ooooo00coo`<002koo`03003ooooo03Koo`03003ooooo00Woo`03003ooooo01[o
o`04003ooooo000:ool00`00ooooo`0:ool00`00ooooo`0Nool000?oo`03003ooooo01;oo`03003o
oooo02Ooo`03003ooooo00Woo`<0037oo`03003ooooo03Koo`03003ooooo00Woo`03003ooooo01_o
o`8000coo`03003ooooo00Soo`<001ooo`000ooo00<00?ooool04_oo00<00?ooool09ooo00<00?oo
ool01_oo0`00=Ooo00<00?ooool0=_oo00<00?ooool02Ooo00<00?ooool06ooo00<00?ooool02ooo
00<00?ooool02Ooo00<00?ooool07Ooo0003ool00`00ooooo`0Cool00`00ooooo`0Vool00`00oooo
o`03ool3000hool00`00ooooo`0fool00`00ooooo`09ool00`00ooooo`0Zool00`00ooooo`08ool0
0`00ooooo`0Mool000;oo`03003ooooo01Coo`03003ooooo02Koo`03003ooooo00<003_oo`03003o
oooo03Ooo`03003ooooo00Soo`03003ooooo02[oo`03003ooooo00Soo`03003ooooo01goo`000_oo
00<00?ooool05Ooo00<00?ooool09?oo1000?ooo00<00?ooool0=_oo00<00?ooool02?oo00<00?oo
ool0:ooo00<00?ooool01_oo0P007ooo0002ool00`00ooooo`0Eool00`00ooooo`0Qool300000ooo
003oo`10ool00`00ooooo`0gool00`00ooooo`07ool00`00ooooo`0/ool00`00ooooo`06ool00`00
ooooo`0Mool000;oo`03003ooooo01Koo`03003ooooo01goo`<000Coo`03003ooooo03ooo`03003o
oooo03Soo`03003ooooo00Koo`03003ooooo02goo`03003ooooo00Goo`03003ooooo01goo`000_oo
00<00?ooool05_oo00<00?ooool06_oo0`001ooo00<00?ooool0@?oo00<00?ooool0>?oo00<00?oo
ool01?oo00<00?ooool0;_oo00<00?ooool01Ooo00<00?ooool07Ooo0002ool00`00ooooo`0Gool0
0`00ooooo`0Fool3000:ool00`00ooooo`10ool00`00ooooo`0iool00`00ooooo`02ool00`00oooo
o`0`ool00`00ooooo`05ool00`00ooooo`0Lool000;oo`03003ooooo01Ooo`03003ooooo01?oo`<0
00goo`03003ooooo043oo`03003ooooo03[oo`05003oooooool0000cool00`00ooooo`02ool3000N
ool000;oo`03003ooooo01Soo`03003ooooo00koo`@0013oo`03003ooooo03Goo`L000Goo`03003o
oooo03[oo`03003oo`0003Goo`03003ooooo00?oo`03003ooooo01coo`000_oo00<00?ooool06Ooo
00<00?ooool02_oo0`004ooo00<00?ooool0=?oo0P001ooo0`000_oo00<00?ooool0>ooo00<00?oo
ool0=?oo00<00?ooool00ooo00<00?ooool07?oo00000ooo003oo`0Kool00`00ooooo`07ool3000F
ool00`00ooooo`0cool00`00ooooo`0:ool3001eool00`00ooooo`02ool00`00ooooo`0Lool00003
ool00?oo01coo`03003ooooo00?oo`<001Woo`03003ooooo037oo`8000ooo`8007Goo`03003ooooo
008001koo`0000?oo`00ool07?oo00<00?ooool00`007?oo00<00?ooool0<?oo00<00?ooool04?oo
00<00?ooool0M?oo00@00?ooool000?oo`@001Ooo`0000?oo`00ool07?oo0`007ooo00<00?ooool0
;ooo00<00?ooool04Ooo0P00MOoo00@00?ooool000Goo`03003ooooo01Koo`0000?oo`00ool06?oo
10000_oo00<00?ooool07Ooo00<00?ooool0;ooo00<00?ooool04Ooo00<00?oo0000MOoo00@00?oo
ool000Coo`03003ooooo01Koo`0000?oo`00ool05Ooo0`001_oo00<00?ooool07Ooo00<00?ooool0
;_oo00<00?ooool04ooo00<00?oo0000MOoo00<00?oo00001?oo00<00?ooool05_oo00000ooo003o
o`0Bool3000:ool00`00ooooo`0Lool00`00ooooo`0/ool2000Fool01000ooooo`00M?oo0`001?oo
00<00?ooool05_oo00000ooo003oo`0?ool3000>ool00`00ooooo`0Kool00`00ooooo`0[ool00`00
ooooo`0Gool00`00ool0001eool20002ool3000Hool00003ool00?oo00coo`<001;oo`03003ooooo
01[oo`03003ooooo02_oo`03003ooooo01Ooo`04003ooooo001dool20004ool00`00ooooo`0Fool0
0003003ooooo00Woo`<001Goo`03003ooooo01[oo`03003ooooo02[oo`03003ooooo01Woo`03003o
o`00073oo`@001ooo`0000<00?ooool01Ooo10006Ooo00<00?ooool06Ooo00<00?ooool0:_oo00<0
0?ooool06Ooo00<00?oo0000K?oo10008ooo00000`00ooooo`02ool3000Nool00`00ooooo`0Hool0
0`00ooooo`0Yool00`00ooooo`0Kool00`00ool0001Xool3000Wool00003003oo`00008002;oo`03
003ooooo01Ooo`03003ooooo02Soo`03003ooooo01coo`03003oo`0006Coo`@002[oo`000P009Ooo
00<00?ooool05_oo00<00?ooool0:?oo00<00?ooool07_oo00<00?oo0000Gooo1000;_oo00000ooo
003oo`0Uool00`00ooooo`0Eool00`00ooooo`0Wool00`00ooooo`0Oool00`00ool0001Lool3000b
ool000;oo`8002Goo`03003ooooo01Coo`03003ooooo02Koo`03003ooooo027oo`8005Soo`@003Go
o`001?oo00<00?ooool08ooo00<00?ooool04ooo00<00?ooool09_oo00<00?ooool0Mooo1000>Ooo
0005ool00`00ooooo`0Sool00`00ooooo`0Bool00`00ooooo`0Uool00`00ooooo`1eool3000mool0
00Koo`03003ooooo02?oo`03003ooooo017oo`03003ooooo02Coo`03003ooooo07;oo`@0043oo`00
1ooo0P009?oo00<00?ooool04?oo00<00?ooool08ooo00<00?ooool0L?oo0`00A?oo0009ool00`00
ooooo`0Rool00`00ooooo`0?ool00`00ooooo`0Rool00`00ooooo`1]ool40017ool000[oo`03003o
oooo02;oo`03003ooooo00koo`03003ooooo027oo`03003ooooo06[oo`@004_oo`002ooo00<00?oo
ool08_oo00<00?ooool03Ooo00<00?ooool08?oo00<00?ooool0J?oo0`00Cooo000<ool2000Sool2
000=ool00`00ooooo`0Nool2001Wool4001Bool000koo`03003ooooo02;oo`03003ooooo00[oo`03
003ooooo01goo`03003ooooo06?oo`@005Koo`003ooo00<00?ooool08_oo0P002_oo00<00?ooool0
7?oo00<00?ooool0HOoo0`00F_oo000@ool00`00ooooo`0Sool00`00ooooo`07ool00`00ooooo`0J
ool2001Pool4001Mool0017oo`8002Coo`8000Koo`03003ooooo01Woo`8005koo`@0067oo`004ooo
00<00?ooool08ooo0P001?oo00<00?ooool06?oo00<00?ooool0Fooo0`00IOoo000Dool00`00oooo
o`0Tool20002ool00`00ooooo`0Fool2001Jool4001Xool001Goo`03003ooooo02Goo`<001Koo`80
05Soo`@006coo`005_oo0P009ooo10003ooo1000Eooo0`00L?oo000Hool00`00ooooo`0Tool01000
oooooooo10001Ooo1P00Eooo1000Looo000Iool00`00ooooo`0Sool00`00ooooo`05ool5001Iool4
001gool001[oo`03003ooooo02;oo`03003ooooo063oo`<007_oo`006ooo0P008_oo00<00?ooool0
G?oo1000O_oo000Mool00`00ooooo`0Oool00`00ooooo`1Iool30022ool001koo`03003ooooo01ko
o`03003ooooo05Goo`@008Goo`007ooo00<00?ooool07Ooo00<00?ooool0DOoo1000ROoo000Pool0
0`00ooooo`0Lool00`00ooooo`1>ool3002=ool0027oo`8001coo`03003ooooo04[oo`@0093oo`00
8ooo00<00?ooool06?oo00<00?ooool0Aooo1000U?oo000Tool00`00ooooo`0Gool00`00ooooo`14
ool3002Hool002Goo`03003ooooo01Koo`03003ooooo043oo`@009_oo`009_oo0P005_oo00<00?oo
ool0??oo1000Wooo000Xool00`00ooooo`0Cool00`00ooooo`0iool3002Sool002Woo`03003ooooo
01;oo`03003ooooo03Goo`@00:Koo`00:_oo00<00?ooool04Ooo00<00?ooool0<Ooo1000Z_oo000[
ool2000Aool00`00ooooo`0^ool3002^ool002goo`03003ooooo00koo`03003ooooo02[oo`@00;7o
o`00;_oo00<00?ooool03Ooo00<00?ooool09_oo1000]Ooo000_ool00`00ooooo`0<ool00`00oooo
o`0Sool3002iool0033oo`8000coo`03003ooooo01ooo`@00;coo`00<_oo00<00?ooool02Ooo00<0
0?ooool07?oo0`00`?oo000cool00`00ooooo`08ool00`00ooooo`0Hool40033ool003Coo`03003o
oooo00Koo`03003ooooo01Goo`@00<Ooo`00=Ooo0P001_oo00<00?ooool04_oo0`00booo000gool0
0`00ooooo`03ool00`00ooooo`0>ool4003>ool003Soo`03003ooooo00;oo`03003ooooo00[oo`@0
0=;oo`00>Ooo00D00?ooooooo`0000Woo`<00=Koo`00>_oo0P0000?oo`00ool01?oo1000fOoo000l
ool200000ooo00000002003Mool003goo`800>7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo
8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {342.938, 156.938}} -> {-0.628379, \
0.763208, 0.00386147, 0.00386147}}]
}, Open  ]],

Cell["Power spectrum:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"ListPlot", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"Simplify", "[", 
              RowBox[{"powspec", "[", 
                RowBox[{"nn", ",", "\[Tau]", ",", "g2"}], "]"}], "]"}], "/", 
            SuperscriptBox[
              RowBox[{"Abs", "[", "Second", "]"}], "2"]}], ",", 
          RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}]}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.047619 0 0.618034 [
[.2619 -0.0125 -3 -9 ]
[.2619 -0.0125 3 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.7381 -0.0125 -6 -9 ]
[.7381 -0.0125 6 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 0 m
.2619 .00625 L
s
[(5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(10)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(15)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(20)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(0.2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(1)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.07143 .3205 Mdot
.11905 .26749 Mdot
.16667 .35951 Mdot
.21429 .26177 Mdot
.2619 .3408 Mdot
.30952 .38334 Mdot
.35714 .25936 Mdot
.40476 .13612 Mdot
.45238 .06851 Mdot
.5 .03809 Mdot
.54762 .02511 Mdot
.59524 .01994 Mdot
.64286 .01894 Mdot
.69048 .02141 Mdot
.7381 .02891 Mdot
.78571 .04656 Mdot
.83333 .08637 Mdot
.88095 .16844 Mdot
.92857 .29773 Mdot
.97619 .39217 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool005Ooo`<003Soo`@000?oo`8003Coo`@000;oo`<003Goo`@000?oo`8000[oo`00F_oo00<00?oo
ool0=ooo00D00?ooooooo`0000;oo`03003ooooo03?oo`03003ooooo00Coo`03003ooooo03;oo`03
003ooooo00?oo`04003ooooo0009ool005[oo`03003ooooo03Ooo`05003oooooool00002ool00`00
ooooo`0cool00`00ooooo`04ool00`00ooooo`0cool00`00ooooo`02ool01000ooooo`002Ooo001G
ool3000jool01@00oooooooo00000_oo00<00?ooool0<ooo00@00?ooooooo`<003Ooo`05003ooooo
ool00002ool00`00ooooo`07ool005Ooo`03003ooooo03[oo`05003oooooool00002ool00`00oooo
o`0cool01@00oooooooo0000>_oo00@00?ooool000;oo`03003ooooo00Ooo`00Eooo00<00?ooool0
>?oo0`000ooo00@00?ooool003?oo`<000?oo`03003ooooo03Goo`04003ooooo0002ool01000oooo
o`002Ooo001Gool4000iool00`00ooooo`02ool2000fool01000oooooooo1000=Ooo0P001?oo0P00
2_oo003ooolQool00?ooob7oo`00oooo8Ooo000Eoooo00050007ool001_oo`03003ooooo00Woo`03
003ooooo00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03
003ooooo00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03
003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00Woo`03003ooooo00[oo`03
003ooooo00Woo`03003ooooo00Woo`03003ooooo00[oo`03003ooooo00Woo`03003ooooo00_oo`00
6ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo097oo`8000[o
o`8006?oo`006ooo00<00?ooool0T_oo00<00?ooool02Ooo00<00?ooool02?oo0P00Eooo000Kool0
0`00ooooo`24ool2000Tool00`00ooooo`1Eool001_oo`03003ooooo08Goo`03003ooooo02koo`80
04[oo`006ooo0P00^?oo00<00?ooool0B?oo000Kool00`00ooooo`1hool20028ool001_oo`03003o
oooo07Woo`03003ooooo08Koo`006ooo00<00?ooool0`_oo0P00?_oo000Kool00`00ooooo`33ool0
0`00ooooo`0lool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0K?oo0P00U?oo000Kool00`00oooo
o`1]ool00`00ooooo`2Bool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0<koo`8003;oo`006ooo00<00?ooool0cooo00<0
0?ooool0<?oo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo
0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo
0ooo0008ool20004ool00`00ooooo`02ool40004ool00`00ooooo`3oool3ool000Ooo`04003ooooo
0008ool00`00ooooo`05ool00`00ooooo`3oool3ool000Ooo`04003ooooo0009ool00`00ooooo`04
ool3003oool3ool000Ooo`04003ooooo000:ool00`00ooooo`03ool00`00ooooo`3oool3ool000Oo
o`04003ooooo000;ool00`00ooooo`02ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool0
1000ooooo`001?oo00<00?ooool0Gooo0P00XOoo0008ool2000:ool20005ool00`00ooooo`1Pool0
0`00ooooo`2Oool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0=_oo`8002Goo`006ooo00<00?ooool0g?oo00<00?ooool08ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0?ooo`?oo`002?oo0P001?oo00<00?ooool01?oo00<00?ooool00ooo
00<00?ooool0oooo0ooo0007ool01000ooooo`002_oo00<00?ooool00ooo00<00?ooool0oooo0ooo
0007ool01000ooooo`001ooo1@001?oo0`00oooo0ooo0007ool01000ooooo`001ooo00@00?ooool0
00Goo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Soo`03003oo`0000Goo`03003ooooo0?oo
o`?oo`001ooo00@00?ooool000Woo`8000Goo`03003ooooo02koo`8002?oo`800:goo`002?oo0P00
2ooo00<00?ooool00ooo00<00?ooool0;ooo00<00?ooool08_oo00<00?ooool0Zooo000Kool00`00
ooooo`0Eool2003[ool001_oo`03003ooooo01Koo`03003ooooo0>Woo`006ooo00<00?ooool0oooo
0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0>Ooo`8001Woo`006ooo00<00?ooool0j?oo00<0
0?ooool05ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo
1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo00Woo`800?Ooo`006ooo00<00?ooool0
2_oo00<00?ooool0mOoo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo
00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`8003_oo`800<Koo`006ooo00<0
0?ooool0>ooo00<00?ooool0a?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?o
o`006ooo00<00?ooool08_oo0P00g_oo000Kool00`00ooooo`0Sool00`00ooooo`3Lool000Soo`80
00Coo`03003ooooo00?oo`8000Goo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Soo`04003o
oooo0004ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool01000ooooo`001?oo0`00oooo
0ooo0007ool01000ooooo`002?oo0`001Ooo00<00?ooool0oooo0ooo0007ool01000ooooo`002?oo
00<00?ooool01Ooo00<00?ooool0oooo0ooo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<0
0?ooool0Aooo0P00^Ooo0008ool2000:ool30004ool00`00ooooo`18ool00`00ooooo`2gool001_o
o`03003ooooo0??oo`8000goo`006ooo00<00?ooool0m?oo00<00?ooool02ooo000Kool00`00oooo
o`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3
ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3
ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_o
o`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_o
o`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_o
o`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0?ooo`?oo`002?oo0P001?oo00<00?ooool00ooo0P001Ooo00<00?ooool0oooo0ooo0007ool0
1000ooooo`002?oo00@00?ooool000Coo`03003ooooo0?ooo`?oo`001ooo00@00?ooool000Soo`04
003ooooo0004ool3003oool3ool000Ooo`04003ooooo0009ool20005ool00`00ooooo`3oool3ool0
00Ooo`04003ooooo0008ool01000ooooo`001?oo00<00?ooool0oooo0ooo0007ool01000ooooo`00
2?oo00@00?ooool000Coo`03003ooooo0?ooo`?oo`002?oo0P002_oo0P001Ooo00<00?ooool0oooo
0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo
0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001?oo`@000Coo`03003ooooo0?ooo`?oo`005Ooo00<00?ooool00ooo00<00?ooool0oooo
0ooo000Eool00`00ooooo`03ool3003oool3ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<00?oo
ool0oooo2Ooo000Cool3003oool;ool001Goo`03003ooooo0?ooo`Woo`00\
\>"],
  ImageRangeCache->{{{131.125, 360.5}, {537.625, 396.312}} -> {-15.5509, \
3.0215, 0.0811648, 0.00625368}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {None, Inherited, None}},
PageHeaderLines->{False, False},
PrintingOptions->{"PrintingMargins"->{{93.5625, 54}, {54, 28.75}},
"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "felix", \
"tmp"}, "qqq.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 26, 0, 54, "Section"],
Cell[1768, 53, 135, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[1928, 60, 27, 0, 45, "Subsection"],
Cell[1958, 62, 67, 1, 27, "Input"],
Cell[2028, 65, 34, 0, 32, "Text"],
Cell[2065, 67, 786, 20, 44, "Input"],
Cell[2854, 89, 31, 0, 32, "Text"],
Cell[2888, 91, 505, 13, 28, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3430, 109, 35, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[3490, 113, 45, 0, 42, "Subsubsection"],
Cell[3538, 115, 634, 18, 60, "Input"],
Cell[4175, 135, 30, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[4230, 139, 490, 13, 27, "Input"],
Cell[4723, 154, 16788, 535, 186, 6560, 404, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[21526, 692, 31, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[21582, 696, 342, 9, 28, "Input"],
Cell[21927, 707, 16042, 513, 186, 6409, 390, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[38018, 1226, 48, 0, 42, "Subsubsection"],
Cell[38069, 1228, 529, 15, 60, "Input"],
Cell[38601, 1245, 30, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[38656, 1249, 926, 23, 43, "Input"],
Cell[39585, 1274, 39516, 1259, 242, 15739, 961, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[79116, 2536, 31, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[79172, 2540, 485, 12, 28, "Input"],
Cell[79660, 2554, 9997, 307, 186, 3040, 217, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

