(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     44018,       1330]*)
(*NotebookOutlinePosition[     44972,       1359]*)
(*  CellTagsIndexPosition[     44928,       1355]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Calculations used to create input data", "Subsection"],

Cell[CellGroupData[{

Cell["Tools", "Subsubsection"],

Cell[BoxData[{
    \(adj[x_] := Transpose[Conjugate[x]]; \), "\[IndentingNewLine]", 
    \(innerProd[\[Psi]_, \[CurlyPhi]_] := \((adj[\[Psi]] . \[CurlyPhi])\)\
\[LeftDoubleBracket]1, 1\[RightDoubleBracket]\), "\[IndentingNewLine]", 
    \(normalize[\[Psi]_] := \[Psi]/\@\((adj[\[Psi]] . \[Psi])\)\
\[LeftDoubleBracket]1, 1\[RightDoubleBracket]; \), "\[IndentingNewLine]", 
    \(terseForm[x_] := 
      NumberForm[MatrixForm[Chop[x]], 
        2]\  (*Nice\ and\ short\ display\ of\ matrices*) \)}], "Input"],

Cell[BoxData[
    \(ket0 = {{1}, {0}}; ket1 = {{0}, {1}}; \)], "Input"],

Cell["Conversion of stuff to strings:", "Text"],

Cell[BoxData[{
    \(complexToString[z_] := "\<(\>" <> ToString[N[Re[z]]] <> "\<,\>" <> 
        ToString[N[Im[z]]] <> "\<)\>"; \), "\[IndentingNewLine]", 
    \(stateToString[\[Psi]_] := 
      complexToString[\[Psi]\[LeftDoubleBracket]1, 
            1\[RightDoubleBracket]] <> "\< \>" <> 
        complexToString[\[Psi]\[LeftDoubleBracket]2, 
            1\[RightDoubleBracket]]; \), "\[IndentingNewLine]", 
    \(opToString[a_] := 
      StringJoin[
        Flatten[Table[
            complexToString[
                a\[LeftDoubleBracket]j, 
                  k\[RightDoubleBracket]] <> "\< \>", {j, 1, 2}, {k, 1, 
              2}]]]\)}], "Input"],

Cell[TextData[{
  "Measurement operators with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          OverscriptBox["p", "_"], "=", 
          FractionBox["1", "2"]}], TraditionalForm]]],
  ":"
}], "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"m0", "[", "\[CapitalDelta]p_", "]"}], ":=", 
        RowBox[{
          RowBox[{
            SqrtBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "-", "\[CapitalDelta]p"}], ")"}], "/", "2"}]], 
            
            RowBox[{"ket0", ".", 
              RowBox[{"adj", "[", "ket0", "]"}]}]}], "+", 
          RowBox[{
            SqrtBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "+", "\[CapitalDelta]p"}], ")"}], "/", "2"}]], 
            
            RowBox[{"ket1", ".", 
              RowBox[{"adj", "[", "ket1", "]"}]}]}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"m1", "[", "\[CapitalDelta]p_", "]"}], ":=", 
        RowBox[{
          RowBox[{
            SqrtBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "+", "\[CapitalDelta]p"}], ")"}], "/", "2"}]], 
            
            RowBox[{"ket0", ".", 
              RowBox[{"adj", "[", "ket0", "]"}]}]}], "+", 
          RowBox[{
            SqrtBox[
              RowBox[{
                RowBox[{"(", 
                  RowBox[{"1", "-", "\[CapitalDelta]p"}], ")"}], "/", "2"}]], 
            
            RowBox[{"ket1", ".", 
              RowBox[{"adj", "[", "ket1", "]"}]}]}]}]}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Miscellaneous", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.0", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.0", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.707107,0.) (0.,0.) (0.,0.) (0.707107,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.707107,0.) (0.,0.) (0.,0.) (0.707107,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.1", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.1", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.67082,0.) (0.,0.) (0.,0.) (0.74162,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.74162,0.) (0.,0.) (0.,0.) (0.67082,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.2", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.2", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.632456,0.) (0.,0.) (0.,0.) (0.774597,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.774597,0.) (0.,0.) (0.,0.) (0.632456,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.3", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.3", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.591608,0.) (0.,0.) (0.,0.) (0.806226,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.806226,0.) (0.,0.) (0.,0.) (0.591608,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.4", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.4", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.547723,0.) (0.,0.) (0.,0.) (0.83666,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.83666,0.) (0.,0.) (0.,0.) (0.547723,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.5", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.5", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.5,0.) (0.,0.) (0.,0.) (0.866025,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.866025,0.) (0.,0.) (0.,0.) (0.5,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.6", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.6", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.447214,0.) (0.,0.) (0.,0.) (0.894427,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.894427,0.) (0.,0.) (0.,0.) (0.447214,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.7", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.7", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.387298,0.) (0.,0.) (0.,0.) (0.921954,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.921954,0.) (0.,0.) (0.,0.) (0.387298,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.8", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.8", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.316228,0.) (0.,0.) (0.,0.) (0.948683,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.948683,0.) (0.,0.) (0.,0.) (0.316228,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "0.9", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "0.9", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.223607,0.) (0.,0.) (0.,0.) (0.974679,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(0.974679,0.) (0.,0.) (0.,0.) (0.223607,0.) \"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"opToString", "[", 
      RowBox[{"m0", "[", "1.0", "]"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "[", 
      RowBox[{"m1", "[", "1.0", "]"}], "]"}]}], "Input"],

Cell[BoxData["\<\"(0.,0.) (0.,0.) (0.,0.) (1.,0.) \"\>"], "Output"],

Cell[BoxData["\<\"(1.,0.) (0.,0.) (0.,0.) (0.,0.) \"\>"], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Position of Peaks in Power Spectrum (",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Tau]", "=", 
            RowBox[{"1", "/", "16"}]}], ",", 
          RowBox[{
            OverscriptBox["p", "_"], "=", 
            RowBox[{"1", "/", "2"}]}]}], TraditionalForm]]],
  ")"
}], "Subsection"],

Cell[BoxData[
    \(\(maximums = {0.392319, 0.393086, 0.393086, 0.389251, 0.390018, 
          0.386183, 0.37161, 0.358571, 0.12694, 0.00652241, 
          0.00192047};\)\)], "Input"],

Cell[BoxData[
    \(\(maximumsScaled = maximums*16. /\((2  \[Pi])\);\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(maximumsScaledWith\[CapitalDelta]p = 
      Transpose[{maximumsScaled, Table[x, {x, 0, 1, 0.1}]}]\)], "Input"],

Cell[BoxData[
    \({{0.9990321299019086`, 0}, {1.0009852793635323`, 
        0.1`}, {1.0009852793635323`, 0.2`}, {0.9912195320554137`, 
        0.30000000000000004`}, {0.9931726815170374`, 
        0.4`}, {0.9834069342089187`, 0.5`}, {0.9462970944380676`, 
        0.6000000000000001`}, {0.9130935535904641`, 
        0.7000000000000001`}, {0.32325005561736314`, 
        0.8`}, {0.016609180677952147`, 0.9`}, {0.004890436696955076`, 
        1.`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[maximumsScaledWith\[CapitalDelta]p, AspectRatio \[Rule] 1, 
        PlotRange \[Rule] {{0, 1.1}, {0, 1.1}}, PlotJoined \[Rule] False, 
        AxesLabel \[Rule] {\*"\"\<\[Omega]/\!\(\[Omega]\_R\)\>\"", \ "\<\
\[CapitalDelta]p\>"}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.3744e-17 0.909091 -2.3744e-17 0.909091 [
[.18182 -0.0125 -9 -9 ]
[.18182 -0.0125 9 0 ]
[.36364 -0.0125 -9 -9 ]
[.36364 -0.0125 9 0 ]
[.54545 -0.0125 -9 -9 ]
[.54545 -0.0125 9 0 ]
[.72727 -0.0125 -9 -9 ]
[.72727 -0.0125 9 0 ]
[.90909 -0.0125 -3 -9 ]
[.90909 -0.0125 3 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 27.5625 4.90625 ]
[-0.0125 .18182 -18 -4.5 ]
[-0.0125 .18182 0 4.5 ]
[-0.0125 .36364 -18 -4.5 ]
[-0.0125 .36364 0 4.5 ]
[-0.0125 .54545 -18 -4.5 ]
[-0.0125 .54545 0 4.5 ]
[-0.0125 .72727 -18 -4.5 ]
[-0.0125 .72727 0 4.5 ]
[-0.0125 .90909 -6 -4.5 ]
[-0.0125 .90909 0 4.5 ]
[0 1.025 -8 0 ]
[0 1.025 8 9.8125 ]
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18182 0 m
.18182 .00625 L
s
[(0.2)] .18182 -0.0125 0 1 Mshowa
.36364 0 m
.36364 .00625 L
s
[(0.4)] .36364 -0.0125 0 1 Mshowa
.54545 0 m
.54545 .00625 L
s
[(0.6)] .54545 -0.0125 0 1 Mshowa
.72727 0 m
.72727 .00625 L
s
[(0.8)] .72727 -0.0125 0 1 Mshowa
.90909 0 m
.90909 .00625 L
s
[(1)] .90909 -0.0125 0 1 Mshowa
.125 Mabswid
.04545 0 m
.04545 .00375 L
s
.09091 0 m
.09091 .00375 L
s
.13636 0 m
.13636 .00375 L
s
.22727 0 m
.22727 .00375 L
s
.27273 0 m
.27273 .00375 L
s
.31818 0 m
.31818 .00375 L
s
.40909 0 m
.40909 .00375 L
s
.45455 0 m
.45455 .00375 L
s
.5 0 m
.5 .00375 L
s
.59091 0 m
.59091 .00375 L
s
.63636 0 m
.63636 .00375 L
s
.68182 0 m
.68182 .00375 L
s
.77273 0 m
.77273 .00375 L
s
.81818 0 m
.81818 .00375 L
s
.86364 0 m
.86364 .00375 L
s
.95455 0 m
.95455 .00375 L
s
1 0 m
1 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 17.8125 translate 1 -1 scale
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
81.000 12.750 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
86.562 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .18182 m
.00625 .18182 L
s
[(0.2)] -0.0125 .18182 1 0 Mshowa
0 .36364 m
.00625 .36364 L
s
[(0.4)] -0.0125 .36364 1 0 Mshowa
0 .54545 m
.00625 .54545 L
s
[(0.6)] -0.0125 .54545 1 0 Mshowa
0 .72727 m
.00625 .72727 L
s
[(0.8)] -0.0125 .72727 1 0 Mshowa
0 .90909 m
.00625 .90909 L
s
[(1)] -0.0125 .90909 1 0 Mshowa
.125 Mabswid
0 .04545 m
.00375 .04545 L
s
0 .09091 m
.00375 .09091 L
s
0 .13636 m
.00375 .13636 L
s
0 .22727 m
.00375 .22727 L
s
0 .27273 m
.00375 .27273 L
s
0 .31818 m
.00375 .31818 L
s
0 .40909 m
.00375 .40909 L
s
0 .45455 m
.00375 .45455 L
s
0 .5 m
.00375 .5 L
s
0 .59091 m
.00375 .59091 L
s
0 .63636 m
.00375 .63636 L
s
0 .68182 m
.00375 .68182 L
s
0 .77273 m
.00375 .77273 L
s
0 .81818 m
.00375 .81818 L
s
0 .86364 m
.00375 .86364 L
s
0 .95455 m
.00375 .95455 L
s
0 1 m
.00375 1 L
s
.25 Mabswid
0 0 m
0 1 L
s
gsave
0 1.025 -69 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 17.8125 translate 1 -1 scale
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.008 w
.90821 0 Mdot
.90999 .09091 Mdot
.90999 .18182 Mdot
.90111 .27273 Mdot
.90288 .36364 Mdot
.89401 .45455 Mdot
.86027 .54545 Mdot
.83009 .63636 Mdot
.29386 .72727 Mdot
.0151 .81818 Mdot
.00445 .90909 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 288},
  ImageMargins->{{41, 0}, {0, 1}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000181000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo000hool2
0004ool00`00ooooo`02ool4000Kool20004ool00`00ooooo`04ool00`00ooooo`0Jool20004ool0
0`00ooooo`03ool2000Mool20004ool00`00ooooo`03ool2000Qool4000fool003Ooo`04003ooooo
0008ool00`00ooooo`0Kool01000ooooo`002_oo00<00?ooool06Ooo00@00?ooool000Soo`04003o
oooo000Kool01000ooooo`002?oo00@00?ooool002;oo`03003ooooo03Goo`00=ooo00@00?ooool0
00Woo`03003ooooo01[oo`04003ooooo0007ool5000Jool01000ooooo`002?oo00@00?ooool001_o
o`04003ooooo0008ool01000ooooo`008_oo00<00?ooool0=Ooo000gool01000ooooo`002_oo00<0
0?ooool06Ooo00@00?ooool000Ooo`04003ooooo000Kool01000ooooo`002?oo0`007?oo00@00?oo
ool000Woo`8002?oo`03003ooooo03Goo`00=ooo00@00?ooool000_oo`03003ooooo01Soo`04003o
oooo0008ool00`00ool0000Kool01000ooooo`002?oo00<00?ooool07?oo00@00?ooool000Soo`04
003ooooo000Rool00`00ooooo`0eool003Ooo`04003ooooo0008ool01000ooooo`006_oo00@00?oo
ool000Woo`8001_oo`04003ooooo0008ool00`00ooooo`0Lool01000ooooo`002?oo00@00?ooool0
023oo`<003Ooo`00>?oo0P002_oo0P007?oo0P002ooo00<00?ooool06_oo0P002_oo0`007?oo0P00
2_oo0P008ooo00<00?ooool0=Ooo003oool=ool00`00ooooo`09ool00`00ool00005ool00?ooo`Oo
o`@000?oo`03003ooooo00;oo`@000;oo`<000Goo`00oooo1_oo00H00?ooool00?oo0002ool01@00
oooooooo00000_oo00L00?oo003oo`00ool00005ool001CoonX000Ooo`06003ooooo003oo`000_oo
00D00?ooooooo`0000;oo`04003oo`00ool30005ool001Coo`03003ooooo00Soo`03003ooooo00So
o`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Goo`03003ooooo00;oo`05003oooooool00002ool00`00ooooo`02ool00`00oooo
o`07ool001Coo`03003ooooo0>ooo`05003oooooool00003ool01@00oooooooo00000ooo00<00?oo
ool01ooo000Dool00`00ooooo`3hool00`00ooooo`0>ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool2003Bool00`00ooooo`0eool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00
oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool000;oo`8000Coo`03003ooooo00;oo`@000?oo`03003o
oooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`03003ooooo00Coo`03003ooooo0?ooo`[oo`0000Go
o`00ooooo`0000Woo`03003ooooo00?oo`<00=7oo`03003ooooo03Goo`0000Goo`00ooooo`0000[o
o`03003ooooo00;oo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000_oo`05003oooooool0003o
ool<ool00005ool00?ooool00008ool01000ooooo`000ooo00<00?ooool0oooo2_oo0002ool2000:
ool20004ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00
ooooo`3?ool00`00ooooo`0gool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo0002ool2
0004ool00`00ooooo`04ool00`00ooooo`02ool00`00ooooo`3oool:ool00005ool00?ooool0000:
ool00`00ooooo`02ool00`00ooooo`3oool:ool00005ool00?ooool00007ool50003ool3003oool:
ool00005ool00?ooool00007ool01000ooooo`001?oo00<00?ooool0cooo00<00?ooool0=ooo0000
1Ooo003ooooo00002?oo00<00?oo00001?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002Ooo
0P001?oo00<00?ooool0oooo2_oo0002ool2000;ool00`00ooooo`02ool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003>ool00`00ooooo`0iool001Co
o`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Co
o`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Co
o`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Co
o`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool000;oo`8000Co
o`03003ooooo00?oo`8000Coo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo
0003ool00`00ooooo`3oool:ool00005ool00?ooool00008ool01000ooooo`000ooo0`00aOoo00<0
0?ooool0@Ooo00001Ooo003ooooo00002?oo0`001?oo00<00?ooool0oooo2_oo00001Ooo003ooooo
00002?oo00<00?ooool01?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00<00?ooool0
1?oo00<00?ooool0oooo2_oo0002ool2000:ool30003ool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo0P00_ooo00<00?ooool0B?oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo0002ool20004ool00`00ooooo`03ool20004ool00`00ooooo`3o
ool:ool00005ool00?ooool00008ool01000ooooo`000ooo00<00?ooool0oooo2_oo00001Ooo003o
oooo00002?oo00@00?ooool000?oo`<0047oo`03003ooooo0<Goo`0000Goo`00ooooo`0000Woo`80
00Coo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool00`00ooooo`3o
ool:ool00005ool00?ooool00008ool01000ooooo`000ooo00<00?ooool0oooo2_oo0002ool2000:
ool20004ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool01@00
oooooooo0000oooo2?oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`003Ooo10000ooo
00<00?ooool0oooo2_oo000?ool00`00ooooo`02ool00`00ooooo`3oool:ool000ooo`03003ooooo
00;oo`<00?ooo`[oo`003ooo00<00?ooool00_oo0P00oooo2ooo000?ool00`00ooooo`02ool00`00
ooooo`3oool:ool000goo`<000Coo`03003ooooo0?ooo`[oo`003ooo00<00?ooool00_oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo000Dool3003oool:ool001Goo`03003ooooo
0?ooo`Woo`003_oo1@000_oo0`00oooo2Ooo000?ool00`00ool00003ool01000ooooo`00oooo2?oo
000?ool00`00ool00003ool01000ooooo`00oooo2?oo000@ool00`00ooooo`02ool01000ooooo`00
oooo2?oo000@ool01P00oooooooo003oo`800?ooo`Woo`004?oo00<00?ooool0oooo3_oo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{89.9375, 319.312}, {334.312, 104.938}} -> {-0.630971, \
0.501121, 0.00473246, 0.00473246}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(maximumsScaledWith\[CapitalDelta]p2 = 
      Transpose[{maximumsScaled, Table[x\^2, {x, 0, 1, 0.1}]}]\)], "Input"],

Cell[BoxData[
    \({{0.9990321299019086`, 0}, {1.0009852793635323`, 
        0.010000000000000002`}, {1.0009852793635323`, 
        0.04000000000000001`}, {0.9912195320554137`, 
        0.09000000000000002`}, {0.9931726815170374`, 
        0.16000000000000003`}, {0.9834069342089187`, 
        0.25`}, {0.9462970944380676`, 
        0.3600000000000001`}, {0.9130935535904641`, 
        0.4900000000000001`}, {0.32325005561736314`, 
        0.6400000000000001`}, {0.016609180677952147`, 
        0.81`}, {0.004890436696955076`, 1.`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[maximumsScaledWith\[CapitalDelta]p2, AspectRatio \[Rule] 1, 
        PlotRange \[Rule] {{0, 1.1}, {0, 1.1}}, PlotJoined \[Rule] False, 
        AxesLabel \[Rule] {\*"\"\<\[Omega]/\!\(\[Omega]\_R\)\>\"", \ "\<\
\[CapitalDelta]p\>"}];\)\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.3744e-17 0.909091 -2.3744e-17 0.909091 [
[.18182 -0.0125 -9 -9 ]
[.18182 -0.0125 9 0 ]
[.36364 -0.0125 -9 -9 ]
[.36364 -0.0125 9 0 ]
[.54545 -0.0125 -9 -9 ]
[.54545 -0.0125 9 0 ]
[.72727 -0.0125 -9 -9 ]
[.72727 -0.0125 9 0 ]
[.90909 -0.0125 -3 -9 ]
[.90909 -0.0125 3 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 27.5625 4.90625 ]
[-0.0125 .18182 -18 -4.5 ]
[-0.0125 .18182 0 4.5 ]
[-0.0125 .36364 -18 -4.5 ]
[-0.0125 .36364 0 4.5 ]
[-0.0125 .54545 -18 -4.5 ]
[-0.0125 .54545 0 4.5 ]
[-0.0125 .72727 -18 -4.5 ]
[-0.0125 .72727 0 4.5 ]
[-0.0125 .90909 -6 -4.5 ]
[-0.0125 .90909 0 4.5 ]
[0 1.025 -8 0 ]
[0 1.025 8 9.8125 ]
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18182 0 m
.18182 .00625 L
s
[(0.2)] .18182 -0.0125 0 1 Mshowa
.36364 0 m
.36364 .00625 L
s
[(0.4)] .36364 -0.0125 0 1 Mshowa
.54545 0 m
.54545 .00625 L
s
[(0.6)] .54545 -0.0125 0 1 Mshowa
.72727 0 m
.72727 .00625 L
s
[(0.8)] .72727 -0.0125 0 1 Mshowa
.90909 0 m
.90909 .00625 L
s
[(1)] .90909 -0.0125 0 1 Mshowa
.125 Mabswid
.04545 0 m
.04545 .00375 L
s
.09091 0 m
.09091 .00375 L
s
.13636 0 m
.13636 .00375 L
s
.22727 0 m
.22727 .00375 L
s
.27273 0 m
.27273 .00375 L
s
.31818 0 m
.31818 .00375 L
s
.40909 0 m
.40909 .00375 L
s
.45455 0 m
.45455 .00375 L
s
.5 0 m
.5 .00375 L
s
.59091 0 m
.59091 .00375 L
s
.63636 0 m
.63636 .00375 L
s
.68182 0 m
.68182 .00375 L
s
.77273 0 m
.77273 .00375 L
s
.81818 0 m
.81818 .00375 L
s
.86364 0 m
.86364 .00375 L
s
.95455 0 m
.95455 .00375 L
s
1 0 m
1 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 17.8125 translate 1 -1 scale
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
69.000 11.250 moveto
(\\220) show
75.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
81.000 12.750 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 7.125 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(R) show
86.562 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .18182 m
.00625 .18182 L
s
[(0.2)] -0.0125 .18182 1 0 Mshowa
0 .36364 m
.00625 .36364 L
s
[(0.4)] -0.0125 .36364 1 0 Mshowa
0 .54545 m
.00625 .54545 L
s
[(0.6)] -0.0125 .54545 1 0 Mshowa
0 .72727 m
.00625 .72727 L
s
[(0.8)] -0.0125 .72727 1 0 Mshowa
0 .90909 m
.00625 .90909 L
s
[(1)] -0.0125 .90909 1 0 Mshowa
.125 Mabswid
0 .04545 m
.00375 .04545 L
s
0 .09091 m
.00375 .09091 L
s
0 .13636 m
.00375 .13636 L
s
0 .22727 m
.00375 .22727 L
s
0 .27273 m
.00375 .27273 L
s
0 .31818 m
.00375 .31818 L
s
0 .40909 m
.00375 .40909 L
s
0 .45455 m
.00375 .45455 L
s
0 .5 m
.00375 .5 L
s
0 .59091 m
.00375 .59091 L
s
0 .63636 m
.00375 .63636 L
s
0 .68182 m
.00375 .68182 L
s
0 .77273 m
.00375 .77273 L
s
0 .81818 m
.00375 .81818 L
s
0 .86364 m
.00375 .86364 L
s
0 .95455 m
.00375 .95455 L
s
0 1 m
.00375 1 L
s
.25 Mabswid
0 0 m
0 1 L
s
gsave
0 1.025 -69 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 17.8125 translate 1 -1 scale
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(D) show
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.008 w
.90821 0 Mdot
.90999 .00909 Mdot
.90999 .03636 Mdot
.90111 .08182 Mdot
.90288 .14545 Mdot
.89401 .22727 Mdot
.86027 .32727 Mdot
.83009 .44545 Mdot
.29386 .58182 Mdot
.0151 .73636 Mdot
.00445 .90909 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 288},
  ImageMargins->{{41, 0}, {0, 1}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P000181000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo000hool2
0004ool00`00ooooo`02ool4000Kool20004ool00`00ooooo`04ool00`00ooooo`0Jool20004ool0
0`00ooooo`03ool2000Mool20004ool00`00ooooo`03ool2000Qool4000fool003Ooo`04003ooooo
0008ool00`00ooooo`0Kool01000ooooo`002_oo00<00?ooool06Ooo00@00?ooool000Soo`04003o
oooo000Kool01000ooooo`002?oo00@00?ooool002;oo`03003ooooo03Goo`00=ooo00@00?ooool0
00Woo`03003ooooo01[oo`04003ooooo0007ool5000Jool01000ooooo`002?oo00@00?ooool001_o
o`04003ooooo0008ool01000ooooo`008_oo00<00?ooool0=Ooo000gool01000ooooo`002_oo00<0
0?ooool06Ooo00@00?ooool000Ooo`04003ooooo000Kool01000ooooo`002?oo0`007?oo00@00?oo
ool000Woo`8002?oo`03003ooooo03Goo`00=ooo00@00?ooool000_oo`03003ooooo01Soo`04003o
oooo0008ool00`00ool0000Kool01000ooooo`002?oo00<00?ooool07?oo00@00?ooool000Soo`04
003ooooo000Rool00`00ooooo`0eool003Ooo`04003ooooo0008ool01000ooooo`006_oo00@00?oo
ool000Woo`8001_oo`04003ooooo0008ool00`00ooooo`0Lool01000ooooo`002?oo00@00?ooool0
023oo`<003Ooo`00>?oo0P002_oo0P007?oo0P002ooo00<00?ooool06_oo0P002_oo0`007?oo0P00
2_oo0P008ooo00<00?ooool0=Ooo003oool=ool00`00ooooo`09ool00`00ool00005ool00?ooo`Oo
o`@000?oo`03003ooooo00;oo`@000;oo`<000Goo`00oooo1_oo00H00?ooool00?oo0002ool01@00
oooooooo00000_oo00L00?oo003oo`00ool00005ool001CoonX000Ooo`06003ooooo003oo`000_oo
00D00?ooooooo`0000;oo`04003oo`00ool30005ool001Coo`03003ooooo00Soo`03003ooooo00So
o`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Soo`03003ooooo00Oo
o`03003ooooo00Goo`03003ooooo00;oo`05003oooooool00002ool00`00ooooo`02ool00`00oooo
o`07ool001Coo`03003ooooo0=7oo`03003ooooo01_oo`05003oooooool00003ool01@00oooooooo
00000ooo00<00?ooool01ooo000Dool00`00ooooo`3hool00`00ooooo`0>ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3Aool00`00ooooo`0eool001Co
o`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0<ooo`03003ooooo03Ooo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0<ooo`03003ooooo03Ooo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Co
o`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool000;o
o`8000Coo`03003ooooo00;oo`@000?oo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`03
003ooooo00Coo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Woo`03003ooooo00?oo`<00?oo
o`[oo`0000Goo`00ooooo`0000[oo`03003ooooo00;oo`03003ooooo0?ooo`[oo`0000Goo`00oooo
o`0000_oo`05003oooooool0003oool<ool00005ool00?ooool00008ool01000ooooo`000ooo00<0
0?ooool0oooo2_oo0002ool2000:ool20004ool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800<koo`03003o
oooo03Woo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`35ool00`00ooooo`11ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo0002ool20004ool00`00ooooo`04ool00`00ooooo`02ool0
0`00ooooo`3oool:ool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`3oool:ool0
0005ool00?ooool00007ool50003ool3003oool:ool00005ool00?ooool00007ool01000ooooo`00
1?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00<00?oo00001?oo00<00?ooool0oooo
2_oo00001Ooo003ooooo00002Ooo0P001?oo00<00?ooool0oooo2_oo0002ool2000;ool00`00oooo
o`02ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0;koo`03003ooooo04Soo`005?oo00<00?ooool0oooo
2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool000;oo`8000Coo`03003ooooo00?oo`8000Coo`03003ooooo0?ooo`[oo`0000Go
o`00ooooo`0000Soo`04003ooooo0003ool00`00ooooo`3oool:ool00005ool00?ooool00008ool0
1000ooooo`000ooo0`00oooo2_oo00001Ooo003ooooo00002?oo0`001?oo00<00?ooool0oooo2_oo
00001Ooo003ooooo00002?oo00<00?ooool01?oo00<00?ooool0oooo2_oo00001Ooo003ooooo0000
2?oo00<00?ooool01?oo00<00?ooool0oooo2_oo0002ool2000:ool30003ool00`00ooooo`3oool:
ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:
ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0@Ooo00<00?ooool0aOoo000Dool2003o
ool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo0002ool20004ool00`00ooooo`03ool20004
ool00`00ooooo`3oool:ool00005ool00?ooool00008ool01000ooooo`000ooo00<00?ooool0oooo
2_oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`<00?ooo`[oo`0000Goo`00ooooo`0000Wo
o`8000Coo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool00`00oooo
o`3oool:ool00005ool00?ooool00008ool01000ooooo`000ooo00D00?ooooooo`000?ooo`Soo`00
0_oo0P002_oo0P001?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`00
5?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`00
5?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`00
5?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`00
5?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`003Ooo1000
0ooo00<00?ooool0oooo2_oo000?ool00`00ooooo`02ool00`00ooooo`3oool:ool000ooo`03003o
oooo00;oo`<00?ooo`[oo`003ooo00<00?ooool00_oo0P00oooo2ooo000?ool00`00ooooo`02ool0
0`00ooooo`3oool:ool000goo`<000Coo`03003ooooo0?ooo`[oo`003ooo00<00?ooool00_oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`00oooo8Ooo003ooolQool00?oo
ob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo000Dool3003oool:ool001Goo`03003o
oooo0?ooo`Woo`003_oo1@000_oo0`00oooo2Ooo000?ool00`00ool00003ool01000ooooo`00oooo
2?oo000?ool00`00ool00003ool01000ooooo`00oooo2?oo000@ool00`00ooooo`02ool01000oooo
o`00oooo2?oo000@ool01P00oooooooo003oo`800?ooo`Woo`004?oo00<00?ooool0oooo3_oo003o
oolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{89.9375, 319.312}, {660.25, 430.875}} -> {-0.630971, \
2.42923, 0.00473246, 0.00473246}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for X",
ScreenRectangle->{{0, 1152}, {0, 864}},
WindowSize->{1124, 836},
WindowMargins->{{-4, Automatic}, {Automatic, 3}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "net", "hubble", \
"Users", "felix", "tmp-work", "0408191106"}, "maximums.nb.ps", \
CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 60, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[1812, 56, 30, 0, 42, "Subsubsection"],
Cell[1845, 58, 507, 8, 98, "Input"],
Cell[2355, 68, 71, 1, 27, "Input"],
Cell[2429, 71, 47, 0, 32, "Text"],
Cell[2479, 73, 653, 14, 59, "Input"],
Cell[3135, 89, 210, 8, 33, "Text"],
Cell[3348, 99, 1392, 41, 52, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4777, 145, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[4840, 149, 205, 4, 43, "Input"],
Cell[5048, 155, 79, 0, 27, "Output"],
Cell[5130, 157, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5246, 162, 205, 4, 43, "Input"],
Cell[5454, 168, 77, 0, 27, "Output"],
Cell[5534, 170, 77, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5648, 175, 205, 4, 43, "Input"],
Cell[5856, 181, 79, 0, 27, "Output"],
Cell[5938, 183, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6054, 188, 205, 4, 43, "Input"],
Cell[6262, 194, 79, 0, 27, "Output"],
Cell[6344, 196, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6460, 201, 205, 4, 43, "Input"],
Cell[6668, 207, 78, 0, 27, "Output"],
Cell[6749, 209, 78, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6864, 214, 205, 4, 43, "Input"],
Cell[7072, 220, 74, 0, 27, "Output"],
Cell[7149, 222, 74, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7260, 227, 205, 4, 43, "Input"],
Cell[7468, 233, 79, 0, 27, "Output"],
Cell[7550, 235, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7666, 240, 205, 4, 43, "Input"],
Cell[7874, 246, 79, 0, 27, "Output"],
Cell[7956, 248, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8072, 253, 205, 4, 43, "Input"],
Cell[8280, 259, 79, 0, 27, "Output"],
Cell[8362, 261, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8478, 266, 205, 4, 43, "Input"],
Cell[8686, 272, 79, 0, 27, "Output"],
Cell[8768, 274, 79, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8884, 279, 205, 4, 43, "Input"],
Cell[9092, 285, 67, 0, 27, "Output"],
Cell[9162, 287, 67, 0, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9290, 294, 333, 11, 45, "Subsection"],
Cell[9626, 307, 183, 3, 27, "Input"],
Cell[9812, 312, 83, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[9920, 317, 130, 2, 27, "Input"],
Cell[10053, 321, 463, 8, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10553, 334, 277, 4, 27, "Input"],
Cell[10833, 340, 16058, 478, 297, 6179, 352, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26928, 823, 134, 2, 31, "Input"],
Cell[27065, 827, 547, 10, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27649, 842, 278, 4, 27, "Input"],
Cell[27930, 848, 16060, 478, 297, 6179, 352, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

