(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     55418,       2785]*)
(*NotebookOutlinePosition[     56487,       2818]*)
(*  CellTagsIndexPosition[     56443,       2814]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[BoxData[
    RowBox[{
      RowBox[{"$TextStyle", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"FontSlant", "\[Rule]", "\"\<Italic\>\""}], ",", 
            RowBox[{"FontSize", "\[Rule]", "8"}]}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{"<<", "Miscellaneous`Units`"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"imgSize", "=", 
        RowBox[{
          RowBox[{"Convert", "[", 
            RowBox[{
              RowBox[{"60", "Milli", " ", "Meter"}], ",", "PrintersPoint"}], 
            "]"}], "/", "PrintersPoint"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Some definitions", "Subsubsection"],

Cell[TextData[{
  "Disturbance ",
  Cell[BoxData[
      FormBox["s", TraditionalForm]]],
  " and postestimation deviation ",
  Cell[BoxData[
      FormBox[
        RowBox[{"vp", "\[Congruent]", 
          SuperscriptBox["v", "\[Prime]"]}], TraditionalForm]]],
  " depending on ",
  Cell[BoxData[
      FormBox["\[Alpha]", TraditionalForm]]],
  " and ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"g1p", "[", "m_", "]"}], "\[Congruent]", 
          RowBox[{
            SubsuperscriptBox["g", "1", "\[Prime]"], "(", "m", ")"}]}], 
        TraditionalForm]]]
}], "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"s", "[", "\[Alpha]_", "]"}], "=", 
        RowBox[{
          FractionBox["1", "6"], 
          RowBox[{"(", 
            RowBox[{
              RowBox[{"2", 
                SuperscriptBox["\[Alpha]", "2"]}], "-", 
              RowBox[{"2", "\[Alpha]"}], "+", "1"}], ")"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"vp", "[", 
          RowBox[{"\[Alpha]_", ",", "g1p_"}], "]"}], "=", 
        RowBox[{
          RowBox[{
            FractionBox["1", "2"], 
            RowBox[{"(", 
              RowBox[{
                SuperscriptBox[
                  RowBox[{"g1p", "[", "0", "]"}], "2"], "+", 
                SuperscriptBox[
                  RowBox[{"g1p", "[", "1", "]"}], "2"], "-", 
                RowBox[{"2", 
                  RowBox[{"g1p", "[", "1", "]"}]}], "+", "1"}], ")"}]}], "-", 
          
          RowBox[{
            RowBox[{"(", 
              RowBox[{
                RowBox[{"g1p", "[", "0", "]"}], "-", 
                RowBox[{"g1p", "[", "1", "]"}], "+", "1"}], ")"}], 
            "\[Alpha]"}], "+", 
          SuperscriptBox["\[Alpha]", "2"]}]}], ";"}]}], "Input"],

Cell["Function that creates a list of tick marks for the plots:", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"ticks", "[", 
            RowBox[{"min_", ",", "max_"}], "]"}], ":=", 
          RowBox[{"Chop", "[", 
            RowBox[{"Range", "[", 
              RowBox[{
                RowBox[{"Floor", "[", "min", "]"}], ",", 
                RowBox[{"Ceiling", "[", "max", "]"}], ",", "0.1"}], "]"}], 
            "]"}]}], ";"}], " "}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot of ",
  Cell[BoxData[
      FormBox["s", TraditionalForm]]],
  "-",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["v", "\[Prime]"], TraditionalForm]]],
  "-curves of estimation procedures with different guesses and different \
ranges of ",
  Cell[BoxData[
      FormBox["\[Alpha]", TraditionalForm]]]
}], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"g1p2a", "[", 
          RowBox[{"m_", ",", "\[Alpha]_"}], "]"}], ":=", 
        RowBox[{"\[Alpha]", " ", "m"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"g1p2b", "[", 
          RowBox[{"m_", ",", "\[Alpha]_"}], "]"}], ":=", "m"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"g1p2c", "[", 
          RowBox[{"m_", ",", "\[Alpha]_"}], "]"}], ":=", "\[Alpha]"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"pa", "=", 
        RowBox[{"ParametricPlot", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"s", "[", "\[Alpha]", "]"}], ",", 
                  RowBox[{"vp", "[", 
                    RowBox[{"\[Alpha]", ",", 
                      RowBox[{
                        RowBox[{"g1p2a", "[", 
                          RowBox[{"#", ",", "\[Alpha]"}], "]"}], "&"}]}], 
                    "]"}]}], "}"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Alpha]", ",", "0.3", ",", "0.8"}], "}"}], ",", 
            RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"pb", "=", 
        RowBox[{"ParametricPlot", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"s", "[", "\[Alpha]", "]"}], ",", 
                RowBox[{"vp", "[", 
                  RowBox[{"\[Alpha]", ",", 
                    RowBox[{
                      RowBox[{"g1p2b", "[", 
                        RowBox[{"#", ",", "\[Alpha]"}], "]"}], "&"}]}], 
                  "]"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Alpha]", ",", "0", ",", "1"}], "}"}], ",", 
            RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"{", 
                RowBox[{"Dashing", "[", 
                  RowBox[{"{", 
                    RowBox[{"0.02", ",", "0.02"}], "}"}], "]"}], "}"}]}], ",",
             
            RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"pc", "=", 
        RowBox[{"ParametricPlot", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"s", "[", "\[Alpha]", "]"}], ",", 
                RowBox[{"vp", "[", 
                  RowBox[{"\[Alpha]", ",", 
                    RowBox[{
                      RowBox[{"g1p2c", "[", 
                        RowBox[{"#", ",", "\[Alpha]"}], "]"}], "&"}]}], 
                  "]"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"\[Alpha]", ",", "0", ",", "0.6"}], "}"}], ",", 
            RowBox[{"PlotStyle", "\[Rule]", 
              RowBox[{"Thickness", "[", "0.01", "]"}]}], ",", 
            RowBox[{"DisplayFunction", "\[Rule]", "Identity"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Show", "[", 
        RowBox[{"pa", ",", "pb", ",", "pc", ",", 
          RowBox[{"DisplayFunction", "\[Rule]", "$DisplayFunction"}], ",", 
          RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"0", ",", "0.2"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", "0.5"}], "}"}]}], "}"}]}], ",", 
          RowBox[{"Ticks", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"ticks", ",", "ticks"}], "}"}]}], ",", 
          RowBox[{"AxesLabel", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"\"\<s\>\"", ",", "\"\<v'\>\""}], "}"}]}], ",", 
          RowBox[{"ImageSize", "\[Rule]", "imgSize"}]}], "]"}], 
      ";"}]}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 170.717 105.509 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 5 0 1.23607 [
[.5 -0.0125 -9.125 -8.25 ]
[.5 -0.0125 9.125 0 ]
[1 -0.0125 -9.125 -8.25 ]
[1 -0.0125 9.125 0 ]
[1.025 0 0 -4.125 ]
[1.025 0 8.75 4.125 ]
[-0.0125 .12361 -18.25 -4.125 ]
[-0.0125 .12361 0 4.125 ]
[-0.0125 .24721 -18.25 -4.125 ]
[-0.0125 .24721 0 4.125 ]
[-0.0125 .37082 -18.25 -4.125 ]
[-0.0125 .37082 0 4.125 ]
[-0.0125 .49443 -18.25 -4.125 ]
[-0.0125 .49443 0 4.125 ]
[-0.0125 .61803 -18.25 -4.125 ]
[-0.0125 .61803 0 4.125 ]
[0 .64303 -7.09375 0 ]
[0 .64303 7.09375 8.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -70.125 -12.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
1 0 m
1 .00625 L
s
gsave
1 -0.0125 -70.125 -12.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
67.750 10.000 moveto
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .12361 m
.00625 .12361 L
s
gsave
-0.0125 .12361 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
0 .24721 m
.00625 .24721 L
s
gsave
-0.0125 .24721 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
0 .37082 m
.00625 .37082 L
s
gsave
-0.0125 .37082 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.3) show
1.000 setlinewidth
grestore
0 .49443 m
.00625 .49443 L
s
gsave
-0.0125 .49443 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
0 0 m
0 .61803 L
s
gsave
0 .64303 -68.0938 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(v) show
68.438 10.000 moveto
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(') show
73.188 10.000 moveto
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.48333 .15451 m
.4705 .14324 L
.45806 .13386 L
.4526 .13039 L
.44787 .1278 L
.44336 .12579 L
.44127 .12504 L
.43942 .12448 L
.43746 .12403 L
.43647 .12386 L
.43597 .12379 L
.43544 .12372 L
.435 .12368 L
.4346 .12365 L
.43441 .12364 L
.4342 .12363 L
.43401 .12362 L
.43383 .12361 L
.4336 .12361 L
.4334 .12361 L
.43318 .12361 L
.43306 .12361 L
.43295 .12361 L
.43276 .12362 L
.43255 .12362 L
.43234 .12363 L
.43215 .12365 L
.43171 .12368 L
.43131 .12373 L
.43043 .12386 L
.42968 .12403 L
.4289 .12424 L
.42762 .12472 L
.42628 .1254 L
.42498 .12627 L
.42388 .12722 L
.42269 .12852 L
.42168 .12991 L
.42003 .13299 L
.41929 .13486 L
.4186 .13705 L
.41804 .13934 L
.41754 .14201 L
.41734 .14331 L
.41717 .14473 L
.41703 .14611 L
.41692 .1474 L
.41682 .1488 L
.41676 .15017 L
.41673 .15093 L
Mistroke
.4167 .15164 L
.41669 .15244 L
.41668 .15289 L
.41667 .1533 L
.41667 .15367 L
.41667 .15406 L
.41667 .15439 L
.41667 .15476 L
.41667 .15516 L
.41667 .15559 L
.41668 .156 L
.41668 .15638 L
.4167 .15711 L
.41671 .15792 L
.41676 .15941 L
.41684 .16113 L
.41693 .16278 L
.41721 .16667 L
.41758 .17063 L
.41808 .1751 L
.42095 .19379 L
.4252 .21454 L
.43064 .23702 L
.43791 .26367 L
.44633 .29197 L
.45683 .32491 L
.46844 .35943 L
.481 .39522 L
.49586 .43607 L
.51163 .4781 L
.52814 .52102 L
.54717 .56941 L
.56667 .61803 L
Mfstroke
[ .02 .02 ] 0 setdash
.83333 0 m
.83128 0 L
.82941 1e-05 L
.82727 2e-05 L
.82524 3e-05 L
.82165 6e-05 L
.81777 .00011 L
.81352 .00018 L
.80904 .00027 L
.80107 .00048 L
.79292 .00076 L
.78537 .00109 L
.76846 .00203 L
.75163 .0033 L
.73658 .00473 L
.70397 .00889 L
.67705 .01356 L
.64917 .01978 L
.62214 .0274 L
.59907 .03538 L
.55658 .05465 L
.5189 .07871 L
.48821 .10598 L
.47474 .12136 L
.46157 .13944 L
.44175 .17599 L
.43407 .19562 L
.42739 .21781 L
.42473 .22904 L
.42226 .24157 L
.42049 .25267 L
.41892 .26521 L
.41834 .27114 L
.4178 .27761 L
.41742 .28327 L
.4171 .28952 L
.41695 .293 L
.41685 .29626 L
.41676 .29973 L
.41673 .30167 L
.4167 .30345 L
.41668 .30511 L
.41667 .30665 L
.41667 .30834 L
.41667 .30919 L
.41667 .31013 L
.41667 .31172 L
.41669 .31342 L
.41671 .31503 L
.41673 .31651 L
Mistroke
.41679 .31981 L
.41687 .32282 L
.41711 .32961 L
.41743 .33596 L
.41779 .34199 L
.41888 .35574 L
.42047 .37089 L
.42454 .39984 L
.43043 .43156 L
.44518 .49185 L
.46673 .56036 L
Mfstroke
.46673 .56036 m
.48844 .61803 L
s
.01 w
[ ] 0 setdash
.83333 .61803 m
.79375 .55933 L
.75284 .49864 L
.71655 .44482 L
.68361 .39596 L
.65065 .34707 L
.62098 .30305 L
.59164 .25953 L
.56553 .2208 L
.54239 .18649 L
.51993 .15316 L
.50038 .12417 L
.48352 .09916 L
.46765 .07562 L
.4544 .05598 L
.4425 .03832 L
.43316 .02447 L
.42615 .01406 L
.4208 .00614 L
.41772 .00157 L
.41693 .00039 L
.41673 9e-05 L
.41668 1e-05 L
.41667 1e-05 L
.41667 0 L
.41667 0 L
.41667 0 L
.41667 0 L
.41669 4e-05 L
.41676 .00013 L
.417 .0005 L
.41782 .00171 L
.421 .00643 L
.42599 .01383 L
.43333 .02472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{170.688, 105.5},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002Z0000JA000`40n007h00OZ_oo002Zool00:[o
o`00D_oo0`000ooo0P000_oo0`00=_oo0`000ooo0P0000Coo`00000000Woo`00D_oo00<00?oo0000
2?oo00<00?ooool0=Ooo00<00?oo00001ooo00<00?ooool02?oo001Bool01000ooooo`001ooo00<0
0?ooool0=Ooo00@00?ooool000Ooo`03003ooooo00Ooo`00Dooo00<00?oo00002?oo00<00?ooool0
=Ooo00<00?oo00002?oo00<00?ooool01_oo001Cool30007ool2000gool30006ool30008ool00:[o
o`00XOoo10001Ooo000Eoon70005ool01000ooooo`001Ooo000Eool00`00ooooo`0fool00`00oooo
o`07ool00`00ooooo`0Gool400000ooo0000000Rool00`00ooooo`04ool20006ool001Goo`03003o
oooo03Koo`03003ooooo01coo`@0037oo`<000Goo`005Ooo00<00?ooool0=ooo00<00?ooool06?oo
00<00?ooool0?Ooo000Eool00`00ooooo`0hool00`00ooooo`0Eool20010ool001Goo`03003ooooo
03Woo`03003ooooo013oo`8004Coo`005Ooo00<00?ooool0>Ooo0P004?oo00<00?ooool0A?oo000E
ool00`00ooooo`0jool00`00ooooo`0:ool20019ool001Goo`03003ooooo03_oo`03003ooooo00So
o`03003ooooo04Woo`005Ooo00<00?ooool0>ooo00<00?ooool0E?oo000Eool00`00ooooo`0lool0
0`00ooooo`04ool2001=ool001Goo`03003ooooo03goo`03003ooooo00;oo`03003ooooo04goo`00
5Ooo00<00?ooool0?Ooo00<00?ooool0D_oo000Eool00`00ooooo`0nool00`00ool0001Aool001Go
o`03003ooooo03koo`8005;oo`005Ooo00<00?ooool0?_oo0P00D_oo0004ool30003ool20002ool3
0004ool00`00ooooo`0gool20006ool2001Aool000Coo`03003oo`0000Soo`03003ooooo00?oo`80
03Ooo`D00003ool00?oo00;oo`03003ooooo04ooo`001?oo00@00?ooool000Ooo`03003ooooo00?o
o`03003ooooo03Goo`8000Coo`8000Coo`03003ooooo04koo`001Ooo00<00?oo00002?oo00<00?oo
ool00_oo00<00?ooool0=Ooo00<00?ooool00ooo0P001Ooo00<00?ooool0COoo0005ool30007ool2
0004ool00`00ooooo`0eool00`00ooooo`03ool00`00ool00005ool00`00ooooo`1<ool001Goo`03
003ooooo03Goo`03003ooooo00Koo`03003ooooo00?oo`03003ooooo04_oo`005Ooo00<00?ooool0
=Ooo00D00?ooooooo`0000[oo`03003ooooo04_oo`005Ooo00<00?ooool0=Ooo00D00?ooooooo`00
00_oo`03003ooooo04[oo`005Ooo00<00?ooool0=Ooo00@00?ooool000coo`03003ooooo04[oo`00
5Ooo00<00?ooool0=_oo00<00?ooool03Ooo00<00?ooool0BOoo000Eool00`00ooooo`0fool00`00
ooooo`0>ool00`00ooooo`18ool001Goo`03003ooooo03Koo`8000ooo`03003ooooo04Soo`005Ooo
00<00?ooool0=_oo0P004?oo00<00?ooool0Aooo000Eool00`00ooooo`0fool2000@ool00`00oooo
o`17ool001Goo`03003ooooo03Koo`03003ooooo013oo`03003ooooo04Koo`005Ooo00<00?ooool0
=ooo00<00?ooool04?oo00<00?ooool0AOoo0004ool30003ool200001?oo000000001Ooo00<00?oo
ool0=_oo0P004_oo0P00A_oo0004ool00`00ool00007ool00`00ooooo`04ool2000gool2000Cool2
0015ool000Coo`04003ooooo0007ool00`00ooooo`03ool00`00ooooo`0fool00`00ool0000Cool0
0`00ooooo`13ool000Goo`03003oo`0000Soo`03003ooooo00;oo`03003ooooo03Goo`04003ooooo
000Dool00`00ooooo`12ool000Goo`<000Koo`<000Coo`03003ooooo03Soo`03003ooooo01;oo`80
04?oo`005Ooo00<00?ooool0=Ooo00@00?ooool001Goo`03003ooooo047oo`005Ooo00<00?ooool0
=Ooo00D00?ooooooo`0001Coo`8004;oo`005Ooo00<00?ooool0=Ooo00D00?ooooooo`0001Goo`03
003ooooo043oo`005Ooo00<00?ooool0=Ooo00D00?ooooooo`0001Goo`80047oo`005Ooo00<00?oo
ool0>Ooo00<00?ooool05?oo00<00?ooool0?ooo000Eool00`00ooooo`0jool00`00ooooo`0Dool0
0`00ooooo`0nool001Goo`03003ooooo03Goo`03003ooooo00;oo`03003ooooo01Coo`03003ooooo
03koo`005Ooo00<00?ooool0=Ooo00<00?ooool00_oo00<00?ooool05Ooo00<00?ooool0?Ooo000E
ool00`00ooooo`0eool00`00ooooo`02ool00`00ooooo`0Fool00`00ooooo`0lool001Goo`03003o
oooo03_oo`03003ooooo01Goo`03003ooooo03coo`005Ooo00<00?ooool0>ooo00<00?ooool05_oo
00<00?ooool0>ooo000Eool00`00ooooo`0eool00`00ooooo`04ool00`00ooooo`0Fool00`00oooo
o`0jool000Coo`<000?oo`800004ool000000005ool00`00ooooo`0eool00`00ooooo`04ool00`00
ooooo`0Gool00`00ooooo`0iool000Coo`03003oo`0000Woo`03003ooooo00;oo`8003Ooo`03003o
oooo00Coo`03003ooooo01Koo`8003[oo`001?oo00@00?ooool000Ooo`03003ooooo00?oo`03003o
oooo03Koo`03003ooooo00Coo`03003ooooo01Ooo`03003ooooo03Soo`001Ooo00<00?oo00002?oo
00<00?ooool00_oo00<00?ooool0?_oo00<00?ooool05ooo00<00?ooool0=ooo0005ool30006ool3
0004ool00`00ooooo`0fool00`00ooooo`05ool00`00ooooo`0Hool00`00ooooo`0fool001Goo`03
003ooooo03Koo`03003ooooo00Goo`03003ooooo01Soo`8003Ooo`005Ooo00<00?ooool0=_oo00<0
0?ooool01_oo00<00?ooool06?oo00<00?ooool0=Ooo000Eool00`00ooooo`0gool00`00ooooo`05
ool00`00ooooo`0Iool00`00ooooo`0dool001Goo`03003ooooo043oo`03003ooooo01Soo`03003o
oooo03Coo`005Ooo00<00?ooool0@?oo00<00?ooool06Ooo00<00?ooool0<ooo000Eool00`00oooo
o`0gool00`00ooooo`06ool00`00ooooo`0Iool2000dool001Goo`03003ooooo03Ooo`03003ooooo
00Ooo`03003ooooo01Woo`03003ooooo03;oo`005Ooo00<00?ooool0>?oo00<00?ooool01_oo00<0
0?ooool06_oo00<00?ooool0<Ooo000Eool00`00ooooo`11ool00`00ooooo`0Jool2000bool001Go
o`03003ooooo04;oo`03003ooooo01[oo`03003ooooo033oo`005Ooo00<00?ooool0>?oo00<00?oo
ool01ooo00<00?ooool06ooo00<00?ooool0;ooo0004ool30003ool20003ool20004ool00`00oooo
o`0hool00`00ooooo`07ool00`00ooooo`0Kool2000`ool000Coo`03003oo`0000Ooo`<000Coo`80
03[oo`03003ooooo00Ooo`03003ooooo01_oo`03003ooooo02koo`001?oo00@00?ooool000Koo`03
003oo`0000Coo`03003ooooo04?oo`03003ooooo01coo`03003ooooo02goo`001Ooo00<00?oo0000
1ooo0P001?oo00<00?ooool0@ooo00<00?ooool07Ooo00<00?ooool0;?oo0005ool30008ool00`00
ooooo`02ool00`00ooooo`0jool00`00ooooo`07ool00`00ooooo`0Lool2000]ool001Goo`03003o
oooo03[oo`03003ooooo00Ooo`03003ooooo01goo`03003ooooo02_oo`005Ooo00<00?ooool0>ooo
00<00?ooool01_oo00<00?ooool07_oo00<00?ooool0:_oo000Eool00`00ooooo`15ool00`00oooo
o`0Nool00`00ooooo`0Yool001Goo`03003ooooo04Goo`03003ooooo01koo`8002[oo`005Ooo00<0
0?ooool0>ooo00<00?ooool02?oo00<00?ooool07_oo00<00?ooool0:?oo000Eool00`00ooooo`0l
ool00`00ooooo`07ool00`00ooooo`0Oool00`00ooooo`0Wool001Goo`03003ooooo03coo`03003o
oooo00Soo`03003ooooo01ooo`03003ooooo02Koo`005Ooo00<00?ooool0?Ooo00<00?ooool01ooo
00<00?ooool07ooo00<00?ooool09_oo000Eool00`00ooooo`0mool00`00ooooo`08ool00`00oooo
o`0Oool00`00ooooo`0Uool001Goo`03003ooooo04Soo`03003ooooo01ooo`8002Koo`005Ooo00<0
0?ooool0?_oo00<00?ooool01ooo00<00?ooool08?oo00<00?ooool09?oo000Eool00`00ooooo`0n
ool00`00ooooo`07ool00`00ooooo`0Qool00`00ooooo`0Sool000Coo`<000?oo`800004ool00000
0005ool00`00ooooo`0oool00`00ooooo`07ool00`00ooooo`0Pool00`00ooooo`0Sool000Coo`03
003oo`0000Woo`03003ooooo00;oo`80043oo`03003ooooo00Ooo`03003ooooo027oo`03003ooooo
02;oo`001?oo00@00?ooool000Soo`03003ooooo09Ooo`001Ooo00<00?oo00001_oo0`00VOoo0005
ool30006ool3002Iool00:[oo`00Z_oo000Aool00`00ooooo`2Fool0017oo`8009Ooo`004?oo00@0
0?ooool009Koo`004?oo0P0000?oo`0000000ooo00<00?ooool0Sooo000Iool00`00ooooo`2>ool0
0:[oo`00\
\>"],
  ImageRangeCache->{{{0, 169.688}, {104.5, 0}} -> {-0.0322318, -0.0598347, \
0.00148986, 0.00602658}}]
}, Open  ]],

Cell[TextData[{
  "The solid thin curve shows a procedure with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              SubsuperscriptBox["g", "1", "\[Prime]"], "(", "m", ")"}], "=", 
            RowBox[{"\[Alpha]", " ", "m"}]}], ",", 
          RowBox[{"\[Alpha]", "\[Element]", 
            RowBox[{"[", 
              RowBox[{"0.3", ",", "0.8"}], "]"}]}]}], TraditionalForm]]],
  ".\nThe dashed curve shows a procedure with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              SubsuperscriptBox["g", "1", "\[Prime]"], "(", "m", ")"}], "=", 
            "m"}], ",", 
          RowBox[{"\[Alpha]", "\[Element]", 
            RowBox[{"[", 
              RowBox[{"0", ",", "1"}], "]"}]}]}], TraditionalForm]]],
  ".\nThe thick curve shows a procedure with ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{
            RowBox[{
              SubsuperscriptBox["g", "1", "\[Prime]"], "(", "m", ")"}], "=", 
            "\[Alpha]"}], ",", 
          RowBox[{"\[Alpha]", "\[Element]", 
            RowBox[{"[", 
              RowBox[{"0", ",", "0.6"}], "]"}]}]}], TraditionalForm]]],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Plot of ",
  Cell[BoxData[
      FormBox["s", TraditionalForm]]],
  "-",
  Cell[BoxData[
      FormBox[
        SuperscriptBox["v", "\[Prime]"], TraditionalForm]]],
  "-curve for guess with deviation ",
  Cell[BoxData[
      FormBox["0", TraditionalForm]]]
}], "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"g1p", "[", 
          RowBox[{"m_", ",", "\[Alpha]_"}], "]"}], ":=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"m", "\[Equal]", "0"}], ",", "\[Alpha]", ",", 
            RowBox[{"1", "-", "\[Alpha]"}]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"ParametricPlot", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"s", "[", "\[Alpha]", "]"}], ",", 
              RowBox[{"vp", "[", 
                RowBox[{"\[Alpha]", ",", 
                  RowBox[{
                    RowBox[{"g1p", "[", 
                      RowBox[{"#", ",", "\[Alpha]"}], "]"}], "&"}]}], "]"}]}],
             "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"\[Alpha]", ",", "0", ",", "1"}], "}"}], ",", 
          RowBox[{"PlotStyle", "\[Rule]", 
            RowBox[{"Thickness", "[", "0.02", "]"}]}], ",", 
          RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"0", ",", "0.2"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", "0.5"}], "}"}]}], "}"}]}], ",", 
          RowBox[{"Ticks", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"ticks", ",", "ticks"}], "}"}]}], ",", 
          RowBox[{"AxesLabel", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{"\"\<s\>\"", ",", "\"\<v'\>\""}], "}"}]}], ",", 
          RowBox[{"ImageSize", "\[Rule]", "imgSize"}]}], "]"}], 
      ";"}]}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
%%ImageSize: 170.717 105.509 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.77556e-17 5 0 1.23607 [
[.5 -0.0125 -9.125 -8.25 ]
[.5 -0.0125 9.125 0 ]
[1 -0.0125 -9.125 -8.25 ]
[1 -0.0125 9.125 0 ]
[1.025 0 0 -4.125 ]
[1.025 0 8.75 4.125 ]
[-0.0125 .12361 -18.25 -4.125 ]
[-0.0125 .12361 0 4.125 ]
[-0.0125 .24721 -18.25 -4.125 ]
[-0.0125 .24721 0 4.125 ]
[-0.0125 .37082 -18.25 -4.125 ]
[-0.0125 .37082 0 4.125 ]
[-0.0125 .49443 -18.25 -4.125 ]
[-0.0125 .49443 0 4.125 ]
[-0.0125 .61803 -18.25 -4.125 ]
[-0.0125 .61803 0 4.125 ]
[0 .64303 -7.09375 0 ]
[0 .64303 7.09375 8.25 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -70.125 -12.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
1 0 m
1 .00625 L
s
gsave
1 -0.0125 -70.125 -12.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(s) show
67.750 10.000 moveto
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .12361 m
.00625 .12361 L
s
gsave
-0.0125 .12361 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
0 .24721 m
.00625 .24721 L
s
gsave
-0.0125 .24721 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
0 .37082 m
.00625 .37082 L
s
gsave
-0.0125 .37082 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.3) show
1.000 setlinewidth
grestore
0 .49443 m
.00625 .49443 L
s
gsave
-0.0125 .49443 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
0 .61803 m
.00625 .61803 L
s
gsave
-0.0125 .61803 -79.25 -8.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
0 0 m
0 .61803 L
s
gsave
0 .64303 -68.0938 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 16.25 translate 1 -1 scale
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique
%%IncludeFont: Courier-Oblique
%%BeginResource: font Courier-Oblique-MISO
%%BeginFont: Courier-Oblique-MISO
/Courier-Oblique /Courier-Oblique-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 10.000 moveto
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(v) show
68.438 10.000 moveto
%%IncludeResource: font Courier-Oblique-MISO
%%IncludeFont: Courier-Oblique-MISO
/Courier-Oblique-MISO findfont 8.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(') show
73.188 10.000 moveto
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.02 w
.83333 0 m
.83128 0 L
s
.83128 0 m
.83005 0 L
s
.82619 0 m
.82524 0 L
s
.82524 0 m
.82339 0 L
s
.80852 0 m
.80764 0 L
s
.80764 0 m
.80663 0 L
s
.80093 0 m
.79897 0 L
.79676 0 L
s
.79676 0 m
.79624 0 L
s
.79215 0 m
.79212 0 L
s
.79212 0 m
.79208 0 L
s
.78882 0 m
.78809 0 L
s
.78809 0 m
.78752 0 L
s
.78633 0 m
.78588 0 L
s
.78588 0 m
.78511 0 L
s
.78303 0 m
.78194 0 L
s
.78194 0 m
.7811 0 L
s
.77487 0 m
.77394 0 L
.77225 0 L
.77041 0 L
s
.77041 0 m
.76999 0 L
s
.76803 0 m
.76742 0 L
s
.76742 0 m
.76644 0 L
s
.76343 0 m
.76224 0 L
.76041 0 L
.75843 0 L
.75731 0 L
s
.75731 0 m
.7568 0 L
s
.75397 0 m
.753 0 L
.7519 0 L
s
.7519 0 m
.75114 0 L
s
.74881 0 m
.74813 0 L
.74631 0 L
s
.74631 0 m
.74545 0 L
s
.74231 0 m
.74162 0 L
s
.74162 0 m
.74144 0 L
s
.73841 0 m
.73685 0 L
.73577 0 L
.73475 0 L
s
.73475 0 m
.73323 0 L
s
.7313 0 m
.73079 0 L
.72971 0 L
.72857 0 L
.72669 0 L
.72493 0 L
.72301 0 L
s
.72301 0 m
.72199 0 L
s
.72189 0 m
.72094 0 L
.71994 0 L
.71887 0 L
s
.71887 0 m
.71782 0 L
s
.71555 0 m
.71467 0 L
s
.71467 0 m
.71425 0 L
s
.71312 0 m
.71261 0 L
s
.71261 0 m
.71219 0 L
s
.70634 0 m
.7061 0 L
s
.7061 0 m
.70554 0 L
s
.70464 0 m
.70397 0 L
.70232 0 L
.70055 0 L
s
.70055 0 m
.69998 0 L
s
.69668 0 m
.69567 0 L
.69383 0 L
s
.69383 0 m
.69363 0 L
s
.69242 0 m
.69192 0 L
.69014 0 L
s
.69014 0 m
.68932 0 L
s
.68875 0 m
.68827 0 L
s
.68827 0 m
.68756 0 L
s
.68711 0 m
.68624 0 L
s
.68624 0 m
.68559 0 L
s
.68304 0 m
.68258 0 L
.68087 0 L
s
.68087 0 m
.67966 0 L
s
.67576 0 m
.67435 0 L
s
.67435 0 m
.67307 0 L
s
.67252 0 m
.67131 0 L
.66995 0 L
s
.66995 0 m
.6696 0 L
s
.66141 0 m
.66118 0 L
.66033 0 L
s
.66033 0 m
.6596 0 L
s
.6563 0 m
.6556 0 L
s
.6556 0 m
.65486 0 L
s
.65012 0 m
.64952 0 L
.6479 0 L
.6462 0 L
s
.6462 0 m
.64559 0 L
s
.64348 0 m
.64317 0 L
s
.64317 0 m
.6428 0 L
s
.64089 0 m
.64031 0 L
.63887 0 L
.6373 0 L
s
.6373 0 m
.63659 0 L
s
.63307 0 m
.63271 0 L
.63106 0 L
.63014 0 L
.6293 0 L
.62768 0 L
s
.62768 0 m
.62702 0 L
s
.62554 0 m
.62471 0 L
.62342 0 L
.62193 0 L
.62054 0 L
.61925 0 L
s
.61925 0 m
.619 0 L
s
.61405 0 m
.61259 0 L
.61133 0 L
.60996 0 L
.60913 0 L
.60837 0 L
.60686 0 L
.6055 0 L
s
.6055 0 m
.60534 0 L
s
.60161 0 m
.60112 0 L
s
.60112 0 m
.60059 0 L
s
.59909 0 m
.59835 0 L
s
.59835 0 m
.59749 0 L
s
.5965 0 m
.59584 0 L
s
.59584 0 m
.59493 0 L
s
.5931 0 m
.5918 0 L
.59051 0 L
s
.59051 0 m
.58984 0 L
s
.589 0 m
.5877 0 L
.58622 0 L
s
.58622 0 m
.58503 0 L
s
.58015 0 m
.57927 0 L
.57794 0 L
s
.57794 0 m
.57766 0 L
s
.57692 0 m
.57648 0 L
.57522 0 L
.57406 0 L
.5728 0 L
.57147 0 L
s
.57147 0 m
.57103 0 L
s
.56992 0 m
.56918 0 L
.56807 0 L
s
.56807 0 m
.56773 0 L
s
.56524 0 m
.56461 0 L
s
.56461 0 m
.56358 0 L
s
.56286 0 m
.5623 0 L
s
.5623 0 m
.56214 0 L
s
.56016 0 m
.55979 0 L
s
.55979 0 m
.55943 0 L
s
.55799 0 m
.55751 0 L
s
.55751 0 m
.55705 0 L
s
.55588 0 m
.55552 0 L
s
.55552 0 m
.55517 0 L
s
.55339 0 m
.55245 0 L
s
.55245 0 m
.55196 0 L
s
.54715 0 m
.54698 0 L
.54575 0 L
.5446 0 L
s
.5446 0 m
.54413 0 L
s
.54286 0 m
.54231 0 L
.54169 0 L
.54101 0 L
.53981 0 L
s
.53981 0 m
.53897 0 L
s
.53851 0 m
.53805 0 L
.53738 0 L
s
.53738 0 m
.53666 0 L
s
.5337 0 m
.53352 0 L
.5323 0 L
.53117 0 L
s
.53117 0 m
.53038 0 L
s
.52644 0 m
.52619 0 L
s
.52619 0 m
.52596 0 L
s
.5224 0 m
.52214 0 L
.52099 0 L
.52035 0 L
.51976 0 L
s
.51976 0 m
.51911 0 L
s
.51526 0 m
.51497 0 L
.51434 0 L
.51374 0 L
s
.51374 0 m
.51337 0 L
s
.51155 0 m
.51106 0 L
s
.51106 0 m
.51074 0 L
s
.5077 0 m
.50753 0 L
.50692 0 L
s
.50692 0 m
.50657 0 L
s
.50614 0 m
.50584 0 L
s
.50584 0 m
.50556 0 L
s
.50371 0 m
.50311 0 L
s
.50311 0 m
.50294 0 L
s
.50204 0 m
.50103 0 L
s
.50103 0 m
.50065 0 L
s
.49538 0 m
0 0 rlineto s
.49222 0 m
0 0 rlineto s
.4871 0 m
.48671 0 L
.48622 0 L
.48532 0 L
.4844 0 L
.48387 0 L
s
.48166 0 m
.48158 0 L
.48076 0 L
s
.48076 0 m
.48027 0 L
s
.47958 0 m
.47904 0 L
.47824 0 L
s
.47824 0 m
.47786 0 L
s
.4763 0 m
.47607 0 L
.4753 0 L
.47457 0 L
.47378 0 L
s
.47177 0 m
0 0 rlineto s
.47063 0 m
.4704 0 L
.46962 0 L
.46886 0 L
s
.46622 0 m
.46595 0 L
s
.46595 0 m
.46552 0 L
s
.46441 0 m
.46392 0 L
.46316 0 L
s
.46316 0 m
.46273 0 L
s
.4618 0 m
.4612 0 L
.46055 0 L
s
.46055 0 m
.46041 0 L
s
.45858 0 m
.45846 0 L
.45778 0 L
s
.45778 0 m
.45753 0 L
s
.45482 0 m
.45463 0 L
s
.45463 0 m
.45449 0 L
s
.45343 0 m
.45299 0 L
.45242 0 L
.45179 0 L
s
.45179 0 m
.45156 0 L
s
.45133 0 m
.45112 0 L
s
.45112 0 m
.45094 0 L
s
.44962 0 m
.44945 0 L
.44914 0 L
s
.4484 0 m
.44817 0 L
s
.44817 0 m
.44803 0 L
s
.44728 0 m
.4471 0 L
s
.4471 0 m
.44695 0 L
s
.44645 0 m
0 0 rlineto s
.44285 0 m
.4427 0 L
.44239 0 L
.44185 0 L
.44135 0 L
.44081 0 L
.44036 0 L
.43986 0 L
.43788 0 L
.43762 0 L
s
.4372 0 m
.43706 0 L
.43679 0 L
s
.43679 0 m
.43663 0 L
s
.43458 0 m
.43447 0 L
.43408 0 L
s
.43408 0 m
.43389 0 L
s
.43293 0 m
.43259 0 L
.43221 0 L
s
.43125 0 m
.43105 0 L
.43065 0 L
s
.42994 0 m
.42984 0 L
.4295 0 L
s
.42904 0 m
.42879 0 L
s
.42879 0 m
.42857 0 L
s
.42816 0 m
.42804 0 L
.42767 0 L
s
.42767 0 m
.42758 0 L
s
.42738 0 m
.42728 0 L
.42692 0 L
s
.4264 0 m
.4263 0 L
s
.4263 0 m
.4262 0 L
s
.4258 0 m
.4257 0 L
.42544 0 L
s
.42544 0 m
.42529 0 L
s
.42425 0 m
.42395 0 L
s
.42395 0 m
.42383 0 L
s
.4237 0 m
.42362 0 L
.42334 0 L
.42306 0 L
s
.42257 0 m
.42232 0 L
.42206 0 L
.42193 0 L
s
.42193 0 m
.42187 0 L
s
.42167 0 m
.42164 0 L
s
.42164 0 m
.42159 0 L
s
.42108 0 m
.421 0 L
s
.421 0 m
.42085 0 L
s
.42035 0 m
.42024 0 L
.42013 0 L
s
.42013 0 m
.42005 0 L
s
.41978 0 m
.41973 0 L
s
.41973 0 m
.41965 0 L
s
.41938 0 m
.41922 0 L
.41906 0 L
s
.41906 0 m
.41896 0 L
s
.41868 0 m
.41862 0 L
.41847 0 L
.41833 0 L
s
.41822 0 m
.41817 0 L
.41805 0 L
.41798 0 L
.41792 0 L
.41781 0 L
s
.41743 0 m
0 0 rlineto s
.41732 0 m
.41728 0 L
s
.41728 0 m
.41724 0 L
s
.41699 0 m
.41695 0 L
.4169 0 L
.41686 0 L
.41682 0 L
.41678 0 L
s
.41674 0 m
.41672 0 L
.4167 0 L
.41669 0 L
s
.41669 0 m
.41668 0 L
s
.41667 0 m
0 0 rlineto s
.41667 0 m
.41668 0 L
s
.41668 0 m
.41668 0 L
s
.4167 0 m
.4167 0 L
.41672 0 L
.41674 0 L
.41677 0 L
.41681 0 L
s
.41681 0 m
.41683 0 L
s
.41707 0 m
.41709 0 L
.41716 0 L
s
.41716 0 m
.41722 0 L
s
.41748 0 m
0 0 rlineto s
.41768 0 m
0 0 rlineto s
.41785 0 m
.4179 0 L
s
.4179 0 m
.41795 0 L
s
.42637 0 m
.42661 0 L
s
.42661 0 m
.42675 0 L
s
.42689 0 m
.42698 0 L
.4273 0 L
.42765 0 L
.42785 0 L
s
.42785 0 m
.42794 0 L
s
.42846 0 m
0 0 rlineto s
.42876 0 m
.42886 0 L
.42922 0 L
.42958 0 L
s
.42958 0 m
.42977 0 L
s
.43014 0 m
.43032 0 L
s
.43032 0 m
.43049 0 L
s
.43074 0 m
.43106 0 L
.43129 0 L
.4315 0 L
.4319 0 L
s
.4319 0 m
.43211 0 L
s
.43252 0 m
.43273 0 L
.43317 0 L
.43356 0 L
.43399 0 L
s
.43399 0 m
.43412 0 L
s
.43449 0 m
.43473 0 L
s
.43473 0 m
.4349 0 L
s
.43591 0 m
0 0 rlineto s
.43624 0 m
.43645 0 L
s
.43645 0 m
.4367 0 L
s
.43724 0 m
.4375 0 L
.43779 0 L
.43811 0 L
s
.43811 0 m
.43825 0 L
s
.43853 0 m
.43868 0 L
.43897 0 L
.43923 0 L
.43974 0 L
.44028 0 L
s
.44134 0 m
.44187 0 L
s
.44298 0 m
.44333 0 L
.44386 0 L
.44436 0 L
.44492 0 L
s
.44492 0 m
.44537 0 L
s
.44577 0 m
.44603 0 L
.4466 0 L
.44722 0 L
.44757 0 L
s
.44757 0 m
.44763 0 L
s
.44929 0 m
.44963 0 L
.4502 0 L
s
.45138 0 m
.45191 0 L
s
.45191 0 m
.45216 0 L
s
.45254 0 m
.45295 0 L
.45354 0 L
.45413 0 L
.45477 0 L
s
.45557 0 m
.45594 0 L
.45661 0 L
s
.45716 0 m
.45733 0 L
.45798 0 L
.45867 0 L
.45934 0 L
.46008 0 L
s
.46008 0 m
.46061 0 L
s
.46211 0 m
.46271 0 L
.46341 0 L
.46407 0 L
.4648 0 L
.46558 0 L
.46625 0 L
s
.46625 0 m
.46643 0 L
s
.46806 0 m
.46824 0 L
s
.46824 0 m
.46845 0 L
s
.47057 0 m
.4709 0 L
.47163 0 L
s
.47314 0 m
.47382 0 L
s
.47382 0 m
.47414 0 L
s
.47515 0 m
.47588 0 L
s
.47588 0 m
.47614 0 L
s
.47803 0 m
.47822 0 L
s
.47822 0 m
.47858 0 L
s
.48 0 m
.48024 0 L
s
.48024 0 m
.48032 0 L
s
.48213 0 m
.48234 0 L
s
.48234 0 m
.48277 0 L
s
.48342 0 m
.48412 0 L
.485 0 L
s
.485 0 m
.48516 0 L
s
.48656 0 m
.48739 0 L
.48826 0 L
.48908 0 L
.49667 0 L
.49749 0 L
.49838 0 L
.49922 0 L
s
.50086 0 m
.50169 0 L
.50259 0 L
.50357 0 L
s
.50664 0 m
0 0 rlineto s
.50814 0 m
.50851 0 L
.50938 0 L
.51038 0 L
s
.51237 0 m
.51296 0 L
.51351 0 L
s
.51351 0 m
.51401 0 L
s
.5195 0 m
.52065 0 L
.5213 0 L
s
.52358 0 m
0 0 rlineto s
.52474 0 m
.52528 0 L
.52645 0 L
.52756 0 L
.52858 0 L
.53046 0 L
.53249 0 L
.53367 0 L
.53474 0 L
.53582 0 L
.53685 0 L
s
.53685 0 m
.53721 0 L
s
.53907 0 m
0 0 rlineto s
.54231 0 m
.54357 0 L
.54477 0 L
.54587 0 L
.54714 0 L
.54833 0 L
.55058 0 L
.55176 0 L
.55302 0 L
s
.5605 0 m
0 0 rlineto s
.56269 0 m
.56385 0 L
.56508 0 L
s
.56765 0 m
0 0 rlineto s
.57045 0 m
.57172 0 L
.57309 0 L
.57438 0 L
.57556 0 L
s
.57669 0 m
.57708 0 L
s
.57708 0 m
.57801 0 L
s
.57963 0 m
.58 0 L
.5808 0 L
.58167 0 L
s
.58167 0 m
.58192 0 L
s
.58323 0 m
.584 0 L
s
.584 0 m
.58424 0 L
s
.58891 0 m
.5902 0 L
s
.5902 0 m
.59088 0 L
s
.5923 0 m
.59304 0 L
s
.59304 0 m
.5939 0 L
s
.59688 0 m
0 0 rlineto s
.59865 0 m
.59959 0 L
s
.59959 0 m
.60034 0 L
s
.60239 0 m
0 0 rlineto s
.60458 0 m
.60535 0 L
.60621 0 L
.60701 0 L
.60855 0 L
.61002 0 L
.6151 0 L
.61645 0 L
s
.61645 0 m
.61718 0 L
s
.61884 0 m
.6193 0 L
.62058 0 L
s
.62291 0 m
.62377 0 L
.62527 0 L
.62668 0 L
.62812 0 L
.62944 0 L
s
.62944 0 m
.6302 0 L
s
.64361 0 m
.64671 0 L
s
.6496 0 m
0 0 rlineto s
.65518 0 m
.65599 0 L
s
.65599 0 m
.65628 0 L
s
.66125 0 m
.6622 0 L
.66323 0 L
s
.66502 0 m
0 0 rlineto s
.66729 0 m
.66839 0 L
s
.66839 0 m
.66902 0 L
s
.67127 0 m
.67171 0 L
.67331 0 L
s
.67518 0 m
0 0 rlineto s
.67839 0 m
.68002 0 L
.68178 0 L
s
.68178 0 m
.68287 0 L
s
.68515 0 m
.68613 0 L
s
.6887 0 m
.69047 0 L
.69206 0 L
.69355 0 L
.6952 0 L
s
.69791 0 m
.69885 0 L
s
.69885 0 m
.69957 0 L
s
.70092 0 m
.70274 0 L
s
.70274 0 m
.70308 0 L
s
.70469 0 m
.70645 0 L
.70837 0 L
.71004 0 L
.71185 0 L
s
.71469 0 m
.7156 0 L
.71671 0 L
.71776 0 L
.71871 0 L
.71974 0 L
.72074 0 L
s
.72074 0 m
.7213 0 L
s
.72302 0 m
.72411 0 L
.72607 0 L
s
.72768 0 m
.72819 0 L
s
.72819 0 m
.72911 0 L
s
.7306 0 m
.73174 0 L
.73363 0 L
.73469 0 L
.73565 0 L
s
.73565 0 m
.7363 0 L
s
.73871 0 m
0 0 rlineto s
.74114 0 m
.74161 0 L
.74361 0 L
s
.74361 0 m
.74455 0 L
s
.74652 0 m
.74754 0 L
.74949 0 L
s
.75219 0 m
.75309 0 L
.75473 0 L
s
.7584 0 m
.76036 0 L
.76243 0 L
.76421 0 L
.76616 0 L
.76827 0 L
.76946 0 L
.77054 0 L
s
.77054 0 m
.77159 0 L
s
.77398 0 m
.77465 0 L
s
.77465 0 m
.77522 0 L
s
.77825 0 m
0 0 rlineto s
.78208 0 m
.78382 0 L
s
.78382 0 m
.78462 0 L
s
.78902 0 m
.79092 0 L
.79294 0 L
.79486 0 L
s
.79486 0 m
.79544 0 L
s
.79866 0 m
.79982 0 L
.80089 0 L
s
.80497 0 m
.80683 0 L
s
.8102 0 m
.81194 0 L
.8156 0 L
.8195 0 L
.82113 0 L
.82289 0 L
.82483 0 L
.82665 0 L
.82834 0 L
.83013 0 L
.83166 0 L
s
.83166 0 m
.8325 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{170.688, 105.5},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002Z0000JA000`40n007h00OZ_oo002Zool00:[o
o`00D_oo0`000ooo0P000_oo0`00=_oo0`000ooo0P0000Coo`00000000Woo`00D_oo00<00?oo0000
2?oo00<00?ooool0=Ooo00<00?oo00001ooo00<00?ooool02?oo001Bool01000ooooo`001ooo00<0
0?ooool0=Ooo00@00?ooool000Ooo`03003ooooo00Ooo`00Dooo00<00?oo00002?oo00<00?ooool0
=Ooo00<00?oo00002?oo00<00?ooool01_oo001Cool30007ool2000gool30006ool30008ool00:[o
o`00XOoo10001Ooo000Eoon70005ool01000ooooo`001Ooo000Eool00`00ooooo`0doolj000Eool0
0`00ooooo`04ool20006ool001Goo`03003ooooo08[oo`<000Goo`005Ooo00<00?ooool0T_oo000E
ool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00oooo
o`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00ooooo`2Bool001Go
o`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00ooooo`2Bool001Goo`03003ooooo
09;oo`005Ooo00<00?ooool0T_oo0004ool30003ool20002ool30004ool00`00ooooo`2Bool000Co
o`03003oo`0000Soo`03003ooooo00?oo`8009?oo`001?oo00@00?ooool000Ooo`03003ooooo00?o
o`03003ooooo09;oo`001Ooo00<00?oo00002?oo00<00?ooool00_oo00<00?ooool0T_oo0005ool3
0007ool20004ool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000E
ool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00oooo
o`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00ooooo`2Bool001Go
o`03003ooooo09;oo`005Ooo00<00?ooool0T_oo0004ool30003ool200001?oo000000001Ooo00<0
0?ooool0T_oo0004ool00`00ool00007ool00`00ooooo`04ool2002Cool000Coo`04003ooooo0007
ool00`00ooooo`03ool00`00ooooo`2Bool000Goo`03003oo`0000Soo`03003ooooo00;oo`03003o
oooo09;oo`001Ooo0`001_oo0`001?oo00<00?ooool0T_oo000Eool00`00ooooo`2Bool001Goo`03
003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00ooooo`2Bool001Goo`03003ooooo09;o
o`005Ooo00<00?ooool0T_oo000Eool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<0
0?ooool0T_oo000Eool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo
0004ool30003ool200001?oo000000001Ooo00<00?ooool0T_oo0004ool00`00ool00009ool00`00
ooooo`02ool2002Cool000Coo`04003ooooo0007ool00`00ooooo`03ool00`00ooooo`2Bool000Go
o`03003oo`0000Soo`03003ooooo00;oo`03003ooooo09;oo`001Ooo0`001_oo0`001?oo00<00?oo
ool0T_oo000Eool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000E
ool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00oooo
o`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00ooooo`2Bool001Go
o`03003ooooo09;oo`001?oo0`000ooo0P000ooo0P001?oo00<00?ooool0T_oo0004ool00`00ool0
0007ool30004ool2002Cool000Coo`04003ooooo0006ool00`00ool00004ool00`00ooooo`2Bool0
00Goo`03003oo`0000Ooo`8000Coo`03003ooooo09;oo`001Ooo0`002?oo00<00?ooool00_oo00<0
0?ooool0T_oo000Eool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo
000Eool00`00ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00
ooooo`2Bool001Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo000Eool00`00ooooo`2Bool0
01Goo`03003ooooo09;oo`005Ooo00<00?ooool0T_oo0004ool30003ool200001?oo000000001Ooo
00<00?ooool0T_oo0004ool00`00ool00009ool00`00ooooo`02ool2002Cool000Coo`04003ooooo
0008ool00`00ooooo`2Gool000Goo`03003oo`0000Koo`<009Woo`001Ooo0`001_oo0`00VOoo002Z
ool00:[oo`004Ooo00<00?ooool0U_oo000Aool2002Gool0013oo`04003ooooo002Fool0013oo`80
0003ool0000000?oo`03003ooooo08ooo`006Ooo00<00?ooool0S_oo002Zool00001\
\>"],
  ImageRangeCache->{{{0, 169.688}, {104.5, 0}} -> {-0.0322318, -0.0598347, \
0.00148986, 0.00602658}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PrintingMargins"->{{84.9375, 54}, {72, 72}},
"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "felix", \
"tmp"}, "mathematica.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 249, 6, 27, "Input"],
Cell[1969, 57, 67, 1, 27, "Input"],
Cell[2039, 60, 270, 7, 27, "Input"],

Cell[CellGroupData[{
Cell[2334, 71, 41, 0, 42, "Subsubsection"],
Cell[2378, 73, 591, 20, 32, "Text"],
Cell[2972, 95, 1208, 34, 80, "Input"],
Cell[4183, 131, 73, 0, 32, "Text"],
Cell[4259, 133, 417, 11, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4713, 149, 353, 12, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[5091, 165, 3782, 94, 139, "Input"],
Cell[8876, 261, 19293, 712, 114, 12945, 630, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[28184, 976, 1216, 35, 68, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29437, 1016, 296, 11, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[29758, 1031, 1585, 40, 59, "Input"],
Cell[31346, 1073, 24044, 1708, 114, 20633, 1663, "GraphicsData", \
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

