(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    106904,       2788]*)
(*NotebookOutlinePosition[    107745,       2819]*)
(*  CellTagsIndexPosition[    107655,       2813]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Some helper functions", "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
      RowBox[{"NumberForm", "[", 
        RowBox[{
          RowBox[{"MatrixForm", "[", 
            RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}], " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"whereAbsMax", "[", "list_", "]"}], ":=", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"fList", "=", 
                RowBox[{"Flatten", "[", "list", "]"}]}], "}"}], ",", 
            RowBox[{"If", "[", "\[IndentingNewLine]", 
              RowBox[{
                RowBox[{
                  RowBox[{"Length", "[", "fList", "]"}], "\[Equal]", "0"}], 
                ",", "\[IndentingNewLine]", 
                RowBox[{"{", 
                  RowBox[{"0", ",", "0"}], "}"}], ",", "\[IndentingNewLine]", 
                
                RowBox[{"Block", "[", 
                  RowBox[{
                    RowBox[{"{", 
                      RowBox[{"whereAbsMaxRest", "=", 
                        RowBox[{"whereAbsMax", "[", 
                          RowBox[{"Rest", "[", "fList", "]"}], "]"}]}], "}"}],
                     ",", 
                    RowBox[{"If", "[", 
                      RowBox[{
                        RowBox[{
                          RowBox[{"Abs", "[", 
                            RowBox[{"First", "[", "fList", "]"}], "]"}], ">", 
                          
                          RowBox[{
                          "whereAbsMaxRest", "\[LeftDoubleBracket]", "1", 
                            "\[RightDoubleBracket]"}]}], ",", 
                        RowBox[{"{", 
                          RowBox[{
                            RowBox[{"First", "[", "fList", "]"}], ",", "1"}], 
                          "}"}], ",", 
                        RowBox[{"whereAbsMaxRest", "+", 
                          RowBox[{"{", 
                            RowBox[{"0", ",", "1"}], "}"}]}]}], "]"}]}], 
                  "]"}]}], "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"whereAbsMax", "[", 
      RowBox[{"{", "}"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"whereAbsMax", "[", 
      RowBox[{"{", "1", "}"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"whereAbsMax", "[", 
      RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], "]"}], "\[IndentingNewLine]", 
    RowBox[{"whereAbsMax", "[", 
      RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "0.5"}], "}"}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"whereAbsMax", "[", 
      RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", 
          RowBox[{"{", 
            RowBox[{"0.5", ",", "3.8"}], "}"}], ",", "0.7"}], "}"}], 
      "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"2", ",", "2"}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"2", ",", "2"}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"3.8`", ",", "4"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          RowBox[{"E", "(", 
            RowBox[{"m", ",", "\[Phi]"}], ")"}], 
          RowBox[{"j", " ", "k"}]], TraditionalForm]]],
  " (called ",
  Cell[BoxData[
      FormBox["ee", TraditionalForm]]],
  " here) according to (1-3)"
}], "Subsection"],

Cell[TextData[{
  "We use the following ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " compatible names for variables: ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"mmm", "=", 
            RowBox[{"M", "(", 
              RowBox[{
                OverscriptBox["m", "\[RightVector]"], ",", "\[CurlyPhi]"}], 
              ")"}]}], ",", " ", 
          RowBox[{"nn", "=", "N"}]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"mmm", "[", 
        RowBox[{"nn_", ",", "j_", ",", "k_"}], "]"}], ":=", 
      RowBox[{
        SubsuperscriptBox["\[Sum]", 
          RowBox[{"n", "=", "0"}], 
          RowBox[{"(", 
            RowBox[{"nn", "-", "1"}], ")"}]], 
        RowBox[{
          RowBox[{"a", "[", 
            RowBox[{"nn", ",", "j", ",", "k", ",", "n"}], "]"}], 
          SuperscriptBox["x", "n"], 
          SuperscriptBox["y", 
            RowBox[{
              RowBox[{"(", 
                RowBox[{"nn", "-", "1"}], ")"}], "-", "n"}]]}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"a", "[", 
          RowBox[{"nn_", ",", "j_", ",", "k_", ",", "n_"}], "]"}], ":=", 
        "\[IndentingNewLine]", "\t", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"nn", "\[LessEqual]", "1"}], ",", "\[IndentingNewLine]", 
            "\t", 
            RowBox[{
              RowBox[{"mm", "\[LeftDoubleBracket]", 
                RowBox[{
                  RowBox[{
                  "m", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
                   "+", "1"}], "\[RightDoubleBracket]"}], 
              "\[LeftDoubleBracket]", 
              RowBox[{
                RowBox[{"j", "+", "1"}], ",", 
                RowBox[{"k", "+", "1"}]}], "\[RightDoubleBracket]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
              SubsuperscriptBox["\[Sum]", 
                RowBox[{"p", "=", "0"}], "1"], 
              RowBox[{
                RowBox[{
                  RowBox[{"mm", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{
                      "m", "\[LeftDoubleBracket]", "nn", 
                        "\[RightDoubleBracket]"}], "+", "1"}], 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"j", "+", "1"}], ",", 
                    RowBox[{"p", "+", "1"}]}], "\[RightDoubleBracket]"}], 
                RowBox[{"(", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"a", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", "p", ",", "k", ",", 
                          RowBox[{"n", "-", "1"}]}], "]"}], 
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{"n", "\[GreaterEqual]", "1"}], ",", "1", 
                          ",", "0"}], "]"}]}], "+", 
                    RowBox[{"I", " ", 
                      RowBox[{"a", "[", 
                        RowBox[{
                          RowBox[{"nn", "-", "1"}], ",", 
                          RowBox[{"1", "-", "p"}], ",", "k", ",", "n"}], 
                        "]"}], 
                      RowBox[{"If", "[", 
                        RowBox[{
                          RowBox[{"n", "\[LessEqual]", 
                            RowBox[{"nn", "-", "2"}]}], ",", "1", ",", "0"}], 
                        "]"}]}]}], ")"}]}]}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"makea", "[", 
          RowBox[{"nn_", ",", "mm_", ",", "m_"}], "]"}], ":=", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"nn", "\[LessEqual]", "1"}], ",", 
            RowBox[{"Table", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"mm", "\[LeftDoubleBracket]", 
                    RowBox[{
                      RowBox[{
                      "m", "\[LeftDoubleBracket]", "1", 
                        "\[RightDoubleBracket]"}], "+", "1"}], 
                    "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]", 
                  RowBox[{
                    RowBox[{"j", "+", "1"}], ",", 
                    RowBox[{"k", "+", "1"}]}], "\[RightDoubleBracket]"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{"j", ",", "0", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"k", ",", "0", ",", "1"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"n", ",", "0", ",", "0"}], "}"}]}], "]"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Block", "[", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"aold", "=", 
                    RowBox[{"makea", "[", 
                      RowBox[{
                        RowBox[{"nn", "-", "1"}], ",", "mm", ",", "m"}], 
                      "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
                RowBox[{"Table", "[", "\[IndentingNewLine]", 
                  RowBox[{
                    RowBox[{
                      SubsuperscriptBox["\[Sum]", 
                        RowBox[{"p", "=", "0"}], "1"], 
                      RowBox[{
                        RowBox[{
                          RowBox[{"mm", "\[LeftDoubleBracket]", 
                            RowBox[{
                              RowBox[{
                              "m", "\[LeftDoubleBracket]", "nn", 
                                "\[RightDoubleBracket]"}], "+", "1"}], 
                            "\[RightDoubleBracket]"}], "\[LeftDoubleBracket]",
                           
                          RowBox[{
                            RowBox[{"j", "+", "1"}], ",", 
                            RowBox[{"p", "+", "1"}]}], 
                          "\[RightDoubleBracket]"}], 
                        RowBox[{"(", 
                          RowBox[{
                            RowBox[{"If", "[", 
                              RowBox[{
                                RowBox[{"n", "\[GreaterEqual]", "1"}], ",", 
                                RowBox[{"aold", "\[LeftDoubleBracket]", 
                                  RowBox[{
                                    RowBox[{"p", "+", "1"}], ",", 
                                    RowBox[{"k", "+", "1"}], ",", 
                                    RowBox[{"n", "-", "1", "+", "1"}]}], 
                                  "\[RightDoubleBracket]"}], ",", "0"}], 
                              "]"}], "+", 
                            RowBox[{"If", "[", 
                              RowBox[{
                                RowBox[{"n", "\[LessEqual]", 
                                  RowBox[{"nn", "-", "2"}]}], ",", 
                                RowBox[{"I", " ", 
                                  RowBox[{"aold", "\[LeftDoubleBracket]", 
                                    RowBox[{
                                      RowBox[{"1", "-", "p", "+", "1"}], ",", 
                                      
                                      RowBox[{"k", "+", "1"}], ",", 
                                      RowBox[{"n", "+", "1"}]}], 
                                    "\[RightDoubleBracket]"}]}], ",", "0"}], 
                              "]"}]}], ")"}]}]}], ",", 
                    RowBox[{"{", 
                      RowBox[{"j", ",", "0", ",", "1"}], "}"}], ",", 
                    RowBox[{"{", 
                      RowBox[{"k", ",", "0", ",", "1"}], "}"}], ",", 
                    RowBox[{"{", 
                      RowBox[{"n", ",", "0", ",", 
                        RowBox[{"nn", "-", "1"}]}], "}"}]}], "]"}]}], "]"}]}],
           "]"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
    FormBox[
      RowBox[{"\[CapitalDelta]p", "=", "0.1"}], 
      TraditionalForm]]]], "Subsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{
          "0", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", 
            ",", "1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "0", ",", 
            "0", ",", "1", ",", "0", ",", "1", ",", "0", ",", "0", ",", "0", 
            ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0", ",", 
            "1", ",", "1", ",", "0", ",", "0", ",", "0", ",", "1", ",", "0", 
            ",", "0", ",", "1", ",", "1", ",", "1", ",", "0", ",", "1", ",", 
            "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0", ",", "0", 
            ",", "1", ",", "1", ",", "0", ",", "1", ",", "1", ",", "1", ",", 
            "0", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0", ",", "0", 
            ",", "0", ",", "0", ",", "0", ",", "1", ",", "1", ",", "0", ",", 
            "0", ",", "1", ",", "0", ",", "0", ",", "0", ",", "1", ",", "0", 
            ",", "1", ",", "0", ",", "1", ",", "0", ",", "1", ",", "1", ",", 
            "1", ",", "0", ",", "0", ",", "1", ",", "0", ",", "1", ",", "1", 
            ",", "0", ",", "0", ",", "1", ",", "1", ",", "1", ",", "1", ",", 
            "0", ",", "0", ",", "0", ",", "0", ",", "1", ",", "0", ",", "1", 
            ",", "0", ",", "0"}], "}"}]}], ";"}]], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox[
                      RowBox[{"11", "/", "20"}]], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox[
                      RowBox[{"9", "/", "20"}]]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox[
                      RowBox[{"9", "/", "20"}]], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox[
                      RowBox[{"11", "/", "20"}]]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mmN", "=", 
        RowBox[{"mm", "//", "N"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mmNhp", "=", 
        RowBox[{"mm", "//", 
          RowBox[{
            RowBox[{"N", "[", 
              RowBox[{"#", ",", "100"}], "]"}], "&"}]}]}], ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"a100Nhp", "=", 
        RowBox[{"makea", "[", 
          RowBox[{"100", ",", "mmNhp", ",", "m"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"a100N", "=", 
        RowBox[{"makea", "[", 
          RowBox[{"100", ",", "mmN", ",", "m"}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"diff", "=", 
        RowBox[{
          RowBox[{
            RowBox[{"a100N", "-", "a100Nhp"}], "//", "N"}], "//", "Abs"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"Max", "[", "diff", "]"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"$RecursionLimit", "=", "1000"}], ";", 
      RowBox[{"whereAbsMax", "[", "diff", "]"}]}], "\[IndentingNewLine]", 
    RowBox[{"diff", "\[LeftDoubleBracket]", 
      RowBox[{"2", ",", "2", ",", "52"}], 
      "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
    RowBox[{"a100Nhp", "\[LeftDoubleBracket]", 
      RowBox[{"2", ",", "2", ",", "52"}], "\[RightDoubleBracket]"}]}], "Input"],

Cell[BoxData["0.1328125`"], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"0.1328125`", ",", "52"}], "}"}]], "Output"],

Cell[BoxData["0.0703125`"], "Output"],

Cell[BoxData[
  "3.4247484624737600494670707173239652025949588988545833211344646296512376506\
34560540641302248528984939315109007396885`98*^13"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"?", "N"}]], "Input"],

Cell[BoxData[
    RowBox[{"\<\"N[expr] gives the numerical value of expr. N[expr, n] \
attempts to give a result with n-digit precision.\"\>", " ", 
      ButtonBox[
        StyleBox["More\[Ellipsis]",
          "SR"],
        ButtonData:>"N",
        Active->True,
        ButtonStyle->"RefGuideLink"]}]], "Print",
  CellTags->"Info3266353134-4839921"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"makea", "[", 
        RowBox[{"30", ",", "mm", ",", "m"}], "]"}], "//", "N"}]], "Input"],

Cell[BoxData["$Aborted"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"(", 
        RowBox[{
          RowBox[{"makea", "[", 
            RowBox[{"20", ",", "mm", ",", "m"}], "]"}], "//", "N"}], ")"}], 
      "-", 
      RowBox[{"makea", "[", 
        RowBox[{"20", ",", "mmN", ",", "m"}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"1.3877787807814457`*^-17", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"-", "8.881784197001252`*^-16"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"1.0658141036401503`*^-14", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"-", "4.263256414560601`*^-14"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"4.263256414560601`*^-14", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"-", "4.263256414560601`*^-14"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"1.0658141036401503`*^-14", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"-", "2.6645352591003757`*^-15"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"5.551115123125783`*^-17", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"-", "3.2526065174565133`*^-19"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"4.336808689942018`*^-19", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{"8.326672684688674`*^-17", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{
                  "1.7763568394002505`*^-15", " ", "\[ImaginaryI]"}]}], ",", 
                "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{
                  "1.0658141036401503`*^-14", " ", "\[ImaginaryI]"}]}], ",", 
                "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"2.842170943040401`*^-14", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{"4.263256414560601`*^-14", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{
                  "2.1316282072803006`*^-14", " ", "\[ImaginaryI]"}]}], ",", 
                "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{"5.329070518200751`*^-15", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"2.220446049250313`*^-16", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{"1.734723475976807`*^-17", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`"}], "}"}]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"2.168404344971009`*^-19", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{
                  "1.3877787807814457`*^-16", " ", "\[ImaginaryI]"}]}], ",", 
                "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{
                  "1.7763568394002505`*^-15", " ", "\[ImaginaryI]"}]}], ",", 
                "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{
                  "2.4868995751603507`*^-14", " ", "\[ImaginaryI]"}]}], ",", 
                "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"2.842170943040401`*^-14", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{"2.842170943040401`*^-14", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"2.842170943040401`*^-14", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{"5.329070518200751`*^-15", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"5.551115123125783`*^-16", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`", ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "-", 
                  RowBox[{"8.673617379884035`*^-18", " ", "\[ImaginaryI]"}]}],
                 ",", "0.`"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"8.673617379884035`*^-18", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"-", "3.3306690738754696`*^-16"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"5.329070518200751`*^-15", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"-", "1.4210854715202004`*^-14"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"2.842170943040401`*^-14", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"-", "2.842170943040401`*^-14"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"7.105427357601002`*^-15", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"-", "8.881784197001252`*^-16"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"8.326672684688674`*^-17", "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{
                  RowBox[{"0.`", "\[InvisibleSpace]"}], "+", 
                  RowBox[{"0.`", " ", "\[ImaginaryI]"}]}], ",", 
                RowBox[{"-", "2.168404344971009`*^-19"}]}], "}"}]}], "}"}]}], 
      "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"a", "[", 
      RowBox[{"10", ",", "0", ",", "0", ",", "0"}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"%", "//", "N"}]}], "Input"],

Cell[BoxData["0"], "Output"],

Cell[BoxData["0.`"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\[Phi] = \[Pi]/16; 
    u = {{Cos[\[Phi]], I\ Sin[\[Phi]]}, {I\ Sin[\[Phi]], 
          Cos[\[Phi]]}};\), "\[IndentingNewLine]", 
    \(\(\(u // N\) // MatrixForm\) // Chop\)}], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {"0.9807852804032304`", \(0.19509032201612825`\ \[ImaginaryI]\)},
            {\(0.19509032201612825`\ \[ImaginaryI]\), "0.9807852804032304`"}
            }], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Compile[{nn}, nn\^2]\)[3]\)], "Input"],

Cell[BoxData[
    \(9.`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Compile[{nn, j, k, n}, a[nn, j, k, n]]\)[10, 0, 0, 0]\)], "Input"],

Cell[BoxData[
    \(0.`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      TagBox[\(CompiledFunction[{nn, j, k, n}, a, "\<-CompiledCode-\>"]\),
        False,
        Editable->False], "[", \(20, 0, 0, 0\), "]"}]], "Input"],

Cell[BoxData[
    \(Syntax::"noinfo" \(\(:\)\(\ \)\) 
      "Input expression contains insufficient information to interpret \
result."\)], "Message"],

Cell[BoxData[
    \(Syntax::"noinfo" \(\(:\)\(\ \)\) 
      "Input expression contains insufficient information to interpret \
result."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"argrx" \(\(:\)\(\ \)\) 
      "\!\(CompiledFunction\) called with \!\(0\) arguments; \!\(6\) \
arguments are expected."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"argrx" \(\(:\)\(\ \)\) 
      "\!\(CompiledFunction\) called with \!\(0\) arguments; \!\(6\) \
arguments are expected."\)], "Message"],

Cell[BoxData[
    \(CompiledFunction::"cfnv" \(\(:\)\(\ \)\) 
      "CompiledFunction object is invalid and cannot be used."\)], "Message"],

Cell[BoxData[
    RowBox[{
      TagBox[\(CompiledFunction[]\),
        False,
        Editable->False], "[", \(20, 0, 0, 0\), "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(a[20, 0, 0, 0]\)], "Input"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[
      terseForm[\[IndentingNewLine]{{a[nn, 0, 0, n], 
            a[nn, 0, 1, n]}, {a[nn, 1, 0, n\ ], 
            a[nn, 1, 1, n]}}\[IndentingNewLine]], {nn, 1, 4}, {n, 0, 
        nn - 1}]\)], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          TagBox[
            TagBox[
              RowBox[{"(", "\[NoBreak]", GridBox[{
                    {
                      InterpretationBox["\<\"0.74\"\>",
                        0.74161984870956632,
                        AutoDelete->True], 
                      InterpretationBox["\<\"0\"\>",
                        0,
                        Editable->False]},
                    {
                      InterpretationBox["\<\"0\"\>",
                        0,
                        Editable->False], 
                      InterpretationBox["\<\"0.67\"\>",
                        0.67082039324993692,
                        AutoDelete->True]}
                    }], "\[NoBreak]", ")"}],
              Function[ BoxForm`e$, 
                MatrixForm[ BoxForm`e$]]],
            (NumberForm[ #, 2]&)], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        RowBox[{
                          InterpretationBox["\<\"0.45\"\>",
                            0.45000000000000001,
                            AutoDelete->True], " ", "\[ImaginaryI]"}]},
                      {
                        RowBox[{
                          InterpretationBox["\<\"0.55\"\>",
                            0.55000000000000004,
                            AutoDelete->True], " ", "\[ImaginaryI]"}], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)], ",", 
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"0.5\"\>",
                          0.49749371855330998,
                          AutoDelete->True], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]},
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        InterpretationBox["\<\"0.5\"\>",
                          0.49749371855330998,
                          AutoDelete->True]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"-0.37\"\>",
                          -0.36895121628746536,
                          AutoDelete->True], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]},
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        InterpretationBox["\<\"-0.33\"\>",
                          -0.33372893191930486,
                          AutoDelete->True]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)], ",", 
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        RowBox[{
                          InterpretationBox["\<\"0.64\"\>",
                            0.63559810888177648,
                            AutoDelete->True], " ", "\[ImaginaryI]"}]},
                      {
                        RowBox[{
                          InterpretationBox["\<\"0.78\"\>",
                            0.77684213307772676,
                            AutoDelete->True], " ", "\[ImaginaryI]"}], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)], ",", 
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"0.33\"\>",
                          0.33372893191930486,
                          AutoDelete->True], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]},
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        InterpretationBox["\<\"0.37\"\>",
                          0.3689512162874653,
                          AutoDelete->True]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        RowBox[{
                          InterpretationBox["\<\"-0.25\"\>",
                            -0.24750000000000003,
                            AutoDelete->True], " ", "\[ImaginaryI]"}]},
                      {
                        RowBox[{
                          InterpretationBox["\<\"-0.25\"\>",
                            -0.24750000000000005,
                            AutoDelete->True], " ", "\[ImaginaryI]"}], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)], ",", 
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"-0.85\"\>",
                          -0.84974309040864104,
                          AutoDelete->True], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]},
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        InterpretationBox["\<\"-0.65\"\>",
                          -0.65024434669797904,
                          AutoDelete->True]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)], ",", 
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        RowBox[{
                          InterpretationBox["\<\"0.74\"\>",
                            0.74499371855330998,
                            AutoDelete->True], " ", "\[ImaginaryI]"}]},
                      {
                        RowBox[{
                          InterpretationBox["\<\"0.74\"\>",
                            0.74499371855330998,
                            AutoDelete->True], " ", "\[ImaginaryI]"}], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)], ",", 
            TagBox[
              TagBox[
                RowBox[{"(", "\[NoBreak]", GridBox[{
                      {
                        InterpretationBox["\<\"0.25\"\>",
                          0.24750000000000003,
                          AutoDelete->True], 
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False]},
                      {
                        InterpretationBox["\<\"0\"\>",
                          0,
                          Editable->False], 
                        InterpretationBox["\<\"0.25\"\>",
                          0.2475,
                          AutoDelete->True]}
                      }], "\[NoBreak]", ")"}],
                Function[ BoxForm`e$, 
                  MatrixForm[ BoxForm`e$]]],
              (NumberForm[ #, 2]&)]}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Simplification of ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(E\&_\)(m\&\[RightVector])\)\_jj\)]],
  ":"
}], "Subsection"],

Cell[BoxData[
    \(eee[
        N_] := \[Sum]\_\(n = 0\)\%\(N - 1\)Conjugate[
            aN[p, j, 
              n]] \(\[Sum]\_\(q = Floor[\((n + 1)\)/2]\)\%\(Ceiling[\((n + N \
- 1 - 1)\)/2]\)aN[p, j, 2  q - n] isc2[q, N - 1 - q]\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(eee[1] // FullSimplify\)], "Input"],

Cell[BoxData[
    \(Abs[aN[p, j, 0]]\^2\ isc2[0, 0]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(eee[2] /. isc2[1, 0] \[Rule] isc2[0, 1] // FullSimplify\)], "Input"],

Cell[BoxData[
    \(\((Abs[aN[p, j, 0]]\^2 + Abs[aN[p, j, 1]]\^2)\)\ isc2[0, 1]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(eee[3] /. isc2[1, 0] \[Rule] isc2[0, 1]\) /. 
        isc2[2, 0] \[Rule] isc2[0, 2] // FullSimplify\)], "Input"],

Cell[BoxData[
    \(\((Abs[aN[p, j, 0]]\^2 + Abs[aN[p, j, 2]]\^2)\)\ isc2[0, 
          2] + \((Abs[aN[p, j, 1]]\^2 + aN[p, j, 2]\ Conjugate[aN[p, j, 0]] + 
            aN[p, j, 0]\ Conjugate[aN[p, j, 2]])\)\ isc2[1, 1]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(\(\(\(eee[4] /. isc2[1, 0] \[Rule] isc2[0, 1]\) /. 
              isc2[2, 0] \[Rule] isc2[0, 2]\) /. 
            isc2[2, 1] \[Rule] isc2[1, 2]\) /. isc2[3, 0] \[Rule] isc2[0, 3] // 
        FullSimplify;\)\), "\[IndentingNewLine]", 
    \(% - \((\((Abs[aN[p, j, 0]]\^2 + Abs[aN[p, j, 3]]\^2)\)\ isc2[0, 
                3] + \[IndentingNewLine]\((Abs[aN[p, j, 1]]\^2 + 
                  Abs[aN[p, j, 2]]\^2 + 
                  2*Re[\[Sum]\_\(r = 0\)\%1 aN[p, j, r] 
                          Conjugate[aN[p, j, 2 + r]\ ]])\)\ isc2[1, 2])\) // 
      FullSimplify\)}], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(mm = {{{1, 0}, {0, 1}}}; \[Phi] = \[Pi]/4; 
    u = {{Cos[\[Phi]], I\ Sin[\[Phi]]}, {I\ Sin[\[Phi]], 
          Cos[\[Phi]]}}; \), "\[IndentingNewLine]", 
    \(u // terseForm\), "\[IndentingNewLine]", 
    \(m = {0, 0, 0, 0}; \)}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]},
              {
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], "\[ImaginaryI]"},
                  {"\[ImaginaryI]", 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], "\[ImaginaryI]"},
                  {"\[ImaginaryI]", 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.25\"\>",
                      0.25099800796022265,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.38\"\>",
                      0.38340579025361626,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.14\"\>",
                      0.13747727084867517,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32078029864690877,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.25\"\>",
                      0.25099800796022265,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.38\"\>",
                      0.38340579025361626,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.14\"\>",
                      0.13747727084867517,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32078029864690877,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.49\"\>",
                        0.49497474683058335,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.21213203435596426,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.066\"\>",
                      0.066203891146696836,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.48\"\>",
                        0.48453390441373473,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20765738760588626,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.066\"\>",
                      -0.066203891146696836,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.055\"\>",
                      -0.054784703604925045,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.33\"\>",
                      -0.32582087381039188,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.49\"\>",
                        0.49497474683058335,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.21213203435596423,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.066\"\>",
                      0.066203891146696836,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.48\"\>",
                        0.48453390441373467,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20765738760588628,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.066\"\>",
                      -0.066203891146696808,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.055\"\>",
                      -0.054784703604925045,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.33\"\>",
                      -0.32582087381039182,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 5", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Phi]", "=", "0.13"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}]},
              {
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}], 
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurement operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm", "=", "tmpM3"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.12\"\>",
                        0.11556295100560166,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.079\"\>",
                          0.078983647783652591,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.14\"\>",
                        -0.14476493145406052,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1054201530221143,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.082\"\>",
                        -0.081812303125227859,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.14\"\>",
                          0.13941908241666626,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.23137397334981527,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.085\"\>",
                          0.085088592206103958,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.038826146894781255,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.06\"\>",
                          0.059520299257066998,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15679433193217271,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.044\"\>",
                          0.043552706959805565,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.017\"\>",
                        -0.017078499979601813,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11321981049548259,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.078\"\>",
                        -0.078018284873153987,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.034\"\>",
                          0.034078738062648661,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.053101810590524066,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.016\"\>",
                          0.015613123710853237,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.085\"\>",
                        -0.084827625921655186,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.054\"\>",
                          0.05406159086005586,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043739384895579103,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.059\"\>",
                          0.059134992020686943,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.045\"\>",
                        0.045357994142567352,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.046\"\>",
                          0.046102257596487922,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.12\"\>",
                        0.11556295100560164,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.079\"\>",
                          0.078983647783652605,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.14\"\>",
                        -0.14476493145406055,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1054201530221143,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.082\"\>",
                        -0.081812303125227887,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.14\"\>",
                          0.13941908241666626,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2313739733498153,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.085\"\>",
                          0.085088592206103958,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.03882614689478122,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.06\"\>",
                          0.059520299257066971,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15679433193217271,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.044\"\>",
                          0.043552706959805586,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.017\"\>",
                        -0.017078499979601827,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1132198104954826,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.078\"\>",
                        -0.078018284873153987,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.034\"\>",
                          0.034078738062648661,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.053101810590524053,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.016\"\>",
                          0.015613123710853214,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.085\"\>",
                        -0.084827625921655214,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.054\"\>",
                          0.054061590860055866,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043739384895579124,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.059\"\>",
                          0.059134992020686949,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.045\"\>",
                        0.045357994142567359,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.046\"\>",
                          0.046102257596487929,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
Evaluator->"psi",
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Magnification->1.25
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Info3266353134-4839921"->{
    Cell[17659, 468, 353, 9, 54, "Print",
      CellTags->"Info3266353134-4839921"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Info3266353134-4839921", 107544, 2806}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 43, 0, 56, "Subsection"],
Cell[1785, 53, 198, 5, 36, "Input"],
Cell[1986, 60, 401, 10, 36, "Input"],

Cell[CellGroupData[{
Cell[2412, 74, 2477, 57, 243, "Input"],
Cell[4892, 133, 80, 2, 36, "Output"],
Cell[4975, 137, 80, 2, 36, "Output"],
Cell[5058, 141, 80, 2, 36, "Output"],
Cell[5141, 145, 80, 2, 36, "Output"],
Cell[5224, 149, 83, 2, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5356, 157, 353, 12, 60, "Subsection"],
Cell[5712, 171, 453, 15, 36, "Text"],
Cell[6168, 188, 580, 16, 45, "Input"],
Cell[6751, 206, 2497, 56, 141, "Input"],
Cell[9251, 264, 4255, 87, 190, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13543, 356, 130, 3, 54, "Subsection"],
Cell[13676, 361, 1320, 20, 82, "Input"],
Cell[14999, 383, 1201, 34, 86, "Input"],
Cell[16203, 419, 165, 4, 36, "Input"],
Cell[16371, 425, 161, 4, 36, "Input"],

Cell[CellGroupData[{
Cell[16557, 433, 684, 15, 128, "Input"],
Cell[17244, 450, 37, 0, 36, "Output"],
Cell[17284, 452, 90, 2, 36, "Output"],
Cell[17377, 456, 37, 0, 36, "Output"],
Cell[17417, 458, 155, 2, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17609, 465, 47, 1, 36, "Input"],
Cell[17659, 468, 353, 9, 54, "Print",
  CellTags->"Info3266353134-4839921"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18049, 482, 130, 3, 36, "Input"],
Cell[18182, 487, 35, 0, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18254, 492, 279, 8, 36, "Input"],
Cell[18536, 502, 10182, 207, 336, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28755, 714, 168, 4, 59, "Input"],
Cell[28926, 720, 28, 0, 36, "Output"],
Cell[28957, 722, 30, 0, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29024, 727, 207, 4, 59, "Input"],
Cell[29234, 733, 335, 7, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29606, 745, 60, 1, 37, "Input"],
Cell[29669, 748, 37, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29743, 754, 88, 1, 36, "Input"],
Cell[29834, 757, 37, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29908, 763, 181, 4, 36, "Input"],
Cell[30092, 769, 150, 3, 28, "Message"],
Cell[30245, 774, 150, 3, 28, "Message"],
Cell[30398, 779, 173, 3, 28, "Message"],
Cell[30574, 784, 173, 3, 28, "Message"],
Cell[30750, 789, 139, 2, 28, "Message"],
Cell[30892, 793, 144, 4, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31073, 802, 47, 1, 36, "Input"],
Cell[31123, 805, 42, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31202, 811, 229, 5, 82, "Input"],
Cell[31434, 818, 9457, 227, 92, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[40940, 1051, 145, 5, 62, "Subsection"],
Cell[41088, 1058, 247, 5, 47, "Input"],

Cell[CellGroupData[{
Cell[41360, 1067, 55, 1, 36, "Input"],
Cell[41418, 1070, 65, 1, 37, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41520, 1076, 88, 1, 36, "Input"],
Cell[41611, 1079, 93, 1, 37, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41741, 1085, 134, 2, 36, "Input"],
Cell[41878, 1089, 233, 3, 60, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42148, 1097, 602, 10, 139, "Input"],
Cell[42753, 1109, 35, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[42837, 1116, 31, 0, 56, "Subsection"],

Cell[CellGroupData[{
Cell[42893, 1120, 262, 5, 82, "Input"],
Cell[43158, 1127, 1178, 34, 73, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44373, 1166, 1569, 38, 174, "Input"],
Cell[45945, 1206, 3964, 106, 73, "Output"],
Cell[49912, 1314, 3964, 106, 73, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[53925, 1426, 31, 0, 56, "Subsection"],
Cell[53959, 1428, 1423, 40, 63, "Input"],

Cell[CellGroupData[{
Cell[55407, 1472, 1569, 38, 161, "Input"],
Cell[56979, 1512, 3052, 82, 49, "Output"],
Cell[60034, 1596, 3052, 82, 49, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[63135, 1684, 31, 0, 56, "Subsection"],
Cell[63169, 1686, 1456, 41, 63, "Input"],

Cell[CellGroupData[{
Cell[64650, 1731, 1569, 38, 161, "Input"],
Cell[66222, 1771, 3550, 88, 51, "Output"],
Cell[69775, 1861, 3550, 88, 51, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[73374, 1955, 31, 0, 56, "Subsection"],

Cell[CellGroupData[{
Cell[73430, 1959, 1653, 43, 119, "Input"],
Cell[75086, 2004, 840, 22, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[75963, 2031, 154, 3, 52, "Subsubsection"],

Cell[CellGroupData[{
Cell[76142, 2038, 5935, 159, 240, "Input"],
Cell[82080, 2199, 666, 20, 49, "Output"],
Cell[82749, 2221, 2474, 62, 49, "Output"],
Cell[85226, 2285, 5447, 122, 51, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[90722, 2413, 38, 0, 52, "Subsubsection"],

Cell[CellGroupData[{
Cell[90785, 2417, 1569, 38, 161, "Input"],
Cell[92357, 2457, 7253, 162, 92, "Output"],
Cell[99613, 2621, 7251, 162, 92, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

