(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     75696,       2027]*)
(*NotebookOutlinePosition[     76325,       2049]*)
(*  CellTagsIndexPosition[     76281,       2045]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Some helper functions", "Subsection"],

Cell[BoxData[
    \(\(adj[x_] := Transpose[Conjugate[x]];\)\)], "Input"],

Cell[BoxData[
    \(\(\(terseForm[x_]\)\(:=\)\(NumberForm[MatrixForm[Chop[x]], 
        2]\)\(\ \)\( (*Nice\ and\ short\ display\ of\ matrices*) \)\)\)], \
"Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      \(TraditionalForm\`M(\(m\& \[RightVector] \), \[CurlyPhi])\)]],
  " (called mmm here) directly"
}], "Subsection"],

Cell[BoxData[
    \(\(\(mmmDirect[nn_]\)\(:=\)\(If[nn \[LessEqual] 1, 
        mm\[LeftDoubleBracket]m\[LeftDoubleBracket]1\[RightDoubleBracket] + 
            1\[RightDoubleBracket], 
        mm\[LeftDoubleBracket]m\[LeftDoubleBracket]nn\[RightDoubleBracket] + 
              1\[RightDoubleBracket] . u . 
          mmmDirect[nn - 1]]\)\(\[IndentingNewLine]\)\( (*\ 
        mm\ is\ the\ set\ of\ measurement\ operators\ for\ a\ single\ \
measurement\ *) \)\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Function for calculating ",
  Cell[BoxData[
      \(TraditionalForm\`\(E(m, \[Phi])\)\_\(j\ k\)\)]],
  " (called ",
  Cell[BoxData[
      \(TraditionalForm\`ee\)]],
  " here) according to (1-3)"
}], "Subsection"],

Cell[TextData[{
  "We use the following ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " compatible names for variables: ",
  Cell[BoxData[
      \(TraditionalForm\`mmm = M(m\&\[RightVector], \[CurlyPhi]), \ 
      nn = N\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(mmm[nn_, j_, 
        k_] := \[Sum]\_\(n = 0\)\%\((nn - 1)\)a[nn, j, k, n] \(x\^n\) 
          y\^\(\((nn - 1)\) - n\)\)], "Input"],

Cell[BoxData[
    \(a[nn_, j_, k_, n_] := \[IndentingNewLine]\t
      If[nn \[LessEqual] 
          1, \[IndentingNewLine]\t\(mm\[LeftDoubleBracket]
            m\[LeftDoubleBracket]1\[RightDoubleBracket] + 
              1\[RightDoubleBracket]\)\[LeftDoubleBracket]j + 1, 
          k + 1\[RightDoubleBracket], \[IndentingNewLine]\[Sum]\_\(p = 0\)\%1\
\( mm\[LeftDoubleBracket]m\[LeftDoubleBracket]nn\[RightDoubleBracket] + 
                  1\[RightDoubleBracket]\)\[LeftDoubleBracket]j + 1, 
              p + 1\[RightDoubleBracket] \((a[nn - 1, p, k, n - 1] 
                  If[n \[GreaterEqual] 1, 1, 0] + 
                I\ a[nn - 1, 1 - p, k, n] 
                  If[n \[LessEqual] nn - 2, 1, 0])\)]; \)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 1", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(a[1, 0, 0, 0]\)], "Input"],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]],

Cell[BoxData[{
    \(mm = {{{1, 0}, {0, 1}}}; \[Phi] = 0; 
    u = {{Cos[\[Phi]], I\ Sin[\[Phi]]}, {I\ Sin[\[Phi]], 
          Cos[\[Phi]]}}; \), "\[IndentingNewLine]", 
    \(m = {0, 0, 0, 0}; \)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(Table[
      terseForm[\[IndentingNewLine]mmmDirect[nn]\[IndentingNewLine]], {nn, 1, 
        4}]\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(Table[
      terseForm[\[IndentingNewLine]{{mmm[nn, 0, 0], 
              mmm[nn, 0, 1]}, {mmm[nn, 1, 0], 
              mmm[nn, 1, 1]}}\[IndentingNewLine] /. {x \[Rule] Cos[\[Phi]], 
            y \[Rule] Sin[\[Phi]]}], {nn, 1, 4}]\)}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 2", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"0", ",", "1"}], "}"}]}], "}"}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]},
              {
                FractionBox["\[ImaginaryI]", 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]], 
                FractionBox[
                  InterpretationBox["\<\"1\"\>",
                    1,
                    Editable->False], 
                  SqrtBox[
                    InterpretationBox["\<\"2\"\>",
                      2,
                      Editable->False]]]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], "\[ImaginaryI]"},
                  {"\[ImaginaryI]", 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"1\"\>",
                      1,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    FractionBox[
                      InterpretationBox["\<\"1\"\>",
                        1,
                        Editable->False], 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], "\[ImaginaryI]"},
                  {"\[ImaginaryI]", 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}], 
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]]},
                  {
                    FractionBox["\[ImaginaryI]", 
                      SqrtBox[
                        InterpretationBox["\<\"2\"\>",
                          2,
                          Editable->False]]], 
                    RowBox[{"-", 
                      FractionBox[
                        InterpretationBox["\<\"1\"\>",
                          1,
                          Editable->False], 
                        SqrtBox[
                          InterpretationBox["\<\"2\"\>",
                            2,
                            Editable->False]]]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 3", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", "0"}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "0", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.25\"\>",
                      0.25099800796022265,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.38\"\>",
                      0.38340579025361626,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.14\"\>",
                      0.13747727084867517,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32078029864690877,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.46\"\>",
                      0.45825756949558399,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.25\"\>",
                      0.25099800796022265,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.38\"\>",
                      0.38340579025361626,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.14\"\>",
                      0.13747727084867517,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32078029864690877,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4", "Subsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"mm", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.3"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.7"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    SqrtBox["0.7"], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.3"]}], "}"}]}], "}"}]}], "}"}]}], ";", 
      RowBox[{"\[Phi]", "=", 
        RowBox[{"\[Pi]", "/", "4"}]}], ";", 
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.49\"\>",
                        0.49497474683058335,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.21213203435596426,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.066\"\>",
                      0.066203891146696836,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.48\"\>",
                        0.48453390441373473,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20765738760588626,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.066\"\>",
                      -0.066203891146696836,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.055\"\>",
                      -0.054784703604925045,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.33\"\>",
                      -0.32582087381039188,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.55\"\>",
                      0.54772255750516607,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False]},
                  {
                    InterpretationBox["\<\"0\"\>",
                      0,
                      Editable->False], 
                    InterpretationBox["\<\"0.84\"\>",
                      0.83666002653407556,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.49\"\>",
                        0.49497474683058335,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.21213203435596423,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"0.32\"\>",
                      0.32403703492039304,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.066\"\>",
                      0.066203891146696836,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.48\"\>",
                        0.48453390441373467,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20765738760588628,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.066\"\>",
                      -0.066203891146696808,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.055\"\>",
                      -0.054784703604925045,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16201851746019655,
                        AutoDelete->True], " ", "\[ImaginaryI]"}], 
                    InterpretationBox["\<\"-0.33\"\>",
                      -0.32582087381039182,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 5", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Phi]", "=", "0.13"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"u", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Cos", "[", "\[Phi]", "]"}], ",", 
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"I", " ", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"Cos", "[", "\[Phi]", "]"}]}], "}"}]}], "}"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"u", "//", "terseForm"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"m", "=", 
        RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "}"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True], 
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}]},
              {
                RowBox[{
                  InterpretationBox["\<\"0.13\"\>",
                    0.12963414261969486,
                    AutoDelete->True], " ", "\[ImaginaryI]"}], 
                InterpretationBox["\<\"0.99\"\>",
                  0.99156189371478809,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Construction of a non minimal set of measurement operators that are \
not diagonal in the {|0>,|1>} basis and non normal:\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"tmpU", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "0.37"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "U"}], " ", "*)"}]}], "\[IndentingNewLine]",
   
    RowBox[{
      RowBox[{
        RowBox[{"tmpW", "=", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
                    RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"-", 
                      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
                    RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "}"}]}], "}"}], "/.",
             
            RowBox[{"\[Theta]", "\[Rule]", "1.2"}]}]}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Arbitrary", " ", "basis", " ", "\"\<converter\>\""}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM1", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "+", 
                      RowBox[{
                        SqrtBox["0.1"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.3"], "+", 
                      RowBox[{
                        SqrtBox["0.2"], "I"}]}]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.1"], "+", 
                      RowBox[{
                        SqrtBox["0.4"], "I"}]}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    SqrtBox["0.4"]}], "}"}]}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{
                      SqrtBox["0.2"], "I"}], ",", "0"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"0", ",", 
                    RowBox[{
                      SqrtBox["0.05"], "+", 
                      RowBox[{
                        SqrtBox["0.05"], "I"}]}]}], "}"}]}], "}"}]}], "}"}]}],
       ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM2", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"#", ".", "tmpU"}], "&"}], ",", "tmpM1"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"tmpM3", "=", 
        RowBox[{"Map", "[", 
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{"adj", "[", "tmpW", "]"}], ".", "#", ".", "tmpW"}], 
              "&"}], ",", "tmpM2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}],
           "+", 
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "tmpM3", "\[LeftDoubleBracket]", "3", "\[RightDoubleBracket]"}],
               "]"}], ".", 
            RowBox[{
            "tmpM3", "\[LeftDoubleBracket]", "3", 
              "\[RightDoubleBracket]"}]}]}], "//", "terseForm"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{"Completeness", " ", "check"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Map", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"terseForm", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}], ".", 
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}]}], "-", 
                RowBox[{
                  RowBox[{
                  "tmpM3", "\[LeftDoubleBracket]", "#", 
                    "\[RightDoubleBracket]"}], ".", 
                  RowBox[{"adj", "[", 
                    RowBox[{
                    "tmpM3", "\[LeftDoubleBracket]", "#", 
                      "\[RightDoubleBracket]"}], "]"}]}]}], "]"}], "&"}], ",",
           
          RowBox[{"{", 
            RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "]"}], 
      RowBox[{"(*", " ", 
        RowBox[{"Non", " ", "normality", " ", "test"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"mm", "=", "tmpM3"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"Map", "[", 
      RowBox[{"terseForm", ",", "mm"}], "]"}]}], "Input"],

Cell[BoxData[
    TagBox[
      TagBox[
        RowBox[{"(", "\[NoBreak]", GridBox[{
              {
                InterpretationBox["\<\"1.\"\>",
                  1.0,
                  AutoDelete->True], 
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False]},
              {
                InterpretationBox["\<\"0\"\>",
                  0,
                  Editable->False], 
                InterpretationBox["\<\"1.\"\>",
                  0.99999999999999989,
                  AutoDelete->True]}
              }], "\[NoBreak]", ")"}],
        (MatrixForm[ #]&)],
      (NumberForm[ #, 2]&)]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.026\"\>",
                      0.026260501618629362,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"0.067\"\>",
                      0.067387052763466404,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.026\"\>",
                      -0.026260501618629362,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314681,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314681,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"-0.013\"\>",
                      -0.013130250809314709,
                      AutoDelete->True], 
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True]},
                  {
                    InterpretationBox["\<\"-0.034\"\>",
                      -0.033693526381733195,
                      AutoDelete->True], 
                    InterpretationBox["\<\"0.013\"\>",
                      0.013130250809314709,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.51\"\>",
                        0.51062771578785471,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41691166777458855,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.16\"\>",
                        0.16164473899988241,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11425562152679138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.2\"\>",
                        -0.19813952774662302,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16181655619802346,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.42\"\>",
                        0.41697846679089179,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.29\"\>",
                          0.29486559033422266,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.21\"\>",
                        0.20841020931068516,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.21\"\>",
                          0.20853925510072138,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.00017\"\>",
                        -0.00016596266908511118,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.16\"\>",
                          0.16188530018640618,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.081\"\>",
                        -0.081025631427745645,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.081\"\>",
                          0.080693706089575434,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.000065\"\>",
                        6.4522895018113581*^-05,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.42\"\>",
                          0.41688494151638844,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations:", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
            RowBox[{"mmmDirect", "[", "nn", "]"}], "\[IndentingNewLine]", 
            "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}], 
      "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"terseForm", "[", "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "0", ",", "1"}], "]"}]}], "}"}], ",",
                 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "0"}], "]"}], ",", 
                    RowBox[{"mmm", "[", 
                      RowBox[{"nn", ",", "1", ",", "1"}], "]"}]}], "}"}]}], 
              "}"}], "\[IndentingNewLine]", "/.", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"x", "\[Rule]", 
                  RowBox[{"Cos", "[", "\[Phi]", "]"}]}], ",", 
                RowBox[{"y", "\[Rule]", 
                  RowBox[{"Sin", "[", "\[Phi]", "]"}]}]}], "}"}]}], "]"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", "4"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.12\"\>",
                        0.11556295100560166,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.079\"\>",
                          0.078983647783652591,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.14\"\>",
                        -0.14476493145406052,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1054201530221143,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.082\"\>",
                        -0.081812303125227859,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.14\"\>",
                          0.13941908241666626,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.23137397334981527,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.085\"\>",
                          0.085088592206103958,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.038826146894781255,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.06\"\>",
                          0.059520299257066998,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15679433193217271,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.044\"\>",
                          0.043552706959805565,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.017\"\>",
                        -0.017078499979601813,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.11321981049548259,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.078\"\>",
                        -0.078018284873153987,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.034\"\>",
                          0.034078738062648661,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.053101810590524066,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.016\"\>",
                          0.015613123710853237,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.085\"\>",
                        -0.084827625921655186,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.054\"\>",
                          0.05406159086005586,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043739384895579103,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.059\"\>",
                          0.059134992020686943,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.045\"\>",
                        0.045357994142567352,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.046\"\>",
                          0.046102257596487922,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.59\"\>",
                        0.58956433824159116,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00018\"\>",
                          0.00018249830643637743,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.11\"\>",
                        0.11411813355002595,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.23\"\>",
                          0.22917509372992317,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2289403870724554,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.00047\"\>",
                          0.0004694133149356004,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.29\"\>",
                        0.29491904285062281,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.59\"\>",
                          0.58947308908837293,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.12\"\>",
                        0.11556295100560164,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.079\"\>",
                          0.078983647783652605,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.14\"\>",
                        -0.14476493145406055,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1054201530221143,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.082\"\>",
                        -0.081812303125227887,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.14\"\>",
                          0.13941908241666626,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.23\"\>",
                        -0.2313739733498153,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.085\"\>",
                          0.085088592206103958,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.039\"\>",
                        0.03882614689478122,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.06\"\>",
                          0.059520299257066971,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.16\"\>",
                        -0.15679433193217271,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.044\"\>",
                          0.043552706959805586,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.017\"\>",
                        -0.017078499979601827,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.11\"\>",
                          0.1132198104954826,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.078\"\>",
                        -0.078018284873153987,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.034\"\>",
                          0.034078738062648661,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.053\"\>",
                        0.053101810590524053,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.016\"\>",
                          0.015613123710853214,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"-0.085\"\>",
                        -0.084827625921655214,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.054\"\>",
                          0.054061590860055866,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.044\"\>",
                        0.043739384895579124,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.059\"\>",
                          0.059134992020686949,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    RowBox[{
                      InterpretationBox["\<\"0.045\"\>",
                        0.045357994142567359,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.046\"\>",
                          0.046102257596487929,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]}
                  }], "\[NoBreak]", ")"}],
            (MatrixForm[ #]&)],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
WindowSize->{1137, 829},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 43, 0, 45, "Subsection"],
Cell[1773, 54, 72, 1, 27, "Input"],
Cell[1848, 57, 164, 3, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2049, 65, 182, 5, 45, "Subsection"],
Cell[2234, 72, 473, 8, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2744, 85, 231, 8, 49, "Subsection"],
Cell[2978, 95, 254, 9, 32, "Text"],
Cell[3235, 106, 151, 3, 35, "Input"],
Cell[3389, 111, 726, 12, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4152, 128, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[4208, 132, 46, 1, 27, "Input"],
Cell[4257, 135, 35, 1, 27, "Output"]
}, Open  ]],
Cell[4307, 139, 208, 4, 43, "Input"],

Cell[CellGroupData[{
Cell[4540, 147, 417, 8, 123, "Input"],
Cell[4960, 157, 2876, 82, 45, "Output"],
Cell[7839, 241, 2876, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10764, 329, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[10820, 333, 1127, 31, 59, "Input"],
Cell[11950, 366, 1178, 34, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13165, 405, 1569, 38, 123, "Input"],
Cell[14737, 445, 3964, 106, 77, "Output"],
Cell[18704, 553, 3964, 106, 77, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[22717, 665, 31, 0, 45, "Subsection"],
Cell[22751, 667, 1423, 40, 50, "Input"],

Cell[CellGroupData[{
Cell[24199, 711, 1569, 38, 123, "Input"],
Cell[25771, 751, 3052, 82, 45, "Output"],
Cell[28826, 835, 3052, 82, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[31927, 923, 31, 0, 45, "Subsection"],
Cell[31961, 925, 1456, 41, 50, "Input"],

Cell[CellGroupData[{
Cell[33442, 970, 1569, 38, 123, "Input"],
Cell[35014, 1010, 3550, 88, 45, "Output"],
Cell[38567, 1100, 3550, 88, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[42166, 1194, 31, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[42222, 1198, 1653, 43, 91, "Input"],
Cell[43878, 1243, 840, 22, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44755, 1270, 154, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[44934, 1277, 5935, 159, 162, "Input"],
Cell[50872, 1438, 666, 20, 45, "Output"],
Cell[51541, 1460, 2474, 62, 45, "Output"],
Cell[54018, 1524, 5447, 122, 45, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[59514, 1652, 38, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[59577, 1656, 1569, 38, 123, "Input"],
Cell[61149, 1696, 7253, 162, 45, "Output"],
Cell[68405, 1860, 7251, 162, 45, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

