(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    206407,       6737]*)
(*NotebookOutlinePosition[    207063,       6761]*)
(*  CellTagsIndexPosition[    207019,       6757]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Miscellaneous calculations used to create tests", "Section"],

Cell[CellGroupData[{

Cell["Basics", "Subsection"],

Cell[CellGroupData[{

Cell["Time spectrum", "Subsubsection"],

Cell[TextData[{
  "The following function returns samples of the time spectrum of the \
function ",
  Cell[BoxData[
      FormBox["g", TraditionalForm]]],
  " at ",
  Cell[BoxData[
      FormBox[
        RowBox[{"0", ",", "\[Tau]", ",", 
          RowBox[{"2", "\[Tau]"}], ",", "\[Ellipsis]", ",", 
          RowBox[{
            RowBox[{"(", 
              RowBox[{"nn", "-", "1"}], ")"}], " ", "\[Tau]"}]}], 
        TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"sample", "[", 
        RowBox[{"g_", ",", "nn_", ",", "\[Tau]_"}], "]"}], ":=", 
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"g", "[", 
            RowBox[{"k", " ", "\[Tau]"}], "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"k", ",", "0", ",", 
              RowBox[{"nn", "-", "1"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Standard Fourier transform and related power spectrum:", "Subsubsection"],

Cell[TextData[{
  "The following functions \"fourier\" and \"power\" return a sample of the \
Fourier spectrum (",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"G", "(", "\[Omega]", ")"}], "=", 
          RowBox[{
            FractionBox["1", 
              SqrtBox[
                RowBox[{"2", "\[Pi]"}]]], 
            RowBox[{
              SubsuperscriptBox["\[Integral]", 
                RowBox[{"-", "\[Infinity]"}], "\[Infinity]"], 
              RowBox[{
                RowBox[{"\[DifferentialD]", "t"}], " ", 
                SuperscriptBox["\[ExponentialE]", 
                  RowBox[{
                    RowBox[{"-", "i"}], " ", "\[Omega]", " ", "t"}]], 
                RowBox[{"g", "(", "t", ")"}]}]}]}]}], TraditionalForm]]],
  ") and power spectrum of the function ",
  Cell[BoxData[
      FormBox["g", TraditionalForm]]],
  " at ",
  Cell[BoxData[
      FormBox["\[Omega]", TraditionalForm]]],
  ". The function \"expI\[Omega]t\" is necessary to avoid problems during \
integration. Note that due to symmetry properties of the real valued Fourier \
transform (i.e. ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"G", "(", 
            RowBox[{"-", "\[Omega]"}], ")"}], "=", 
          SuperscriptBox[
            RowBox[{"G", "(", "\[Omega]", ")"}], "*"]}], TraditionalForm]]],
  ") the part of the Fourier spectrum for negative frequencies is omitted."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"expMinusI\[Omega]t", "[", 
          RowBox[{"\[Omega]_", ",", "t_"}], "]"}], ":=", 
        RowBox[{"Exp", "[", 
          RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"If", "[", 
              RowBox[{
                RowBox[{"\[Omega]", "\[Equal]", "1."}], ",", "1", ",", 
                "\[Omega]"}], "]"}], "t"}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"fourier", "[", 
        RowBox[{"g_", ",", "\[Omega]_", ",", 
          RowBox[{"range_:", "\[Infinity]"}]}], "]"}], ":=", 
      RowBox[{
        FractionBox["1", 
          SqrtBox[
            RowBox[{"2", "\[Pi]"}]]], 
        RowBox[{"N", "[", 
          RowBox[{"Integrate", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"expMinusI\[Omega]t", "[", 
                  RowBox[{"\[Omega]", ",", "t"}], "]"}], 
                RowBox[{"g", "[", "t", "]"}]}], ",", 
              RowBox[{"{", 
                RowBox[{"t", ",", 
                  RowBox[{"-", "range"}], ",", "range"}], "}"}]}], "]"}], 
          "]"}]}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(fourier\\)\\\" is similar to existing symbol \\\"\\!\\(Fourier\\)\\\
\".\"\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"power", "[", 
        RowBox[{"g_", ",", "\[Omega]_", ",", 
          RowBox[{"range_:", "\[Infinity]"}]}], "]"}], ":=", 
      SuperscriptBox[
        RowBox[{"Abs", "[", 
          RowBox[{"fourier", "[", 
            RowBox[{"g", ",", "\[Omega]", ",", "range"}], "]"}], "]"}], 
        "2"]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(power\\)\\\" is similar to existing symbol \
\\\"\\!\\(Power\\)\\\".\"\>"}]], "Message"]
}, Open  ]],

Cell[TextData[{
  "The following function returns samples of the power spectrum of the \
function ",
  Cell[BoxData[
      FormBox["g", TraditionalForm]]],
  " at ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"\[Omega]", "=", "0"}], ",", " ", 
          RowBox[{"\[Omega]", "=", 
            RowBox[{"2", "\[Pi]", " ", 
              FractionBox["1", 
                RowBox[{"nn", " ", "\[Tau]"}]]}]}], ",", "\[Ellipsis]", ",", 
          RowBox[{"2", "\[Pi]", " ", 
            FractionBox[
              RowBox[{"nn", "-", "1"}], 
              RowBox[{"nn", " ", "\[Tau]"}]]}]}], TraditionalForm]]],
  "."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"powerSpec", "[", 
        RowBox[{"g_", ",", "nn_", ",", "\[Tau]_", ",", 
          RowBox[{"range_:", "\[Infinity]"}]}], "]"}], ":=", 
      RowBox[{"Table", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"2", "\[Pi]", " ", 
                FractionBox["n", 
                  RowBox[{"nn", " ", "\[Tau]"}]]}], ",", 
              RowBox[{"power", "[", 
                RowBox[{"g", ",", 
                  RowBox[{"2", "\[Pi]", " ", 
                    FractionBox["n", 
                      RowBox[{"nn", " ", "\[Tau]"}]]}], ",", "range"}], 
                "]"}]}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"n", ",", "0", ",", 
              RowBox[{"nn", "/", "2"}]}], "}"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Discrete Fourier transform (",
  Cell[BoxData[
      FormBox[
        RowBox[{
          FractionBox["1", 
            SqrtBox[
              RowBox[{"2", "\[Pi]"}]]], 
          RowBox[{
            SubsuperscriptBox["\[Sum]", 
              RowBox[{"k", "=", "0"}], 
              RowBox[{"N", "-", "1"}]], 
            RowBox[{"\[Tau]", " ", 
              SuperscriptBox["\[ExponentialE]", 
                RowBox[{
                  RowBox[{"-", "i"}], " ", 
                  SubscriptBox["\[Omega]", "n"], " ", "k", " ", "\[Tau]"}]], 
              RowBox[{"g", "(", 
                RowBox[{"k", " ", "\[Tau]"}], ")"}]}]}]}], 
        TraditionalForm]]],
  ", ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          SubscriptBox["\[Omega]", "n"], "=", 
          RowBox[{"2", "\[Pi]", " ", 
            FractionBox["n", 
              RowBox[{"N", " ", "\[Tau]"}]]}]}], TraditionalForm]]],
  ") and power spectrum:"
}], "Subsubsection"],

Cell[TextData[{
  "The frequency spectra (i.e. Fourier spectrum and power spectrum) are lists \
of pairs. The first element of each pair is the frequency ",
  Cell[BoxData[
      FormBox[
        SubscriptBox["\[Omega]", "n"], TraditionalForm]]],
  " and the second element is the corresponding amplitude. Due to symmetry \
(see 0403211714), the part of the spectra for negative frequencies is \
omitted."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"discreteFourierSpec", "[", 
          RowBox[{"samples_", ",", "\[Tau]_"}], "]"}], ":=", 
        "\[IndentingNewLine]", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"nn", "=", 
                RowBox[{"Length", "[", "samples", "]"}]}], "}"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Table", "[", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"2", "\[Pi]", " ", 
                      FractionBox["n", 
                        RowBox[{"nn", " ", "\[Tau]"}]]}], ",", 
                    RowBox[{
                      FractionBox["1", 
                        SqrtBox[
                          RowBox[{"2", "\[Pi]"}]]], "\[Tau]", " ", 
                      RowBox[{"Sum", "[", 
                        RowBox[{
                          RowBox[{
                            RowBox[{"Exp", "[", 
                              RowBox[{
                                RowBox[{"-", "2"}], "\[Pi]", " ", "I", " ", 
                                FractionBox["n", "nn"], "k"}], "]"}], 
                            RowBox[{"samples", "\[LeftDoubleBracket]", 
                              RowBox[{"k", "+", "1"}], 
                              "\[RightDoubleBracket]"}]}], ",", 
                          RowBox[{"{", 
                            RowBox[{"k", ",", "0", ",", 
                              RowBox[{"nn", "-", "1"}]}], "}"}]}], "]"}]}]}], 
                  "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"n", ",", "0", ",", 
                    RowBox[{"nn", "/", "2"}]}], "}"}]}], "]"}]}], "]"}]}], 
      ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"discretePowerSpec", "[", 
        RowBox[{"samples_", ",", "\[Tau]_"}], "]"}], ":=", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"fourSpec", "=", 
              RowBox[{"discreteFourierSpec", "[", 
                RowBox[{"samples", ",", "\[Tau]"}], "]"}]}], "}"}], ",", 
          RowBox[{
            RowBox[{
              RowBox[{"{", 
                RowBox[{
                  RowBox[{
                  "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}],
                   ",", 
                  SuperscriptBox[
                    RowBox[{"Abs", "[", 
                      RowBox[{
                      "#", "\[LeftDoubleBracket]", "2", 
                        "\[RightDoubleBracket]"}], "]"}], "2"]}], "}"}], 
              "&"}], "/@", "fourSpec"}]}], "]"}]}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plots", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"powerPlot", "[", 
          RowBox[{"spec_", ",", 
            RowBox[{"plotRange_:", "Automatic"}]}], "]"}], ":=", 
        RowBox[{"ListPlot", "[", 
          RowBox[{"spec", ",", 
            RowBox[{"PlotJoined", "\[Rule]", "True"}], ",", 
            RowBox[{"PlotRange", "\[Rule]", "plotRange"}], ",", 
            RowBox[{"AxesLabel", "\[Rule]", 
              RowBox[{"{", 
                RowBox[{"\"\<\[Omega]=2\[Pi] f\>\"", ",", "\"\<\>\""}], 
                "}"}]}]}], "]"}]}], ";"}]], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Miscellaneous fourier transforms", "Subsection"],

Cell[CellGroupData[{

Cell["sin sampled 4 times during each period", "Subsubsection"],

Cell[TextData[{
  "The peak should be at ",
  Cell[BoxData[
      FormBox[
        RowBox[{"\[Omega]", "=", "1"}], TraditionalForm]]],
  " since then the frequency is ",
  Cell[BoxData[
      FormBox[
        RowBox[{"f", "=", 
          RowBox[{
            RowBox[{
              RowBox[{"\[Omega]", "/", "2"}], "\[Pi]"}], "=", 
            RowBox[{
              RowBox[{"1", "/", "2"}], "\[Pi]"}]}]}], TraditionalForm]]],
  " which is the oscillation frequency of ",
  Cell[BoxData[
      FormBox[
        RowBox[{"sin", "(", "t", ")"}], TraditionalForm]]],
  ". The height of the peak is limited here by the integration range, though \
actually it is divergent."
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Tau]", "=", 
        RowBox[{"0.25", "\[Times]", "2", "\[Pi]"}]}], ";", 
      RowBox[{"nn", "=", "100"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ps", "=", 
        RowBox[{"powerSpec", "[", 
          RowBox[{"Sin", ",", "nn", ",", "\[Tau]", ",", "1000."}], "]"}]}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"ps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "150000"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -2.70508e-17 4.12023e-06 [
[.2619 -0.0125 -9 -9 ]
[.2619 -0.0125 9 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -9 -9 ]
[.7381 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .0824 -30 -4.5 ]
[.01131 .0824 0 4.5 ]
[.01131 .16481 -30 -4.5 ]
[.01131 .16481 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .32962 -30 -4.5 ]
[.01131 .32962 0 4.5 ]
[.01131 .41202 -36 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .49443 -36 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .57683 -36 -4.5 ]
[.01131 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 0 m
.2619 .00625 L
s
[(0.5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(1)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(1.5)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(2)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .0824 m
.03006 .0824 L
s
[(20000)] .01131 .0824 1 0 Mshowa
.02381 .16481 m
.03006 .16481 L
s
[(40000)] .01131 .16481 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(60000)] .01131 .24721 1 0 Mshowa
.02381 .32962 m
.03006 .32962 L
s
[(80000)] .01131 .32962 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(100000)] .01131 .41202 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(120000)] .01131 .49443 1 0 Mshowa
.02381 .57683 m
.03006 .57683 L
s
[(140000)] .01131 .57683 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 0 m
.04286 0 L
.0619 0 L
.08095 0 L
.1 0 L
.11905 0 L
.1381 0 L
.15714 0 L
.17619 0 L
.19524 0 L
.21429 0 L
.23333 0 L
.25238 0 L
.27143 0 L
.29048 0 L
.30952 0 L
.32857 0 L
.34762 0 L
.36667 0 L
.38571 1e-05 L
.40476 1e-05 L
.42381 0 L
.44286 1e-05 L
.4619 .0001 L
.48095 .00023 L
s
.48095 .00023 m
.49892 .61803 L
s
.50108 .61803 m
.51905 .00024 L
.5381 .0001 L
.55714 1e-05 L
.57619 0 L
.59524 1e-05 L
.61429 1e-05 L
.63333 0 L
.65238 0 L
.67143 1e-05 L
.69048 0 L
.70952 0 L
.72857 0 L
.74762 0 L
.76667 0 L
.78571 0 L
.80476 0 L
.82381 0 L
.84286 0 L
.8619 0 L
.88095 0 L
.9 0 L
.91905 0 L
.9381 0 L
.95714 0 L
.97619 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool0057oo`8000Coo`03003ooooo00;oo`<002Soo`@002So
o`@000?oo`03003ooooo00;oo`<002Soo`@003;oo`00D?oo00@00?ooool000_oo`03003ooooo02Oo
o`03003ooooo02Woo`03003ooooo00[oo`03003ooooo02Goo`03003ooooo03?oo`00D?oo00@00?oo
ool000_oo`03003ooooo02Ooo`03003ooooo02Woo`03003ooooo00[oo`03003ooooo02Koo`03003o
oooo03;oo`00D?oo00@00?ooool000Soo`<002[oo`03003ooooo02Woo`03003ooooo00Ooo`<002[o
o`03003ooooo037oo`00D?oo00@00?ooool000Soo`03003ooooo02[oo`03003ooooo02Woo`03003o
oooo00Ooo`03003ooooo02_oo`03003ooooo033oo`00D?oo00@00?ooool000Soo`03003ooooo02So
o`<002Woo`<000Woo`03003ooooo02Soo`04003ooooo000bool0057oo`8000Woo`@002Woo`03003o
oooo02Woo`03003ooooo00Ooo`@002Soo`8003?oo`00oooo8Ooo003jool40002ool40002ool40000
0ooo003oo`03ool00`00ooooo`05ool40004ool00?Woo`06003ooooo003oo`001ooo00<00?ooool0
0ooo00@00?ooool000Woo`03003ooooo00Coo`008Oood@001ooo00L00?ooool00?oo003oo`040003
ool00`00ooooo`03ool00`00ool00009ool00`00ooooo`04ool002Koo`03003ooooo02ooo`03003o
oooo02[oo`05003oooooool00003ool00`00ooooo`0[ool00`00ooooo`0^ool00`00ooooo`0:ool0
0`00ooooo`02ool00`00ooooo`07ool00`00ooooo`02ool00`00ool00009ool00`00ooooo`04ool0
02Koo`03003ooooo05coo`03003ooooo00Goo`03003ooooo06[oo`05003oooooool0000:ool00`00
ooooo`050007ool40004ool002Koo`03003ooooo05coo`03003ooooo00Goo`03003ooooo07Koo`04
003ooooo000?ool00`00ooooo`04ool002Koo`8005goo`03003ooooo00Goo`03003ooooo07Ooo`80
017oo`8000Coo`009_oo00<00?ooool0G?oo00<00?ooool01Ooo00<00?ooool0T?oo000Vool00`00
ooooo`1Lool00`00ooooo`05ool00`00ooooo`2@ool002Koo`03003ooooo05coo`03003ooooo00Go
o`03003ooooo093oo`009_oo0P00GOoo00<00?ooool01Ooo00<00?ooool0T?oo000Vool00`00oooo
o`1Lool00`00ooooo`05ool00`00ooooo`2@ool002Koo`03003ooooo05coo`03003ooooo00Goo`03
003ooooo093oo`009_oo00<00?ooool0G?oo00<00?ooool01Ooo00<00?ooool0T?oo000Vool2001M
ool00`00ooooo`05ool00`00ooooo`2@ool002Koo`03003ooooo05coo`03003ooooo00Goo`03003o
oooo093oo`009_oo00<00?ooool0G?oo00<00?ooool01Ooo00<00?ooool0T?oo0007ool40003ool2
0004ool20004ool20004ool20004ool00`00ooooo`1Lool00`00ooooo`05ool00`00ooooo`2@ool0
00Ooo`03003ooooo00?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003o
oooo0003ool00`00ooooo`1Lool00`00ooooo`05ool00`00ooooo`2@ool000Soo`03003ooooo00;o
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool2001Nool0
0`00ooooo`03ool00`00ooooo`2Aool000Woo`05003oooooool00002ool01000ooooo`000_oo00@0
0?ooool000;oo`04003ooooo0002ool01@00oooooooo0000Gooo00<00?ooool00ooo00<00?ooool0
TOoo000:ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00D00?ooooooo`0005ooo`03003ooooo00?oo`03003ooooo097oo`001ooo00@00?ooool000;oo`04
003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`1M
ool00`00ooooo`03ool00`00ooooo`2Aool000Soo`8000Coo`8000Coo`8000Coo`8000Coo`8000Co
o`8005koo`03003ooooo00?oo`03003ooooo097oo`009_oo00<00?ooool0GOoo00<00?ooool00ooo
00<00?ooool0TOoo000Vool00`00ooooo`1Mool00`00ooooo`03ool00`00ooooo`2Aool002Koo`03
003ooooo05goo`03003ooooo00?oo`03003ooooo097oo`009_oo0P00G_oo00<00?ooool00ooo00<0
0?ooool0TOoo000Vool00`00ooooo`1Mool00`00ooooo`03ool00`00ooooo`2Aool002Koo`03003o
oooo05goo`03003ooooo00?oo`03003ooooo097oo`009_oo00<00?ooool0GOoo00<00?ooool00ooo
00<00?ooool0TOoo000Vool00`00ooooo`1Mool00`00ooooo`03ool00`00ooooo`2Aool002Koo`80
05koo`03003ooooo00?oo`03003ooooo097oo`009_oo00<00?ooool0GOoo00<00?ooool00ooo00<0
0?ooool0TOoo0009ool00`00ooooo`02ool20004ool20004ool20004ool20004ool00`00ooooo`1M
ool00`00ooooo`03ool00`00ooooo`2Aool000Woo`05003oooooool00002ool01000ooooo`000_oo
00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo0000Gooo00<00?ooool00ooo00<00?oo
ool0TOoo0006ool50002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`000ooo0P00G_oo00<00?ooool00ooo00<00?ooool0TOoo0006ool01000ooooo`000ooo00@0
0?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`03003ooooo05go
o`03003ooooo00?oo`03003ooooo097oo`001ooo00<00?oo00000ooo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`000_oo00@00?ooool000?oo`03003ooooo05goo`03003ooooo00?oo`03
003ooooo097oo`002?oo0P000ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000?oo`03003ooooo05goo`03003ooooo00?oo`03003ooooo097oo`002Ooo00<00?oo
ool00_oo0P001?oo0P001?oo0P001?oo0P001?oo0P00G_oo00<00?ooool00ooo00<00?ooool0TOoo
000Vool00`00ooooo`1Mool00`00ooooo`03ool00`00ooooo`2Aool002Koo`03003ooooo05goo`03
003ooooo00?oo`03003ooooo097oo`009_oo00<00?ooool0GOoo00<00?ooool00ooo00<00?ooool0
TOoo000Vool2001Nool00`00ooooo`03ool00`00ooooo`2Aool002Koo`03003ooooo05goo`03003o
oooo00?oo`03003ooooo097oo`009_oo00<00?ooool0GOoo00<00?ooool00ooo00<00?ooool0TOoo
000Vool00`00ooooo`1Mool00`00ooooo`03ool00`00ooooo`2Aool002Koo`03003ooooo05goo`03
003ooooo00?oo`03003ooooo097oo`009_oo0P00G_oo00<00?ooool00ooo00<00?ooool0TOoo000V
ool00`00ooooo`1Mool00`00ooooo`03ool00`00ooooo`2Aool000Soo`8000Coo`8000Coo`8000Co
o`8000Coo`8000Coo`03003ooooo05koo`05003oooooool0002Dool000Ooo`04003ooooo0002ool0
1000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?ooool0
G_oo00D00?ooooooo`0009Coo`001ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0003ool2001Oool01@00oooooooo0000U?oo0007ool30003
ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?oo
ool0G_oo00D00?ooooooo`0009Coo`001ooo00<00?ooool00ooo00@00?ooool000;oo`04003ooooo
0002ool01000ooooo`000_oo00@00?ooool000?oo`03003ooooo05koo`05003oooooool0002Dool0
00Ooo`03003ooooo00?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003o
oooo0003ool00`00ooooo`1Nool01@00oooooooo0000U?oo0008ool30003ool20004ool20004ool2
0004ool20004ool2001Oool01@00oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo
0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00
oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo000Vool2001Oool01@00
oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo000Vool00`00ooooo`1N
ool01@00oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo000Vool2001O
ool01@00oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo0008ool20004
ool20004ool20004ool20004ool20004ool00`00ooooo`1Nool01@00oooooooo0000U?oo0007ool0
1000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool0
00?oo`03003ooooo05koo`05003oooooool0002Dool000Ooo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo0P00Gooo00D00?ooooooo`00
09Coo`002?oo0P000ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?oo
ool000?oo`03003ooooo05koo`05003oooooool0002Dool000Ooo`04003ooooo0002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?ooool0G_oo00D0
0?ooooooo`0009Coo`001ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@0
0?ooool000;oo`04003ooooo0003ool00`00ooooo`1Nool01@00oooooooo0000U?oo0008ool20004
ool20004ool20004ool20004ool20004ool2001Oool01@00oooooooo0000U?oo000Vool00`00oooo
o`1Nool01@00oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo000Vool0
0`00ooooo`1Nool01@00oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo
000Vool2001Oool01@00oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo
000Vool00`00ooooo`1Nool01@00oooooooo0000U?oo000Vool00`00ooooo`1Nool01@00oooooooo
0000U?oo000Vool2001Pool00`00ool0002Eool002Koo`03003ooooo05ooo`03003oo`0009Goo`00
00?oo`0000000P000ooo0P001?oo0P001?oo0P001?oo0P001?oo0P001?oo00<00?ooool0Gooo00<0
0?oo0000UOoo0003ool01@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000;oo`05003oooooool0001Qool00`00ool0002Eool000?oo`05003o
ooooool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooooooo`80063oo`03003oo`0009Goo`000ooo00D00?ooooooo`0000;oo`04003ooooo
0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo0000HOoo
00<00?oo0000UOoo0003ool01@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo0002ool0
1000ooooo`000_oo00@00?ooool000;oo`05003oooooool0001Qool00`00ool0002Eool00004ool0
00000003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000?oo`03003ooooo05ooo`03003oo`0009Goo`000ooo00<00?ooool00_oo0P001?oo
0P001?oo0P001?oo0P001?oo0P001?oo00<00?ooool0Gooo00<00?oo0000UOoo000Vool2001Pool0
0`00ool0002Eool002Koo`03003ooooo05ooo`03003oo`0009Goo`009_oo00<00?ooool0Gooo00<0
0?oo0000UOoo000Vool00`00ooooo`1Oool00`00ool0002Eool002Koo`80063oo`03003oo`0009Go
o`009_oo00<00?ooool0Gooo00<00?oo0000UOoo000Vool00`00ooooo`1Oool00`00ool0002Eool0
02Koo`03003ooooo05ooo`03003oo`0009Goo`009_oo0P00H?oo00<00?oo0000UOoo000Vool00`00
ooooo`1Oool00`00ool0002Eool00003ool00000008000;oo`@000?oo`8000Coo`8000Coo`8000Co
o`8000Coo`03003ooooo05ooo`03003oo`0009Goo`000ooo00D00?ooooooo`0000Goo`04003ooooo
0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`1Oool00`00
ool0002Eool000?oo`03003ooooo00;oo`03003ooooo00;oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0003ool2001Pool00`00ool0002Eool000?oo`03003ooooo
00?oo`05003oooooool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool0
1@00oooooooo0000HOoo00<00?oo0000UOoo0003ool00`00ooooo`04ool01000ooooo`000_oo00@0
0?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00D00?ooooooo`00067oo`03003oo`00
09Goo`0000Coo`00000000?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04
003ooooo0002ool01000ooooo`000ooo00<00?ooool0Gooo00<00?oo0000UOoo0003ool00`00oooo
o`02ool20004ool20004ool20004ool20004ool20004ool00`00ooooo`1Oool00`00ool0002Eool0
02Koo`80063oo`03003oo`0009Goo`009_oo00<00?ooool0Gooo00<00?oo0000UOoo000Vool00`00
ooooo`1Oool00`00ool0002Eool002Koo`03003ooooo05ooo`03003oo`0009Goo`009_oo0P00H?oo
00<00?oo0000UOoo000Vool00`00ooooo`1Oool00`00ool0002Eool002Koo`03003ooooo063oo`03
003ooooo09Coo`009_oo00<00?ooool0H?oo00<00?ooool0U?oo000Vool2001Qool00`00ooooo`2D
ool002Koo`03003ooooo063oo`03003ooooo09Coo`009_oo00<00?ooool0H?oo00<00?ooool0U?oo
00000ooo000000020004ool00`00ooooo`02ool20004ool20004ool20004ool20004ool00`00oooo
o`1Pool00`00ooooo`2Dool000?oo`03003ooooo00?oo`05003oooooool00002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo0000H_oo00<00?ooool0U?oo0003
ool00`00ooooo`050002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`000ooo0P00HOoo00<00?ooool0U?oo0003ool01000ooooo`000_oo00D00?ooooooo`0000;o
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`05003oooooool0001Rool00`00
ooooo`2Dool000?oo`07003oooooool00?oo00000ooo00@00?ooool000;oo`04003ooooo0002ool0
1000ooooo`000_oo00@00?ooool000?oo`03003ooooo063oo`03003ooooo09Coo`0000Coo`000000
00Coo`8000?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003
ool00`00ooooo`1Pool00`00ooooo`2Dool000?oo`03003ooooo00?oo`03003ooooo00;oo`8000Co
o`8000Coo`8000Coo`8000Coo`80067oo`03003ooooo09Coo`009_oo00<00?ooool0H?oo00<00?oo
ool0U?oo000Vool00`00ooooo`1Pool00`00ooooo`2Dool002Koo`03003ooooo063oo`03003ooooo
09Coo`009_oo00<00?ooool0H?oo00<00?ooool0U?oo003ooolQool00?ooob7oo`00oooo8Ooo003o
oolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo0000
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.389463, -34920.8, \
0.010089, 1166.02}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"samp", "=", 
        RowBox[{"sample", "[", 
          RowBox[{"Sin", ",", "nn", ",", "\[Tau]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"dps", "=", 
        RowBox[{"discretePowerSpec", "[", 
          RowBox[{"samp", ",", "\[Tau]"}], "]"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"dps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "1000"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 -1.51788e-17 0.000618034 [
[.2619 -0.0125 -9 -9 ]
[.2619 -0.0125 9 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -9 -9 ]
[.7381 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -24 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 0 m
.2619 .00625 L
s
[(0.5)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(1)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(1.5)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(2)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .12361 m
.03006 .12361 L
s
[(200)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(400)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(600)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(800)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(1000)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 0 m
.04286 0 L
.0619 0 L
.08095 0 L
.1 0 L
.11905 0 L
.1381 0 L
.15714 0 L
.17619 0 L
.19524 0 L
.21429 0 L
.23333 0 L
.25238 0 L
.27143 0 L
.29048 0 L
.30952 0 L
.32857 0 L
.34762 0 L
.36667 0 L
.38571 0 L
.40476 0 L
.42381 0 L
.44286 0 L
.4619 0 L
.48095 0 L
.5 .60675 L
.51905 0 L
.5381 0 L
.55714 0 L
.57619 0 L
.59524 0 L
.61429 0 L
.63333 0 L
.65238 0 L
.67143 0 L
.69048 0 L
.70952 0 L
.72857 0 L
.74762 0 L
.76667 0 L
.78571 0 L
.80476 0 L
.82381 0 L
.84286 0 L
.8619 0 L
.88095 0 L
.9 0 L
.91905 0 L
.9381 0 L
.95714 0 L
Mistroke
.97619 0 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool004Soo`8000Coo`03003ooooo00;o
o`<002_oo`@002[oo`@000?oo`03003ooooo00;oo`<002coo`@003;oo`00Aooo00@00?ooool000_o
o`03003ooooo02[oo`03003ooooo02_oo`03003ooooo00[oo`03003ooooo02Woo`03003ooooo03?o
o`00Aooo00@00?ooool000_oo`03003ooooo02[oo`03003ooooo02_oo`03003ooooo00[oo`03003o
oooo02[oo`03003ooooo03;oo`00Aooo00@00?ooool000Soo`<002goo`03003ooooo02_oo`03003o
oooo00Ooo`<002koo`03003ooooo037oo`00Aooo00@00?ooool000Soo`03003ooooo02goo`03003o
oooo02_oo`03003ooooo00Ooo`03003ooooo02ooo`03003ooooo033oo`00Aooo00@00?ooool000So
o`03003ooooo02_oo`<002_oo`<000Woo`03003ooooo02coo`04003ooooo000bool004Soo`8000Wo
o`@002coo`03003ooooo02_oo`03003ooooo00Ooo`@002coo`8003?oo`00oooo8Ooo003jool40002
ool40002ool400000ooo003oo`03ool00`00ooooo`05ool40004ool00?Woo`06003ooooo003oo`00
1ooo00<00?ooool00ooo00@00?ooool000Woo`03003ooooo00Coo`005Ooog@001ooo00L00?ooool0
0?oo003oo`040003ool00`00ooooo`03ool00`00ool00009ool00`00ooooo`04ool001[oo`03003o
oooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003o
oooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00?oo`05003o
ooooool00003ool00`00ooooo`03ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool0
0`00ooooo`07ool00`00ooooo`08ool00`00ooooo`07ool00`00ooooo`08ool00`00ooooo`07ool0
0`00ooooo`08ool00`00ooooo`0:ool00`00ooooo`02ool00`00ooooo`07ool00`00ooooo`02ool0
0`00ool00009ool00`00ooooo`04ool001[oo`03003ooooo06;oo`03003ooooo00Goo`03003ooooo
073oo`05003oooooool0000:ool00`00ooooo`050007ool40004ool001[oo`03003ooooo06;oo`03
003ooooo00Goo`03003ooooo07coo`04003ooooo000?ool00`00ooooo`04ool001[oo`03003ooooo
06;oo`03003ooooo00Goo`03003ooooo07goo`80017oo`8000Coo`006_oo00<00?ooool0H_oo00<0
0?ooool01Ooo00<00?ooool0U_oo000Jool2001Sool00`00ooooo`05ool00`00ooooo`2Fool001[o
o`03003ooooo06;oo`03003ooooo00Goo`03003ooooo09Koo`006_oo00<00?ooool0H_oo00<00?oo
ool01Ooo00<00?ooool0U_oo000Jool00`00ooooo`1Rool00`00ooooo`05ool00`00ooooo`2Fool0
01[oo`03003ooooo06;oo`03003ooooo00Goo`03003ooooo09Koo`006_oo00<00?ooool0H_oo00<0
0?ooool01Ooo00<00?ooool0U_oo000Jool00`00ooooo`1Rool00`00ooooo`05ool00`00ooooo`2F
ool001[oo`8006?oo`03003ooooo00Goo`03003ooooo09Koo`006_oo00<00?ooool0H_oo00<00?oo
ool01Ooo00<00?ooool0U_oo000Jool00`00ooooo`1Rool00`00ooooo`05ool00`00ooooo`2Fool0
01[oo`03003ooooo06;oo`03003ooooo00Goo`03003ooooo09Koo`006_oo00<00?ooool0Hooo00<0
0?ooool00ooo00<00?ooool0Uooo000Jool00`00ooooo`1Sool00`00ooooo`03ool00`00ooooo`2G
ool001[oo`03003ooooo06?oo`03003ooooo00?oo`03003ooooo09Ooo`006_oo0P00I?oo00<00?oo
ool00ooo00<00?ooool0Uooo000Jool00`00ooooo`1Sool00`00ooooo`03ool00`00ooooo`2Gool0
01[oo`03003ooooo06?oo`03003ooooo00?oo`03003ooooo09Ooo`006_oo00<00?ooool0Hooo00<0
0?ooool00ooo00<00?ooool0Uooo000Jool00`00ooooo`1Sool00`00ooooo`03ool00`00ooooo`2G
ool000Ooo`@000?oo`8000Coo`8000Coo`03003ooooo06?oo`03003ooooo00?oo`03003ooooo09Oo
o`001ooo00<00?ooool00ooo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`1Sool00`00
ooooo`03ool00`00ooooo`2Gool000Soo`03003ooooo00;oo`04003ooooo0002ool01000ooooo`00
0ooo0`00Hooo00<00?ooool00ooo00<00?ooool0Uooo0009ool01@00oooooooo00000_oo00@00?oo
ool000;oo`05003oooooool0001Uool00`00ooooo`03ool00`00ooooo`2Gool000[oo`04003ooooo
0002ool01000ooooo`000_oo00D00?ooooooo`0006Goo`03003ooooo00?oo`03003ooooo09Ooo`00
1ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?ooool0Hooo00<00?oo
ool00ooo00<00?ooool0Uooo0008ool20004ool20004ool20004ool00`00ooooo`1Sool00`00oooo
o`03ool00`00ooooo`2Gool001[oo`03003ooooo06?oo`03003ooooo00?oo`03003ooooo09Ooo`00
6_oo00<00?ooool0Hooo00<00?ooool00ooo00<00?ooool0Uooo000Jool2001Tool00`00ooooo`03
ool00`00ooooo`2Gool001[oo`03003ooooo06?oo`03003ooooo00?oo`03003ooooo09Ooo`006_oo
00<00?ooool0Hooo00<00?ooool00ooo00<00?ooool0Uooo000Jool00`00ooooo`1Sool00`00oooo
o`03ool00`00ooooo`2Gool001[oo`03003ooooo06?oo`03003ooooo00?oo`03003ooooo09Ooo`00
6_oo00<00?ooool0Hooo00<00?ooool00ooo00<00?ooool0Uooo000Jool2001Tool00`00ooooo`03
ool00`00ooooo`2Gool001[oo`03003ooooo06?oo`03003ooooo00?oo`03003ooooo09Ooo`006_oo
00<00?ooool0Hooo00<00?ooool00ooo00<00?ooool0Uooo000Jool00`00ooooo`1Sool00`00oooo
o`03ool00`00ooooo`2Gool001[oo`03003ooooo06?oo`03003ooooo00?oo`03003ooooo09Ooo`00
6_oo00<00?ooool0Hooo00<00?ooool00ooo00<00?ooool0Uooo000Jool00`00ooooo`1Sool00`00
ooooo`03ool00`00ooooo`2Gool001[oo`8006Coo`03003ooooo00?oo`03003ooooo09Ooo`006_oo
00<00?ooool0Hooo00<00?ooool00ooo00<00?ooool0Uooo000Jool00`00ooooo`1Sool00`00oooo
o`03ool00`00ooooo`2Gool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[oo`03003o
oooo06Coo`05003oooooool0002Jool000Woo`03003ooooo00;oo`8000Coo`8000Coo`03003ooooo
06Coo`05003oooooool0002Jool000Woo`05003oooooool00002ool01000ooooo`000_oo00D00?oo
ooooo`0006Koo`05003oooooool0002Jool000Koo`D000;oo`04003ooooo0002ool01000ooooo`00
0ooo0`00I?oo00D00?ooooooo`0009[oo`001_oo00@00?ooool000?oo`04003ooooo0002ool01000
ooooo`000ooo00<00?ooool0I?oo00D00?ooooooo`0009[oo`001ooo00<00?oo00000ooo00@00?oo
ool000;oo`04003ooooo0003ool00`00ooooo`1Tool01@00oooooooo0000V_oo0008ool20003ool0
1000ooooo`000_oo00@00?ooool000?oo`03003ooooo06Coo`05003oooooool0002Jool000Woo`03
003ooooo00;oo`8000Coo`8000Coo`03003ooooo06Coo`05003oooooool0002Jool001[oo`03003o
oooo06Coo`05003oooooool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[o
o`8006Goo`05003oooooool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[o
o`03003ooooo06Coo`05003oooooool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002J
ool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[oo`03003ooooo06Coo`05003ooooo
ool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[oo`8006Goo`05003ooooo
ool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[oo`03003ooooo06Coo`05
003oooooool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[oo`03003ooooo
06Coo`05003oooooool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[oo`80
06Goo`05003oooooool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002Jool001[oo`03
003ooooo06Coo`05003oooooool0002Jool001[oo`03003ooooo06Coo`05003oooooool0002Jool0
01[oo`03003ooooo06Coo`05003oooooool0002Jool000Soo`8000Coo`8000Coo`8000Coo`03003o
oooo06Coo`05003oooooool0002Jool000Ooo`04003ooooo0002ool01000ooooo`000_oo00@00?oo
ool000?oo`03003ooooo06Coo`05003oooooool0002Jool000Ooo`04003ooooo0002ool01000oooo
o`000_oo00@00?ooool000?oo`<006Coo`05003oooooool0002Jool000Ooo`<000?oo`04003ooooo
0002ool01000ooooo`000ooo00<00?ooool0I?oo00D00?ooooooo`0009[oo`001ooo00<00?ooool0
0ooo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`1Tool01@00oooooooo0000V_oo0007
ool00`00ooooo`03ool01000ooooo`000_oo00@00?ooool000?oo`03003ooooo06Goo`03003oo`00
09_oo`002?oo0`000ooo0P001?oo0P001?oo00<00?ooool0IOoo00<00?oo0000Vooo000Jool00`00
ooooo`1Uool00`00ool0002Kool001[oo`03003ooooo06Goo`03003oo`0009_oo`006_oo0P00I_oo
00<00?oo0000Vooo000Jool00`00ooooo`1Uool00`00ool0002Kool001[oo`03003ooooo06Goo`03
003oo`0009_oo`006_oo00<00?ooool0IOoo00<00?oo0000Vooo000Jool00`00ooooo`1Uool00`00
ool0002Kool001[oo`03003ooooo06Goo`03003oo`0009_oo`006_oo00<00?ooool0IOoo00<00?oo
0000Vooo000Jool2001Vool00`00ool0002Kool001[oo`03003ooooo06Goo`03003oo`0009_oo`00
6_oo00<00?ooool0IOoo00<00?oo0000Vooo000Jool00`00ooooo`1Uool00`00ool0002Kool001[o
o`03003ooooo06Goo`03003oo`0009_oo`006_oo00<00?ooool0IOoo00<00?oo0000Vooo000Jool0
0`00ooooo`1Uool00`00ool0002Kool001[oo`8006Koo`03003oo`0009_oo`006_oo00<00?ooool0
IOoo00<00?oo0000Vooo000Jool00`00ooooo`1Uool00`00ool0002Kool001[oo`03003ooooo06Go
o`03003oo`0009_oo`002?oo0P001?oo0P001?oo0P001?oo00<00?ooool0IOoo00<00?oo0000Vooo
0007ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`1Uool00`00
ool0002Kool000Ooo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`<006Goo`03
003oo`0009_oo`002?oo0P000ooo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`1Uool0
0`00ool0002Kool000Ooo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`03003o
oooo06Goo`03003oo`0009_oo`001ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0ooo00<00?ooool0IOoo00<00?oo0000Vooo0008ool20004ool20004ool20004ool00`00ooooo`1U
ool00`00ool0002Kool001[oo`03003ooooo06Goo`03003oo`0009_oo`006_oo00<00?ooool0IOoo
00<00?oo0000Vooo000Jool2001Vool00`00ool0002Kool001[oo`03003ooooo06Goo`03003oo`00
09_oo`006_oo00<00?ooool0I_oo00<00?ooool0V_oo000Jool00`00ooooo`1Vool00`00ooooo`2J
ool001[oo`03003ooooo06Koo`03003ooooo09[oo`006_oo00<00?ooool0I_oo00<00?ooool0V_oo
000Jool00`00ooooo`1Vool00`00ooooo`2Jool001[oo`8006Ooo`03003ooooo09[oo`006_oo00<0
0?ooool0I_oo00<00?ooool0V_oo000Jool00`00ooooo`1Vool00`00ooooo`2Jool001[oo`03003o
oooo06Koo`03003ooooo09[oo`006_oo00<00?ooool0I_oo00<00?ooool0V_oo000Jool00`00oooo
o`1Vool00`00ooooo`2Jool001[oo`03003ooooo06Koo`03003ooooo09[oo`006_oo0P00Iooo00<0
0?ooool0V_oo000Jool00`00ooooo`1Vool00`00ooooo`2Jool001[oo`03003ooooo06Koo`03003o
oooo09[oo`006_oo00<00?ooool0I_oo00<00?ooool0V_oo000Jool00`00ooooo`1Vool00`00oooo
o`2Jool00003ool00000008000?oo`8000Coo`8000Coo`8000Coo`03003ooooo0?ooo`Coo`000ooo
00D00?ooooooo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00D00?ooooooo`000?ooo`Ko
o`000ooo00D00?ooooooo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooooooo`<0
0?ooo`Coo`000ooo00D00?ooooooo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00<00?oo
ool0oooo2?oo0003ool01@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo0002ool00`00
ooooo`3oool8ool00004ool000000003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo
003oool:ool000?oo`03003ooooo00;oo`8000Coo`8000Coo`800?ooo`_oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.255365, -177.367, \
0.00954606, 7.35516}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "]"}], ",", 
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
                "]"}]}], "}"}], "&"}], "/@", "dps"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Export", "[", 
        RowBox[{"\"\<tmp.txt\>\"", ",", 
          RowBox[{"ToString", "[", "%", "]"}]}], "]"}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["sin sampled 100 times during one period", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Tau]", "=", 
        RowBox[{"0.01", "\[Times]", "2", "\[Pi]"}]}], ";", 
      RowBox[{"nn", "=", "100"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ps", "=", 
        RowBox[{"powerSpec", "[", 
          RowBox[{"Sin", ",", "nn", ",", "\[Tau]", ",", "1000."}], "]"}]}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"ps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "150000"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 -2.70508e-17 4.12023e-06 [
[.21429 -0.0125 -6 -9 ]
[.21429 -0.0125 6 0 ]
[.40476 -0.0125 -6 -9 ]
[.40476 -0.0125 6 0 ]
[.59524 -0.0125 -6 -9 ]
[.59524 -0.0125 6 0 ]
[.78571 -0.0125 -6 -9 ]
[.78571 -0.0125 6 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .0824 -30 -4.5 ]
[.01131 .0824 0 4.5 ]
[.01131 .16481 -30 -4.5 ]
[.01131 .16481 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .32962 -30 -4.5 ]
[.01131 .32962 0 4.5 ]
[.01131 .41202 -36 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .49443 -36 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .57683 -36 -4.5 ]
[.01131 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 0 m
.21429 .00625 L
s
[(10)] .21429 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(20)] .40476 -0.0125 0 1 Mshowa
.59524 0 m
.59524 .00625 L
s
[(30)] .59524 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(40)] .78571 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(50)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.0619 0 m
.0619 .00375 L
s
.1 0 m
.1 .00375 L
s
.1381 0 m
.1381 .00375 L
s
.17619 0 m
.17619 .00375 L
s
.25238 0 m
.25238 .00375 L
s
.29048 0 m
.29048 .00375 L
s
.32857 0 m
.32857 .00375 L
s
.36667 0 m
.36667 .00375 L
s
.44286 0 m
.44286 .00375 L
s
.48095 0 m
.48095 .00375 L
s
.51905 0 m
.51905 .00375 L
s
.55714 0 m
.55714 .00375 L
s
.63333 0 m
.63333 .00375 L
s
.67143 0 m
.67143 .00375 L
s
.70952 0 m
.70952 .00375 L
s
.74762 0 m
.74762 .00375 L
s
.82381 0 m
.82381 .00375 L
s
.8619 0 m
.8619 .00375 L
s
.9 0 m
.9 .00375 L
s
.9381 0 m
.9381 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .0824 m
.03006 .0824 L
s
[(20000)] .01131 .0824 1 0 Mshowa
.02381 .16481 m
.03006 .16481 L
s
[(40000)] .01131 .16481 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(60000)] .01131 .24721 1 0 Mshowa
.02381 .32962 m
.03006 .32962 L
s
[(80000)] .01131 .32962 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(100000)] .01131 .41202 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(120000)] .01131 .49443 1 0 Mshowa
.02381 .57683 m
.03006 .57683 L
s
[(140000)] .01131 .57683 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 0 m
0 0 rlineto s
.02381 0 m
.04178 .61803 L
s
.04394 .61803 m
.0619 0 L
.08095 0 L
.1 0 L
.11905 0 L
.1381 0 L
.15714 0 L
.17619 0 L
.19524 0 L
.21429 0 L
.23333 0 L
.25238 0 L
.27143 0 L
.29048 0 L
.30952 0 L
.32857 0 L
.34762 0 L
.36667 0 L
.38571 0 L
.40476 0 L
.42381 0 L
.44286 0 L
.4619 0 L
.48095 0 L
.5 0 L
.51905 0 L
.5381 0 L
.55714 0 L
.57619 0 L
.59524 0 L
.61429 0 L
.63333 0 L
.65238 0 L
.67143 0 L
.69048 0 L
.70952 0 L
.72857 0 L
.74762 0 L
.76667 0 L
.78571 0 L
.80476 0 L
.82381 0 L
.84286 0 L
.8619 0 L
.88095 0 L
.9 0 L
.91905 0 L
.9381 0 L
.95714 0 L
.97619 0 L
Mistroke
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool004Woo`@000?oo`8001koo`@000?oo`80023oo`8000Co
o`80027oo`03003ooooo00;oo`8001koo`<000Coo`80033oo`00Booo00D00?ooooooo`0000;oo`03
003ooooo01_oo`03003ooooo00?oo`04003ooooo000Nool01000ooooo`000_oo00@00?ooool0023o
o`05003oooooool00002ool00`00ooooo`0Nool01000ooooo`000_oo00<00?ooool0;Ooo001;ool0
1@00oooooooo00000_oo00<00?ooool07?oo00<00?ooool00_oo00@00?ooool0027oo`04003ooooo
0002ool00`00ooooo`0Kool50002ool01000ooooo`008?oo00@00?ooool000;oo`03003ooooo02go
o`00Booo00D00?ooooooo`0000;oo`03003ooooo01goo`05003oooooool00002ool00`00ooooo`0M
ool20003ool01000ooooo`007Ooo00@00?ooool000?oo`04003ooooo000Mool30003ool01000oooo
o`00;ooo001;ool01@00oooooooo00000_oo00<00?ooool07_oo00@00?ooool000;oo`03003ooooo
01ooo`04003ooooo0002ool00`00ooooo`0Lool00`00ool00003ool01000ooooo`007Ooo00<00?oo
ool00ooo00@00?ooool002ooo`00BOoo0`000ooo00@00?ooool001goo`04003ooooo0002ool01000
ooooo`007_oo00@00?ooool000;oo`04003ooooo000Oool20003ool01000ooooo`007Ooo00<00?oo
ool00ooo00@00?ooool002ooo`00Booo00<00?ooool00_oo0P007ooo0P001?oo0P008?oo0P001?oo
0P008Ooo00<00?ooool00_oo0P007_oo10000ooo0P00<?oo003ooolQool00?[oo`@000;oo`@000;o
o`@00003ool00?oo00?oo`03003ooooo00Goo`@000Coo`009_oo00<00?ooool0d?oo00H00?ooool0
0?oo0007ool00`00ooooo`03ool01000ooooo`002Ooo00<00?ooool01?oo000QoooA0007ool01`00
ooooo`00ool00?oo00@000?oo`03003ooooo00?oo`03003oo`0000Woo`03003ooooo00Coo`009_oo
00<00?ooool01Ooo00<00?ooool07Ooo00<00?ooool09?oo00<00?ooool09Ooo00<00?ooool09Ooo
00<00?ooool09?oo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool01ooo00<00?ooool00_oo
00<00?oo00002Ooo00<00?ooool01?oo000Vool00`00ooooo`05ool00`00ooooo`39ool01@00oooo
oooo00002_oo00<00?ooool01@001ooo10001?oo000Vool00`00ooooo`05ool00`00ooooo`3Eool0
1000ooooo`003ooo00<00?ooool01?oo000Vool20006ool00`00ooooo`3Fool2000Aool20004ool0
02Koo`03003ooooo00Goo`03003ooooo0>ooo`009_oo00<00?ooool01Ooo00<00?ooool0kooo000V
ool00`00ooooo`05ool00`00ooooo`3_ool002Koo`8000Koo`03003ooooo0>ooo`009_oo00<00?oo
ool01Ooo00<00?ooool0kooo000Vool00`00ooooo`05ool00`00ooooo`3_ool002Koo`03003ooooo
00Goo`03003ooooo0>ooo`009_oo0P001_oo00<00?ooool0kooo000Vool00`00ooooo`05ool00`00
ooooo`3_ool002Koo`03003ooooo00Goo`03003ooooo0>ooo`001ooo10000ooo0P001?oo0P001?oo
0P001?oo0P001?oo00<00?ooool01Ooo00<00?ooool0kooo0007ool00`00ooooo`03ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?ooool01Ooo00<0
0?ooool0kooo0008ool00`00ooooo`02ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo
0002ool01000ooooo`000ooo0P001Ooo00<00?ooool0l?oo0009ool01@00oooooooo00000_oo00@0
0?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooooooo`8000Goo`03003ooooo
0?3oo`002_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;o
o`04003oooooool20005ool00`00ooooo`3`ool000Ooo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo0P001Ooo00<00?ooool0l?oo0008
ool20004ool20004ool20004ool20004ool20004ool20005ool00`00ooooo`3`ool002Koo`8000Go
o`03003ooooo0?3oo`009_oo0P001Ooo00<00?ooool0l?oo000Vool20005ool00`00ooooo`3`ool0
02Koo`8000Goo`03003ooooo0?3oo`009_oo0P001Ooo00<00?ooool0l?oo000Vool20005ool00`00
ooooo`3`ool002Koo`8000Goo`03003ooooo0?3oo`009_oo0P001Ooo00<00?ooool0l?oo000Vool2
0005ool00`00ooooo`3`ool002Koo`8000Goo`03003ooooo0?3oo`002Ooo00<00?ooool00_oo0P00
1?oo0P001?oo0P001?oo0P001?oo0P001Ooo00<00?ooool0l?oo0009ool01@00oooooooo00000_oo
00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooooooo`8000Goo`03003o
oooo0?3oo`001_oo1@000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@0
0?ooool000?oo`8000Goo`03003ooooo0?3oo`001_oo00@00?ooool000?oo`04003ooooo0002ool0
1000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool20005ool00`00ooooo`3`ool000Oo
o`03003oo`0000?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo
0003ool20005ool00`00ooooo`3`ool000Soo`8000?oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000;oo`04003ooooo0003ool20005ool00`00ooooo`3`ool000Woo`03003ooooo00;o
o`8000Coo`8000Coo`8000Coo`8000Coo`8000Goo`03003ooooo0?3oo`009_oo0P001Ooo00<00?oo
ool0l?oo000Vool20005ool00`00ooooo`3`ool002Koo`8000Goo`03003ooooo0?3oo`009_oo0P00
1Ooo00<00?ooool0l?oo000Vool20005ool00`00ooooo`3`ool002Koo`8000Goo`03003ooooo0?3o
o`009_oo0P001Ooo00<00?ooool0l?oo000Vool20005ool00`00ooooo`3`ool002Koo`8000Goo`03
003ooooo0?3oo`009_oo0P001Ooo00<00?ooool0l?oo0008ool20004ool20004ool20004ool20004
ool20004ool00`00ool00003ool00`00ooooo`3aool000Ooo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?oo00000ooo00<00?oo
ool0lOoo0007ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000?oo`<000?oo`03003ooooo0?7oo`001ooo0`000ooo00@00?ooool000;oo`04
003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`03003oo`0000?oo`03003ooooo0?7o
o`001ooo00<00?ooool00ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@0
0?ooool000?oo`03003oo`0000?oo`03003ooooo0?7oo`001ooo00<00?ooool00ooo00@00?ooool0
00;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`03003oo`0000?oo`03003o
oooo0?7oo`002?oo0`000ooo0P001?oo0P001?oo0P001?oo0P001?oo0`000ooo00<00?ooool0lOoo
000Vool00`00ool00003ool00`00ooooo`3aool002Koo`03003oo`0000?oo`03003ooooo0?7oo`00
9_oo00<00?oo00000ooo00<00?ooool0lOoo000Vool00`00ool00003ool00`00ooooo`3aool002Ko
o`<000?oo`03003ooooo0?7oo`009_oo00<00?oo00000ooo00<00?ooool0lOoo000Vool00`00ool0
0003ool00`00ooooo`3aool002Koo`03003oo`0000?oo`03003ooooo0?7oo`009_oo0`000ooo00<0
0?ooool0lOoo000Vool00`00ool00003ool00`00ooooo`3aool000Soo`8000Coo`8000Coo`8000Co
o`8000Coo`8000Coo`03003oo`0000?oo`03003ooooo0?7oo`001ooo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00ool00003ool0
0`00ooooo`3aool000Ooo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`000ooo0`000ooo00<00?ooool0lOoo0008ool20003ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?oo00000ooo00<00?oo
ool0lOoo0007ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000?oo`03003oo`0000?oo`03003ooooo0?7oo`001ooo00@00?ooool000;oo`04
003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00ool00003
ool00`00ooooo`3aool000Soo`8000Coo`8000Coo`8000Coo`8000Coo`8000Coo`<000?oo`03003o
oooo0?7oo`009_oo00<00?oo00000ooo00<00?ooool0lOoo000Vool00`00ool00003ool00`00oooo
o`3aool002Koo`03003oo`0000?oo`03003ooooo0?7oo`009_oo00<00?oo00000ooo00<00?ooool0
lOoo000Vool30003ool00`00ooooo`3aool002Koo`03003oo`0000?oo`03003ooooo0?7oo`009_oo
00<00?oo00000ooo00<00?ooool0lOoo000Vool00`00ool00003ool00`00ooooo`3aool002Koo`80
0004ool00?oo003dool002Koo`06003ooooo003oo`00m?oo00000ooo000000020003ool20004ool2
0004ool20004ool20004ool20004ool01P00ooooo`00ool00?Coo`000ooo00D00?ooooooo`0000;o
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooo
oooo00000_oo00<00?oo0000m?oo0003ool01@00oooooooo00000_oo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003oooooool200001?oo003oo`00m?oo
0003ool01@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000;oo`05003oooooool00002ool00`00ool0003dool000?oo`05003oooooool00002
ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00D00?oo
ooooo`0000;oo`03003oo`000?Coo`0000Coo`00000000?oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00H00?ooool00?oo003dool0
00?oo`03003ooooo00;oo`8000Coo`8000Coo`8000Coo`8000Coo`8000Coo`06003ooooo003oo`00
m?oo000Vool200001?oo003oo`00m?oo000Vool01P00ooooo`00ool00?Coo`009_oo00H00?ooool0
0?oo003dool002Koo`06003ooooo003oo`00m?oo000Vool200001?oo003oo`00m?oo000Vool01P00
ooooo`00ool00?Coo`009_oo00H00?ooool00?oo003dool002Koo`06003ooooo003oo`00m?oo000V
ool200001?oo003oo`00m?oo000Vool01P00ooooo`00ool00?Coo`0000?oo`0000000P000_oo1000
0ooo0P001?oo0P001?oo0P001?oo0P001?oo00H00?ooool00?oo003dool000?oo`05003oooooool0
0005ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00H0
0?ooool00?oo003dool000?oo`03003ooooo00;oo`03003ooooo00;oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool200001?oo003oo`00m?oo0003ool00`00
ooooo`03ool01@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0_oo00D00?ooooooo`0000;oo`03003oo`000?Coo`000ooo00<00?ooool01?oo00@00?ooool000;o
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`05003oooooool00002ool00`00
ool0003dool00004ool000000003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002
ool01000ooooo`000_oo00@00?ooool000?oo`06003ooooo003oo`00m?oo0003ool00`00ooooo`02
ool20004ool20004ool20004ool20004ool20004ool01P00ooooo`00ool00?Coo`009_oo0P0000Co
o`00ool00?Coo`009_oo00H00?ooool00?oo003dool002Koo`06003ooooo003oo`00m?oo000Vool0
1P00ooooo`00ool00?Coo`009_oo0P0000Coo`00ool00?Coo`009_oo00H00?ooool00?oo003dool0
02Koo`05003oooooool0003eool002Koo`05003oooooool0003eool002Koo`8000;oo`03003ooooo
0??oo`009_oo00D00?ooooooo`000?Goo`009_oo00D00?ooooooo`000?Goo`0000?oo`0000000P00
1?oo00<00?ooool00_oo0P001?oo0P001?oo0P001?oo0P001?oo00D00?ooooooo`000?Goo`000ooo
00<00?ooool00ooo00D00?ooooooo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?oo
ool000;oo`05003oooooool00003ool00`00ooooo`3cool000?oo`03003ooooo00D000;oo`04003o
oooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool20002ool00`00oooo
o`3cool000?oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo
0002ool01000ooooo`000_oo00D00?ooooooo`0000?oo`03003ooooo0??oo`000ooo00L00?oooooo
o`00ool00003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`00
0ooo00D00?ooooooo`000?Goo`0000Coo`00000000Coo`8000?oo`04003ooooo0002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0003ool01@00oooooooo0000mOoo0003ool00`00oooo
o`03ool00`00ooooo`02ool20004ool20004ool20004ool20004ool20002ool00`00ooooo`3cool0
02Koo`05003oooooool0003eool002Koo`05003oooooool0003eool002Koo`05003oooooool0003e
ool002Koo`05003oooooool0003eool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo
8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0
0?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-9.73656, -34920.8, \
0.252224, 1166.02}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"samp", "=", 
        RowBox[{"sample", "[", 
          RowBox[{"Sin", ",", "nn", ",", "\[Tau]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"dps", "=", 
        RowBox[{"discretePowerSpec", "[", 
          RowBox[{"samp", ",", "\[Tau]"}], "]"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"dps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "3"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0190476 0 0.206011 [
[.21429 -0.0125 -6 -9 ]
[.21429 -0.0125 6 0 ]
[.40476 -0.0125 -6 -9 ]
[.40476 -0.0125 6 0 ]
[.59524 -0.0125 -6 -9 ]
[.59524 -0.0125 6 0 ]
[.78571 -0.0125 -6 -9 ]
[.78571 -0.0125 6 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .10301 -18 -4.5 ]
[.01131 .10301 0 4.5 ]
[.01131 .20601 -6 -4.5 ]
[.01131 .20601 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .51503 -18 -4.5 ]
[.01131 .51503 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 0 m
.21429 .00625 L
s
[(10)] .21429 -0.0125 0 1 Mshowa
.40476 0 m
.40476 .00625 L
s
[(20)] .40476 -0.0125 0 1 Mshowa
.59524 0 m
.59524 .00625 L
s
[(30)] .59524 -0.0125 0 1 Mshowa
.78571 0 m
.78571 .00625 L
s
[(40)] .78571 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(50)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.0619 0 m
.0619 .00375 L
s
.1 0 m
.1 .00375 L
s
.1381 0 m
.1381 .00375 L
s
.17619 0 m
.17619 .00375 L
s
.25238 0 m
.25238 .00375 L
s
.29048 0 m
.29048 .00375 L
s
.32857 0 m
.32857 .00375 L
s
.36667 0 m
.36667 .00375 L
s
.44286 0 m
.44286 .00375 L
s
.48095 0 m
.48095 .00375 L
s
.51905 0 m
.51905 .00375 L
s
.55714 0 m
.55714 .00375 L
s
.63333 0 m
.63333 .00375 L
s
.67143 0 m
.67143 .00375 L
s
.70952 0 m
.70952 .00375 L
s
.74762 0 m
.74762 .00375 L
s
.82381 0 m
.82381 .00375 L
s
.8619 0 m
.8619 .00375 L
s
.9 0 m
.9 .00375 L
s
.9381 0 m
.9381 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .10301 m
.03006 .10301 L
s
[(0.5)] .01131 .10301 1 0 Mshowa
.02381 .20601 m
.03006 .20601 L
s
[(1)] .01131 .20601 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(1.5)] .01131 .30902 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(2)] .01131 .41202 1 0 Mshowa
.02381 .51503 m
.03006 .51503 L
s
[(2.5)] .01131 .51503 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(3)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 0 m
.04286 .3236 L
.0619 0 L
.08095 0 L
.1 0 L
.11905 0 L
.1381 0 L
.15714 0 L
.17619 0 L
.19524 0 L
.21429 0 L
.23333 0 L
.25238 0 L
.27143 0 L
.29048 0 L
.30952 0 L
.32857 0 L
.34762 0 L
.36667 0 L
.38571 0 L
.40476 0 L
.42381 0 L
.44286 0 L
.4619 0 L
.48095 0 L
.5 0 L
.51905 0 L
.5381 0 L
.55714 0 L
.57619 0 L
.59524 0 L
.61429 0 L
.63333 0 L
.65238 0 L
.67143 0 L
.69048 0 L
.70952 0 L
.72857 0 L
.74762 0 L
.76667 0 L
.78571 0 L
.80476 0 L
.82381 0 L
.84286 0 L
.8619 0 L
.88095 0 L
.9 0 L
.91905 0 L
.9381 0 L
.95714 0 L
Mistroke
.97619 0 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00>_oo10000ooo0P008_oo10000ooo0P008ooo0P001?oo0P00
9?oo00<00?ooool00_oo0P008_oo0`001?oo0P00<Ooo000lool01@00oooooooo00000_oo00<00?oo
ool07ooo00<00?ooool00ooo00@00?ooool0027oo`04003ooooo0002ool01000ooooo`008ooo00D0
0?ooooooo`0000;oo`03003ooooo02;oo`04003ooooo0002ool00`00ooooo`0^ool003coo`05003o
ooooool00002ool00`00ooooo`0Pool00`00ooooo`02ool01000ooooo`009?oo00@00?ooool000;o
o`03003ooooo01koo`D000;oo`04003ooooo000Tool01000ooooo`000_oo00<00?ooool0;_oo000l
ool01@00oooooooo00000_oo00<00?ooool08Ooo00D00?ooooooo`0000;oo`03003ooooo023oo`80
00?oo`04003ooooo000Pool01000ooooo`000ooo00@00?ooool0027oo`<000?oo`04003ooooo000`
ool003coo`05003oooooool00002ool00`00ooooo`0Rool01000ooooo`000_oo00<00?ooool08_oo
00@00?ooool000;oo`03003ooooo01ooo`03003oo`0000?oo`04003ooooo000Qool00`00ooooo`03
ool01000ooooo`00<?oo000jool30003ool01000ooooo`008Ooo00@00?ooool000;oo`04003ooooo
000Qool01000ooooo`000_oo00@00?ooool002;oo`8000?oo`04003ooooo000Qool00`00ooooo`03
ool01000ooooo`00<?oo000lool00`00ooooo`02ool2000Sool20004ool2000Sool20004ool2000T
ool00`00ooooo`02ool2000Rool40003ool2000aool00?ooob7oo`00oooo8Ooo003jool40002ool4
0002ool400000ooo003oo`03ool00`00ooooo`05ool40004ool000ooon<000Ooo`06003ooooo003o
o`001ooo00<00?ooool00ooo00@00?ooool000Woo`03003ooooo00Coo`005?oo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo
00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool02ooo
00L00?ooool00?oo003oo`040003ool00`00ooooo`03ool00`00ool00009ool00`00ooooo`04ool0
01Coo`03003ooooo00Koo`03003ooooo0=Woo`03003ooooo00;oo`03003ooooo00Ooo`03003ooooo
00;oo`03003oo`0000Woo`03003ooooo00Coo`005?oo00<00?ooool01_oo00<00?ooool0f_oo00D0
0?ooooooo`0000[oo`03003ooooo00D000Ooo`@000Coo`005?oo00<00?ooool01_oo00<00?ooool0
i_oo00@00?ooool000ooo`03003ooooo00Coo`005?oo0P001ooo00<00?ooool0iooo0P004Ooo0P00
1?oo000Dool00`00ooooo`06ool00`00ooooo`3oool1ool001Coo`03003ooooo00Koo`03003ooooo
0?ooo`7oo`005?oo0P001ooo00<00?ooool0oooo0Ooo000Dool20007ool00`00ooooo`3oool1ool0
01Coo`8000Koo`03003ooooo0?ooo`;oo`005?oo0P001_oo00<00?ooool0oooo0_oo000Dool20006
ool00`00ooooo`3oool2ool001Coo`8000Koo`03003ooooo0?ooo`;oo`005?oo0P001_oo00<00?oo
ool0oooo0_oo000Dool20006ool00`00ooooo`3oool2ool001Coo`8000Koo`03003ooooo0?ooo`;o
o`005?oo0P001_oo00<00?ooool0oooo0_oo000Dool20006ool00`00ooooo`3oool2ool001Coo`80
00Koo`03003ooooo0?ooo`;oo`005?oo0P001_oo00<00?ooool0oooo0_oo0002ool20004ool00`00
ooooo`02ool30004ool20006ool00`00ooooo`3oool2ool00005ool00?ooool0000;ool01`00oooo
oooo003oo`0000Goo`03003ooooo0?ooo`;oo`0000Goo`00ooooo`0000_oo`04003oooooool30005
ool00`00ooooo`3oool2ool00005ool00?ooool00008ool30004ool00`00ool00005ool00`00oooo
o`3oool2ool00005ool00?ooool00008ool00`00ooooo`04ool00`00ool00005ool00`00ooooo`3o
ool2ool00005ool00?ooool00008ool00`00ooooo`04ool00`00ool00005ool00`00ooooo`3oool2
ool000;oo`8000Woo`@000?oo`03003oo`0000Goo`03003ooooo0?ooo`;oo`005?oo0`001?oo00<0
0?ooool0oooo0ooo000Dool00`00ool00004ool00`00ooooo`3oool3ool001Coo`03003oo`0000Co
o`03003ooooo0?ooo`?oo`005?oo00<00?oo00001?oo00<00?ooool0oooo0ooo000Dool30004ool0
0`00ooooo`3oool3ool001Coo`03003oo`0000Coo`03003ooooo0?ooo`?oo`005?oo00<00?oo0000
1?oo00<00?ooool0oooo0ooo000Dool00`00ool00004ool00`00ooooo`3oool3ool001Coo`03003o
o`0000Coo`03003ooooo0?ooo`?oo`005?oo0P0000?oo`00ool00_oo00<00?ooool0oooo0ooo000D
ool01000ooooo`000ooo00<00?ooool0oooo0ooo000Dool01000ooooo`000ooo00<00?ooool0oooo
0ooo000Dool01000ooooo`000ooo00<00?ooool0oooo0ooo000Dool01000ooooo`000ooo00<00?oo
ool0oooo0ooo000Dool200000ooo003oo`02ool00`00ooooo`3oool3ool001Coo`04003ooooo0003
ool00`00ooooo`3oool3ool000goo`@000?oo`04003ooooo0003ool00`00ooooo`3oool3ool000oo
o`03003ooooo00;oo`04003ooooo0003ool00`00ooooo`3oool3ool000ooo`03003ooooo00;oo`@0
00;oo`03003ooooo0?ooo`Coo`003ooo00<00?ooool00_oo00@00?ooool000;oo`03003ooooo0?oo
o`Coo`003ooo00<00?ooool00_oo00@00?ooool000;oo`03003ooooo0?ooo`Coo`003Ooo0`001?oo
00@00?ooool000;oo`03003ooooo0?ooo`Coo`003ooo00<00?ooool00_oo00@00?ooool000;oo`03
003ooooo0?ooo`Coo`005?oo0P0000Goo`00ooooo`000?ooo`Koo`005?oo00L00?ooooooo`00ool0
003oool6ool001Coo`07003oooooool00?oo0000oooo1_oo000Dool01`00oooooooo003oo`000?oo
o`Koo`005?oo00L00?ooooooo`00ool0003oool6ool001Coo`8000;oo`03003oo`000?ooo`Koo`00
5?oo00L00?ooooooo`00ool0003oool6ool001Coo`07003oooooool00?oo0000oooo1_oo000Dool0
1`00oooooooo003oo`000?ooo`Koo`005?oo0P000_oo00<00?oo0000oooo1_oo000Dool01`00oooo
oooo003oo`000?ooo`Koo`005?oo00L00?ooooooo`00ool0003oool6ool001Coo`07003oooooool0
0?oo0000oooo1_oo000Dool01000oooooooo0P00oooo1ooo000Dool20002ool2003oool7ool001Co
o`03003ooooo00;oo`03003ooooo0?ooo`Goo`005?oo00<00?ooool00_oo00<00?ooool0oooo1Ooo
00000ooo000000020003ool00`00ooooo`02ool30004ool00`00ooooo`02ool00`00ooooo`3oool5
ool000?oo`03003ooooo00[oo`05003oooooool00004ool00`00ooooo`3oool5ool000?oo`03003o
oooo00[oo`04003oooooool30002ool00`00ooooo`3oool5ool000?oo`03003ooooo00Ooo`<000Co
o`03003ooooo00;oo`03003ooooo0?ooo`Goo`000ooo00<00?ooool01ooo00<00?ooool01?oo00<0
0?ooool00_oo00<00?ooool0oooo1Ooo00001?oo000000002Ooo00<00?ooool01?oo00<00?ooool0
0_oo00<00?ooool0oooo1Ooo0003ool00`00ooooo`07ool40003ool2003oool;ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`00
5?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`00
5?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`003Ooo1000
0ooo00<00?ooool0oooo2_oo000=ool00`00ooooo`04ool00`00ooooo`3oool:ool000koo`03003o
oooo00?oo`<00?ooo`[oo`003ooo00<00?ooool00_oo00<00?ooool0oooo2_oo000@ool01@00oooo
oooo0000oooo3?oo000=ool01000ooooo`000ooo00<00?ooool0oooo2_oo000>ool20004ool00`00
ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo
0?ooo`[oo`0000?oo`0000000P000ooo00<00?ooool00_oo0`001?oo00<00?ooool0oooo2_oo0000
0ooo003oo`0=ool01@00oooooooo0000oooo3?oo0002ool00`00ooooo`0;ool01000oooooooo0`00
oooo2_oo0003ool00`00ooooo`07ool30004ool00`00ooooo`3oool:ool000Coo`03003ooooo00Ko
o`03003ooooo00Coo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`03003ooooo00Coo`03
003ooooo0?ooo`[oo`000_oo0P002Ooo10000ooo00<00?ooool0oooo2_oo000Dool2003oool;ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool000koo`8000Coo`03003ooooo
0?ooo`[oo`003Ooo00@00?ooool000?oo`03003ooooo0?ooo`[oo`004?oo00@00?ooooooo`<00?oo
o`[oo`003_oo0P00oooo4Ooo000@ool00`00ooooo`3oool>ool000goo`04003ooooo003oool@ool0
00koo`800?oooa7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-4.83967, -0.479654, \
0.232399, 0.0214874}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "]"}], ",", 
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
                "]"}]}], "}"}], "&"}], "/@", "dps"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Export", "[", 
        RowBox[{"\"\<tmp.txt\>\"", ",", 
          RowBox[{"ToString", "[", "%", "]"}]}], "]"}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The constant 0.5 sampled 19 times", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Tau]", "=", "1."}], ";", 
      RowBox[{"nn", "=", "19"}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"ps", "=", 
        RowBox[{"powerSpec", "[", 
          RowBox[{
            RowBox[{"0.5", "&"}], ",", "nn", ",", "\[Tau]", ",", "1000."}], 
          "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"ps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "150000"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.319994 -2.70508e-17 4.12023e-06 [
[.18381 -0.0125 -9 -9 ]
[.18381 -0.0125 9 0 ]
[.3438 -0.0125 -3 -9 ]
[.3438 -0.0125 3 0 ]
[.5038 -0.0125 -9 -9 ]
[.5038 -0.0125 9 0 ]
[.6638 -0.0125 -3 -9 ]
[.6638 -0.0125 3 0 ]
[.82379 -0.0125 -9 -9 ]
[.82379 -0.0125 9 0 ]
[.98379 -0.0125 -3 -9 ]
[.98379 -0.0125 3 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .0824 -30 -4.5 ]
[.01131 .0824 0 4.5 ]
[.01131 .16481 -30 -4.5 ]
[.01131 .16481 0 4.5 ]
[.01131 .24721 -30 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .32962 -30 -4.5 ]
[.01131 .32962 0 4.5 ]
[.01131 .41202 -36 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .49443 -36 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .57683 -36 -4.5 ]
[.01131 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18381 0 m
.18381 .00625 L
s
[(0.5)] .18381 -0.0125 0 1 Mshowa
.3438 0 m
.3438 .00625 L
s
[(1)] .3438 -0.0125 0 1 Mshowa
.5038 0 m
.5038 .00625 L
s
[(1.5)] .5038 -0.0125 0 1 Mshowa
.6638 0 m
.6638 .00625 L
s
[(2)] .6638 -0.0125 0 1 Mshowa
.82379 0 m
.82379 .00625 L
s
[(2.5)] .82379 -0.0125 0 1 Mshowa
.98379 0 m
.98379 .00625 L
s
[(3)] .98379 -0.0125 0 1 Mshowa
.125 Mabswid
.05581 0 m
.05581 .00375 L
s
.08781 0 m
.08781 .00375 L
s
.11981 0 m
.11981 .00375 L
s
.15181 0 m
.15181 .00375 L
s
.21581 0 m
.21581 .00375 L
s
.24781 0 m
.24781 .00375 L
s
.2798 0 m
.2798 .00375 L
s
.3118 0 m
.3118 .00375 L
s
.3758 0 m
.3758 .00375 L
s
.4078 0 m
.4078 .00375 L
s
.4398 0 m
.4398 .00375 L
s
.4718 0 m
.4718 .00375 L
s
.5358 0 m
.5358 .00375 L
s
.5678 0 m
.5678 .00375 L
s
.5998 0 m
.5998 .00375 L
s
.6318 0 m
.6318 .00375 L
s
.6958 0 m
.6958 .00375 L
s
.7278 0 m
.7278 .00375 L
s
.7598 0 m
.7598 .00375 L
s
.7918 0 m
.7918 .00375 L
s
.85579 0 m
.85579 .00375 L
s
.88779 0 m
.88779 .00375 L
s
.91979 0 m
.91979 .00375 L
s
.95179 0 m
.95179 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .0824 m
.03006 .0824 L
s
[(20000)] .01131 .0824 1 0 Mshowa
.02381 .16481 m
.03006 .16481 L
s
[(40000)] .01131 .16481 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(60000)] .01131 .24721 1 0 Mshowa
.02381 .32962 m
.03006 .32962 L
s
[(80000)] .01131 .32962 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(100000)] .01131 .41202 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(120000)] .01131 .49443 1 0 Mshowa
.02381 .57683 m
.03006 .57683 L
s
[(140000)] .01131 .57683 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.0299 .61803 m
.12963 0 L
.23545 0 L
.34127 0 L
.44709 0 L
.55291 0 L
.65873 0 L
.76455 0 L
.87037 0 L
.97619 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{311.875, 192.688},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004g0000`1000`40n007h00Ooooo>?oo003ooolh
ool00?ooocSoo`00oooo>?oo003ooolhool00?ooocSoo`00oooo>?oo003ooolhool00?ooocSoo`00
oooo>?oo003ooolhool00?ooocSoo`00oooo>?oo003ooolhool00?ooocSoo`00oooo>?oo003ooolh
ool00?ooocSoo`00oooo>?oo003ooolhool004Coo`8000Coo`03003ooooo00;oo`<001goo`@001_o
o`@000?oo`03003ooooo00;oo`<001coo`@001_oo`@000?oo`03003ooooo00;oo`<001goo`80037o
o`00@ooo00@00?ooool000_oo`03003ooooo01coo`03003ooooo01coo`03003ooooo00[oo`03003o
oooo01Woo`03003ooooo01coo`03003ooooo00coo`03003ooooo01Woo`04003ooooo000`ool004?o
o`04003ooooo000;ool00`00ooooo`0Lool00`00ooooo`0Lool00`00ooooo`0:ool00`00ooooo`0J
ool00`00ooooo`0Lool00`00ooooo`0;ool00`00ooooo`0Lool00`00ooooo`0^ool004?oo`04003o
oooo0008ool3000Oool00`00ooooo`0Lool00`00ooooo`07ool3000Nool00`00ooooo`0Lool00`00
ooooo`07ool3000Mool2000aool004?oo`04003ooooo0008ool00`00ooooo`0Oool00`00ooooo`0L
ool00`00ooooo`07ool00`00ooooo`0Oool00`00ooooo`0Lool00`00ooooo`06ool00`00ooooo`0O
ool00`00ooooo`0^ool004?oo`04003ooooo0008ool00`00ooooo`0Mool3000Lool30009ool00`00
ooooo`0Lool01000ooooo`006ooo00@00?ooool000Soo`03003ooooo01coo`04003ooooo000`ool0
04Coo`8000Woo`@001koo`03003ooooo01coo`03003ooooo00Ooo`@001coo`8001goo`8000Woo`@0
01coo`80037oo`00oooo>?oo003ooolhool00?oooa;oo`@000;oo`@000;oo`@00003ool00?oo00?o
o`03003ooooo00Goo`@000Coo`008Oooj@001_oo00H00?ooool00?oo0007ool00`00ooooo`03ool0
1000ooooo`002Ooo00<00?ooool01?oo000Vool00`00ooooo`05ool00`00ooooo`04ool00`00oooo
o`05ool00`00ool00004ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`04ool00`00oooo
o`05ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`04ool00`00oooo
o`05ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`04ool00`00oooo
o`05ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`05ool00`00oooo
o`04ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`04ool00`00ooooo`05ool00`00oooo
o`04ool00`00ooooo`04ool00`00ooooo`05ool00`00ooooo`08ool01`00ooooo`00ool00?oo00@0
00?oo`03003ooooo00?oo`03003oo`0000Woo`03003ooooo00Coo`009_oo00<00?ooool05_oo00<0
0?ooool0c_oo00<00?ooool00_oo00<00?ooool01ooo00<00?ooool00_oo00<00?oo00002Ooo00<0
0?ooool01?oo000Vool00`00ooooo`0Fool00`00ooooo`3?ool01@00oooooooo00002_oo00<00?oo
ool01@001ooo10001?oo000Vool00`00ooooo`0Eool00`00ooooo`3Lool01000ooooo`003ooo00<0
0?ooool01?oo000Vool2000Fool00`00ooooo`3Mool2000Aool20004ool002Koo`03003ooooo01Go
o`03003ooooo0?Koo`009_oo00<00?ooool05Ooo00<00?ooool0m_oo000Vool00`00ooooo`0Eool0
0`00ooooo`3fool002Koo`03003ooooo01Goo`03003ooooo0?Koo`009_oo0P005Ooo00<00?ooool0
mooo000Vool00`00ooooo`0Dool00`00ooooo`3gool002Koo`03003ooooo01Coo`03003ooooo0?Oo
o`009_oo00<00?ooool05?oo00<00?ooool0mooo000Vool2000Eool00`00ooooo`3gool002Koo`03
003ooooo01Coo`03003ooooo0?Ooo`009_oo00<00?ooool04ooo00<00?ooool0n?oo0007ool40003
ool20004ool20004ool20004ool20004ool00`00ooooo`0Cool00`00ooooo`3hool000Ooo`03003o
oooo00?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool0
0`00ooooo`0Cool00`00ooooo`3hool000Soo`03003ooooo00;oo`04003ooooo0002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0003ool3000Cool00`00ooooo`3hool000Woo`05003o
ooooool00002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo
00005Ooo00<00?ooool0n?oo000:ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002
ool01000ooooo`000_oo00D00?ooooooo`0001Goo`03003ooooo0?Soo`001ooo00@00?ooool000;o
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00oooo
o`0Bool00`00ooooo`3iool000Soo`8000Coo`8000Coo`8000Coo`8000Coo`8000Coo`03003ooooo
01;oo`03003ooooo0?Woo`009_oo0P004ooo00<00?ooool0nOoo000Vool00`00ooooo`0Bool00`00
ooooo`3iool002Koo`03003ooooo01;oo`03003ooooo0?Woo`009_oo00<00?ooool04_oo00<00?oo
ool0nOoo000Vool00`00ooooo`0Aool00`00ooooo`3jool002Koo`8001;oo`03003ooooo0?[oo`00
9_oo00<00?ooool04Ooo00<00?ooool0n_oo000Vool00`00ooooo`0Aool00`00ooooo`3jool002Ko
o`03003ooooo017oo`03003ooooo0?[oo`009_oo0P004_oo00<00?ooool0n_oo000Vool00`00oooo
o`0Aool00`00ooooo`3jool002Koo`03003ooooo013oo`03003ooooo0?_oo`002Ooo00<00?ooool0
0_oo0P001?oo0P001?oo0P001?oo0P001?oo00<00?ooool04?oo00<00?ooool0nooo0009ool01@00
oooooooo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00D00?oooooo
o`0001;oo`03003ooooo0?_oo`001_oo1@000_oo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000?oo`<0013oo`03003ooooo0?_oo`001_oo00@00?ooool000?oo`04
003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`0@
ool00`00ooooo`3kool000Ooo`03003oo`0000?oo`04003ooooo0002ool01000ooooo`000_oo00@0
0?ooool000;oo`04003ooooo0003ool00`00ooooo`0@ool00`00ooooo`3kool000Soo`8000?oo`04
003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`0?
ool00`00ooooo`3lool000Woo`03003ooooo00;oo`8000Coo`8000Coo`8000Coo`8000Coo`03003o
oooo00ooo`03003ooooo0?coo`009_oo0P004?oo00<00?ooool0o?oo000Vool00`00ooooo`0?ool0
0`00ooooo`3lool002Koo`03003ooooo00ooo`03003ooooo0?coo`009_oo00<00?ooool03ooo00<0
0?ooool0o?oo000Vool00`00ooooo`0>ool00`00ooooo`3mool002Koo`8000ooo`03003ooooo0?go
o`009_oo00<00?ooool03_oo00<00?ooool0oOoo000Vool00`00ooooo`0>ool00`00ooooo`3mool0
02Koo`03003ooooo00koo`03003ooooo0?goo`009_oo00<00?ooool03_oo00<00?ooool0oOoo000V
ool2000>ool00`00ooooo`3nool002Koo`03003ooooo00goo`03003ooooo0?koo`002?oo0P001?oo
0P001?oo0P001?oo0P001?oo0P001?oo00<00?ooool03Ooo00<00?ooool0o_oo0007ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`03
003ooooo00goo`03003ooooo0?koo`001ooo00@00?ooool000;oo`04003ooooo0002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0003ool3000=ool00`00ooooo`3nool000Ooo`<000?o
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool00`00oooo
o`0=ool00`00ooooo`3nool000Ooo`03003ooooo00?oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`0=ool00`00ooooo`3nool000Ooo`03003o
oooo00?oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool0
0`00ooooo`0<ool00`00ooooo`3oool000Soo`<000?oo`8000Coo`8000Coo`8000Coo`8000Coo`03
003ooooo00coo`03003ooooo0?ooo`009_oo0P003Ooo00<00?ooool0oooo000Vool00`00ooooo`0<
ool00`00ooooo`3oool002Koo`03003ooooo00coo`03003ooooo0?ooo`009_oo00<00?ooool03?oo
00<00?ooool0oooo000Vool00`00ooooo`0;ool00`00ooooo`3oool1ool002Koo`8000coo`03003o
oooo0?ooo`7oo`009_oo00<00?ooool02ooo00<00?ooool0oooo0Ooo000Vool00`00ooooo`0;ool0
0`00ooooo`3oool1ool002Koo`03003ooooo00_oo`03003ooooo0?ooo`7oo`009_oo00<00?ooool0
2ooo00<00?ooool0oooo0Ooo000Vool2000;ool00`00ooooo`3oool2ool002Koo`03003ooooo00[o
o`03003ooooo0?ooo`;oo`002?oo0P001?oo0P001?oo0P001?oo0P001?oo0P001?oo00<00?ooool0
2_oo00<00?ooool0oooo0_oo0007ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002
ool01000ooooo`000_oo00@00?ooool000?oo`03003ooooo00[oo`03003ooooo0?ooo`;oo`001ooo
00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo
0003ool3000:ool00`00ooooo`3oool2ool000Soo`8000?oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`0:ool00`00ooooo`3oool2ool000Oo
o`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000oooo
o`000ooo00<00?ooool02Ooo00<00?ooool0oooo0ooo0007ool01000ooooo`000_oo00@00?ooool0
00;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000?oo`03003ooooo00Woo`03003o
oooo0?ooo`?oo`002?oo0P001?oo0P001?oo0P001?oo0P001?oo0P001?oo00<00?ooool02Ooo00<0
0?ooool0oooo0ooo000Vool2000:ool00`00ooooo`3oool3ool002Koo`03003ooooo00Woo`03003o
oooo0?ooo`?oo`009_oo00<00?ooool02Ooo00<00?ooool0oooo0ooo000Vool00`00ooooo`08ool0
0`00ooooo`3oool4ool002Koo`03003ooooo00Soo`03003ooooo0?ooo`Coo`009_oo0P002Ooo00<0
0?ooool0oooo1?oo000Vool00`00ooooo`08ool00`00ooooo`3oool4ool002Koo`03003ooooo00So
o`03003ooooo0?ooo`Coo`009_oo00<00?ooool02?oo00<00?ooool0oooo1?oo000Vool00`00oooo
o`08ool00`00ooooo`3oool4ool002Koo`8000Soo`03003ooooo0?ooo`Goo`009_oo00<00?ooool0
1ooo00<00?ooool0oooo1Ooo00000ooo000000020003ool20004ool20004ool20004ool20004ool2
0004ool00`00ooooo`07ool00`00ooooo`3oool5ool000?oo`05003oooooool00002ool01000oooo
o`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00D00?ooooooo`0000Wo
o`03003ooooo0?ooo`Goo`000ooo00D00?ooooooo`0000;oo`04003ooooo0002ool01000ooooo`00
0_oo00@00?ooool000;oo`04003ooooo0002ool01000oooooooo0`001ooo00<00?ooool0oooo1Ooo
0003ool01@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000;oo`05003oooooool00009ool00`00ooooo`3oool5ool000?oo`05003oooooool0
0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo00D0
0?ooooooo`0000Soo`03003ooooo0?ooo`Koo`0000Coo`00000000?oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000ooo00<00?ooool01_oo
00<00?ooool0oooo1_oo0003ool00`00ooooo`02ool20004ool20004ool20004ool20004ool20004
ool00`00ooooo`06ool00`00ooooo`3oool6ool002Koo`8000Ooo`03003ooooo0?ooo`Koo`009_oo
00<00?ooool01_oo00<00?ooool0oooo1_oo000Vool00`00ooooo`06ool00`00ooooo`3oool6ool0
02Koo`03003ooooo00Goo`03003ooooo0?ooo`Ooo`009_oo00<00?ooool01Ooo00<00?ooool0oooo
1ooo000Vool20006ool00`00ooooo`3oool7ool002Koo`03003ooooo00Goo`03003ooooo0?ooo`Oo
o`009_oo00<00?ooool01Ooo00<00?ooool0oooo1ooo000Vool00`00ooooo`05ool00`00ooooo`3o
ool7ool002Koo`03003ooooo00Coo`03003ooooo0?ooo`Soo`009_oo0P001Ooo00<00?ooool0oooo
2?oo000Vool00`00ooooo`04ool00`00ooooo`3oool8ool002Koo`03003ooooo00Coo`03003ooooo
0?ooo`Soo`0000?oo`0000000P000_oo10000ooo0P001?oo0P001?oo0P001?oo0P001?oo00<00?oo
ool01?oo00<00?ooool0oooo2?oo0003ool01@00oooooooo00001Ooo00@00?ooool000;oo`04003o
oooo0002ool01000ooooo`000_oo00@00?ooool000?oo`03003ooooo00Coo`03003ooooo0?ooo`So
o`000ooo00<00?ooool00_oo00<00?ooool00_oo00@00?ooool000;oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000?oo`<000Coo`03003ooooo0?ooo`Soo`000ooo00<00?ooool00ooo
00D00?ooooooo`0000;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`05003o
ooooool00005ool00`00ooooo`3oool9ool000?oo`03003ooooo00Coo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01@00oooooooo00001Ooo00<00?ooool0
oooo2Ooo00001?oo000000000ooo00@00?ooool000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000;oo`04003ooooo0003ool00`00ooooo`03ool00`00ooooo`3oool9ool000?oo`03
003ooooo00;oo`8000Coo`8000Coo`8000Coo`8000Coo`8000Coo`8000Coo`03003ooooo0?ooo`Wo
o`009_oo00<00?ooool00ooo00<00?ooool0oooo2Ooo000Vool00`00ooooo`03ool00`00ooooo`3o
ool9ool002Koo`03003ooooo00;oo`03003ooooo0?ooo`[oo`009_oo00<00?ooool00_oo00<00?oo
ool0oooo2_oo000Vool20003ool00`00ooooo`3oool:ool002Koo`03003ooooo00;oo`03003ooooo
0?ooo`[oo`009_oo00<00?ooool00_oo00<00?ooool0oooo2_oo000Vool00`00ooooo`02ool00`00
ooooo`3oool:ool002Koo`05003oooooool0003oool=ool002Koo`8000;oo`03003ooooo0?ooo`_o
o`009_oo00D00?ooooooo`000?ooo`goo`009_oo00D00?ooooooo`000?ooo`goo`0000?oo`000000
0P001?oo00<00?ooool00_oo0P001?oo0P001?oo0P001?oo0P001?oo00D00?ooooooo`000?ooo`go
o`000ooo00<00?ooool00ooo00D00?ooooooo`0000;oo`04003ooooo0002ool01000ooooo`000_oo
00@00?ooool000;oo`05003oooooool00003ool00`00ooooo`3oool;ool000?oo`03003ooooo00D0
00;oo`04003ooooo0002ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool4003o
ool>ool000?oo`04003ooooo0002ool01@00oooooooo00000_oo00@00?ooool000;oo`04003ooooo
0002ool01000ooooo`000_oo00D00?ooooooo`0000;oo`03003ooooo0?ooo`coo`000ooo00L00?oo
ooooo`00ool00003ool01000ooooo`000_oo00@00?ooool000;oo`04003ooooo0002ool01000oooo
o`000ooo00@00?ooool00?ooo`koo`0000Coo`00000000Coo`8000?oo`04003ooooo0002ool01000
ooooo`000_oo00@00?ooool000;oo`04003ooooo0003ool01000ooooo`00oooo3_oo0003ool00`00
ooooo`03ool00`00ooooo`02ool20004ool20004ool20004ool20004ool200000ooo003oo`3oool=
ool002Koo`04003ooooo003oool>ool002Koo`03003oo`000?ooo`ooo`009_oo00<00?oo0000oooo
3ooo000Vool00`00ool0003oool?ool002Koo`03003oo`000?ooo`ooo`00oooo>?oo003ooolhool0
0?ooocSoo`00oooo>?oo003ooolhool00?ooocSoo`00oooo>?oo003ooolhool00?ooocSoo`00oooo
>?oo003ooolhool00?ooocSoo`00oooo>?oo003ooolhool00?ooocSoo`00oooo>?oo003ooolhool0
0?ooocSoo`00\
\>"],
  ImageRangeCache->{{{0, 310.875}, {191.688, 0}} -> {-0.524627, -31626.9, \
0.0134875, 1047.49}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"samp", "=", 
        RowBox[{"sample", "[", 
          RowBox[{
            RowBox[{"0.5", "&"}], ",", "nn", ",", "\[Tau]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"dps", "=", 
        RowBox[{"discretePowerSpec", "[", 
          RowBox[{"samp", ",", "\[Tau]"}], "]"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"dps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "19"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.319994 -6.93889e-18 0.0325281 [
[.18381 -0.0125 -9 -9 ]
[.18381 -0.0125 9 0 ]
[.3438 -0.0125 -3 -9 ]
[.3438 -0.0125 3 0 ]
[.5038 -0.0125 -9 -9 ]
[.5038 -0.0125 9 0 ]
[.6638 -0.0125 -3 -9 ]
[.6638 -0.0125 3 0 ]
[.82379 -0.0125 -9 -9 ]
[.82379 -0.0125 9 0 ]
[.98379 -0.0125 -3 -9 ]
[.98379 -0.0125 3 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .08132 -18 -4.5 ]
[.01131 .08132 0 4.5 ]
[.01131 .16264 -6 -4.5 ]
[.01131 .16264 0 4.5 ]
[.01131 .24396 -18 -4.5 ]
[.01131 .24396 0 4.5 ]
[.01131 .32528 -12 -4.5 ]
[.01131 .32528 0 4.5 ]
[.01131 .4066 -24 -4.5 ]
[.01131 .4066 0 4.5 ]
[.01131 .48792 -12 -4.5 ]
[.01131 .48792 0 4.5 ]
[.01131 .56924 -24 -4.5 ]
[.01131 .56924 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18381 0 m
.18381 .00625 L
s
[(0.5)] .18381 -0.0125 0 1 Mshowa
.3438 0 m
.3438 .00625 L
s
[(1)] .3438 -0.0125 0 1 Mshowa
.5038 0 m
.5038 .00625 L
s
[(1.5)] .5038 -0.0125 0 1 Mshowa
.6638 0 m
.6638 .00625 L
s
[(2)] .6638 -0.0125 0 1 Mshowa
.82379 0 m
.82379 .00625 L
s
[(2.5)] .82379 -0.0125 0 1 Mshowa
.98379 0 m
.98379 .00625 L
s
[(3)] .98379 -0.0125 0 1 Mshowa
.125 Mabswid
.05581 0 m
.05581 .00375 L
s
.08781 0 m
.08781 .00375 L
s
.11981 0 m
.11981 .00375 L
s
.15181 0 m
.15181 .00375 L
s
.21581 0 m
.21581 .00375 L
s
.24781 0 m
.24781 .00375 L
s
.2798 0 m
.2798 .00375 L
s
.3118 0 m
.3118 .00375 L
s
.3758 0 m
.3758 .00375 L
s
.4078 0 m
.4078 .00375 L
s
.4398 0 m
.4398 .00375 L
s
.4718 0 m
.4718 .00375 L
s
.5358 0 m
.5358 .00375 L
s
.5678 0 m
.5678 .00375 L
s
.5998 0 m
.5998 .00375 L
s
.6318 0 m
.6318 .00375 L
s
.6958 0 m
.6958 .00375 L
s
.7278 0 m
.7278 .00375 L
s
.7598 0 m
.7598 .00375 L
s
.7918 0 m
.7918 .00375 L
s
.85579 0 m
.85579 .00375 L
s
.88779 0 m
.88779 .00375 L
s
.91979 0 m
.91979 .00375 L
s
.95179 0 m
.95179 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .08132 m
.03006 .08132 L
s
[(2.5)] .01131 .08132 1 0 Mshowa
.02381 .16264 m
.03006 .16264 L
s
[(5)] .01131 .16264 1 0 Mshowa
.02381 .24396 m
.03006 .24396 L
s
[(7.5)] .01131 .24396 1 0 Mshowa
.02381 .32528 m
.03006 .32528 L
s
[(10)] .01131 .32528 1 0 Mshowa
.02381 .4066 m
.03006 .4066 L
s
[(12.5)] .01131 .4066 1 0 Mshowa
.02381 .48792 m
.03006 .48792 L
s
[(15)] .01131 .48792 1 0 Mshowa
.02381 .56924 m
.03006 .56924 L
s
[(17.5)] .01131 .56924 1 0 Mshowa
.125 Mabswid
.02381 .01626 m
.02756 .01626 L
s
.02381 .03253 m
.02756 .03253 L
s
.02381 .04879 m
.02756 .04879 L
s
.02381 .06506 m
.02756 .06506 L
s
.02381 .09758 m
.02756 .09758 L
s
.02381 .11385 m
.02756 .11385 L
s
.02381 .13011 m
.02756 .13011 L
s
.02381 .14638 m
.02756 .14638 L
s
.02381 .1789 m
.02756 .1789 L
s
.02381 .19517 m
.02756 .19517 L
s
.02381 .21143 m
.02756 .21143 L
s
.02381 .2277 m
.02756 .2277 L
s
.02381 .26022 m
.02756 .26022 L
s
.02381 .27649 m
.02756 .27649 L
s
.02381 .29275 m
.02756 .29275 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34155 m
.02756 .34155 L
s
.02381 .35781 m
.02756 .35781 L
s
.02381 .37407 m
.02756 .37407 L
s
.02381 .39034 m
.02756 .39034 L
s
.02381 .42287 m
.02756 .42287 L
s
.02381 .43913 m
.02756 .43913 L
s
.02381 .45539 m
.02756 .45539 L
s
.02381 .47166 m
.02756 .47166 L
s
.02381 .50419 m
.02756 .50419 L
s
.02381 .52045 m
.02756 .52045 L
s
.02381 .53671 m
.02756 .53671 L
s
.02381 .55298 m
.02756 .55298 L
s
.02381 .58551 m
.02756 .58551 L
s
.02381 .60177 m
.02756 .60177 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .46723 m
.12963 0 L
.23545 0 L
.34127 0 L
.44709 0 L
.55291 0 L
.65873 0 L
.76455 0 L
.87037 0 L
.97619 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo000fool2
0004ool00`00ooooo`02ool3000Kool4000Iool40003ool00`00ooooo`02ool3000Jool4000Iool4
0003ool00`00ooooo`02ool3000Kool2000bool003Goo`04003ooooo000;ool00`00ooooo`0Jool0
0`00ooooo`0Jool00`00ooooo`0:ool00`00ooooo`0Gool00`00ooooo`0Jool00`00ooooo`0<ool0
0`00ooooo`0Gool01000ooooo`00<Ooo000eool01000ooooo`002ooo00<00?ooool06_oo00<00?oo
ool06_oo00<00?ooool02_oo00<00?ooool06?oo00<00?ooool06_oo00<00?ooool02ooo00<00?oo
ool06_oo00<00?ooool0;ooo000eool01000ooooo`002?oo0`007Ooo00<00?ooool06_oo00<00?oo
ool01ooo0`007?oo00<00?ooool06_oo00<00?ooool01ooo0`006ooo0P00<_oo000eool01000oooo
o`002?oo00<00?ooool07Ooo00<00?ooool06_oo00<00?ooool01ooo00<00?ooool07Ooo00<00?oo
ool06_oo00<00?ooool01_oo00<00?ooool07Ooo00<00?ooool0;ooo000eool01000ooooo`002?oo
00<00?ooool06ooo0`006_oo0`002Ooo00<00?ooool06_oo00@00?ooool001Woo`04003ooooo0008
ool00`00ooooo`0Jool01000ooooo`00<Ooo000fool20009ool4000Lool00`00ooooo`0Jool00`00
ooooo`07ool4000Jool2000Kool20009ool4000Jool2000bool00?ooob7oo`00oooo8Ooo003jool4
0002ool40002ool400000ooo003oo`03ool00`00ooooo`05ool40004ool001Goomd000Ooo`06003o
oooo003oo`001ooo00<00?ooool00ooo00@00?ooool000Woo`03003ooooo00Coo`006_oo00<00?oo
ool01?oo00<00?ooool01?oo00<00?ooool01?oo00@00?ooool000?oo`03003ooooo00Coo`03003o
oooo00Coo`03003ooooo00Goo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003o
oooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003o
oooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003o
oooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003o
oooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003ooooo00Coo`03003o
oooo00Woo`07003ooooo003oo`00ool010000ooo00<00?ooool00ooo00<00?oo00002Ooo00<00?oo
ool01?oo000Jool00`00ooooo`0Eool00`00ooooo`34ool00`00ooooo`02ool00`00ooooo`07ool0
0`00ooooo`02ool00`00ool00009ool00`00ooooo`04ool001[oo`03003ooooo01Coo`03003ooooo
0<Koo`05003oooooool0000:ool00`00ooooo`050007ool40004ool001[oo`8001Goo`03003ooooo
0=;oo`04003ooooo000?ool00`00ooooo`04ool001[oo`03003ooooo01Coo`03003ooooo0=?oo`80
017oo`8000Coo`006_oo00<00?ooool05?oo00<00?ooool0k?oo000Jool2000Dool00`00ooooo`3]
ool001[oo`03003ooooo01?oo`03003ooooo0>goo`006_oo00<00?ooool04ooo00<00?ooool0kOoo
000Jool00`00ooooo`0Cool00`00ooooo`3]ool001[oo`8001?oo`03003ooooo0>koo`006_oo00<0
0?ooool04_oo00<00?ooool0k_oo000Jool00`00ooooo`0Bool00`00ooooo`3^ool001[oo`8001?o
o`03003ooooo0>koo`006_oo00<00?ooool04_oo00<00?ooool0k_oo0007ool40003ool00`00oooo
o`02ool30004ool00`00ooooo`0Aool00`00ooooo`3_ool000Ooo`03003ooooo00coo`05003ooooo
ool0000Cool00`00ooooo`3_ool000Soo`03003ooooo00_oo`04003oooooool3000Aool00`00oooo
o`3_ool000Woo`03003ooooo00Ooo`<000Coo`03003ooooo017oo`03003ooooo0>ooo`002_oo00<0
0?ooool01_oo00<00?ooool01?oo00<00?ooool04?oo00<00?ooool0l?oo0007ool01000ooooo`00
2?oo00<00?ooool01?oo0P004Ooo00<00?ooool0l?oo0008ool20009ool40003ool00`00ooooo`0@
ool00`00ooooo`3`ool001[oo`03003ooooo013oo`03003ooooo0?3oo`006_oo00<00?ooool03ooo
00<00?ooool0lOoo000Jool2000@ool00`00ooooo`3aool001[oo`03003ooooo00ooo`03003ooooo
0?7oo`006_oo00<00?ooool03ooo00<00?ooool0lOoo000Jool00`00ooooo`0>ool00`00ooooo`3b
ool001[oo`8000ooo`03003ooooo0?;oo`006_oo00<00?ooool03_oo00<00?ooool0l_oo000Jool0
0`00ooooo`0>ool00`00ooooo`3bool001[oo`8000ooo`03003ooooo0?;oo`006_oo00<00?ooool0
3Ooo00<00?ooool0looo000Cool30004ool00`00ooooo`0=ool00`00ooooo`3cool001Koo`05003o
ooooool0000?ool00`00ooooo`3cool001Koo`04003oooooool3000=ool00`00ooooo`3cool001?o
o`<000Coo`03003ooooo00coo`03003ooooo0?Coo`004ooo00<00?ooool01?oo00<00?ooool03?oo
00<00?ooool0m?oo000Cool00`00ooooo`04ool2000=ool00`00ooooo`3dool001?oo`@000?oo`03
003ooooo00coo`03003ooooo0?Coo`006_oo00<00?ooool02ooo00<00?ooool0mOoo000Jool00`00
ooooo`0;ool00`00ooooo`3eool001[oo`8000coo`03003ooooo0?Goo`006_oo00<00?ooool02ooo
00<00?ooool0mOoo000Jool00`00ooooo`0;ool00`00ooooo`3eool001[oo`8000_oo`03003ooooo
0?Koo`006_oo00<00?ooool02_oo00<00?ooool0m_oo000Jool00`00ooooo`0:ool00`00ooooo`3f
ool001[oo`03003ooooo00[oo`03003ooooo0?Koo`006_oo0P002_oo00<00?ooool0mooo000Jool0
0`00ooooo`09ool00`00ooooo`3gool000Soo`03003ooooo00?oo`03003ooooo00;oo`<000Coo`03
003ooooo00Woo`03003ooooo0?Ooo`002?oo00<00?ooool02ooo00D00?ooooooo`0000_oo`03003o
oooo0?Ooo`002Ooo00<00?ooool02_oo00@00?ooooooo`<000Soo`03003ooooo0?Soo`002Ooo00<0
0?ooool01ooo0`001?oo00<00?ooool02?oo00<00?ooool0n?oo000:ool00`00ooooo`06ool00`00
ooooo`04ool00`00ooooo`08ool00`00ooooo`3hool000Ooo`04003ooooo0008ool00`00ooooo`04
ool20009ool00`00ooooo`3hool000Ooo`@000Soo`@000?oo`03003ooooo00Ooo`03003ooooo0?Wo
o`006_oo00<00?ooool01ooo00<00?ooool0nOoo000Jool00`00ooooo`07ool00`00ooooo`3iool0
01[oo`8000Soo`03003ooooo0?Woo`006_oo00<00?ooool01ooo00<00?ooool0nOoo000Jool00`00
ooooo`06ool00`00ooooo`3jool001[oo`8000Ooo`03003ooooo0?[oo`006_oo00<00?ooool01_oo
00<00?ooool0n_oo000Jool00`00ooooo`06ool00`00ooooo`3jool001[oo`03003ooooo00Goo`03
003ooooo0?_oo`006_oo0P001_oo00<00?ooool0nooo000=ool40003ool20004ool00`00ooooo`05
ool00`00ooooo`3kool000ooo`05003oooooool00002ool01@00oooooooo00001ooo00<00?ooool0
nooo000?ool01@00oooooooo00000_oo00@00?ooooooo`<000Coo`03003ooooo0?coo`003ooo00D0
0?ooooooo`0000;oo`05003oooooool00006ool00`00ooooo`3lool000ooo`05003oooooool00002
ool01@00oooooooo00001_oo00<00?ooool0o?oo000=ool30003ool01000ooooo`000ooo00<00?oo
ool01?oo00<00?ooool0o?oo000?ool00`00ooooo`02ool20004ool20005ool00`00ooooo`3lool0
01[oo`03003ooooo00?oo`03003ooooo0?goo`006_oo00<00?ooool00ooo00<00?ooool0oOoo000J
ool00`00ooooo`03ool00`00ooooo`3mool001[oo`8000Coo`03003ooooo0?goo`006_oo00<00?oo
ool00_oo00<00?ooool0o_oo000Jool00`00ooooo`02ool00`00ooooo`3nool001[oo`8000?oo`03
003ooooo0?koo`006_oo00<00?ooool00_oo00<00?ooool0o_oo000Jool01@00oooooooo0000oooo
0_oo000Jool01@00oooooooo0000oooo0_oo000Jool20002ool00`00ooooo`3oool00003ool00000
008000;oo`@000?oo`03003ooooo00;oo`<000Coo`05003oooooool0003oool2ool000?oo`05003o
ooooool0000>ool01@00oooooooo00000_oo00<00?ooool0oooo0Ooo0003ool00`00ooooo`02ool0
0`00ooooo`0;ool01000oooooooo1000oooo0ooo0003ool00`00ooooo`03ool00`00ooooo`07ool3
0004ool01000ooooo`00oooo0ooo0003ool00`00ooooo`04ool00`00ooooo`06ool00`00ooooo`04
ool01000ooooo`00oooo0ooo00001?oo000000000ooo00@00?ooool000Soo`03003ooooo00Coo`04
003ooooo003oool3ool000?oo`03003ooooo00;oo`8000Woo`@000?oo`<00?ooo`Coo`006_oo00<0
0?oo0000oooo1?oo000Jool00`00ool0003oool4ool001[oo`03003oo`000?ooo`Coo`006_oo0P00
oooo1Ooo000Jool2003oool5ool001[oo`800?ooo`Goo`006_oo0P00oooo1Ooo000Jool00`00oooo
o`3oool4ool001[oo`03003ooooo0?ooo`Coo`006_oo00<00?ooool0oooo1?oo000Jool2003oool5
ool000goo`@000;oo`<000Coo`03003ooooo0?ooo`Coo`003ooo00<00?ooool01?oo00D00?oooooo
o`000?ooo`Koo`003ooo00<00?ooool01?oo00@00?ooooooo`<00?ooo`Coo`003ooo00@00?oooooo
o`<000Coo`03003ooooo0?ooo`Coo`003ooo00D00?ooooooo`0000Koo`03003ooooo0?ooo`Coo`00
3Ooo0`000ooo00<00?ooool01?oo00<00?ooool0oooo1?oo000?ool01000oooooooo10000ooo0P00
oooo1Ooo000Jool00`00ooooo`3oool4ool001[oo`03003ooooo0?ooo`Coo`006_oo0P00oooo1Ooo
000Jool00`00ooooo`3oool4ool001[oo`03003ooooo0?ooo`Coo`006_oo00<00?ooool0oooo1?oo
000Jool2003oool5ool001[oo`03003ooooo0?ooo`Coo`006_oo00<00?ooool0oooo1?oo000Jool0
0`00ooooo`3oool4ool001[oo`800?ooo`Goo`0000?oo`0000000P000ooo00<00?ooool00ooo00<0
0?ooool00_oo0`001?oo00<00?ooool0oooo1?oo0003ool00`00ooooo`02ool00`00ooooo`0;ool0
1@00oooooooo0000oooo1_oo0003ool00`00ooooo`03ool00`00ooooo`0:ool01000oooooooo0`00
oooo1?oo0003ool00`00ooooo`03ool00`00ooooo`07ool30004ool00`00ooooo`3oool4ool000?o
o`03003ooooo00Coo`03003ooooo00Koo`03003ooooo00Coo`03003ooooo0?ooo`Coo`0000Coo`00
000000?oo`04003ooooo0008ool00`00ooooo`04ool00`00ooooo`3oool4ool000?oo`04003ooooo
ool40008ool40003ool2003oool5ool001[oo`03003ooooo0?ooo`Coo`006_oo00<00?ooool0oooo
1?oo000Jool2003oool5ool001[oo`03003ooooo0?ooo`Coo`006_oo00<00?ooool0oooo1?oo000J
ool00`00ooooo`3oool4ool001[oo`03003ooooo0?ooo`Coo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003o
oolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.380014, -3.68441, \
0.0142057, 0.139748}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "]"}], ",", 
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
                "]"}]}], "}"}], "&"}], "/@", "dps"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Export", "[", 
        RowBox[{"\"\<tmp.txt\>\"", ",", 
          RowBox[{"ToString", "[", "%", "]"}]}], "]"}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shifted Gaussian", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Tau]", "=", "0.413"}], ";", 
      RowBox[{"nn", "=", "12"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"gauss", "[", "t_", "]"}], ":=", 
        RowBox[{"Exp", "[", 
          RowBox[{"-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"3", "-", "t"}], ")"}], "2"]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Plot", "[", 
        RowBox[{
          RowBox[{"gauss", "[", "t", "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"t", ",", "0", ",", 
              RowBox[{"\[Tau]", " ", "nn"}]}], "}"}], ",", 
          RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"-", "1"}], ",", "2"}], "}"}]}]}], "]"}], 
      ";"}]}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.192167 0.206011 0.206011 [
[.21598 .19351 -3 -9 ]
[.21598 .19351 3 0 ]
[.40814 .19351 -3 -9 ]
[.40814 .19351 3 0 ]
[.60031 .19351 -3 -9 ]
[.60031 .19351 3 0 ]
[.79248 .19351 -3 -9 ]
[.79248 .19351 3 0 ]
[.98465 .19351 -3 -9 ]
[.98465 .19351 3 0 ]
[.01131 0 -12 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .10301 -24 -4.5 ]
[.01131 .10301 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .51503 -18 -4.5 ]
[.01131 .51503 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21598 .20601 m
.21598 .21226 L
s
[(1)] .21598 .19351 0 1 Mshowa
.40814 .20601 m
.40814 .21226 L
s
[(2)] .40814 .19351 0 1 Mshowa
.60031 .20601 m
.60031 .21226 L
s
[(3)] .60031 .19351 0 1 Mshowa
.79248 .20601 m
.79248 .21226 L
s
[(4)] .79248 .19351 0 1 Mshowa
.98465 .20601 m
.98465 .21226 L
s
[(5)] .98465 .19351 0 1 Mshowa
.125 Mabswid
.06224 .20601 m
.06224 .20976 L
s
.10068 .20601 m
.10068 .20976 L
s
.13911 .20601 m
.13911 .20976 L
s
.17754 .20601 m
.17754 .20976 L
s
.25441 .20601 m
.25441 .20976 L
s
.29284 .20601 m
.29284 .20976 L
s
.33128 .20601 m
.33128 .20976 L
s
.36971 .20601 m
.36971 .20976 L
s
.44658 .20601 m
.44658 .20976 L
s
.48501 .20601 m
.48501 .20976 L
s
.52344 .20601 m
.52344 .20976 L
s
.56188 .20601 m
.56188 .20976 L
s
.63874 .20601 m
.63874 .20976 L
s
.67718 .20601 m
.67718 .20976 L
s
.71561 .20601 m
.71561 .20976 L
s
.75405 .20601 m
.75405 .20976 L
s
.83091 .20601 m
.83091 .20976 L
s
.86935 .20601 m
.86935 .20976 L
s
.90778 .20601 m
.90778 .20976 L
s
.94621 .20601 m
.94621 .20976 L
s
.25 Mabswid
0 .20601 m
1 .20601 L
s
.02381 0 m
.03006 0 L
s
[(-1)] .01131 0 1 0 Mshowa
.02381 .10301 m
.03006 .10301 L
s
[(-0.5)] .01131 .10301 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(0.5)] .01131 .30902 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(1)] .01131 .41202 1 0 Mshowa
.02381 .51503 m
.03006 .51503 L
s
[(1.5)] .01131 .51503 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(2)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .20604 m
.03279 .20604 L
.04262 .20606 L
.05288 .20607 L
.06244 .20609 L
.0731 .20612 L
.0828 .20616 L
.094 .20621 L
.10458 .20628 L
.11428 .20635 L
.12455 .20646 L
.14299 .20673 L
.1524 .20691 L
.16253 .20716 L
.18083 .20777 L
.19093 .20821 L
.20196 .20881 L
.2125 .20952 L
.22205 .21029 L
.24173 .21235 L
.2627 .21542 L
.28195 .21925 L
.30277 .22475 L
.3227 .23157 L
.34377 .24068 L
.38174 .26251 L
.42216 .29324 L
.46107 .32788 L
.50243 .36495 L
.52313 .38133 L
.54228 .39407 L
.56086 .40352 L
.57025 .40704 L
.5753 .40856 L
.58061 .40987 L
.58543 .41079 L
.58798 .41118 L
.5907 .41151 L
.59321 .41174 L
.5955 .41189 L
.5966 .41195 L
.59779 .41199 L
.59892 .41201 L
.59996 .41202 L
.60116 .41202 L
.60245 .412 L
.60367 .41196 L
.60478 .41191 L
.60608 .41184 L
.6075 .41173 L
Mistroke
.61009 .41149 L
.61278 .41116 L
.61565 .41071 L
.62081 .40969 L
.63112 .40679 L
.64226 .40243 L
.66245 .39157 L
.7035 .36042 L
.74304 .32467 L
.78107 .29106 L
.82154 .26075 L
.8605 .23895 L
.88178 .23012 L
.90192 .22355 L
.92187 .21854 L
.9403 .21502 L
.95737 .21254 L
.97619 .2105 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Cool4003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool00ooo0P00oooo1?oo000=ool40004ool00`00ooooo`03ool00`00ooooo`3oool3ool001Go
o`03003ooooo00?oo`03003ooooo0?ooo`?oo`004ooo0`001Ooo00<00?ooool0oooo0ooo000Eool0
0`00ooooo`03ool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo
000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo
000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2
003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4
ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo0008ool20004ool00`00oooo
o`02ool30005ool00`00ooooo`3oool3ool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00
ooooo`3oool3ool000Ooo`04003ooooo000;ool00`00ooooo`02ool2003oool4ool00003ool00000
008000;oo`04003ooooo0008ool30005ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool0
0`00ooooo`05ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00
ooooo`3oool3ool000Soo`8000Woo`@000Coo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo
0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00;ooo1000;ooo1000<?oo0P00<_oo
00<00?ooool0;_oo0`001Ooo000Kool00`00ooooo`0`ool00`00ooooo`0^ool00`00ooooo`0`ool0
1000ooooo`00<Ooo00<00?ooool0<Ooo00<00?ooool00_oo000Kool00`00ooooo`0`ool00`00oooo
o`0_ool00`00ooooo`0bool00`00ooooo`0/ool5000bool00`00ooooo`02ool001_oo`03003ooooo
033oo`03003ooooo033oo`03003ooooo02ooo`8002ooo`04003ooooo000`ool30005ool001_oo`03
003ooooo033oo`03003ooooo037oo`03003ooooo033oo`03003ooooo02goo`03003oo`00033oo`03
003ooooo00Goo`006ooo0P00;ooo0`00<?oo00@00?ooool002ooo`04003ooooo000`ool2000`ool0
0`00ooooo`05ool001_oo`03003ooooo033oo`03003ooooo02ooo`80037oo`8003;oo`03003ooooo
02koo`@000Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003o
oooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Doooo000=000001_oo`03003ooooo00Ooo`03
003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Goo`d000Woo`03003ooooo
00Soo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo
00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo
00Ooo`03003ooooo00Ooo`03003ooooo00Ooo`03003ooooo00Soo`03003ooooo00Ooo`03003ooooo
00Ooo`03003ooooo00Ooo`T00003ool00?oo00Coo`006ooo00<00?ooool0>_oo1@00/?oo10003ooo
000Kool00`00ooooo`0oool4002Vool6000Cool001_oo`03003ooooo04?oo`<009ooo`@001Woo`00
6ooo0P00Aooo1000VOoo0P007Ooo000Kool00`00ooooo`1:ool3002Cool3000Oool001_oo`03003o
oooo04goo`8008koo`<002;oo`006ooo00<00?ooool0Cooo0P00R_oo0P009Ooo000Kool00`00oooo
o`1Aool20026ool2000Wool001_oo`03003ooooo05?oo`8008;oo`8002Woo`006ooo0P00E_oo0P00
Oooo00<00?ooool0:Ooo000Kool00`00ooooo`1Gool00`00ooooo`1jool2000/ool001_oo`03003o
oooo05Soo`8007Soo`8002koo`006ooo00<00?ooool0F_oo0P00MOoo00<00?ooool0;_oo000Kool0
0`00ooooo`1Lool00`00ooooo`1`ool2000aool001_oo`8005koo`8006ooo`03003ooooo037oo`00
6ooo00<00?ooool0Gooo00<00?ooool0Jooo00<00?ooool0<_oo000Kool00`00ooooo`1Pool00`00
ooooo`1Xool2000eool001_oo`03003ooooo067oo`8006Ooo`03003ooooo03Goo`006ooo00<00?oo
ool0Hooo00<00?ooool0Hooo00<00?ooool0=_oo000Kool00`00ooooo`1Tool00`00ooooo`1Pool2
000iool001_oo`8006Koo`8005ooo`03003ooooo03Woo`006ooo00<00?ooool0Iooo00<00?ooool0
Fooo00<00?ooool0>_oo000Kool00`00ooooo`1Xool00`00ooooo`1Iool00`00ooooo`0kool000So
o`8000Coo`03003ooooo00;oo`<000Goo`03003ooooo06Woo`03003ooooo05Ooo`03003ooooo03co
o`001ooo00@00?ooool000_oo`03003ooooo00;oo`03003ooooo06[oo`03003ooooo05Coo`8003oo
o`001ooo00@00?ooool000_oo`03003ooooo00;oo`8006coo`03003ooooo05;oo`03003ooooo03oo
o`001ooo00@00?ooool000Soo`<000Goo`03003ooooo06coo`80057oo`03003ooooo043oo`001ooo
00@00?ooool000Soo`03003ooooo00Goo`03003ooooo06koo`03003ooooo04goo`03003ooooo047o
o`001ooo00@00?ooool000Soo`03003ooooo00Goo`03003ooooo06ooo`03003ooooo04_oo`03003o
oooo04;oo`002?oo0P002Ooo10001?oo00<00?ooool0L?oo00<00?ooool0BOoo00<00?ooool0@ooo
000Kool00`00ooooo`1aool00`00ooooo`17ool00`00ooooo`14ool001_oo`8007?oo`03003ooooo
04Goo`03003ooooo04Goo`006ooo00<00?ooool0Looo00<00?ooool0@ooo00<00?ooool0A_oo000K
ool00`00ooooo`1dool00`00ooooo`11ool00`00ooooo`17ool001_oo`03003ooooo07Goo`03003o
oooo03ooo`03003ooooo04Soo`006ooo00<00?ooool0M_oo0P00?_oo00<00?ooool0BOoo000Kool2
001iool00`00ooooo`0jool00`00ooooo`1:ool001_oo`03003ooooo07Woo`03003ooooo03Soo`03
003ooooo04_oo`006ooo00<00?ooool0N_oo00<00?ooool0=_oo00<00?ooool0C?oo000Kool00`00
ooooo`1kool00`00ooooo`0dool00`00ooooo`1=ool001_oo`03003ooooo07coo`03003ooooo037o
o`80053oo`006ooo0P00O_oo0P00<?oo00<00?ooool0D?oo000Kool00`00ooooo`1oool00`00oooo
o`0/ool00`00ooooo`1Aool001_oo`03003ooooo083oo`8002[oo`8005Coo`006ooo00<00?ooool0
P_oo00<00?ooool09_oo00<00?ooool0E?oo000Kool00`00ooooo`23ool00`00ooooo`0Tool00`00
ooooo`1Eool001_oo`03003ooooo08Coo`8002;oo`8005Soo`006ooo0P00Qooo00<00?ooool07Ooo
0P00F_oo000Kool00`00ooooo`27ool2000Kool2001Lool001_oo`03003ooooo08Woo`8001Ooo`80
05koo`004ooo10001?oo00<00?ooool0Rooo0P004ooo0P00H?oo000Eool00`00ooooo`03ool00`00
ooooo`2=ool2000>ool3001Rool001Goo`03003ooooo00?oo`80093oo`h006Goo`005Ooo00<00?oo
ool00ooo00<00?ooool0oooo0ooo000Eool00`00ooooo`03ool00`00ooooo`3oool3ool001?oo`<0
00Goo`03003ooooo0?ooo`?oo`005Ooo00<00?ooool00ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool0
01_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool0
01_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03
003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03
003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo
0?ooo`?oo`006ooo00<00?ooool0oooo0ooo0007ool40003ool00`00ooooo`02ool30005ool00`00
ooooo`3oool3ool000Woo`03003ooooo00[oo`03003ooooo00;oo`03003ooooo0?ooo`?oo`002Ooo
00<00?ooool02_oo00<00?ooool00_oo0P00oooo1?oo0009ool00`00ooooo`07ool30005ool00`00
ooooo`3oool3ool000Woo`03003ooooo00Ooo`03003ooooo00Goo`03003ooooo0?ooo`?oo`001ooo
0`002Ooo00<00?ooool01Ooo00<00?ooool0oooo0ooo0009ool00`00ooooo`07ool40004ool00`00
ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3o
ool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool0
01_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03
003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03
003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4ool001_oo`03003ooooo
0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Cool40004ool00`00ooooo`3oool3ool001?oo`03
003ooooo00Goo`03003ooooo0?ooo`?oo`005?oo00<00?ooool01?oo0P00oooo1?oo000Eool00`00
ooooo`3oool9ool001Koo`03003ooooo0?ooo`Soo`004ooo00@00?ooool00?ooo`[oo`005?oo0P00
oooo2ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.534577, -1.1144, \
0.0195628, 0.0182481}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"Off", "[", 
        RowBox[{
          RowBox[{"Integrate", "::", "idiv"}], ",", 
          RowBox[{"NIntegrate", "::", "ncvb"}]}], "]"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"ps", "=", 
        RowBox[{"powerSpec", "[", 
          RowBox[{"gauss", ",", "nn", ",", "\[Tau]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"On", "[", 
        RowBox[{
          RowBox[{"NIntegrate", "::", "ncvb"}], ",", 
          RowBox[{"Integrate", "::", "idiv"}]}], "]"}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"ps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "1"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.125202 0 0.618034 [
[.14901 -0.0125 -3 -9 ]
[.14901 -0.0125 3 0 ]
[.27421 -0.0125 -3 -9 ]
[.27421 -0.0125 3 0 ]
[.39942 -0.0125 -3 -9 ]
[.39942 -0.0125 3 0 ]
[.52462 -0.0125 -3 -9 ]
[.52462 -0.0125 3 0 ]
[.64982 -0.0125 -3 -9 ]
[.64982 -0.0125 3 0 ]
[.77502 -0.0125 -3 -9 ]
[.77502 -0.0125 3 0 ]
[.90022 -0.0125 -3 -9 ]
[.90022 -0.0125 3 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.14901 0 m
.14901 .00625 L
s
[(1)] .14901 -0.0125 0 1 Mshowa
.27421 0 m
.27421 .00625 L
s
[(2)] .27421 -0.0125 0 1 Mshowa
.39942 0 m
.39942 .00625 L
s
[(3)] .39942 -0.0125 0 1 Mshowa
.52462 0 m
.52462 .00625 L
s
[(4)] .52462 -0.0125 0 1 Mshowa
.64982 0 m
.64982 .00625 L
s
[(5)] .64982 -0.0125 0 1 Mshowa
.77502 0 m
.77502 .00625 L
s
[(6)] .77502 -0.0125 0 1 Mshowa
.90022 0 m
.90022 .00625 L
s
[(7)] .90022 -0.0125 0 1 Mshowa
.125 Mabswid
.04885 0 m
.04885 .00375 L
s
.07389 0 m
.07389 .00375 L
s
.09893 0 m
.09893 .00375 L
s
.12397 0 m
.12397 .00375 L
s
.17405 0 m
.17405 .00375 L
s
.19909 0 m
.19909 .00375 L
s
.22413 0 m
.22413 .00375 L
s
.24917 0 m
.24917 .00375 L
s
.29925 0 m
.29925 .00375 L
s
.32429 0 m
.32429 .00375 L
s
.34933 0 m
.34933 .00375 L
s
.37437 0 m
.37437 .00375 L
s
.42446 0 m
.42446 .00375 L
s
.4495 0 m
.4495 .00375 L
s
.47454 0 m
.47454 .00375 L
s
.49958 0 m
.49958 .00375 L
s
.54966 0 m
.54966 .00375 L
s
.5747 0 m
.5747 .00375 L
s
.59974 0 m
.59974 .00375 L
s
.62478 0 m
.62478 .00375 L
s
.67486 0 m
.67486 .00375 L
s
.6999 0 m
.6999 .00375 L
s
.72494 0 m
.72494 .00375 L
s
.74998 0 m
.74998 .00375 L
s
.80006 0 m
.80006 .00375 L
s
.8251 0 m
.8251 .00375 L
s
.85014 0 m
.85014 .00375 L
s
.87518 0 m
.87518 .00375 L
s
.92526 0 m
.92526 .00375 L
s
.9503 0 m
.9503 .00375 L
s
.97534 0 m
.97534 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .12361 m
.03006 .12361 L
s
[(0.2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(1)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.18254 .13834 L
.34127 .01241 L
.5 .00022 L
.65873 0 L
.81746 0 L
.97619 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00;ooo10006?oo10006Ooo0P006ooo00<00?ooool06?oo0`00
6_oo0P006_oo00<00?ooool0A?oo000aool00`00ooooo`0Gool00`00ooooo`0Iool01000ooooo`00
6_oo00<00?ooool06ooo00<00?ooool05_oo00@00?ooool001Woo`03003ooooo04Coo`00<Ooo00<0
0?ooool06?oo00<00?ooool06ooo00<00?ooool05Ooo1@007?oo00<00?ooool05_oo00@00?ooool0
01[oo`03003ooooo04?oo`00<Ooo00<00?ooool06Ooo00<00?ooool06?oo0P006?oo00@00?ooool0
01[oo`<001Woo`<001_oo`03003ooooo04?oo`00<Ooo00<00?ooool06_oo00<00?ooool06Ooo00<0
0?ooool05_oo00<00?oo00006_oo00<00?ooool06Ooo00<00?ooool07?oo00<00?ooool0@_oo000_
ool3000Iool01000ooooo`006?oo00@00?ooool001Woo`8001[oo`03003ooooo01Woo`03003ooooo
01Woo`04003ooooo0014ool0037oo`03003ooooo01Soo`8001[oo`8001_oo`03003ooooo01Soo`@0
01Woo`<001Soo`@004Coo`00oooo8Ooo003ooolQool00?[oo`@000;oo`@000;oo`@00003ool00?oo
00?oo`03003ooooo00Goo`@000Coo`003oooh`001ooo00H00?ooool00?oo0007ool00`00ooooo`03
ool01000ooooo`002Ooo00<00?ooool01?oo000Dool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00
ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`02ool=0005ool00`00ooooo`02
ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03
ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03
ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02
ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`0;ool01`00ooooo`00
ool00?oo00@000?oo`03003ooooo00?oo`03003oo`0000Woo`03003ooooo00Coo`005?oo00<00?oo
ool0Booo3000Rooo00<00?ooool00_oo00<00?ooool01ooo00<00?ooool00_oo00<00?oo00002Ooo
00<00?ooool01?oo000Dool00`00ooooo`15ool6002Hool01@00oooooooo00002_oo00<00?ooool0
1@001ooo10001?oo000Dool00`00ooooo`14ool00`00ooooo`2Xool01000ooooo`003ooo00<00?oo
ool01?oo000Dool00`00ooooo`12ool2002/ool2000Aool20004ool001Coo`03003ooooo047oo`03
003ooooo0<Goo`005?oo0P00@Ooo00<00?ooool0a_oo000Dool00`00ooooo`0nool20039ool001Co
o`03003ooooo03goo`03003ooooo0<Woo`005?oo00<00?ooool0??oo00<00?ooool0b_oo000Dool0
0`00ooooo`0kool00`00ooooo`3;ool001Coo`03003ooooo03Woo`800<koo`005?oo00<00?ooool0
>?oo00<00?ooool0c_oo000Dool2000hool00`00ooooo`3?ool001Coo`03003ooooo03Goo`800=;o
o`005?oo00<00?ooool0=?oo00<00?ooool0d_oo000Dool00`00ooooo`0cool00`00ooooo`3Cool0
01Coo`03003ooooo03;oo`03003ooooo0=Coo`005?oo00<00?ooool0<?oo0P00eooo000Dool00`00
ooooo`0_ool00`00ooooo`3Gool001Coo`8002ooo`03003ooooo0=Soo`005?oo00<00?ooool0;?oo
0P00fooo000Dool00`00ooooo`0[ool00`00ooooo`3Kool001Coo`03003ooooo02[oo`03003ooooo
0=coo`005?oo00<00?ooool0:Ooo00<00?ooool0gOoo0002ool20004ool00`00ooooo`02ool40003
ool00`00ooooo`0Wool2003Pool00005ool00?ooool00008ool00`00ooooo`04ool00`00ooooo`0V
ool00`00ooooo`3Pool00005ool00?ooool00009ool00`00ooooo`03ool3000Uool00`00ooooo`3Q
ool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`0Sool2003Tool00005ool00?oo
ool0000;ool01@00oooooooo00009?oo00<00?ooool0i?oo00001Ooo003ooooo00002?oo00@00?oo
ool000?oo`03003ooooo027oo`03003ooooo0>Goo`000_oo0P002_oo0P001?oo00<00?ooool08?oo
00<00?ooool0i_oo000Dool00`00ooooo`0Oool00`00ooooo`3Wool001Coo`03003ooooo01koo`03
003ooooo0>Soo`005?oo0P007_oo00<00?ooool0jOoo000Dool00`00ooooo`0Lool00`00ooooo`3Z
ool001Coo`03003ooooo01_oo`03003ooooo0>_oo`005?oo00<00?ooool06ooo00<00?ooool0jooo
000Dool00`00ooooo`0Jool00`00ooooo`3/ool001Coo`03003ooooo01Woo`03003ooooo0>goo`00
5?oo00<00?ooool06?oo00<00?ooool0k_oo000Dool2000Hool00`00ooooo`3_ool001Coo`03003o
oooo01Koo`03003ooooo0?3oo`005?oo00<00?ooool05Ooo00<00?ooool0lOoo000Dool00`00oooo
o`0Dool00`00ooooo`3bool001Coo`03003ooooo01?oo`03003ooooo0??oo`005?oo00<00?ooool0
4_oo00<00?ooool0m?oo000Dool00`00ooooo`0Aool00`00ooooo`3eool001Coo`80017oo`03003o
oooo0?Koo`005?oo00<00?ooool03ooo00<00?ooool0mooo000Dool00`00ooooo`0?ool00`00oooo
o`3gool001Coo`03003ooooo00koo`03003ooooo0?Soo`005?oo00<00?ooool03Ooo00<00?ooool0
nOoo0002ool20004ool00`00ooooo`04ool00`00ooooo`02ool00`00ooooo`0<ool00`00ooooo`3j
ool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`0;ool00`00ooooo`3kool00005
ool00?ooool00007ool50003ool3000:ool00`00ooooo`3lool00005ool00?ooool00007ool01000
ooooo`001?oo00<00?ooool02Ooo00<00?ooool0oOoo00001Ooo003ooooo00002?oo00<00?oo0000
1?oo00<00?ooool02?oo00<00?ooool0o_oo00001Ooo003ooooo00002Ooo0P001?oo00<00?ooool0
1ooo00<00?ooool0oooo0002ool2000;ool00`00ooooo`02ool00`00ooooo`06ool00`00ooooo`3o
ool1ool001Coo`03003ooooo00Goo`03003ooooo0?ooo`;oo`005?oo00<00?ooool01?oo00<00?oo
ool0oooo0ooo000Dool20004ool00`00ooooo`3oool4ool001Coo`03003ooooo00?oo`03003ooooo
0?ooo`Coo`005?oo00<00?ooool00_oo00<00?ooool0oooo1Ooo000Dool01@00oooooooo0000oooo
2?oo000Dool01000ooooo`00oooo2Ooo000Dool00`00ool0003oool:ool001Coo`800?ooo`_oo`00
5?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`000_oo0P001?oo00<0
0?ooool00ooo0P001?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00@00?ooool000?o
o`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool3003oool:ool00005
ool00?ooool00008ool30004ool00`00ooooo`3oool:ool00005ool00?ooool00008ool00`00oooo
o`04ool00`00ooooo`3oool:ool00005ool00?ooool00008ool00`00ooooo`04ool00`00ooooo`3o
ool:ool000;oo`8000[oo`<000?oo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`000_oo0P001?oo00<00?ooool00ooo0P001?oo00<00?ooool0oooo
2_oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo0?ooo`[oo`0000Goo`00oooo
o`0000Soo`04003ooooo0003ool3003oool:ool00005ool00?ooool00009ool20004ool00`00oooo
o`3oool:ool00005ool00?ooool00008ool01000ooooo`000ooo00<00?ooool0oooo2_oo00001Ooo
003ooooo00002?oo00@00?ooool000?oo`03003ooooo0?ooo`[oo`000_oo0P002_oo0P001?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00
oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`003Ooo10000ooo00<00?ooool0oooo2_oo
000?ool00`00ooooo`02ool00`00ooooo`3oool:ool000ooo`03003ooooo00;oo`<00?ooo`[oo`00
3ooo00<00?ooool0oooo3ooo000?ool00`00ooooo`3oool?ool000goo`<00?oooa7oo`003ooo00<0
0?ooool0oooo3ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo
003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.736283, -0.159885, \
0.035356, 0.00716245}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"samp", "=", 
        RowBox[{"sample", "[", 
          RowBox[{"gauss", ",", "nn", ",", "\[Tau]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"dps", "=", 
        RowBox[{"discretePowerSpec", "[", 
          RowBox[{"samp", ",", "\[Tau]"}], "]"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"dps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "1"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.125202 0 0.618034 [
[.14901 -0.0125 -3 -9 ]
[.14901 -0.0125 3 0 ]
[.27421 -0.0125 -3 -9 ]
[.27421 -0.0125 3 0 ]
[.39942 -0.0125 -3 -9 ]
[.39942 -0.0125 3 0 ]
[.52462 -0.0125 -3 -9 ]
[.52462 -0.0125 3 0 ]
[.64982 -0.0125 -3 -9 ]
[.64982 -0.0125 3 0 ]
[.77502 -0.0125 -3 -9 ]
[.77502 -0.0125 3 0 ]
[.90022 -0.0125 -3 -9 ]
[.90022 -0.0125 3 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.14901 0 m
.14901 .00625 L
s
[(1)] .14901 -0.0125 0 1 Mshowa
.27421 0 m
.27421 .00625 L
s
[(2)] .27421 -0.0125 0 1 Mshowa
.39942 0 m
.39942 .00625 L
s
[(3)] .39942 -0.0125 0 1 Mshowa
.52462 0 m
.52462 .00625 L
s
[(4)] .52462 -0.0125 0 1 Mshowa
.64982 0 m
.64982 .00625 L
s
[(5)] .64982 -0.0125 0 1 Mshowa
.77502 0 m
.77502 .00625 L
s
[(6)] .77502 -0.0125 0 1 Mshowa
.90022 0 m
.90022 .00625 L
s
[(7)] .90022 -0.0125 0 1 Mshowa
.125 Mabswid
.04885 0 m
.04885 .00375 L
s
.07389 0 m
.07389 .00375 L
s
.09893 0 m
.09893 .00375 L
s
.12397 0 m
.12397 .00375 L
s
.17405 0 m
.17405 .00375 L
s
.19909 0 m
.19909 .00375 L
s
.22413 0 m
.22413 .00375 L
s
.24917 0 m
.24917 .00375 L
s
.29925 0 m
.29925 .00375 L
s
.32429 0 m
.32429 .00375 L
s
.34933 0 m
.34933 .00375 L
s
.37437 0 m
.37437 .00375 L
s
.42446 0 m
.42446 .00375 L
s
.4495 0 m
.4495 .00375 L
s
.47454 0 m
.47454 .00375 L
s
.49958 0 m
.49958 .00375 L
s
.54966 0 m
.54966 .00375 L
s
.5747 0 m
.5747 .00375 L
s
.59974 0 m
.59974 .00375 L
s
.62478 0 m
.62478 .00375 L
s
.67486 0 m
.67486 .00375 L
s
.6999 0 m
.6999 .00375 L
s
.72494 0 m
.72494 .00375 L
s
.74998 0 m
.74998 .00375 L
s
.80006 0 m
.80006 .00375 L
s
.8251 0 m
.8251 .00375 L
s
.85014 0 m
.85014 .00375 L
s
.87518 0 m
.87518 .00375 L
s
.92526 0 m
.92526 .00375 L
s
.9503 0 m
.9503 .00375 L
s
.97534 0 m
.97534 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .12361 m
.03006 .12361 L
s
[(0.2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(1)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30529 m
.18254 .1404 L
.34127 .01216 L
.5 .00021 L
.65873 1e-05 L
.81746 1e-05 L
.97619 1e-05 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00;ooo10006?oo10006Ooo0P006ooo00<00?ooool06?oo0`00
6_oo0P006_oo00<00?ooool0A?oo000aool00`00ooooo`0Gool00`00ooooo`0Iool01000ooooo`00
6_oo00<00?ooool06ooo00<00?ooool05_oo00@00?ooool001Woo`03003ooooo04Coo`00<Ooo00<0
0?ooool06?oo00<00?ooool06ooo00<00?ooool05Ooo1@007?oo00<00?ooool05_oo00@00?ooool0
01[oo`03003ooooo04?oo`00<Ooo00<00?ooool06Ooo00<00?ooool06?oo0P006?oo00@00?ooool0
01[oo`<001Woo`<001_oo`03003ooooo04?oo`00<Ooo00<00?ooool06_oo00<00?ooool06Ooo00<0
0?ooool05_oo00<00?oo00006_oo00<00?ooool06Ooo00<00?ooool07?oo00<00?ooool0@_oo000_
ool3000Iool01000ooooo`006?oo00@00?ooool001Woo`8001[oo`03003ooooo01Woo`03003ooooo
01Woo`04003ooooo0014ool0037oo`03003ooooo01Soo`8001[oo`8001_oo`03003ooooo01Soo`@0
01Woo`<001Soo`@004Coo`00oooo8Ooo003ooolQool00?[oo`@000;oo`@000;oo`@00003ool00?oo
00?oo`03003ooooo00Goo`@000Coo`003oooh`001ooo00H00?ooool00?oo0007ool00`00ooooo`03
ool01000ooooo`002Ooo00<00?ooool01?oo000Dool00`00ooooo`03ool00`00ooooo`03ool00`00
ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00
ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00
ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`02ool=0005ool00`00ooooo`02
ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03
ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03
ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02
ool00`00ooooo`03ool00`00ooooo`03ool00`00ooooo`02ool00`00ooooo`0;ool01`00ooooo`00
ool00?oo00@000?oo`03003ooooo00?oo`03003oo`0000Woo`03003ooooo00Coo`005?oo00<00?oo
ool0Booo3000Rooo00<00?ooool00_oo00<00?ooool01ooo00<00?ooool00_oo00<00?oo00002Ooo
00<00?ooool01?oo000Dool00`00ooooo`15ool6002Hool01@00oooooooo00002_oo00<00?ooool0
1@001ooo10001?oo000Dool00`00ooooo`14ool00`00ooooo`2Xool01000ooooo`003ooo00<00?oo
ool01?oo000Dool00`00ooooo`12ool2002/ool2000Aool20004ool001Coo`03003ooooo047oo`03
003ooooo0<Goo`005?oo0P00@Ooo00<00?ooool0a_oo000Dool00`00ooooo`0oool00`00ooooo`37
ool001Coo`03003ooooo03goo`800<[oo`005?oo00<00?ooool0??oo00<00?ooool0b_oo000Dool0
0`00ooooo`0kool00`00ooooo`3;ool001Coo`03003ooooo03[oo`03003ooooo0<coo`005?oo00<0
0?ooool0>?oo0P00cooo000Dool2000hool00`00ooooo`3?ool001Coo`03003ooooo03Koo`03003o
oooo0=3oo`005?oo00<00?ooool0=Ooo00<00?ooool0dOoo000Dool00`00ooooo`0cool2003Dool0
01Coo`03003ooooo03;oo`03003ooooo0=Coo`005?oo00<00?ooool0<Ooo00<00?ooool0eOoo000D
ool00`00ooooo`0`ool00`00ooooo`3Fool001Coo`80033oo`03003ooooo0=Ooo`005?oo00<00?oo
ool0;Ooo0P00f_oo000Dool00`00ooooo`0/ool00`00ooooo`3Jool001Coo`03003ooooo02_oo`03
003ooooo0=_oo`005?oo00<00?ooool0:_oo00<00?ooool0g?oo0002ool20004ool00`00ooooo`02
ool40003ool00`00ooooo`0Xool2003Oool00005ool00?ooool00008ool00`00ooooo`04ool00`00
ooooo`0Wool00`00ooooo`3Oool00005ool00?ooool00009ool00`00ooooo`03ool3000Vool00`00
ooooo`3Pool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`0Uool00`00ooooo`3Q
ool00005ool00?ooool0000;ool01@00oooooooo00009Ooo0P00i?oo00001Ooo003ooooo00002?oo
00@00?ooool000?oo`03003ooooo02;oo`03003ooooo0>Coo`000_oo0P002_oo0P001?oo00<00?oo
ool08Ooo00<00?ooool0iOoo000Dool00`00ooooo`0Pool00`00ooooo`3Vool001Coo`03003ooooo
01ooo`03003ooooo0>Ooo`005?oo0P007ooo00<00?ooool0j?oo000Dool00`00ooooo`0Mool00`00
ooooo`3Yool001Coo`03003ooooo01coo`03003ooooo0>[oo`005?oo00<00?ooool06ooo00<00?oo
ool0jooo000Dool00`00ooooo`0Jool00`00ooooo`3/ool001Coo`03003ooooo01Woo`03003ooooo
0>goo`005?oo00<00?ooool06?oo00<00?ooool0k_oo000Dool2000Hool00`00ooooo`3_ool001Co
o`03003ooooo01Koo`03003ooooo0?3oo`005?oo00<00?ooool05Ooo00<00?ooool0lOoo000Dool0
0`00ooooo`0Dool00`00ooooo`3bool001Coo`03003ooooo01?oo`03003ooooo0??oo`005?oo00<0
0?ooool04_oo00<00?ooool0m?oo000Dool00`00ooooo`0Aool00`00ooooo`3eool001Coo`80017o
o`03003ooooo0?Koo`005?oo00<00?ooool03ooo00<00?ooool0mooo000Dool00`00ooooo`0?ool0
0`00ooooo`3gool001Coo`03003ooooo00koo`03003ooooo0?Soo`005?oo00<00?ooool03Ooo00<0
0?ooool0nOoo0002ool20004ool00`00ooooo`04ool00`00ooooo`02ool00`00ooooo`0<ool00`00
ooooo`3jool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`0;ool00`00ooooo`3k
ool00005ool00?ooool00007ool50003ool3000:ool00`00ooooo`3lool00005ool00?ooool00007
ool01000ooooo`001?oo00<00?ooool02Ooo00<00?ooool0oOoo00001Ooo003ooooo00002?oo00<0
0?oo00001?oo00<00?ooool02?oo00<00?ooool0o_oo00001Ooo003ooooo00002Ooo0P001?oo00<0
0?ooool01ooo00<00?ooool0oooo0002ool2000;ool00`00ooooo`02ool00`00ooooo`06ool00`00
ooooo`3oool1ool001Coo`03003ooooo00Goo`03003ooooo0?ooo`;oo`005?oo00<00?ooool01?oo
00<00?ooool0oooo0ooo000Dool20004ool00`00ooooo`3oool4ool001Coo`03003ooooo00;oo`03
003ooooo0?ooo`Goo`005?oo00D00?ooooooo`000?ooo`Soo`005?oo00@00?ooool00?ooo`Woo`00
5?oo00<00?oo0000oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00
oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`000_oo0P001?oo00<00?ooool0
0ooo0P001?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03003o
oooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool3003oool:ool00005ool00?oo
ool00008ool30004ool00`00ooooo`3oool:ool00005ool00?ooool00008ool00`00ooooo`04ool0
0`00ooooo`3oool:ool00005ool00?ooool00008ool00`00ooooo`04ool00`00ooooo`3oool:ool0
00;oo`8000[oo`<000?oo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`000_oo0P001?oo00<00?ooool00ooo0P001?oo00<00?ooool0oooo2_oo0000
1Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000So
o`04003ooooo0003ool3003oool:ool00005ool00?ooool00009ool20004ool00`00ooooo`3oool:
ool00005ool00?ooool00008ool01000ooooo`000ooo00<00?ooool0oooo2_oo00001Ooo003ooooo
00002?oo00@00?ooool000?oo`03003ooooo0?ooo`[oo`000_oo0P002_oo0P001?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo
000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00oooo
o`3oool:ool001Coo`03003ooooo0?ooo`[oo`003Ooo10000ooo00<00?ooool0oooo2_oo000?ool0
0`00ooooo`02ool00`00ooooo`3oool:ool000ooo`03003ooooo00;oo`<00?ooo`[oo`003ooo00<0
0?ooool0oooo3ooo000?ool00`00ooooo`3oool?ool000goo`<00?oooa7oo`003ooo00<00?ooool0
oooo3ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.736283, -0.159885, \
0.035356, 0.00716245}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "]"}], ",", 
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
                "]"}]}], "}"}], "&"}], "/@", "dps"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Export", "[", 
        RowBox[{"\"\<tmp.txt\>\"", ",", 
          RowBox[{"ToString", "[", "%", "]"}]}], "]"}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shifted Gaussian whose main part is only partially sampled", \
"Subsubsection"],

Cell["\<\
The discrete Fourier transform of the sampled part is approximately \
equal to the Fourier transform of the corresponding interval.\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Tau]", "=", "0.25"}], ";", 
      RowBox[{"nn", "=", "12"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"gauss", "[", "t_", "]"}], ":=", 
        RowBox[{"Exp", "[", 
          RowBox[{"-", 
            SuperscriptBox[
              RowBox[{"(", 
                RowBox[{"3", "-", "t"}], ")"}], "2"]}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Plot", "[", 
        RowBox[{
          RowBox[{"gauss", "[", "t", "]"}], ",", 
          RowBox[{"{", 
            RowBox[{"t", ",", "0", ",", 
              RowBox[{"\[Tau]", " ", "nn"}]}], "}"}], ",", 
          RowBox[{"PlotRange", "\[Rule]", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"-", "1"}], ",", "2"}], "}"}]}]}], "]"}], 
      ";"}]}], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.31746 0.206011 0.206011 [
[.18254 .19351 -9 -9 ]
[.18254 .19351 9 0 ]
[.34127 .19351 -3 -9 ]
[.34127 .19351 3 0 ]
[.5 .19351 -9 -9 ]
[.5 .19351 9 0 ]
[.65873 .19351 -3 -9 ]
[.65873 .19351 3 0 ]
[.81746 .19351 -9 -9 ]
[.81746 .19351 9 0 ]
[.97619 .19351 -3 -9 ]
[.97619 .19351 3 0 ]
[.01131 0 -12 -4.5 ]
[.01131 0 0 4.5 ]
[.01131 .10301 -24 -4.5 ]
[.01131 .10301 0 4.5 ]
[.01131 .30902 -18 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .51503 -18 -4.5 ]
[.01131 .51503 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .20601 m
.18254 .21226 L
s
[(0.5)] .18254 .19351 0 1 Mshowa
.34127 .20601 m
.34127 .21226 L
s
[(1)] .34127 .19351 0 1 Mshowa
.5 .20601 m
.5 .21226 L
s
[(1.5)] .5 .19351 0 1 Mshowa
.65873 .20601 m
.65873 .21226 L
s
[(2)] .65873 .19351 0 1 Mshowa
.81746 .20601 m
.81746 .21226 L
s
[(2.5)] .81746 .19351 0 1 Mshowa
.97619 .20601 m
.97619 .21226 L
s
[(3)] .97619 .19351 0 1 Mshowa
.125 Mabswid
.05556 .20601 m
.05556 .20976 L
s
.0873 .20601 m
.0873 .20976 L
s
.11905 .20601 m
.11905 .20976 L
s
.15079 .20601 m
.15079 .20976 L
s
.21429 .20601 m
.21429 .20976 L
s
.24603 .20601 m
.24603 .20976 L
s
.27778 .20601 m
.27778 .20976 L
s
.30952 .20601 m
.30952 .20976 L
s
.37302 .20601 m
.37302 .20976 L
s
.40476 .20601 m
.40476 .20976 L
s
.43651 .20601 m
.43651 .20976 L
s
.46825 .20601 m
.46825 .20976 L
s
.53175 .20601 m
.53175 .20976 L
s
.56349 .20601 m
.56349 .20976 L
s
.59524 .20601 m
.59524 .20976 L
s
.62698 .20601 m
.62698 .20976 L
s
.69048 .20601 m
.69048 .20976 L
s
.72222 .20601 m
.72222 .20976 L
s
.75397 .20601 m
.75397 .20976 L
s
.78571 .20601 m
.78571 .20976 L
s
.84921 .20601 m
.84921 .20976 L
s
.88095 .20601 m
.88095 .20976 L
s
.9127 .20601 m
.9127 .20976 L
s
.94444 .20601 m
.94444 .20976 L
s
.25 Mabswid
0 .20601 m
1 .20601 L
s
.02381 0 m
.03006 0 L
s
[(-1)] .01131 0 1 0 Mshowa
.02381 .10301 m
.03006 .10301 L
s
[(-0.5)] .01131 .10301 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(0.5)] .01131 .30902 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(1)] .01131 .41202 1 0 Mshowa
.02381 .51503 m
.03006 .51503 L
s
[(1.5)] .01131 .51503 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(2)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0206 m
.02756 .0206 L
s
.02381 .0412 m
.02756 .0412 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .0824 m
.02756 .0824 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .14421 m
.02756 .14421 L
s
.02381 .16481 m
.02756 .16481 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26781 m
.02756 .26781 L
s
.02381 .28842 m
.02756 .28842 L
s
.02381 .32962 m
.02756 .32962 L
s
.02381 .35022 m
.02756 .35022 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .45322 m
.02756 .45322 L
s
.02381 .47383 m
.02756 .47383 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .53563 m
.02756 .53563 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57683 m
.02756 .57683 L
s
.02381 .59743 m
.02756 .59743 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .20604 m
.04262 .20605 L
.06244 .20606 L
.08255 .20609 L
.10458 .20612 L
.12415 .20616 L
.14509 .20623 L
.16639 .20632 L
.18653 .20643 L
.20651 .20659 L
.22495 .20677 L
.24367 .20702 L
.2643 .20736 L
.28371 .20778 L
.30459 .20836 L
.32572 .20911 L
.34581 .21001 L
.384 .21236 L
.40368 .21398 L
.42465 .21608 L
.46623 .22161 L
.5063 .22905 L
.54485 .23853 L
.58585 .25144 L
.62534 .26675 L
.66331 .284 L
.70374 .30464 L
.74264 .32592 L
.78401 .34881 L
.82385 .36965 L
.86218 .38709 L
.88156 .39451 L
.90296 .40135 L
.91271 .40395 L
.92321 .40637 L
.9323 .40812 L
.94223 .40968 L
.9508 .41071 L
.95877 .4114 L
.96318 .41168 L
.96719 .41186 L
.96941 .41193 L
.97058 .41196 L
.97182 .41198 L
.9729 .412 L
.97409 .41201 L
.97518 .41202 L
.97619 .41202 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo000Cool4003oool:ool001Goo`03003ooooo0?ooo`Woo`005Ooo00<0
0?ooool00ooo0P00oooo1?oo000=ool40004ool00`00ooooo`03ool00`00ooooo`3oool3ool001Go
o`03003ooooo00?oo`03003ooooo0?ooo`?oo`004ooo0`001Ooo00<00?ooool0oooo0ooo000Eool0
0`00ooooo`03ool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo
000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo
000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000Kool0
0`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2
003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00oooo
o`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool2003oool4
ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo0008ool20004ool00`00oooo
o`02ool30005ool00`00ooooo`3oool3ool000Ooo`04003ooooo000;ool00`00ooooo`02ool00`00
ooooo`3oool3ool000Ooo`04003ooooo000;ool00`00ooooo`02ool2003oool4ool00003ool00000
008000;oo`04003ooooo0008ool30005ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool0
0`00ooooo`05ool00`00ooooo`3oool3ool000Ooo`04003ooooo0008ool00`00ooooo`05ool00`00
ooooo`3oool3ool000Soo`8000Woo`@000Coo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo
0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P00oooo1?oo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00
ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo0P008Ooo0P001?oo00<00?ooool00_oo
0`008Ooo10008?oo10000ooo00<00?ooool00_oo0`008_oo10008?oo10000ooo00<00?ooool00_oo
0`008_oo0P001ooo000Kool00`00ooooo`0Oool01000ooooo`002ooo00<00?ooool08?oo00<00?oo
ool08Ooo00<00?ooool02_oo00<00?ooool07ooo00<00?ooool08Ooo00<00?ooool03?oo00<00?oo
ool07_oo00@00?ooool000Koo`006ooo00<00?ooool07ooo00@00?ooool000_oo`03003ooooo023o
o`03003ooooo027oo`03003ooooo00[oo`03003ooooo023oo`03003ooooo027oo`03003ooooo00_o
o`03003ooooo027oo`03003ooooo00Coo`006ooo00<00?ooool07ooo00@00?ooool000Soo`<002?o
o`03003ooooo027oo`03003ooooo00Ooo`<002Coo`03003ooooo027oo`03003ooooo00Ooo`<002;o
o`8000Ooo`006ooo00<00?ooool07ooo00@00?ooool000Soo`03003ooooo02?oo`03003ooooo027o
o`03003ooooo00Ooo`03003ooooo02Goo`03003ooooo027oo`03003ooooo00Koo`03003ooooo02Co
o`03003ooooo00Coo`006ooo0P008?oo00@00?ooool000Soo`03003ooooo027oo`<0027oo`<000Wo
o`03003ooooo02;oo`04003ooooo000Pool01000ooooo`002?oo00<00?ooool08Ooo00@00?ooool0
00Koo`006ooo00<00?ooool08?oo0P002Ooo10008_oo00<00?ooool08Ooo00<00?ooool01ooo1000
8_oo0P008_oo0P002Ooo10008Ooo0P001ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo
0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001Coool000d00000
6ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool0
1Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool0
1?oo40000_oo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<0
0?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<0
0?ooool01_oo00<00?ooool01Ooo00<00?ooool01_oo00<00?ooool01Ooo00<00?ooool01Ooo000K
ool00`00ooooo`1Pool:002Hool001_oo`03003ooooo06[oo`H009;oo`006ooo00<00?ooool0L?oo
1P00S?oo000Kool2001gool50027ool001_oo`03003ooooo07_oo`D008;oo`006ooo00<00?ooool0
P?oo1@00OOoo000Kool00`00ooooo`25ool4001iool001_oo`03003ooooo08Woo`<007Koo`006ooo
00<00?ooool0S?oo0P00M?oo000Kool2002?ool3001aool001_oo`03003ooooo097oo`<006koo`00
6ooo00<00?ooool0U?oo0`00Jooo000Kool00`00ooooo`2Gool2001Yool001_oo`03003ooooo09Wo
o`<006Koo`006ooo0P00WOoo0`00Hooo000Kool00`00ooooo`2Oool2001Qool001_oo`03003ooooo
0:7oo`<005koo`006ooo00<00?ooool0Y?oo0P00G?oo000Kool00`00ooooo`2Vool2001Jool001_o
o`03003ooooo0:Soo`8005Soo`006ooo0P00Zooo0P00E_oo000Kool00`00ooooo`2/ool2001Dool0
01_oo`03003ooooo0:koo`8005;oo`002?oo0P001?oo00<00?ooool00_oo0`001Ooo00<00?ooool0
/?oo0P00D?oo0007ool01000ooooo`002ooo00<00?ooool00_oo00<00?ooool0/_oo00<00?ooool0
COoo0007ool01000ooooo`002ooo00<00?ooool00_oo0P00]?oo0P00COoo0007ool01000ooooo`00
2?oo0`001Ooo00<00?ooool0]Ooo0P00Booo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<0
0?ooool0]ooo00<00?ooool0B?oo0007ool01000ooooo`002?oo00<00?ooool01Ooo00<00?ooool0
^?oo0P00B?oo0008ool20009ool40004ool00`00ooooo`2jool20016ool001_oo`03003ooooo0;co
o`03003ooooo04?oo`006ooo0P00__oo0P00@ooo000Kool00`00ooooo`2oool20011ool001_oo`03
003ooooo0<7oo`8003ooo`006ooo00<00?ooool0`ooo0P00?Ooo000Kool00`00ooooo`35ool2000k
ool001_oo`800<Soo`8003Woo`006ooo00<00?ooool0bOoo0P00=ooo000Kool00`00ooooo`3;ool2
000eool001_oo`03003ooooo0<goo`8003?oo`006ooo00<00?ooool0cooo0P00<Ooo000Kool2003B
ool2000_ool001_oo`03003ooooo0=?oo`8002goo`006ooo00<00?ooool0eOoo0P00:ooo000Kool0
0`00ooooo`3Gool2000Yool001_oo`03003ooooo0=Woo`8002Ooo`006ooo00<00?ooool0fooo0`00
9?oo000Kool2003Oool2000Rool001_oo`03003ooooo0>3oo`<001ooo`006ooo00<00?ooool0hooo
0`007?oo000Cool40004ool00`00ooooo`3Vool5000Gool001Goo`03003ooooo00?oo`03003ooooo
0>_oo`D001;oo`005Ooo00<00?ooool00ooo0P00lOoo2`001ooo000Eool00`00ooooo`03ool00`00
ooooo`3oool3ool001Goo`03003ooooo00?oo`03003ooooo0?ooo`?oo`004ooo0`001Ooo00<00?oo
ool0oooo0ooo000Eool00`00ooooo`03ool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?o
o`006ooo0P00oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo
00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?oo
ool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?oo
ool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo
0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo
0ooo000Kool2003oool4ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool000Ooo`@000?oo`03003ooooo00;oo`<000Goo`03003ooooo0?ooo`?o
o`002Ooo00<00?ooool02_oo00<00?ooool00_oo00<00?ooool0oooo0ooo0009ool00`00ooooo`0:
ool00`00ooooo`02ool2003oool4ool000Woo`03003ooooo00Ooo`<000Goo`03003ooooo0?ooo`?o
o`002Ooo00<00?ooool01ooo00<00?ooool01Ooo00<00?ooool0oooo0ooo0007ool30009ool00`00
ooooo`05ool00`00ooooo`3oool3ool000Woo`03003ooooo00Ooo`@000Coo`03003ooooo0?ooo`?o
o`006ooo0P00oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo
00<00?ooool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo
0P00oooo1?oo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?oo
ool0oooo0ooo000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo
0ooo000Kool00`00ooooo`3oool3ool001_oo`03003ooooo0?ooo`?oo`006ooo00<00?ooool0oooo
0ooo000Kool00`00ooooo`3oool3ool001_oo`800?ooo`Coo`006ooo00<00?ooool0oooo0ooo000K
ool00`00ooooo`3oool3ool001?oo`@000Coo`03003ooooo0?ooo`?oo`004ooo00<00?ooool01Ooo
00<00?ooool0oooo0ooo000Dool00`00ooooo`04ool2003oool4ool001Goo`03003ooooo0?ooo`Wo
o`005_oo00<00?ooool0oooo2?oo000Cool01000ooooo`00oooo2_oo000Dool2003oool;ool00?oo
ob7oo`00oooo8Ooo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.323594, -1.1144, \
0.0118419, 0.0182481}}]
}, Open  ]],

Cell["Power spectrum of the entire main part:", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"Off", "[", 
        RowBox[{
          RowBox[{"Integrate", "::", "idiv"}], ",", 
          RowBox[{"NIntegrate", "::", "ncvb"}]}], "]"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"psWhole", "=", 
        RowBox[{"powerSpec", "[", 
          RowBox[{"gauss", ",", "nn", ",", "\[Tau]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"On", "[", 
        RowBox[{
          RowBox[{"NIntegrate", "::", "ncvb"}], ",", 
          RowBox[{"Integrate", "::", "idiv"}]}], "]"}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"psWhole", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "1"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0833669 0 0.618034 [
[.19054 -0.0125 -3 -9 ]
[.19054 -0.0125 3 0 ]
[.35728 -0.0125 -3 -9 ]
[.35728 -0.0125 3 0 ]
[.52401 -0.0125 -3 -9 ]
[.52401 -0.0125 3 0 ]
[.69074 -0.0125 -3 -9 ]
[.69074 -0.0125 3 0 ]
[.85748 -0.0125 -6 -9 ]
[.85748 -0.0125 6 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.19054 0 m
.19054 .00625 L
s
[(2)] .19054 -0.0125 0 1 Mshowa
.35728 0 m
.35728 .00625 L
s
[(4)] .35728 -0.0125 0 1 Mshowa
.52401 0 m
.52401 .00625 L
s
[(6)] .52401 -0.0125 0 1 Mshowa
.69074 0 m
.69074 .00625 L
s
[(8)] .69074 -0.0125 0 1 Mshowa
.85748 0 m
.85748 .00625 L
s
[(10)] .85748 -0.0125 0 1 Mshowa
.125 Mabswid
.06549 0 m
.06549 .00375 L
s
.10718 0 m
.10718 .00375 L
s
.14886 0 m
.14886 .00375 L
s
.23223 0 m
.23223 .00375 L
s
.27391 0 m
.27391 .00375 L
s
.31559 0 m
.31559 .00375 L
s
.39896 0 m
.39896 .00375 L
s
.44064 0 m
.44064 .00375 L
s
.48233 0 m
.48233 .00375 L
s
.56569 0 m
.56569 .00375 L
s
.60738 0 m
.60738 .00375 L
s
.64906 0 m
.64906 .00375 L
s
.73243 0 m
.73243 .00375 L
s
.77411 0 m
.77411 .00375 L
s
.81579 0 m
.81579 .00375 L
s
.89916 0 m
.89916 .00375 L
s
.94085 0 m
.94085 .00375 L
s
.98253 0 m
.98253 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .12361 m
.03006 .12361 L
s
[(0.2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(1)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.21429 .02272 L
.40476 1e-05 L
.59524 0 L
.78571 0 L
.97619 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00>?oo10009?oo00<00?ooool08Ooo0P009?oo0P008?oo1000
0ooo0P00Booo000hool00`00ooooo`0Uool00`00ooooo`0Pool01000ooooo`008_oo00@00?ooool0
027oo`05003oooooool00002ool00`00ooooo`18ool003Woo`03003ooooo027oo`D0027oo`04003o
oooo000Rool01000ooooo`008Ooo00D00?ooooooo`0000;oo`03003ooooo04Soo`00>_oo00<00?oo
ool08?oo00@00?ooool002;oo`<002Coo`8002;oo`05003oooooool00002ool00`00ooooo`18ool0
03_oo`03003ooooo023oo`03003oo`0002;oo`03003ooooo02?oo`04003ooooo000Qool01@00oooo
oooo00000_oo00<00?ooool0B?oo000hool01000ooooo`008ooo0P008_oo00<00?ooool08ooo00@0
0?ooool001ooo`<000?oo`04003ooooo001:ool003Woo`8002Goo`03003ooooo027oo`<002?oo`80
02;oo`03003ooooo00;oo`8004_oo`00oooo8Ooo003ooolQool00?[oo`@000;oo`@000;oo`@00003
ool00?oo00?oo`03003ooooo00Goo`@000Coo`003oooh`001ooo00H00?ooool00?oo0007ool00`00
ooooo`03ool01000ooooo`002Ooo00<00?ooool01?oo000Dool00`00ooooo`07ool00`00ooooo`06
ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`06
ool00`00ooooo`05ool80003ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`06ool00`00
ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`06ool00`00
ooooo`07ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`06ool00`00
ooooo`07ool00`00ooooo`09ool01`00ooooo`00ool00?oo00@000?oo`03003ooooo00?oo`03003o
o`0000Woo`03003ooooo00Coo`005?oo00<00?ooool0?_oo2@00Vooo00<00?ooool00_oo00<00?oo
ool01ooo00<00?ooool00_oo00<00?oo00002Ooo00<00?ooool01?oo000Dool00`00ooooo`0eool9
002Uool01@00oooooooo00002_oo00<00?ooool01@001ooo10001?oo000Dool00`00ooooo`0]ool8
002jool01000ooooo`003ooo00<00?ooool01?oo000Dool00`00ooooo`0Xool50033ool2000Aool2
0004ool001Coo`03003ooooo02Ooo`03003ooooo0=ooo`005?oo0P00:?oo00<00?ooool0gooo000D
ool00`00ooooo`0Vool00`00ooooo`3Pool001Coo`03003ooooo02Goo`03003ooooo0>7oo`005?oo
00<00?ooool09Ooo00<00?ooool0hOoo000Dool00`00ooooo`0Tool00`00ooooo`3Rool001Coo`03
003ooooo02?oo`03003ooooo0>?oo`005?oo00<00?ooool08ooo00<00?ooool0hooo000Dool2000S
ool00`00ooooo`3Tool001Coo`03003ooooo027oo`03003ooooo0>Goo`005?oo00<00?ooool08Ooo
00<00?ooool0iOoo000Dool00`00ooooo`0Pool00`00ooooo`3Vool001Coo`03003ooooo01ooo`03
003ooooo0>Ooo`005?oo00<00?ooool07ooo00<00?ooool0iooo000Dool00`00ooooo`0Nool00`00
ooooo`3Xool001Coo`8001koo`03003ooooo0>Woo`005?oo00<00?ooool07Ooo00<00?ooool0jOoo
000Dool00`00ooooo`0Lool00`00ooooo`3Zool001Coo`03003ooooo01_oo`03003ooooo0>_oo`00
5?oo00<00?ooool06ooo00<00?ooool0jooo0002ool20004ool00`00ooooo`02ool40003ool00`00
ooooo`0Jool00`00ooooo`3/ool00005ool00?ooool00008ool00`00ooooo`04ool00`00ooooo`0I
ool00`00ooooo`3]ool00005ool00?ooool00009ool00`00ooooo`03ool3000Iool00`00ooooo`3]
ool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`0Hool00`00ooooo`3^ool00005
ool00?ooool0000;ool01@00oooooooo00006Ooo00<00?ooool0kooo00001Ooo003ooooo00002?oo
00@00?ooool000?oo`03003ooooo01Ooo`03003ooooo0>ooo`000_oo0P002_oo0P001?oo00<00?oo
ool05_oo00<00?ooool0l?oo000Dool00`00ooooo`0Eool00`00ooooo`3aool001Coo`03003ooooo
01Goo`03003ooooo0?7oo`005?oo0P005Ooo00<00?ooool0l_oo000Dool00`00ooooo`0Cool00`00
ooooo`3cool001Coo`03003ooooo01?oo`03003ooooo0??oo`005?oo00<00?ooool04_oo00<00?oo
ool0m?oo000Dool00`00ooooo`0Bool00`00ooooo`3dool001Coo`03003ooooo017oo`03003ooooo
0?Goo`005?oo00<00?ooool04?oo00<00?ooool0m_oo000Dool2000Aool00`00ooooo`3fool001Co
o`03003ooooo00ooo`03003ooooo0?Ooo`005?oo00<00?ooool03_oo00<00?ooool0n?oo000Dool0
0`00ooooo`0>ool00`00ooooo`3hool001Coo`03003ooooo00goo`03003ooooo0?Woo`005?oo00<0
0?ooool03?oo00<00?ooool0n_oo000Dool00`00ooooo`0<ool00`00ooooo`3jool001Coo`8000co
o`03003ooooo0?_oo`005?oo00<00?ooool02_oo00<00?ooool0o?oo000Dool00`00ooooo`0:ool0
0`00ooooo`3lool001Coo`03003ooooo00Woo`03003ooooo0?goo`005?oo00<00?ooool02?oo00<0
0?ooool0o_oo0002ool20004ool00`00ooooo`04ool00`00ooooo`02ool00`00ooooo`08ool00`00
ooooo`3nool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`07ool00`00ooooo`3o
ool00005ool00?ooool00007ool50003ool30006ool00`00ooooo`3oool1ool00005ool00?ooool0
0007ool01000ooooo`001?oo00<00?ooool01_oo00<00?ooool0oooo0Ooo00001Ooo003ooooo0000
2?oo00<00?oo00001?oo00<00?ooool01Ooo00<00?ooool0oooo0_oo00001Ooo003ooooo00002Ooo
0P001?oo00<00?ooool01?oo00<00?ooool0oooo0ooo0002ool2000;ool00`00ooooo`02ool00`00
ooooo`04ool00`00ooooo`3oool3ool001Coo`03003ooooo00?oo`03003ooooo0?ooo`Coo`005?oo
00<00?ooool00_oo00<00?ooool0oooo1Ooo000Dool20003ool00`00ooooo`3oool5ool001Coo`05
003oooooool0003oool8ool001Coo`04003ooooo003oool9ool001Coo`04003ooooo003oool9ool0
01Coo`03003oo`000?ooo`[oo`005?oo0P00oooo2ooo000Dool2003oool;ool001Coo`800?ooo`_o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[o
o`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool000;oo`8000Coo`03003ooooo00?oo`80
00Coo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool00`00ooooo`3o
ool:ool00005ool00?ooool00008ool01000ooooo`000ooo0`00oooo2_oo00001Ooo003ooooo0000
2?oo0`001?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00<00?ooool01?oo00<00?oo
ool0oooo2_oo00001Ooo003ooooo00002?oo00<00?ooool01?oo00<00?ooool0oooo2_oo0002ool2
000:ool30003ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo
000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool0
0`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00oooo
o`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00oooo
o`3oool:ool000;oo`8000Coo`03003ooooo00?oo`8000Coo`03003ooooo0?ooo`[oo`0000Goo`00
ooooo`0000Soo`04003ooooo0003ool00`00ooooo`3oool:ool00005ool00?ooool00008ool01000
ooooo`000ooo0`00oooo2_oo00001Ooo003ooooo00002Ooo0P001?oo00<00?ooool0oooo2_oo0000
1Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000So
o`04003ooooo0003ool00`00ooooo`3oool:ool000;oo`8000[oo`8000Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool000goo`@000?oo`03003ooooo0?ooo`[oo`003ooo00<00?oo
ool00_oo00<00?ooool0oooo2_oo000?ool00`00ooooo`02ool3003oool:ool000ooo`03003ooooo
0?ooo`ooo`003ooo00<00?ooool0oooo3ooo000=ool3003ooolAool000ooo`03003ooooo0?ooo`oo
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003o
oolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.10576, -0.159885, \
0.0530983, 0.00716245}}]
}, Open  ]],

Cell["\<\
Power spectrum of the part of the main part that corresponds to the \
sampled interval:\
\>", "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{"psInterval", "=", 
        RowBox[{"powerSpec", "[", 
          RowBox[{
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"gauss", "[", "#", "]"}], 
                  RowBox[{"UnitStep", "[", "#", "]"}], 
                  RowBox[{"UnitStep", "[", 
                    RowBox[{
                      RowBox[{"\[Tau]", " ", "nn"}], "-", " ", "#"}], "]"}]}],
                 ")"}], "&"}], ",", "nn", ",", "\[Tau]"}], "]"}]}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"psInterval", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "1"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0833669 0 0.618034 [
[.19054 -0.0125 -3 -9 ]
[.19054 -0.0125 3 0 ]
[.35728 -0.0125 -3 -9 ]
[.35728 -0.0125 3 0 ]
[.52401 -0.0125 -3 -9 ]
[.52401 -0.0125 3 0 ]
[.69074 -0.0125 -3 -9 ]
[.69074 -0.0125 3 0 ]
[.85748 -0.0125 -6 -9 ]
[.85748 -0.0125 6 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.19054 0 m
.19054 .00625 L
s
[(2)] .19054 -0.0125 0 1 Mshowa
.35728 0 m
.35728 .00625 L
s
[(4)] .35728 -0.0125 0 1 Mshowa
.52401 0 m
.52401 .00625 L
s
[(6)] .52401 -0.0125 0 1 Mshowa
.69074 0 m
.69074 .00625 L
s
[(8)] .69074 -0.0125 0 1 Mshowa
.85748 0 m
.85748 .00625 L
s
[(10)] .85748 -0.0125 0 1 Mshowa
.125 Mabswid
.06549 0 m
.06549 .00375 L
s
.10718 0 m
.10718 .00375 L
s
.14886 0 m
.14886 .00375 L
s
.23223 0 m
.23223 .00375 L
s
.27391 0 m
.27391 .00375 L
s
.31559 0 m
.31559 .00375 L
s
.39896 0 m
.39896 .00375 L
s
.44064 0 m
.44064 .00375 L
s
.48233 0 m
.48233 .00375 L
s
.56569 0 m
.56569 .00375 L
s
.60738 0 m
.60738 .00375 L
s
.64906 0 m
.64906 .00375 L
s
.73243 0 m
.73243 .00375 L
s
.77411 0 m
.77411 .00375 L
s
.81579 0 m
.81579 .00375 L
s
.89916 0 m
.89916 .00375 L
s
.94085 0 m
.94085 .00375 L
s
.98253 0 m
.98253 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .12361 m
.03006 .12361 L
s
[(0.2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(1)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .04046 m
.21429 .02006 L
.40476 .00518 L
.59524 .00202 L
.78571 .00109 L
.97619 .00069 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00>?oo10009?oo00<00?ooool08Ooo0P009?oo0P008?oo1000
0ooo0P00Booo000hool00`00ooooo`0Uool00`00ooooo`0Pool01000ooooo`008_oo00@00?ooool0
027oo`05003oooooool00002ool00`00ooooo`18ool003Woo`03003ooooo027oo`D0027oo`04003o
oooo000Rool01000ooooo`008Ooo00D00?ooooooo`0000;oo`03003ooooo04Soo`00>_oo00<00?oo
ool08?oo00@00?ooool002;oo`<002Coo`8002;oo`05003oooooool00002ool00`00ooooo`18ool0
03_oo`03003ooooo023oo`03003oo`0002;oo`03003ooooo02?oo`04003ooooo000Qool01@00oooo
oooo00000_oo00<00?ooool0B?oo000hool01000ooooo`008ooo0P008_oo00<00?ooool08ooo00@0
0?ooool001ooo`<000?oo`04003ooooo001:ool003Woo`8002Goo`03003ooooo027oo`<002?oo`80
02;oo`03003ooooo00;oo`8004_oo`00oooo8Ooo003ooolQool00?[oo`@000;oo`@000;oo`@00003
ool00?oo00?oo`03003ooooo00Goo`@000Coo`003oooh`001ooo00H00?ooool00?oo0007ool00`00
ooooo`03ool01000ooooo`002Ooo00<00?ooool01?oo000Dool00`00ooooo`07ool00`00ooooo`06
ool00`00ooooo`07ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`06
ool00`00ooooo`07ool00`00ooooo`0M0005ool00`00ooooo`07ool00`00ooooo`06ool00`00oooo
o`07ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`06ool00`00oooo
o`07ool00`00ooooo`06ool00`00ooooo`06ool00`00ooooo`07ool00`00ooooo`09ool01`00oooo
o`00ool00?oo00@000?oo`03003ooooo00?oo`03003oo`0000Woo`03003ooooo00Coo`005?oo00<0
0?ooool0?_oo3P00U_oo00<00?ooool00_oo00<00?ooool01ooo00<00?ooool00_oo00<00?oo0000
2Ooo00<00?ooool01?oo000Dool00`00ooooo`0`ool>002Uool01@00oooooooo00002_oo00<00?oo
ool01@001ooo10001?oo000Dool00`00ooooo`0Tool<002oool01000ooooo`003ooo00<00?ooool0
1?oo000Dool00`00ooooo`0Lool8003<ool2000Aool20004ool001Coo`03003ooooo01?oo`T00>go
o`005?oo0P002ooo2@00m_oo000Dool00`00ooooo`02ool8003oool001Coo`D00?ooo`Soo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo0002ool20004ool00`00ooooo`02ool40003ool00`00ooooo`3oool:ool00005ool00?ooool0
0008ool00`00ooooo`04ool00`00ooooo`3oool:ool00005ool00?ooool00009ool00`00ooooo`03
ool3003oool:ool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`3oool:ool00005
ool00?ooool0000;ool01@00oooooooo0000oooo3?oo00001Ooo003ooooo00002?oo00@00?ooool0
00?oo`03003ooooo0?ooo`[oo`000_oo0P002_oo0P001?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo0002ool20004ool00`00ooooo`04ool00`00
ooooo`02ool00`00ooooo`3oool:ool00005ool00?ooool0000:ool00`00ooooo`02ool00`00oooo
o`3oool:ool00005ool00?ooool00007ool50003ool3003oool:ool00005ool00?ooool00007ool0
1000ooooo`001?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00<00?oo00001?oo00<0
0?ooool0oooo2_oo00001Ooo003ooooo00002Ooo0P001?oo00<00?ooool0oooo2_oo0002ool2000;
ool00`00ooooo`02ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?oo
ool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo
2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool000;oo`8000Coo`03003ooooo00?oo`8000Coo`03003ooooo0?ooo`[oo`0000Go
o`00ooooo`0000Soo`04003ooooo0003ool00`00ooooo`3oool:ool00005ool00?ooool00008ool0
1000ooooo`000ooo0`00oooo2_oo00001Ooo003ooooo00002?oo0`001?oo00<00?ooool0oooo2_oo
00001Ooo003ooooo00002?oo00<00?ooool01?oo00<00?ooool0oooo2_oo00001Ooo003ooooo0000
2?oo00<00?ooool01?oo00<00?ooool0oooo2_oo0002ool2000:ool30003ool00`00ooooo`3oool:
ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Co
o`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Co
o`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Co
o`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003o
oooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool000;oo`8000Coo`03003o
oooo00?oo`8000Coo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool0
0`00ooooo`3oool:ool00005ool00?ooool00008ool01000ooooo`000ooo0`00oooo2_oo00001Ooo
003ooooo00002Ooo0P001?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00@00?ooool0
00?oo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool00`00ooooo`3o
ool:ool000;oo`8000[oo`8000Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000D
ool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
00goo`@000?oo`03003ooooo0?ooo`[oo`003ooo00<00?ooool00_oo00<00?ooool0oooo2_oo000?
ool00`00ooooo`02ool3003oool:ool000ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo
3ooo000=ool3003ooolAool000ooo`03003ooooo0?ooo`ooo`00oooo8Ooo003ooolQool00?ooob7o
o`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003o
oolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.10576, -0.159885, \
0.0530983, 0.00716245}}]
}, Open  ]],

Cell["Discrete power spectrum:", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"samp", "=", 
        RowBox[{"sample", "[", 
          RowBox[{"gauss", ",", "nn", ",", "\[Tau]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"dps", "=", 
        RowBox[{"discretePowerSpec", "[", 
          RowBox[{"samp", ",", "\[Tau]"}], "]"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"powerPlot", "[", 
        RowBox[{"dps", ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "1"}], "}"}]}], "]"}], ";"}]], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0757881 0 0.618034 [
[.17539 -0.0125 -3 -9 ]
[.17539 -0.0125 3 0 ]
[.32696 -0.0125 -3 -9 ]
[.32696 -0.0125 3 0 ]
[.47854 -0.0125 -3 -9 ]
[.47854 -0.0125 3 0 ]
[.63011 -0.0125 -3 -9 ]
[.63011 -0.0125 3 0 ]
[.78169 -0.0125 -6 -9 ]
[.78169 -0.0125 6 0 ]
[.93327 -0.0125 -6 -9 ]
[.93327 -0.0125 6 0 ]
[1.025 0 0 -4.90625 ]
[1.025 0 40 4.90625 ]
[.01131 .12361 -18 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -18 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -18 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -18 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -6 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 0 m
.17539 .00625 L
s
[(2)] .17539 -0.0125 0 1 Mshowa
.32696 0 m
.32696 .00625 L
s
[(4)] .32696 -0.0125 0 1 Mshowa
.47854 0 m
.47854 .00625 L
s
[(6)] .47854 -0.0125 0 1 Mshowa
.63011 0 m
.63011 .00625 L
s
[(8)] .63011 -0.0125 0 1 Mshowa
.78169 0 m
.78169 .00625 L
s
[(10)] .78169 -0.0125 0 1 Mshowa
.93327 0 m
.93327 .00625 L
s
[(12)] .93327 -0.0125 0 1 Mshowa
.125 Mabswid
.0617 0 m
.0617 .00375 L
s
.0996 0 m
.0996 .00375 L
s
.13749 0 m
.13749 .00375 L
s
.21328 0 m
.21328 .00375 L
s
.25117 0 m
.25117 .00375 L
s
.28907 0 m
.28907 .00375 L
s
.36486 0 m
.36486 .00375 L
s
.40275 0 m
.40275 .00375 L
s
.44064 0 m
.44064 .00375 L
s
.51643 0 m
.51643 .00375 L
s
.55433 0 m
.55433 .00375 L
s
.59222 0 m
.59222 .00375 L
s
.66801 0 m
.66801 .00375 L
s
.7059 0 m
.7059 .00375 L
s
.7438 0 m
.7438 .00375 L
s
.81958 0 m
.81958 .00375 L
s
.85748 0 m
.85748 .00375 L
s
.89537 0 m
.89537 .00375 L
s
.97116 0 m
.97116 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
1.025 0 -61 -8.90625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 17.8125 translate 1 -1 scale
63.000 11.250 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
%%BeginResource: font Courier-MISO
%%BeginFont: Courier-MISO
/Courier /Courier-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(w) show
69.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(=) show
75.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
81.000 11.250 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(p) show
93.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(f) show
99.000 11.250 moveto
%%IncludeResource: font Courier-MISO
%%IncludeFont: Courier-MISO
/Courier-MISO findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .12361 m
.03006 .12361 L
s
[(0.2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(0.4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(0.6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(0.8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(1)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .057 m
.18254 .02973 L
.34127 .00795 L
.5 .00332 L
.65873 .0021 L
.81746 .00166 L
.97619 .00154 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40n007h00Ooooo8Ooo003ooolQ
ool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00
oooo8Ooo003ooolQool00?ooob7oo`00=Ooo10008?oo00<00?ooool07_oo0P008?oo0P007Ooo1000
0ooo0P006Ooo10000_oo1000>Ooo000eool00`00ooooo`0Qool00`00ooooo`0Mool01000ooooo`00
7_oo00@00?ooool001koo`05003oooooool00002ool00`00ooooo`0Hool01@00oooooooo0000??oo
000fool00`00ooooo`0Mool5000Nool01000ooooo`007_oo00@00?ooool001koo`05003oooooool0
0002ool00`00ooooo`0Hool00`00ooooo`02ool00`00ooooo`0iool003Ooo`03003ooooo01coo`04
003ooooo000Oool3000Pool2000Oool01@00oooooooo00000_oo00<00?ooool06?oo00<00?ooool0
0ooo00<00?ooool0>?oo000hool00`00ooooo`0Lool00`00ool0000Oool00`00ooooo`0Oool01000
ooooo`007_oo00D00?ooooooo`0000;oo`03003ooooo01Soo`03003ooooo00Coo`03003ooooo03Oo
o`00=Ooo00@00?ooool001ooo`8001ooo`03003ooooo01ooo`04003ooooo000Lool30003ool01000
ooooo`006?oo0`000ooo00@00?ooool003Woo`00=_oo0P008Ooo00<00?ooool07_oo0`007ooo0P00
7ooo00<00?ooool00_oo0P006ooo00<00?ooool00_oo0P00>_oo003ooolQool00?ooob7oo`00n_oo
10000_oo10000_oo100000?oo`00ool00ooo00<00?ooool01Ooo10001?oo000?oooS0007ool01P00
ooooo`00ool000Ooo`03003ooooo00?oo`04003ooooo0009ool00`00ooooo`04ool001Coo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Koo`03003ooooo00Goo`03003o
oooo00Koo`03003ooooo00Goo`03003ooooo00Koo`03003ooooo00Goo`03003ooooo00Koo`04003o
oooooolT0003ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool00`00ooooo`05ool00`00ooooo`06ool0
0`00ooooo`05ool00`00ooooo`0<ool01`00ooooo`00ool00?oo00@000?oo`03003ooooo00?oo`03
003oo`0000Woo`03003ooooo00Coo`005?oo00<00?ooool0@_oo5@00Rooo00<00?ooool00_oo00<0
0?ooool01ooo00<00?ooool00_oo00<00?oo00002Ooo00<00?ooool01?oo000Dool00`00ooooo`0k
ool7002Qool01@00oooooooo00002_oo00<00?ooool01@001ooo10001?oo000Dool00`00ooooo`0c
ool8002dool01000ooooo`003ooo00<00?ooool01?oo000Dool00`00ooooo`0/ool7002mool2000A
ool20004ool001Coo`03003ooooo02Goo`L00=goo`005?oo0P007ooo1`00i?oo000Dool00`00oooo
o`0Hool6003[ool001Coo`03003ooooo01;oo`H00?7oo`005?oo00<00?ooool03?oo1P00mooo000D
ool00`00ooooo`06ool6003mool001Coo`03003ooooo00H00?ooo`Coo`005?oo0`00oooo2_oo000D
ool2003oool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00
ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo0002ool20004ool00`00
ooooo`02ool40003ool00`00ooooo`3oool:ool00005ool00?ooool00008ool00`00ooooo`04ool0
0`00ooooo`3oool:ool00005ool00?ooool00009ool00`00ooooo`03ool3003oool:ool00005ool0
0?ooool0000:ool00`00ooooo`02ool00`00ooooo`3oool:ool00005ool00?ooool0000;ool01@00
oooooooo0000oooo3?oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo0?ooo`[o
o`000_oo0P002_oo0P001?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo
00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo
00<00?ooool0oooo2_oo0002ool20004ool00`00ooooo`04ool00`00ooooo`02ool00`00ooooo`3o
ool:ool00005ool00?ooool0000:ool00`00ooooo`02ool00`00ooooo`3oool:ool00005ool00?oo
ool00007ool50003ool3003oool:ool00005ool00?ooool00007ool01000ooooo`001?oo00<00?oo
ool0oooo2_oo00001Ooo003ooooo00002?oo00<00?oo00001?oo00<00?ooool0oooo2_oo00001Ooo
003ooooo00002Ooo0P001?oo00<00?ooool0oooo2_oo0002ool2000;ool00`00ooooo`02ool00`00
ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003o
ool;ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3o
ool:ool001Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`03003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool000;oo`80
00Coo`03003ooooo00?oo`8000Coo`03003ooooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003o
oooo0003ool00`00ooooo`3oool:ool00005ool00?ooool00008ool01000ooooo`000ooo0`00oooo
2_oo00001Ooo003ooooo00002?oo0`001?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo
00<00?ooool01?oo00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00<00?ooool01?oo00<0
0?ooool0oooo2_oo0002ool2000:ool30003ool00`00ooooo`3oool:ool001Coo`03003ooooo0?oo
o`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[oo`00
5?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`00
5?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`800?ooo`_oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<0
0?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo0P00
oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0
oooo2_oo000Dool00`00ooooo`3oool:ool000;oo`8000Coo`03003ooooo00?oo`8000Coo`03003o
oooo0?ooo`[oo`0000Goo`00ooooo`0000Soo`04003ooooo0003ool00`00ooooo`3oool:ool00005
ool00?ooool00008ool01000ooooo`000ooo0`00oooo2_oo00001Ooo003ooooo00002Ooo0P001?oo
00<00?ooool0oooo2_oo00001Ooo003ooooo00002?oo00@00?ooool000?oo`03003ooooo0?ooo`[o
o`0000Goo`00ooooo`0000Soo`04003ooooo0003ool00`00ooooo`3oool:ool000;oo`8000[oo`80
00Coo`03003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool0
01Coo`800?ooo`_oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03
003ooooo0?ooo`[oo`005?oo0P00oooo2ooo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool001Coo`03003ooooo
0?ooo`[oo`005?oo00<00?ooool0oooo2_oo000Dool2003oool;ool001Coo`03003ooooo0?ooo`[o
o`005?oo00<00?ooool0oooo2_oo000Dool00`00ooooo`3oool:ool000goo`@000?oo`03003ooooo
0?ooo`[oo`003ooo00<00?ooool00_oo00<00?ooool0oooo2_oo000?ool00`00ooooo`02ool3003o
ool:ool000ooo`03003ooooo0?ooo`ooo`003ooo00<00?ooool0oooo3ooo000=ool3003ooolAool0
00ooo`03003ooooo0?ooo`ooo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool0
0?ooob7oo`00oooo8Ooo003ooolQool00?ooob7oo`00oooo8Ooo003ooolQool00001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.21634, -0.159885, \
0.0584081, 0.00716245}}]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
                "]"}], ",", 
              RowBox[{"CForm", "[", 
                RowBox[{
                "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}], 
                "]"}]}], "}"}], "&"}], "/@", "dps"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"Export", "[", 
        RowBox[{"\"\<tmp.txt\>\"", ",", 
          RowBox[{"ToString", "[", "%", "]"}]}], "]"}], ";"}]}], "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
Magnification->1
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 66, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[1830, 55, 28, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[1883, 59, 38, 0, 42, "Subsubsection"],
Cell[1924, 61, 456, 15, 32, "Text"],
Cell[2383, 78, 387, 10, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2807, 93, 79, 0, 42, "Subsubsection"],
Cell[2889, 95, 1429, 37, 77, "Text"],
Cell[4321, 134, 426, 11, 27, "Input"],

Cell[CellGroupData[{
Cell[4772, 149, 705, 19, 48, "Input"],
Cell[5480, 170, 262, 6, 22, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5779, 181, 348, 9, 28, "Input"],
Cell[6130, 192, 258, 6, 22, "Message"]
}, Open  ]],
Cell[6403, 201, 644, 19, 34, "Text"],
Cell[7050, 222, 804, 20, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7891, 247, 968, 29, 48, "Subsubsection"],
Cell[8862, 278, 417, 9, 50, "Text"],
Cell[9282, 289, 1758, 40, 91, "Input"],
Cell[11043, 331, 883, 22, 28, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11963, 358, 30, 0, 42, "Subsubsection"],
Cell[11996, 360, 566, 13, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[12611, 379, 54, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[12690, 383, 63, 0, 42, "Subsubsection"],
Cell[12756, 385, 679, 20, 50, "Text"],
Cell[13438, 407, 167, 4, 27, "Input"],
Cell[13608, 413, 187, 5, 27, "Input"],

Cell[CellGroupData[{
Cell[13820, 422, 186, 5, 27, "Input"],
Cell[14009, 429, 17234, 512, 186, 5816, 366, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[31258, 944, 340, 9, 43, "Input"],

Cell[CellGroupData[{
Cell[31623, 957, 185, 5, 27, "Input"],
Cell[31811, 964, 15018, 455, 186, 5267, 330, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[46844, 1422, 663, 18, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47544, 1445, 64, 0, 42, "Subsubsection"],
Cell[47611, 1447, 167, 4, 27, "Input"],
Cell[47781, 1453, 187, 5, 27, "Input"],

Cell[CellGroupData[{
Cell[47993, 1462, 186, 5, 27, "Input"],
Cell[48182, 1469, 16453, 518, 186, 6021, 385, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[64650, 1990, 340, 9, 43, "Input"],

Cell[CellGroupData[{
Cell[65015, 2003, 182, 5, 27, "Input"],
Cell[65200, 2010, 13808, 483, 186, 5892, 381, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[79023, 2496, 663, 18, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[79723, 2519, 58, 0, 42, "Subsubsection"],
Cell[79784, 2521, 119, 3, 27, "Input"],
Cell[79906, 2526, 219, 6, 27, "Input"],

Cell[CellGroupData[{
Cell[80150, 2536, 186, 5, 27, "Input"],
Cell[80339, 2543, 17657, 509, 201, 5732, 357, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[98011, 3055, 368, 10, 43, "Input"],

Cell[CellGroupData[{
Cell[98404, 3069, 183, 5, 27, "Input"],
Cell[98590, 3076, 14290, 488, 186, 5985, 381, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[112895, 3567, 663, 18, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[113595, 3590, 41, 0, 42, "Subsubsection"],
Cell[113639, 3592, 122, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[113786, 3599, 720, 21, 44, "Input"],
Cell[114509, 3622, 12009, 415, 186, 4526, 318, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[126533, 4040, 579, 16, 59, "Input"],

Cell[CellGroupData[{
Cell[127137, 4060, 181, 5, 27, "Input"],
Cell[127321, 4067, 13141, 446, 186, 5551, 348, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[140477, 4516, 342, 9, 43, "Input"],

Cell[CellGroupData[{
Cell[140844, 4529, 182, 5, 27, "Input"],
Cell[141029, 4536, 13144, 446, 186, 5562, 348, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[154188, 4985, 663, 18, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[154888, 5008, 85, 1, 42, "Subsubsection"],
Cell[154976, 5011, 155, 3, 32, "Text"],
Cell[155134, 5016, 121, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[155280, 5023, 720, 21, 44, "Input"],
Cell[156003, 5046, 11570, 408, 186, 4423, 315, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[167588, 5457, 55, 0, 32, "Text"],
Cell[167646, 5459, 584, 16, 59, "Input"],

Cell[CellGroupData[{
Cell[168255, 5479, 186, 5, 27, "Input"],
Cell[168444, 5486, 12294, 391, 186, 4959, 296, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[180753, 5880, 111, 3, 32, "Text"],
Cell[180867, 5885, 541, 14, 27, "Input"],

Cell[CellGroupData[{
Cell[181433, 5903, 189, 5, 27, "Input"],
Cell[181625, 5910, 11549, 382, 186, 4975, 296, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[193189, 6295, 40, 0, 32, "Text"],
Cell[193232, 6297, 342, 9, 43, "Input"],

Cell[CellGroupData[{
Cell[193599, 6310, 182, 5, 27, "Input"],
Cell[193784, 6317, 11905, 394, 186, 5113, 306, "GraphicsData", "PostScript", \
"Graphics"]
}, Open  ]],
Cell[205704, 6714, 663, 18, 43, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

