(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     21598,        600]*)
(*NotebookOutlinePosition[     22254,        624]*)
(*  CellTagsIndexPosition[     22210,        620]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\<\
This file contains some tools that can be used to calculate some \
testing data. Similar files can be found in some directories of individual \
tests.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Tools", "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"innerProd", "[", 
        RowBox[{"\[Psi]_", ",", "\[CurlyPhi]_"}], "]"}], ":=", 
      RowBox[{
        RowBox[{"(", 
          RowBox[{
            RowBox[{"adj", "[", "\[Psi]", "]"}], ".", "\[CurlyPhi]"}], ")"}], 
        "\[LeftDoubleBracket]", 
        RowBox[{"1", ",", "1"}], 
        "\[RightDoubleBracket]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"normalize", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{"\[Psi]", "/", 
          SqrtBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"adj", "[", "\[Psi]", "]"}], ".", "\[Psi]"}], ")"}],
               "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"terseForm", "[", "x_", "]"}], ":=", 
        RowBox[{"NumberForm", "[", 
          RowBox[{
            RowBox[{"MatrixForm", "[", 
              RowBox[{"Chop", "[", "x", "]"}], "]"}], ",", "2"}], "]"}]}], 
      " ", 
      RowBox[{"(*", 
        RowBox[{
        "Nice", " ", "and", " ", "short", " ", "display", " ", "of", " ", 
          "matrices"}], "*)"}]}]}], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"randomComplex", ":=", 
        RowBox[{
          RowBox[{"Random", "[", "]"}], 
          RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
              RowBox[{"Random", "[", "]"}]}], "]"}]}]}], 
      RowBox[{"(*", " ", 
        RowBox[{"Range", " ", "is", " ", "chosen", " ", "arbitrarily"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"randomState", ":=", 
        RowBox[{"normalize", "[", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", "randomComplex", "}"}], ",", 
              RowBox[{"{", "randomComplex", "}"}]}], "}"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"randomOrthogonalState", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"\[CurlyPhi]", "=", "randomState"}], "}"}], ",", 
            RowBox[{"normalize", "[", 
              RowBox[{"\[CurlyPhi]", "-", 
                RowBox[{
                  RowBox[{"innerProd", "[", 
                    RowBox[{"\[Psi]", ",", "\[CurlyPhi]"}], "]"}], 
                  "\[Psi]"}]}], "]"}]}], "]"}]}], ";"}]}], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"randomHSystem", ":=", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"Random", "[", 
                    RowBox[{"Real", ",", 
                      RowBox[{"{", 
                        RowBox[{
                          RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}], ",",
                   
                  RowBox[{"Random", "[", 
                    RowBox[{"Real", ",", 
                      RowBox[{"{", 
                        RowBox[{
                          RowBox[{"-", "2"}], ",", "2"}], "}"}]}], "]"}]}], 
                "}"}], ",", 
              RowBox[{"Block", "[", 
                RowBox[{
                  RowBox[{"{", 
                    RowBox[{"eState1", "=", "randomState"}], "}"}], ",", 
                  RowBox[{"{", 
                    RowBox[{"eState1", ",", 
                      RowBox[{
                      "randomOrthogonalState", "[", "eState1", "]"}]}], 
                    "}"}]}], "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
        RowBox[{
          RowBox[{
          "The", " ", "range", " ", "for", " ", "the", " ", "eigenvalues", 
            " ", "is", " ", "chosen", " ", "arbitrarily"}], " ", "-", "  ", 
          RowBox[{
          "it", " ", "could", " ", "be", " ", "something", " ", "completely", 
            " ", "else"}]}], " ", "*)"}]}]], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"u", "[", 
        RowBox[{"es_", ",", "eStates_", ",", "\[Tau]_"}], "]"}], ":=", 
      RowBox[{"Sum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"Exp", "[", 
              RowBox[{
                RowBox[{"-", "I"}], " ", "\[Tau]", " ", 
                RowBox[{
                "es", "\[LeftDoubleBracket]", "j", 
                  "\[RightDoubleBracket]"}]}], "]"}], 
            RowBox[{
              RowBox[{
              "eStates", "\[LeftDoubleBracket]", "j", 
                "\[RightDoubleBracket]"}], ".", 
              RowBox[{"adj", "[", 
                RowBox[{
                "eStates", "\[LeftDoubleBracket]", "j", 
                  "\[RightDoubleBracket]"}], "]"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"j", ",", "1", ",", "2"}], "}"}]}], 
        "]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"c12", "[", "\[Psi]_", "]"}], ":=", 
      RowBox[{
        SuperscriptBox[
          RowBox[{"Abs", "[", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}], "}"}], ".", "\[Psi]"}], 
              ")"}], "]"}], "2"], "\[LeftDoubleBracket]", 
        RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]}]}], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"randomUnitaryOp", ":=", "\[IndentingNewLine]", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"\[Psi]1", "=", "randomState"}], "}"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"Block", "[", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"\[Psi]", "=", 
                    RowBox[{"{", 
                      RowBox[{"\[Psi]1", ",", 
                        RowBox[{
                        "randomOrthogonalState", "[", "\[Psi]1", "]"}]}], 
                      "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
                RowBox[{"Sum", "[", 
                  RowBox[{
                    RowBox[{
                      RowBox[{"Exp", "[", 
                        RowBox[{"I", " ", 
                          RowBox[{"Random", "[", 
                            RowBox[{"Real", ",", 
                              RowBox[{"{", 
                                RowBox[{"0", ",", 
                                  RowBox[{"2", "\[Pi]"}]}], "}"}]}], "]"}]}], 
                        "]"}], 
                      RowBox[{
                        RowBox[{
                        "\[Psi]", "\[LeftDoubleBracket]", "j", 
                          "\[RightDoubleBracket]"}], ".", 
                        RowBox[{"adj", "[", 
                          RowBox[{
                          "\[Psi]", "\[LeftDoubleBracket]", "j", 
                            "\[RightDoubleBracket]"}], "]"}]}]}], ",", 
                    RowBox[{"{", 
                      RowBox[{"j", ",", "1", ",", "2"}], "}"}]}], "]"}]}], 
              "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{"randomDiagonalOp", ":=", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"randomComplex", ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
            RowBox[{"0", ",", "randomComplex"}], "}"}]}], 
        "}"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"randomPosDiagonalOp", ":=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"Random", "[", "]"}], ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"0", ",", 
                RowBox[{"Random", "[", "]"}]}], "}"}]}], "}"}]}], 
      RowBox[{"(*", " ", 
        RowBox[{"Range", " ", "is", " ", "chosen", " ", "arbitrarily"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"oneOp", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"1", ",", "0"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ";"}]}], "Input"],

Cell["Generators and other tools for measurement operators:", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"completeness", "[", "ms_", "]"}], ":=", 
      RowBox[{"Sum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"adj", "[", 
              RowBox[{
              "ms", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}], 
              "]"}], ".", 
            RowBox[{
            "ms", "\[LeftDoubleBracket]", "j", "\[RightDoubleBracket]"}]}], 
          ",", 
          RowBox[{"{", 
            RowBox[{"j", ",", "1", ",", 
              RowBox[{"Length", "[", "ms", "]"}]}], "}"}]}], 
        "]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"randomRealsThatSumToOne", "[", "n_", "]"}], ":=", 
      "\[IndentingNewLine]", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"t", "=", 
              RowBox[{"Table", "[", 
                RowBox[{
                  RowBox[{"Random", "[", "]"}], ",", 
                  RowBox[{"{", 
                    RowBox[{"j", ",", "1", ",", "n"}], "}"}]}], "]"}]}], 
            "}"}], ",", 
          RowBox[{"t", "/", 
            RowBox[{"(", 
              RowBox[{"Plus", "@@", "t"}], ")"}]}]}], "]"}]}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      RowBox[{"randomPosDiagMeasurementOps", "[", "n_", "]"}], ":=", 
      "\[IndentingNewLine]", 
      RowBox[{"Block", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{"reals", "=", 
              RowBox[{"Transpose", "[", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"Sqrt", "[", 
                      RowBox[{"randomRealsThatSumToOne", "[", "n", "]"}], 
                      "]"}], ",", 
                    RowBox[{"Sqrt", "[", 
                      RowBox[{"randomRealsThatSumToOne", "[", "n", "]"}], 
                      "]"}]}], "}"}], "]"}]}], "}"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
            RowBox[{
              RowBox[{"{", 
                RowBox[{
                  RowBox[{"{", 
                    RowBox[{
                      RowBox[{
                      "#", "\[LeftDoubleBracket]", "1", 
                        "\[RightDoubleBracket]"}], ",", "0"}], "}"}], ",", 
                  RowBox[{"{", 
                    RowBox[{"0", ",", 
                      RowBox[{
                      "#", "\[LeftDoubleBracket]", "2", 
                        "\[RightDoubleBracket]"}]}], "}"}]}], "}"}], "&"}], "/@",
             "reals"}]}], "]"}]}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"General", "::", "\<\"spell1\"\>"}], 
      RowBox[{
      ":", " "}], "\<\"Possible spelling error: new symbol name \
\\\"\\!\\(reals\\)\\\" is similar to existing symbol \
\\\"\\!\\(Reals\\)\\\".\"\>"}]], "Message"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"randomMeasurementOps", "[", "n_", "]"}], ":=", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"u", "=", "randomUnitaryOp"}], ",", 
                RowBox[{"v", "=", "randomUnitaryOp"}]}], "}"}], ",", 
            RowBox[{
              RowBox[{
                RowBox[{"u", ".", "#", ".", "v"}], "&"}], "/@", 
              RowBox[{"randomPosDiagMeasurementOps", "[", "n", "]"}]}]}], 
          "]"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"randomProjMeasurementOps", ":=", 
        RowBox[{"Block", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"\[Psi]", "=", "randomState"}], "}"}], ",", 
            RowBox[{"Block", "[", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"\[CurlyPhi]", "=", 
                    RowBox[{"randomOrthogonalState", "[", "\[Psi]", "]"}]}], 
                  "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    RowBox[{"\[Psi]", ".", 
                      RowBox[{"adj", "[", "\[Psi]", "]"}]}], ",", 
                    RowBox[{"\[CurlyPhi]", ".", 
                      RowBox[{"adj", "[", "\[CurlyPhi]", "]"}]}]}], "}"}]}], 
              "]"}]}], "]"}]}], ";"}]], "Input"],

Cell["Conversion of stuff to strings:", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"complexToString", "[", "z_", "]"}], ":=", 
        RowBox[{"\"\<(\>\"", "<>", 
          RowBox[{"ToString", "[", 
            RowBox[{"N", "[", 
              RowBox[{"Re", "[", "z", "]"}], "]"}], "]"}], "<>", "\"\<,\>\"", 
          "<>", 
          RowBox[{"ToString", "[", 
            RowBox[{"N", "[", 
              RowBox[{"Im", "[", "z", "]"}], "]"}], "]"}], "<>", 
          "\"\<)\>\""}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"stateToString", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{
          RowBox[{"complexToString", "[", 
            RowBox[{"\[Psi]", "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
          "<>", "\"\< \>\"", "<>", 
          RowBox[{"complexToString", "[", 
            RowBox[{"\[Psi]", "\[LeftDoubleBracket]", 
              RowBox[{"2", ",", "1"}], "\[RightDoubleBracket]"}], "]"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"opToString", "[", "a_", "]"}], ":=", 
      RowBox[{"StringJoin", "[", 
        RowBox[{"Flatten", "[", 
          RowBox[{"Table", "[", 
            RowBox[{
              RowBox[{
                RowBox[{"complexToString", "[", 
                  RowBox[{"a", "\[LeftDoubleBracket]", 
                    RowBox[{"j", ",", "k"}], "\[RightDoubleBracket]"}], "]"}],
                 "<>", "\"\< \>\""}], ",", 
              RowBox[{"{", 
                RowBox[{"j", ",", "1", ",", "2"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"k", ",", "1", ",", "2"}], "}"}]}], "]"}], "]"}], 
        "]"}]}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Temporary data", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"hSystem", "=", "randomHSystem"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"hSystem", "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "1"}], 
      "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
    RowBox[{"hSystem", "\[LeftDoubleBracket]", 
      RowBox[{"1", ",", "2"}], 
      "\[RightDoubleBracket]"}], "\[IndentingNewLine]", 
    RowBox[{"stateToString", "[", 
      RowBox[{"hSystem", "\[LeftDoubleBracket]", 
        RowBox[{"2", ",", "1"}], "\[RightDoubleBracket]"}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"stateToString", "[", 
      RowBox[{"hSystem", "\[LeftDoubleBracket]", 
        RowBox[{"2", ",", "2"}], "\[RightDoubleBracket]"}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"-", "0.7293548799420551`"}]], "Output"],

Cell[BoxData[
    RowBox[{"-", "0.12383874730583733`"}]], "Output"],

Cell[BoxData["\<\"(0.731971,0.492626) (0.447448,0.146045)\"\>"], "Output"],

Cell[BoxData["\<\"(-0.462748,-0.0860432) (0.878488,-0.0819798)\"\>"], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Psi]0", "=", 
        RowBox[{"normalize", "[", 
          RowBox[{"{", 
            RowBox[{
              RowBox[{"{", 
                RowBox[{"1", ",", "0.1"}], "}"}], ",", 
              RowBox[{"{", 
                RowBox[{"1.1", ",", "0.1"}], "}"}]}], "}"}], "]"}]}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"stateToString", "[", "\[Psi]0", "]"}]], "Input"],

Cell[BoxData["\<\"(0.672673,0.) (0.73994,0.)\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"terseForm", "/@", "randomProjMeasurementOps"}]], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.6\"\>",
                      0.60263563869281445,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"0.49\"\>",
                        0.48789128623659672,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.038\"\>",
                          0.037789131830690187,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"0.49\"\>",
                        0.48789128623659672,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.038\"\>",
                          0.037789131830690187,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.4\"\>",
                      0.39736436130718555,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)], ",", 
        TagBox[
          TagBox[
            RowBox[{"(", "\[NoBreak]", GridBox[{
                  {
                    InterpretationBox["\<\"0.4\"\>",
                      0.39736436130718544,
                      AutoDelete->True], 
                    RowBox[{
                      InterpretationBox["\<\"-0.49\"\>",
                        -0.48789128623659678,
                        AutoDelete->True], "-", 
                      RowBox[{
                        InterpretationBox["\<\"0.038\"\>",
                          0.037789131830690104,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}]},
                  {
                    RowBox[{
                      InterpretationBox["\<\"-0.49\"\>",
                        -0.48789128623659678,
                        AutoDelete->True], "+", 
                      RowBox[{
                        InterpretationBox["\<\"0.038\"\>",
                          0.037789131830690104,
                          AutoDelete->True], " ", "\[ImaginaryI]"}]}], 
                    InterpretationBox["\<\"0.6\"\>",
                      0.60263563869281467,
                      AutoDelete->True]}
                  }], "\[NoBreak]", ")"}],
            Function[ BoxForm`e$, 
              MatrixForm[ BoxForm`e$]]],
          (NumberForm[ #, 2]&)]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"randomProjMeasurementOps", ";"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "/@", "%"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"\<\"(0.33296,0.) (0.444598,-0.156303) (0.444598,0.156303) \
(0.66704,0.) \"\>", 
        ",", "\<\"(0.66704,0.) (-0.444598,0.156303) (-0.444598,-0.156303) \
(0.33296,0.) \"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"randomMeasurementOps", "[", "2", "]"}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{"opToString", "/@", "%"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"\<\"(0.490442,-0.0101741) (0.169873,0.198824) \
(-0.0992031,-0.258005) (-0.279689,0.445353) \"\>", 
        ",", "\<\"(0.734983,-0.0205484) (0.244237,0.301167) \
(-0.126575,-0.355404) (-0.373628,0.604231) \"\>"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
Evaluator->"phi",
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 174, 4, 32, "Text"],

Cell[CellGroupData[{
Cell[1916, 57, 30, 0, 42, "Subsubsection"],
Cell[1949, 59, 1456, 40, 105, "Input"],
Cell[3408, 101, 1218, 31, 59, "Input"],
Cell[4629, 134, 1484, 36, 59, "Input"],
Cell[6116, 172, 1335, 35, 45, "Input"],
Cell[7454, 209, 2758, 67, 123, "Input"],
Cell[10215, 278, 69, 0, 32, "Text"],
Cell[10287, 280, 1174, 32, 59, "Input"],

Cell[CellGroupData[{
Cell[11486, 316, 1296, 32, 59, "Input"],
Cell[12785, 350, 258, 6, 23, "Message"]
}, Open  ]],
Cell[13058, 359, 556, 14, 27, "Input"],
Cell[13617, 375, 806, 19, 27, "Input"],
Cell[14426, 396, 47, 0, 32, "Text"],
Cell[14476, 398, 1682, 40, 59, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16195, 443, 39, 0, 42, "Subsubsection"],
Cell[16237, 445, 92, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[16354, 451, 634, 13, 75, "Input"],
Cell[16991, 466, 66, 1, 27, "Output"],
Cell[17060, 469, 67, 1, 27, "Output"],
Cell[17130, 472, 74, 0, 27, "Output"],
Cell[17207, 474, 79, 0, 27, "Output"]
}, Open  ]],
Cell[17301, 477, 343, 10, 27, "Input"],

Cell[CellGroupData[{
Cell[17669, 491, 75, 1, 27, "Input"],
Cell[17747, 494, 61, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17845, 499, 84, 1, 27, "Input"],
Cell[17932, 502, 2732, 64, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20701, 571, 135, 2, 43, "Input"],
Cell[20839, 575, 243, 5, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21119, 585, 170, 4, 43, "Input"],
Cell[21292, 591, 278, 5, 43, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

