(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     19799,        595]*)
(*NotebookOutlinePosition[     20455,        619]*)
(*  CellTagsIndexPosition[     20411,        615]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\<\
This file contains some tools that can be used to calculate some \
testing data. Similar files can be found in some directories of individual \
tests.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Some definitions", "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"normalize", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{"\[Psi]", "/", 
          SqrtBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  RowBox[{"adj", "[", "\[Psi]", "]"}], ".", "\[Psi]"}], ")"}],
               "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"innerprod", "[", 
          RowBox[{"\[Psi]_", ",", "\[CurlyPhi]_"}], "]"}], ":=", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"adj", "[", "\[Psi]", "]"}], ".", "\[CurlyPhi]"}], 
            ")"}], "\[LeftDoubleBracket]", 
          RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]}], 
      ";"}]}], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"u", "[", 
        RowBox[{"es_", ",", "eStates_", ",", "\[Tau]_"}], "]"}], ":=", 
      RowBox[{"Sum", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"Exp", "[", 
              RowBox[{
                RowBox[{"-", "I"}], " ", "\[Tau]", " ", 
                RowBox[{
                "es", "\[LeftDoubleBracket]", "j", 
                  "\[RightDoubleBracket]"}]}], "]"}], 
            RowBox[{
              RowBox[{
              "eStates", "\[LeftDoubleBracket]", "j", 
                "\[RightDoubleBracket]"}], ".", 
              RowBox[{"adj", "[", 
                RowBox[{
                "eStates", "\[LeftDoubleBracket]", "j", 
                  "\[RightDoubleBracket]"}], "]"}]}]}], ",", 
          RowBox[{"{", 
            RowBox[{"j", ",", "1", ",", "2"}], "}"}]}], 
        "]"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"c12", "[", "\[Psi]_", "]"}], ":=", 
      RowBox[{
        SuperscriptBox[
          RowBox[{"Abs", "[", 
            RowBox[{"(", 
              RowBox[{
                RowBox[{"{", 
                  RowBox[{"{", 
                    RowBox[{"0", ",", "1"}], "}"}], "}"}], ".", "\[Psi]"}], 
              ")"}], "]"}], "2"], "\[LeftDoubleBracket]", 
        RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]}]}], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"complexToString", "[", "z_", "]"}], ":=", 
        RowBox[{"\"\<(\>\"", "<>", 
          RowBox[{"ToString", "[", 
            RowBox[{"N", "[", 
              RowBox[{"Re", "[", "z", "]"}], "]"}], "]"}], "<>", "\"\<,\>\"", 
          "<>", 
          RowBox[{"ToString", "[", 
            RowBox[{"N", "[", 
              RowBox[{"Im", "[", "z", "]"}], "]"}], "]"}], "<>", 
          "\"\<)\>\""}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"stateToString", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{
          RowBox[{"complexToString", "[", 
            RowBox[{"\[Psi]", "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
          "<>", "\"\< \>\"", "<>", 
          RowBox[{"complexToString", "[", 
            RowBox[{"\[Psi]", "\[LeftDoubleBracket]", 
              RowBox[{"2", ",", "1"}], "\[RightDoubleBracket]"}], "]"}]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"opToString", "[", "a_", "]"}], ":=", 
        RowBox[{"StringJoin", "[", 
          RowBox[{"Flatten", "[", 
            RowBox[{"Table", "[", 
              RowBox[{
                RowBox[{
                  RowBox[{"complexToString", "[", 
                    RowBox[{"a", "\[LeftDoubleBracket]", 
                      RowBox[{"j", ",", "k"}], "\[RightDoubleBracket]"}], 
                    "]"}], "<>", "\"\< \>\""}], ",", 
                RowBox[{"{", 
                  RowBox[{"j", ",", "1", ",", "2"}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{"k", ",", "1", ",", "2"}], "}"}]}], "]"}], "]"}], 
          "]"}]}], ";"}]}], "Input"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"complexToCString", "[", "z_", "]"}], ":=", 
        RowBox[{"\"\<dx(\>\"", "<>", 
          RowBox[{"ToString", "[", 
            RowBox[{"N", "[", 
              RowBox[{"Re", "[", "z", "]"}], "]"}], "]"}], "<>", "\"\<,\>\"", 
          "<>", 
          RowBox[{"ToString", "[", 
            RowBox[{"N", "[", 
              RowBox[{"Im", "[", "z", "]"}], "]"}], "]"}], "<>", 
          "\"\<)\>\""}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"stateToCString", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{"\"\<state_tp(\>\"", "<>", 
          RowBox[{"complexToCString", "[", 
            RowBox[{"\[Psi]", "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
          "<>", "\"\<,\>\"", "<>", 
          RowBox[{"complexToCString", "[", 
            RowBox[{"\[Psi]", "\[LeftDoubleBracket]", 
              RowBox[{"2", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], 
          "<>", "\"\<)\>\""}]}], ";"}]}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Miscellaneous calculations", "Subsubsection"],

Cell[BoxData[
    RowBox[{
      RowBox[{"m0", "=", 
        RowBox[{"(", GridBox[{
              {"1", "0"},
              {"0", "0"}
              }], ")"}]}], ";", 
      RowBox[{"m1", "=", 
        RowBox[{"(", GridBox[{
              {"0", "0"},
              {"0", "1"}
              }], ")"}]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"\[Psi]", "=", 
        RowBox[{"(", GridBox[{
              {"1"},
              {"0"}
              }], ")"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"adj", "[", "\[Psi]", "]"}], ".", 
      RowBox[{"adj", "[", "m0", "]"}], ".", "m0", ".", 
      "\[Psi]"}], "\[IndentingNewLine]", 
    RowBox[{"\[Psi]0", "=", 
      RowBox[{"normalize", "[", 
        RowBox[{"m0", ".", "\[Psi]"}], "]"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"{", "1", "}"}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", "1", "}"}], ",", 
        RowBox[{"{", "0", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[ns are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[         0,          0]*)
(*NotebookOutlinePosition[         0,          0]*)
(*  CellTagsIndexPosition[         0,          0]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\<\
This file contains some tools that can be used to calculate some \
testing data. Similar files can be found in some directories of individual \
tests.\
\>", "Text"],

Cell[BoxData[{
    RowBox[{
      RowBox[{
        RowBox[{"adj", "[", "x_", "]"}], ":=", 
        RowBox[{"Transpose", "[", 
          RowBox[{"Conjugate", "[", "x", "]"}], "]"}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"normalize", "[", "\[Psi]_", "]"}], ":=", 
        RowBox[{"\[Psi]", "/", 
          SqrtBox[
            RowBox[{
              RowBox[{"(", 
                RowBox[{
                  Row,

Cell[CellGroupData[{

, "]"}], ".", "\[Psi]"}], ")"}],
               "\[LeftDoubleBracket]", 
              RowBox[{"1", ",", "1"}], "\[RightDoubleBracket]"}]]}]}], 
      ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"innerprod", "[", 
          RowBox[{"\[Psi]_", ",", "\[CurlyPhi]_"}], "]"}], ":=", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{
              RowBox[{"adj", "["Cell[BoxDat,

    RowBox[{
      RowBox[{"p0", "=", 
        RowBox[{"8", "/", "10"}]}], ";", 
      RowBox[{"p1", "=", 
        SqrtBox[
          RowBox[{"1", "-", 
            SuperscriptBox["p0", 
              RowBox[{"2", " "}]]}]]}], ";"}]], "Input"],

Cell[BoxData[
    RowBox[{
      RowBox[{"m0", "=", 
        RowBox[{"(", GridBox[{
              {"1", "0"},
              {"0", "0"}
              }], ")"}]}], ";", 
      RowBox[{,

", "=", 
        RowBox[{"(", GridBox[{
              {"0", "0"},
              {",

 "1"}
              }], ")"}]}], ";"}]], "Input"],

Cel
}, Open  ]],

Cell[CellGroupData[{

{"\[Psi]", "=", 
        RowBox[{"(", GridBox[{
              {"1"},
              {"0"}
              }], ")"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{
      RowBox[{"adj", "[", "\[Psi]", "]"}], ".", 
      RowBox[{"adj", "[", "m0", "]"}], ".", "m0", ".", 
      "\[Psi]"}], "\[IndentingNewLine]", 
    RowBox[{"\[Psi]0", "=", 
      RowBox[{"normalize", "[", 
        RowBox[{"m0", ".", "\[Psi]"}], "]"}]}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"{", "1", "}"}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", "1", "}"}], ",", 
        RowBox[{"{", "0", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[            "2"]}], "]"}]}]], "Input"],

Cell[BoxData["0.`"], "Output"]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"e0state", "=", 
        RowBox[{"(", GridBox[{
              {
                RowBox[{"0.500927", "+", 
                  RowBox[{"0.274041", "I"}]}]},
              {
                RowBox[{"0.701081", "+", 
                  RowBox[{"0.427151", "I"}]}]}
              }], ")"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"e1state", "=", 
        RowBox[{"(", GridBox[{
              {
                RowBox[{"0.593112", "+", 
                  RowBox[{"0.567619", "I"}]}]},
              {
                RowBox[{
                  RowBox[{"-", "0.39367"}], "-", 
                  RowBox[{"0.413582", "I"}]}]}
              }], ")"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"e0", "=", 
        RowBox[{"-", "1.8722"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"e1", "=", "0.664564"}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"\[Psi]init", "=", 
        RowBox[{"(", GridBox[{
              {"0.672673"},
              {"0.73994"}
              }], ")"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"one", "=", 
        RowBox[{"(", GridBox[{
              {"0"},
              {"1"}
              }], ")"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"2", 
      RowBox[{"Abs", "[", 
        RowBox[{
          RowBox[{
            RowBox[{"innerprod", "[", 
              RowBox[{"one", ",", "e0state"}], "]"}], 
            RowBox[{"innerprod", "[", 
              RowBox[{"e0state", ",", "\[Psi]init"}], "]"}], 
            RowBox[{"innerprod", "[", 
              RowBox[{"\[Psi]init", ",", "one"}], "]"}]}], "-", 
          SuperscriptBox[
            RowBox[{"Abs", "[", 
              RowBox[{
                RowBox[{"innerprod", "[", 
                  RowBox[{"one", ",", "e0state"}], "]"}], 
                RowBox[{"innerprod", "[", 
                  RowBox[{"e0state", ",", "\[Psi]init"}], "]"}]}], "]"}], 
            "2"]}], "]"}]}]], "Input"],

Cell[BoxData["0.122375256861016`"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"2", 
      FractionBox["\[Pi]", "2.54"]}]], "Input"],

Cell[BoxData["2.4736950028266085`"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Test 3-1", "Subsubsection"],

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[Psi]init", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"0.6976712087122579`", "\[InvisibleSpace]"}], "+", 
                RowBox[{"0.2579955279298219`", " ", "\[ImaginaryI]"}]}], 
              "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"0.6257768863591852`", "\[InvisibleSpace]"}], "+", 
                RowBox[{"0.2347263951940495`", " ", "\[ImaginaryI]"}]}], 
              "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"e0", "=", 
        RowBox[{"-", "3.5"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"e1", "=", 
        RowBox[{"-", "0.3"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"e0state", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"0.7789630666468557`", "\[InvisibleSpace]", "+", 
                RowBox[{"0.3426969182136192`", " ", "\[ImaginaryI]"}]}], 
              "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"0.3420257320367741`", "\[InvisibleSpace]"}], "+", 
                RowBox[{"0.3984893495085197`", " ", "\[ImaginaryI]"}]}], 
              "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"e1state", "=", 
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                RowBox[{"0.4183095967888088`", "\[InvisibleSpace]"}], "-", 
                RowBox[{"0.3174782579664265`", " ", "\[ImaginaryI]"}]}], 
              "}"}], ",", 
            RowBox[{"{", 
              RowBox[{
                RowBox[{"-", "0.833682387731856`"}], "+", 
                RowBox[{"0.17087513961573986`", " ", "\[ImaginaryI]"}]}], 
              "}"}]}], "}"}]}], ";"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"\[Omega]", "=", 
      RowBox[{"e1", "-", "e0"}]}], "\[IndentingNewLine]", 
    RowBox[{"\[CapitalDelta]tMax", "=", 
      FractionBox[
        RowBox[{"2", "\[Pi]"}], "\[Omega]"]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"\[CapitalDelta]t", "=", "1.3"}], ";"}], "\[IndentingNewLine]", 
  
    RowBox[{
      RowBox[{
        RowBox[{"nRecPoints", "=", "3"}], ";"}], 
      RowBox[{"(*", " ", 
        RowBox[{"number", " ", "of", " ", "recording", " ", "points"}], " ", 
        "*)"}]}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{
        RowBox[{"nn", "=", "2"}], ";"}], " ", 
      RowBox[{"(*", " ", 
        RowBox[{
          RowBox[{"2", "nn"}], " ", "=", " ", 
          "\"\<number of measurement points\>\""}], " ", "*)"}]}]}], "Input"],

Cell[BoxData["3.2`"], "Output"],

Cell[BoxData["1.9634954084936207`"], "Output"]
}, Open  ]],

Cell[BoxData[{
    RowBox[{
      RowBox[{"\[CapitalDelta]tRec", "=", 
        RowBox[{
          RowBox[{"(", 
            RowBox[{"\[CapitalDelta]t", "/", "2"}], ")"}], "/", 
          "nRecPoints"}]}], ";"}], "\[IndentingNewLine]", 
    RowBox[{
      RowBox[{"uu", "=", 
        RowBox[{"u", "[", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{"e0", ",", "e1"}], "}"}], ",", 
            RowBox[{"{", 
              RowBox[{"e0state", ",", "e1state"}], "}"}], ",", 
            "\[CapitalDelta]tRec"}], "]"}]}], ";"}]}], "Input"],

Cell[TextData[{
  "First ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          RowBox[{"3", "+", "6", "+", "3", "+", "6"}], "=", "18"}], 
        TraditionalForm]]],
  " values of ",
  Cell[BoxData[
      FormBox[
        RowBox[{"t", ",", 
          SuperscriptBox[
            RowBox[{"\[LeftBracketingBar]", 
              SubscriptBox[
                OverscriptBox["c", "~"], "1"], "\[RightBracketingBar]"}], 
            "2"]}], TraditionalForm]]],
  ":"
}], "Text"],

Cell[BoxData[
    RowBox[{
      RowBox[{
        RowBox[{"applyU", "[", 
          RowBox[{"uu_", ",", "n_", ",", "\[Psi]_"}], "]"}], ":=", 
        RowBox[{"If", "[", 
          RowBox[{
            RowBox[{"n", "\[LessEqual]", "0"}], ",", "\[Psi]", ",", 
            RowBox[{"applyU", "[", 
              RowBox[{"uu", ",", 
                RowBox[{"n", "-", "1"}], ",", 
                RowBox[{"normalize", "[", 
                  RowBox[{"uu", ".", "\[Psi]"}], "]"}]}], "]"}]}], "]"}]}], 
      ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[{
    RowBox[{"Table", "[", 
      RowBox[{
        RowBox[{"n", "*", "\[CapitalDelta]tRec"}], ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "0", ",", "17"}], "}"}]}], 
      "]"}], "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
      RowBox[{
        SuperscriptBox[
          RowBox[{"Abs", "[", 
            RowBox[{
              RowBox[{"applyU", "[", 
                RowBox[{"uu", ",", "n", ",", "\[Psi]init"}], "]"}], 
              "\[LeftDoubleBracket]", 
              RowBox[{"2", ",", "1"}], "\[RightDoubleBracket]"}], "]"}], "2"],
         ",", 
        RowBox[{"{", 
          RowBox[{"n", ",", "0", ",", "17"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
      "0", ",", "0.21666666666666667`", ",", "0.43333333333333335`", ",", 
        "0.65`", ",", "0.8666666666666667`", ",", "1.0833333333333335`", ",", 
        "1.3`", ",", "1.5166666666666666`", ",", "1.7333333333333334`", ",", 
        "1.9500000000000002`", ",", "2.166666666666667`", ",", 
        "2.3833333333333333`", ",", "2.6`", ",", "2.816666666666667`", ",", 
        "3.033333333333333`", ",", "3.25`", ",", "3.466666666666667`", ",", 
        "3.6833333333333336`"}], "}"}]], "Output"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
      "0.44669319210218955`", ",", "0.2933706591785191`", ",", 
        "0.14735360188769683`", ",", "0.07606687306839091`", ",", 
        "0.1124278406580929`", ",", "0.23964646016272534`", ",", 
        "0.3989782474953762`", ",", "0.5168501385719678`", ",", 
        "0.5388335956784039`", ",", "0.45477753518248615`", ",", 
        "0.303495693056497`", ",", "0.1548439897782196`", ",", 
        "0.07746385261455682`", ",", "0.10708634242714882`", ",", 
        "0.23003297498006908`", ",", "0.38953189936291344`", ",", 
        "0.5119328742684616`", ",", "0.5407160117230836`"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
Evaluator->"phi",
WindowSize->{1012, 740},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 174, 4, 32, "Text"],

Cell[CellGroupData[{
Cell[1916, 57, 41, 0, 42, "Subsubsection"],
Cell[1960, 59, 1058, 29, 73, "Input"],
Cell[3021, 90, 1335, 35, 45, "Input"],
Cell[4359, 127, 1736, 41, 59, "Input"],
Cell[6098, 170, 1064, 24, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7199, 199, 51, 0, 42, "Subsubsection"],
Cell[7253, 201, 319, 11, 45, "Input"],
Cell[7575, 214, 172, 6, 45, "Input"],

Cell[CellGroupData[{
Cell[7772, 224, 295, 7, 43, "Input"],
Cell[8070, 233, 80, 2, 27, "Output"],
Cell[8153, 237, 138, 4, 27, "Output"]
}, Open  ]],
Cell[8306, 244, 1035, 31, 119, "Input"],

Cell[CellGroupData[{
Cell[9366, 279, 416, 11, 61, "Input"],
Cell[9785, 292, 428, 11, 27, "Output"],
Cell[10216, 305, 82, 2, 27, "Output"],
Cell[10301, 309, 55, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10393, 315, 734, 18, 45, "Input"],
Cell[11130, 335, 30, 0, 27, "Output"]
}, Open  ]],
Cell[11175, 338, 1270, 38, 191, "Input"],

Cell[CellGroupData[{
Cell[12470, 380, 734, 18, 45, "Input"],
Cell[13207, 400, 45, 0, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13289, 405, 79, 2, 40, "Input"],
Cell[13371, 409, 46, 0, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[13466, 415, 33, 0, 42, "Subsubsection"],
Cell[13502, 417, 1884, 47, 91, "Input"],

Cell[CellGroupData[{
Cell[15411, 468, 796, 21, 112, "Input"],
Cell[16210, 491, 31, 0, 27, "Output"],
Cell[16244, 493, 46, 0, 27, "Output"]
}, Open  ]],
Cell[16305, 496, 554, 15, 43, "Input"],
Cell[16862, 513, 479, 17, 32, "Text"],
Cell[17344, 532, 517, 13, 27, "Input"],

Cell[CellGroupData[{
Cell[17886, 549, 681, 18, 45, "Input"],
Cell[18570, 569, 547, 9, 43, "Output"],
Cell[19120, 580, 651, 11, 43, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

